*DECK ASMDRV
      SUBROUTINE ASMDRV(IPSYS ,IPTRK ,IPMACR,IFTRAK,CDOOR ,IPRNTP,
     >                  NGROUP,NBMIX ,NREGIO,NANI  ,LEAKSW,ITRANC,
     >                  TITRE ,LNORM ,LALBS ,LR2   ,IPHASE,ISCATR,
     >                  KNORM ,CNAME ,
     >                  MATCOD,VOLUME,XSSIGT,XSSIGW,NSBG  ,IFUITE,
     >                  NALBP ,ALBP  )
C
C---------------------------  ASMDRV  ---------------------------------
C
C 1- PROGRAMME STATISTICS:
C
C     NAME     :  ASMDRV
C     USE      :  DRIVER FOR DRAGON ASSEMBLY AND PIJ PHASES
C     MODIFIED :  94-10-17 (AH)
C                 94-07-21 (IP AND GM)
C                 94-06-15 (IP) (PIJK)
C                 91-08-26 (GM)
C                 91-04-02 (RR)
C     AUTHOR   :  G.MARLEAU
C
C 2- ROUTINE PARAMETERS:
C   INPUT
C     IPSYS    : POINTER TO THE PIJ                      I
C     IPTRK    : POINTER TO THE TRACKING                 I
C     IPMACR   : POINTER TO THE MACROLIB                 I
C     IFTRAK   : FILE UNIT NUMBER FOR TRACKS.            I
C     CDOOR    : NAME OF THE PIJ CALCULATION DOOR        C*12
C     IPRNTP   : PRINT OPTION FOR PIJ CALCULATIONS       I
C     NGROUP   : NUMBER OF GROUPS TREATED                I
C     NBMIX    : NUMBER OF MIXTURES CONSIDERED           I
C     NREGIO   : NUMBER OF REGIONS CONSIDERED            I
C     NANI     : NUMBER OF ANISOTROPY ORDERS FOR XS      I
C     LEAKSW   : LEAKAGE SWITCH                          L
C     ITRANC   : TYPE OF TRANSPORT CORRECTIONS           I
C     TITRE    : EXECUTION TITLE                         C*72
C     LNORM    : SWITCH FOR REMOVING LEAKAGE FROM CP     L
C     LALBS    : SWITCH FOR KEEPING THE PIS              L
C     LR2      : =.TRUE. TO STORE INFORMATION REQUIRED   L
C                TO COMPUTE THE L2 NORM OF THE SOLUTION
C     IPHASE   : = -1 FOR ASS AND 0 FOR PIJ 1 FOR PIJK   I
C     ISCATR   : = 0 SCATTERING REDUCED/1 DIRECT         I
C     KNORM    : TYPE OF PIJ NORMALIZATION               I
C     CNAME    : NAME USED TO SAVE ADDITIONAL PIJ DATA   C*12
C     MATCOD   : MIXTURE CODE IN REGION                  I(NREGIO)
C     VOLUME   : VOLUME OF REGION                        R(NREGIO)
C     IFUITE   : Leakage probalility flag
C     NALBP    : Number of physical albedos              I
C   WORK VECTOR
C     XSSIGT   : GROUP TOTAL CROSS SECTION               R(NBMIX+1,
C                                                          NGROUP)
C     XSSIGW   : WITHIN GROUP SCATTERING                 R(NBMIX+1,
C                                                     NANI,NGROUP)
C     ALBP     : Physical albedos                        R(0:NALBP,
C                                                          NGROUP)
C
C
C---------------------------   ASMDRV  --------------------------------
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER     IOUT,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='ASMDRV')
C----
C MEMORY ALLOCATION
C----
      INTEGER      IBASE(1)
      REAL         RBASE(1)
      COMMON       RBASE
      EQUIVALENCE (RBASE(1),IBASE(1))
      INTEGER      IPIJSY,IPIS,IPCSCT,JSIG1
      INTEGER      KPIJSY
C-----
C INTERFACE VARIABLES
C-----
      INTEGER     IPRNTP,NGROUP,NBMIX,NREGIO,NANI,ITRANC,IGR,ILCMLN,
     >            IPHASE,ISCATR
      INTEGER     NSBG,ISG,JSG
      INTEGER     MATCOD(*)
      REAL        VOLUME(*),XSSIGT(0:NBMIX,NSBG),
     >            XSSIGW(0:NBMIX,NANI,NSBG)
      INTEGER     IFUITE
      INTEGER     NALBP
      REAL        ALBP(0:NALBP,NGROUP)
C----
C LOCAL VARIABLES
C----
      INTEGER     IPIJK,I,IREGIO,NELPIJ,IL,IPMACR,IPSYS,ITYLCM,IMAT,
     >            IMIX,IPTRK,IFTRAK,KNORM,IOP,NRG,IJKS,KS,NRGI
C
      CHARACTER   CDOOR*12,TITRE*72,CGRPNM*12,CNAME*12,CTEMP*12,
     >            HSMG*130,CMS(3)*1,NDC*4,CM*2
      LOGICAL     LEAKSW,LNORM,LALBS,LR2
      INTEGER     NNREG,NPST,KPST,PROBKS
      INTEGER     JGL
      INTEGER     IALB
C----
C DATA
C-----
      SAVE CMS
      DATA CMS    /'1','2','3'/
C----
C SET DIMENSION FOR DIRECTIONAL CP
C----
      IPIJK=1
      IF(IPHASE .EQ. 1) THEN
        IPIJK=4
      ENDIF
C----
C  CHECK MIXTURE INDICES
C----
      DO 10 I=1,NREGIO
      IF (MATCOD(I).GT.NBMIX) THEN
         WRITE(HSMG,9000) NAMSBR,NBMIX
         CALL XABORT(HSMG)
      ENDIF
   10 CONTINUE
C----
C  READ X-SECTIONS AND COMPUTE TRANSPORT CORRECTED X-SECTIONS
C----
      IF(IPHASE .EQ. -1) THEN
        IF(CDOOR(1:3).NE.'JPM') THEN
          CALL XABORT('ASMDRV: ASSEMBLY ERROR - INVALID DOOR '//CDOOR)
        ENDIF
      ENDIF
C----
C  WRITE CALCULATION OPTIONS
C----
      IF(IPRNTP.GE.1) THEN
        IF(IPHASE .EQ. -1) THEN
          WRITE(IOUT,6200) CDOOR
        ELSE
          WRITE(IOUT,6201) CDOOR
        ENDIF
        IF(ITRANC.GT.0) THEN
          WRITE(IOUT,6100)
        ENDIF
      ENDIF
      IF(IPRNTP.GE.2) THEN
        WRITE(IOUT,6000)
        WRITE(IOUT,6001) (IREGIO,VOLUME(IREGIO),MATCOD(IREGIO),
     >                      IREGIO=1,NREGIO)
      ENDIF
      IF(IPHASE .GE. 0) THEN
C----
C ALLOCATE MEMORY FOR (DOOR)P TO COMPUTE CP MATRICES.
C----
        NELPIJ=NREGIO*(NREGIO+1)/2
        IF( IPRNTP .GT. 1 )THEN
          WRITE(IOUT,6300) (NELPIJ*IPIJK+4*NREGIO*NREGIO)/256
        ENDIF
        CALL SETARA(RBASE,NELPIJ*IPIJK*NSBG,IPIJSY)
        CALL SETARA(RBASE,NREGIO,IPIS)
      ENDIF
      CALL SETARA(RBASE,NBMIX+1,JSIG1)
      IF(IPIJK .EQ. 4) THEN
        NNREG=NREGIO*NREGIO
        NPST=3*NNREG
        CALL SETARA(RBASE,NPST*NSBG,PROBKS)
      ELSE
        NNREG=1
        NPST=1
        CALL SETARA(RBASE,NPST*NSBG,PROBKS)
      ENDIF
      RBASE(JSIG1)=0.0
C----
C  SET CROSS SECTION FOR VOID TO 0.0  (GROUP INDEPENDENT)
C----
      DO 20 ISG=1,NSBG
        XSSIGT(0,ISG)=0.0
        ALBP(0,ISG)=1.0
        DO 21 IL=1,NANI
          XSSIGW(0,IL,ISG)=0.0
 21     CONTINUE
        DO 22 IALB=1,NALBP
          ALBP(IALB,ISG)=0.0
 22     CONTINUE
 20   CONTINUE
      IF(NALBP .GT. 0) THEN
        CALL LCMLEN(IPMACR,'PHYSALBEDO',ILCMLN,ITYLCM)
        IF(ILCMLN .GT. 0) THEN
          CALL LCMGET(IPMACR,'PHYSALBEDO',ALBP(1,1))
          DO ISG=2,NSBG
            DO IALB=1,NALBP
              ALBP(IALB,ISG)=ALBP(IALB,1)
            ENDDO
          ENDDO
        ENDIF
      ENDIF
      DO 150 IGR=1,NGROUP/NSBG
        JGL=(IGR-1)*NSBG
C----
C  READ X-SECTIONS AND COMPUTE TRANSPORT CORRECTED X-SECTIONS
C----
        DO 151 ISG=1,NSBG
          JSG=(IGR-1)*NSBG+ISG
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') JSG,NGROUP
          CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
          CALL LCMSIX(IPSYS,CGRPNM,ILCMUP)
          CALL LCMLEN(IPMACR,'TOTAL',ILCMLN,ITYLCM)
          IF(ILCMLN.GT.0) THEN
            CALL LCMGET(IPMACR,'TOTAL',XSSIGT(1,ISG))
          ELSE
            CALL LCMLIB(IPMACR)
            CALL XABORT('ASMDRV: READ ERROR ON LCM RECORD= TOTAL')
          ENDIF
          DO 110 IL=1,NANI
            WRITE(CM,'(I2)') IL-1
            CALL LCMLEN(IPMACR,'SIGW'//CM,ILCMLN,ITYLCM)
            IF(ILCMLN.GT.0) THEN
              CALL LCMGET(IPMACR,'SIGW'//CM,XSSIGW(1,IL,ISG))
            ELSE
              CALL XDRSET(XSSIGW(1,IL,ISG),NBMIX,0.0)
            ENDIF
 110      CONTINUE
          IF(ITRANC.NE.0) THEN
            CALL LCMLEN(IPMACR,'TRANC',ILCMLN,ITYLCM)
            IF(ILCMLN.GT.0) THEN
              CALL LCMGET(IPMACR,'TRANC',RBASE(JSIG1+1))
              DO 125 IMAT=1,NBMIX
                XSSIGT(IMAT,ISG)=XSSIGT(IMAT,ISG)-RBASE(JSIG1+IMAT)
                XSSIGW(IMAT,1,ISG)=XSSIGW(IMAT,1,ISG)-RBASE(JSIG1+IMAT)
 125          CONTINUE
            ENDIF
          ENDIF
          CALL LCMPUT(IPSYS,'DRAGON-TXSC',NBMIX+1,2,XSSIGT(0,ISG))
          CALL LCMPUT(IPSYS,'DRAGON-S0XSC',NBMIX+1,2,XSSIGW(0,1,ISG))
          IF(IPRNTP.GE.3) THEN
            WRITE(IOUT,6002)  JSG
            WRITE(IOUT,6003) (IMIX,XSSIGT(IMIX,ISG),XSSIGW(IMIX,1,ISG),
     >      IMIX=1,NBMIX)
          ENDIF
          IF(NALBP .GT. 0) THEN
            CALL LCMLEN(IPMACR,'PHYSALBEDO',ILCMLN,ITYLCM)
            IF(ILCMLN .GT. 0) THEN
              CALL LCMGET(IPMACR,'PHYSALBEDO',ALBP(1,ISG))
            ENDIF
          ENDIF
          CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
          CALL LCMSIX(IPSYS,CGRPNM,ILCMDN)
 151    CONTINUE
        IF(IPHASE .GE. 0) THEN
C----
C  CALL (DOOR)P TO COMPUTE CP MATRICES.
C----
          IF(CDOOR.EQ.'EXCELL') THEN
            CALL EXCELP(IPSYS ,IPTRK ,IFTRAK,IPRNTP,NREGIO,NBMIX ,
     >                  MATCOD,VOLUME,KNORM ,NELPIJ,IPIJK ,LEAKSW,
     >                  TITRE ,XSSIGT,RBASE(IPIJSY),
     >                  NSBG  ,NPST  ,RBASE(PROBKS),JGL,NGROUP,
     >                  IFUITE,
     >                  NALBP ,ALBP  )
          ELSE IF(CDOOR(1:3).EQ.'JPM') THEN
            KPIJSY=IPIJSY
            DO 153 ISG=1,NSBG
              CALL JPMP  (IPSYS ,IPTRK ,IFTRAK,IPRNTP,NREGIO,NBMIX ,
     >                    MATCOD,VOLUME,KNORM ,NELPIJ,IPIJK ,LEAKSW,
     >                    TITRE ,XSSIGT(0,ISG),RBASE(KPIJSY))
              KPIJSY=KPIJSY+NELPIJ*IPIJK
 153        CONTINUE
          ELSE IF(CDOOR.EQ.'SYBIL') THEN
            KPIJSY=IPIJSY
            DO 154 ISG=1,NSBG
              CALL SYBILP(IPSYS ,IPTRK ,IFTRAK,IPRNTP,NREGIO,NBMIX ,
     >                    MATCOD,VOLUME,KNORM ,NELPIJ,IPIJK ,LEAKSW,
     >                    TITRE ,XSSIGT(0,ISG),RBASE(KPIJSY))
              KPIJSY=KPIJSY+NELPIJ*IPIJK
 154        CONTINUE
          ELSE
            CALL XABORT('ASMDRV: '//CDOOR//' IS AN INVALID PIJ DOOR.')
          ENDIF
          CALL SETARA(RBASE,4*NREGIO*NREGIO,IPCSCT)
          KPIJSY=IPIJSY
          KPST=PROBKS
          DO 155 ISG=1,NSBG
          JSG=(IGR-1)*NSBG+ISG
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') JSG,NGROUP
          CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
          CALL LCMSIX(IPSYS,CGRPNM,ILCMUP)
C----
C  REMOVE LEAKAGE FROM THE CP MATRIX.
C----
          IF(LNORM.OR.LALBS) THEN
            CALL PIJNRM(NREGIO,MATCOD,VOLUME,XSSIGT(1,ISG),
     >                  RBASE(KPIJSY),
     >                  RBASE(IPIS))
            CALL LCMPUT(IPSYS,'DRAGON-PIS',NREGIO,2,RBASE(IPIS))
          ELSE
            CALL PIJCPL(NREGIO,NBMIX ,NELPIJ,MATCOD,VOLUME,
     >                  XSSIGT(0,ISG),
     >                  RBASE(KPIJSY),RBASE(IPIS)  )
            CALL LCMPUT(IPSYS,'DRAGON-PIS  ',NREGIO,2,RBASE(IPIS))
          ENDIF
          IF(CNAME.NE.' ')THEN
            CALL LCMPUT(IPSYS,CNAME,NELPIJ,2,RBASE(KPIJSY))
            IF(IPIJK .EQ. 4) THEN
              CTEMP=CNAME(1:8)//'PIJK'
              CALL LCMPUT(IPSYS,CTEMP,(IPIJK-1)*NELPIJ,2,
     >        RBASE(KPIJSY+NELPIJ))
            ENDIF
          ENDIF
C----
C  COMPUTE THE SCATTERING MODIFIED CP MATRIX.
C----
          IOP=1
          IF(ISCATR .EQ. 0) THEN
            CALL PIJSMD(IPRNTP,NBMIX,NREGIO,MATCOD,VOLUME,
     >                  XSSIGW(0,1,ISG),XSSIGT(0,ISG),
     >                  LEAKSW,RBASE(KPIJSY),RBASE(IPCSCT),IOP)
            CALL XDRSDB(NREGIO*NREGIO,RBASE(IPCSCT),RBASE(IPCSCT),1)
          ELSE
            CALL PIJNOS(NREGIO,VOLUME,RBASE(KPIJSY),RBASE(IPCSCT))
          ENDIF
          CALL LCMPUT(IPSYS,'DRAGON-PCSCT',NREGIO*NREGIO,2,
     >    RBASE(IPCSCT))
C-------
          IF(IPIJK .EQ. 4)THEN
            IOP=4
            NRG=NREGIO*NREGIO
            NRGI=3*NRG
            IF(ISCATR .EQ. 0)THEN
              DO 200 IJKS=1,3
                WRITE(NDC,'(2HDC,I2)') IJKS
                CALL LCMLEN(IPMACR,NDC,ILCMLN,ITYLCM)
                IF(ILCMLN.GT.0) THEN
                  CALL LCMGET(IPMACR,NDC,RBASE(JSIG1+1))
                ELSE
                  CALL XDRSET(RBASE(JSIG1+1),NBMIX,0.0)
                ENDIF
                DO 135 IMAT=1,NBMIX
                  RBASE(JSIG1+IMAT)=XSSIGW(IMAT,2,ISG)
     >                             +RBASE(JSIG1+IMAT)
 135            CONTINUE
                KS=KPIJSY+NELPIJ*IJKS
                CALL PIJSMD(IPRNTP,NBMIX,NREGIO,MATCOD,VOLUME,
     >                      RBASE(JSIG1),XSSIGT(0,ISG),LEAKSW,
     >                      RBASE(KS),RBASE(IPCSCT),IOP)
                CALL XDRSDB(NREGIO*NREGIO,RBASE(IPCSCT),
     >                      RBASE(IPCSCT),1)
                CALL LCMPUT(IPSYS,'DRAGON'//CMS(IJKS)//'PCSCT',NRG,2,
     >                      RBASE(IPCSCT))
 200          CONTINUE
            ELSE
              DO 300 IJKS=1,3
                KS=KPIJSY+NELPIJ*IJKS
                CALL PIJNOS(NREGIO,VOLUME,RBASE(KS),RBASE(IPCSCT))
                CALL LCMPUT(IPSYS,'DRAGON'//CMS(IJKS)//'PCSCT',NRG,2,
     >                      RBASE(IPCSCT))
 300          CONTINUE
            ENDIF
            CALL LCMPUT(IPSYS,'DRAGON1P*SCT',NNREG,2,
     >                  RBASE(KPST))
            CALL LCMPUT(IPSYS,'DRAGON2P*SCT',NNREG,2,
     >                  RBASE(KPST+NNREG))
            CALL LCMPUT(IPSYS,'DRAGON3P*SCT',NNREG,2,
     >                  RBASE(KPST+2*NNREG))
          ENDIF
          KPST=KPST+NPST
          KPIJSY=KPIJSY+NELPIJ*IPIJK
          CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
          CALL LCMSIX(IPSYS,CGRPNM,ILCMDN)
 155      CONTINUE
          CALL RLSARA(RBASE(IPCSCT))
       ELSE
        DO 156 ISG=1,NSBG
          JSG=(IGR-1)*NSBG+ISG
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') JSG,NGROUP
          CALL LCMSIX(IPSYS,CGRPNM,ILCMUP)
C----
C  CALL (DOOR)A TO ASSEMBLE RESPONSE MATRICES
C----
          IF(CDOOR(1:3).EQ.'JPM') THEN
            CALL JPMA(IPSYS,IPTRK,IFTRAK,IPRNTP,LR2,NREGIO,NBMIX,NANI,
     >                MATCOD,VOLUME,KNORM,
     >                XSSIGT(0,ISG),XSSIGW(0,1,ISG),LEAKSW,TITRE)
          ELSE
            CALL XABORT('ASMDRV: '//CDOOR//' IS AN INVALID ASM DOOR.')
          ENDIF
          CALL LCMSIX(IPSYS,CGRPNM,ILCMDN)
 156    CONTINUE
        ENDIF
 150  CONTINUE
      IF(IPIJK .EQ. 4) THEN
        CALL RLSARA(RBASE(PROBKS))
      ENDIF
      CALL RLSARA(RBASE(JSIG1))
      IF(IPHASE .GE. 0) THEN
C----
C  RELEASE MEMORY FOR (DOOR)P TO COMPUTE CP MATRICES.
C----
        CALL RLSARA(RBASE(IPIS))
        CALL RLSARA(RBASE(IPIJSY))
      ENDIF
      IF(LNORM.OR.LALBS) LEAKSW=.FALSE.
      RETURN
C----
C  FORMATS
C----
 6000 FORMAT(//30X,' EDITION REGION/VOLUME/MIXTURE '//
     >3(5X,'REGION',5X,'VOLUME  ',5X,'MIXTURE')/)
 6001 FORMAT(1P,3(5X,I4,4X,E12.5,4X,I4,4X))
 6002 FORMAT(//30X,' G R O U P : ',I5//
     >30X,' TOTAL MACROSCOPIC CROSS SECTIONS PER MIXTURE '/)
 6003 FORMAT(3(1X,'MIXTURE',4X,'TOTAL',11X,'SIGW',3X)/
     >1P,3(1X,I4,3X,E12.5,3X,E12.5))
 6100 FORMAT(//' USE TRANSPORT CORRECTED CROSS-SECTIONS')
 6200 FORMAT(//' COMPUTATION OF DRAGON RESPONSE MATRICES BY DOOR =',
     >3X,A12)
 6201 FORMAT(//' COMPUTATION OF DRAGON COMPLETE CP BY DOOR =',
     >3X,A12)
 6300 FORMAT(' *** SPACE REQUIRED FOR CP EVALUATION = ',I10,' K ***')
C----
C  ABORT FORMAT
C----
 9000 FORMAT(1X,A6,': INVALID VALUE OF NBMIX (',I5,').')
      END
