*DECK B1DIF
      SUBROUTINE B1DIF(OPTION,TYPE,NGRO,ST,SFNU,XHI,IJJ0,IJJ1,NJJ0,NJJ1,
     1 SCAT0,SCAT1,REFKEF,IMPX,D,B2,ALAM1,CAET,A2,PHI,ASTOC,BSTOC,
     2 CSTOC,SA,IND)
*
*-----------------------------------------------------------------------
*
* SOLVE THE B-N EQUATIONS AND PERFORM A BUCKLING SEARCH IF REQUIRED.
*
* INPUT PARAMETERS:
*  OPTION  : CAN BE 'LKRD', 'B0', 'P0', 'B1', 'P1' OR 'B0TR'.
*            'LKRD' IS USED TO IMPOSE A LEAKAGE COEFFICIENT.
*  TYPE    : CAN BE 'DIFF', 'K', 'B' OR 'L'.
*  NGRO    : NUMBER OF ENERGY GROUPS.
*  ST      : MACROSCOPIC TOTAL CROSS SECTIONS. DIMENSION ST(NGRO)
*  SFNU    : NU*MACROSCOPIC FISSION CROSS SECTIONS. DIMENSION SFNU(NGRO)
*  XHI     : FISSION SPECTRUM NORMALIZED TO ONE. DIMENSION XHI(NGRO)
*  IJJ0,IJJ1 : MOST THERMAL GROUP IN BAND FOR P0 AND P1 SCATTERING.
*              DIMENSION IJJ0(NGRO),IJJ1(NGRO)
*  NJJ0,NJJ1 : NUMBER OF GROUPS IN BAND FOR P0 AND P1 SCATTERING.
*              DIMENSION NJJ0(NGRO),NJJ1(NGRO)
*  SCAT0   : PACKED DIFFUSION P0 MACROSCOPIC CROSS SECTIONS.
*  SCAT1   : PACKED DIFFUSION P1 MACROSCOPIC CROSS SECTIONS.
*            DIMENSION SCAT0(SUM(NJJ0(I))),SCAT1(SUM(NJJ1(I)))
*  REFKEF  : TARGET K-EFFECTIVE FOT TYPE B OR TYPE L CALCULATIONS.
*  IMPX    : PRINT FLAG.
*
* OUTPUT PARAMETERS:
*  D       : DIFFUSION COEFFICIENTS. DIMENSION D(NGRO)
*  B2      : BUCKLING. (DOUBLE PREC.)
*  ALAM1   : EFFECTIVE MULTIPLICATION FACTOR. (DOUBLE PREC.)
*  CAET    : INFINITE MULTIPLICATION FACTOR. (DOUBLE PREC.)
*  A2      : MIGRATION AREA. (DOUBLE PREC.)
*  PHI     : FUNDAMENTAL FLUX. DIMENSION PHI(2*NGRO)  (DOUBLE PREC.)
*
* SCRATCH STORAGE:
*  ASTOC   : DIMENSION ASTOC(2*NGRO*(NGRO+1))
*  BSTOC   : DIMENSION BSTOC(2*NGRO*NGRO)
*  CSTOC   : DIMENSION CSTOC(2*NGRO)
*  SA      : DIMENSION SA(2*NGRO)
*
*-------------------------------------- AUTHOR: A. HEBERT 21/01/91 -----
*
*      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IMPLICIT NONE
*----
*  Routine parameters
*----
      CHARACTER        OPTION*4,TYPE*4
      INTEGER          NGRO
      REAL             ST(NGRO),SFNU(NGRO),XHI(NGRO)
      INTEGER          IJJ0(NGRO),IJJ1(NGRO),NJJ0(NGRO),NJJ1(NGRO)
      REAL             SCAT0(NGRO*NGRO),SCAT1(NGRO*NGRO)
      DOUBLE PRECISION REFKEF
      INTEGER          IMPX
      REAL             D(NGRO)
      DOUBLE PRECISION B2,ALAM1,CAET,A2,PHI(NGRO),
     >                 ASTOC(NGRO,NGRO+1),BSTOC(NGRO,NGRO),
     >                 CSTOC(NGRO),SA(NGRO)
      INTEGER          IND(NGRO)
*----
*  Local parameters
*----
      INTEGER           IOUT
      DOUBLE PRECISION  EPS
      CHARACTER         NAMSBR*6
      PARAMETER        (IOUT=6,EPS=1.0D-6,NAMSBR='B1DIF ')
*----
*  Functions
*----
      DOUBLE PRECISION  B1BETA
*----
*  Local variables
*----
      CHARACTER         HSMG*131
      INTEGER           IAPROX,I,J,K,IGAR,ITEX,IER,IT
      DOUBLE PRECISION  BIL1,BIL2,TOTCHI,ST2,DD,BETA,ALAM2,
     >                  XX,B2XX,YY,B2YY,ERR3,ZXC,B2OLD
*
      IF((IMPX.GT.0).AND.(TYPE.EQ.'DIFF')) THEN
         WRITE(IOUT,400) NAMSBR
      ELSE IF(IMPX.GT.0) THEN
         WRITE(IOUT,410) NAMSBR,OPTION,TYPE
      ENDIF
      IAPROX=2
      IF((OPTION.EQ.'P0').OR.(OPTION.EQ.'P1')) IAPROX=1
      IF(OPTION.EQ.'LKRD') IAPROX=0
      BIL1=0.0D0
      DO 5 I=1,NGRO
      BIL1=BIL1+XHI(I)
    5 SA(I)=ST(I)
*----
*  Save fission spectrum over energy
*----
      TOTCHI=BIL1
      IF((BIL1.GT.0.9D0).AND.(ABS(BIL1-1.0D0).GT.EPS)) THEN
        IF(IMPX.GT.5) WRITE(IOUT,460) NAMSBR
      ENDIF
      IGAR=0
      DO 10 I=1,NGRO
      DO 10 J=IJJ0(I),IJJ0(I)-NJJ0(I)+1,-1
      IGAR=IGAR+1
   10 SA(J)=SA(J)-SCAT0(IGAR)
      IF(TYPE.EQ.'DIFF') THEN
         DO 15 I=1,NGRO
         ST2=ST(I)
         DD=D(I)
         BETA=B1BETA(IAPROX,B2,ST2,DD)
   15    D(I)=BETA*ST2/(1.0D0-B2*BETA)
         RETURN
      ELSE IF((TYPE.EQ.'B').OR.(TYPE.EQ.'L')) THEN
*        COMPUTE THE INITIAL BUCKLING.
         BIL1=0.0D0
         BIL2=0.0D0
         DO 20 I=1,NGRO
         BIL1=BIL1+(TOTCHI*SFNU(I)/REFKEF-SA(I))*PHI(I)
   20    BIL2=BIL2+D(I)*PHI(I)
         B2=BIL1/BIL2
         IF(B2.EQ.0.0D0) CALL XABORT('B1DIF: THE INITIAL BUCKLING IS '
     1   //'ZERO.')
         DO 25 I=1,NGRO
         ST2=-0.99D0*(ST(I)**2)
         IF(B2.LT.ST2) THEN
            IF(IMPX.GT.0) WRITE(IOUT,415) NAMSBR,B2,ST2
            B2=ST2
         ENDIF
   25    PHI(I)=1.0D0
      ENDIF
      IF(IMPX.GT.1) WRITE(IOUT,420) NAMSBR,B2
      B2XX=B2
      B2YY=B2XX
      XX=0.0
      IF(TYPE.EQ.'L') GO TO 160
*
* COMPUTE THE FUNDAMENTAL FLUX WITH TYPE K OR TYPE B.
      ITEX=0
   30 ITEX=ITEX+1
      IF(ITEX.GT.50) THEN
        WRITE(IOUT,470) TYPE
        GO TO 300
      ENDIF
      IGAR=0
      DO 50 I=1,NGRO
      ST2=ST(I)
      DD=D(I)
      BETA=B1BETA(IAPROX,B2,ST2,DD)
      DO 40 J=1,NGRO
      ASTOC(I,J)=0.0D0
   40 BSTOC(I,J)=0.0D0
      ASTOC(I,I)=ST2
      ASTOC(I,NGRO+1)=(1.0D0-B2*BETA)*XHI(I)
      DO 50 J=IJJ0(I),IJJ0(I)-NJJ0(I)+1,-1
      IGAR=IGAR+1
      ASTOC(I,J)=ASTOC(I,J)-(1.0D0-B2*BETA)*SCAT0(IGAR)
   50 BSTOC(I,J)=SCAT0(IGAR)
      IF((OPTION.EQ.'P1').OR.(OPTION.EQ.'B1')) THEN
         DO 70 J=1,NGRO
         DO 60 K=1,NGRO
         CSTOC(K)=BSTOC(K,J)
   60    BSTOC(K,J)=0.0D0
         IGAR=0
         DO 70 I=1,NGRO
         DO 70 K=IJJ1(I),IJJ1(I)-NJJ1(I)+1,-1
         IGAR=IGAR+1
   70    BSTOC(I,J)=BSTOC(I,J)+3.0D0*SCAT1(IGAR)*CSTOC(K)
         IGAR=0
         DO 90 I=1,NGRO
         ST2=ST(I)
         DD=D(I)
         BETA=B1BETA(IAPROX,B2,ST2,DD)*ST2
         DO 80 J=1,NGRO
   80    ASTOC(I,J)=ASTOC(I,J)+BETA*BSTOC(I,J)
         DO 90 J=IJJ1(I),IJJ1(I)-NJJ1(I)+1,-1
         IGAR=IGAR+1
         ASTOC(I,NGRO+1)=ASTOC(I,NGRO+1)-3.0D0*BETA*SCAT1(IGAR)*XHI(J)
   90    ASTOC(I,J)=ASTOC(I,J)-3.0D0*BETA*SCAT1(IGAR)*ST(J)
      ENDIF
      CALL B1SOL(NGRO,ASTOC,IER)
      IF(IER.NE.0) CALL XABORT('B1DIF: SINGULAR MATRIX(1).')
      ALAM1=0.0D0
      CAET=0.0D0
      DO 130 I=1,NGRO
      ALAM1=ALAM1+SFNU(I)*ASTOC(I,NGRO+1)
  130 CAET=CAET+SA(I)*ASTOC(I,NGRO+1)
      IF(IMPX.GT.1) WRITE(IOUT,430) NAMSBR,ITEX,ALAM1
      YY=XX
      XX=REFKEF-ALAM1
      IF(TYPE.EQ.'K') THEN
         DO 140 I=1,NGRO
  140    PHI(I)=ASTOC(I,NGRO+1)/ALAM1
      ELSE IF(TYPE.EQ.'B') THEN
*        COMPUTE THE EXTRAPOLATED BUCKLING.
         ERR3=ABS((REFKEF-ALAM1)/REFKEF)
         IF(ITEX .EQ. 1) THEN
           B2=B2*(ALAM1/REFKEF-CAET)/(1.0D0-CAET)
         ELSE
           IF(ERR3 .GT. 5.0D0*EPS) THEN
             B2=B2*(ALAM1/REFKEF-CAET)/(1.0D0-CAET)
           ELSE IF(ABS(XX-YY) .GT. EPS) THEN
             B2=-XX*(B2XX-B2YY)/(XX-YY)+B2XX
           ENDIF
         ENDIF
         B2YY=B2XX
         B2XX=B2
*        CHECK THE CONVERGENCE.
         BIL1=0.0D0
         BIL2=0.0D0
         DO 150 I=1,NGRO
         ST2=ST(I)**2
         IF(B2.LT.-0.995D0*ST2) THEN
            WRITE(HSMG,9000) NAMSBR,B2,-0.995D0*ST2,ITEX
            CALL XABORT(HSMG)
         ENDIF
         BIL1=MAX(BIL1,ABS(ASTOC(I,NGRO+1)/ALAM1))
         BIL2=MAX(BIL2,ABS(PHI(I)-ASTOC(I,NGRO+1)/ALAM1))
  150    PHI(I)=ASTOC(I,NGRO+1)/ALAM1
         IF((BIL2.GE.10*EPS*BIL1).OR.(ERR3.GE.EPS)) GO TO 30
      ENDIF
      GO TO 300
*
* COMPUTE THE FUNDAMENTAL FLUX WITH TYPE L.
  160 ITEX=0
  170 ITEX=ITEX+1
      IF(ITEX.GT.50) THEN
        WRITE(IOUT,470) TYPE
        GO TO 300
      ENDIF 
      IGAR=0
      DO 200 I=1,NGRO
      ST2=ST(I)
      DD=D(I)
      BETA=B1BETA(IAPROX,B2,ST2,DD)
      DO 180 J=1,NGRO
  180 BSTOC(I,J)=XHI(I)*SFNU(J)/REFKEF
      DO 190 J=IJJ0(I),IJJ0(I)-NJJ0(I)+1,-1
      IGAR=IGAR+1
  190 BSTOC(I,J)=BSTOC(I,J)+SCAT0(IGAR)
      DO 200 J=1,NGRO
  200 ASTOC(I,J)=BSTOC(I,J)
      IF((OPTION.EQ.'P1').OR.(OPTION.EQ.'B1')) THEN
         DO 220 J=1,NGRO
         DO 210 K=1,NGRO
         CSTOC(K)=BSTOC(K,J)
  210    BSTOC(K,J)=0.0D0
         IGAR=0
         DO 220 I=1,NGRO
         DO 220 K=IJJ1(I),IJJ1(I)-NJJ1(I)+1,-1
         IGAR=IGAR+1
  220    BSTOC(I,J)=BSTOC(I,J)+3.0D0*SCAT1(IGAR)*CSTOC(K)
         IGAR=0
         DO 240 I=1,NGRO
         ST2=ST(I)
         DD=D(I)
         BETA=B1BETA(IAPROX,B2,ST2,DD)*ST2
         DO 230 J=1,NGRO
         ZXC=ASTOC(I,J)
         ASTOC(I,J)=ZXC-BETA*BSTOC(I,J)
  230    BSTOC(I,J)=ZXC
         DO 240 J=IJJ1(I),IJJ1(I)-NJJ1(I)+1,-1
         IGAR=IGAR+1
  240    ASTOC(I,J)=ASTOC(I,J)+3.0D0*BETA*SCAT1(IGAR)*ST(J)
      ENDIF
      DO 250 I=1,NGRO
      ST2=ST(I)
      DD=D(I)
      BETA=B1BETA(IAPROX,B2,ST2,DD)
      ASTOC(I,I)=ASTOC(I,I)-ST2
      ASTOC(I,NGRO+1)=PHI(I)
      DO 250 J=1,NGRO
  250 BSTOC(I,J)=BETA*BSTOC(I,J)
      B2OLD=B2
      CALL ALEIGD(ASTOC,BSTOC,NGRO,B2,ASTOC(1,NGRO+1),
     >            EPS,IT,CSTOC,IND)
      IF(IMPX.GT.1) WRITE(IOUT,440) NAMSBR,ITEX,IT,B2
      BIL1=0.0D0
      BIL2=0.0D0
      DO 260 I=1,NGRO
      ST2=ST(I)**2
      IF(B2.LT.-0.995D0*ST2) THEN
         WRITE(HSMG,9000) NAMSBR,B2,-0.995D0*ST2,ITEX
         CALL XABORT(HSMG)
      ENDIF
      BIL1=MAX(BIL1,ABS(ASTOC(I,NGRO+1)))
      BIL2=MAX(BIL2,ABS(PHI(I)-ASTOC(I,NGRO+1)))
  260 PHI(I)=ASTOC(I,NGRO+1)
      ERR3=ABS(B2-B2OLD)
      IF((BIL2.GE.10*EPS*BIL1).OR.(ERR3.GE.EPS)) GO TO 170
*
* COMPUTE THE LEAKAGE COEFFICIENTS:
  300 IF((OPTION.EQ.'P1').OR.(OPTION.EQ.'B1')) THEN
         IGAR=0
         DO 320 I=1,NGRO
         ST2=ST(I)
         DD=D(I)
         BETA=B1BETA(IAPROX,B2,ST2,DD)
         BETA=BETA*ST2/(1.0D0-B2*BETA)
         DO 310 J=1,NGRO
  310    ASTOC(I,J)=0.0D0
         ASTOC(I,I)=1.0D0
         ASTOC(I,NGRO+1)=BETA
         DO 320 J=IJJ1(I),IJJ1(I)-NJJ1(I)+1,-1
         IGAR=IGAR+1
  320    ASTOC(I,J)=ASTOC(I,J)-3.0D0*BETA*SCAT1(IGAR)*PHI(J)/PHI(I)
         CALL B1SOL(NGRO,ASTOC,IER)
         IF(IER.NE.0) CALL XABORT('B1DIF: SINGULAR MATRIX(2).')
         DO 330 I=1,NGRO
  330    D(I)=ASTOC(I,NGRO+1)
      ELSE IF(OPTION.NE.'LKRD') THEN
         DO 340 I=1,NGRO
         ST2=ST(I)
         DD=D(I)
         BETA=B1BETA(IAPROX,B2,ST2,DD)
  340    D(I)=BETA*ST2/(1.0D0-B2*BETA)
      ENDIF
      A2=0.0D0
      CAET=0.0D0
      ZXC=0.0D0
      DO 350 I=1,NGRO
      A2=A2+D(I)*PHI(I)
      CAET=CAET+SA(I)*PHI(I)
  350 ZXC=ZXC+TOTCHI*SFNU(I)*PHI(I)/REFKEF
      A2=A2/CAET
      CAET=ZXC/CAET
      ALAM2=CAET/(1.0D0+A2*B2)
      IF(TYPE.EQ.'L') ALAM1=ALAM2
      IF(IMPX.GT.0) THEN
         WRITE(IOUT,450) ITEX,B2,ALAM1,ALAM2,CAET,A2
      ENDIF
      RETURN
*
  400 FORMAT(1X,A6,': DIFFUSION COEFFICIENT CALCULATION.')
  410 FORMAT(1X,A6,': SOLUTION OF ',A4,' EQUATIONS WITH TYPE ',A4)
  415 FORMAT(1X,A6,': THE INITIAL BUCKLING WAS INCREASED FROM',1P,
     > E13.5,' TO ',E13.5)
  420 FORMAT(1X,A6,': INITIAL BUCKLING =',1P,E13.5)
  430 FORMAT(1X,A6,': K-EFFECTIVE ITERATION NO.',I3,
     >'. K-EFFECTIVE =',F10.6)
  440 FORMAT(1X,A6,': BUCKLING ITERATION NO.',I3,' CONVERGED IN',I5,
     > ' INNER ITERATIONS. BUCKLING =',1P,E13.5,5X,'Precision =',E13.5)
  450 FORMAT(8X,'NUMBER OF ITERATIONS =',I3/
     >       8X,'BUCKLING =',1P,E13.5,0P/
     >       8X,'K-EFFECTIVE =',F10.6,'  (',F10.6,' )'/
     >       8X,'K-INFINITE =',F10.6/
     >       8X,'MIGRATION AREA =',1P,E13.5/)
  460 FORMAT(1X,A6,': WARNING INCONSISTENT FISSION SPECTRUM')
  470 FORMAT(1X,A6,': WARNING '/
     >       10X,'Convergence was not reached for ',A4,' search')
 9000 FORMAT(A6,': THE EXTRAPOLATED BUCKLING (',1P,E12.4,')',
     >       ' IS SMALLER THAN',E12.4,' AT ITERATION',I5,'.')
      END
