*DECK B1HOM
      SUBROUTINE B1HOM (IPMACR,NUNKNO,OPTION,TYPE,NGRO,IPAS,NBM,NFISSI,
     1                  VOL,MAT,KEYFLX,FLUX,REFKEF,IMPX,D,ALAM1,ST,PHI,
     2                  XSCAT,IJJ,NJJ,IPOS,INORM,B2)
*
*-----------------------------------------------------------------------
*
* HOMOGENIZATION OF THE UNIT CELL AND SOLUTION OF THE B-N EQUATIONS.
* THE CROSS SECTION INFORMATION IS FOUND ON LCM.
*
* INPUT PARAMETERS:
*  IPMACR  : POINTER TO THE MACROLIB (L_MACROLIB SIGNATURE).
*  NUNKNO  : FIRST DIMENSION OF MATRIX FLUX. MAXIMUM NUMBER OF UNKNOWNS.
*  OPTION  : CAN BE 'LKRD' (IMPOSED LEAKAGE COEFFICIENTS),
*            'B0' (B-0), 'P0' (P-0), 'B1' (B-1), 'P1' (P-1)
*            OR 'B0TR' (B-0 WITH TRANSPORT CORRECTION).
*  TYPE    : CAN BE 'DIFF' (DO A B-0 CALCULATION OF D(NGRO) AND EXIT),
*                   'K' (DO A B-N CALCULATION WITH KEFF SEARCH),
*                   'B' (DO A B-N CALCULATION WITH BUCKLING SEARCH),
*                   'L' (DO A B-N CALCULATION WITH BUCKLING SEARCH
*                        FOR A PROBLEM WITH FEW OR NO FISSION).
*  NGRO    : NUMBER OF GROUPS.
*  IPAS    : NUMBER OF VOLUMES.
*  NBM     : NUMBER OF MIXTURES.
*  NFISSI  : MAXIMUM NUMBER OF FISSION SPECTRUM ASSIGNED TO A MIXTURE.
*  VOL     : VOLUMES. DIMENSION VOL(IPAS)
*  MAT     : MIXTURE NUMBER OF EACH VOLUME. DIMENSION MAT(IPAS)
*  KEYFLX  : POSITION OF EACH FLUX IN THE UNKNOWN VECTOR.
*            DIMENSION KEYFLX(IPAS)
*  FLUX    : DIRECT UNKNOWN VECTOR. DIMENSION FLUX(NUNKNO,NGRO)
*  REFKEF  : TARGET K-EFFECTIVE FOT TYPE B OR TYPE L CALCULATIONS.
*  IMPX    : PRINT FLAG.
*  INORM   : FLUX NORMALIZATION OPTION
*            =-1: FOR B1 HETEREGENEOUS; =2: FOR B1 HOMOGENEOUS.
*  B2      : ORIGINAL DIRECTION DEPENDANT B2 DIMENSION B2(4)
*
* OUTPUT PARAMETERS:
*  D       : DIFFUSION COEFFICIENTS. DIMENSION D(NGRO)
*  ALAM1   : EFFECTIVE MULTIPLICATION FACTOR. (DOUBLE PREC.)
*  ST      : HOMOGENEIZED TOTAL XS .           ST(NGRO)
*
* SCRATCH STORAGE:
*  PHI     : DIMENSION PHI(2*NGRO)            (DOUBLE PREC.)
*  XSCAT   : SCATTERING VECTOR                    R(0:NBM*NGRO)
*            XSCAT(0) = 0.0
*  NJJ     : NUMBER OF SCATTERING GROUP           I(0:NBM)
*            NJJ(0) = -NGRO
*  IJJ     : LAST SCATTERING GROUP                I(0:NBM)
*            IJJ(0) = 0
*  IPOS    : POSITION SELF SCATTERING IN XSCAT    I(0:NBM)
*            IPOS(0)= NGRO+1
*
*-------------------------------------- AUTHOR: A. HEBERT 16/02/88 -----
*
      CHARACTER*4 OPTION,TYPE
      DOUBLE PRECISION B2HOM,ALAM1,CAET,A2,PHI(*),REFKEF,CURN,B2T(3)
      DIMENSION VOL(*),MAT(*),KEYFLX(*),FLUX(NUNKNO,*),D(*),ST(*),
     1          IDEL(2),B2(4),XSCAT(0:NBM*NGRO)
      INTEGER   IJJ(0:NBM),NJJ(0:NBM),IPOS(0:NBM)
      COMMON BASE(1)
*
      IAN=0
      IF ((OPTION.EQ.'B0').OR.(OPTION.EQ.'P0')) THEN
         IAN=0
      ELSE IF ((OPTION.EQ.'B1').OR.(OPTION.EQ.'P1')) THEN
         IAN=1
      ELSE IF (OPTION.EQ.'B0TR') THEN
         IAN=-1
      ENDIF
      CALL SETARA(BASE,NGRO,IIJJ0)
      CALL SETARA(BASE,NGRO,IIJJ1)
      CALL SETARA(BASE,NGRO,INJJ0)
      CALL SETARA(BASE,NGRO,INJJ1)
      CALL B1HXS1(NUNKNO,IPMACR,IPAS,NGRO,NBM,IAN,NFISSI,VOL,MAT,KEYFLX,
     1 FLUX,BASE(IIJJ0),BASE(IIJJ1),BASE(INJJ0),BASE(INJJ1),IDEL,IJJ,
     2 NJJ)
*
      CALL SETARA(BASE,NBM+1,IGAR)
      CALL SETARA(BASE,NBM*NFISSI,ISIGF0)
      BASE(IGAR)=0.0
      CALL SETARA(BASE,2*NGRO,IGAF)
      CALL SETARA(BASE,2*NFISSI*(NBM+1),IA14)
      CALL SETARA(BASE,NGRO,ISA)
      CALL SETARA(BASE,NGRO,ISFNU)
      CALL SETARA(BASE,NGRO,IXHI)
      CALL SETARA(BASE,IDEL(1),ISCAT0)
      CALL SETARA(BASE,IDEL(2),ISCAT1)
      CALL SETARA(BASE,2*NGRO,ICUR)
      IF(INORM.LT.2) THEN
         CALL B1HXS3(NUNKNO,IPMACR,IPAS,NGRO,NBM,IAN,NFISSI,VOL,MAT,
     1   KEYFLX,FLUX,BASE(IIJJ0),BASE(IIJJ1),BASE(INJJ0),BASE(INJJ1),
     2   IDEL,XSCAT,IJJ,NJJ,IPOS,BASE(IGAR),BASE(IGAF),BASE(IA14),
     3   PHI,BASE(ISA),ST,BASE(ISFNU),BASE(IXHI),BASE(ISCAT0),
     4   BASE(ISCAT1),BASE(ISIGF0),BASE(ICUR),NGROIN,INORM,B2,B2HOM)
      ENDIF
      CALL B1HXS2(NUNKNO,IPMACR,IPAS,NGRO,NBM,IAN,NFISSI,VOL,MAT,
     1 KEYFLX,FLUX,BASE(IIJJ0),BASE(IIJJ1),BASE(INJJ0),BASE(INJJ1),
     2 IDEL,XSCAT,IJJ,NJJ,IPOS,BASE(IGAR),BASE(IGAF),BASE(IA14),
     3 PHI,BASE(ISA),ST,BASE(ISFNU),BASE(IXHI),BASE(ISCAT0),
     4 BASE(ISCAT1),BASE(ISIGF0),BASE(ICUR),NGROIN,INORM,B2,B2HOM)
      CALL RLSARA(BASE(ICUR))
      CALL RLSARA(BASE(IA14))
      CALL RLSARA(BASE(IGAF))
      CALL RLSARA(BASE(ISIGF0))
      CALL RLSARA(BASE(IGAR))
*
      CALL SETARA(BASE,2*NGRO*(NGRO+1),IASTOC)
      CALL SETARA(BASE,2*NGRO*NGRO,IBSTOC)
      CALL SETARA(BASE,2*NGRO,ICSTOC)
      CALL SETARA(BASE,2*NGRO,ISAG)
      CALL SETARA(BASE,NGRO,IIND)
      CALL B1DIF(OPTION,TYPE,NGRO,ST,BASE(ISFNU),BASE(IXHI),
     1 BASE(IIJJ0),BASE(IIJJ1),BASE(INJJ0),BASE(INJJ1),BASE(ISCAT0),
     2 BASE(ISCAT1),REFKEF,IMPX,D,B2HOM,ALAM1,CAET,A2,PHI,
     3 BASE(IASTOC),BASE(IBSTOC),BASE(ICSTOC),BASE(ISAG),
     4 BASE(IIND))
      CALL RLSARA(BASE(IIND))
      CALL RLSARA(BASE(ISAG))
      CALL RLSARA(BASE(ICSTOC))
      CALL RLSARA(BASE(IBSTOC))
      CALL RLSARA(BASE(IASTOC))
      B2(4)=B2HOM
*
      IF (TYPE.EQ.'DIFF') GO TO 70
*
      IF(INORM.EQ.2) THEN
*
* NORMALIZE THE DRAGON FLUX USING THE FUNDAMENTAL B1 FLUX.
        DO 30 I=1,NGRO
          CAET=0.0D0
          DO 10 L=1,IPAS
            CAET=CAET+VOL(L)*FLUX(KEYFLX(L),I)
   10     CONTINUE
          CAET=PHI(I)/CAET
          DO 20 L=1,NUNKNO
            FLUX(L,I)=FLUX(L,I)*CAET
   20     CONTINUE
   30   CONTINUE
      ELSE IF(INORM.LE.0) THEN
        IF(B2(4).EQ.0.0.OR.
     >    (B2(1).EQ.0.0.AND.B2(2).EQ.0.0.AND.B2(3).EQ.0.0)) THEN
          B2T(1)=0.33333333333333D0
          B2T(2)=B2T(1)
          B2T(3)=B2T(1)
        ELSE IF(INORM.EQ.-1) THEN
          B2HOM=1.0D0/(DBLE(B2(1))+DBLE(B2(2))+DBLE(B2(3)))
          B2T(1)=B2HOM*DBLE(B2(1))
          B2T(2)=B2HOM*DBLE(B2(2))
          B2T(3)=B2HOM*DBLE(B2(3))
        ENDIF
*
* NORMALIZE THE DRAGON FLUX USING THE FUNDAMENTAL B1 FLUX.
        DO 60 I=1,NGRO
          CAET=0.0D0
          CURN=0.0D0
          DO 40 L=1,IPAS
            CAET=CAET+VOL(L)*FLUX(KEYFLX(L),I)
            CURN=CURN+B2T(1)*FLUX(KEYFLX(L)+IPAS,I)*VOL(L)
     >               +B2T(2)*FLUX(KEYFLX(L)+2*IPAS,I)*VOL(L)
     >               +B2T(3)*FLUX(KEYFLX(L)+3*IPAS,I)*VOL(L)
   40     CONTINUE
          CAET=PHI(I)/CAET
          CURN=PHI(I)*D(I)/CURN
          DO 50 L=1,IPAS
            FLUX(KEYFLX(L),I)=FLUX(KEYFLX(L),I)*CAET
            FLUX(KEYFLX(L)+IPAS,I)=FLUX(KEYFLX(L)+IPAS,I)*CURN
            FLUX(KEYFLX(L)+2*IPAS,I)=FLUX(KEYFLX(L)+2*IPAS,I)*CURN
            FLUX(KEYFLX(L)+3*IPAS,I)=FLUX(KEYFLX(L)+3*IPAS,I)*CURN
   50     CONTINUE
          IF(IMPX .GT. 100) THEN
            WRITE(6,6000) I,CAET,CURN
 6000       FORMAT('Renormalisation factors for group ',I10,1P,2E15.7)
          ENDIF
   60   CONTINUE
      ENDIF
*
   70 CONTINUE
      CALL RLSARA(BASE(ISCAT1))
      CALL RLSARA(BASE(ISCAT0))
      CALL RLSARA(BASE(IXHI))
      CALL RLSARA(BASE(ISFNU))
      CALL RLSARA(BASE(ISA))
      CALL RLSARA(BASE(INJJ1))
      CALL RLSARA(BASE(INJJ0))
      CALL RLSARA(BASE(IIJJ1))
      CALL RLSARA(BASE(IIJJ0))
      RETURN
      END
