*DECK BIVCOL
      SUBROUTINE BIVCOL (IPTRK,IMPX,IELEM,ICOL)
C
C-----------------------------------------------------------------------
C
C SELECTION OF THE UNIT MATRICES (MASS, STIFFNESS, ETC.) FOR A FINITE
C ELEMENT APPROXIMATION.
C
C INPUT PARAMETERS:
C  IPTRK  : L_TRACK POINTER TO THE TRACKING INFORMATION.
C  IMPX   : PRINT PARAMETER.
C  IELEM  : DEGREE OF THE FINITE ELEMENTS. =1 (LINEAR POLYNOMIALS) ;
C           =2 (PARABOLIC POLYNOMIALS) ;   =3 (CUBIC POLYNOMIALS) ;
C           =4 (QUARTIC POLYNOMIALS).
C  ICOL   : TYPE OF QUADRATURE USED TO INTEGRATE THE MASS MATRICES.
C           =1 (ANALYTIC INTEGRATION) ;  =2 (GAUSS-LOBATTO QUADRATURE) ;
C           =3 (GAUSS-LEGENDRE QUADRATURE).
C           IELEM=1 AND ICOL=2 IS EQUIVALENT TO FINITE DIFFERENCES.
C
C THE BIVAC REFERENCE ELEMENT IS DEFINED BETWEEN POINTS -1/2 AND +1/2.
C THE COLLOCATION POLYNOMIALS CORRESPONDING TO APPROXIMATIONS LL2$, PL3,
C PL3$, PL3#, CL4, CL4$, CL4# AND QL5$ ARE PARTIALLY OR COMPLETELY
C ORTHONORMALIZED IN ORDER TO PRODUCE A SPARSE MASS MATRIX.
C
C------------------------------------------ AUTHOR: A. HEBERT 1977 -----
C
      IMPLICIT NONE
C----
C LOCAL AND INTERFACE VARIABLES
C----
      INTEGER   LC,I,J,K,IELEM,ICOL,IPTRK,IMPX
      REAL      EL(2,2),TL(2),TSL(2),RL(2,2),RSL(2,2),QSL(2,2),TSL1(2),
     &          TSL2(2),RL2(2,2),RSL2(2,2)
      REAL      RHA6(6,6),QHA6(6,6),RHL6(6,6),QHL6(6,6),RTA(3,3),
     &          QTA(3,3),RTL(3,3),QTL(3,3)
      REAL      EP(3,3),TP(3),TSP(3),RP(3,3),VP(3,2),RSP(3,3),QSP(3,3),
     &          EP1(3,3),TP1(3),TSP1(3),VP1(3,2),QSP1(3,3),
     &          EP2(3,3),TP2(3),
     &          TSP2(3),RP2(3,3),VP2(3,2),RSP2(3,3),QSP2(3,3)
      REAL      EC(4,4),TC(4),TSC(4),RC(4,4),VC(4,3),RSC(4,4),QSC(4,4),
     &          EC1(4,4),TC1(4),TSC1(4),VC1(4,3),QSC1(4,4),
     &          EC2(4,4),TC2(4),TSC2(4),RC2(4,4),VC2(4,3),
     &          RSC2(4,4),QSC2(4,4)
      REAL      EQ(5,5),VQ(5,4),TQ(5),TSQ(5),QSQ(5,5)
      CHARACTER*40 HTYPE
C
C****************** FINITE ELEMENT BASIC MATRICES **********************
C                                                                      *
      REAL      T(5),TS(5),R(25),RS(25),Q(25),QS(25),V(20),E(25),
     &          RH(6,6),QH(6,6),RT(3,3),QT(3,3)
C                                                                      *
C LC      : NUMBER OF POLYNOMIALS IN A COMPLETE 1-D BASIS.             *
C T       : CARTESIAN LINEAR PRODUCT VECTOR.                           *
C TS      : CYLINDRICAL LINEAR PRODUCT VECTOR.                         *
C R       : CARTESIAN MASS MATRIX.                                     *
C RS      : CYLINDRICAL MASS MATRIX.                                   *
C Q       : CARTESIAN STIFFNESS MATRIX.                                *
C QS      : CYLINDRICAL STIFFNESS MATRIX.                              *
C V       : NODAL COUPLING MATRIX.                                     *
C E       : POLYNOMIAL COEFFICIENTS.                                   *
C RH      : HEXAGONAL MASS MATRIX.                                     *
C QH      : HEXAGONAL STIFFNESS MATRIX.                                *
C RT      : TRIANGULAR MASS MATRIX.                                    *
C QT      : TRIANGULAR STIFFNESS MATRIX.                               *
C                                                                      *
C***********************************************************************
C
C LINEAR LAGRANGIAN POLYNOMIALS.
      DATA EL/0.5,-1.0,0.5,1.0/
      DATA TL/0.5,0.5/
      DATA RL/
     $ 0.333333333333, 0.166666666667, 0.166666666667, 0.333333333333/
*     CYLINDRICAL OPTION MATRICES:
      DATA TSL/-0.083333333333, 0.083333333333/
      DATA RSL/
     $-0.083333333333, 0.000000000000, 0.000000000000, 0.083333333333/
      DATA QSL/
     $ 0.000000000000, 0.000000000000, 0.000000000000, 0.000000000000/
*     GAUSS-LOBATTO (FINITE DIFFERENCES) MATRICES:
      DATA TSL1/-0.25,0.25/
*     GAUSS-LEGENDRE (SUPERCONVERGENT) MATRICES:
      DATA TSL2/0.0,0.0/
      DATA RL2/0.25,0.25,0.25,0.25/
      DATA RSL2/0.0,0.0,0.0,0.0/
*
*     ANALYTIC INTEGRATION FOR HEXAGON AND TRIANGLE.
      DATA RHA6/
     > 0.158470 , 0.086580 , 0.036760 , 0.027870 , 0.036760 , 0.086580,
     > 0.086580 , 0.158470 , 0.086580 , 0.036760 , 0.027870 , 0.036760,
     > 0.036760 , 0.086580 , 0.158470 , 0.086580 , 0.036760 , 0.027870,
     > 0.027870 , 0.036760 , 0.086580 , 0.158470 , 0.086580 , 0.036760,
     > 0.036760 , 0.027870 , 0.036760 , 0.086580 , 0.158470 , 0.086580,
     > 0.086580 , 0.036760 , 0.027870 , 0.036760 , 0.086580 , 0.158470/
      DATA QHA6/
     > 0.760640 ,-0.161980 ,-0.169310 ,-0.098060 ,-0.169310 ,-0.161980,
     >-0.161980 , 0.760640 ,-0.161980 ,-0.169310 ,-0.098060 ,-0.169310,
     >-0.169310 ,-0.161980 , 0.760640 ,-0.161980 ,-0.169310 ,-0.098060,
     >-0.098060 ,-0.169310 ,-0.161980 , 0.760640 ,-0.161980 ,-0.169310,
     >-0.169310 ,-0.098060 ,-0.169310 ,-0.161980 , 0.760640 ,-0.161980,
     >-0.161980 ,-0.169310 ,-0.098060 ,-0.169310 ,-0.161980 , 0.760640/
      DATA RTA/
     > 1.0, 0.5, 0.5, 0.5, 1.0, 0.5, 0.5, 0.5, 1.0/
      DATA QTA/
     > 1.0,-0.5,-0.5,-0.5, 1.0,-0.5,-0.5,-0.5, 1.0/
*
*     GAUSS-LOBATTO INTEGRATION FOR HEXAGON AND TRIANGLE.
      DATA RHL6/
     > 1.000000 , 0.000000 , 0.000000 , 0.000000 , 0.000000 , 0.000000,
     > 0.000000 , 1.000000 , 0.000000 , 0.000000 , 0.000000 , 0.000000,
     > 0.000000 , 0.000000 , 1.000000 , 0.000000 , 0.000000 , 0.000000,
     > 0.000000 , 0.000000 , 0.000000 , 1.000000 , 0.000000 , 0.000000,
     > 0.000000 , 0.000000 , 0.000000 , 0.000000 , 1.000000 , 0.000000,
     > 0.000000 , 0.000000 , 0.000000 , 0.000000 , 0.000000 , 1.000000/
      DATA QHL6/
     > 1.666667 ,-1.000000 , 0.166667 ,-1.000000 , 0.166667 , 0.000000,
     >-1.000000 , 1.666667 ,-1.000000 , 0.166667 , 0.000000 , 0.166667,
     > 0.166667 ,-1.000000 , 1.666667 , 0.000000 , 0.166667 ,-1.000000,
     >-1.000000 , 0.166667 , 0.000000 , 1.666667 ,-1.000000 , 0.166667,
     > 0.166667 , 0.000000 , 0.166667 ,-1.000000 , 1.666667 ,-1.000000,
     > 0.000000 , 0.166667 ,-1.000000 , 0.166667 ,-1.000000 , 1.666667/
      DATA RTL/
     > 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0/
      DATA QTL/
     > 1.0,-0.5,-0.5,-0.5, 1.0,-0.5,-0.5,-0.5, 1.0/
*
* PARABOLIC LAGRANGIAN POLYNOMIALS.
      DATA EP/
     $-0.125000000000,-1.000000000000, 2.500000000000,
     $ 1.250000000000, 0.000000000000,-5.000000000000,
     $-0.125000000000, 1.000000000000, 2.500000000000/
      DATA TP/
     $ 0.083333333333, 0.833333333333, 0.083333333333/
      DATA RP/
     $ 0.125000000000, 0.000000000000,-0.041666666667,
     $ 0.000000000000, 0.833333333333, 0.000000000000,
     $-0.041666666667, 0.000000000000, 0.125000000000/
      DATA VP/
     $-1.000000000000, 0.000000000000, 1.000000000000,
     $ 1.443375672974,-2.886751345948, 1.443375672974/
*     CYLINDRICAL OPTION MATRICES:
      DATA TSP/
     $-0.083333333333, 0.000000000000, 0.083333333333/
      DATA RSP/
     $-0.041666666667,-0.041666666667, 0.000000000000,
     $-0.041666666667, 0.000000000000, 0.041666666667,
     $ 0.000000000000, 0.041666666667, 0.041666666667/
      DATA QSP/
     $-0.833333333333, 0.833333333333, 0.000000000000,
     $ 0.833333333333, 0.000000000000,-0.833333333333,
     $ 0.000000000000,-0.833333333333, 0.833333333333/
*     GAUSS-LOBATTO (VARIATIONAL COLLOCATION METHOD) MATRICES:
      DATA EP1/0.0,-1.0,2.0,1.0,0.0,-4.0,0.0,1.0,2.0/
      DATA TP1/
     $ 0.166666666667, 0.666666666667, 0.166666666667/
      DATA VP1/
     $-1.000000000000, 0.0000000000000, 1.000000000000,
     $ 1.154700538379,-2.3094010767585, 1.154700538379/
*     CYLINDRICAL GAUSS-LOBATTO (VARIATIONAL COLLOCATION METHOD)
*     MATRICES.
      DATA TSP1/
     $-0.083333333333, 0.000000000000, 0.083333333333/
      DATA QSP1/
     $-0.666666666667, 0.666666666667, 0.000000000000,
     $ 0.666666666667, 0.000000000000,-0.666666666667,
     $ 0.000000000000,-0.666666666667, 0.666666666667/
*     GAUSS-LEGENDRE (SUPERCONVERGENT) MATRICES:
      DATA EP2/
     $-0.250000000000,-1.000000000000, 3.000000000000,
     $ 1.500000000000, 0.000000000000,-6.000000000000,
     $-0.250000000000, 1.000000000000, 3.000000000000/
      DATA TP2/
     $ 0.000000000000, 1.000000000000, 0.000000000000/
      DATA RP2/
     $ 0.083333333333, 0.000000000000,-0.083333333333,
     $ 0.000000000000, 1.000000000000, 0.000000000000,
     $-0.083333333333, 0.000000000000, 0.083333333333/
      DATA VP2/
     $-1.000000000000, 0.000000000000, 1.000000000000,
     $ 1.732050807569,-3.464101615138, 1.732050807569/
*     CYLINDRICAL GAUSS-LEGENDRE (SUPERCONVERGENT) MATRICES:
      DATA TSP2/
     $-0.083333333333, 0.000000000000, 0.083333333333/
      DATA RSP2/
     $ 0.000000000000,-0.083333333333, 0.000000000000,
     $-0.083333333333, 0.000000000000, 0.083333333333,
     $ 0.000000000000, 0.083333333333, 0.000000000000/
      DATA QSP2/
     $-1.000000000000, 1.000000000000, 0.000000000000,
     $ 1.000000000000, 0.000000000000,-1.000000000000,
     $ 0.000000000000,-1.000000000000, 1.000000000000/
*
* CUBIC LAGRANGIAN POLYNOMIALS.
      DATA EC/
     $-0.125000000000, 0.750000000000, 2.500000000000, -7.000000000000,
     $ 0.625000000000,-3.307189138831,-2.500000000000, 13.228756555323,
     $ 0.625000000000, 3.307189138831,-2.500000000000,-13.228756555323,
     $-0.125000000000,-0.750000000000, 2.500000000000,  7.000000000000/
      DATA TC/
     $ 0.083333333333, 0.416666666667, 0.416666666667, 0.083333333333/
      DATA RC/
     $ 0.066666666667, 0.000000000000, 0.000000000000, 0.016666666667,
     $ 0.000000000000, 0.416666666667, 0.000000000000, 0.000000000000,
     $ 0.000000000000, 0.000000000000, 0.416666666667, 0.000000000000,
     $ 0.016666666667, 0.000000000000, 0.000000000000, 0.066666666667/
      DATA VC/
     $-1.000000000000, 0.000000000000, 0.000000000000,1.000000000000,
     $ 1.443375672974,-1.443375672974,-1.443375672974,1.443375672974,
     $-1.565247584250, 2.958039891550,-2.958039891550,1.565247584250/
*     CYLINDRICAL OPTION MATRICES:
      DATA TSC /
     $-0.025000000000,-0.110239637961, 0.110239637961, 0.025000000000/
      DATA RSC/
     $-0.025000000000,-0.015748519709, 0.015748519709, 0.000000000000,
     $-0.015748519709,-0.078742598544, 0.000000000000,-0.015748519709,
     $ 0.015748519709, 0.000000000000, 0.078742598544, 0.015748519709,
     $ 0.000000000000,-0.015748519709, 0.015748519709, 0.025000000000/
      DATA QSC/
     $-2.000000000000, 2.102396379610,-0.102396379610, 0.000000000000,
     $ 2.102396379610,-2.204792759220, 0.000000000000, 0.102396379610,
     $-0.102396379610, 0.000000000000, 2.204792759220,-2.102396379610,
     $ 0.000000000000, 0.102396379610,-2.102396379610, 2.000000000000/
*     GAUSS-LOBATTO (VARIATIONAL COLLOCATION METHOD) MATRICES:
      DATA EC1/
     $-0.125000000000, 0.250000000000, 2.500000000000, -5.000000000000,
     $ 0.625000000000,-2.795084971875,-2.500000000000, 11.180339887499,
     $ 0.625000000000, 2.795084971875,-2.500000000000,-11.180339887499,
     $-0.125000000000,-0.250000000000, 2.500000000000,  5.000000000000/
      DATA TC1/
     $ 0.083333333333, 0.416666666667, 0.416666666667, 0.083333333333/
      DATA VC1/
     $-1.000000000000, 0.000000000000, 0.000000000000,1.000000000000,
     $ 1.443375672974,-1.443375672974,-1.443375672974,1.443375672974,
     $-1.118033988750, 2.500000000000,-2.500000000000,1.118033988750/
*     CYLINDRICAL GAUSS-LOBATTO (VARIATIONAL COLLOCATION METHOD)
*     MATRICES:
      DATA TSC1/
     $-0.041666666667,-0.093169499062, 0.093169499062, 0.041666666667/
      DATA QSC1/
     $-1.666666666667, 1.765028323958,-0.098361657292, 0.000000000000,
     $ 1.765028323958,-1.863389981250, 0.000000000000, 0.098361657292,
     $-0.098361657292, 0.000000000000, 1.863389981250,-1.765028323958,
     $ 0.000000000000, 0.098361657292,-1.765028323958, 1.666666666667/
*     GAUSS-LEGENDRE (SUPERCONVERGENT) MATRICES:
      DATA EC2/
     $-0.125000000000, 1.500000000000, 2.500000000000,-10.000000000000,
     $ 0.625000000000,-3.952847075210,-2.500000000000, 15.811388300842,
     $ 0.625000000000, 3.952847075210,-2.500000000000,-15.811388300842,
     $-0.125000000000,-1.500000000000, 2.500000000000, 10.000000000000/
      DATA TC2/
     $ 0.083333333333, 0.416666666667, 0.416666666667, 0.083333333333/
      DATA RC2/
     $ 0.041666666667, 0.000000000000, 0.000000000000, 0.041666666667,
     $ 0.000000000000, 0.416666666667, 0.000000000000, 0.000000000000,
     $ 0.000000000000, 0.000000000000, 0.416666666667, 0.000000000000,
     $ 0.041666666667, 0.000000000000, 0.000000000000, 0.041666666667/
      DATA VC2/
     $-1.000000000000, 0.000000000000, 0.000000000000,1.000000000000,
     $ 1.443375672974,-1.443375672974,-1.443375672974,1.443375672974,
     $-2.236067977500, 3.535533905933,-3.535533905933,2.236067977500/
*     CYLINDRICAL GAUSS-LEGENDRE (SUPERCONVERGENT) MATRICES:
      DATA TSC2/
     $ 0.000000000000,-0.131761569174, 0.131761569174, 0.000000000000/
      DATA RSC2/
     $ 0.000000000000,-0.032940392293, 0.032940392293, 0.000000000000,
     $-0.032940392293,-0.065880784587, 0.000000000000,-0.032940392293,
     $ 0.032940392293, 0.000000000000, 0.065880784587, 0.032940392293,
     $ 0.000000000000,-0.032940392293, 0.032940392293, 0.000000000000/
      DATA QSC2/
     $-2.500000000000, 2.567615691737,-0.067615691737, 0.000000000000,
     $ 2.567615691737,-2.635231383474, 0.000000000000, 0.067615691737,
     $-0.067615691737, 0.000000000000, 2.635231383474,-2.567615691737,
     $ 0.000000000000, 0.067615691737,-2.567615691737, 2.500000000000/
*
* QUARTIC LAGRANGIAN POLYNOMIALS.
      DATA EQ/
     $  0.000000000000, 0.750000000000,-1.500000000000,-7.000000000000,
     $ 14.000000000000, 0.000000000000,-2.673169155391, 8.166666666667,
     $ 10.692676621564,-32.666666666667, 1.000000000000, 0.000000000000,
     $-13.333333333333, 0.000000000000,37.333333333333, 0.000000000000,
     $  2.673169155391,8.166666666667,-10.692676621564,-32.666666666667,
     $  0.000000000000,-0.750000000000,-1.500000000000, 7.000000000000,
     $ 14.000000000000/
      DATA TQ/
     $ 0.050000000000, 0.272222222222, 0.355555555556, 0.272222222222,
     $ 0.050000000000/
      DATA VQ/
     $-1.000000000000, 0.00000000000, 0.000000000000, 0.00000000000,
     $ 1.000000000000, 1.55884572681,-0.943005439677,-1.23168057427,
     $-0.943005439677, 1.55884572681,-1.565247584250, 2.39095517873,
     $ 0.000000000000,-2.39095517873, 1.565247584250, 1.058300524426,
     $-2.46936789032 , 2.8221347318 ,-2.46936789032 , 1.058300524426/
*     CYLINDRICAL GAUSS-LOBATTO (VARIATIONAL COLLOCATION METHOD)
*     MATRICES:
      DATA TSQ/
     $-0.025000000000,-0.089105638513, 0.000000000000, 0.089105638513,
     $ 0.025000000000/
      DATA QSQ/
     $-3.000000000000, 3.237234826568,-0.266666666667, 0.029431840099,
     $ 0.000000000000, 3.237234826568,-4.158263130608, 0.950460144139,
     $ 0.000000000000,-0.029431840099,-0.266666666667, 0.950460144139,
     $ 0.000000000000,-0.950460144139, 0.266666666667, 0.029431840099,
     $ 0.000000000000,-0.950460144139, 4.158263130608,-3.237234826568,
     $ 0.000000000000,-0.029431840099, 0.266666666667,-3.237234826568,
     $ 3.000000000000/
*
      LC=IELEM+1
      IF((IELEM.EQ.1).AND.(ICOL.EQ.1)) THEN
*        LL2
         HTYPE='LINEAR LAGRANGIAN POLYNOMIALS'
         DO 90 I=1,LC
         T(I)=TL(I)
         TS(I)=TSL(I)
         DO 90 J=1,LC
         R((J-1)*LC+I)=RL(I,J)
         RS((J-1)*LC+I)=RSL(I,J)
         QS((J-1)*LC+I)=QSL(I,J)
   90    E((J-1)*LC+I)=EL(I,J)
         V(1)=-1.0
         V(2)=1.0
         DO 91 I=1,6
         DO 91 J=1,6
         RH(I,J)=RHA6(I,J)
   91    QH(I,J)=QHA6(I,J)
         DO 92 I=1,3
         DO 92 J=1,3
         RT(I,J)=RTA(I,J)*SQRT(3.0)/24.0
   92    QT(I,J)=QTA(I,J)/SQRT(3.0)
      ELSE IF((IELEM.EQ.1).AND.(ICOL.EQ.2)) THEN
*        LL2$
         HTYPE='FINITE DIFFERENCES'
         DO 110 I=1,LC
         T(I)=TL(I)
         TS(I)=TSL1(I)
         DO 100 J=1,LC
         R((J-1)*LC+I)=0.0
         RS((J-1)*LC+I)=0.0
         QS((J-1)*LC+I)=QSL(I,J)
  100    E((J-1)*LC+I)=EL(I,J)
         R((I-1)*LC+I)=TL(I)
  110    RS((I-1)*LC+I)=TSL1(I)
         V(1)=-1.0
         V(2)=1.0
         DO 111 I=1,6
         DO 111 J=1,6
         RH(I,J)=RHL6(I,J)*SQRT(3.0)/4.0
  111    QH(I,J)=QHL6(I,J)*SQRT(3.0)
         DO 112 I=1,3
         DO 112 J=1,3
         RT(I,J)=RTL(I,J)*SQRT(3.0)/12.0
  112    QT(I,J)=QTL(I,J)/SQRT(3.0)
      ELSE IF((IELEM.EQ.1).AND.(ICOL.EQ.3)) THEN
*        LL2#
         HTYPE='SUPERCONVERGENT LINEAR POLYNOMIALS'
         DO 120 I=1,LC
         T(I)=TL(I)
         TS(I)=TSL2(I)
         DO 120 J=1,LC
         R((J-1)*LC+I)=RL2(I,J)
         RS((J-1)*LC+I)=RSL2(I,J)
         QS((J-1)*LC+I)=QSL(I,J)
  120    E((J-1)*LC+I)=EL(I,J)
         V(1)=-1.0
         V(2)=1.0
      ELSE IF((IELEM.EQ.2).AND.(ICOL.EQ.1)) THEN
*        PL3
         HTYPE='PARABOLIC LAGRANGIAN POLYNOMIALS'
         DO 130 I=1,LC
         T(I)=TP(I)
         TS(I)=TSP(I)
         DO 125 J=1,LC-1
  125    V((J-1)*LC+I)=VP(I,J)
         DO 130 J=1,LC
         R((J-1)*LC+I)=RP(I,J)
         RS((J-1)*LC+I)=RSP(I,J)
         QS((J-1)*LC+I)=QSP(I,J)
  130    E((J-1)*LC+I)=EP(I,J)
      ELSE IF((IELEM.EQ.2).AND.(ICOL.EQ.2)) THEN
*        PL3$
         HTYPE='PARABOLIC COLLOCATION METHOD'
         DO 150 I=1,LC
         T(I)=TP1(I)
         TS(I)=TSP1(I)
         DO 135 J=1,LC-1
  135    V((J-1)*LC+I)=VP1(I,J)
         DO 140 J=1,LC
         R((J-1)*LC+I)=0.0
         RS((J-1)*LC+I)=0.0
         QS((J-1)*LC+I)=QSP1(I,J)
  140    E((J-1)*LC+I)=EP1(I,J)
         R((I-1)*LC+I)=TP1(I)
  150    RS((I-1)*LC+I)=TSP1(I)
      ELSE IF((IELEM.EQ.2).AND.(ICOL.EQ.3)) THEN
*        PL3#
         HTYPE='PARABOLIC SUPERCONVERGENT POLYNOMIALS'
         DO 160 I=1,LC
         T(I)=TP2(I)
         TS(I)=TSP2(I)
         DO 155 J=1,LC-1
  155    V((J-1)*LC+I)=VP2(I,J)
         DO 160 J=1,LC
         R((J-1)*LC+I)=RP2(I,J)
         RS((J-1)*LC+I)=RSP2(I,J)
         QS((J-1)*LC+I)=QSP2(I,J)
  160    E((J-1)*LC+I)=EP2(I,J)
      ELSE IF((IELEM.EQ.3).AND.(ICOL.EQ.1)) THEN
*        CL4
         HTYPE='CUBIC LAGRANGIAN POLYNOMIALS'
         DO 170 I=1,LC
         T(I)=TC(I)
         TS(I)=TSC(I)
         DO 165 J=1,LC-1
  165    V((J-1)*LC+I)=VC(I,J)
         DO 170 J=1,LC
         R((J-1)*LC+I)=RC(I,J)
         RS((J-1)*LC+I)=RSC(I,J)
         QS((J-1)*LC+I)=QSC(I,J)
  170    E((J-1)*LC+I)=EC(I,J)
      ELSE IF((IELEM.EQ.3).AND.(ICOL.EQ.2)) THEN
*        CL4$
         HTYPE='CUBIC COLLOCATION METHOD'
         DO 190 I=1,LC
         T(I)=TC1(I)
         TS(I)=TSC1(I)
         DO 175 J=1,LC-1
  175    V((J-1)*LC+I)=VC1(I,J)
         DO 180 J=1,LC
         R((J-1)*LC+I)=0.0
         RS((J-1)*LC+I)=0.0
         QS((J-1)*LC+I)=QSC1(I,J)
  180    E((J-1)*LC+I)=EC1(I,J)
         R((I-1)*LC+I)=TC1(I)
  190    RS((I-1)*LC+I)=TSC1(I)
      ELSE IF((IELEM.EQ.3).AND.(ICOL.EQ.3)) THEN
*        CL4#
         HTYPE='SUPERCONVERGENT CUBIC POLYNOMIALS'
         DO 200 I=1,LC
         T(I)=TC2(I)
         TS(I)=TSC2(I)
         DO 195 J=1,LC-1
  195    V((J-1)*LC+I)=VC2(I,J)
         DO 200 J=1,LC
         R((J-1)*LC+I)=RC2(I,J)
         RS((J-1)*LC+I)=RSC2(I,J)
         QS((J-1)*LC+I)=QSC2(I,J)
  200    E((J-1)*LC+I)=EC2(I,J)
      ELSE IF((IELEM.EQ.4).AND.(ICOL.EQ.2)) THEN
*        QL5$
         HTYPE='QUARTIC COLLOCATION METHOD'
         DO 220 I=1,LC
         T(I)=TQ(I)
         TS(I)=TSQ(I)
         DO 205 J=1,LC-1
  205    V((J-1)*LC+I)=VQ(I,J)
         DO 210 J=1,LC
         R((J-1)*LC+I)=0.0
         RS((J-1)*LC+I)=0.0
         QS((J-1)*LC+I)=QSQ(I,J)
  210    E((J-1)*LC+I)=EQ(I,J)
         R((I-1)*LC+I)=TQ(I)
  220    RS((I-1)*LC+I)=TSQ(I)
      ELSE
         CALL XABORT('BIVCOL: TYPE OF FINITE ELEMENT NOT AVAILABLE.')
      ENDIF
*
* COMPUTE THE CARTESIAN STIFFNESS MATRIX FROM THE TENSORIAL PRODUCT OF
* TWO NODAL COUPLING MATRICES.
      DO 230 I=1,LC
      DO 230 J=1,LC
      Q((J-1)*LC+I)=0.0
      DO 230 K=1,LC-1
  230 Q((J-1)*LC+I)=Q((J-1)*LC+I)+V((K-1)*LC+I)*V((K-1)*LC+J)
      IF(IMPX.GT.0) WRITE (6,'(/9H BIVCOL: ,A40)') HTYPE
*
* SAVE THE UNIT MATRICES ON LCM.
      CALL LCMSIX(IPTRK,'BIVCOL',1)
      CALL LCMPUT(IPTRK,'T',LC,2,T)
      CALL LCMPUT(IPTRK,'TS',LC,2,TS)
      CALL LCMPUT(IPTRK,'R',LC*LC,2,R)
      CALL LCMPUT(IPTRK,'RS',LC*LC,2,RS)
      CALL LCMPUT(IPTRK,'Q',LC*LC,2,Q)
      CALL LCMPUT(IPTRK,'QS',LC*LC,2,QS)
      CALL LCMPUT(IPTRK,'V',LC*(LC-1),2,V)
      CALL LCMPUT(IPTRK,'E',LC*LC,2,E)
      IF((IELEM.EQ.1).AND.(ICOL.LE.2)) THEN
         CALL LCMPUT(IPTRK,'RH',36,2,RH)
         CALL LCMPUT(IPTRK,'QH',36,2,QH)
         CALL LCMPUT(IPTRK,'RT',9,2,RT)
         CALL LCMPUT(IPTRK,'QT',9,2,QT)
      ENDIF
      CALL LCMSIX(IPTRK,' ',2)
      RETURN
      END
