*DECK BIVF2D
      SUBROUTINE BIVF2D (DIFF0,SIGMA,CYLIND,NREG,NUN,LL4,XX,YY,DD,MAT,
     1 KN,QFR,BFR,VOL,IDL,MU,SOURCE,LC,T,TS,R,RS,Q,QS,FUNKNO,A11)
C
C-----------------------------------------------------------------------
C
C  MESH CORNER FINITE DIFFERENCE OR FINITE ELEMENT FLUX CALCULATION IN
C  A SINGLE ENERGY GROUP. CARTESIAN GEOMETRY.
C
C  INPUT PARAMETERS:
C    DIFF0  : DIFFUSION COEFFICIENTS. DIMENSION DIFF0(NBMIX)
C    SIGMA  : REMOVAL MACROSCOPIC X-SECTIONS. DIMENSION SIGMA(NBMIX)
C             WHERE NBMIX IS THE NUMBER OF MIXTURES.
C    CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
C    NREG   : NUMBER OF ELEMENTS IN BIVAC.
C    NUN    : DIMENSION OF VECTOR FUNKNO.
C    LL4    : ORDER OF THE MATRIX A11.
C    XX     : X-DIRECTED MESH SPACINGS. DIMENSION XX(NREG)
C    YY     : Y-DIRECTED MESH SPACINGS. DIMENSION YY(NREG)
C    DD     : USED WITH CYLINDRICAL GEOMETRY. DIMENSION DD(NREG)
C    MAT    : MIXTURE INDEX PER REGION. DIMENSION MAT(NREG)
C    KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(LLC NREG)
C             WHERE LL IS THE NUMBER OF UNKNOWN PER ELEMENT.
C    QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. QFR(4C NREG)
C    BFR    : ELEMENT-ORDERED SURFACE FRACTIONS. BFR(4C NREG)
C    VOL    : VOLUME OF REGIONS. DIMENSION VOL(NREG)
C    IDL    : POSITION OF INTEGRATED FLUXES INTO UNKNOWN VECTOR.
C             DIMENSION IDL(NREG)
C    MU     : INDICES USED WITH COMPRESSED DIAGONAL STORAGE MODE MATRIX
C             A11. DIMENSION MU(LL4)
C    SOURCE : FISSION AND DIFFUSION SOURCES. DIMENSION SOURCE(LL4)
C    LC     : NUMBER OF POLYNOMIALS IN A COMPLETE 1-D BASIS.
C    T      : CARTESIAN LINEAR PRODUCT VECTOR. DIMENSION T(LC)
C    TS     : CYLINDRICAL LINEAR PRODUCT VECTOR. DIMENSION TS(LC)
C    R      : CARTESIAN MASS MATRIX. DIMENSION R(LC,LC)
C    RS     : CYLINDRICAL MASS MATRIX. DIMENSION RS(LC,LC)
C    Q      : CARTESIAN STIFFNESS MATRIX. DIMENSION Q(LC,LC)
C    QS     : CYLINDRICAL STIFFNESS MATRIX. DIMENSION QS(LC,LC)
C
C  OUTPUT PARAMETER:
C    FUNKNO : FIRST LL4 VALUES CONTAINS THE FINITE ELEMENT UNKNOWNS;
C             NEXT NREG VALUES CONTAINS ELEMENT AVERAGED FLUXES.
C             SURFACE-AVERAGED FLUX IN POSITION FUNKNO(NUN)
C             DIMENSION FUNKNO(NUN)
C
C  SCRATCH STORAGE:
C    A11    : DIMENSION A11(MU(LL4))
C
C------------------------------------------ AUTHOR: A. HEBERT 1977 -----
C
      IMPLICIT NONE
C---
C LOCAL VARIABLES
C----
      INTEGER     LC,LL,I,I1,I2,J,J2,LL4,NUM1,NUM2,K,J1,NREG,L,IND1,
     &            KEY1,IND2,KEY,IC,NUN
      INTEGER     MAT(*),KN(*),IDL(*),MU(*),IJ1(25),IJ2(25),ISR(4,5)
      REAL        DX,DY,QQ,RR,SS,BFR1,CRZ,QFR1
      REAL        DIFF0(*),SIGMA(*),XX(*),YY(*),DD(*),QFR(*),
     1            BFR(*),VOL(*),SOURCE(*),T(*),TS(*),R(LC,*),RS(LC,*),
     2            Q(LC,*),QS(LC,*),FUNKNO(*),A11(*),
     3            Q2DP1(25,25),Q2DP2(25,25),R2DP(25,25),Q2DC1(25,25),
     4            Q2DC2(25,25),R2DC(25,25)
      LOGICAL     CYLIND
C
C COMPUTE VECTORS IJ1, IJ2 AND MATRIX ISR.
C
      LL=LC*LC
      DO 10 I=1,LL
      IJ1(I)=1+MOD(I-1,LC)
   10 IJ2(I)=1+(I-IJ1(I))/LC
      DO 20 I=1,LC
      ISR(1,I)=(I-1)*LC+1
      ISR(2,I)=I*LC
      ISR(3,I)=I
   20 ISR(4,I)=LL-LC+I
*
* COMPUTE THE CARTESIAN 2-D MASS AND STIFFNESS MATRICES FROM TENSORIAL
* PRODUCTS OF 1-D MATRICES.
      DO 30 I=1,LL
      I1=IJ1(I)
      I2=IJ2(I)
      DO 30 J=1,LL
      J1=IJ1(J)
      J2=IJ2(J)
      Q2DP1(I,J)=Q(I1,J1)*R(I2,J2)
      Q2DP2(I,J)=R(I1,J1)*Q(I2,J2)
      R2DP(I,J)=R(I1,J1)*R(I2,J2)
      Q2DC1(I,J)=QS(I1,J1)*R(I2,J2)
      Q2DC2(I,J)=RS(I1,J1)*Q(I2,J2)
   30 R2DC(I,J)=RS(I1,J1)*R(I2,J2)
*
* ASSEMBLY OF THE MAIN COEFFICIENT MATRIX.
      DO 40 I=1,MU(LL4)
   40 A11(I)=0.0
      NUM1=0
      NUM2=0
      DO 110 K=1,NREG
      L=MAT(K)
      IF(L.EQ.0) GO TO 110
      IF(VOL(K).EQ.0.0) GO TO 100
      DX=XX(K)
      DY=YY(K)
      DO 60 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 60
      KEY1=MU(IND1)-IND1
      DO 50 J=1,LL
      IND2=KN(NUM1+J)
      IF((IND2.EQ.0).OR.(IND2.GT.IND1)) GO TO 50
      IF(CYLIND) THEN
         QQ=(Q2DP1(I,J)+Q2DC1(I,J)*DX/DD(K))/(DX*DX)
     1     +(Q2DP2(I,J)+Q2DC2(I,J)*DX/DD(K))/(DY*DY)
         RR=(R2DP(I,J)+R2DC(I,J)*DX/DD(K))
      ELSE
         QQ=Q2DP1(I,J)/(DX*DX)+Q2DP2(I,J)/(DY*DY)
         RR=R2DP(I,J)
      ENDIF
      IF((QQ.EQ.0.0).AND.(RR.EQ.0.0)) GO TO 50
      KEY=KEY1+IND2
      A11(KEY)=A11(KEY)+(QQ*DIFF0(L)+RR*SIGMA(L))*VOL(K)
   50 CONTINUE
   60 CONTINUE
      DO 90 IC=1,4
      QFR1=QFR(NUM2+IC)
      IF(QFR1.EQ.0.0) GO TO 90
      DO 80 I1=1,LC
      IND1=KN(NUM1+ISR(IC,I1))
      IF(IND1.EQ.0) GO TO 80
      KEY1=MU(IND1)-IND1
      DO 70 J1=1,LC
      IND2=KN(NUM1+ISR(IC,J1))
      IF((IND2.EQ.0).OR.(IND2.GT.IND1)) GO TO 70
      IF(CYLIND) THEN
         IF(IC.EQ.1) THEN
            CRZ=-0.5*R(I1,J1)
         ELSE IF(IC.EQ.2) THEN
            CRZ=0.5*R(I1,J1)
         ELSE IF(IC.EQ.3) THEN
            CRZ=RS(I1,J1)
         ELSE IF(IC.EQ.4) THEN
            CRZ=RS(I1,J1)
         ENDIF
         RR=(R(I1,J1)+CRZ*DX/DD(K))
      ELSE
         RR=R(I1,J1)
      ENDIF
      IF(RR.EQ.0.0) GO TO 70
      KEY=KEY1+IND2
      A11(KEY)=A11(KEY)+RR*QFR1
   70 CONTINUE
   80 CONTINUE
   90 CONTINUE
  100 NUM1=NUM1+LL
      NUM2=NUM2+4
  110 CONTINUE
*
* L-D-L(T) FACTORIZATION AND RESOLUTION.
      CALL ALLDLF(LL4,A11,MU)
      DO 120 I=1,LL4
  120 FUNKNO(I)=SOURCE(I)
      CALL ALLDLS(LL4,MU,A11,FUNKNO)
*
* CALCULATION OF ELEMENT-AVERAGED AND SURFACE-AVERAGED FLUXES.
      FUNKNO(NUN)=0.0
      NUM1=0
      NUM2=0
      DO 170 K=1,NREG
      IF(MAT(K).EQ.0) GO TO 170
      FUNKNO(IDL(K))=0.0
      IF(VOL(K).EQ.0.0) GO TO 160
*
      DO 130 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 130
      IF(CYLIND) THEN
         SS=(T(IJ1(I))+TS(IJ1(I))*XX(K)/DD(K))*T(IJ2(I))
      ELSE
         SS=T(IJ1(I))*T(IJ2(I))
      ENDIF
      FUNKNO(IDL(K))=FUNKNO(IDL(K))+SS*FUNKNO(IND1)
  130 CONTINUE
*
      DO 150 IC=1,4
      BFR1=BFR(NUM2+IC)
      IF(BFR1.EQ.0.0) GO TO 150
      DO 140 I1=1,LC
      IND1=KN(NUM1+ISR(IC,I1))
      IF(IND1.GT.0) FUNKNO(NUN)=FUNKNO(NUN)+T(I1)*FUNKNO(IND1)*BFR1
  140 CONTINUE
  150 CONTINUE
*
  160 NUM1=NUM1+LL
      NUM2=NUM2+4
  170 CONTINUE
      RETURN
      END
