*DECK BIVFSO
      SUBROUTINE BIVFSO (IEX,NGCOND,IGR,NMERGE,FLXMER,SUNMER,CYLIND,
     1 NREG,NUN,XX,DD,MAT,KN,QFR,VOL,MERG,COUR,FUNKNO,LC,T,TS,R,RS,
     2 SOURCE,ZGAR1,ZGAR2)
C
C-----------------------------------------------------------------------
C
C  SOURCE TERM CALCULATION FOR FINITE ELEMENT OR MESH CORNER FINITE
C  DIFFERENCES IN CARTESIAN GEOMETRY.
C
C  INPUT PARAMETERS:
C    IEX    : ITERATION NUMBER.
C    NGCOND : NUMBER OF GROUPS CONDENSED.
C    IGR    : ENERGY GROUP INDEX.
C    NMERGE : NUMBER OF MIXTURES.
C    FLXMER : FLUX ESTIMATE PER MIXTURE. DIMENSION FLXMER(NMERGE,NGCOND)
C    SUNMER : INCOMING SOURCE (SCATTERING+FISSION) X-SECTIONS.
C             DIMENSION SUNMER(NMERGE,NGCOND)
C    CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
C    NREG   : NUMBER OF REGIONS IN BIVAC.
C    NUN    : NUMBER OF UNKNOWN PER GROUP IN BIVAC.
C    XX     : X-DIRECTED MESH SPACINGS. DIMENSION XX(NREG)
C    DD     : USED WITH CYLINDRICAL GEOMETRY. DIMENSION DD(NREG)
C    MAT    : MIXTURE INDEX PER REGION. DIMENSION MAT(NREG)
C    KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(LLC NREG)
C             WHERE LL IS THE NUMBER OF UNKNOWN PER ELEMENT.
C    QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS.  QFR(4C NREG)
C    VOL    : VOLUME OF REGIONS. DIMENSION VOL(NREG)
C    MERG   : INDEX OF MERGED REGIONS. DIMENSION MERG(NREG)
C    COUR   : FOUR TIMES THE INCOMING CURRENT PER UNIT SURFACE.
C    FUNKNO : PREVIOUSLY CALCULATED FLUXES. DIMENSION FUNKNO(NUN,NGCOND)
C    LC     : ORDER OF THE UNIT MATRICES.
C    T,TS,R,RS : UNIT MATRICES. T(LC),TS(LC),R(LC,LC),RS(LC,LC)
C
C  OUTPUT PARAMETER:
C    SOURCE : FISSION AND DIFFUSION SOURCES. DIMENSION SOURCE(NUN)
C
C  SCRATCH STORAGE:
C    ZGAR1  : DIMENSION ZGAR1(NBMIX) WHERE NBMIX=MAX(MAT(I))
C    ZGAR2  : DIMENSION ZGAR2(NBMIX)
C
C------------------------------------------ AUTHOR: A. HEBERT 1977 -----
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      REAL       DPI
      PARAMETER (DPI=6.283185307)
C----
C INTERFACE AND LOCAL VARIABLES
C----
      INTEGER LL,LC,I,NMERGE,K,NUN,J,NBMIX,NUM1,NUM2,IC,I1,
     &        NREG,IND1,IGR2,NGCOND,IEX,IGR,NL,L,I2,IND2,J1
      INTEGER MAT(*),KN(*),MERG(*),IJ1(25),IJ2(25),ISR(4,5)
C
      REAL    SS,COUR,QFR1,CRZ,RR,PV
      REAL    FLXMER(NMERGE,*),SUNMER(NMERGE,*),XX(*),DD(*),
     1        QFR(*),VOL(*),FUNKNO(NUN,*),T(*),TS(*),R(LC,*),
     2        RS(LC,*),SOURCE(*),ZGAR1(*),ZGAR2(*)
      LOGICAL CYLIND
C
C COMPUTE VECTORS IJ1, IJ2 AND MATRIX ISR.
C
      LL=LC*LC
      DO 10 I=1,LL
      IJ1(I)=1+MOD(I-1,LC)
   10 IJ2(I)=1+(I-IJ1(I))/LC
      DO 20 I=1,LC
      ISR(1,I)=(I-1)*LC+1
      ISR(2,I)=I*LC
      ISR(3,I)=I
   20 ISR(4,I)=LL-LC+I
*
      NBMIX=0
      DO 30 I=1,NREG
   30 NBMIX=MAX(NBMIX,MAT(I))
      DO 40 I=1,NUN
   40 SOURCE(I)=0.0
*
* INCOMING CURRENT SOURCE.
      IF(COUR.NE.0.0) THEN
         NUM1=0
         NUM2=0
         DO 80 K=1,NREG
         IF(MAT(K).EQ.0) GO TO 80
         IF(VOL(K).EQ.0.0) GO TO 70
         DO 60 IC=1,4
         QFR1=QFR(NUM2+IC)
         IF(QFR1.EQ.0.0) GO TO 60
         DO 50 I1=1,LC
         IND1=KN(NUM1+ISR(IC,I1))
         IF(IND1.EQ.0) GO TO 50
         IF(CYLIND) THEN
            IF(IC.EQ.1) THEN
               CRZ=-0.5*T(I1)
            ELSE IF(IC.EQ.2) THEN
               CRZ=0.5*T(I1)
            ELSE IF(IC.EQ.3) THEN
               CRZ=TS(I1)
            ELSE IF(IC.EQ.4) THEN
               CRZ=TS(I1)
            ENDIF
            RR=DPI*(XX(K)*CRZ+DD(K)*T(I1))
         ELSE
            RR=T(I1)
         ENDIF
         SOURCE(IND1)=SOURCE(IND1)+COUR*QFR1*RR
   50    CONTINUE
   60    CONTINUE
   70    NUM1=NUM1+LL
         NUM2=NUM2+4
   80    CONTINUE
      ENDIF
*
* DIFFUSION AND FISSION SOURCE.
      IGR2=NGCOND
      IF(IEX.EQ.1) IGR2=IGR-1
      DO 160 NL=1,IGR2
      DO 90 L=1,NBMIX
      ZGAR1(L)=0.0
   90 ZGAR2(L)=0.0
      NUM1=0
      DO 120 K=1,NREG
      L=MAT(K)
      IF(L.EQ.0) GO TO 120
      IF(VOL(K).EQ.0.0) GO TO 110
      ZGAR1(L)=ZGAR1(L)+VOL(K)
      DO 100 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 100
      SS=T(IJ1(I))*T(IJ2(I))*VOL(K)
      IF(CYLIND) THEN
         SS=DPI*(T(IJ1(I))*DD(K)+TS(IJ1(I))*XX(K))*T(IJ2(I))*VOL(K)
      ENDIF
      ZGAR2(L)=ZGAR2(L)+SS*FUNKNO(IND1,NL)
  100 CONTINUE
  110 NUM1=NUM1+LL
  120 CONTINUE
      NUM1=0
      DO 160 K=1,NREG
      L=MAT(K)
      IF(L.EQ.0) GO TO 160
      IF(VOL(K).EQ.0.0) GO TO 150
      PV=SUNMER(MERG(K),NL)*FLXMER(MERG(K),NL)*ZGAR1(L)/ZGAR2(L)
      DO 140 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 140
      I1=IJ1(I)
      I2=IJ2(I)
      DO 130 J=1,LL
      IND2=KN(NUM1+J)
      IF(IND2.EQ.0) GO TO 130
      IF(CYLIND) THEN
         J1=IJ1(J)
         RR=DPI*(R(I1,J1)*DD(K)+RS(I1,J1)*XX(K))*R(I2,IJ2(J))
      ELSE
         RR=R(I1,IJ1(J))*R(I2,IJ2(J))
      ENDIF
      SOURCE(IND1)=SOURCE(IND1)+RR*FUNKNO(IND2,NL)*VOL(K)*PV
  130 CONTINUE
  140 CONTINUE
  150 NUM1=NUM1+LL
  160 CONTINUE
      IF(IEX.GT.1) RETURN
      NUM1=0
      DO 200 K=1,NREG
      IF(MAT(K).EQ.0) GO TO 200
      IF(VOL(K).EQ.0.0) GO TO 190
      PV=0.0
      DO 170 NL=IGR,NGCOND
  170 PV=PV+SUNMER(MERG(K),NL)*FLXMER(MERG(K),NL)
      DO 180 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 180
      SS=T(IJ1(I))*T(IJ2(I))*VOL(K)
      IF(CYLIND) THEN
         SS=DPI*(T(IJ1(I))*DD(K)+TS(IJ1(I))*XX(K))*T(IJ2(I))*VOL(K)
      ENDIF
      SOURCE(IND1)=SOURCE(IND1)+SS*PV
  180 CONTINUE
  190 NUM1=NUM1+LL
  200 CONTINUE
      RETURN
      END
