*DECK BIVG2D
      SUBROUTINE BIVG2D (DIFF0,SIGMA,CYLIND,IELEM,ICOL,NREG,NUN,LL4,XX,
     1 YY,DD,MAT,KN,QFR,BFR,VOL,MU,SOURCE,LC,R,V,FUNKNO,A11)
C
C-----------------------------------------------------------------------
C
C  MIXED-DUAL FINITE ELEMENT FLUX CALCULATION IN A SINGLE ENERGY GROUP.
C  CARTESIAN GEOMETRY.
C
C  INPUT PARAMETERS:
C    DIFF0  : DIFFUSION COEFFICIENTS. DIMENSION DIFF0(NBMIX)
C    SIGMA  : REMOVAL MACROSCOPIC X-SECTIONS. DIMENSION SIGMA(NBMIX)
C             WHERE NBMIX IS THE NUMBER OF MIXTURES.
C    CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
C    IELEM  : DEGREE OF THE LAGRANGIAN FINITE ELEMENTS. =1 (LINEAR);
C             =2 (PARABOLIC); =3 (CUBIC); =4 (QUARTIC).
C    ICOL   : TYPE OF QUADRATURE. =1 (ANALYTICAL INTEGRATION);
C             =2 (GAUSS-LOBATTO); =3 (GAUSS-LEGENDRE).
C    NREG   : NUMBER OF ELEMENTS IN BIVAC.
C    NUN    : DIMENSION OF VECTOR FUNKNO.
C    LL4    : NUMBER OF UNKNOWNS PER GROUP IN BIVAC.
C    XX     : X-DIRECTED MESH SPACINGS. DIMENSION XX(NREG)
C    YY     : Y-DIRECTED MESH SPACINGS. DIMENSION YY(NREG)
C    DD     : USED WITH CYLINDRICAL GEOMETRY. DIMENSION DD(NREG)
C    MAT    : MIXTURE INDEX PER REGION. DIMENSION MAT(NREG)
C    KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(5C NREG)
C    QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. QFR(4C NREG)
C    BFR    : ELEMENT-ORDERED SURFACE FRACTIONS. DIMENSION BFR(4C NREG)
C    VOL    : VOLUME OF REGIONS. DIMENSION VOL(NREG)
C    MU     : INDICES USED WITH COMPRESSED DIAGONAL STORAGE MODE MATRIX
C             A11. DIMENSION MU(LL4)
C    SOURCE : FISSION AND DIFFUSION SOURCES. DIMENSION SOURCE(LL4)
C    LC     : NUMBER OF POLYNOMIALS IN A COMPLETE 1D BASIS.
C    R      : CARTESIAN MASS MATRIX. DIMENSION R(LC,LC)
C    V      : NODAL COUPLING MATRIX. DIMENSION V(LC,LC-1)
C
C  OUTPUT PARAMETER:
C    FUNKNO : NEUTRON FLUXES. SURFACE-AVERAGED FLUX ARE IN POSITION
C             FUNKNO(NUN). DIMENSION FUNKNO(NUN)
C
C  SCRATCH STORAGE:
C    A11    : DIMENSION A11(MU(NREG))
C
C------------------------------------- AUTHOR: A. HEBERT ; 11/08/92 ----
C
      IMPLICIT NONE
C----
C INTERFACE AND LOCAL VARIABLES
C----
      INTEGER LC,IELEM,ICOL,I,I0,J0,K0,NUM1,NUM2,LL4,K,NREG,
     &        L,INX1,INX2,INY1,INY2,JND1,KEY,KND1,NUN,IND1
      INTEGER MAT(*),KN(*),MU(*)
      REAL    VOL0,DX,DY,DIN,DOT,SG
      REAL    DIFF0(*),SIGMA(*),XX(*),YY(*),DD(*),
     1        QFR(*),BFR(*),VOL(*),SOURCE(*),FUNKNO(*),A11(*),R(LC,*),
     2        V(LC,*),QQ(5,5)
      LOGICAL CYLIND
C
C----
C
      IF((CYLIND).AND.((IELEM.GT.1).OR.(ICOL.NE.2)))
     1 CALL XABORT('BIVG2D: TYPE OF DISCRETIZATION NOT IMPLEMENTED.')
      DO 10 I0=1,IELEM
      DO 10 J0=1,IELEM
      QQ(I0,J0)=0.0
      DO 10 K0=2,IELEM
   10 QQ(I0,J0)=QQ(I0,J0)+V(K0,I0)*V(K0,J0)/R(K0,K0)
*
* ASSEMBLY OF THE MAIN COEFFICIENT MATRIX.
      DO 15 I=1,MU(LL4)
   15 A11(I)=0.0
      NUM1=0
      NUM2=0
      DO 80 K=1,NREG
      L=MAT(K)
      IF(L.EQ.0) GO TO 80
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 70
      DX=XX(K)
      DY=YY(K)
      IF(CYLIND) THEN
         DIN=1.0-0.5*DX/DD(K)
         DOT=1.0+0.5*DX/DD(K)
      ELSE
         DIN=1.0
         DOT=1.0
      ENDIF
*
      DO 60 I0=1,IELEM
      INX1=ABS(KN(NUM1+2))+I0-1
      INX2=ABS(KN(NUM1+3))+I0-1
      INY1=ABS(KN(NUM1+4))+I0-1
      INY2=ABS(KN(NUM1+5))+I0-1
      DO 50 J0=1,IELEM
      JND1=KN(NUM1+1)+(I0-1)*IELEM+J0-1
      KEY=MU(JND1)
      A11(KEY)=A11(KEY)+VOL0*SIGMA(L)
      DO 20 K0=1,J0
      IF(QQ(J0,K0).EQ.0.0) GO TO 20
      KND1=KN(NUM1+1)+(I0-1)*IELEM+K0-1
      KEY=MU(JND1)-JND1+KND1
      A11(KEY)=A11(KEY)+VOL0*QQ(J0,K0)*DIFF0(L)/(DX*DX)
   20 CONTINUE
      IF(KN(NUM1+2).NE.0) THEN
         IF(JND1.GT.INX1) KEY=MU(JND1)-JND1+INX1
         IF(JND1.LT.INX1) KEY=MU(INX1)-INX1+JND1
         SG=REAL(SIGN(1,KN(NUM1+2)))
         A11(KEY)=A11(KEY)+SG*(VOL0/DX)*DIN*V(1,J0)
      ENDIF
      IF(KN(NUM1+3).NE.0) THEN
         IF(INX2.GT.JND1) KEY=MU(INX2)-INX2+JND1
         IF(INX2.LT.JND1) KEY=MU(JND1)-JND1+INX2
         SG=REAL(SIGN(1,KN(NUM1+3)))
         A11(KEY)=A11(KEY)+SG*(VOL0/DX)*DOT*V(IELEM+1,J0)
      ENDIF
      JND1=KN(NUM1+1)+(J0-1)*IELEM+I0-1
      DO 30 K0=1,J0
      IF(QQ(J0,K0).EQ.0.0) GO TO 30
      KND1=KN(NUM1+1)+(K0-1)*IELEM+I0-1
      KEY=MU(JND1)-JND1+KND1
      A11(KEY)=A11(KEY)+VOL0*QQ(J0,K0)*DIFF0(L)/(DY*DY)
   30 CONTINUE
      IF(KN(NUM1+4).NE.0) THEN
         IF(JND1.GT.INY1) KEY=MU(JND1)-JND1+INY1
         IF(JND1.LT.INY1) KEY=MU(INY1)-INY1+JND1
         SG=REAL(SIGN(1,KN(NUM1+4)))
         A11(KEY)=A11(KEY)+SG*(VOL0/DY)*V(1,J0)
      ENDIF
      IF(KN(NUM1+5).NE.0) THEN
         IF(INY2.GT.JND1) KEY=MU(INY2)-INY2+JND1
         IF(INY2.LT.JND1) KEY=MU(JND1)-JND1+INY2
         SG=REAL(SIGN(1,KN(NUM1+5)))
         A11(KEY)=A11(KEY)+SG*(VOL0/DY)*V(IELEM+1,J0)
      ENDIF
   50 CONTINUE
      IF(KN(NUM1+2).NE.0) THEN
         KEY=MU(INX1)
         A11(KEY)=A11(KEY)-DIN*(VOL0*R(1,1)/DIFF0(L)+QFR(NUM2+1))
      ENDIF
      IF(KN(NUM1+3).NE.0) THEN
         KEY=MU(INX2)
         A11(KEY)=A11(KEY)-DOT*(VOL0*R(IELEM+1,IELEM+1)/DIFF0(L)
     1   +QFR(NUM2+2))
      ENDIF
      IF(KN(NUM1+4).NE.0) THEN
         KEY=MU(INY1)
         A11(KEY)=A11(KEY)-VOL0*R(1,1)/DIFF0(L)-QFR(NUM2+3)
      ENDIF
      IF(KN(NUM1+5).NE.0) THEN
         KEY=MU(INY2)
         A11(KEY)=A11(KEY)-VOL0*R(IELEM+1,IELEM+1)/DIFF0(L)-QFR(NUM2+4)
      ENDIF
      IF(ICOL.NE.2) THEN
         IF((KN(NUM1+2).NE.0).AND.(KN(NUM1+3).NE.0)) THEN
            IF(INX2.GT.INX1) KEY=MU(INX2)-INX2+INX1
            IF(INX2.LE.INX1) KEY=MU(INX1)-INX1+INX2
            SG=REAL(SIGN(1,KN(NUM1+2))*SIGN(1,KN(NUM1+3)))
            IF(INX1.EQ.INX2) SG=2.0*SG
            A11(KEY)=A11(KEY)-SG*VOL0*R(IELEM+1,1)/DIFF0(L)
         ENDIF
         IF((KN(NUM1+4).NE.0).AND.(KN(NUM1+5).NE.0)) THEN
            IF(INY2.GT.INY1) KEY=MU(INY2)-INY2+INY1
            IF(INY2.LE.INY1) KEY=MU(INY1)-INY1+INY2
            SG=REAL(SIGN(1,KN(NUM1+4))*SIGN(1,KN(NUM1+5)))
            IF(INY1.EQ.INY2) SG=2.0*SG
            A11(KEY)=A11(KEY)-SG*VOL0*R(IELEM+1,1)/DIFF0(L)
         ENDIF
      ENDIF
   60 CONTINUE
   70 NUM1=NUM1+5
      NUM2=NUM2+4
   80 CONTINUE
*
* L-D-L(T) FACTORIZATION AND RESOLUTION.
      CALL ALLDLF (LL4,A11,MU)
      DO 90 I=1,LL4
   90 FUNKNO(I)=SOURCE(I)
      CALL ALLDLS (LL4,MU,A11,FUNKNO)
*
* CALCULATION OF THE SURFACE-AVERAGED FLUX.
      FUNKNO(NUN)=0.0
      NUM1=0
      NUM2=0
      DO 160 K=1,NREG
      IF(MAT(K).EQ.0) GO TO 160
      IF(VOL(K).EQ.0.0) GO TO 150
      IF(BFR(NUM2+1).GT.0.0) THEN
         SG=1.0
         DO 110 I0=1,IELEM
         IND1=KN(NUM1+1)+I0-1
         FUNKNO(NUN)=FUNKNO(NUN)+SG*SQRT(REAL(2*I0-1))*FUNKNO(IND1)*
     1   BFR(NUM2+1)
  110    SG=-SG
      ENDIF
      IF(BFR(NUM2+2).GT.0.0)  THEN
         DO 120 I0=1,IELEM
         IND1=KN(NUM1+1)+I0-1
  120    FUNKNO(NUN)=FUNKNO(NUN)+SQRT(REAL(2*I0-1))*FUNKNO(IND1)*
     1   BFR(NUM2+2)
      ENDIF
      IF(BFR(NUM2+3).GT.0.0)  THEN
         SG=1.0
         DO 130 I0=1,IELEM
         IND1=KN(NUM1+1)+(I0-1)*IELEM
         FUNKNO(NUN)=FUNKNO(NUN)+SG*SQRT(REAL(2*I0-1))*FUNKNO(IND1)*
     1   BFR(NUM2+3)
  130    SG=-SG
      ENDIF
      IF(BFR(NUM2+4).GT.0.0)  THEN
         DO 140 I0=1,IELEM
         IND1=KN(NUM1+1)+(I0-1)*IELEM
  140    FUNKNO(NUN)=FUNKNO(NUN)+SQRT(REAL(2*I0-1))*FUNKNO(IND1)*
     1   BFR(NUM2+4)
      ENDIF
  150 NUM1=NUM1+5
      NUM2=NUM2+4
  160 CONTINUE
      RETURN
      END
