*DECK BIVGSH
      SUBROUTINE BIVGSH (IEX,NGCOND,IGR,NMERGE,FLXMER,SUNMER,NREG,NUN,
     1 LL4,ISPLH,SIDE,MAT,KN,QFR,VOL,DIFF0,MERG,COUR,FUNKNO,SOURCE,
     2 ZGAR1,ZGAR2)
C
C-----------------------------------------------------------------------
C
C SOURCE TERM CALCULATION FOR MESH CENTERED FINITE DIFFERENCES IN
C HEXAGONAL GEOMETRY.
C
C INPUT PARAMETERS:
C  IEX    : ITERATION NUMBER.
C  NGCOND : NUMBER OF GROUPS CONDENSED.
C  IGR    : ENERGY GROUP INDEX.
C  NMERGE : NUMBER OF MERGED REGIONS.
C  FLXMER : FLUX ESTIMATE PER MIXTURE. DIMENSION FLXMER(NMERGE,NGCOND)
C  SUNMER : INCOMING SOURCE (SCATTERING+FISSION) X-SECTIONS.
C           DIMENSION SUNMER(NMERGE,NGCOND)
C  NREG   : NUMBER OF HEXAGONS IN BIVAC.
C  NUN    : NUMBER OF UNKNOWNS PER GROUP IN BIVAC.
C  LL4    : ORDER OF THE SYSTEM MATRICES IN BIVAC. EQUAL TO THE NUMBER
C           OF FINITE ELEMENTS (HEXAGONS OR TRIANGLES) EXCLUDING THE
C           VIRTUAL ELEMENTS.
C  ISPLH  : =1 : HEXAGONAL ELEMENTS ; .GT.1 : TRIANGULAR ELEMENTS.
C  SIDE   : SIDE OF THE HEXAGONS.
C  MAT    : MIXTURE INDEX PER HEXAGON. DIMENSION MAT(NREG)
C  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN((LH+1)*LL4)
C           WHERE LH=6 (HEXAGONS) OR 3 (TRIANGLES).
C  QFR    : ELEMENT-ORDERED INFORMATION. DIMENSION QFR((LH+1)*LL4)
C  VOL    : VOLUME OF HEXAGONS. DIMENSION VOL(NREG)
C  DIFF0  : DIFFUSION COEFFICIENTS. DIMENSION DIFF0(NBMIX) WHERE
C           NBMIX=MAX(MAT(I))
C  MERG   : INDEX OF MERGED HEXAGONS. DIMENSION MERG(NREG)
C  COUR   : FOUR TIMES THE INCOMING CURRENT PER UNIT SURFACE.
C  FUNKNO : PREVIOUSLY CALCULATED FLUXES. DIMENSION FUNKNO(NUN,NGCOND)
C
C OUTPUT PARAMETER:
C  SOURCE : FISSION AND DIFFUSION SOURCES. DIMENSION SOURCE(NREG)
C
C SCRATCH STORAGE:
C  ZGAR1  : DIMENSION ZGAR1(NBMIX) WHERE NBMIX=MAX(MAT(I))
C  ZGAR2  : DIMENSION ZGAR2(NBMIX)
C
C------------------------------------- AUTHOR: A. HEBERT ; 18/07/94 ----
C
      IMPLICIT NONE
C----
C INTERFACE VARIABLES
C-----
      INTEGER   NUN,IEX,NGCOND,IGR,NMERGE,NREG,ISPLH
      INTEGER   MAT(*),KN(*),MERG(*)
      REAL      COUR
      REAL      FLXMER(NMERGE,*),SUNMER(NMERGE,*),QFR(*),
     1          VOL(*),DIFF0(*),FUNKNO(NUN,*),SOURCE(*),
     2          ZGAR1(*),ZGAR2(*)
C----
C LOCAL VARIABLES
C----
      INTEGER          NSURF,NBMIX,I,IND1,NUM1,LL4,KHEX,L,
     &                 IC,NL,IGR2
      REAL             FACT,DS,X1,X2,DIF1,DIF2,SIDE,VOL0,SIDEB,PV
      DOUBLE PRECISION A
C----
C INTERNAL FUNCTION
C----
      DOUBLE PRECISION DHARM
      DHARM(X1,X2,DIF1,DIF2)=2.0D0*DIF1*DIF2/(X1*DIF2+X2*DIF1)
C
C---
C
      IF(ISPLH.EQ.1) THEN
         DS=SQRT(3.0)*SIDE
         FACT=2.0/(3.0*DS)
         NSURF=6
      ELSE
         DS=SIDE/(SQRT(3.0)*REAL(ISPLH-1))
         FACT=4.0/(3.0*DS)
         NSURF=3
      ENDIF
*
      NBMIX=0
      DO 10 I=1,NREG
   10 NBMIX=MAX(NBMIX,MAT(I))
      DO 20 IND1=1,LL4
   20 SOURCE(IND1)=0.0
*
* INCOMING CURRENT SOURCE.
      IF(COUR.NE.0.0) THEN
         NUM1=0
         DO 40 IND1=1,LL4
         KHEX=KN(NUM1+NSURF+1)
         IF(VOL(KHEX).EQ.0.0) GO TO 40
         L=MAT(KHEX)
         VOL0=QFR(NUM1+NSURF+1)
         SIDEB=FACT*VOL0
         DO 30 IC=1,NSURF
         IF(KN(NUM1+IC).EQ.-1) THEN
            A=DHARM(DS,DS,DIFF0(L),0.5*DS*QFR(NUM1+IC))
            SOURCE(IND1)=SOURCE(IND1)+A*COUR*SIDEB
         ENDIF
   30    CONTINUE
   40    NUM1=NUM1+NSURF+1
      ENDIF
*
* DIFFUSION AND FISSION SOURCE.
      IGR2=NGCOND
      IF(IEX.EQ.1) IGR2=IGR-1
      DO 90 NL=1,IGR2
      DO 70 L=1,NBMIX
      ZGAR1(L)=0.0
   70 ZGAR2(L)=0.0
      NUM1=0
      DO 80 IND1=1,LL4
      KHEX=KN(NUM1+NSURF+1)
      IF(VOL(KHEX).EQ.0.0) GO TO 80
      L=MAT(KHEX)
      VOL0=QFR(NUM1+NSURF+1)
      ZGAR1(L)=ZGAR1(L)+VOL0
      ZGAR2(L)=ZGAR2(L)+FUNKNO(IND1,NL)*VOL0
   80 NUM1=NUM1+NSURF+1
      NUM1=0
      DO 90 IND1=1,LL4
      KHEX=KN(NUM1+NSURF+1)
      IF(VOL(KHEX).EQ.0.0) GO TO 90
      L=MAT(KHEX)
      VOL0=QFR(NUM1+NSURF+1)
      PV=SUNMER(MERG(KHEX),NL)*FLXMER(MERG(KHEX),NL)*ZGAR1(L)/ZGAR2(L)
      SOURCE(IND1)=SOURCE(IND1)+FUNKNO(IND1,NL)*VOL0*PV
   90 NUM1=NUM1+NSURF+1
      IF(IEX.GT.1) RETURN
      NUM1=0
      DO 110 IND1=1,LL4
      KHEX=KN(NUM1+NSURF+1)
      IF(VOL(KHEX).EQ.0.0) GO TO 110
      VOL0=QFR(NUM1+NSURF+1)
      PV=0.0
      DO 100 NL=IGR,NGCOND
  100 PV=PV+SUNMER(MERG(KHEX),NL)*FLXMER(MERG(KHEX),NL)
      SOURCE(IND1)=SOURCE(IND1)+VOL0*PV
  110 NUM1=NUM1+NSURF+1
      RETURN
      END
