*DECK CPOMIC
      SUBROUTINE CPOMIC(IPCPO ,IPEDIT,IPRINT,NGROUP,NMERGE,NBMICR,
     >                  NL    ,IMRG  ,ISOR  ,NPROC ,ISOTMP,IDIMIX,
     >                  INDPRO,ITYPRO,DENCPO,DENTMP,DXSMIC,DSCMIC,
     >                  EMJTMP,DMJCPO,DFLUX ,XSREC ,XSCAT ,SCATC ,
     >                  IJJ   ,NJJ   )
C
C---------------------------  CPOMIC  ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : CPOMIC
C      USE      : COMPUTE MICROSCOPIC CROSS SECTION
C      MODIFIED : 97-07-18 (G.M.)
C      AUTHOR   : G.MARLEAU
C
C  2- ROUTINE PARAMETERS:
C    INPUT
C      IPCPO    : POINTER TO THE CPO                 I
C      IPEDIT   : POINTER TO THE EDIT                I
C      IPRINT   : PRINT LEVEL                        i
C      NGROUP   : NUMBER OF GROUPS CONDENSED         I
C      NMERGE   : NUMBER OF REGIONS MERGED           I
C      NBMICR   : MAXIMUM NUMBER OF ISOTOPES         I
C      NL       : NUMBER OF LEGENDRE ORDERS          I
C      IMRG     : MERGE REGION                       I
C      ISOR     : CPO ISOTOPE NUMBER                 I
C      NPROC    : NUMBER OF MIC XS TO PROCESS        I
C      ISOTMP   : NAME OF ISOTOPES IN EDIT           I(3,NBMICR)
C      IDIMIX   : ISOTOPES IDENTIFIER IN EACH        I(NMERGE,NBMICR)
C                 CPO MATERIAL
C      INDPRO   : IDENTIFIER FOR XS PROCESSING       I(NPROC)
C      ITYPRO   : IDENTIFIER FOR XS PROCESSED        I(NPROC)
C      DENCPO   : CPO ISOTOPES CONCENTRATION         D(NBMICR)
C      DENTMP   : DRAGON ISOTOPES CONCENTRATION      R(NBMICR)
C      EMJTMP   : FISSION ENERGY PER ISOTOPES        R(NBMICR)
C    INPUT/OUTPUT
C      DXSMIC   : MICRO VECTOR XS                    D(NGROUP,NPROC)
C      DSCMIC   : MICRO SCAT MATRIX XS               D(NGROUP,NGROUP,NL)
C      DFLUX    : FLUX                               D(NGROUP)
C      DMJCPO   : FISSION ENERGY FOR MACRO           D(2,NBMICR)
C    WORK
C      XSREC    : MICRO VECTOR XS                    R(NGROUP,NPROC)
C      XSCAT    : COMPRESS SCATTERING                R(NGROUP,NGROUP,NL)
C      SCATC    : COMPRESS SCATTERING                R(NGROUP*NGROUP)
C      IJJ      : POSITION OF FIRST DIFFUSION GROUP  I(NGROUP)
C      NJJ      : NUMBER OF DIFFUSION GROUP          I(NGROUP)
C
C---------------------------   CPOMIC  --------------------------------
C
      IMPLICIT         NONE
      INTEGER          IPCPO ,IPEDIT,IPRINT,NGROUP,NMERGE,NBMICR,
     >                 NL,IMRG,ISOR,NPROC,ISOTMP(3,NBMICR),
     >                 IDIMIX(NMERGE,NBMICR),INDPRO(NPROC),
     >                 ITYPRO(NPROC),IJJ(NGROUP),NJJ(NGROUP)
      REAL             DENTMP(NBMICR),XSREC(NGROUP,NPROC),
     >                 EMJTMP(NBMICR),XSCAT(NGROUP,NGROUP,NL),
     >                 SCATC(NGROUP*NGROUP)
      DOUBLE PRECISION DENCPO(NBMICR),
     >                 DXSMIC(NGROUP,NPROC),
     >                 DSCMIC(NGROUP,NGROUP,NL),
     >                 DMJCPO(2,NBMICR),
     >                 DFLUX(NGROUP)
C----
C  MEMORY ALLOCATION PARAMETERS
C----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
C----
C  LOCAL PARAMETERS
C----
      INTEGER          IOUT,NDPROC
      PARAMETER       (IOUT=6,NDPROC=20)
      INTEGER          NBISOE,ISOE,IADDXS,ITC,IXSR,JXSR,KXSR,IL,
     >                 IGR,JGR
      CHARACTER        NAMISO*12
      DOUBLE PRECISION DSCPO,DNUFI,FACDEN,FACAXS
C----
C  COMPUTE FINAL DENSITY OF ISOTOPE ISOR IN MIXTURE IMRG
C----
      NBISOE=0
      DO 100 ISOE=1,NBMICR
        IF(IDIMIX(IMRG,ISOE).EQ.ISOR) THEN
          NBISOE=NBISOE+1
          DENCPO(ISOR)=DENCPO(ISOR)+DENTMP(ISOE)
        ENDIF
 100  CONTINUE
      IF(DENCPO(ISOR).EQ.0.0D0) THEN
        FACDEN=1.0D0/DBLE(NBISOE)
        IADDXS=0
      ELSE
        FACDEN=1.0D0/DENCPO(ISOR)
        IADDXS=1
      ENDIF
C----
C  SCAN ALL ISOTOPES FOR THIS MERGE REGION AND
C  LOCATE ADD XS FOR THOSE ASSOCIATED WITH ISOR
C----
      DSCPO=0.0D0
      DO 110 ISOE=1,NBMICR
        IF(IDIMIX(IMRG,ISOE).EQ.ISOR) THEN
          IF(IADDXS.EQ.0) THEN
            FACAXS=FACDEN
          ELSE
            FACAXS=FACDEN*DBLE(DENTMP(ISOE))
          ENDIF
C----
C  READ MICRO XS
C----
          WRITE(NAMISO,'(3A4)') (ISOTMP(ITC,ISOE),ITC=1,3)
          IF(IPRINT.GE.10) THEN
            WRITE(IOUT,6000) NAMISO
          ENDIF
          CALL LCMSIX(IPEDIT,NAMISO,1)
          CALL XDRLGS(IPEDIT,-1,NPROC,0,NL-1,1,NGROUP,INDPRO,
     >                ITYPRO,XSREC,XSCAT,SCATC,IJJ,NJJ)
          CALL LCMSIX(IPEDIT,NAMISO,2)
C----
C  ADD MICRO XS TO CPO ISOTOPE
C----
          DO 120 IXSR=1,4
            IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
              DO 121 IGR=1,NGROUP
                DXSMIC(IGR,IXSR)=DXSMIC(IGR,IXSR)
     >            +FACAXS*DBLE(XSREC(IGR,IXSR))
 121          CONTINUE
            ENDIF
 120      CONTINUE
          IXSR=4
          IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
C----
C  COMPUTE TOTAL NEUTRON PRODUCED AND TOTAL ENERGY GENERATED
C----
            DO 122 IGR=1,NGROUP
              DMJCPO(1,ISOR)=DMJCPO(1,ISOR)+DENTMP(ISOE)
     >            *DFLUX(IGR)*DBLE(XSREC(IGR,IXSR)*EMJTMP(ISOE))
              DMJCPO(2,ISOR)=DMJCPO(2,ISOR)+DENTMP(ISOE)
     >            *DFLUX(IGR)*DBLE(XSREC(IGR,IXSR))
 122        CONTINUE
          ENDIF
          IXSR=5
          IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
            JXSR=3
            DNUFI=0.0D0
            DO 130 IGR=1,NGROUP
              DNUFI=DNUFI+DFLUX(IGR)*DBLE(XSREC(IGR,JXSR))
 130        CONTINUE
            DNUFI=DNUFI*FACAXS
            DSCPO=DSCPO+DNUFI
            DO 140 IGR=1,NGROUP
              DXSMIC(IGR,IXSR)=DXSMIC(IGR,IXSR)
     >          +DNUFI*DBLE(XSREC(IGR,IXSR))
 140        CONTINUE
          ENDIF
          DO 150 IXSR=7,NDPROC
            IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
              DO 151 IGR=1,NGROUP
                DXSMIC(IGR,IXSR)=DXSMIC(IGR,IXSR)
     >            +FACAXS*DBLE(XSREC(IGR,IXSR))
 151          CONTINUE
            ENDIF
 150      CONTINUE
          IL=0
          DO 160 IXSR=NDPROC+1,NDPROC+NL
            IL=IL+1
            IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
              DO 170 IGR=1,NGROUP
                DXSMIC(IGR,IXSR)=DXSMIC(IGR,IXSR)
     >            +FACAXS*DBLE(XSREC(IGR,IXSR))
                DO 171 JGR=1,NGROUP
                  DSCMIC(IGR,JGR,IL)=DSCMIC(IGR,JGR,IL)
     >              +FACAXS*DBLE(XSCAT(IGR,JGR,IL))
 171            CONTINUE
 170          CONTINUE
            ENDIF
 160      CONTINUE
        ENDIF
 110  CONTINUE
C----
C   SAVE AVERAGE XS
C----
      CALL XDRSET(XSREC,NGROUP*NPROC,0.0)
      CALL XDRSET(XSCAT,NGROUP*NGROUP*NL,0.0)
      DO 180 IXSR=1,4
        IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
          DO 181 IGR=1,NGROUP
            XSREC(IGR,IXSR)=DXSMIC(IGR,IXSR)
 181      CONTINUE
        ENDIF
 180  CONTINUE
      IXSR=5
      IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
        IF(DSCPO.NE.0.0D0) THEN
          DO 190 IGR=1,NGROUP
            XSREC(IGR,IXSR)=DXSMIC(IGR,IXSR)/DSCPO
 190      CONTINUE
        ENDIF
      ENDIF
      IXSR=6
      IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
        JXSR=4
        KXSR=3
        DO 191 IGR=1,NGROUP
          IF(DXSMIC(IGR,JXSR).GT.0.0) THEN
            XSREC(IGR,IXSR)=DXSMIC(IGR,KXSR)/DXSMIC(IGR,JXSR)
          ENDIF
 191    CONTINUE
      ENDIF
      DO 200 IXSR=7,NDPROC
        IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
          DO 201 IGR=1,NGROUP
            XSREC(IGR,IXSR)=DXSMIC(IGR,IXSR)
 201      CONTINUE
        ENDIF
 200  CONTINUE
      IL=0
      DO 210 IXSR=NDPROC+1,NDPROC+NL
        IL=IL+1
        IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
          DO 220 IGR=1,NGROUP
            XSREC(IGR,IXSR)=DXSMIC(IGR,IXSR)
            DO 221 JGR=1,NGROUP
              XSCAT(IGR,JGR,IL)=DSCMIC(IGR,JGR,IL)
 221        CONTINUE
 220      CONTINUE
        ENDIF
 210  CONTINUE
C----
C  SAVE CPO MICRO
C----
      CALL XDRLGS(IPCPO,1,NPROC,0,NL-1,1,NGROUP,INDPRO,
     >            ITYPRO,XSREC,XSCAT,SCATC,IJJ,NJJ)
      RETURN
C----
C  PRINT FORMAT
C----
 6000 FORMAT(' CPOMIC: INCLUDE ISOTOPE          = ',A12)
      END
