*DECK CROWH1
      SUBROUTINE CROWH1(Z,ZI,COTE,SIGT,R,IAPP,TRONC,PIS,PSS,PII)
C
C---------------------------  CROWH1  ---------------------------------
C
C   1-  PROGRAM STATISTICS:
C
C          FUNCTION   :  COMPUTE THE COLLISION PROBABILITIES OF AN
C                        HOMOGENEOUS 2D HEXAGONAL CROWN WITH DP0
C                        SURFACE ANGULAR EXPANSION ON THE HEXAGONAL
C                        BOUNDARY. THE TRACKS ARE COMPUTED BY SYBHTK
C          REVISED    :  18-06-1990
C          AUTHOR     :  M.OUISLOUMEN
C
C   2-  PROGRAM PARAMETERS:
C
C INPUT PARAMETERS.
C   IAPP    : TYPE OF ANGULAR FLUX EXPANSION ON THE INTERNAL SURFACE
C            = 2   => DP1 , = 1  => DP0                  L
C   COTE    : LENGTH OF ONE OF SIDES OF THE HEXAGONE     R
C   SIGT    : TOTAL CROSS SECTION.                       R
C   Z,ZI    : INTEGRATION MESH                           R
C   R       : RADIUS OF INTERNAL TUBE                    R
C
C INTRINSEC FUNCTIONS
C   FUNC3   : COMPUTE THE BICKLEY NAYLOR FUNCTION OF ORDER 3
C   FUNC4   : COMPUTE THE BICKLEY NAYLOR FUNCTION OF ORDER 4
C
C OUTPUT PARAMETERS.
C   PIS     : LEAKAGE PROBABILITY                        R(8)
C             PIS(I)  -> VOLUME TO SIDE I,(I=1,8)
C   PSS     : TRANSMISSION PROBABILITY                   R(8,8)
C             PSS(I,J) -> SIDE I TO SIDE J (I=1,8,J=1,8)
C   PII     : VOLUME TO VOLUME REDUCED PROBABILITY.
C
C
C                                            SIDE A,B
C                      SIDE 4               DIR A -> ISOTROPIC
C                     XXXXXXXX
C                    X        X             DIR B -> NORMAL  TO SURFACE
C           SIDE 5  X          X SIDE 3
C                  X   SIDE 1,2 X
C                 X      O       X
C                  X            X
C            SIDE 6 X          X SIDE 8
C                    X        X
C                     XXXXXXXX
C                      SIDE 7
C
C---------------------------   CROWH1  --------------------------------
C
      PARAMETER (MKI3=600,MKI4=600)
      PARAMETER (SQRT3=1.732050807568877,PI=3.141592653589793,
     +            PI4=PI*.25,PI12=PI/12.,C1=2./3.)
C
      REAL      PIS(*),PSS(6+IAPP,*),Z(*),P(7),PVS(3)
      INTEGER   ZI(*),IROT1(8,8),IROT0(7,7),IPVS0(7),IPVS1(8)
      LOGICAL   LGP1,LG6,LG5,LG51,SERIE
C
C ASSUME THAT BICKLEY KI TABLES HAVE THE SAME TABULATION POINTS AND
C THE SAME TRUNCATION LIMIT.
C
      COMMON /BICKL3/BI3(0:MKI3,3),PAS3,XLIM3,L3
      COMMON /BICKL4/BI4(0:MKI4,3),PAS4,XLIM4,L4
C
      SAVE IROT1,IROT0,IPVS1,IPVS0
      DATA IROT1/
     +           0,0,6,6,6,6,6,6,
     +           0,0,7,7,7,7,7,7,
     +           4,5,0,1,2,3,2,1,
     +           4,5,1,0,1,2,3,2,
     +           4,5,2,1,0,1,2,3,
     +           4,5,3,2,1,0,1,2,
     +           4,5,2,3,2,1,0,1,
     +           4,5,1,2,3,2,1,0/
      DATA IPVS1/1,2,3,3,3,3,3,3/
      DATA IPVS0/1,3,3,3,3,3,3/
      DATA IROT0/
     +           0,6,6,6,6,6,6,
     +           4,0,1,2,3,2,1,
     +           4,1,0,1,2,3,2,
     +           4,2,1,0,1,2,3,
     +           4,3,2,1,0,1,2,
     +           4,2,3,2,1,0,1,
     +           4,1,2,3,2,1,0/
C
      FUNC3(X,K)=BI3(K,1)+X*(BI3(K,2)+X*BI3(K,3))
      FUNC4(X,K)=BI4(K,1)+X*(BI4(K,2)+X*BI4(K,3))
C
C    INITIALIZATION
C
      DO 10 I=1,7
      P(I)=0.
   10 CONTINUE
C
      LGP1=.TRUE.
      IF(IAPP.NE.2)LGP1=.FALSE.
      RI5=.5/R
      NA=ZI(1)
      NX=ZI(2)
      IZ0=2
      IZ=4
      VOLUME=4.*(1.5*COTE*COTE*SQRT3-PI*R*R)
      SERIE=SIGT*VOLUME/(6.*COTE).LE.TRONC
      IF(SERIE) THEN
         PVS(1)=0.
         PVS(2)=0.
         P60S=0.
         P60I=0.
         P50S=0.
         P50I=0.
         P40S=0.
         P40I=0.
         P70I=0.
      ENDIF
C
C      INTEGRATE OVER ANGELES
C
          DO 150 IA=1,NA
C
      LG6=.TRUE.
      LG5=.FALSE.
      LG51=.FALSE.
      IZ0=IZ0+3
      NR5=ZI(IZ0)
      IMIN=ZI(IZ0-1)
      IMAX=NR5+IMIN-1
      IMIN6=ZI(IZ0-2)
      NR6=3-IMIN6
      IZZ1=IZ
      IZ=IZ+NR5+NR6+IMIN
      IF(IMIN6.EQ.1) LG6=.FALSE.
      IF(IMAX.EQ.2)LG5=.TRUE.
      IF(IMIN.EQ.2)LG51=.TRUE.
C
C       INTEGRATE OVER DISTANCE X
C
         DO 120 IX=1,NX
C
      IZ=IZ+9
      W600=Z(IZ)
C
C     TRANSMISSION PAR LES DIFFERENTES FACES: 7->I, I=1,6,5,4
C     SURFACE 6
C
      IZ1=IZZ1
      IZ=IZ+2
      IZZ=IZ
      RI=Z(IZ-1)
      IZ1=IZ1+1
      DELR=Z(IZ1)
      TAU=SIGT*RI
      WW6=W600*DELR
      IF(SERIE)THEN
C
C        SERIES EXANSION FOR BICKLEY-NAYLOR FUNCTIONS
C
         XXF3=PI4*RI*RI
         XF3=-RI+SIGT*XXF3
         P60S=P60S+WW6*XF3
         P60I=P60I+WW6*XXF3
         P(1)=P(1)+WW6*(PI4+SIGT*XF3)
      ELSE
C
C         DIRECT KI CALCULATIONS
C
         IF(TAU.LT.XLIM3) THEN
            K=NINT(PAS3*TAU)
            P(1)=P(1)+WW6*FUNC3(TAU,K)
         ENDIF
      ENDIF
C
      IF(LG6) GOTO 8
C
C           CONTRIBUTION TO FACE 1
C
      IZ1=IZ1+1
      DELR=Z(IZ1)
      IZ=IZ+1
      RI=Z(IZ)
      IZ=IZ+2
      RC=Z(IZ)
      TAU7C=SIGT*RC
      RD=Z(IZ-1)-RC
      TAU72=SIGT*RD
      WW6=DELR*W600
      COS6=RI*RI5
      IF(SERIE)THEN
C
C         SERIES EXPANSIONS FOR KI FUNCTIONS
C
         XXF3=PI4*(RC*RC+RD*RD)
         XF3=-RC-RD+SIGT*XXF3
         PVS(1)=PVS(1)+WW6*XF3
         P70I=P70I+WW6*XXF3
         P(4)=P(4)+WW6*(2.*PI4+SIGT*XF3)
         IF(LGP1) THEN
            XF4=RC*(-PI4+TAU7C*(.5-PI12*TAU7C))
            XF42=RD*(-PI4+TAU72*(.5-PI12*TAU72))
            WWW6=WW6*COS6
            P(5)=P(5)+WWW6*(2.*C1+SIGT*(XF42+XF4))
            PVS(2)=PVS(2)+WWW6*(XF4+XF42)
         ENDIF
      ELSE
C
C      DIRECT KI CALCULATIONS
C
         IF(TAU7C.LT.XLIM3) THEN
            K=NINT(PAS3*TAU7C)
            P(4)=P(4)+WW6*FUNC3(TAU7C,K)
            IF(LGP1) P(5)=P(5)+WW6*COS6*FUNC4(TAU7C,K)
         ENDIF
         IF(TAU72.LT.XLIM3) THEN
            K=NINT(PAS3*TAU72)
            P(4)=P(4)+WW6*FUNC3(TAU72,K)
            IF(LGP1)P(5)=P(5)+COS6*WW6*FUNC4(TAU72,K)
         ENDIF
      ENDIF
C
C     SURFACE 5
C
    8 IF(LG5) THEN
         IZ=IZ+2
         R0=Z(IZ-1)
         IZ1=IZ1+1
         DELR=Z(IZ1)
         TAU=SIGT*R0
         WW6=W600*DELR
         IF(SERIE)THEN
C
C       SERIES EXPANSION FOR KI FUNCTIONS
C
            XXF3=PI4*R0*R0
            XF3=-R0+SIGT*XXF3
            P50S=P50S+WW6*XF3
            P50I=P50I+WW6*XXF3
            P(2)=P(2)+WW6*(PI4+SIGT*XF3)
            IF(LG51) GOTO 11
         ELSE
C
C         DIRECT KI CALCULATIONS
C
            IF(TAU.LT.XLIM3) THEN
               K=NINT(PAS3*TAU)
               P(2)=P(2)+WW6*FUNC3(TAU,K)
            ENDIF
            IF(LG51) GOTO 11
         ENDIF
      ENDIF
C
C         CONTRIBUTION TO FACE 1
C
      IZ=IZ+1
      RI=Z(IZ)
      IZ=IZ+2
      RC=Z(IZ)
      IZ1=IZ1+1
      DELR=Z(IZ1)
      TAU7C=SIGT*RC
      RD=Z(IZ-1)-RC
      TAU72=SIGT*RD
      WW6=DELR*W600
      COS6=RI*RI5
      IF(SERIE)THEN
C
C          SERIES EXPANSION FOR KI FUNCTIONS
C
         XXF3=PI4*(RC*RC+RD*RD)
         XF3=-RC-RD+SIGT*XXF3
         PVS(1)=PVS(1)+WW6*XF3
         P70I=P70I+WW6*XXF3
         P(4)=P(4)+WW6*(2.*PI4+SIGT*XF3)
         IF(LGP1) THEN
            WWW6=WW6*COS6
            XF4=RC*(-PI4+TAU7C*(.5-PI12*TAU7C))
            XF42=RD*(-PI4+TAU72*(.5-PI12*TAU72))
            P(5)=P(5)+WWW6*(2.*C1+SIGT*(XF4+XF42))
            PVS(2)=PVS(2)+WWW6*(XF4+XF42)
         ENDIF
         GOTO 17
      ELSE
C
C          DIRECT KI CALCULATIONS
C
         IF(TAU7C.LT.XLIM3) THEN
            K=NINT(PAS3*TAU7C)
            P(4)=P(4)+WW6*FUNC3(TAU7C,K)
            IF(LGP1)P(5)=P(5)+WW6*COS6*FUNC4(TAU7C,K)
         ENDIF
         IF(TAU72.LT.XLIM3) THEN
            K=NINT(PAS3*TAU72)
            P(4)=P(4)+WW6*FUNC3(TAU72,K)
            IF(LGP1)P(5)=P(5)+COS6*WW6*FUNC4(TAU72,K)
         ENDIF
         GOTO 17
      ENDIF
C
C     SUFACE 4
C
   11 IZ=IZ+2
      R0=Z(IZ-1)
      IZ1=IZ1+1
      DELR=Z(IZ1)
      TAU=SIGT*R0
      WW6=W600*DELR
      IF(SERIE) THEN
C
C       SERIES EXPANSION FOR KI FUNCTIONS
C
         XXF3=PI4*R0*R0
         XF3=-R0+SIGT*XXF3
         P40S=P40S+WW6*XF3
         P40I=P40I+WW6*XXF3
         P(3)=(3)+WW6*(PI4+SIGT*XF3)
      ELSE
C
C       DIRECT KI CALCULATIONS
C
         IF(TAU.LT.XLIM3) THEN
            K=NINT(PAS3*TAU)
            P(3)=P(3)+WW6*FUNC3(TAU,K)
         ENDIF
      ENDIF
C
C       CONTRIBUTION TO FACE 1
C
   17 IZ=IZ+3
      RC=Z(IZ)
      RI=Z(IZ-2)
      IZ1=IZ1+1
      DELR=.5*Z(IZ1)
      TAU7C=RC*SIGT
      RD=Z(IZ-1)-RC
      TAU72=SIGT*RD
      WW6=DELR*W600
      COS6=RI*RI5
      IF(SERIE) THEN
C
C        SERIES EXPANSION FOR KI FUNCTIONS
C
         XXF3=PI4*(RC*RC+RD*RD)
         XF3=-RC-RD+SIGT*XXF3
         PVS(1)=PVS(1)+WW6*XF3
         P70I=P70I+WW6*XXF3
         P(4)=P(4)+WW6*(2.*PI4+SIGT*XF3)
         IF(LGP1) THEN
            XF4=-PI4*RC+RC*TAU7C*(.5-PI12*TAU7C)
            XF42=-PI4*RD+RD*TAU72*(.5-PI12*TAU72)
            F4=C1+SIGT*XF4
            F42=C1+SIGT*XF42
            P(5)=P(5)+WW6*COS6*(F4+F42)
            PVS(2)=PVS(2)+WW6*COS6*(XF4+XF42)
         ENDIF
      ELSE
C
C        DIRECT KI CALCULATIONS
C
         IF(TAU7C.LT.XLIM3) THEN
            K=NINT(PAS3*TAU7C)
            P(4)=P(4)+WW6*FUNC3(TAU7C,K)
            IF(LGP1) P(5)=P(5)+WW6*COS6*FUNC4(TAU7C,K)
         ENDIF
         IF(TAU72.LT.XLIM3) THEN
            K=NINT(PAS3*TAU72)
            P(4)=P(4)+WW6*FUNC3(TAU72,K)
            IF(LGP1)P(5)=P(5)+WW6*COS6*FUNC4(TAU72,K)
         ENDIF
      ENDIF
  120 CONTINUE
  150 CONTINUE
C
C     PSS ORTHONORMALIZATION
C
      E1=Z(1)
      E2=Z(2)
      E3=Z(3)
      COEF=COTE/(2.*PI*R)
      AUX=1./(6.*COTE)
      P4=AUX*P(4)
      P(4)=E1*E1*P4
C
C    LES PROBABILITES PSS(I->1) SONT OBTENUES PAR SYMETRIE
C    ET LES PSS(1->I) PAR RELATION DE RECIPROCITE
C
      P(6)=COEF*P(4)
      IF(LGP1) THEN
         P(5)=AUX*E1*E2*P(5)-P4*E1*E3
         P(7)=COEF*P(5)
         PVS(2)=AUX*E1*(PVS(2)*E2-PVS(1)*E3)
      ENDIF
      IF(SERIE) THEN
         P70I=AUX*P70I*E1*E1
         PVS(1)=AUX*PVS(1)*E1*E1
      ENDIF
C
C     NORMALIZATION OF PSS(7->I),I=6,5,4
C
      AUX=E1*E1*AUX
      P(1)=P(1)*AUX
      P(2)=P(2)*AUX
      P(3)=P(3)*AUX
      IF(SERIE) THEN
         P60S=AUX*P60S
         P60I=AUX*P60I
         P50I=AUX*P50I
         P50S=AUX*P50S
         P40S=AUX*P40S
         P40I=AUX*P40I
      ENDIF
C
C                  * STOCKAGE DES PSS *
C
C    COMPUTATION OF PIS AND PII
C
      CORX=1./VOLUME
      COR=COTE*CORX
      IF(.NOT.SERIE) THEN
         COR=COR/SIGT
         PVS(3)=COR*(1.-2.*(P(1)+P(2))-P(3)-P(4))
         COR=2.*PI*R*CORX/SIGT
         PVS(1)=COR*(1.-6.*P(6))
         IF(LGP1) PVS(2)=-6.*COR*P(7)
         PII=(1.-6.*PVS(3)-PVS(1))/SIGT
      ELSE
         PVS(3)=COR*(-2.*(P60S+P50S)-P40S-PVS(1))
         PVS(1)=-COR*6.*PVS(1)
         IF(LGP1) PVS(2)=-6.*COR*PVS(2)
         PII=6.*COR*(2.*(P60I+P50I+P70I)+P40I)
      ENDIF
C
C     TRANSMISSION MATRIX
C
      IF(LGP1)THEN
         DO 58 I=1,8
            PIS(I)=PVS(IPVS1(I))
         DO 58 J=1,8
            IB=IROT1(J,I)
            IF(IB.GT.0)THEN
               PSS(I,J)=P(IB)
            ELSE
               PSS(I,J)=0.
            ENDIF
   58    CONTINUE
      ELSE
         DO 60 I=1,7
            PIS(I)=PVS(IPVS0(I))
         DO 60 J=1,7
            IB=IROT0(J,I)
            IF(IB.GT.0)THEN
               PSS(I,J)=P(IB)
            ELSE
               PSS(I,J)=0.
            ENDIF
   60    CONTINUE
      ENDIF
      RETURN
      END
