*DECK EDISTA
      SUBROUTINE EDISTA(IPRINT,NMERGE,ITYPE,VOLMER,VOLREL,VOLTOT,
     >                  FLXNEW,FLXOLD,RATNEW,RATOLD,VALERR)
C
C---------------------------  EDISTA  ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : EDISTA
C      USE      : STATISTICS EDIT
C      MODIFIED : 97-02-10 (G.M.)
C                 91-04-08
C      AUTHOR   : G.MARLEAU
C
C  2- ROUTINE PARAMETERS
C    INPUT
C      IPRINT   : PRINT LEVEL                        I
C                  = 1  PRINT STATS RESULTS ONLY
C                  = 2  PRINT ERRORS ONLY
C                 >= 3  PRINT ALL
C      NMERGE   : NUMBER OF REGIONS                  I
C      ITYPE    : TYPE OF STATISTICS                 I
C                 = 1  FLUX RELATIVE ERRORS
C                 = 2  REACTION RATES RELATIVE ERRORS
C                 = 3  DELTA SIGMA
C      VOLMER   : CURRENT REGION MERGED VOLUMES      D(NMERGE)
C      VOLREL   : OLD VOLUME/NEW VOLUME              D
C      VOLTOT   : TOTAL OLD VOLUME                   R
C      FLXNEW   : NEW INTEGRATED FLUX                D(NMERGE)
C      FLXOLD   : OLD INTEGRATED FLUX                R(NMERGE)
C      RATNEW   : NEW HOMOGENIZED X-S                D(NMERGE)
C      RATOLD   : OLD HOMOGENIZED X-S                R(NMERGE)
C      VALERR   : RELATIVE ERROR OR DELTA SIGMA      R(NMERGE)
C
C
C---------------------------   EDISTA  --------------------------------
C
      IMPLICIT          NONE
      INTEGER           IOUT
      CHARACTER         NAMSBR*6
      PARAMETER        (IOUT=6,NAMSBR='EDISTA')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER           IPRINT,NMERGE,ITYPE
      REAL              VOLTOT,FLXOLD(NMERGE),RATOLD(NMERGE),
     >                  VALERR(NMERGE)
      DOUBLE PRECISION  VOLREL,VOLMER(NMERGE),FLXNEW(NMERGE),
     >                  RATNEW(NMERGE)
C----
C  LOCAL PARAMETERS
C----
      INTEGER           IREG
      REAL              EPSMAX,EPSAVG,RPSAVG,CURVAL,OLDVAL
C----
C  PRINT COMMENT IF REQUIRED
C----
      IF(IPRINT .GT. 2) THEN
        IF(ITYPE .NE. 3) THEN
          WRITE(IOUT,6000)
        ELSE
          WRITE(IOUT,6001)
        ENDIF
      ENDIF
      EPSMAX=0.0
      EPSAVG=0.0
      RPSAVG=0.0
      DO 100 IREG=1,NMERGE
        IF(ITYPE .EQ. 3) THEN
          CURVAL=RATNEW(IREG)
          OLDVAL=RATOLD(IREG)
          VALERR(IREG)=CURVAL-OLDVAL
        ELSE
          IF(ITYPE .EQ. 1) THEN
            CURVAL=VOLREL*FLXNEW(IREG)
            OLDVAL=FLXOLD(IREG)
          ELSE IF(ITYPE .EQ. 2) THEN
            CURVAL=VOLREL*RATNEW(IREG)*FLXNEW(IREG)
            OLDVAL=RATOLD(IREG)*FLXOLD(IREG)
          ENDIF
          IF(OLDVAL .NE. 0.0) THEN
            VALERR(IREG)=100.0*(CURVAL-OLDVAL)/OLDVAL
          ELSE IF(CURVAL .NE. 0.0) THEN
            VALERR(IREG)=100.0*(CURVAL-OLDVAL)/CURVAL
          ELSE
            VALERR(IREG)=0.0
          ENDIF
        ENDIF
        IF(IPRINT .GT. 2) THEN
          WRITE(IOUT,6002) IREG,CURVAL,OLDVAL,VALERR(IREG)
        ENDIF
        IF(ITYPE .NE. 3) THEN
          EPSMAX=MAX(EPSMAX,ABS(VALERR(IREG)))
          EPSAVG=EPSAVG+ABS(VALERR(IREG))*VOLMER(IREG)*VOLREL
          RPSAVG=RPSAVG+VALERR(IREG)*VALERR(IREG)
        ENDIF
 100  CONTINUE
      IF(ITYPE .NE. 3) THEN
        IF(IPRINT .EQ. 2) THEN
          WRITE(IOUT,6003)
          WRITE(IOUT,6006) (VALERR(IREG),IREG=1,NMERGE)
        ENDIF
        EPSAVG=EPSAVG/VOLTOT
        WRITE(IOUT,6005) EPSMAX,EPSAVG,SQRT(RPSAVG/NMERGE)
      ELSE IF(IPRINT .LE. 2) THEN
        WRITE(IOUT,6004)
        WRITE(IOUT,6006) (VALERR(IREG),IREG=1,NMERGE)
      ENDIF
C
C------    FORMAT
C
 6000 FORMAT(
     >4X,'REGION',13X,'CURRENT VALUE',10X,'REFERENCE',16X,' ERROR (%) ')
 6001 FORMAT(
     >4X,'REGION',13X,'CURRENT VALUE',10X,'REFERENCE',16X,'DELTA SIGMA')
 6002 FORMAT(4X,I5,10X,1P,E14.4,8X,E14.4,8X,E14.4)
 6003 FORMAT(' RELATIVE ERROR (NEW-OLD) ON FLUXES (%)')
 6004 FORMAT(' DELTA SIGMA (NEW-OLD)')
 6005 FORMAT(/4X,'                MAXIMUM ERROR=',F8.2,' %'/
     >        4X,'VOLUME WEIGHTED AVERAGE ERROR=',F8.2,' %'/
     >        4X,'                   RMS  ERROR=',F8.2,' %')
 6006 FORMAT(1P,7(3X,E15.7))
      RETURN
      END
