*DECK EVOGET
      SUBROUTINE EVOGET(IPRINT,ITYPE ,ITIXS ,IEXTR ,IGLOB ,INR   ,
     >                  RPAR  ,XT    )
C
C----
C
C  READER FOR ISOTOPIC DEPLETION CALCULATION OPTIONS
C
C INPUT/OUTPUT
C  IPRINT  : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.     I
C  ITYPE   : SOLUTION METHOD                             I
C            =1 FIFTH-ORDER CASH-KARP ALGORITHM.
C            =2 FOURTH-ORDER KAPS-RENTROP ALGORITHM.
C  ITIXS   : FLAG FOR TIME DEPENDENT XS                  I
C            = 0 ON
C            = 1 OFF
C  IEXTR   : FLAG FOR POWER EXTRAPOLATION                I
C            = 0 ON
C            = 1 OFF
C  IGLOB   : FLAG FOR POWER COMPUTATION OPTION           I
C            = 0 FUEL POWER ONLY
C            = 1 TOTAL CELL POWER
C  INR     : BURNUP OPTION                               I
C            = 0 OUT-OF-CORE DEPLETION.
C            = 1 CONSTANT FLUX DEPLETION.
C            = 2 CONSTANT POWER DEPLETION.
C  RPAR    : BURNUP PARAMETERS                           R(5)
C            RPAR(1) = EPS1   ACCURACY FOR ODE SOLVER.
C            RPAR(2) = EPS2   ACCURACY FOR CONSTANT
C                             POWER  ITERATIONS.
C            RPAR(3) = EXPMAX ISOTOPE SATURATING LIMIT.
C            RPAR(4) = H1     GUESSED FIRST STEPSIZE.
C            RPAR(5) = FIT    FLUX (N/CM**2/S) OR
C                             POWER (kW/kG INITIAL HEAVY)
C                             NORMALIZATIO FACTOR.
C  XT      : TIME TABLE                                  R(5)
C            XT(1)   = INITIAL TIME FOR DEPLETION
C            XT(2)   = FINAL TIME FOR DEPLETION
C            XT(3)   = TIME FOR SAVE
C            XT(4)   = LAST SET TIME
C            XT(5)   = TIME FOR CURRENT SET
C
C----
C   REVISION HISTORY
C    1) AUTHOR    : A. HEBERT     1994/11/09
C    2) MODIFIED  : G. MARLEAU    1997/02/12
C    3) MODIFIED  : G. MARLEAU    2000/02/29
C                   EXTRACTED FROM EVO FOR IMPROVED CLARITY
C----
      IMPLICIT         NONE
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      REAL             CSEC,CDAY,CYEAR
      PARAMETER       (IOUT=6,CSEC=1.0E-8,CDAY=8.64E-4,
     >                 CYEAR=3.1536E-1,NAMSBR='EVOGET')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER          IPRINT,ITYPE,ITIXS,IEXTR,IGLOB,INR
      REAL             RPAR(5),XT(5)
C----
C  INPUT FILE PARAMETERS
C----
      CHARACTER        CARLIR*12
      INTEGER          ITYPLU,INTLIR
      REAL             REALIR
      DOUBLE PRECISION DBLLIR
C----
C  LOCAL PARAMETERS
C----
      INTEGER          IRDT,ISAVE,ISET
C----
C  READ THE BURNUP INPUT DATA.
C----
      ISAVE=0
      ISET=0
 100  CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
 101  CONTINUE
      IF(ITYPLU .EQ. 10) GO TO 105
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': CHARACTER KEYWORD EXPECTED.')
      IF(CARLIR(1:4) .EQ. ';') THEN
        GO TO 105
      ELSE IF(CARLIR(1:4) .EQ. 'EDIT') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': INTEGER IPRINT EXPECTED')
        IPRINT=INTLIR
      ELSE IF(CARLIR(1:4) .EQ. 'EPS1') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': REAL EPS1 EXPECTED')
        RPAR(1)=REALIR
      ELSE IF(CARLIR(1:4) .EQ. 'EPS2') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': REAL EPS2 EXPECTED')
        RPAR(2)=REALIR
      ELSE IF(CARLIR(1:4) .EQ. 'EXPM') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': REAL EXPM EXPECTED')
        RPAR(3)=REALIR
      ELSE IF(CARLIR(1:6) .EQ. 'SATOFF') THEN
        RPAR(3)=0.0
      ELSE IF(CARLIR(1:4) .EQ. 'H1') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  'REAL H1 EXPECTED')
        RPAR(4)=REALIR
      ELSE IF(CARLIR(1:4) .EQ. 'RUNG') THEN
        ITYPE=1
      ELSE IF(CARLIR(1:4) .EQ. 'KAPS') THEN
        ITYPE=2
      ELSE IF(CARLIR(1:4) .EQ. 'TIXS') THEN
        ITIXS=1
      ELSE IF(CARLIR(1:4) .EQ. 'TDXS') THEN
        ITIXS=0
      ELSE IF(CARLIR(1:4) .EQ. 'NOEX') THEN
        IEXTR=0
      ELSE IF(CARLIR(1:4) .EQ. 'EXTR') THEN
        IEXTR=1
      ELSE IF(CARLIR(1:4) .EQ. 'NOGL') THEN
        IGLOB=0
      ELSE IF(CARLIR(1:4) .EQ. 'GLOB') THEN
        IGLOB=1
      ELSE IF(CARLIR(1:4) .EQ. 'NOSA') THEN
        ISAVE=-1
      ELSE IF(CARLIR(1:4) .EQ. 'SAVE') THEN
        ISAVE=1
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': SAVE TIME EXPECTED')
        XT(3)=REALIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': TIME UNITS EXPECTED')
        IF(CARLIR(1:4) .EQ. 'S') THEN
          XT(3)=XT(3)*CSEC
        ELSE IF(CARLIR(1:4) .EQ. 'DAY') THEN
          XT(3)=XT(3)*CDAY
        ELSE IF(CARLIR(1:4) .EQ. 'YEAR') THEN
          XT(3)=XT(3)*CYEAR
        ELSE
          CALL XABORT(NAMSBR//': INVALID TIME UNITS')
        ENDIF
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': SAVE NORMALIZATION EXPECTED')
        IF(CARLIR(1:4) .EQ. 'FLUX') THEN
          INR=1
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >    ': BURNUP FLUX LEVEL EXPECTED')
          RPAR(5)=REALIR
          IF(RPAR(5) .EQ. 0.0) THEN
            INR=0
          ENDIF
        ELSE IF(CARLIR(1:4) .EQ. 'POWR') THEN
          INR=2
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >    ': BURNUP POWER EXPECTED')
          RPAR(5)=REALIR
          IF(RPAR(5) .EQ. 0.0) THEN
            INR=0
          ENDIF
        ELSE
          GO TO 101
        ENDIF
      ELSE IF(CARLIR(1:4) .EQ. 'DEPL') THEN
        IRDT=2
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': INITIAL OR INCREMENTAL DEPLETE TIME EXPECTED')
        XT(1)=REALIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .EQ. 1) THEN
          CALL XABORT(NAMSBR//': UNITS OR FINAL TIME EXPECTED')
        ELSE IF(ITYPLU .EQ. 2) THEN
          XT(2)=REALIR
          IRDT=1
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        ENDIF
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': TIME UNITS EXPECTED')
        IF(CARLIR(1:4) .EQ. 'S') THEN
          XT(1)=XT(1)*CSEC
          XT(2)=XT(2)*CSEC
        ELSE IF(CARLIR(1:4) .EQ. 'DAY') THEN
          XT(1)=XT(1)*CDAY
          XT(2)=XT(2)*CDAY
        ELSE IF(CARLIR(1:4) .EQ. 'YEAR') THEN
          XT(1)=XT(1)*CYEAR
          XT(2)=XT(2)*CYEAR
        ELSE
          CALL XABORT(NAMSBR//': INVALID TIME UNITS')
        ENDIF
        IF(IRDT .EQ. 2) THEN
          XT(2)=XT(4)+XT(1)
          XT(1)=XT(4)
        ENDIF
        IF(XT(2) .LE. XT(1)) CALL XABORT(NAMSBR//
     >  ': FINAL TIME IS LESS OR EQUAL TO THE INITIAL TIME.')
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': DEPLETION NORMALIZATION LEVEL EXPECTED')
        IF(CARLIR(1:4) .EQ. 'COOL') THEN
          INR=0
        ELSE IF(CARLIR(1:4) .EQ. 'FLUX') THEN
          INR=1
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >    ': FLUX LEVEL EXPECTED')
          RPAR(5)=REALIR
          IF(RPAR(5) .EQ. 0.0) THEN
            INR=0
          ENDIF
        ELSE IF(CARLIR(1:4) .EQ. 'POWR') THEN
          INR=2
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >    ': POWER LEVEL EXPECTED')
          RPAR(5)=REALIR
          IF(RPAR(5) .EQ. 0.0) THEN
            INR=0
          ENDIF
        ELSE
          GO TO 101
        ENDIF
      ELSE IF(CARLIR(1:4) .EQ. 'SET') THEN
        ISET=1
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': SET TIME EXPECTED')
        XT(5)=REALIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': TIME UNITS EXPECTED')
        IF(CARLIR(1:4) .EQ. 'S') THEN
          XT(5)=XT(5)*CSEC
        ELSE IF(CARLIR(1:4) .EQ. 'DAY') THEN
          XT(5)=XT(5)*CDAY
        ELSE IF(CARLIR(1:4) .EQ. 'YEAR') THEN
          XT(5)=XT(5)*CYEAR
        ELSE
          CALL XABORT(NAMSBR//': INVALID TIME UNITS')
        ENDIF
      ELSE
        CALL XABORT(NAMSBR//': '//CARLIR//' IS AN INVALID KEYWORD')
      ENDIF
      GO TO 100
C----
C  INPUT DATA READ COMPLETE
C----
 105  CONTINUE
      XT(4)=0.0
      IF(ISAVE .EQ. -1) THEN
        XT(4)=-1.0
      ELSE IF(ISAVE .EQ. 0) THEN
        XT(3)=XT(1)
      ENDIF
      IF(ISET .EQ. 0) THEN
        XT(5)=XT(2)
      ENDIF
      RETURN
      END
