*DECK EVOKAP
      SUBROUTINE EVOKAP(Y,N,X,HTRY,EPS,YSCAL,HDID,HNEXT,MU1,IMA,MAXA,
     1 NSUPF,NFISS,KFISS,YSF,ADPL,BDPL,DYDX,TEMP,YSAV,DYSAV,DFDX,AK,ASS)
C
C-----------------------------------------------------------------------
C
C  FOURTH-ORDER KAPS-RENTROP STEP FOR INTEGRATING STIFF O.D.E.'S, WITH
C  MONITORING OF LOCAL TRUNCATION ERROR TO ADJUST STEPSIZE.
C  SPECIAL VERSION FOR ISOTOPIC DEPLETION CALCULATIONS.
C
C  INPUT AND OUTPUT PARAMETERS:
C   Y       : DEPENDENT VARIABLE VECTOR. DIMENSION Y(N)
C   N       : DIMENSION OF THE DEPENDENT VARIABLE VECTOR.
C   X       : INDEPENDENT VARIABLE.
C   HTRY    : STEPSIZE TO BE ATTEMPTED.
C   EPS     : REQUIRED ACCURACY.
C   YSCAL   : VECTOR AGAINST WHICH THE ERROR IS SCALED.
C             DIMENSION YSCAL(N)
C   HDID    : STEPSIZE THAT WAS ACTUALLY ACCOMPLISHED.
C   HNEXT   : ESTIMATED NEXT STEPSIZE.
C
C  INPUT DATA RELATED TO THE ISOTOPIC DEPLETION CALCULATION:
C   MU1     : POSITION OF EACH DIAGONAL ELEMENT IN VECTORS ADPL AND ASS.
C   IMA     : POSITION OF THE FIRST NON-ZERO COLUMN ELEMENT IN VECTORS
C             ADPL AND ASS. DIMENSION MU1(N),IMA(N)
C   MAXA    : FIRST DIMENSION OF ADPL.
C   NSUPF   : NUMBER OF DEPLETING FISSION PRODUCTS.
C   NFISS   : NUMBER OF FISSILE ISOTOPES PRODUCING FISSION PRODUCTS.
C   KFISS   : POSITION IN CHAIN OF THE FISSILE ISOTOPES.
C             DIMENSION KFISS(NFISS)
C   YSF     : THE PRODUCT OF THE FISSION YIELDS AND FISSION
C             RATES. DIMENSION YSF(NFISS,NSUPF,2)
C   ADPL    : DEPLETION MATRIX COMPONENTS. DIMENSION ADPL(MAXA,2)
C   BDPL    : DEPLETION SOURCE COMPONENTS. DIMENSION BDPL(N,2)
C
C
C  SCRATCH STORAGE:
C   DYDX    : DIMENSION DYDX(N)
C   TEMP    : DIMENSION TEMP(N)
C   YSAV    : DIMENSION YSAV(N)
C   DYSAV   : DIMENSION DYSAV(N)
C   DFDX    : DIMENSION DFDX(N)
C   AK      : DIMENSION AK(N,4)
C   ASS     : DIMENSION ASS(IMA(N))
C
C  REFERENCE:
C   W.H. PRESS AND S.A. TEUKOLSKY, 'INTEGRATING STIFF ORDINARY DIFFEREN-
C   TIAL EQUATIONS', COMPUTERS IN PHYSICS, 88 (MAY/JUNE 1989).
C
C-----------------------------------------------------------------------
C
      IMPLICIT NONE
C----
C PARAMETERS
C-----
      INTEGER  MAXTRY
      REAL     SAFETY,GROW,PGROW,SHRNK,PSHRNK,ERRCON,GAM,GAM21,
     1         GAM31,GAM32,GAM41,GAM42,GAM43,ALF21,ALF31,ALF32,
     2         C1,C2,C3,C4,CC1,CC2,CC3,GAM2X,GAM3X,GAM4X
C
      PARAMETER (SAFETY=0.9,GROW=1.5,PGROW=-.25,SHRNK=0.5,PSHRNK=-1./3.,
     1           ERRCON=.1296,MAXTRY=20)
      PARAMETER (GAM=.231,GAM21=-.270629667752/GAM,
     1           GAM31=.311254483294/GAM,GAM32=.852445628482E-2,
     2           GAM41=.282816832044/GAM,GAM42=-.457959483281/GAM,
     3           GAM43=-.111208333333/GAM,ALF21=.462,
     4           ALF31=-.815668168327E-1,ALF32=.961775150166,
     5           C1=.217487371653,C2=.486229037990,C3=0.,
     6           C4=.296283590357,CC1=-.717088504499,CC2=1.77617912176,
     7           CC3=-.590906172617E-1,GAM2X=GAM*(1.+GAM21),
     8           GAM3X=GAM*(1.+GAM31+GAM32),
     9           GAM4X=GAM*(1.+GAM41+GAM42+GAM43))
C----
C INTERFACE VARIABLES
C----
      INTEGER    NFISS,NSUPF,MAXA,N
      INTEGER    MU1(*),IMA(*),KFISS(*)
      REAL       Y(*),YSCAL(*),YSF(NFISS,NSUPF,*),
     1           ADPL(MAXA,*),BDPL(N,*),DYDX(*),TEMP(*),YSAV(*),
     2           DYSAV(*),DFDX(*),AK(N,*),ASS(*)
C----
C LOCAL VARIABLES
C----
      INTEGER    I,J,NSUPL,JTRY
      REAL       XSAV,H,HTRY,X,HSQ,ERRMAX,EPS,HDID,HNEXT
C
C----
C
      XSAV=X
      NSUPL=N-NSUPF
      CALL ALLUM(N,ADPL(1,1),Y,DYDX,MU1,IMA)
      CALL ALLUM(N,ADPL(1,2),Y,TEMP,MU1,IMA)
      DO 10 I=1,NSUPF
      DO 10 J=1,NFISS
      DYDX(NSUPL+I)=DYDX(NSUPL+I)+YSF(J,I,1)*Y(KFISS(J))
   10 TEMP(NSUPL+I)=TEMP(NSUPL+I)+YSF(J,I,2)*Y(KFISS(J))
      DO 20 I=1,N
      DYDX(I)=DYDX(I)+X*TEMP(I)+BDPL(I,1)+X*BDPL(I,2)
      YSAV(I)=Y(I)
   20 DYSAV(I)=DYDX(I)
      H=HTRY
      DO 200 JTRY=1,MAXTRY
      HSQ=H*H
      CALL ALLUM(N,ADPL(1,2),YSAV,DFDX,MU1,IMA)
      DO 30 I=1,NSUPF
      DO 30 J=1,NFISS
   30 DFDX(NSUPL+I)=DFDX(NSUPL+I)+YSF(J,I,2)*YSAV(KFISS(J))
      DO 50 I=1,IMA(N)
   50 ASS(I)=-H*GAM*(ADPL(I,1)+XSAV*ADPL(I,2))
      DO 60 I=1,N
      DFDX(I)=DFDX(I)+BDPL(I,2)
   60 ASS(MU1(I))=1.+ASS(MU1(I))
      CALL ALLUF(N,ASS,MU1,IMA)
      DO 70 I=1,N
   70 AK(I,1)=H*DYSAV(I)+HSQ*GAM*DFDX(I)
      CALL ALLUS(NSUPL,MU1,IMA,ASS,AK(1,1))
      IF(NSUPF.GT.0) THEN
         DO 75 I=1,NSUPF
         DO 75 J=1,NFISS
   75    AK(NSUPL+I,1)=AK(NSUPL+I,1)+H*GAM*(YSF(J,I,1)+XSAV*YSF(J,I,2))
     1   *AK(KFISS(J),1)
         CALL ALLUS(NSUPF,MU1(NSUPL+1),IMA(NSUPL+1),ASS,AK(NSUPL+1,1))
      ENDIF
      DO 80 I=1,N
   80 Y(I)=YSAV(I)+ALF21*AK(I,1)
      X=XSAV+ALF21*H
      CALL ALLUM(N,ADPL(1,1),Y,DYDX,MU1,IMA)
      CALL ALLUM(N,ADPL(1,2),Y,TEMP,MU1,IMA)
      DO 90 I=1,NSUPF
      DO 90 J=1,NFISS
      DYDX(NSUPL+I)=DYDX(NSUPL+I)+YSF(J,I,1)*Y(KFISS(J))
   90 TEMP(NSUPL+I)=TEMP(NSUPL+I)+YSF(J,I,2)*Y(KFISS(J))
      DO 100 I=1,N
      DYDX(I)=DYDX(I)+X*TEMP(I)+BDPL(I,1)+X*BDPL(I,2)
  100 AK(I,2)=H*DYDX(I)+HSQ*GAM2X*DFDX(I)+GAM21*AK(I,1)
      CALL ALLUS(NSUPL,MU1,IMA,ASS,AK(1,2))
      IF(NSUPF.GT.0) THEN
         DO 105 I=1,NSUPF
         DO 105 J=1,NFISS
  105    AK(NSUPL+I,2)=AK(NSUPL+I,2)+H*GAM*(YSF(J,I,1)+XSAV*YSF(J,I,2))
     1   *AK(KFISS(J),2)
         CALL ALLUS(NSUPF,MU1(NSUPL+1),IMA(NSUPL+1),ASS,AK(NSUPL+1,2))
      ENDIF
      DO 110 I=1,N
      AK(I,2)=AK(I,2)-GAM21*AK(I,1)
  110 Y(I)=YSAV(I)+ALF31*AK(I,1)+ALF32*AK(I,2)
      X=XSAV+(ALF31+ALF32)*H
      CALL ALLUM(N,ADPL(1,1),Y,DYDX,MU1,IMA)
      CALL ALLUM(N,ADPL(1,2),Y,TEMP,MU1,IMA)
      DO 120 I=1,NSUPF
      DO 120 J=1,NFISS
      DYDX(NSUPL+I)=DYDX(NSUPL+I)+YSF(J,I,1)*Y(KFISS(J))
  120 TEMP(NSUPL+I)=TEMP(NSUPL+I)+YSF(J,I,2)*Y(KFISS(J))
      DO 130 I=1,N
      DYDX(I)=DYDX(I)+X*TEMP(I)+BDPL(I,1)+X*BDPL(I,2)
      TEMP(I)=GAM31*AK(I,1)+GAM32*AK(I,2)
  130 AK(I,3)=H*DYDX(I)+GAM3X*HSQ*DFDX(I)+TEMP(I)
      CALL ALLUS(NSUPL,MU1,IMA,ASS,AK(1,3))
      IF(NSUPF.GT.0) THEN
         DO 135 I=1,NSUPF
         DO 135 J=1,NFISS
  135    AK(NSUPL+I,3)=AK(NSUPL+I,3)+H*GAM*(YSF(J,I,1)+XSAV*YSF(J,I,2))
     1   *AK(KFISS(J),3)
         CALL ALLUS(NSUPF,MU1(NSUPL+1),IMA(NSUPL+1),ASS,AK(NSUPL+1,3))
      ENDIF
      DO 140 I=1,N
      AK(I,3)=AK(I,3)-TEMP(I)
      TEMP(I)=GAM41*AK(I,1)+GAM42*AK(I,2)+GAM43*AK(I,3)
  140 AK(I,4)=H*DYDX(I)+HSQ*GAM4X*DFDX(I)+TEMP(I)
      CALL ALLUS(NSUPL,MU1,IMA,ASS,AK(1,4))
      IF(NSUPF.GT.0) THEN
         DO 145 I=1,NSUPF
         DO 145 J=1,NFISS
  145    AK(NSUPL+I,4)=AK(NSUPL+I,4)+H*GAM*(YSF(J,I,1)+XSAV*YSF(J,I,2))
     1   *AK(KFISS(J),4)
         CALL ALLUS(NSUPF,MU1(NSUPL+1),IMA(NSUPL+1),ASS,AK(NSUPL+1,4))
      ENDIF
      DO 150 I=1,N
      AK(I,4)=AK(I,4)-TEMP(I)
      Y(I)=YSAV(I)+C1*AK(I,1)+C2*AK(I,2)+C3*AK(I,3)+C4*AK(I,4)
  150 TEMP(I)=YSAV(I)+CC1*AK(I,1)+CC2*AK(I,2)+CC3*AK(I,3)
      X=XSAV+H
      IF (X.EQ.XSAV) CALL XABORT('EVOKAP: STEPSIZE NOT SIGNIFICANT.')
      ERRMAX=0.
      DO 160 I=1,N
  160 ERRMAX=MAX(ERRMAX,ABS((Y(I)-TEMP(I))/YSCAL(I)))
      ERRMAX=ERRMAX/EPS
      IF (ERRMAX.LE.1.) THEN
         HDID=H
         IF (ERRMAX.GT.ERRCON) THEN
            HNEXT=SAFETY*H*ERRMAX**PGROW
         ELSE
            HNEXT=GROW*H
         ENDIF
         RETURN
      ELSE
         HNEXT=SAFETY*H*ERRMAX**PSHRNK
         IF (HNEXT.LT.SHRNK*H) HNEXT=SHRNK*H
         H=HNEXT
      ENDIF
  200 CONTINUE
      CALL XABORT('EVOKAP: EXCEEDED MAXTRY.')
      END
