*DECK EVOODE
      SUBROUTINE EVOODE(YSTART,NVAR,X1,X2,EPS,H1,NOK,NBAD,ITYPE,MU1,
     1 IMA,MAXA,NSUPF,NFISS,KFISS,YSF,ADPL,BDPL,YSCAL,Y)
C
C-----------------------------------------------------------------------
C
C  RUNGE-KUTTA OR KAPS-RENTROP DRIVER WITH ADAPTIVE STEPSIZE CONTROL.
C  SPECIAL VERSION FOR ISOTOPIC DEPLETION CALCULATIONS.
C
C  INPUT AND OUTPUT PARAMETERS:
C   YSTART  : DEPENDENT VARIABLE VECTOR. DIMENSION YSTART(NVAR)
C   NVAR    : DIMENSION OF THE DEPENDENT VARIABLE VECTOR (NUMBER OF
C             DEPLETING ISOTOPES).
C   X1,X2   : INITIAL AND FINAL VALUE OF THE INDEPENDENT VARIABLE.
C   EPS     : REQUIRED ACCURACY.
C   H1      : GUESSED FIRST STEPSIZE.
C   NOK     : NUMBER OF GOOD STEPS TAKEN.
C   NBAD    : NUMBER OF BAD STEPS TAKEN.
C   ITYPE   : =1 FIFTH-ORDER RUNGE-KUTTA METHOD.
C             =2 FOURTH-ORDER KAPS-RENTROP METHOD.
C
C  INPUT DATA RELATED TO THE ISOTOPIC DEPLETION CALCULATION:
C   MU1     : POSITION OF EACH DIAGONAL ELEMENT IN VECTOR ADPL.
C   IMA     : POSITION OF THE FIRST NON-ZERO COLUMN ELEMENT IN VECTOR
C             ADPL. DIMENSION MU1(NVAR),IMA(NVAR)
C   MAXA    : FIRST DIMENSION OF ADPL.
C   NSUPF   : NUMBER OF DEPLETING FISSION PRODUCTS.
C   NFISS   : NUMBER OF FISSILE ISOTOPES PRODUCING FISSION PRODUCTS.
C   KFISS   : POSITION IN CHAIN OF THE FISSILE ISOTOPES.
C             DIMENSION KFISS(NFISS)
C   YSF     : THE PRODUCT OF THE FISSION YIELDS AND FISSION
C             RATES. DIMENSION YSF(NFISS,NSUPF,2)
C   ADPL    : DEPLETION MATRIX COMPONENTS. DIMENSION ADPL(MAXA,2)
C   BDPL    : DEPLETION SOURCE COMPONENTS. DIMENSION BDPL(NVAR,2)
C
C  SCRATCH STORAGE:
C   YSCAL   : DIMENSION YSCAL(NVAR)
C   Y       : DIMENSION Y(NVAR)
C
C  REFERENCE:
C   W.H. PRESS, B.P. FLANNERY, S.A. TEUKOLSKY AND W.T. VETTERLING,
C   'NUMERICAL RECIPIES (FORTRAN VERSION)', CAMBRIDGE UNIVERSITY PRESS,
C   CHAPTER 15, CAMBRIDGE (1990).
C
C--------------------------------------- AUTHOR: A. HEBERT 18/06/90 ----
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER    MAXSTP
      REAL       HMIN,SMIN
      PARAMETER (MAXSTP=50000,HMIN=0.0,SMIN=1.0E-2)
C----
C MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL         BASE(1)
      COMMON       BASE
      EQUIVALENCE (BASE(1),IBASE(1))
      INTEGER     IDYDX,ITEMP,IYSAV,IDYSAV,IDFDX,IAK,IASS
C----
C INTERFACE VARIABLES
C----
      INTEGER    NFISS,NSUPF,MAXA,NVAR
      INTEGER    MU1(*),IMA(*),KFISS(*)
      REAL       YSTART(*),YSF(NFISS,NSUPF,*),
     1           ADPL(MAXA,2),BDPL(NVAR,2),YSCAL(*),Y(*)
C----
C LOCAL VARIABLES
C----
      INTEGER    I,NOK,NBAD,NSTP,ITYPE
      REAL       X,X1,H,H1,X2,EPS,HDID,HNEXT
C
C----
C
      X=X1
      H=SIGN(H1,X2-X1)
      NOK=0
      NBAD=0
      DO 11 I=1,NVAR
        Y(I)=YSTART(I)
11    CONTINUE
      CALL SETARA(BASE,NVAR,IDYDX)
      CALL SETARA(BASE,NVAR,ITEMP)
      CALL SETARA(BASE,NVAR,IYSAV)
      CALL SETARA(BASE,NVAR,IDYSAV)
      CALL SETARA(BASE,NVAR,IDFDX)
      CALL SETARA(BASE,4*NVAR,IAK)
      CALL SETARA(BASE,IMA(NVAR),IASS)
      DO 16 NSTP=1,MAXSTP
        DO 12 I=1,NVAR
          YSCAL(I)=MAX(SMIN,ABS(Y(I)))
12      CONTINUE
        IF((X+H-X2)*(X+H-X1).GT.0.0) H=X2-X
        IF (ITYPE.EQ.1) THEN
           CALL EVORK(Y,NVAR,X,H,EPS,YSCAL,HDID,HNEXT,MU1,IMA,MAXA,
     1     NSUPF,NFISS,KFISS,YSF,ADPL,BDPL,BASE(IDYDX),BASE(ITEMP),
     2     BASE(IYSAV),BASE(IDYSAV),BASE(IDFDX),BASE(IAK))
        ELSE IF (ITYPE.EQ.2) THEN
           CALL EVOKAP(Y,NVAR,X,H,EPS,YSCAL,HDID,HNEXT,MU1,IMA,MAXA,
     1     NSUPF,NFISS,KFISS,YSF,ADPL,BDPL,BASE(IDYDX),BASE(ITEMP),
     2     BASE(IYSAV),BASE(IDYSAV),BASE(IDFDX),BASE(IAK),BASE(IASS))
        ENDIF
        IF(HDID.EQ.H)THEN
          NOK=NOK+1
        ELSE
          NBAD=NBAD+1
        ENDIF
        IF((X-X2)*(X2-X1).GE.0.0)THEN
          DO 14 I=1,NVAR
            YSTART(I)=Y(I)
14        CONTINUE
          CALL RLSARA(BASE(IASS))
          CALL RLSARA(BASE(IAK))
          CALL RLSARA(BASE(IDFDX))
          CALL RLSARA(BASE(IDYSAV))
          CALL RLSARA(BASE(IYSAV))
          CALL RLSARA(BASE(ITEMP))
          CALL RLSARA(BASE(IDYDX))
          RETURN
        ENDIF
        IF(ABS(HNEXT).LT.HMIN) CALL XABORT('EVOODE: STEPSIZE SMALLER T'
     1  //'HAN MINIMUM.')
        H=HNEXT
16    CONTINUE
      CALL XABORT('EVOODE: TOO MANY STEPS.')
      END
