*DECK EVOPRE
      SUBROUTINE EVOPRE(NDEPL,NDFI,NDFP,NBXR,NHEAVY,IDR,KPAR,BPAR,
     1                  YIELD,SIG,QF,LCOOL,NDEPLT,NID,NIF,NIH,NIL,
     2                  LP,MU1,IMA,MXDADP,ADPL,BDPL,KFISS,YSF)
C
C-----------------------------------------------------------------------
C
C  STORE DEPLETION MATRIX IN SPARSE MODE FOR A SINGLE MIXTURE.
C
C  INPUT PARAMETERS:
C   NDEPL   : NUMBER OF DEPLETING NUCLIDES.
C   NDFI    : NUMBER OF DIRECT FISSILE ISOTOPES.
C   NDFP    : NUMBER OF DIRECT FISSION PRODUCTS.
C   NHEAVY  : NUMBER OF HEAVY ELEMENTS.
C   NBXR    : MAXIMUM NUMBER OF PARENT REACTION.
C   IDR     : REACTION IDENTIFIER. IDR(MAXR,NDEPL)
C   KPAR    : REDUCED REACTION TYPE MATRIX. KPAR(NBXR,NDEPL)
C   BPAR    : REDUCED BRANCHING RATIO MATRIX. BPAR(NBXR,NDEPL)
C   YIELD   : FISSION YIELD. YIELD(NDFI,NDFP)
C   SIG     : REACTION RATES. SIG(NDEPL,5,2)
C   QF      : FISSION ENERGY QF(NHEAVY)
C   LCOOL   : LOGICAL FOR OUT OF CORE DEPLETION
C   NDEPLT  : NUMBER OF DEPLETING ISOTOPES TO TREAT
C   NID     : NUMBER OF DEPLETING ISOTOPES IN THIS FUEL REGION
C   NIF     : NUMBER OF FISSILE ISOTOPES IN THIS FUEL REGION
C   NIH     : NUMBER OF HEAVY ISOTOPES IN THIS FUEL REGION
C   NIL     : NUMBER OF FISSILE PRODUCTS IN THIS FUEL REGION
C   LP      : DIMENSION LP(NDEPL)
C   MU1     : POSITION  DIAGONAL ELEMENT IN  ADPL. MU1(NID+1)
C   IMA     : POSITION OF THE FIRST ELEMENT IN  ADPL. IMA(NID+1)
C   MXDADP  : FIRST DIMENSION OF VECTOR ADPL.
C
C  OUTPUT PARAMETERS:
C   KFISS   : ISOTOPE ID FOR FISSILE ISOTOPE KFISS(NIF)
C   ADPL    : DEPLETION MATRIX IN SPARSE FORMAT. ADPL(MXDADP,2)
C   BDPL    : DIMENSION BDPL(NID+1,2)
C   YSF     : FISSION PRODUCTION MATRIX YSF(NIF,NIL+1,2)
C
C-------------------------------------- AUTHOR: A. HEBERT  17/07/90 ----
C-------------------------------------- MODIF : G. MARLEAU 97/02/14 ----
C-------------------------------------- MODIF : G. MARLEAU 98/11/10 ----
C                             CORRECT FOR HEAVY FISSION PRODUCT (WIMSD4)
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER           IUNOUT,MAXR,MXTIMS,KFISSI,KFISSP,KDECAY,IFISSH
      PARAMETER        (IUNOUT=6,MAXR=5,MXTIMS=2,KFISSI=6,KFISSP=2,
     >                  KDECAY=1,IFISSH=1)
C----
C INTERFACE AND LOCAL VARIABLES
C----
      INTEGER           NDEPL,NDFI,NDFP,NBXR,NHEAVY,NDEPLT,
     1                  NID,NIF,NIH,NIL,IFI,IDEPL,IP,IXR,JDEPL,
     2                  KREAC,IFPS,IFIS,ILOC,ISOF,ISOFP,IFP
      INTEGER           IDR(MAXR,NDEPL),KPAR(NBXR,NDEPL),
     1                  LP(NDEPL),MU1(NID+1),
     2                  IMA(NID+1),MXDADP,KFISS(NIF)
C
      REAL              RREAC
      REAL              BPAR(NBXR,NDEPL),YIELD(NDFI,NDFP),
     1                  SIG(NDEPL,5,MXTIMS),QF(NHEAVY),
     2                  ADPL(MXDADP,MXTIMS),BDPL(NID+1,MXTIMS),
     3                  YSF(NIF,NIL+1,MXTIMS)
      LOGICAL           LCOOL
C----
C  BUILT ADPL AND BDPL VECTORS AT XT(1) AND XT(2)
C  BUILT YSF AND KFISS VECTORS
C----
      CALL XDRSET(ADPL,MXDADP*MXTIMS,0.0)
      CALL XDRSET(BDPL,(NID+1)*MXTIMS,0.0)
      CALL XDRSET(YSF,NIF*(NIL+1)*MXTIMS,0.0)
      IFI=0
      DO 100 IDEPL=1,NDEPLT
        IF((LP(IDEPL).EQ.0).OR.(LP(IDEPL).GT.NID)) GO TO 105
C----
C  DIAGONAL COMPONENT OF ADPL MATRIX - DEPLETION
C----
        DO 101 IP=1,MXTIMS
          ADPL(MU1(LP(IDEPL)),IP)=-(SIG(IDEPL,1,IP)+SIG(IDEPL,2,IP)+
     >                              SIG(IDEPL,3,IP)+SIG(IDEPL,4,IP)+
     >                              SIG(IDEPL,5,IP))
 101    CONTINUE
C----
C  OFF DIAGONAL COMPONENT OF ADPL MATRIX - CREATION OTHER THAN FISSION
C  98/11/10 - CORRECT FOR HEAVY FISSION PRODUCT (WIMSD4)
C             TO SIMULATE N->2N REACTION
C----
        DO 110 IXR=1,NBXR
          JDEPL=KPAR(IXR,IDEPL)/10
          KREAC=MOD(KPAR(IXR,IDEPL),10)
          IF(LCOOL.AND.KREAC.NE.KDECAY) GO TO 115
          RREAC=BPAR(IXR,IDEPL)
C----
C  98/11/10 - CORRECT FOR HEAVY FISSION PRODUCT (WIMSD4)
C             TO SIMULATE N->2N REACTION
C----
          IF(KREAC .EQ. KFISSP ) THEN
            IF(JDEPL .EQ. IFISSH) THEN
              IFPS=IDR(5,IDEPL)/10
              IF(IFPS .GT. 0) THEN
                DO 350 IFIS=1,NDFI
                  RREAC=YIELD(IFIS,IFPS)
                  IF(RREAC .NE. 0.0) THEN
                    DO 351 JDEPL =1,NDEPLT
                      IF(MOD(IDR(5,JDEPL),10) .EQ. KFISSI  .AND.
     >                   IDR(5,JDEPL)/10      .EQ. IFIS  ) THEN
                        IF(LP(JDEPL) .EQ.   0 .OR.
     >                     LP(JDEPL) .GT. NID) GO TO 355
                        IF(JDEPL.LE.IDEPL) THEN
                          ILOC=MU1(LP(IDEPL))-LP(IDEPL)+LP(JDEPL)
                        ELSE
                          ILOC=MU1(LP(JDEPL))+LP(JDEPL)-LP(IDEPL)
                        ENDIF
                        DO 352 IP=1,MXTIMS
                          ADPL(ILOC,IP)=RREAC*SIG(JDEPL,KFISSP,IP)
 352                    CONTINUE
                      ENDIF
 355                  CONTINUE
 351                CONTINUE
                  ENDIF
 350            CONTINUE
              ENDIF
            ENDIF
          ELSE IF(JDEPL.GT.0) THEN
            IF(LP(JDEPL).EQ.0.OR.LP(JDEPL).GT.NID) GO TO 115
            IF(JDEPL.LE.IDEPL) THEN
              ILOC=MU1(LP(IDEPL))-LP(IDEPL)+LP(JDEPL)
            ELSE
              ILOC=MU1(LP(JDEPL))+LP(JDEPL)-LP(IDEPL)
            ENDIF
            DO 111 IP=1,MXTIMS
              ADPL(ILOC,IP)=RREAC*SIG(JDEPL,KREAC,IP)
 111        CONTINUE
          ENDIF
 115      CONTINUE
 110    CONTINUE
C----
C  COMPONENT OF YSF MATRIX - CREATION BY FISSION
C----
        IF(MOD(IDR(5,IDEPL),10).EQ.KFISSI) THEN
          ISOF=IDR(5,IDEPL)/10
          IFI=IFI+1
          IF(IFI.GT.NIF) CALL XABORT
     >      ('EVOPRE: INVALID NUMBER OF FISSILE ISOTOPES')
          IF(IDEPL.GT.NHEAVY) CALL XABORT
     >      ('EVOPRE: INVALID NUMBER OF FISSILE ISOTOPES')
          KFISS(IFI)=LP(IDEPL)
          DO 120 IP=1,MXTIMS
            YSF(IFI,NIL+1,IP)=SIG(IDEPL,2,IP)*QF(IDEPL)
 120      CONTINUE
          IF(ISOF.NE.0) THEN
            DO 121 JDEPL=1,NDEPLT
              IF((LP(JDEPL).EQ.0).OR.(LP(JDEPL).GT.NID)) GO TO 125
              IF(MOD(IDR(5,JDEPL),10).EQ.KFISSP) THEN
                DO 360 IXR=1,NBXR
                  IF(MOD(KPAR(IXR,JDEPL),10) .EQ. KFISSP .AND.
     >               KPAR(IXR,JDEPL)/10      .NE. IFISSH) THEN
                    ISOFP=IDR(5,JDEPL)/10
                    IFP=LP(JDEPL)-NIH
                    IF(IFP.GT.NIL) CALL XABORT
     >                ('EVOPRE: INVALID NUMBER OF FISSION PRODUCTS')
                    DO 122 IP=1,MXTIMS
                      YSF(IFI,IFP,IP)=SIG(IDEPL,KFISSP,IP)
     >                               *YIELD(ISOF,ISOFP)
 122                CONTINUE
                  ENDIF
 360            CONTINUE
              ENDIF
 125          CONTINUE
 121        CONTINUE
          ENDIF
        ENDIF
 105    CONTINUE
 100  CONTINUE
      RETURN
      END
