*DECK EVORCP
      SUBROUTINE EVORCP(IMPX,NCOMB,NDEPL,NDFI,NDFP,NBXR,EXPMAX,XT,IDR,
     >                  RER,KPAR,YIELD,SIG,LCOOL,NDEPLT,NID,NIF,LP,
     >                  NGROUP,MU1,IMA,IPERM,DIAG)
C
C-----------------------------------------------------------------------
C
C  DETERMINE THE ROW AND COLUMN PROFILE OF THE ADPL MATRIX
C
C  INPUT AND OUTPUT PARAMETERS:
C   IMPX    : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
C   NCOMB   : NUMBER OF DEPLETING MIXTURES.
C   NDEPL   : NUMBER OF DEPLETING NUCLIDES
C   NDFI    : NUMBER OF DIRECT FISSILE ISOTOPES.
C   NDFP    : NUMBER OF DIRECT FISSION PRODUCTS.
C   NBXR    : MAXIMUM NUMBER OF PARENT REACTION.
C   EXPMAX  : ISOTOPE SATURATING LIMIT.
C             EXPMAX=0.0 MEANS THAT THE SATURATION
C             MODEL IS NOT USED 
C   XT      : INITIAL AND FINAL TIME FOR THE DEPLETION. XT(2)
C   IDR     : REACTION IDENTIFIER. IDR(MAXR,NDEPL)
C   RER     : REACTION CONSTANT. RER(MAXR,NDEPL)
C   KPAR    : REDUCED REACTION TYPE MATRIX. KPAR(NBXR,NDEPL)
C   YIELD   : FISSION YIELD. YIELD(NDFI,NDFP)
C   SIG     : DIMENSION SIG(NDEPL,5,2,NGROUP)
C   LCOOL   : LOGICAL FOR OUT OF CORE DEPLETION
C   NDEPLT  : NUMBER OF DEPLETING ISOTOPES TO TREAT
C   NID     : NUMBER OF DEPLETING ISOTOPES IN THIS FUEL REGION
C   NIF     : NUMBER OF FISSILE ISOTOPES IN THIS FUEL REGION
C   LP      : DIMENSION LP(NDEPL)
C
C  SCRATCH STORAGE:
C   MU1     : DIMENSION MU1(NDEPL+1)
C   IMA     : DIMENSION IMA(NDEPL+1)
C   IPERM   : DIMENSION IPERM(NDEPLT)
C   DIAG    : DIMENSION DIAG(NDEPLT)
C
C-------------------------------------- AUTHOR: G. MARLEAU 96/08/29 ----
C-------------------------------------- MODIF : A. HEBERT  96/12/11 ----
C-------------------------------------- MODIF : G. MARLEAU 97/02/13 ----
C-------------------------------------- MODIF : G. MARLEAU 98/11/10 ----
C                             CORRECT FOR HEAVY FISSION PRODUCT (WIMSD4)
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER           IOUT,MAXR,KCAPTU,KDECAY,KFISSP,KFISSI,
     >                  KN2N,KN3N,IFISSH,NGROUP
      PARAMETER        (IOUT=6,MAXR=5,KCAPTU=3,KDECAY=1,KFISSP=2,
     >                  KFISSI=6,KN2N=4,KN3N=5,IFISSH=1)
C-----
C LOCAL AND INTERFACE VARIABLES
C-----
      INTEGER           IMPX,NCOMB,NDEPL,NDFI,NDFP,NBXR,NDEPLT,NID,NIF,
     >                  IDEP,IXR,IREA,JDEP,IFP,IFI,IDI,I,NTER,INDSAT,
     >                  N,MAXA,ITER,JMIN,JMAX,J,II,IS,MAXROW,MAXCOL,
     >                  IMIN,IMAX,NFISS2,NVARM,IMAM1
      INTEGER           LP(NDEPL),IDR(MAXR,NDEPL),KPAR(NBXR,NDEPL),
     >                  MU1(NDEPL+1),IMA(NDEPL+1),IPERM(NDEPLT)
C
      REAL              SIGE1,SIGE2,RATD
      REAL              EXPMAX,XT(2),RER(MAXR,NDEPL),YIELD(NDFI,NDFP),
     >                  SIG(NDEPL,5,2,NGROUP),DIAG(NDEPLT)
C
      CHARACTER         SHOW(65,65)*2
      LOGICAL           LCOOL
      INTEGER           IG
C----
C  DETERMINE THE ROW AND COLUMN PROFILE OF THE ADPL MATRIX
C  PRODUCTION FROM ALL REACTION EXCEPT FROM FISSION
C----
      DO 330 IDEP=1,NDEPLT
        DO 340 IXR=1,NBXR
          IREA=MOD(KPAR(IXR,IDEP),10)
          JDEP=KPAR(IXR,IDEP)/10
          IF(IREA.EQ.0) GO TO 335
          IF(JDEP.EQ.0) GO TO 345
          IF(LCOOL.AND.IREA.NE.KDECAY) GO TO 345
C----
C  98/11/10 - CORRECT FOR HEAVY FISSION PRODUCT (WIMSD4)
C             TO SIMULATE N->2N REACTION
C----
          IF(IREA .EQ. KFISSP) THEN
            IF(JDEP.EQ.IFISSH) THEN
              IFP=IDR(5,IDEP)/10
              IF(IFP .GT. 0) THEN
                DO 350 IFI=1,NDFI
                  IF(YIELD(IFI,IFP) .NE. 0.0) THEN
                    DO 351 JDEP =1,NDEPLT
                      IF(MOD(IDR(5,JDEP),10) .EQ. KFISSI .AND.
     >                   IDR(5,JDEP)/10      .EQ. IFI   ) THEN
                         IF(LP(JDEP).NE.0) THEN
                           MU1(LP(IDEP))=MAX(MU1(LP(IDEP)),
     >                                       LP(IDEP)-LP(JDEP)+1)
                           IMA(LP(JDEP))=MAX(IMA(LP(JDEP)),
     >                                       LP(JDEP)-LP(IDEP)+1)
                         ENDIF
                      ENDIF
 351                CONTINUE
                  ENDIF
 350            CONTINUE
              ENDIF
            ENDIF
          ELSE
            IF(LP(JDEP).NE.0) THEN
              MU1(LP(IDEP))=MAX(MU1(LP(IDEP)),LP(IDEP)-LP(JDEP)+1)
              IMA(LP(JDEP))=MAX(IMA(LP(JDEP)),LP(JDEP)-LP(IDEP)+1)
            ENDIF
          ENDIF
 345      CONTINUE
 340    CONTINUE
 335    CONTINUE
 330  CONTINUE
C----
C  FIND SATURATING FACTOR FOR DEPLETING ISOTOPES
C  IDENTIFY FISSILE ISOTOPES AND FISSION PRODUCTS
C----
      IDI=0
      NIF=0
      DO 400 IDEP=1,NDEPLT
        DIAG(IDEP)=0.0
        IF(LP(IDEP).NE.0) THEN
          RATD=0.0
          IF(MOD(IDR(4,IDEP),10).EQ.KDECAY) THEN
            RATD=RER(4,IDEP)
          ENDIF
          SIGE1=RATD
          SIGE2=RATD
          IF(.NOT.LCOOL) THEN
            DO IG=1,NGROUP
              SIGE1=SIGE1+SIG(IDEP,1,1,IG)+SIG(IDEP,2,1,IG)+
     >              SIG(IDEP,3,1,IG)+SIG(IDEP,4,1,IG)+SIG(IDEP,5,1,IG)
              SIGE2=SIGE2+SIG(IDEP,1,2,IG)+SIG(IDEP,2,2,IG)+
     >              SIG(IDEP,3,2,IG)+SIG(IDEP,4,2,IG)+SIG(IDEP,5,2,IG)
            ENDDO
          ENDIF
          DIAG(IDEP)=(MIN(SIGE1,SIGE2))*(XT(2)-XT(1))
          IDI=IDI+1
          IF(MOD(IDR(5,IDEP),10).EQ.KFISSI) THEN
            NIF=NIF+1
          ENDIF
        ENDIF
 400  CONTINUE
C----
C  COMPUTE THE LUMPING INDEX VECTOR IPERM.
C----
      DO 410 I=1,NID
        IPERM(I)=I
 410  CONTINUE
      NTER=0
 420  CONTINUE
      NTER=NTER+1
      INDSAT=0
      DO 421 IDEP=1,NDEPLT
        IF(LP(IDEP).GT.0) THEN
          IF(IPERM(LP(IDEP)).GE.0) THEN
            IF(EXPMAX .GT. 0.0 .AND. DIAG(IDEP).GT.EXPMAX) THEN
              IF(MOD(IDR(5,IDEP),10).NE.KFISSI) THEN
                IPERM(LP(IDEP))=0
                IF(INDSAT.EQ.0) THEN
                  IPERM(LP(IDEP))=-NTER
                  INDSAT=LP(IDEP)
                ENDIF
              ENDIF
            ENDIF
          ENDIF
        ENDIF
 421  CONTINUE
      IF(INDSAT.NE.0) THEN
        DO 430 I=INDSAT+1,NID
          DO 431 J=MIN(I-MU1(I)+1,I-IMA(I)+1),I-1
            IF((IPERM(I).EQ.0).AND.(IPERM(J).EQ.-NTER)) THEN
              IPERM(I)=-NTER
              GO TO 435
            ENDIF
 431      CONTINUE
 435      CONTINUE
 430    CONTINUE
        GO TO 420
      ELSE
        NTER=NTER-1
      ENDIF
      N=0
      DO 440 I=1,NID
        IF(IPERM(I).GT.0) THEN
          N=N+1
          IPERM(I)=N
        ENDIF
 440  CONTINUE
C----
C  COMPUTE MAXIMUM DIMENSION OF SATURATION VECTOR MAXA
C----
      MAXA=0
      DO 450 ITER=1,NTER
        JMIN=NID
        JMAX=1
        DO 460 I=1,NID
          IF(-IPERM(I).EQ.ITER) THEN
            DO 461 J=1,NID
              IF( ((J.LE.I).AND.(J.GT.I-MU1(I))) .OR.
     >            ((I.LE.J).AND.(I.GT.J-IMA(J))) ) THEN
                JMIN=MIN(JMIN,J)
                JMAX=MAX(JMAX,J)
              ENDIF
 461        CONTINUE
          ENDIF
 460    CONTINUE
        IMIN=NID
        IMAX=1
        DO 470 I=1,NID
          DO 471 J=1,NID
            IF(-IPERM(J).EQ.ITER) THEN
              IF( ((J.LE.I).AND.(J.GT.I-MU1(I))) .OR.
     >            ((I.LE.J).AND.(I.GT.J-IMA(J))) ) THEN
                IMIN=MIN(IMIN,I)
                IMAX=MAX(IMAX,I)
              ENDIF
            ENDIF
 471      CONTINUE
 470    CONTINUE
        DO 480 I=IMIN,IMAX
          IF(-IPERM(I).NE.ITER) THEN
            DO 481 J=JMIN,JMAX
              IF(-IPERM(J).EQ.ITER) GO TO 481
              IF((J.LE.I).AND.(J.GT.I-MU1(I))) GO TO 481
              IF((I.LE.J).AND.(I.GE.J-IMA(J))) GO TO 481
              MAXA=MAXA+1
  481       CONTINUE
          ENDIF
  480   CONTINUE
  450 CONTINUE
      MAXA=MAXA+IMA(NID)+1
      MAXROW=1
      MAXCOL=1
      II=0
      DO 490 IS=1,NID
        MAXROW=MAX(MAXROW,MU1(I))
        MAXCOL=MAX(MAXCOL,IMA(I))
        II=II+MU1(IS)
        MU1(IS)=II
        II=II+IMA(IS)-1
        IMA(IS)=II
 490  CONTINUE
      MU1(NID+1)=IMA(NID)+1
      IMA(NID+1)=IMA(NID)+1
      NFISS2=0
      IF(IMPX.GE.5) WRITE(IOUT,6000) MAXROW,MAXCOL
      IF(IMPX.GE.10) THEN
        NVARM=MIN(NID,65)
        WRITE(IOUT,6001)
        DO 600 I=1,NVARM
          DO 601 J=1,NVARM
            SHOW(I,J)=' '
 601      CONTINUE
 600    CONTINUE
        IMAM1=0
        DO 610 I=1,NVARM
          DO 611 J=I-MU1(I)+IMAM1+1,I
            IF(IPERM(I).GT.0) THEN
              SHOW(I,J)='*'
            ELSE
              SHOW(I,J)='+'
            ENDIF
  611     CONTINUE
          DO 612 J=I-IMA(I)+MU1(I),I
            IF(IPERM(J).GT.0) THEN
              SHOW(J,I)='*'
            ELSE
              SHOW(J,I)='+'
            ENDIF
  612     CONTINUE
          IMAM1=IMA(I)
  610   CONTINUE
        DO 620 I=1,NVARM
          WRITE(IOUT,6002) (SHOW(I,J),J=1,NVARM)
  620   CONTINUE
        IF(NID.GT.65) WRITE(IOUT,6003)
      ENDIF
C----
C  RETURN
C----
      RETURN
C----
C  FORMATS
C----
 6000 FORMAT(/' EVORCP: MAXIMUM ROW PROFILE WIDTH    = ',I10
     >       /' EVORCP: MAXIMUM COLUMN PROFILE WIDTH = ',I10)
 6001 FORMAT(/' EVORCP: DEPLETION MATRIX PROFILE')
 6002 FORMAT(1X,65A2)
 6003 FORMAT(' MATRIX TRUNCATED')
      END
