*DECK EVORK4
      SUBROUTINE EVORK4(Y,DYDX,N,X,H,YOUT,MU1,IMA,MAXA,NSUPF,NFISS,
     1 KFISS,YSF,ADPL,BDPL,AK,TEMR)
C
C-----------------------------------------------------------------------
C
C USE THE FOURTH-ORDER RUNGE-KUTTA METHOD TO ADVANCE THE SOLUTION OVER
C AN INTERVAL H AND RETURN THE INCREMENTED VARIABLES AS YOUT.
C SPECIAL VERSION FOR ISOTOPIC DEPLETION CALCULATIONS.
C
C INPUT AND OUTPUT PARAMETERS:
C  Y       : INITIAL DEPENDENT VARIABLE VECTOR. DIMENSION Y(N)
C  DYDX    : DERIVATIVE OF THE INITIAL DEPENDENT VARIABLE VECTOR.
C            DIMENSION DYDX(N)
C  N       : DIMENSION OF THE DEPENDENT VARIABLE VECTOR.
C  X       : INDEPENDENT VARIABLE.
C  H       : STEPSIZE.
C  YOUT    : FINAL DEPENDENT VARIABLE VECTOR. DIMENSION YOUT(N)
C
C INPUT DATA RELATED TO THE ISOTOPIC DEPLETION CALCULATION:
C  MU1     : POSITION OF EACH DIAGONAL ELEMENT IN VECTOR ADPL.
C  IMA     : POSITION OF THE FIRST NON-ZERO COLUMN ELEMENT IN VECTOR
C            ADPL. DIMENSION MU1(N),IMA(N)
C  MAXA    : FIRST DIMENSION OF ADPL.
C  NSUPF   : NUMBER OF DEPLETING FISSION PRODUCTS.
C  NFISS   : NUMBER OF FISSILE ISOTOPES PRODUCING FISSION PRODUCTS.
C  KFISS   : POSITION IN CHAIN OF THE FISSILE ISOTOPES.
C            DIMENSION KFISS(NFISS)
C  YSF     : COMPONENTS OF THE PRODUCT OF THE FISSION YIELDS AND FISSION
C            RATES. DIMENSION YSF(NFISS,NSUPF,2)
C  ADPL    : DEPLETION MATRIX COMPONENTS. DIMENSION ADPL(MAXA,2)
C  BDPL    : DEPLETION SOURCE COMPONENTS. DIMENSION BDPL(N,2)
C
C
C SCRATCH STORAGE:
C  AK      : DIMENSION AK(N,3)
C  TEMR    : DIMENSION TEMR(N)
C
C REFERENCE:
C  W.H. PRESS, B.P. FLANNERY, S.A. TEUKOLSKY AND W.T. VETTERLING,
C  'NUMERICAL RECIPIES (FORTRAN VERSION)', CAMBRIDGE UNIVERSITY PRESS,
C  CHAPTER 15, CAMBRIDGE (1990).
C
C-----------------------------------------------------------------------
C
      IMPLICIT NONE
C----
C INTERFACE VARIABLES
C----
      INTEGER   MAXA,N,NFISS,NSUPF
      INTEGER   MU1(*),IMA(*),KFISS(*)
      REAL      Y(*),DYDX(*),YOUT(*),YSF(NFISS,NSUPF,*),
     1          ADPL(MAXA,*),BDPL(N,*),AK(N,*),TEMR(*)
C----
C LOCAL VARIABLES
C----
      INTEGER   NSUPL,I,J
      REAL      HH,H,H6,XH,X
C
C----
C
      NSUPL=N-NSUPF
      HH=H*0.5
      H6=H/6.
      XH=X+HH
      DO 10 I=1,N
   10 AK(I,1)=Y(I)+HH*DYDX(I)
      CALL ALLUM(N,ADPL(1,1),AK(1,1),AK(1,2),MU1,IMA)
      CALL ALLUM(N,ADPL(1,2),AK(1,1),TEMR,MU1,IMA)
      DO 20 I=1,NSUPF
      DO 20 J=1,NFISS
      AK(NSUPL+I,2)=AK(NSUPL+I,2)+YSF(J,I,1)*AK(KFISS(J),1)
   20 TEMR(NSUPL+I)=TEMR(NSUPL+I)+YSF(J,I,2)*AK(KFISS(J),1)
      DO 30 I=1,N
      AK(I,2)=AK(I,2)+XH*TEMR(I)+BDPL(I,1)+XH*BDPL(I,2)
   30 AK(I,1)=Y(I)+HH*AK(I,2)
      CALL ALLUM(N,ADPL(1,1),AK(1,1),AK(1,3),MU1,IMA)
      CALL ALLUM(N,ADPL(1,2),AK(1,1),TEMR,MU1,IMA)
      DO 40 I=1,NSUPF
      DO 40 J=1,NFISS
      AK(NSUPL+I,3)=AK(NSUPL+I,3)+YSF(J,I,1)*AK(KFISS(J),1)
   40 TEMR(NSUPL+I)=TEMR(NSUPL+I)+YSF(J,I,2)*AK(KFISS(J),1)
      DO 50 I=1,N
      AK(I,3)=AK(I,3)+XH*TEMR(I)+BDPL(I,1)+XH*BDPL(I,2)
      AK(I,1)=Y(I)+H*AK(I,3)
   50 AK(I,3)=AK(I,2)+AK(I,3)
      CALL ALLUM(N,ADPL(1,1),AK(1,1),AK(1,2),MU1,IMA)
      CALL ALLUM(N,ADPL(1,2),AK(1,1),TEMR,MU1,IMA)
      DO 60 I=1,NSUPF
      DO 60 J=1,NFISS
      AK(NSUPL+I,2)=AK(NSUPL+I,2)+YSF(J,I,1)*AK(KFISS(J),1)
   60 TEMR(NSUPL+I)=TEMR(NSUPL+I)+YSF(J,I,2)*AK(KFISS(J),1)
      DO 70 I=1,N
      AK(I,2)=AK(I,2)+(X+H)*TEMR(I)+BDPL(I,1)+(X+H)*BDPL(I,2)
   70 YOUT(I)=Y(I)+H6*(DYDX(I)+AK(I,2)+2.*AK(I,3))
      RETURN
      END
