*DECK FLUALB
      SUBROUTINE FLUALB(IPSYS,NREGIO,MATCOD,VOLUME,KEYFLX,FUNKNO,SUNKNO,
     > SIGS0,SIGT0,F1,F2,WORK)
*
*-----------------------------------------------------------------------
*
*  THIS SUBROUTINE COMPUTES INFORMATION RELATED TO AN ALBEDO SEARCH.
*
*  INPUT PARAMETERS:
*   IPSYS  : POINTER TO THE PIJ (L_PIJ SIGNATURE).
*   NREGIO : TOTAL NUMBER OF VOLUMES IN THE DOMAIN.
*   MATCOD : MIXTURE NUMBER IN EACH VOLUME. DIMENSION MATCOD(NREGIO)
*   VOLUME : VOLUMES. DIMENSION VOLUME(NREGIO)
*   KEYFLX : INDEX POINTING TO THE AVERAGE FLUXES IN VECTOR FUNKNO.
*            DIMENSION KEYFLX(NREGIO)
*   FUNKNO : UNKNOWNS. DIMENSION FUNKNO(NUNKNO) WHERE
*            NUNKNO=MAX(KEYFLX(I))
*   SUNKNO : SOURCES. DIMENSION SUNKNO(NUNKNO)
*   SIGS0  : WITHIN GROUP SCATTERING MACROSCOPIC CROSS SECTIONS OF EACH
*            MIXTURE.
*   SIGT0  : TOTAL MACROSCOPIC CROSS SECTIONS OF EACH MIXTURE.
*            DIMENSION SIGS0(0:IR),SIGT0(0:IR) WHERE IR=MAX(MATCOD(I))
*
*  OUTPUT PARAMETERS:
*   F1     : FIRST PART ON THE NEUTRON FLUX. DIMENSION F1(NREGIO)
*   F2     : SECOND PART ON THE NEUTRON FLUX. DIMENSION F2(NREGIO)
*
*  SCRATCH STORAGE:
*   WORK   : DIMENSION WORK(NREGIO**2)
*
*--------------------------------------- AUTHOR: A. HEBERT 06/10/88 ----
*
      DIMENSION MATCOD(*),VOLUME(*),KEYFLX(*),FUNKNO(*),SUNKNO(*),
     > SIGS0(0:*),SIGT0(0:*),F1(*),F2(*),WORK(*)
*
* READ PIS MATRIX.
      CALL LCMLEN(IPSYS,'DRAGON-PIS',ILCMLN,ITYLCM)
      IF(ILCMLN.EQ.NREGIO) THEN
        CALL LCMGET(IPSYS,'DRAGON-PIS',WORK)
      ELSE
        CALL XABORT('FLUALB: THE ALBS OPTION OF THE ASM: MODULE HAVE N'
     >  //'OT BEEN ACTIVATED.')
      ENDIF
*
      ZNUM=0.0
      ZDEN=0.0
      DO 10 I=1,NREGIO
      ZNUM=ZNUM+VOLUME(I)*FUNKNO(KEYFLX(I))
      ZDEN=ZDEN+VOLUME(I)*SIGT0(MATCOD(I))*WORK(I)
      F1(I)=WORK(I)
   10 F2(I)=0.0
      ZNUM=-ZNUM/ZDEN
*
* READ SCATTERING MODIFIED CP MATRIX.
      CALL LCMLEN(IPSYS,'DRAGON-PCSCT',ILCMLN,ITYLCM)
      IF(ILCMLN.EQ.NREGIO**2) THEN
        CALL LCMGET(IPSYS,'DRAGON-PCSCT',WORK)
      ELSE
        CALL LCMLIB(IPSYS)
        CALL XABORT('FLUALB: THE SCATTERING MODIFIED PIJ ARE ABSENT FR'
     >  //'OM LCM.')
      ENDIF
*
      DO 40 J=1,NREGIO
      SSS=F1(J)*SIGS0(MATCOD(J))
      IOF=(J-1)*NREGIO
      DO 30 I=1,NREGIO
   30 F2(I)=F2(I)+WORK(IOF+I)*SSS
   40 F2(J)=F1(J)+F2(J)
*
      DO 50 I=1,NREGIO
      F2(I)=F2(I)*ZNUM
   50 F1(I)=0.0
      DO 60 J=1,NREGIO
      SSS=SUNKNO(KEYFLX(J))
      IOF=(J-1)*NREGIO
      DO 60 I=1,NREGIO
   60 F1(I)=F1(I)+WORK(IOF+I)*SSS
      RETURN
      END
