*DECK GEO
      SUBROUTINE GEO(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* To analyze and track a geometry data structure using the 
* new EXCELL type procedure.
*
*Copyright:
* Copyright (C) 1989 Ecole Polytechnique de Montreal.
*
*Author(s): 
* A. H\'{e}bert
*
*Parameters: input
* NENTRY  number of data structures transfered to this module.
* HENTRY  name of the data structures.
* IENTRY  data structure type where:
*         \begin{itemize} 
*         \item IENTRY=1 for LCM memory object; 
*         \item IENTRY=2 for XSM file; 
*         \item IENTRY=3 for sequential binary file;
*         \item IENTRY=4 for sequential ASCII file.  
*         \end{itemize}
*JENTRY   access permission for the data structure where:
*         \begin{itemize} 
*         \item JENTRY=0 for a data structure in creation mode;
*         \item JENTRY=1 for a data structure in modifications mode;
*         \item JENTRY=2 for a data structure in read-only mode.
*         \end{itemize} 
* KENTRY  data structure pointer.
* 
*Comments:
* Instructions for the use of the GEO: module:
*   Create or update a geometry 
*   NEWGEO  := GEO: [ NEWGEO ] [ OLDGEO ] :: (geoin1) ;
*   where
*     NEWGEO   : new or updated geometry data structure
*                (signature L_GEOM)
*     OLDGEO   : old geometry data structure
*                (signature L_GEOM)
*     (geoin1) : Processing options
*                (read geometry description from input using GEOIN1)
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  Subroutine arguments
*----
      INTEGER          NENTRY                  	
      CHARACTER        HENTRY(NENTRY)*12
      INTEGER          IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='GEO   ')
*----
*  Local variables
*----
      INTEGER          KCHAR(3),ITYPE,IPGEOM,IMPX,IPGEO2,I1,MAXMIX
      CHARACTER        SIGN*12,OLDGEO*12,NEWGEO*12
C
C PARAMETER VALIDATION.
C
      IF(NENTRY.LT.1) CALL XABORT(NAMSBR//
     >': AT LEAST ONE DATA STRUCTURE REQUIRED')
      IF(IENTRY(1).NE.1.AND.IENTRY(1).NE.2) CALL XABORT(NAMSBR//
     >': FIRST DATA STRUCTURE NOT A LINKED LIST OR XSM FILE')
      IF(JENTRY(1).NE.0.AND.JENTRY(1).NE.1) CALL XABORT(NAMSBR//
     >': FIRST DATA STRUCTURE NOT IN CREATE OR MODIFY MODE')
      ITYPE=JENTRY(1)
      IPGEOM=KENTRY(1)
*
      IMPX=1
      IF((ITYPE.EQ.0).AND.(NENTRY.GT.1)) THEN
*----
*  CREATE A NEW GEOMETRY BASED ON AN EXISTING ONE.
*----
        IF(IENTRY(2).NE.1.AND.IENTRY(2).NE.2) CALL XABORT(NAMSBR//
     >  ': SECOND DATA STRUCTURE NOT A LINKED LIST OR XSM FILE')
        IF(JENTRY(2).NE.2) CALL XABORT(NAMSBR//
     >  ': SECOND DATA STRUCTURE NOT IN READ-ONLY MODE')
         IPGEO2=KENTRY(2)
         OLDGEO=HENTRY(2)
         CALL LCMGET(IPGEO2,'SIGNATURE',KCHAR)
         WRITE(SIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(SIGN.NE.'L_GEOM') CALL XABORT(NAMSBR//
     >': SIGNATURE OF '//OLDGEO//' IS '//SIGN//'. L_GEOM EXPECTED(1).')
         CALL LCMEQU(IPGEO2,IPGEOM)
      ELSE IF(ITYPE.EQ.1) THEN
*        MODIFY AN EXISTING GEOMETRY USING THE SAME NAME.
         CALL LCMGET(IPGEOM,'SIGNATURE',KCHAR)
         OLDGEO=HENTRY(1)
         WRITE(SIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(SIGN.NE.'L_GEOM') CALL XABORT(NAMSBR//
     >': SIGNATURE OF '//OLDGEO//' IS '//SIGN//'. L_GEOM EXPECTED(2).')
      ENDIF
*
      NEWGEO='/'
      CALL GEOIN1 (NEWGEO,IPGEOM,1,IMPX,MAXMIX)
      RETURN
      END
