*DECK GEOIN1
      SUBROUTINE GEOIN1(GEONAM,IPLIST,LEVEL,IMPX,MAXMIX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* To read a level 1 general geometry.
*
*Copyright:
* Copyright (C) 1989 Ecole Polytechnique de Montreal.
*
*Author(s): 
* A. H\'{e}bert
*
*Update(s): 
* 2005/10/14: Modified to generalized the definition of TUBE.
*
*
*Parameters: input
* GEONAM   name of the geometry.
* IPLIST   pointer to the GEOMETRY data structure.
* LEVEL    read level for the geometry.
* IMPX     print level.
*
*Parameters: input/output
* MAXMIX   maximum mixture number in geometry. 
*
*----
*
      IMPLICIT NONE
*----
*  Subroutine arguments
*----
      CHARACTER        GEONAM*12
      INTEGER          IPLIST,LEVEL,IMPX,MAXMIX
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='GEOIN1')
      INTEGER          MAXCOD,MAXHEX,MAXTUR,MAXTYP,MXCL,NSTATE
      PARAMETER       (MAXCOD=10,MAXHEX=9,MAXTUR=12,MAXTYP=30,
     >                 MXCL=500,NSTATE=40)
*----
*  Memory allocation variables
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          IMIX,IMESH,ICYL,ICENT,ICELL,IIGEN,IMERGE,
     >                 ITURN,INS,IRS,IMIXDL,IMIXGR,IMILIE,IFRACT,
     >                 IPOURC,IPROCE,ARPIN
*----
*  Functions
*----
      INTEGER      KDROPN,KDRCLS
*----
*  Variables for input via REDGET
*----
      INTEGER          ITYPLU,INTLIR
      CHARACTER        CARLIR*12
      REAL             REALIR
      DOUBLE PRECISION DBLLIR
*----
*  Local variables
*----
      REAL             ZCODE(6)
      INTEGER          ISTATE(NSTATE),NCODE(6),ICODE(6),KCHAR(3),
     >                 MINMIX,MINICO,ILONG,ITC,
     >                 ITYX,I,J,K,I1,IRLXY,IRLYZ,IFILE,
     >                 IRC,LMESH,LCYL,IKG,JKG,NPIN,MICRO,NG,
     >                 IOFJ,IOFK,NMILG,NSMAX,ISURF,IHEX,IPIN
      INTEGER          IMPX2,MAXMI2
      CHARACTER        COND(MAXCOD)*4,CHEX(MAXHEX)*8,
     >                 CTUR(MAXTUR)*1,TYPE(0:MAXTYP)*16,
     >                 NAMT*12,DIR*1,OLDGEO*12,NEWGEO*12
      LOGICAL          LHEX,LCM
      INTEGER          NBR,NBRR,IREP,II,JREP
*----
*  Data
*----
      SAVE             COND,CHEX,CTUR,TYPE
      DATA             COND
     >      /'VOID','REFL','DIAG','TRAN','SYME',
     1       'ALBE','ZERO','CYLI','ACYL','SSYM'/
      DATA             CHEX
     >      /'S30     ','SA60    ','SB60    ','S90     ','R120    ',
     >       'R180    ','SA180   ','SB180   ','COMPLETE'/
      DATA             CTUR
     >      /'A','B','C','D','E','F','G','H','I','J','K','L'/
      DATA             TYPE
     >      /'VIRTUAL         ','HOMOGENEOUS     ','CARTESIAN 1-D   ',
     >       'TUBE 1-D        ','SPHERE 1-D      ','CARTESIAN 2-D   ',
     >       'TUBE 2-D (Z)    ','CARTESIAN 3-D   ','HEXAGONAL 2-D   ',
     >       'HEXAGONE 3-D (Z)','TUBE 2-D (X)    ','TUBE 2-D (Y)    ',
     >       '                ','                ','                ',
     >       '                ','                ','                ',
     >       '                ','                ','2-D RECT. CELL  ',
     >       '3-D RECT. CELL X','3-D RECT. CELL Y','3-D RECT. CELL X',
     >       '2-D HEX. CELL   ','3-D HEX. CELL Z ','                ',
     >       '                ','                ','                ',
     >       'DO-IT-YOURSELF  '/
*
      MINMIX=0
      MINICO=1
      NPIN=0
      CALL LCMLEN(IPLIST,'SIGNATURE',ILONG,ITYX)
      IF(ILONG.EQ.0) THEN
*        INPUT A NEW GEOMETRY.
         CALL XDISET(ISTATE,NSTATE,0)
         LHEX=.FALSE.
         DO 20 I=1,6
         NCODE(I)=0
         ZCODE(I)=0.0
   20    ICODE(I)=0
      ELSE
*        MODIFY AN EXISTING GEOMETRY.
         CALL LCMGET(IPLIST,'SIGNATURE',KCHAR)
         WRITE(NAMT,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(NAMT.NE.'L_GEOM') THEN
            CALL XABORT(NAMSBR//': SIGNATURE OF '//GEONAM//' IS '
     1      //NAMT//'. L_GEOM EXPECTED.')
         ENDIF
         CALL XDISET(ISTATE,NSTATE,0)
         CALL LCMGET(IPLIST,'STATE-VECTOR',ISTATE)
         LHEX=(ISTATE(1).EQ.8).OR.(ISTATE(1).EQ.9).OR.(ISTATE(1).EQ.24)
     1   .OR.(ISTATE(1).EQ.25)
         CALL LCMGET(IPLIST,'NCODE',NCODE)
         CALL LCMGET(IPLIST,'ZCODE',ZCODE)
         CALL LCMGET(IPLIST,'ICODE',ICODE)
         IF(LEVEL.EQ.1) GO TO 30
      ENDIF
*
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
      IF(ITYPLU.NE.3) CALL XABORT
     >  (NAMSBR//': CHARACTER DATA EXPECTED(1).')
      IF(CARLIR.EQ.'VIRTUAL') THEN
         ISTATE(1)=0
      ELSE IF(CARLIR.EQ.'HOMOGE') THEN
         ISTATE(1)=1
         ISTATE(6)=1
      ELSE IF(CARLIR.EQ.'CAR1D') THEN
         ISTATE(1)=2
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(3)=INTLIR
         ISTATE(6)=ISTATE(3)
      ELSE IF(CARLIR.EQ.'SPHERE') THEN
         ISTATE(1)=4
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(2)=INTLIR
         ISTATE(6)=ISTATE(2)
      ELSE IF(CARLIR.EQ.'CAR2D') THEN
         ISTATE(1)=5
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(3)=INTLIR
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(4)=INTLIR
         ISTATE(6)=ISTATE(3)*ISTATE(4)
      ELSE IF(CARLIR.EQ.'CAR3D') THEN
         ISTATE(1)=7
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(3)=INTLIR
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(4)=INTLIR
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(5)=INTLIR
         ISTATE(6)=ISTATE(3)*ISTATE(4)*ISTATE(5)
      ELSE IF(CARLIR.EQ.'HEX') THEN
         ISTATE(1)=8
         LHEX=.TRUE.
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(3)=INTLIR
         ISTATE(6)=ISTATE(3)
      ELSE IF(CARLIR.EQ.'HEXZ') THEN
         ISTATE(1)=9
         LHEX=.TRUE.
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(3)=INTLIR
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(5)=INTLIR
         ISTATE(6)=ISTATE(3)*ISTATE(5)
      ELSE IF(CARLIR(1:4).EQ.'TUBE') THEN
         DIR=CARLIR(5:5)
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(2)=INTLIR
         IF(DIR .EQ. ' ') THEN
           ISTATE(1)=3
           ISTATE(3)=1
           ISTATE(4)=1
           IRLXY=1
           CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
           IF(ITYPLU .EQ. 3) THEN
             IRLXY=0
           ELSE IF(ITYPLU.EQ. 2) THEN
             CALL XABORT(NAMSBR//': INVALID REAL DATA.')
           ELSE
             ISTATE(3)=INTLIR
             CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
             IF(ITYPLU.NE.1) CALL XABORT
     >         (NAMSBR//': INTEGER DATA EXPECTED.')
             ISTATE(4)=INTLIR
           ENDIF
           ISTATE(6)=ISTATE(2)*ISTATE(4)*ISTATE(3)
           IF(IRLXY .EQ. 0 ) GO TO 35
         ELSE
           ISTATE(3)=1
           ISTATE(4)=1
           ISTATE(5)=1
           CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
           IF(ITYPLU.NE.1) CALL XABORT
     >       (NAMSBR//': INTEGER DATA EXPECTED.')
           ISTATE(3)=INTLIR
           CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
           IF(ITYPLU .EQ. 3) THEN
             IRLYZ=0
           ELSE IF(ITYPLU.EQ. 2) THEN
             CALL XABORT(NAMSBR//': INVALID REAL DATA.')
           ELSE
             ISTATE(4)=INTLIR
             IRLYZ=1
             CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
             IF(ITYPLU.NE.1) CALL XABORT
     >         (NAMSBR//': INTEGER DATA EXPECTED.')
             ISTATE(5)=INTLIR
           ENDIF
           ISTATE(6)=ISTATE(2)*ISTATE(4)*ISTATE(5)*ISTATE(3)
           IF(DIR .EQ. 'X') THEN
             ISTATE(1)=10
             IF(IRLYZ .EQ. 0 ) THEN
               GO TO 35
             ENDIF
           ELSE IF(DIR .EQ. 'Y') THEN
             ISTATE(1)=11
             IF(IRLYZ .EQ. 0 ) THEN
               ISTATE(4)=ISTATE(3)
               ISTATE(3)=1
               GO TO 35
             ENDIF
           ELSE IF(DIR .EQ. 'Z') THEN
             ISTATE(1)=6
             IF(IRLYZ .EQ.0 ) THEN
               ISTATE(5)=ISTATE(3)
               ISTATE(3)=1
               GO TO 35
             ENDIF
           ELSE
             GO TO 500
           ENDIF
         ENDIF
      ELSE IF(CARLIR(1:6).EQ.'CARCEL') THEN
         DIR=CARLIR(7:7)
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(2)=INTLIR
         IF(DIR .EQ. ' ') THEN
           ISTATE(1)=20
           ISTATE(3)=1
           ISTATE(4)=1
           IRLXY=1
           CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
           IF(ITYPLU .EQ. 3) THEN
             IRLXY=0
           ELSE IF(ITYPLU.EQ. 2) THEN
             CALL XABORT(NAMSBR//': INVALID REAL DATA.')
           ELSE
             ISTATE(3)=INTLIR
             CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
             IF(ITYPLU.NE.1) CALL XABORT
     >         (NAMSBR//': INTEGER DATA EXPECTED.')
             ISTATE(4)=INTLIR
           ENDIF
           ISTATE(6)=(ISTATE(2)+1)*ISTATE(4)*ISTATE(3)
           IF(IRLXY .EQ. 0 ) GO TO 35
         ELSE
           ISTATE(3)=1
           ISTATE(4)=1
           ISTATE(5)=1
           CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
           IF(ITYPLU.NE.1) CALL XABORT
     >       (NAMSBR//': INTEGER DATA EXPECTED.')
           ISTATE(3)=INTLIR
           CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
           IF(ITYPLU .EQ. 3) THEN
             IRLYZ=0
           ELSE IF(ITYPLU.EQ. 2) THEN
             CALL XABORT(NAMSBR//': INVALID REAL DATA.')
           ELSE
             ISTATE(4)=INTLIR
             IRLYZ=1
             CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
             IF(ITYPLU.NE.1) CALL XABORT
     >         (NAMSBR//': INTEGER DATA EXPECTED.')
             ISTATE(5)=INTLIR
           ENDIF
           ISTATE(6)=(ISTATE(2)+1)*ISTATE(4)*ISTATE(5)*ISTATE(3)
           IF(DIR .EQ. 'X') THEN
             ISTATE(1)=21
             IF(IRLYZ .EQ.0 ) THEN
               GO TO 35
             ENDIF
           ELSE IF(DIR .EQ. 'Y') THEN
             ISTATE(1)=22
             IF(IRLYZ .EQ.0 ) THEN
               ISTATE(4)=ISTATE(3)
               ISTATE(3)=1
               GO TO 35
             ENDIF
           ELSE IF(DIR .EQ. 'Z') THEN
             ISTATE(1)=23
             IF(IRLYZ .EQ.0 ) THEN
               ISTATE(5)=ISTATE(3)
               ISTATE(3)=1
               GO TO 35
             ENDIF
           ELSE
             GO TO 500
           ENDIF
         ENDIF
      ELSE IF(CARLIR(1:6).EQ.'HEXCEL') THEN
         DIR=CARLIR(7:7)
         LHEX=.TRUE.
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(2)=INTLIR
         ISTATE(3)=1
         IF(DIR.EQ.' ') THEN
            ISTATE(1)=24
            ISTATE(6)=ISTATE(2)+1
         ELSE IF(DIR.EQ.'Z') THEN
            ISTATE(1)=25
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU.NE.1) CALL XABORT
     >        (NAMSBR//': INTEGER DATA EXPECTED.')
            ISTATE(5)=INTLIR
            ISTATE(6)=(ISTATE(2)+1)*ISTATE(5)
         ELSE
            GO TO 500
         ENDIF
      ELSE IF(CARLIR.EQ.'GROUP') THEN
*        DO-IT-YOURSELF OPTION.
         ISTATE(1)=30
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         ISTATE(3)=INTLIR
         ISTATE(6)=ISTATE(3)
      ELSE IF(CARLIR.NE.GEONAM) THEN
*        COPY ATTRIBUTES FROM AN EXISTING GEOMETRY LOCATED ON A PARALLEL
*        DIRECTORY OF THE LINKED LIST POINTED BY IPLIST.
         IF(LEVEL.EQ.1) CALL XABORT
     >     (NAMSBR//': THE GEOMETRY NAME SHOULD APPEARS BEFORE THE ::')
         CALL LCMSIX(IPLIST,' ',2)
         CALL LCMLEN(IPLIST,CARLIR,ILONG,ITYX)
         IF(ILONG.EQ.0) CALL XABORT(NAMSBR//': UNKNOWN GEOMETRY.')
         CALL LCMSIX(IPLIST,CARLIR,1)
         IFILE=KDROPN('DUMMYSQ',0,2,0,0)
         IF(IFILE.LE.0) CALL XABORT(NAMSBR//': KDROPN FAILURE.')
         CALL LCMEXP(IPLIST,0,IFILE,1,1)
         REWIND(IFILE)
         CALL LCMSIX(IPLIST,' ',2)
         CALL LCMSIX(IPLIST,GEONAM,1)
         CALL LCMEXP(IPLIST,0,IFILE,1,2)
         IRC=KDRCLS(IFILE,2)
         IF(IRC.LT.0) CALL XABORT(NAMSBR//': KDRCLS FAILURE.')
         CALL XDISET(ISTATE,NSTATE,0)
         CALL LCMGET(IPLIST,'STATE-VECTOR',ISTATE)
         LHEX=(ISTATE(1).EQ.8).OR.(ISTATE(1).EQ.9).OR.(ISTATE(1).EQ.24)
     1   .OR.(ISTATE(1).EQ.25)
         CALL LCMGET(IPLIST,'NCODE',NCODE)
         CALL LCMGET(IPLIST,'ZCODE',ZCODE)
         CALL LCMGET(IPLIST,'ICODE',ICODE)
      ENDIF
*
   30 CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
      IF(ITYPLU.NE.3) 
     >CALL XABORT(NAMSBR//': CHARACTER DATA EXPECTED(2).')
   35 IF(CARLIR.EQ.'EDIT') THEN
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         IMPX=INTLIR
      ELSE IF(CARLIR.EQ.'MIX') THEN
*        INPUT MIXTURE NUMBERS.
         CALL SETARA(IBASE,ISTATE(6),IMIX)
         CALL XDISET(IBASE(IMIX),ISTATE(6),0)
         I=0
   40    I=I+1
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.EQ.3) GO TO 50
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         IF(I.GT.ISTATE(6)) GO TO 500
         IBASE(IMIX+I-1)=INTLIR
         ISTATE(7)=MAX(ISTATE(7),IBASE(IMIX+I-1))
         MINMIX=MIN(MINMIX,IBASE(IMIX+I-1))
         GO TO 40
   50    CONTINUE
         IF(CARLIR .EQ. 'REPEAT') THEN
           NBR=ISTATE(6)/(I-1)
           NBRR=NBR*(I-1)
           IF(NBRR .NE. ISTATE(6) ) THEN
             WRITE(IOUT,9000) NAMSBR,I-1,ISTATE(6)
             CALL XABORT(NAMSBR//
     >       ': Impossible to repeat an integer number of times')
           ENDIF
           JREP=I-1
           DO IREP=1,NBR-1
             DO II=1,I-1
               JREP=JREP+1
               IBASE(IMIX+JREP-1)=IBASE(IMIX+II-1)
             ENDDO
           ENDDO
           I=JREP+1
           CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
           IF(ITYPLU.NE.3) CALL XABORT(NAMSBR//
     >     ': CHARACTER DATA EXPECTED.')
         ENDIF
         ISTATE(6)=I-1
         CALL LCMPUT(IPLIST,'MIX',ISTATE(6),1,IBASE(IMIX))
         CALL RLSARA(IBASE(IMIX))
         GO TO 35
      ELSE IF(CARLIR(1:4).EQ.'MESH') THEN
         NAMT=CARLIR
         DIR=NAMT(5:5)
*        INPUT CARTESIAN COORDINATES.
         IF(DIR.EQ.'X') THEN
            IF(ISTATE(3).EQ.0) GO TO 500
            LMESH=ISTATE(3)+1
         ELSE IF(DIR.EQ.'Y') THEN
            IF(ISTATE(4).EQ.0) GO TO 500
            LMESH=ISTATE(4)+1
         ELSE IF(DIR.EQ.'Z') THEN
            IF(ISTATE(5).EQ.0) GO TO 500
            LMESH=ISTATE(5)+1
         ELSE
            GO TO 500
         ENDIF
         CALL SETARA(RBASE,LMESH,IMESH)
         DO 60 I=1,LMESH
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.2) CALL XABORT(NAMSBR//': REAL DATA EXPECTED.')
         RBASE(IMESH+I-1)=REALIR
         IF(I.GT.1) THEN
            IF(RBASE(IMESH+I-1).LE.RBASE(IMESH+I-2)) GO TO 500
         ENDIF
   60    CONTINUE
         CALL LCMPUT(IPLIST,NAMT,LMESH,2,RBASE(IMESH))
         CALL RLSARA(RBASE(IMESH))
      ELSE IF(CARLIR.EQ.'RADIUS') THEN
*        INPUT TUBE RADIUS.
         IF(ISTATE(2).EQ.0) GO TO 500
         LCYL=ISTATE(2)+1
         CALL SETARA(RBASE,LCYL,ICYL)
         DO 70 I=1,LCYL
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.2) CALL XABORT(NAMSBR//': REAL DATA EXPECTED.')
         RBASE(ICYL+I-1)=REALIR
         IF(I.GT.1) THEN
            IF(RBASE(ICYL+I-1).LE.RBASE(ICYL+I-2)) GO TO 500
         ENDIF
   70    CONTINUE
         IF(RBASE(ICYL).NE.0.0) GO TO 500
         CALL LCMPUT(IPLIST,'RADIUS',LCYL,2,RBASE(ICYL))
         CALL RLSARA(RBASE(ICYL))
      ELSE IF(CARLIR.EQ.'OFFCENTER') THEN
*        INPUT TUBE CENTER LOCATION (USE FOR CARCEL* ONLY).
         IF(ISTATE(2).EQ.0) GO TO 500
         CALL SETARA(RBASE,3,ICENT)
         CALL XDRSET(RBASE(ICENT),3,0.0)
         DO 71 I=1,3
           CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
           IF(ITYPLU.NE.2) GO TO 72
           RBASE(ICENT+I-1)=REALIR
 71      CONTINUE
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
 72      CONTINUE
         CALL LCMPUT(IPLIST,'OFFCENTER',3,2,RBASE(ICENT))
         CALL RLSARA(RBASE(ICENT))
         GO TO 35
      ELSE IF(CARLIR.EQ.'SIDE') THEN
*        INPUT THE SIDE LENGTH IN HEXAGONAL GEOMETRY.
         IF(.NOT.LHEX) CALL XABORT(NAMSBR//': SIDE PROHIBITED.')
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.2) CALL XABORT(NAMSBR//': REAL DATA EXPECTED.')
         CALL LCMPUT(IPLIST,'SIDE',1,2,REALIR)
      ELSE IF(CARLIR(1:5).EQ.'SPLIT') THEN
         NAMT=CARLIR
         DIR=CARLIR(6:6)
*        INPUT MESH SPLITTING FACTORS.
         ISTATE(11)=1
         IF(DIR.EQ.'X') THEN
            IF(ISTATE(3).EQ.0) GO TO 500
            LMESH=ISTATE(3)
         ELSE IF(DIR.EQ.'Y') THEN
            IF(ISTATE(4).EQ.0) GO TO 500
            LMESH=ISTATE(4)
         ELSE IF(DIR.EQ.'Z') THEN
            IF(ISTATE(5).EQ.0) GO TO 500
            LMESH=ISTATE(5)
         ELSE IF(DIR.EQ.'R') THEN
            IF(ISTATE(2).EQ.0) GO TO 500
            LMESH=ISTATE(2)
         ELSE
            GO TO 500
         ENDIF
         CALL SETARA(IBASE,LMESH,IMESH)
         DO 80 I=1,LMESH
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         IBASE(IMESH+I-1)=INTLIR 
         IF(((IBASE(IMESH+I-1).LE.0).AND.(DIR.NE.'R')).OR.
     >       (IBASE(IMESH+I-1).EQ.0)) CALL XABORT
     >    (NAMSBR//': INVALID MESH-SPLITTING INDEX.')
   80    CONTINUE
         CALL LCMPUT(IPLIST,NAMT,LMESH,1,IBASE(IMESH))
         CALL RLSARA(IBASE(IMESH))
      ELSE IF(CARLIR.EQ.'CELL') THEN
*        FORCE SUB GEOMETRIES AT SPECIFIC LOCATIONS.
         ISTATE(8)=1
         CALL SETARA(IBASE,3*ISTATE(6),ICELL)
         CALL SETARA(IBASE,ISTATE(6),IIGEN)
         CALL XDISET(IBASE(IIGEN),ISTATE(6),0)
         I=0
         IKG=0
  110    CONTINUE
         I=I+1
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU .EQ. 2) CALL XABORT(NAMSBR//
     >   ': Cell name or mixture number expected')
         IF(ITYPLU .EQ. 3) THEN
           IF((CARLIR(2:2).EQ.'-').OR.(CARLIR(2:2).EQ.'+').OR.
     >        (CARLIR.EQ.'HBC').OR.(CARLIR(1:4).EQ.'MESH').OR.
     >        (CARLIR(1:5).EQ.'SPLIT').OR.(CARLIR.EQ.'SIDE').OR.
     >        (CARLIR.EQ.'MERGE').OR.(CARLIR.EQ.'TURN').OR.
     >        (CARLIR.EQ.'CLUSTER').OR.(CARLIR(2:4).EQ.'PIN').OR.
     >        (CARLIR.EQ.'BIHET').OR.(CARLIR.EQ.'POURCE').OR.
     >        (CARLIR.EQ.'PROCEL').OR.(CARLIR.EQ.';').OR.
     >        (CARLIR.EQ.':::').OR.(CARLIR.EQ.'RADIUS')
     >        ) GO TO 120
           IF(I .GT. ISTATE(6)) GO TO 500
           NEWGEO=CARLIR
           DO 115 J=1,I-1
             JKG=IBASE(IIGEN+J-1)
             WRITE(OLDGEO,'(3A4)') (IBASE(ICELL+3*(JKG-1)+ITC),ITC=0,2)
             IF(NEWGEO .EQ. OLDGEO) THEN
               IBASE(IIGEN+I-1)=JKG
               GO TO 110
             ENDIF
  115      CONTINUE
           IKG=IKG+1
           IBASE(IIGEN+I-1)=IKG
           READ(NEWGEO,'(3A4)') (IBASE(ICELL+3*(IKG-1)+ITC),ITC=0,2)
         ELSE
           IBASE(IIGEN+I-1)=-INTLIR
         ENDIF
         GO TO 110
  120    CONTINUE
         ISTATE(6)=I-1
         CALL LCMPUT(IPLIST,'CELL',3*IKG,3,IBASE(ICELL))
         CALL LCMPUT(IPLIST,'GENERATING',ISTATE(6),1,IBASE(IIGEN))
         CALL RLSARA(IBASE(IIGEN))
         CALL RLSARA(IBASE(ICELL))
         GO TO 35
      ELSE IF(CARLIR.EQ.'MERGE') THEN
*        INPUT CELL-MERGING ITYPLUES.
         ISTATE(10)=1
         CALL SETARA(IBASE,ISTATE(6),IMERGE)
         I=0
  130    I=I+1
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.EQ.3) GO TO 140
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         IF(I.GT.ISTATE(6)) GO TO 500
         IBASE(IMERGE+I-1)=INTLIR
         GO TO 130
  140    ISTATE(6)=I-1
         CALL LCMPUT(IPLIST,'MERGE',ISTATE(6),1,IBASE(IMERGE))
         CALL RLSARA(IBASE(IMERGE))
         GO TO 35
      ELSE IF(CARLIR.EQ.'TURN') THEN
*        INPUT ORIENTATION INFORMATION.
         CALL SETARA(IBASE,ISTATE(6),ITURN)
         I=0
  150    I=I+1
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.3)
     >   CALL XABORT(NAMSBR//': CHARACTER DATA EXPECTED.')
         DO 160 J=1,MAXTUR
         IF(CARLIR.EQ.CTUR(J)) THEN
            IF(I.GT.ISTATE(6)) GO TO 500
            IBASE(ITURN+I-1)=J
            GO TO 150
         ELSE IF(CARLIR.EQ.'-'//CTUR(J)) THEN
            IF(I.GT.ISTATE(6)) GO TO 500
            IBASE(ITURN+I-1)=MAXTUR+J
            GO TO 150
         ENDIF
  160    CONTINUE
         ISTATE(6)=I-1
         CALL LCMPUT(IPLIST,'TURN',ISTATE(6),1,IBASE(ITURN))
         CALL RLSARA(IBASE(ITURN))
         GO TO 35
      ELSE IF(CARLIR.EQ.'CLUSTER') THEN
*        DEFINE CLUSTER SUB GEOMETRIES.
         CALL SETARA(IBASE,3*MXCL,ICELL)
         I=0
  180    I=I+1
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.3) 
     >   CALL XABORT(NAMSBR//': CHARACTER DATA EXPECTED.')
         IF((CARLIR(2:2).EQ.'-').OR.(CARLIR(2:2).EQ.'+').OR.
     1   (CARLIR.EQ.'HBC').OR.(CARLIR(1:4).EQ.'MESH').OR.(CARLIR(1:5)
     2   .EQ.'SPLIT').OR.(CARLIR.EQ.'SIDE').OR.(CARLIR.EQ.'MIX').OR.
     3   (CARLIR.EQ.'CELL').OR.(CARLIR.EQ.'MERGE').OR.(CARLIR.EQ.'TURN')
     4   .OR.(CARLIR(2:4).EQ.'PIN').OR.(CARLIR.EQ.'BIHET').OR.
     5   (CARLIR.EQ.'POURCE').OR.(CARLIR.EQ.'PROCEL').OR.(CARLIR.EQ.
     6   ';').OR.(CARLIR.EQ.':::')) GO TO 190
         IF(I.GT.MXCL) GO TO 500
         NEWGEO=CARLIR
         READ(NEWGEO,'(3A4)') (IBASE(ICELL+3*(I-1)+ITC),ITC=0,2)
         GO TO 180
  190    CALL LCMPUT(IPLIST,'CLUSTER',3*(I-1),3,IBASE(ICELL))
         ISTATE(13)=I-1
         CALL RLSARA(IBASE(ICELL))
         GO TO 35
      ELSE IF(CARLIR.EQ.'NPIN') THEN
*        INPUT NUMBER OF PINS IN CLUSTER RING.
         IF(ISTATE(1) .NE.  3 .AND. ISTATE(1) .NE.  6 .AND.
     >      ISTATE(1) .NE. 10 .AND. ISTATE(1) .NE. 11 ) GO TO 500
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         NPIN=INTLIR
         CALL LCMPUT(IPLIST,'NPIN',1,1,NPIN)
      ELSE IF((CARLIR.EQ.'RPIN').OR.(CARLIR.EQ.'APIN')) THEN
         NAMT=CARLIR
         IF(NPIN .EQ. 0) NPIN=1
*----
*  Allocate memory for APIN and RPIN
*----
         CALL SETARA(RBASE,NPIN,ARPIN)
*        INPUT RADIUS/ANGLE OF CLUSTER RING.
         IF(ISTATE(1) .NE.  3 .AND. ISTATE(1) .NE.  6 .AND.
     >      ISTATE(1) .NE. 10 .AND. ISTATE(1) .NE. 11 ) GO TO 500
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.2) CALL XABORT(NAMSBR//': REAL DATA EXPECTED.')
         RBASE(ARPIN)=REALIR
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.2) THEN
           CALL LCMPUT(IPLIST,NAMT,1,2,RBASE(ARPIN))
           CALL RLSARA(RBASE(ARPIN))
           GO TO 35
         ENDIF
         IF(NPIN .EQ. 1) CALL XABORT(NAMSBR//
     >   ': Only one APIN or RPIN permitted.') 
         RBASE(ARPIN+1)=REALIR
         DO 191 IPIN=2,NPIN-1 
           CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
           IF(ITYPLU.NE.2) CALL XABORT(NAMSBR//': REAL DATA EXPECTED.')
           RBASE(ARPIN+IPIN)=REALIR
 191     CONTINUE
         CALL LCMPUT(IPLIST,NAMT,NPIN,2,RBASE(ARPIN))
         CALL RLSARA(RBASE(ARPIN))
      ELSE IF(CARLIR.EQ.'BIHET') THEN
*        DOUBLE HETEROGENEITY OPTION.
         ISTATE(12)=1
         IF(LEVEL.NE.1) CALL XABORT
     >    (NAMSBR//': BIHET DATA SHOULD BE WRITTEN ON FIRST '//
     >             'DIRECTORY LEVEL.')
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.3) 
     >   CALL XABORT(NAMSBR//': CHARACTER DATA EXPECTED.')
         IF(CARLIR.EQ.'TUBE') THEN
            MICRO=3
         ELSE IF(CARLIR.EQ.'SPHE') THEN
            MICRO=4
         ELSE
            CALL XABORT(NAMSBR//': PROHIBITED TYPE OF MICRO GEOMETRY.')
         ENDIF
         CALL LCMPUT(IPLIST,'MICRO',1,1,MICRO)
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         NG=INTLIR
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.') 
         NMILG=INTLIR
*
         CALL SETARA(IBASE,NG,INS)
         NSMAX=0
         DO 200 I=1,NG
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         IBASE(INS+I-1)=INTLIR
  200    NSMAX=MAX(NSMAX,IBASE(INS+I-1))
         CALL LCMPUT(IPLIST,'NS',NG,1,IBASE(INS))
*
         CALL SETARA(RBASE,(NSMAX+1)*NG,IRS)
         DO 205 IOFJ=1,(NSMAX+1)*NG
  205    RBASE(IRS+IOFJ-1)=0.0
         DO 210 I=1,NG
         DO 210 J=1,IBASE(INS+I-1)+1
         IOFJ=(I-1)*(NSMAX+1)+J
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.2) CALL XABORT(NAMSBR//': REAL DATA EXPECTED.')
         RBASE(IRS+IOFJ-1)=REALIR
  210    CONTINUE
         CALL LCMPUT(IPLIST,'RS',(NSMAX+1)*NG,2,RBASE(IRS))
         CALL RLSARA(RBASE(IRS))
*
         CALL SETARA(IBASE,NMILG,IMILIE)
         DO 220 I=1,NMILG
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         IBASE(IMILIE+I-1)=INTLIR
  220    CONTINUE
         CALL LCMPUT(IPLIST,'MILIE',NMILG,1,IBASE(IMILIE))
         CALL RLSARA(IBASE(IMILIE))
*
         CALL SETARA(IBASE,NMILG,IMIXDL)
         DO 230 I=1,NMILG
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//': INTEGER DATA EXPECTED.')
         IBASE(IMIXDL+I-1)=INTLIR
  230    CONTINUE
         CALL LCMPUT(IPLIST,'MIXDIL',NMILG,1,IBASE(IMIXDL))
         CALL RLSARA(IBASE(IMIXDL))
*
         CALL SETARA(IBASE,NSMAX*NG*NMILG,IMIXGR)
         DO 235 IOFK=1,NSMAX*NG*NMILG
  235    IBASE(IMIXGR+IOFK-1)=0
         CALL SETARA(RBASE,NG*NMILG,IFRACT)
         DO 250 I=1,NMILG
         DO 240 J=1,NG
         IOFJ=(I-1)*NG+J
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.2) CALL XABORT(NAMSBR//': REAL DATA EXPECTED.')
         RBASE(IFRACT+IOFJ-1)=REALIR
  240    CONTINUE
         DO 250 J=1,NG
         IOFJ=(I-1)*NG+J
         IF(RBASE(IFRACT+IOFJ-1).GT.0.0) THEN
            DO 245 K=1,IBASE(INS+J-1)
            IOFK=((I-1)*NG+(J-1))*NSMAX+K
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU.NE.1) CALL XABORT
     >        (NAMSBR//': INTEGER DATA EXPECTED.')
            IBASE(IMIXGR+IOFK-1)=INTLIR
  245       CONTINUE
         ENDIF
  250    CONTINUE
         CALL LCMPUT(IPLIST,'FRACT',NG*NMILG,2,RBASE(IFRACT))
         CALL RLSARA(RBASE(IFRACT))
         CALL LCMPUT(IPLIST,'MIXGR',NSMAX*NG*NMILG,1,IBASE(IMIXGR))
         CALL RLSARA(IBASE(IMIXGR))
*
         CALL RLSARA(IBASE(INS))
      ELSE IF(CARLIR.EQ.'POURCE') THEN
*        CELL PROPORTIONS FOR DO-IT-YOURSELF OPTION.
         IF(ISTATE(1).NE.30) GO TO 500
         CALL SETARA(RBASE,ISTATE(3),IPOURC)
         DO 260 I=1,ISTATE(3)
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.2) CALL XABORT(NAMSBR//': REAL DATA EXPECTED.')
         RBASE(IPOURC+I-1)=REALIR
  260    CONTINUE
         CALL LCMPUT(IPLIST,'POURCE',ISTATE(3),2,RBASE(IPOURC))
         CALL RLSARA(RBASE(IPOURC))
      ELSE IF(CARLIR.EQ.'PROCEL') THEN
*        CELL PROBABILITIES FOR DO-IT-YOURSELF OPTION.
         IF(ISTATE(1).NE.30) GO TO 500
         ILONG=ISTATE(3)*ISTATE(3)
         CALL SETARA(RBASE,ILONG,IPROCE)
         DO 270 I=1,ISTATE(3)
         DO 270 J=1,ISTATE(3)
         IOFJ=(J-1)*ISTATE(3)+I
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.2) CALL XABORT(NAMSBR//': REAL DATA EXPECTED.')
         RBASE(IPROCE+IOFJ-1)=REALIR
  270    CONTINUE
         CALL LCMPUT(IPLIST,'PROCEL',ILONG,2,RBASE(IPROCE))
         CALL RLSARA(RBASE(IPROCE))
      ELSE IF((CARLIR(2:2).EQ.'+').OR.(CARLIR(2:2).EQ.'-').OR.
     1   (CARLIR.EQ.'HBC')) THEN
         NAMT=CARLIR
*        INPUT BOUNDARY CONDITIONS.
         IF(CARLIR.EQ.'X-') THEN
            ISURF=1
            IF(ISTATE(3).EQ.0) GO TO 500
         ELSE IF(CARLIR.EQ.'X+') THEN
            ISURF=2
            IF(ISTATE(3).EQ.0) GO TO 500
         ELSE IF(CARLIR.EQ.'R+') THEN
            ISURF=2
            IF(ISTATE(2).EQ.0) GO TO 500
         ELSE IF(CARLIR.EQ.'Y-') THEN
            ISURF=3
            IF(ISTATE(4).EQ.0) GO TO 500
         ELSE IF(CARLIR.EQ.'Y+') THEN
            ISURF=4
            IF(ISTATE(4).EQ.0) GO TO 500
         ELSE IF(CARLIR.EQ.'Z-') THEN
            ISURF=5
            IF(ISTATE(5).EQ.0) GO TO 500
         ELSE IF(CARLIR.EQ.'Z+') THEN
            ISURF=6
            IF(ISTATE(5).EQ.0) GO TO 500
         ELSE IF(CARLIR.EQ.'HBC') THEN
            ISURF=1
            IF(.NOT.LHEX) CALL XABORT(NAMSBR//': HBC PROHIBITED.')
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU.NE.3) CALL XABORT
     >        (NAMSBR//': CHARACTER DATA EXPECTED.')
            DO 280 I=1,MAXHEX
            IF(CARLIR.EQ.CHEX(I)) THEN
               IHEX=I
               GO TO 285
            ENDIF
  280       CONTINUE
            CALL XABORT(NAMSBR//': INVALID TYPE OF HEXAGONAL SYMMETRY.')
  285       CALL LCMPUT(IPLIST,'IHEX',1,1,IHEX)
         ELSE
            GO TO 500
         ENDIF
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.3) 
     >   CALL XABORT(NAMSBR//': CHARACTER DATA EXPECTED.')
         NAMT=CARLIR
         DO 290 I=1,MAXCOD
         IF(NAMT.EQ.COND(I)) THEN
            NCODE(ISURF)=I
            GO TO 295
         ENDIF
  290    CONTINUE
         CALL XABORT(NAMSBR//': INVALID TYPE OF BOUNDARY CONDITION.')
  295    IF(NAMT.EQ.'ALBE') THEN
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU.EQ.1) THEN
               ICODE(ISURF)=INTLIR
               MINICO=MIN(MINICO,INTLIR)
            ELSE IF(ITYPLU.EQ.2) THEN
               ZCODE(ISURF)=REALIR
            ELSE
               CALL XABORT(NAMSBR//': INTEGER OR REAL DATA EXPECTED.')
            ENDIF
         ELSE IF(NAMT.EQ.'REFL') THEN
            ZCODE(ISURF)=1.0
         ELSE IF(NAMT.EQ.'VOID') THEN
            ZCODE(ISURF)=0.0
         ENDIF
      ELSE IF(CARLIR.EQ.';') THEN
*        END-OF-GEOMETRY.
         GO TO 320
      ELSE IF(CARLIR.EQ.':::') THEN
*----
*  INPUT A SUB GEOMETRY.
*  Uncomment next command for last level 
*        CALL XABORT(NAMSBR//': No subgeometry permitted.')
*  otherwise uncomment next 20 lines
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU.NE.3) CALL XABORT
     >     (NAMSBR//': CHARACTER DATA EXPECTED.')
         NEWGEO=CARLIR
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF((ITYPLU.NE.3).OR.(CARLIR.NE.':=')) GO TO 500
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF((ITYPLU.NE.3).OR.(CARLIR.NE.'GEO:')) GO TO 500
         CALL LCMLEN(IPLIST,NEWGEO,ILONG,ITYX)
         IF(ILONG.NE.0) THEN
            IF(ITYX.NE.0) CALL XABORT
     >        (NAMSBR//': INVALID GEOMETRY NAME.')
         ELSE
            ISTATE(9)=ISTATE(9)+1
         ENDIF
         CALL LCMSIX(IPLIST,NEWGEO,1)
         IMPX2=IMPX
         CALL GEOIN2(NEWGEO,IPLIST,LEVEL+1,IMPX2,MAXMI2)
         CALL LCMSIX(IPLIST,' ',2)
         ISTATE(7)=MAX(ISTATE(7),MAXMI2)
      ELSE
         CALL XABORT(NAMSBR//
     >   ': '//CARLIR//' IS AN INVALID KEY WORD.')
      ENDIF
      GO TO 30
*
  320 NAMT='L_GEOM'
      READ(NAMT,'(3A4)') (KCHAR(I1),I1=1,3)
      CALL LCMPUT(IPLIST,'SIGNATURE',3,3,KCHAR)
      CALL LCMPUT(IPLIST,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL LCMPUT(IPLIST,'NCODE',6,1,NCODE)
      CALL LCMPUT(IPLIST,'ZCODE',6,2,ZCODE)
      CALL LCMPUT(IPLIST,'ICODE',6,1,ICODE)
      IF(MINMIX.LT.0)
     >  CALL XABORT(NAMSBR//': NEGATIVE MIXTURE NUMBERS INVALID')
      IF(MINICO.LT.1)
     >  CALL XABORT(NAMSBR//': ALBEDO NUMBER MUST BE GREATER THAN 0')
      MAXMIX=ISTATE(7)
      IF(IMPX.GT.0) THEN
        NAMT=' '
        CALL LCMNXT(IPLIST,NAMT,NEWGEO,OLDGEO,LCM)
        WRITE(IOUT,6000) LEVEL,GEONAM,NEWGEO,TYPE(ISTATE(1))
      ENDIF
      IF(IMPX.GT.1) THEN
        WRITE(IOUT,6010) ISTATE(1),TYPE(ISTATE(1)),(ISTATE(I),I=2,13)
      ENDIF
      IF((ISTATE(8).EQ.1).AND.(ISTATE(9).EQ.0)) GO TO 500
      RETURN
  500 CALL XABORT(NAMSBR//': INVALID DATA.')
*
 6000 FORMAT(/' CREATION OF A LEVEL ',I3,' GEOMETRY'/
     >        ' GEOMETRY NAME    = ',4X,A12/
     >        ' LINKED LIST NAME = ',4X,A12/
     >        ' GEOMETRY TYPE    = ',4X,A16)
 6010 FORMAT(/' GEOMETRY PARAMETER VECTOR:'/
     > ' ITYPE = ',I6,' (',A16,')'/
     > ' LR    = ',I6,' (NUMBER OF TUBES)'/
     > ' LX    = ',I6,' (X-DIMENSION INDEX)'/
     > ' LY    = ',I6,' (Y-DIMENSION INDEX)'/
     > ' LZ    = ',I6,' (Z-DIMENSION INDEX)'/
     > ' LREG  = ',I6,' (NUMBER OF REGIONS)'/
     > ' MAXMIX= ',I6,' (MAX. NB. OF MIXTURES/0=TRANSPARENT GEOMETRY)'/
     > ' ISUB1 = ',I6,' (1=COMMAND CELL IS USED/0=ELSE)'/
     > ' ISUB2 = ',I6,' (NUMBER OF SUB GEOMETRIES)'/
     > ' IMERGE= ',I6,' (1=CELL-MERGING/0=ELSE)'/
     > ' ISPLIT= ',I6,' (1=MESH-SPLITTING/0=ELSE)'/
     > ' IBIHET= ',I6,' (1=DOUBLE HETEROGENEITY/0=ELSE)'/
     > ' ICLUST= ',I6,' (NUMBER OF CLUSTER RINGS)'/)
 9000 FORMAT(' ***** Error in ',A6,' *****'/
     >       ' Initial number of mixtures ',I10/
     >       ' Cannot be repeated an integer number of times',
     >       ' to fill ',I10,' mixtures')
      END
