*DECK JPMCP
      SUBROUTINE JPMCP (ILK,IMPX,NMBLK,IFR,ALB,NMERGE,IJAT,INUM,VOL,MIX,
     1 DVX,NGEN,IGEN,ISURF,CHORD,SIGT,PII,PIS,PSS,P,PSSB,IGAT,IGAU,
     2 IGAU2)
*
*-----------------------------------------------------------------------
*
*   THIS SUBROUTINE COMPUTES THE COMPLETE REDUCED PIJ MATRIX USING
*   THE INTERFACE CURRENT METHOD.
*
*   ILK    : ILK=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
*            IS PRESENT.
*   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*
*   INPUT PARAMETERS RELATED TO BLOCKS DEFINITION:
*   NMBLK  : TOTAL NUMBER OF BLOCKS IN THE DOMAIN.
*   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAS) WHERE
*            IJAS IS THE TOTAL NUMBER OF IN-CURRENTS SURROUNDING ALL
*            THE BLOCKS.
*   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
*            DIMENSION ALB(IJAS)
*
*   INPUT PARAMETERS RELATED TO THE MERGED BLOCKS DEFINITION:
*   NMERGE : TOTAL NUMBER OF MERGED BLOCKS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*            MANY BLOCKS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*            OWN THE SAME GENERATING BLOCK. THIS ALLOWS SOME REDUCTION
*            IN CPU TIME AND MEMORY. NMERGE.LE.NMBLK
*   IJAT   : TOTAL NUMBER OF DISTINCT OUT-CURRENTS.
*   INUM   : INDEX-NUMBER OF THE MERGED BLOCK ASSOCIATED TO EACH
*            BLOCK. DIMENSION INUM(NMBLK)
*   VOL    : VOLUMES. DIMENSION VOL(NMERGE)
*   MIX    : INDEX-NUMBER OF OUT-CURRENTS. DIMENSION MIX(IJAR) WHERE
*            IJAR IS THE TOTAL NUMBER OF OUT-CURRENTS BELONGING TO THE
*            MERGED BLOCKS.
*   DVX    : WEIGHT ASSOCIATED WITH EACH OUT-CURRENT.
*            DIMENSION DVX(IJAR)
*
*            REMARK:
*            IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
*            GEOMETRICAL "A" MATRIX.
*
*   INPUT PARAMETERS RELATED TO THE GENERATING BLOCKS DEFINITION:
*   NGEN   : TOTAL NUMBER OF GENERATING BLOCKS. A GENERATING BLOCK IS
*            DEFINED BY ITS MIXTURE AND DIMENSIONS IRRESPECTIVE OF
*            ITS POSITION IN THE DOMAIN. NGEN.LE.NMERGE
*   IGEN   : INDEX-NUMBER OF THE GENERATING BLOCK ASSOCIATED WITH EACH
*            MERGED BLOCK. DIMENSION IGEN(NMERGE)
*   ISURF  : NUMBER OF SURFACES ASSOCIATED WITH EACH GENERATING BLOCK.
*            DIMENSION ISURF(NGEN)
*   CHORD  : MEAN CHORD LENGTH ASSOCIATED WITH EACH SURFACE.
*            DIMENSION CHORD(NPIS) WHERE NPIS=SUM(ISURF(I)) IS THE
*            TOTAL NUMBER OF SURFACES SURROUNDING ALL THE GENERATING
*            BLOCKS.
*   SIGT   : TOTAL CROSS SECTIONS OF THE GENERATING BLOCKS.
*            DIMENSION SIGT(NGEN)
*   PII    : VOLUME TO VOLUME REDUCED COLLISION PROBABILITIES.
*            DIMENSION PII(NGEN)
*   PIS    : VOLUME TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PIS(NPIS) WHERE NPIS=SUM(ISURF(I))
*   PSS    : SURFACE TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PSS(NPSS) WHERE NPSS=SUM(ISURF(I)**2)
*
*   OUTPUT PARAMETER:
*   P      : REDUCED PIJ MATRIX. DIMENSION P(NMERGE,NMERGE)
*
*   SCRATCH STORAGE:
*   PSSB   : DIMENSION PSSB(IJAT,2*IJAT)
*   IGAT   : DIMENSION IGAT(NMERGE)
*   IGAU   : DIMENSION IGAU(NGEN)
*   IGAU2  : DIMENSION IGAU2(NGEN)
*
*-----------------------------------------------------------------------
*
      PARAMETER (EPS1=1.0E-4)
      LOGICAL ILK
      DIMENSION IFR(*),ALB(*),INUM(*),VOL(*),MIX(*),DVX(*),IGEN(*),
     1 ISURF(*),CHORD(*),SIGT(*),PII(*),PIS(*),PSS(*),P(NMERGE,*),
     2 PSSB(IJAT,*),IGAT(*),IGAU(*),IGAU2(*)
*
      IJAR=0
      DO 10 I=1,NMERGE
      IGAT(I)=IJAR
   10 IJAR=IJAR+ISURF(IGEN(I))
      NPIS=0
      NPSS=0
      DO 20 I=1,NGEN
      IGAU(I)=NPIS
      IGAU2(I)=NPSS
      NPIS=NPIS+ISURF(I)
   20 NPSS=NPSS+ISURF(I)**2
*
*     COMPUTATION OF PSSB=A*(I-PSS*A)**-1
      DO 40 I=1,IJAT
      DO 30 J=1,IJAT
      PSSB(I,J)=0.0
   30 PSSB(I,IJAT+J)=0.0
   40 PSSB(I,I)=1.0
      IS=0
      DO 60 ICEL=1,NMBLK
      IKK=INUM(ICEL)
      IKG=IGEN(IKK)
      NCOUR=ISURF(IKG)
      IT=IGAT(IKK)
      DO 50 JP=1,NCOUR
      J1=IFR(IS+JP)
      J2=MIX(IT+JP)
      ALBEDO=ALB(IS+JP)
      PSSB(J1,IJAT+J2)=PSSB(J1,IJAT+J2)+ALBEDO*DVX(IT+JP)
      DO 50 IP=1,NCOUR
      J2=MIX(IT+IP)
      IOFSET=IGAU2(IKG)+(JP-1)*NCOUR+IP
   50 PSSB(J1,J2)=PSSB(J1,J2)-PSS(IOFSET)*ALBEDO*DVX(IT+IP)
   60 IS=IS+NCOUR
      CALL ALSB (IJAT,IJAT,PSSB,IER,IJAT)
      IF (IER.NE.0) CALL XABORT('JPMCP: SINGULAR MATRIX.')
*
*     COMPUTATION OF PIS*PSSB*PSJ
      DO 80 IKK=1,NMERGE
      DO 70 JKK=1,NMERGE
   70 P(IKK,JKK)=0.0
      IKG=IGEN(IKK)
      P(IKK,IKK)=PII(IKG)
      NCOUR=ISURF(IKG)
      IT=IGAT(IKK)
      DO 80 IP=1,NCOUR
      ICC=MIX(IT+IP)
      ZZZ=PIS(IGAU(IKG)+IP)
      DO 80 JKK=1,NMERGE
      JKG=IGEN(JKK)
      NCOUS=ISURF(JKG)
      JT=IGAT(JKK)
      DO 80 JP=1,NCOUS
      JCC=MIX(JT+JP)
      PBJ=PIS(IGAU(JKG)+JP)*CHORD(IGAU(JKG)+JP)
   80 P(IKK,JKK)=P(IKK,JKK)+ZZZ*DVX(JT+JP)*PSSB(JCC,IJAT+ICC)*PBJ
*
      IF (IMPX.GE.7) THEN
         WRITE (6,130) (J,J=1,NMERGE)
         DO 90 I=1,NMERGE
         WRITE (6,140) I,(P(I,J),J=1,NMERGE)
   90    CONTINUE
         WRITE (6,'(//)')
      ENDIF
      IF ((IMPX.GE.10).OR.(IMPX.LT.0)) THEN
*        CHECK THE RECIPROCITY CONDITIONS.
         VOLTOT=0.0
         DO 95 I=1,NMERGE
   95    VOLTOT=VOLTOT+VOL(I)
         VOLTOT=VOLTOT/REAL(NMERGE)
         WRK=0.0
         DO 100 I=1,NMERGE
         DO 100 J=1,NMERGE
  100    WRK=MAX(WRK,ABS(P(I,J)*VOL(I)-P(J,I)*VOL(J))/VOLTOT)
         IF (WRK.GE.EPS1) WRITE (6,150) WRK
*        CHECK THE CONSERVATION CONDITIONS.
         IF (.NOT.ILK) THEN
            WRK=0.0
            DO 120 I=1,NMERGE
            F1=1.0
            DO 110 J=1,NMERGE
  110       F1=F1-P(I,J)*SIGT(IGEN(J))
  120       WRK=AMAX1(WRK,ABS(F1))
            IF (WRK.GE.EPS1) WRITE (6,160) WRK
         ENDIF
      ENDIF
      RETURN
*
  130 FORMAT (//41H REDUCED COLLISION PROBABILITIES MATRIX ://
     1 (11X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,
     2 I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,
     3 2HJ=,I4,:,5X,2HJ=,I4))
  140 FORMAT (3H I=,I4,2H: ,1P,11E11.3/(9X,11E11.3))
  150 FORMAT (/50H THE REDUCED PIJ DO NOT MEET THE RECIPROCITY CONDI,
     1 14HTIONS. RECIP =,1P,E10.3/)
  160 FORMAT (/50H THE REDUCED PIJ DO NOT MEET THE CONSERVATION COND,
     1 14HITIONS. LEAK =,1P,E10.3/)
      END
