*DECK JPMDIT
      SUBROUTINE JPMDIT(IPTRK,ITG,IMPX,NGEN,IFTRAK,ISURF,CHORD,SIGT,
     1 MASK,PII,PIS,PSS)
*
*-----------------------------------------------------------------------
*
* THIS SUBROUTINE COMPUTES THE COLLISION PROBABILITIES FOR THE
* INTERFACE CURRENT ANALYSIS OF AN ARBITRARY GEOMETRY.
*
* INPUT PARAMETERS:
*   IPTRK  : POINTER TO THE JPM TRACKING (L_TRACK SIGNATURE).
*   ITG    : TYPE OF GEOMETRY: 1 : HOMOGENEOUS.
*                              2 : 1-D/2-D/3-D PURE GEOMETRY.
*                              3 : DO-IT-YOURSELF MULTICELL.
*                              4 : 2-D ASSEMBLY (EURYDICE-2).
*                              5 : 2-D CLUSTER.
*   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*   NGEN   : TOTAL NUMBER OF GENERATING BLOCKS.
*   IFTRAK : FILE UNIT NUMBER USED TO STORE CLUSTER TRACKS.
*   ISURF  : NUMBER OF SURFACES SURROUNDING EACH GENERATING BLOCK.
*            DIMENSION ISURF(NGEN)
*   CHORD  : MEAN CHORD LENGTH ASSOCIATED WITH EACH SURFACE.
*            DIMENSION CHORD(NPIS) WHERE NPIS=SUM(ISURF(I)) IS THE
*            TOTAL NUMBER OF SURFACES SURROUNDING ALL THE GENERATING
*            BLOCKS.
*   SIGT   : MACROSCOPIC CROSS SECTIONS. DIMENSION SIGT(NGEN)
*   MASK   : COLLISION PROBABILITIES ARE COMPUTED ONLY IF MASK=.TRUE.
*            DIMENSION MASK(NGEN)
*
* OUTPUT PARAMETERS:
*   PII    : VOLUME TO VOLUME REDUCED COLLISION PROBABILITIES.
*            DIMENSION PII(NGEN)
*   PIS    : VOLUME TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PIS(NPIS) WHERE NPIS=SUM(ISURF(I))
*   PSS    : SURFACE TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PSS(NPSS) WHERE NPSS=SUM(ISURF(I)**2)
*
*----------------------------------------------- AUTHOR: A. HEBERT -----
*
      PARAMETER (EPS1=1.0E-4)
      LOGICAL MASK(*)
      DIMENSION ISURF(*),CHORD(*),SIGT(*),PII(*),PIS(*),PSS(*),IPAR(15)
      COMMON BASE(1)
*
      IF (ITG.EQ.1) THEN
         IF (MASK(1)) THEN
            PII(1)=1.0/SIGT(1)
            PIS(1)=0.0
            PSS(1)=0.0
         ENDIF
      ELSE IF (ITG.EQ.2) THEN
         CALL LCMSIX(IPTRK,'PURE-GEOM',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         ITYPE=IPAR(1)
         IAPP=IPAR(3)
         IQUAD=IPAR(4)
         IXX2=LCMIOF(IPTRK,'XX')
         IYY2=LCMIOF(IPTRK,'YY')
         IZZ2=LCMIOF(IPTRK,'ZZ')
         CALL LCMSIX(IPTRK,' ',2)
*
         CALL JPM2(ITYPE,IAPP,IQUAD,BASE(IXX2),BASE(IYY2),BASE(IZZ2),
     1   NGEN,ISURF,SIGT,MASK,PII,PIS,PSS)
      ELSE IF (ITG.EQ.3) THEN
         CALL LCMSIX(IPTRK,'DOITYOURSELF',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         NSUPCE=IPAR(1)
         IAPP=IPAR(2)
         IQUAD=IPAR(3)
         INMC3=LCMIOF(IPTRK,'NMC')
         IRAYR3=LCMIOF(IPTRK,'RAYRE')
         IPROC=LCMIOF(IPTRK,'PROCEL')
         CALL LCMSIX(IPTRK,' ',2)
*
         CALL JPM3(NSUPCE,IAPP,IQUAD,BASE(INMC3),BASE(IRAYR3),
     1   BASE(IPROC),NGEN,ISURF,SIGT,MASK,PII,PIS,PSS)
      ELSE IF (ITG.EQ.4) THEN
         CALL LCMSIX(IPTRK,'EURYDICE',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         IWIGN=IPAR(2)
         LMAIL=IPAR(10)
         IITGEN=LCMIOF(IPTRK,'ITGEN')
         ICOORD=LCMIOF(IPTRK,'COORD')
         IMAIL=LCMIOF(IPTRK,'MAIL')
         IF(LMAIL.GT.0) THEN
            IIZMAI=LCMIOF(IPTRK,'ZMAILI')
            IRZMAI=LCMIOF(IPTRK,'ZMAILR')
         ELSE
*           THIS INFO IS NOT REQUIRED IN THE CALLED ROUTINE.
            IIZMAI=IMAIL
            IRZMAI=IMAIL
         ENDIF
         CALL LCMSIX(IPTRK,' ',2)
*
         CALL JPM4(IWIGN,LMAIL,IPAR(4),BASE(IITGEN),BASE(ICOORD),
     1   BASE(IMAIL),BASE(IIZMAI),BASE(IRZMAI),NGEN,ISURF,SIGT,MASK,
     2   PII,PIS,PSS)
      ELSE IF (ITG.EQ.5) THEN
         CALL LCMSIX(IPTRK,'CLUSTER',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         IAPP=IPAR(1)
         IROT=IPAR(2)
         NAG=IPAR(6)
         NRT=IPAR(7)
         MSROD=IPAR(8)
         NSURF=IPAR(9)
         NSOUT=IPAR(10)
         MTRK=IPAR(11)
         CALL LCMGET(IPTRK,'YSIDE',COTE)
         NRINFO=LCMIOF(IPTRK,'NRINFO')
         JRAN=LCMIOF(IPTRK,'RAN')
         NRODS=LCMIOF(IPTRK,'NRODS')
         JRODS=LCMIOF(IPTRK,'RODS')
         JRODR=LCMIOF(IPTRK,'RODR')
         CALL LCMSIX(IPTRK,' ',2)
*
         REWIND IFTRAK
         CALL SETARA(BASE,4*MTRK,JTRAK)
         CALL XCSPIJ(IAPP,IROT,NGEN,NAG,NRT,MSROD,BASE(NRINFO),
     1               BASE(JRAN),BASE(NRODS),BASE(JRODS),BASE(JRODR),
     2               IPAR(3),IFTRAK,CHORD,NSURF,ISURF,NSOUT,COTE,MASK,
     3               BASE(JTRAK),SIGT,PII,PIS,PSS,IMPX)
         CALL RLSARA(BASE(JTRAK))
      ENDIF
*
*     PRINT THE BLOCKWISE COLLISION PROBABLILTIES.
      IF (IMPX.GE.8) THEN
         NPIS=0
         NPSS=0
         DO 35 IKG=1,NGEN
           NCOUR=ISURF(IKG)
           WRITE (6,100) IKG,SIGT(IKG),PII(IKG),MASK(IKG)
           WRITE (6,'(//14H MATRIX PIS(I)/)')
           WRITE (6,'(1X,1P,11E11.3)') (PIS(NPIS+I),I=1,NCOUR)
           WRITE (6,110) (J,J=1,NCOUR)
           DO 30 I=1,NCOUR
             WRITE (6,120) I,(PSS(NPSS+(J-1)*NCOUR+I),J=1,NCOUR)
   30      CONTINUE
           WRITE (6,'(///)')
         NPIS=NPIS+NCOUR
         NPSS=NPSS+NCOUR*NCOUR
   35    CONTINUE
      ENDIF
      IF ((IMPX.GE.10).OR.(IMPX.LE.-10)) THEN
*        CHECK THE RECIPROCITY AND CONSERVATION CONDITIONS.
         NPIS=0
         NPSS=0
         DO 60 IKG=1,NGEN
         NCOUR=ISURF(IKG)
C        IF (MASK(IKG)) THEN
            RECIP=0.0
            FLEAK1=1.0-PII(IKG)*SIGT(IKG)
            FLEAK2=0.0
            DO 50 I=1,NCOUR
            FLEAK1=FLEAK1-PIS(NPIS+I)
            WRK=1.0-PIS(NPIS+I)*CHORD(NPIS+I)*SIGT(IKG)
            DO 40 J=1,NCOUR
            WRK=WRK-PSS(NPSS+(J-1)*NCOUR+I)
            PSS1=PSS(NPSS+(J-1)*NCOUR+I)/CHORD(NPIS+I)
            PSS2=PSS(NPSS+(I-1)*NCOUR+J)/CHORD(NPIS+J)
   40       RECIP=MAX(RECIP,ABS(PSS1-PSS2))
   50       FLEAK2=MAX(FLEAK2,ABS(WRK))
            FLEAK1=ABS(FLEAK1)
            IF (RECIP.GE.EPS1) WRITE (6,130) IKG,RECIP
            IF (IMPX.GE.11) THEN
               IF (FLEAK1.GE.EPS1) WRITE (6,140) IKG,'1',FLEAK1
               IF (FLEAK2.GE.EPS1) WRITE (6,140) IKG,'2',FLEAK2
            ENDIF
C        ENDIF
         NPIS=NPIS+NCOUR
         NPSS=NPSS+NCOUR*NCOUR
   60    CONTINUE
      ENDIF
      RETURN
*
  100 FORMAT (//24H GENERATING BLOCK NUMBER,I5//19H TOTAL MACROSCOPIC ,
     1 15HCROSS SECTION =,1P,E11.3,5X,5HPII =,E11.3,5X,6HMASK =,L2)
  110 FORMAT (//29H MATRIX PSS(I,J) (I TOWARD J)//(11X,2HJ=,I4,:,5X,
     1 2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,
     2 5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4))
  120 FORMAT (3H I=,I4,2H: ,1P,11E11.3/(9X,11E11.3))
  130 FORMAT (/51H THE SURFACE TO SURFACE COLLISION PROBABILITIES DO ,
     1 56HNOT MEET THE RECIPROCITY CONDITIONS IN GENERATING BLOCK ,
     2 6HNUMBER,I4,1H./8H RECIP =,1P,E10.3/)
  140 FORMAT (/52H THE COLLISION PROBABILITIES DO NOT MEET THE CONSERV,
     1 43HATION CONDITIONS IN GENERATING BLOCK NUMBER,I4,6H. LEAK,A1,
     2 2H =,1P,E10.3/)
      END
