*DECK JPMG3
      SUBROUTINE JPMG3 (MAXPTS,MAXJ,IPGEOM,IR,MAT,VOL,ILK,NSUPCE,IAPP,
     1 IQUAD,IHALT,ISTAT,NMC,RAYRE,PROCEL,IMPX,NGEN,IFR,ALB,IJAT,INUM,
     2 MIX,DVX,IGEN,ISURF,CHORD,POURCE,SURFA,IWRK)
*
*-----------------------------------------------------------------------
*
* RECOVER AND ANALYSIS OF THE GEOMETRY FOR THE DESCRIPTION OF AN
* ARBITRARY MULTICELL USING THE 'DO-IT-YOURSELF' APPROACH.
*
* REFERENCE: A. HOFFMANN, F. JEANPIERRE, A. KAVENOKY, M. LIVOLANT
* AND H. LORAIN, 'APOLLO, CODE MULTIGROUPE DE RESOLUTION DE L'EQUATION
* DU TRANSPORT POUR LES NEUTRONS RAPIDES ET THERMIQUES', REPORT
* CEA-N-1610, COMMISSARIAT A L'ENERGIE ATOMIQUE, P. 13 (1973).
*
* GENERAL PARAMETERS:
*   MAXPTS : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION NSUPCE OR NGEN.
*   MAXJ   : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION IJAT OR IJAS
*            WHERE IJAT IS THE TOTAL NUMBER OF DISTINCT OUT-CURRENTS
*            AND IJAS IS THE TOTAL NUMBER OF OUT-CURRENTS LEAVING ALL
*            THE BLOCKS.
*   IPGEOM : POINTER TO THE GEOMETRY (L_GEOM SIGNATURE).
*   IR     : NUMBER OF MIXTURES.
*   MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*            DIMENSION MAT(NGEN)
*   VOL    : VOLUMES. DIMENSION VOL(NGEN)
*   ILK    : ILK=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
*            IS PRESENT.
*   NGEOM  : TYPE OF 1-D GEOMETRY FOR THE CELLS.
*   NSUPCE : NUMBER OF CELLS.
*   IAPP   : TYPE OF ANGULAR SURFACE FLUX REPRESENTATION.
*   IQUAD  : QUADRATURE PARAMETER.
*   IHALT  : =1 TO STOP ET THE END OF SUBROUTINE JPMG3.
*   ISTAT  : ISTAT=1 FOR THE STATISTICAL APPROXIMATION.
*   NMC    : OFFSET OF THE FIRST VOLUME IN EACH CELL.
*            DIMENSION NMC(NSUPCE+1)
*   RAYRE  : RADIUS OF THE TUBES IN EACH CELL.
*            DIMENSION RAYRE(NSUPCE+SUM(NMC(I+1)-NMC(I)))
*   PROCEL : USER SUPPLIED GEOMETRICAL MATRIX.
*            DIMENSION PROCEL(NSUPCE**2)
*   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*
*   PARAMETERS RELATED TO BLOCKS DEFINITION:
*   NGEN   : TOTAL NUMBER OF BLOCKS IN THE DOMAIN.
*   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAS) WHERE
*            IJAS IS THE TOTAL NUMBER OF IN-CURRENTS SURROUNDING ALL
*            THE BLOCKS.
*   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
*            DIMENSION ALB(IJAS)
*   IJAT   : TOTAL NUMBER OF DISTINCT OUT-CURRENTS.
*   INUM   : INDEX-NUMBER OF THE MERGED BLOCK ASSOCIATED TO EACH BLOCK
*            (INUM(I)=I IN GEOM3). DIMENSION INUM(NGEN)
*   MIX    : INDEX-NUMBER OF OUT-CURRENTS. DIMENSION MIX(IJAS)
*   DVX    : WEIGHT ASSOCIATED WITH EACH OUT-CURRENT.
*            DIMENSION DVX(IJAS)
*   IGEN   : INDEX-NUMBER OF THE GENERATING BLOCK ASSOCIATED WITH EACH
*            BLOCK (IGEN(I)=I IN GEOM3). DIMENSION IGEN(NGEN)
*   ISURF :  NUMBER OF SURFACES ASSOCIATED WITH EACH BLOCK.
*            DIMENSION ISURF(NGEN)
*   CHORD  : MEAN CHORD LENGTH ASSOCIATED WITH EACH OUT-CURRENT.
*            DIMENSION CHORD(IJAS)
*
*            REMARK:
*            IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
*            GEOMETRICAL "A" MATRIX.
*
* SCRATCH STORAGE:
*   POURCE : DIMENSION POURCE(MAXPTS)
*   SURFA  : DIMENSION SURFA(MAXPTS)
*   IWRK   : DIMENSION IWRK(MAXPTS)
*
*---------------------------------------- AUTHOR: A. HEBERT 26/05/86 ---
*
      PARAMETER (PI=3.141592654,EPS1=1.0E-5,EPS2=1.0E-4)
      LOGICAL ILK
      DIMENSION MAT(*),VOL(*),NMC(*),RAYRE(*),PROCEL(*),IFR(*),ALB(*),
     1 INUM(*),MIX(*),DVX(*),IGEN(*),ISURF(*),CHORD(*),POURCE(*),
     2 SURFA(*),IWRK(*)
      COMMON BASE(1)
*
      CALL SETARA(BASE,MAXPTS,INBREG)
      CALL SETARA(BASE,MAXPTS,IMILIE)
      CALL SETARA(BASE,MAXPTS,IRAYZO)
      CALL SETARA(BASE,3*MAXPTS,ICELL)
      CALL READMT (MAXPTS,IPGEOM,IR,MAT,VOL,ILK,ISTAT,NSUPCE,NGEN,NMC,
     1 RAYRE,PROCEL,POURCE,SURFA,IMPX,BASE(INBREG),BASE(IMILIE),
     2 BASE(IRAYZO),BASE(ICELL))
      CALL RLSARA(BASE(ICELL))
      CALL RLSARA(BASE(IRAYZO))
      CALL RLSARA(BASE(IMILIE))
      CALL RLSARA(BASE(INBREG))
      IF(ISTAT.EQ.1) CALL XABORT('JPMG3: STAT OPTION NOT IMPLEMENTED.')
*
* IWRK IS THE INDEX-NUMBER OF THE CELL'S EXTERNAL IN-CURRENT.
      IJAT=0
      DO 140 IKK=1,NSUPCE
      I2=NMC(IKK+1)-NMC(IKK)
      IF(IAPP.EQ.1) THEN
         IJAT=IJAT+2*I2-1
         IWRK(IKK)=IJAT
      ELSE IF(IAPP.EQ.2) THEN
         IJAT=IJAT+4*I2-3
         IWRK(IKK)=IJAT
      ENDIF
  140 CONTINUE
      IF(IMPX.GE.1) THEN
         WRITE (6,'(/23H QUADRATURE PARAMETER =,I4)') IQUAD
         WRITE (6,'(/4H DP-,I1,36H ANGULAR FLUX REPRESENTATION BETWEEN,
     1   11H THE TUBES.)') IAPP-1
         WRITE (6,'(/34H NUMBER OF DISTINCT OUT-CURRENTS =,I4,5X,
     1   17HAVAILABLE SPACE =,I4)') IJAT,MAXJ
      ENDIF
      IF(IJAT.GT.MAXJ) CALL XABORT('JPMG3: INSUFFICIENT MAXJ.')
*
* COMPUTATION OF IFR, ALB, MIX, DVX, CHORD AND ISURF.
      I1=0
      IJAS=0
      DO 170 IKK=1,NSUPCE
      RINT=0.0
      I2=NMC(IKK+1)-NMC(IKK)
      DO 160 IP=1,I2
      NIS=0
      IBLK=I1+IP
      INUM(IBLK)=IBLK
      IGEN(IBLK)=IBLK
      ROUT=RAYRE(IKK+IBLK)
      VOLUME=VOL(IBLK)
      IF(IAPP.EQ.1) THEN
*        DP-0 APPROXIMATION.
         IF(IP.GT.1) THEN
            NIS=NIS+1
            MIX(IJAS+NIS)=IWRK(IKK)-2*(I2-IP)-1
            DVX(IJAS+NIS)=1.0
            IFR(IJAS+NIS)=MIX(IJAS+NIS)-1
            ALB(IJAS+NIS)=1.0
            CHORD(IJAS+NIS)=4.0*VOLUME/(2.0*PI*RINT*POURCE(IKK))
         ENDIF
         IF(IP.LT.I2) THEN
            NIS=NIS+1
            MIX(IJAS+NIS)=IWRK(IKK)-2*(I2-IP)
            DVX(IJAS+NIS)=1.0
            IFR(IJAS+NIS)=MIX(IJAS+NIS)+1
            ALB(IJAS+NIS)=1.0
            CHORD(IJAS+NIS)=4.0*VOLUME/(2.0*PI*ROUT*POURCE(IKK))
         ENDIF
      ELSE IF(IAPP.EQ.2) THEN
*        DP-1 APPROXIMATION.
         IF(IP.GT.1) THEN
            NIS=NIS+1
            MIX(IJAS+NIS)=IWRK(IKK)-4*(I2-IP)-2
            DVX(IJAS+NIS)=1.0
            IFR(IJAS+NIS)=MIX(IJAS+NIS)-2
            ALB(IJAS+NIS)=1.0
            CHORD(IJAS+NIS)=4.0*VOLUME/(2.0*PI*RINT*POURCE(IKK))
            NIS=NIS+1
            MIX(IJAS+NIS)=MIX(IJAS+NIS-1)+1
            DVX(IJAS+NIS)=1.0
            IFR(IJAS+NIS)=MIX(IJAS+NIS)-2
            ALB(IJAS+NIS)=1.0
            CHORD(IJAS+NIS)=CHORD(IJAS+NIS-1)
         ENDIF
         IF(IP.LT.I2) THEN
            NIS=NIS+1
            MIX(IJAS+NIS)=IWRK(IKK)-4*(I2-IP)
            DVX(IJAS+NIS)=1.0
            IFR(IJAS+NIS)=MIX(IJAS+NIS)+2
            ALB(IJAS+NIS)=1.0
            CHORD(IJAS+NIS)=4.0*VOLUME/(2.0*PI*ROUT*POURCE(IKK))
            NIS=NIS+1
            MIX(IJAS+NIS)=MIX(IJAS+NIS-1)+1
            DVX(IJAS+NIS)=1.0
            IFR(IJAS+NIS)=MIX(IJAS+NIS)+2
            ALB(IJAS+NIS)=1.0
            CHORD(IJAS+NIS)=CHORD(IJAS+NIS-1)
         ENDIF
      ENDIF
      IF(IP.EQ.I2) THEN
         DO 150 JKK=1,NSUPCE
         IOF=(JKK-1)*NSUPCE+IKK
         IF(PROCEL(IOF).NE.0.0) THEN
            NIS=NIS+1
            IFR(IJAS+NIS)=IWRK(JKK)
            ALB(IJAS+NIS)=1.0
            MIX(IJAS+NIS)=IWRK(IKK)
            DVX(IJAS+NIS)=PROCEL(IOF)
            CHORD(IJAS+NIS)=4.0*VOLUME/(PROCEL(IOF)*SURFA(IKK))
         ENDIF
  150    CONTINUE
      ENDIF
      ISURF(IBLK)=NIS
      IJAS=IJAS+NIS
  160 RINT=ROUT
  170 I1=I1+I2
*
* COMPRESSION OF THE USER SUPPLIED GEOMETRICAL MATRIX.
      DO 200 IKK=1,NSUPCE
      DO 200 JKK=1,NSUPCE
      IOF=(JKK-1)*NSUPCE+IKK
      IF(PROCEL(IOF).EQ.0.0) THEN
         DO 190 J1=JKK+1,NSUPCE
         IF(PROCEL((J1-1)*NSUPCE+IKK).NE.0.0) THEN
            DO 180 J2=0,NSUPCE-J1
  180       PROCEL((JKK+J2-1)*NSUPCE+IKK)=PROCEL((J1+J2-1)*NSUPCE+IKK)
            GO TO 200
         ENDIF
  190    CONTINUE
      ENDIF
  200 CONTINUE
*
      IF(IMPX.GE.1) WRITE (6,'(/31H TOTAL NUMBER OF OUT-CURRENTS =,I4,
     1 5X,17HAVAILABLE SPACE =,I4/)') IJAS,MAXJ
      IF(IJAS.GT.MAXJ) CALL XABORT('JPMG3: INSUFFICIENT MAXJ.')
      IF(IHALT.EQ.1) CALL XABORT('JPMG3: STOP REQUESTED BY USER.')
      RETURN
      END
