*DECK JPMGEO
      SUBROUTINE JPMGEO (IPTRK,IPGEOM,MAXPTS,MAXJ,MAXZ,IR,ITG,VOL,MAT,
     1 ILK,IMPX,IAPP,MULTC,IWIGN,IHALT,ILIGN,INORM,IRECT,IQW,JQUA1,
     2 JQUA2,IFTRAK,ITRAK,IROT,IBPP,JQUAB,NMBLK,IJAS,IFR,ALB,NMERGE,
     3 IJAT,IJAR,INUM,MIX,DVX,NGEN,NPIS,IGEN,ISURF,CHORD)
*
*-----------------------------------------------------------------------
*
* DESCRIPTION OF THE GEOMETRY FOR JPM.
*
* GENERAL PARAMETERS:
*  IPTRK  : POINTER TO THE SYBIL TRACKING (L_TRACK SIGNATURE).
*  IPGEOM : POINTER TO THE GEOMETRY (L_GEOM SIGNATURE).
*  MAXPTS : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION NMBLK, NMERGE OR
*           NGEN.
*  MAXJ   : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION IJAT OR IJAS.
*  MAXZ   : ALLOCATED STORAGE FOR STORING THE TRACKING INFORMATION.
*  IR     : NUMBER OF MIXTURES.
*  ITG    : TYPE OF GEOMETRY: 1 : HOMOGENEOUS.
*                             2 : 1-D/2-D/3-D PURE GEOMETRY.
*                             3 : DO-IT-YOURSELF MULTICELL.
*                             4 : 2-D ASSEMBLY (EURYDICE-2).
*                             5 : 2-D CLUSTER (TRAFIC).
*                            10 : DOUBLE HETEROGENEITY (BIHET-2).
*  VOL    : VOLUMES. DIMENSION VOL(NMERGE)
*  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*           DIMENSION MAT(NMERGE)
*  ILK    : ILK=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
*           IS PRESENT.
*  IMPX   : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
*
* TRACKING OPTIONS:
*  IAPP   : =1 FOR DP-0  ;  =2 FOR DP-1  ;
*           =3 DP-1 WITH ISOTROPIC BOUNDARY CONDITION.
*  MULTC  : TYPE OF MULTICELL APPROXIMATION IN EURYDICE.
*  IWIGN  : =1/2/3: ASKEW/WIGNER/SANCHEZ CYLINDERIZATION IN EURYDICE.
*  IHALT  : =1 TO STOP AFTER CALCULATION OF THE TRACKING.
*  ILIGN  : =1 TO PRINT THE TRACKING.
*  INORM  : =1 TO AVOID TRACK NORMALIZATION.
*  IRECT  : =1 TO AVOID CONSIDERING SYMMETRIES OF SQUARE CELLS.
*  IQW    : =1 TO USE EQUAL WEIGHT QUADRATURES IN ANGLE AND SPACE.
*  JQUA1  : QUADRATURE PARAMETER FOR 1-D GEOMETRIES.
*  JQUA2  : QUADRATURE PARAMETER FOR 2-D GEOMETRIES (ANGLE AND SPACE).
*           DIMENSION JQUA2(2)
*  IFTRAK : UNIT NUMBER OF THE TRACKING FILE CFTRAK FOR CLUSTERS.
*  ITRAK  : =1  PERFORM A NEW TRACKING AND BUILD FILE CFTRAK;
*           =0  RECOVER THE TRACKING OF AN EXISTING CFTRAK FILE (OLD);
*           =-1 RECOVER THE TRACKING OF AN EXISTING CFTRAK FILE (MOD).
*  IROT   : =1 DIRECT CLUSTER RECONSTRUCTION ;
*           =2 SURFACE SPLITTING CLUSTER RECONSTRUCTION.
*  IBPP   : =1 FOR DP-0 IN BIHET ;  =2 FOR DP-1 IN BIHET
*  JQUAB  : QUADRATURE PARAMETER FOR BIHET.
*
* PARAMETERS RELATED TO BLOCK DEFINITION:
*  NMBLK  : TOTAL NUMBER OF BLOCKS IN THE DOMAIN.
*  IJAS   : TOTAL NUMBER OF DISTINCT INCOMING CURRENTS SURROUNDING ALL
*           THE BLOCKS (MERGED OR NOT).
*  IFR    : INDEX-NUMBER OF INCOMING CURRENTS. DIMENSION IFR(IJAS)
*  ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH INCOMING CURRENT.
*           DIMENSION ALB(IJAS)
*
* PARAMETERS RELATED TO THE MERGED BLOCK DEFINITION:
*  NMERGE : TOTAL NUMBER OF MERGED BLOCKS FOR WHICH SPECIFIC VALUES
*           OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*           MANY BLOCKS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*           BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*           OWN THE SAME GENERATING BLOCK. THIS ALLOWS SOME REDUCTION
*           IN CPU TIME AND MEMORY. NMERGE.LE.NMBLK
*  IJAT   : TOTAL NUMBER OF DISTINCT OUTCOMING CURRENTS.
*  IJAR   : TOTAL NUMBER OF OUTCOMING CURRENTS LEAVING ALL THE MERGED
*           BLOCKS.
*  INUM   : INDEX-NUMBER OF THE MERGED BLOCK ASSOCIATED TO EACH BLOCK.
*           DIMENSION INUM(NMBLK)
*  MIX    : INDEX-NUMBER OF OUTCOMING CURRENTS. DIMENSION MIX(IJAR)
*  DVX    : WEIGHT ASSOCIATED WITH EACH OUTCOMING CURRENT.
*           DIMENSION DVX(IJAR)
*
*           REMARK:
*           IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
*           GEOMETRICAL "A" MATRIX.
*
* PARAMETERS RELATED TO THE GENERATING BLOCK DEFINITION:
*  NGEN   : TOTAL NUMBER OF GENERATING BLOCKS. A GENERATING BLOCK IS
*           DEFINED BY ITS MIXTURE AND DIMENSIONS IRRESPECTIVE OF
*           ITS POSITION IN THE DOMAIN. NGEN.LE.NMERGE
*  NPIS   : TOTAL NUMBER OF SURFACES SURROUNDING ALL THE GENERATING
*           BLOCKS.
*  IGEN   : INDEX-NUMBER OF THE GENERATING BLOCK ASSOCIATED WITH EACH
*           MERGED BLOCK. DIMENSION IGEN(NMERGE)
*  ISURF  : NUMBER OF SURFACES ASSOCIATED WITH EACH GENERATING BLOCK.
*           DIMENSION ISURF(NGEN)
*  CHORD  : MEAN CHORD LENGTH ASSOCIATED WITH EACH SURFACE.
*           DIMENSION CHORD(NPIS)
*
*---------------------------------------------- A. HEBERT 21/01/87 -----
*
      PARAMETER(NSTATE=40)
      LOGICAL ILK
      DIMENSION VOL(*),MAT(*),IFR(*),ALB(*),INUM(*),MIX(*),DVX(*),
     1 IGEN(*),ISURF(*),CHORD(*),JQUA2(*),ISTATE(NSTATE),IPARAM(15),
     2 IQUAD(4),NCODE(6),ICODE(6),ALBEDO(6),ZCODE(6),IBASE(1)
      COMMON       BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      CALL LCMGET(IPGEOM,'STATE-VECTOR',ISTATE)
      ITYGEO=ISTATE(1)
      ITYNRE=ISTATE(6)
      ITYEUR=ISTATE(8)
      ITYASM=ISTATE(9)
      ITYGDH=ISTATE(12)
      ITYCLS=ISTATE(13)
      IF(ITYGEO.EQ.1) THEN
         ITG=1
         IF(ITYNRE.NE.1) CALL XABORT('JPMGEO: INVALID NUMBER OF REG'
     1   //'IONS.')
         CALL LCMLEN(IPGEOM,'MIX',NMILG,ITYLCM)
         IF(NMILG.NE.1) CALL XABORT('JPMGEO: INVALID MIX VECTOR.')
         CALL LCMGET(IPGEOM,'MIX',MAT(1))
         IR=MAT(1)
         VOL(1)=1.0
         ILK=.FALSE.
         NMBLK=1
         IFR(1)=1
         ALB(1)=1.0
         NMERGE=1
         IJAT=1
         INUM(1)=1
         MIX(1)=1
         DVX(1)=1.0
         NGEN=1
         IGEN(1)=1
         ISURF(1)=1
         CHORD(1)=1.0E10
      ELSE IF((ITYGEO.GT.1).AND.(ITYGEO.LT.8).AND.(ITYASM.EQ.
     1 0)) THEN
         ITG=2
         ITYPE=ITYGEO
         CALL SETARA(BASE,MAXPTS,IXX2)
         CALL SETARA(BASE,MAXPTS,IYY2)
         CALL SETARA(BASE,MAXPTS,IZZ2)
*
         CALL SETARA(BASE,MAXPTS+1,IXXX)
         CALL SETARA(BASE,MAXPTS+1,IYYY)
         CALL SETARA(BASE,MAXPTS+1,IZZZ)
         CALL SETARA(BASE,MAXPTS,IIORI)
         CALL JPMG2(MAXPTS,MAXJ,IPGEOM,IR,MAT,VOL,ILK,ITYGEO,
     1   IHEX,IAPP,JQUA1,IHALT,BASE(IXX2),BASE(IYY2),BASE(IZZ2),
     2   IMPX,NMBLK,IFR,ALB,NMERGE,IJAT,INUM,MIX,DVX,NGEN,IGEN,
     3   ISURF,CHORD,BASE(IXXX),BASE(IYYY),BASE(IZZZ),BASE(IIORI))
         CALL RLSARA(BASE(IIORI))
         CALL RLSARA(BASE(IZZZ))
         CALL RLSARA(BASE(IYYY))
         CALL RLSARA(BASE(IXXX))
*
         IPARAM(1)=ITYGEO
         IPARAM(2)=IHEX
         IPARAM(3)=IAPP
         IPARAM(4)=JQUA1
         CALL LCMSIX(IPTRK,'PURE-GEOM',1)
         CALL LCMPUT(IPTRK,'PARAM',4,1,IPARAM)
         CALL LCMPUT(IPTRK,'XX',NGEN,2,BASE(IXX2))
         CALL LCMPUT(IPTRK,'YY',NGEN,2,BASE(IYY2))
         CALL LCMPUT(IPTRK,'ZZ',NGEN,2,BASE(IZZ2))
         CALL LCMSIX(IPTRK,' ',2)
         CALL RLSARA(BASE(IZZ2))
         CALL RLSARA(BASE(IYY2))
         CALL RLSARA(BASE(IXX2))
      ELSE IF(ITYGEO.EQ.30) THEN
         ITG=3
         CALL SETARA(IBASE,1+MAXPTS,INMC3)
         CALL SETARA(BASE,2*MAXPTS,IRAYR3)
         CALL SETARA(BASE,MAXPTS**2,IPROCE)
*
         CALL SETARA(BASE,MAXPTS,IPOURC)
         CALL SETARA(BASE,MAXPTS,ISURFA)
         CALL SETARA(BASE,MAXPTS,IIWRK)
         CALL JPMG3(MAXPTS,MAXJ,IPGEOM,IR,MAT,VOL,ILK,NSUPCE,IAPP,
     1   JQUA1,IHALT,ISTAT,IBASE(INMC3),BASE(IRAYR3),BASE(IPROCE),
     2   IMPX,NMBLK,IFR,ALB,IJAT,INUM,MIX,DVX,IGEN,ISURF,CHORD,
     3   BASE(IPOURC),BASE(ISURFA),BASE(IIWRK))
         CALL RLSARA(BASE(IIWRK))
         CALL RLSARA(BASE(ISURFA))
         CALL RLSARA(BASE(IPOURC))
         NGEN=NMBLK
         NMERGE=NMBLK
*
         IPARAM(1)=NSUPCE
         IPARAM(2)=IAPP
         IPARAM(3)=JQUA1
         IPARAM(4)=ISTAT
         CALL LCMSIX(IPTRK,'DOITYOURSELF',1)
         CALL LCMPUT(IPTRK,'PARAM',4,1,IPARAM)
         CALL LCMPUT(IPTRK,'NMC',1+NSUPCE,1,IBASE(INMC3))
         CALL LCMPUT(IPTRK,'RAYRE',NMBLK+NSUPCE,2,BASE(IRAYR3))
         CALL LCMPUT(IPTRK,'PROCEL',NSUPCE**2,2,BASE(IPROCE))
         CALL LCMSIX(IPTRK,' ',2)
         CALL RLSARA(BASE(IPROCE))
         CALL RLSARA(BASE(IRAYR3))
         CALL RLSARA(IBASE(INMC3))
      ELSE IF( ((ITYGEO.EQ. 5).AND.(ITYEUR.EQ.1)) .OR.
     1         ((ITYGEO.EQ. 8).AND.(ITYEUR.EQ.1)) .OR.
     2         ((ITYGEO.EQ.20).AND.(ITYCLS.EQ.0)) .OR.
     3         ((ITYGEO.EQ.24).AND.(ITYCLS.EQ.0)) ) THEN
         ITG=4
         IF(IAPP.EQ.3) IAPP=2
         CALL SETARA(IBASE,MAXPTS,IITGEN)
         CALL SETARA(BASE,3*MAXPTS,ICOORD)
         CALL SETARA(IBASE,MAXPTS,IMAIL)
         CALL SETARA(IBASE,MAXZ,IIZMAI)
         CALL SETARA(BASE,MAXZ,IRZMAI)
         IQUAD(1)=JQUA2(1)
         IQUAD(2)=JQUA2(2)
         IQUAD(3)=JQUA1
         IQUAD(4)=JQUA1
*
         CALL SETARA(BASE,MAXPTS,IXX4)
         CALL SETARA(BASE,MAXPTS,IYY4)
         CALL SETARA(BASE,MAXPTS,IRAYR)
         CALL SETARA(BASE,MAXPTS+1,INMC)
         CALL SETARA(BASE,MAXPTS,IIORI)
         CALL JPMEUR(MAXPTS,MAXJ,MAXZ,IPGEOM,IR,MAT,VOL,ILK,LMAIL,
     1   IBASE(IITGEN),BASE(ICOORD),IBASE(IMAIL),IBASE(IIZMAI),
     2   BASE(IRZMAI),IMPX,IAPP,MULTC,IWIGN,IHALT,ILIGN,INORM,IRECT,
     3   IQW,IQUAD,NMBLK,IFR,ALB,NMERGE,IJAT,INUM,MIX,DVX,NGEN,IGEN,
     4   ISURF,CHORD,BASE(IXX4),BASE(IYY4),BASE(IRAYR),BASE(INMC),
     5   BASE(IIORI))
         CALL RLSARA(BASE(IIORI))
         CALL RLSARA(BASE(INMC))
         CALL RLSARA(BASE(IRAYR))
         CALL RLSARA(BASE(IYY4))
         CALL RLSARA(BASE(IXX4))
*
         IPARAM(1)=MULTC
         IPARAM(2)=IWIGN
         IPARAM(3)=IAPP
         IPARAM(4)=IQUAD(1)
         IPARAM(5)=IQUAD(2)
         IPARAM(6)=IQUAD(3)
         IPARAM(7)=IQUAD(4)
         IPARAM(8)=INORM
         IPARAM(9)=IQW
         IPARAM(10)=LMAIL
         CALL LCMSIX(IPTRK,'EURYDICE',1)
         CALL LCMPUT(IPTRK,'PARAM',10,1,IPARAM)
         CALL LCMPUT(IPTRK,'ITGEN',NGEN,1,IBASE(IITGEN))
         CALL LCMPUT(IPTRK,'COORD',3*NGEN,2,BASE(ICOORD))
         CALL LCMPUT(IPTRK,'MAIL',NGEN,1,IBASE(IMAIL))
         IF(LMAIL.GT.0) THEN
            CALL LCMPUT(IPTRK,'ZMAILI',LMAIL,1,IBASE(IIZMAI))
            CALL LCMPUT(IPTRK,'ZMAILR',LMAIL,2,BASE(IRZMAI))
         ENDIF
         CALL LCMSIX(IPTRK,' ',2)
         CALL RLSARA(BASE(IRZMAI))
         CALL RLSARA(IBASE(IIZMAI))
         CALL RLSARA(IBASE(IMAIL))
         CALL RLSARA(BASE(ICOORD))
         CALL RLSARA(IBASE(IITGEN))
      ELSE IF( ((ITYGEO.EQ. 3)  .OR.  (ITYGEO.EQ.20).OR.
     1          (ITYGEO.EQ.24)) .AND. (ITYCLS.GE.1) ) THEN
         ITG=5
         IQUAD(1)=JQUA2(1)
         IQUAD(2)=JQUA2(2)
         IQUAD(3)=JQUA1
         IF(ITYGEO.EQ.3) THEN
           NSOUT=1
         ELSE IF(ITYGEO.EQ.20) THEN
           NSOUT=4
         ELSE IF(ITYGEO.EQ.24) THEN
           NSOUT=6
         ENDIF
         CALL XCGDIM(IPGEOM,MAXPTS,NSOUT,IROT,IAPP,MAXJ,NGEN,NAG,MNAN,
     1   NRT,MSROD,MAROD,NSURF)
*
*        ALLOCATE BLOCKS FOR GEOMETRY.
         NMBLK=NGEN
         NMERGE=NGEN
         CALL SETARA(IBASE,2*NAG,NRINFO)
         CALL SETARA(BASE,NAG,JRAN)
         CALL SETARA(IBASE,3*NRT,NRODS)
         CALL SETARA(BASE,2*NRT,JRODS)
         CALL SETARA(BASE,MSROD*NRT,JRODR)
*
*        ALLOCATE TEMPORARY BLOCKS
         CALL SETARA(IBASE,NSOUT+NGEN+1,MATALB)
         CALL SETARA(BASE,NSOUT+NGEN+1,JVS)
         CALL SETARA(IBASE,NSOUT+NGEN+1,KEYMRG)
         CALL SETARA(IBASE,NAG,MATANN)
         CALL SETARA(IBASE,MSROD*NRT,MATROD)
         CALL SETARA(IBASE,MNAN,ISPLIT)
         CALL SETARA(BASE,MNAN,JRAD)
         CALL SETARA(IBASE,NRT,NXRS)
         CALL SETARA(IBASE,NRT*NAG,NXRI)
         CALL SETARA(BASE,2*NRT,JVRG)
         CALL SETARA(IBASE,NRT,NRODR)
         CALL XCGGEO(IPGEOM,IROT,NSOUT,NGEN,NAG,MNAN,NRT,MSROD,IMPX,
     1   ILK,IR,BASE(JRAN),IBASE(MATANN),IBASE(NRODS),BASE(JRODS),
     2   IBASE(MATROD),IBASE(NRODR),BASE(JRODR),IBASE(NRINFO),
     3   IBASE(MATALB),BASE(JVS),IBASE(ISPLIT),BASE(JRAD),COTE,
     4   RADMIN,NCODE,ICODE,ZCODE,ALBEDO,IBASE(KEYMRG),IBASE(NXRS),
     5   IBASE(NXRI),BASE(JVRG))
         CALL RLSARA(IBASE(NRODR))
         CALL RLSARA(BASE(JVRG))
         CALL RLSARA(IBASE(NXRI))
         CALL RLSARA(IBASE(NXRS))
         CALL RLSARA(BASE(JRAD))
         CALL RLSARA(IBASE(ISPLIT))
         CALL RLSARA(IBASE(MATROD))
         CALL RLSARA(IBASE(MATANN))
         CALL RLSARA(IBASE(KEYMRG))
*
*        CARRY OUT SURFACE NUMBERING
         CALL SETARA(BASE,NGEN,JVOLW)
         CALL XCSNUM(NSOUT,NGEN,NAG,NRT,MSROD,IMPX,IROT,IAPP,NSURF,
     1   BASE(JRAN),COTE,IBASE(NRODS),BASE(JRODR),IBASE(NRINFO),
     2   IBASE(MATALB),BASE(JVS),ZCODE,MAT,VOL,ISURF,BASE(JVOLW),
     3   CHORD,IFR,MIX,ALB,DVX)
         CALL RLSARA(BASE(JVOLW))
         CALL RLSARA(BASE(JVS))
         CALL RLSARA(IBASE(MATALB))
         IJAT=NSURF
         DO 10 IREG=1,NGEN
           IGEN(IREG)=IREG
           INUM(IREG)=IREG
   10    CONTINUE
*
*        ALLOCATE TRACKING BLOCKS
         IF(IFTRAK.EQ.0) CALL XABORT('JPMGEO: NO SEQUENTIAL BINARY FIL'
     1   //'E DEFINED TO HOLD THE TRACKS.')
         MRTRK=2*MAROD+3
         NMTRAK=3*MAX(IQUAD(1),IQUAD(2))+4*IQUAD(1)*IQUAD(2)*MRTRK
         IF(ITRAK.EQ.1) THEN
*          OPEN NEW TRACKING UNIT
           CALL XCSANA(NAG,NRT,MSROD,IBASE(NRINFO),BASE(JRAN),
     1     IBASE(NRODS),BASE(JRODS),BASE(JRODR),IQUAD,IFTRAK,IMPX,
     2     MRTRK,NMTRAK,MTRK,NSOUT,COTE)
         ELSE
*          OPEN OLD TRACKING UNIT
           MTRK=NMTRAK
         ENDIF
*
         IPARAM(1)=IAPP
         IPARAM(2)=IROT
         IPARAM(3)=IQUAD(1)
         IPARAM(4)=IQUAD(2)
         IPARAM(5)=IQUAD(3)
         IPARAM(6)=NAG
         IPARAM(7)=NRT
         IPARAM(8)=MSROD
         IPARAM(9)=NSURF
         IPARAM(10)=NSOUT
         IPARAM(11)=MTRK
         CALL LCMSIX(IPTRK,'CLUSTER',1)
         CALL LCMPUT(IPTRK,'PARAM',11,1,IPARAM)
         CALL LCMPUT(IPTRK,'YSIDE',1,2,COTE)
         CALL LCMPUT(IPTRK,'NRINFO',2*NAG,1,IBASE(NRINFO))
         CALL LCMPUT(IPTRK,'RAN',NAG,2,BASE(JRAN))
         CALL LCMPUT(IPTRK,'NRODS',3*NRT,1,IBASE(NRODS))
         CALL LCMPUT(IPTRK,'RODS',2*NRT,2,BASE(JRODS))
         CALL LCMPUT(IPTRK,'RODR',MSROD*NRT,2,BASE(JRODR))
         CALL LCMSIX(IPTRK,' ',2)
         CALL RLSARA(BASE(JRODR))
         CALL RLSARA(BASE(JRODS))
         CALL RLSARA(IBASE(NRODS))
         CALL RLSARA(BASE(JRAN))
         CALL RLSARA(IBASE(NRINFO))
      ELSE
        CALL XABORT('JPMGEO: INVALID GEOMETRY MODULE.')
      ENDIF
*
      IF(ITYGDH.EQ.1) THEN
*        DOUBLE HETEROGENEITY OPTION:
         ITGOLD=ITG
         ITG=10
         CALL LCMLEN(IPGEOM,'NS',NG,ITYLCM)
         CALL SETARA(IBASE,NG,INS)
         CALL LCMGET(IPGEOM,'NS',IBASE(INS))
         NSMAX=0
         DO 15 I=1,NG
   15    NSMAX=MAX(NSMAX,IBASE(INS+I-1))
         NGEN2=NGEN
         IR2=IR
         CALL SETARA(IBASE,NGEN2,IIGI)
         CALL SETARA(BASE,NG*(1+NSMAX),IRS)
         CALL SETARA(BASE,NG*IR2,IFRACT)
         CALL SETARA(IBASE,NGEN2,ISF2)
         CALL SETARA(BASE,MAXJ,ICH2)
*
         CALL SETARA(BASE,NG*NSMAX*IR,IITEM)
         CALL SETARA(BASE,NMERGE,IIBI)
         CALL JPMGG (MAXPTS,MAXJ,IPGEOM,IR,MAT,VOL,NGEN2,NG,NSMAX,
     1   MICRO,JQUAB,IBPP,IHALT,IBASE(INS),IBASE(IIGI),BASE(IRS),
     2   BASE(IFRACT),IBASE(ISF2),BASE(ICH2),IMPX,NMBLK,IFR,ALB,NMERGE,
     3   IJAT,INUM,MIX,DVX,NGEN,IGEN,ISURF,CHORD,BASE(IITEM),BASE(IIBI))
         CALL RLSARA(BASE(IIBI))
         CALL RLSARA(BASE(IITEM))
*
         NPIS2=0
         DO 20 I=1,NGEN2
   20    NPIS2=NPIS2+IBASE(ISF2+I-1)
         IPARAM(1)=IR2
         IPARAM(2)=NGEN2
         IPARAM(3)=NG
         IPARAM(4)=NSMAX
         IPARAM(5)=ITGOLD
         IPARAM(6)=MICRO
         IPARAM(7)=JQUAB
         IPARAM(8)=IBPP
         IPARAM(9)=NPIS2
         CALL LCMSIX(IPTRK,'BIHET',1)
         CALL LCMPUT(IPTRK,'PARAM',9,1,IPARAM)
         CALL LCMPOF(IPTRK,'NS',NG,1,INS)
         CALL LCMPOF(IPTRK,'IGI',NGEN2,1,IIGI)
         CALL LCMPOF(IPTRK,'RS',NG*(1+NSMAX),2,IRS)
         CALL LCMPOF(IPTRK,'FRACT',NG*IR2,2,IFRACT)
         CALL LCMPOF(IPTRK,'ISURF2',NGEN2,1,ISF2)
         CALL LCMPUT(IPTRK,'CHORD2',NPIS2,2,BASE(ICH2))
         CALL LCMSIX(IPTRK,' ',2)
         CALL RLSARA(BASE(ICH2))
      ENDIF
      NPIS=0
      DO 30 I=1,NGEN
   30 NPIS=NPIS+ISURF(I)
      IJAR=0
      DO 40 I=1,NMERGE
   40 IJAR=IJAR+ISURF(IGEN(I))
      IJAS=0
      DO 50 I=1,NMBLK
   50 IJAS=IJAS+ISURF(IGEN(INUM(I)))
      CALL LCMPUT(IPTRK,'IFR',IJAS,1,IFR)
      CALL LCMPUT(IPTRK,'ALB',IJAS,2,ALB)
      CALL LCMPUT(IPTRK,'INUM',NMBLK,1,INUM)
      CALL LCMPUT(IPTRK,'MIX',IJAR,1,MIX)
      CALL LCMPUT(IPTRK,'DVX',IJAR,2,DVX)
      CALL LCMPUT(IPTRK,'IGEN',NMERGE,1,IGEN)
      CALL LCMPUT(IPTRK,'ISURF',NGEN,1,ISURF)
      CALL LCMPUT(IPTRK,'CHORD',NPIS,2,CHORD)
*
      IF(IMPX.GE.4) THEN
         WRITE (6,'(//19H GENERATING BLOCKS:)')
         I1=1
         DO 60 I=1,1+(NPIS-1)/8
         I2=I1+7
         IF(I2.GT.NPIS) I2=NPIS
         WRITE (6,120) (J,J=I1,I2)
         WRITE (6,130) (CHORD(J),J=I1,I2)
   60    I1=I1+8
         WRITE (6,'(////15H MERGED BLOCKS:)')
         I1=1
         DO 70 I=1,1+(IJAR-1)/8
         I2=I1+7
         IF(I2.GT.IJAR) I2=IJAR
         WRITE (6,120) (J,J=I1,I2)
         WRITE (6,150) (MIX(J),J=I1,I2)
         WRITE (6,160) (DVX(J),J=I1,I2)
   70    I1=I1+8
         WRITE (6,'(////8H BLOCKS:)')
         I1=1
         DO 80 I=1,1+(IJAS-1)/8
         I2=I1+7
         IF(I2.GT.IJAS) I2=IJAS
         WRITE (6,170) (J,J=I1,I2)
         WRITE (6,180) (IFR(J),J=I1,I2)
         WRITE (6,190) (ALB(J),J=I1,I2)
   80    I1=I1+8
         WRITE (6,'(//)')
      ENDIF
      IF(IMPX.GE.2) THEN
         WRITE (6,'(////30H VALUES OF INUM, IGEN AND MAT:/7H (A MER,
     1   55HGED BLOCK IN JPM IS EQUIVALENT TO A REGION IN THE MAIN ,
     2   7HDRIVER))')
         I1=1
         DO 90 I=1,1+(NMBLK-1)/11
         I2=I1+10
         IF(I2.GT.NMBLK) I2=NMBLK
         WRITE (6,200) (J,J=I1,I2)
         WRITE (6,210) (INUM(J),J=I1,I2)
         WRITE (6,220) (IGEN(INUM(J)),J=I1,I2)
         WRITE (6,230) (MAT(INUM(J)),J=I1,I2)
   90    I1=I1+11
         WRITE (6,'(//)')
      ENDIF
      RETURN
*
  120 FORMAT (///13H OUT-CURRENT:,8(I7,6X,1HI))
  130 FORMAT (   13H CHORD      :,1P,8(E12.5,2H I))
  150 FORMAT (   13H MIX        :,8(I7,6X,1HI))
  160 FORMAT (   13H DVX        :,1P,8(E12.5,2H I))
  170 FORMAT (///13H IN-CURRENT :,8(I7,6X,1HI))
  180 FORMAT (   13H IFR        :,8(I7,6X,1HI))
  190 FORMAT (   13H ALB        :,1P,8(E12.5,2H I))
  200 FORMAT (///25H BLOCK NB.              :,11(I5,3X,1HI))
  210 FORMAT (   25H MERGED BLOCK (INUM)    :,11(I5,3X,1HI))
  220 FORMAT (   25H GENERATING BLOCK (IGEN):,11(I5,3X,1HI))
  230 FORMAT (   25H MIXTURE NB. (MAT)      :,11(I5,3X,1HI))
      END
