*DECK JPMGG
      SUBROUTINE JPMGG (MAXPTS,MAXJ,IPGEOM,IR,MAT,VOL,NGEN2,NG,NSMAX,
     1 MICRO,IQUAD,IAPP,IHALT,NS,IGI,RS,FRACT,ISURF2,CHORD2,IMPX,NMBLK,
     2 IFR,ALB,NMERGE,IJAT,INUM,MIX,DVX,NGEN,IGEN,ISURF,CHORD,ITEMP,IBI)
*
*-----------------------------------------------------------------------
*
* RECOVERY AND ANALYSIS OF THE GEOMETRY FOR THE DOUBLE HETEROGENEITY
* OPTION OF JPM.
*
* GENERAL PARAMETERS:
*   MAXPTS : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION NMBLK, NMERGE OR
*            NGEN.
*   MAXJ   : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION IJAT OR IJAS
*            WHERE IJAT IS THE TOTAL NUMBER OF DISTINCT OUT-CURRENTS
*            AND IJAS IS THE TOTAL NUMBER OF OUT-CURRENTS LEAVING ALL
*            THE BLOCKS (MERGED OR NOT).
*   IPGEOM : POINTER TO THE GEOMETRY (L_GEOM SIGNATURE).
*   IR     : NUMBER OF MIXTURES.
*   MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*            DIMENSION MAT(NMERGE)
*   VOL    : VOLUMES. DIMENSION VOL(NMERGE)
*   NGEN2  : NUMBER OF GENERATING BLOCKS IN THE MACRO GEOMETRY.
*   NG     : NUMBER OF DIFFERENT KIND OF MICRO STRUCTURES. A KIND OF
*            MICRO STRUCTURE IS CHARACTERIZED BY THE RADIUS OF ITS
*            MICRO BLOCKS. ALL THE MICRO BLOCKS OF THE SAME KIND
*            SHOULD OWN THE SAME NUCLEAR PROPERTIES IN A GIVEN
*            GENERATING BLOCK OF THE MACRO GEOMETRY.
*   NSMAX  : MAXIMUM NUMBER OF BLOCKS (TUBES OR SHELLS) IN EACH KIND
*            OF MICRO STRUCTURE.
*   MICRO  : TYPE OF MICRO BLOCKS: =3: CYLINDER ; =4: SPHERE.
*   IQUAD  : QUADRATURE PARAMETER FOR THE CP CALCULATION OF THE MICRO
*            BLOCKS.
*   IAPP   : TYPE OF ANGULAR FLUX EXPANSION BETWEEN THE MICRO BLOCKS.
*   IHALT  : STOP REQUESTED BY USER IF IHALT=1.
*   NS     : NUMBER OF BLOCKS IN EACH KIND OF MICRO STRUCTURE.
*            DIMENSION NS(NG)
*   IGI    : TYPE OF MIXTURE IN EACH GENERATING BLOCK OF THE MACRO
*            GEOMETRY. IF IGI(IKG) IS NEGATIVE, THE GENERATING BLOCK
*            CONTAIN A MICRO STRUCTURE. DIMENSION IGI(NGEN2)
*   RS     : RADIUS OF THE MICRO BLOCKS.
*            DIMENSION RS(NG*(1+NSMAX))
*   FRACT  : VOLUMIC FRACTIONS OF THE MICRO BLOCKS.
*            DIMENSION FRACT(NG*IR)
*   ISURF2 : NUMBER OF SURFACES SURROUNDING EACH GENERATING BLOCK OF
*            THE MACRO GEOMETRY. DIMENSION ISURF2(NGEN2)
*   CHORD2 : MEAN CHORD LENGTH ASSOCIATED WITH EACH SURFACE OF THE
*            MACRO GEOMETRY. DIMENSION CHORD2(NPIS2) WHERE
*            NPIS2=SUM(ISURF2(I)) IS THE TOTAL NUMBER OF SURFACES SUR-
*            ROUNDING ALL THE GENERATING BLOCKS OF THE MACRO GEOMETRY.
*   IMPX   : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
*
* PARAMETERS RELATED TO BLOCKS DEFINITION:
*   NMBLK  : TOTAL NUMBER OF BLOCKS IN THE DOMAIN.
*   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAS) WHERE
*            IJAS IS THE TOTAL NUMBER OF IN-CURRENTS SURROUNDING ALL
*            THE BLOCKS (MERGED OR NOT).
*   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
*            DIMENSION ALB(IJAS)
*
* PARAMETERS RELATED TO THE MERGED BLOCKS DEFINITION:
*   NMERGE : TOTAL NUMBER OF MERGED BLOCKS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*            MANY BLOCKS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*            OWN THE SAME GENERATING BLOCK. THIS ALLOWS SOME REDUCTION
*            IN CPU TIME AND MEMORY. NMERGE.LE.NMBLK
*   IJAT   : TOTAL NUMBER OF DISTINCT OUT-CURRENTS.
*   INUM   : INDEX-NUMBER OF THE MERGED BLOCK ASSOCIATED TO EACH BLOCK.
*            DIMENSION INUM(NMBLK)
*   MIX    : INDEX-NUMBER OF OUT-CURRENTS. DIMENSION MIX(IJAR) WHERE
*            IJAR IS THE TOTAL NUMBER OF OUT-CURRENTS LEAVING ALL THE
*            MERGED BLOCKS.
*   DVX    : WEIGHT ASSOCIATED WITH EACH OUT-CURRENT.
*            DIMENSION DVX(IJAR)
*
*            REMARK:
*            IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
*            GEOMETRICAL "A" MATRIX.
*
* PARAMETERS RELATED TO THE GENERATING BLOCKS DEFINITION:
*   NGEN   : TOTAL NUMBER OF GENERATING BLOCKS. A GENERATING BLOCK IS
*            DEFINED BY ITS MIXTURE AND DIMENSIONS IRRESPECTIVE OF
*            ITS POSITION IN THE DOMAIN. NGEN.LE.NMERGE
*   IGEN   : INDEX-NUMBER OF THE GENERATING BLOCK ASSOCIATED WITH EACH
*            MERGED BLOCK. DIMENSION IGEN(NMERGE)
*   ISURF  : NUMBER OF SURFACES ASSOCIATED WITH EACH GENERATING BLOCK.
*            DIMENSION ISURF(NGEN)
*   CHORD  : MEAN CHORD LENGTH ASSOCIATED WITH EACH SURFACE.
*            DIMENSION CHORD(NPIS) WHERE NPIS=SUM(ISURF(I)) IS THE
*            TOTAL NUMBER OF SURFACES SURROUNDING ALL THE GENERATING
*            BLOCKS.
*
* SCRATCH STORAGE.
*  ITEMP   : DIMENSION ITEMP(NSMAX*NG*IR)
*  IBI     : DIMENSION IBI(NMERGE)
*
*---------------------------------------- AUTHOR: A. HEBERT 20/02/87 ---
*
      DIMENSION MAT(*),VOL(*),NS(*),IGI(*),RS(1+NSMAX,*),FRACT(NG,*),
     1 ISURF2(*),CHORD2(*),IFR(*),ALB(*),INUM(*),MIX(*),DVX(*),IGEN(*),
     2 ISURF(*),CHORD(*),ITEMP(NSMAX,NG,*),IBI(*)
      COMMON BASE(1)
*
* BACKUP THE VALUES RELATIVE TO THE MACRO GEOMETRY.
      NGEN2=NGEN
      NMERG2=NMERGE
      NMBLK2=NMBLK
      IR2=IR
      DO 20 IKK=1,NMERG2
20    IBI(IKK)=MAT(IKK)
*
* READ THE INPUT DATA.
      CALL SETARA(BASE,NG*NSMAX,IVOLK)
      CALL SETARA(BASE,IR2,IMILIE)
      CALL SETARA(BASE,IR2,IIDIL)
      CALL SETARA(BASE,IR2,IFTEMP)
      CALL READBH(MAXPTS,IPGEOM,IR,IR2,NMERGE,NMERG2,MAT,VOL,NG,NSMAX,
     1 MICRO,NS,IBI,RS,FRACT,BASE(IVOLK),IMPX,BASE(IMILIE),BASE(IIDIL),
     2 ITEMP,BASE(IFTEMP))
      CALL RLSARA(BASE(IFTEMP))
      CALL RLSARA(BASE(IIDIL))
      CALL RLSARA(BASE(IMILIE))
      CALL RLSARA(BASE(IVOLK))
      IF (IMPX.GE.1) THEN
         IF (IAPP.EQ.1) THEN
            WRITE (6,'(44H DP-0 ANGULAR FLUX APPROXIMATION BETWEEN THE,
     1      14H MICRO BLOCKS./)')
         ELSE IF (IAPP.EQ.2) THEN
            WRITE (6,'(44H DP-1 ANGULAR FLUX APPROXIMATION BETWEEN THE,
     1      14H MICRO BLOCKS./)')
         ENDIF
         WRITE (6,'(47H QUADRATURE PARAMETER FOR THE C.P. CALCULATION ,
     1   21HOF THE MICRO BLOCKS =,I3/)') IQUAD
      ENDIF
      DO 30 IKK=1,NMERG2
      IND=IBI(IKK)
      IF (IND.GT.IR) THEN
         IGI(IGEN(IKK))=-IND
      ELSE
         IGI(IGEN(IKK))=IND
      ENDIF
30    CONTINUE
      NMBLK=NMBLK2+(NMERGE-NMERG2)
      IND2=0
      DO 40 I=1,NGEN2
      IF (IGI(I).LT.0) IND2=MAX(IND2,-IGI(I))
40    CONTINUE
*
* SET-UP THE NEW GENERATING BLOCKS.
      IJAW=0
      DO 70 IKG=1,NGEN2
      ISURF2(IKG)=ISURF(IKG)
70    IJAW=IJAW+ISURF2(IKG)
      DO 80 I=1,IJAW
80    CHORD2(I)=CHORD(I)
      NPIS=0
      DO 130 IKG=1,NGEN2
      IND=-IGI(IKG)
      IF (IND.GT.0) THEN
         DILF=1.0
         DO 90 J=1,NG
90       DILF=DILF-FRACT(J,IND)
         DO 100 I=1,ISURF2(IKG)
100      CHORD(NPIS+I)=CHORD(NPIS+I)*DILF
         J0=0
         DO 120 J=1,NG
         IF (FRACT(J,IND).GT.0.00001) THEN
            ISURF(IKG)=ISURF(IKG)+1
            J0=J0+1
            DO 110 I=IJAW,NPIS+ISURF2(IKG)+J0,-1
110         CHORD(I+1)=CHORD(I)
            ZZZ=4.0*DILF*RS(NS(J)+1,J)/(REAL(MICRO-1)*FRACT(J,IND))
            CHORD(NPIS+ISURF2(IKG)+J0)=ZZZ
            IJAW=IJAW+1
         ENDIF
120      CONTINUE
      ENDIF
130   NPIS=NPIS+ISURF(IKG)
      DO 150 IND=IR+1,IND2
      DO 150 J=1,NG
      IF (FRACT(J,IND).GT.0.00001) THEN
         W=0.0
         DO 140 K=1,NS(J)
         ITEMP(K,J,IND)=NGEN+K
         ISURF(NGEN+K)=2*IAPP
         ZZZ=RS(K+1,J)**(MICRO-1)
         FACT=4.0*(ZZZ-W)/REAL(MICRO-1)
         IF (IAPP.EQ.1) THEN
            IF (K.GT.1) CHORD(NPIS+2*K-2)=FACT/(RS(K,J)**(MICRO-2))
            CHORD(NPIS+2*K-1)=FACT/(RS(K+1,J)**(MICRO-2))
         ELSE IF (IAPP.EQ.2) THEN
            IF (K.GT.1) THEN
               CHORD(NPIS+4*K-5)=FACT/(RS(K,J)**(MICRO-2))
               CHORD(NPIS+4*K-4)=CHORD(NPIS+4*K-5)
            ENDIF
            CHORD(NPIS+4*K-3)=FACT/(RS(K+1,J)**(MICRO-2))
            IF (K.LT.NS(J)) THEN
               CHORD(NPIS+4*K-2)=CHORD(NPIS+4*K-3)
            ENDIF
         ENDIF
140      W=ZZZ
         IF (NS(J).EQ.1) THEN
            ISURF(NGEN+1)=1
         ELSE
            ISURF(NGEN+1)=IAPP
            ISURF(NGEN+NS(J))=IAPP+1
         ENDIF
         NGEN=NGEN+NS(J)
         NPIS=NPIS+2*IAPP*(NS(J)-1)+1
      ENDIF
150   CONTINUE
      DO 160 I=1,NMERGE-NMERG2
160   INUM(NMBLK2+I)=NMERG2+I
      WRITE (6,310) NMBLK,NMERGE,NGEN,MAXPTS
      IF (NMERGE.GT.MAXPTS) CALL XABORT('JPMGG: INSUFFICIENT MAXPTS.')
*
* SET-UP THE NEW INTERFACE CURRENTS.
*     PART 1.
      IJAS=0
      DO 170 IBLK=1,NMBLK2
170   IJAS=IJAS+ISURF2(IGEN(INUM(IBLK)))
      IJ=0
      DO 200 IBLK=1,NMBLK2
      IKG=IGEN(INUM(IBLK))
      IND=-IGI(IKG)
      IF (IND.GT.0) THEN
         J0=0
         DO 190 J=1,NG
         IF (FRACT(J,IND).GT.0.00001) THEN
            J0=J0+1
            DO 180 I=IJAS,IJ+ISURF2(IKG)+J0,-1
            IFR(I+1)=IFR(I)
180         ALB(I+1)=ALB(I)
            IJAS=IJAS+1
         ENDIF
190      CONTINUE
      ENDIF
200   IJ=IJ+ISURF(IKG)
*
*     PART 2.
      IJAR=0
      DO 210 IKK=1,NMERG2
210   IJAR=IJAR+ISURF2(IGEN(IKK))
      NM=NMERG2
      IJ=0
      DO 280 IKK=1,NMERG2
      IKG=IGEN(IKK)
      IND=-IGI(IKG)
      IF (IND.GT.0) THEN
         J0=0
         DO 270 J=1,NG
         IF (FRACT(J,IND).GT.0.00001) THEN
            J0=J0+1
            DO 220 I=IJAR,IJ+ISURF2(IKG)+J0,-1
            MIX(I+1)=MIX(I)
220         DVX(I+1)=DVX(I)
            IF (IAPP.EQ.1) THEN
               MIX(IJ+ISURF2(IKG)+J0)=IJAT+2*NS(J)
            ELSE IF (IAPP.EQ.2) THEN
               MIX(IJ+ISURF2(IKG)+J0)=IJAT+4*NS(J)-2
            ENDIF
            DVX(IJ+ISURF2(IKG)+J0)=1.0
            IJAR=IJAR+1
            IFRACT=0
            DO 230 IBLK=1,NMBLK2
            IF (INUM(IBLK).EQ.IKK) IFRACT=IFRACT+1
230         CONTINUE
            IK=0
            DO 235 IBLK=1,NMBLK2
            IF (INUM(IBLK).EQ.IKK) THEN
               IFR(IK+ISURF2(IKG)+J0)=MIX(IJ+ISURF2(IKG)+J0)-1
               ALB(IK+ISURF2(IKG)+J0)=1.0/REAL(IFRACT)
            ENDIF
235         IK=IK+ISURF(IGEN(INUM(IBLK)))
*
            DO 240 K=1,NS(J)
            NM=NM+1
240         IGEN(NM)=ITEMP(K,J,IND)
            DO 250 K=1,1+2*IAPP*(NS(J)-1)
            MIX(IJAR+K)=IJAT+K
            DVX(IJAR+K)=1.0
250         ALB(IJAS+K)=1.0
            IF (IAPP.EQ.1) THEN
               DO 260 K=1,2*NS(J)-2,2
               IFR(IJAS+K)=IJAT+K+1
260            IFR(IJAS+K+1)=IJAT+K
               IFR(IJAS+2*NS(J)-1)=IJAT+2*NS(J)
            ELSE IF (IAPP.EQ.2) THEN
               DO 265 K=1,4*NS(J)-4,4
               IFR(IJAS+K)=IJAT+K+2
               IFR(IJAS+K+1)=IJAT+K+3
               IFR(IJAS+K+2)=IJAT+K
265            IFR(IJAS+K+3)=IJAT+K+1
               IFR(IJAS+4*NS(J)-3)=IJAT+4*NS(J)-2
            ENDIF
            IJAR=IJAR+2*IAPP*(NS(J)-1)+1
            IJAS=IJAS+2*IAPP*(NS(J)-1)+1
            IJAT=IJAT+2*IAPP*(NS(J)-1)+2
         ENDIF
270      CONTINUE
      ENDIF
280   IJ=IJ+ISURF(IKG)
      WRITE (6,320) IJAT,IJAR,IJAS,MAXJ
      IF (IJAS.GT.MAXJ) CALL XABORT('JPMGG: INSUFFICIENT MAXJ.')
      IF (IHALT.EQ.1) CALL XABORT('JPMGG: STOP REQUESTED BY USER.')
      RETURN
*
310   FORMAT (/39H TOTAL NUMBER OF BLOCKS IN THE DOMAIN =,I5/
     1         32H TOTAL NUMBER OF MERGED BLOCKS =,I5/
     2         36H TOTAL NUMBER OF GENERATING BLOCKS =,I5,5X,
     3         26HAVAILABLE STORAGE MAXPTS =,I5/)
320   FORMAT (/40H TOTAL NUMBER OF DISTINCT OUT-CURRENTS =,I5/
     1 61H TOTAL NUMBER OF OUT-CURRENTS LEAVING ALL THE MERGED BLOCKS =,
     2 I5/57H TOTAL NUMBER OF IN-CURRENTS SURROUNDING ALL THE BLOCKS =,
     3 I5,5X,24HAVAILABLE STORAGE MAXJ =,I5/)
      END
