*DECK JPMGR1
      SUBROUTINE JPMGR1(NGEN2,NG,NSMAX,MICRO,IQUAD,NS,IGI,RS,FRACT,SIGT,
     1 MASK,SIG2,RRRR)
*
*-----------------------------------------------------------------------
*
* CALCULATION OF THE BLOCK COLLISION PROBABILITIES FOR THE DOUBLE
* HETEROGENEITY OPTION OF JPM. PART 1.
*
* INPUT PARAMETERS:
*   NGEN2  : NUMBER OF GENERATING BLOCKS IN THE MACRO GEOMETRY.
*   NG     : NUMBER OF DIFFERENT KIND OF MICRO STRUCTURES. A KIND
*            OF MICRO STRUCTURE IS CHARACTERIZED BY THE RADIUS OF
*            ITS MICRO BLOCKS. ALL THE MICRO BLOCKS OF THE SAME
*            KIND SHOULD OWN THE SAME NUCLEAR PROPERTIES IN A GIVEN
*            MACRO BLOCK.
*   NSMAX  : MAXIMUM NUMBER OF TUBES OR SHELLS IN A MICRO STRUCTURE.
*   MICRO  : TYPE OF MICRO BLOCKS: =3: CYLINDER ; =4: SPHERE.
*   IQUAD  : QUADRATURE PARAMETER FOR THE CP CALCULATION OF THE
*            MICRO BLOCKS.
*   NS     : NUMBER OF MICRO BLOCKS IN EACH KIND OF MICRO STRUCTURE.
*            DIMENSION NS(NG)
*   IGI    : TYPE OF MIXTURE IN EACH GENERATING BLOCK OF THE MACRO
*            GEOMETRY. DIMENSION IGI(NGEN2)
*   RS     : RADIUS OF THE MICRO BLOCKS. DIMENSION RS(NSMAX+1,NG)
*   FRACT  : VOLUMIC FRACTION OF THE MICRO BLOCKS.
*            DIMENSION FRACT(NG,IR2) WHERE IR2 IS THE NUMBER OF
*            ORDINARY AND COMPOSITE MIXTURES.
*   SIGT   : MACROSCOPIC CROSS SECTIONS. DIMENSION SIGT(NGEN2)
*   MASK   : COLLISION PROBABILITIES ARE COMPUTED ONLY IF MASK=.TRUE.
*            DIMENSION MASK(NGEN2)
*
* OUTPUT PARAMETERS.
*   SIG2   : MODIFIED MACROSCOPIC CROSS SECTIONS. DIMENSION SIG2(NGEN2)
*   RRRR   : VOLUMIC SELF-SHIELDED FACTOR FOR JPMG2.
*            DIMENSION RRRR(NGEN2)
*
*----------------------------------------------- AUTHOR: A. HEBERT -----
*
      PARAMETER (SMALL=1.0E-2)
      LOGICAL MASK(*)
      DOUBLE PRECISION DP0,DP1,FACT,CHORD
      DIMENSION NS(*),IGI(*),RS(NSMAX+1,*),FRACT(NG,*),SIGT(*),SIG2(*),
     1 RRRR(*),RGAR(2)
*
      DO 30 I=1,NGEN2
      RRRR(I)=1.0
      SIG2(I)=SIGT(I)
      MIL=-IGI(I)
      IF ((MIL.GT.0).AND.MASK(I)) THEN
         DILF=1.0
         DO 10 J=1,NG
   10    DILF=DILF-FRACT(J,MIL)
         DP0=DILF*SIGT(I)
         DP1=DP0
         DO 20 J=1,NG
         FRT=FRACT(J,MIL)
         IF (FRT.GT.0.00001) THEN
            RGAR(1)=0.0
            RGAR(2)=RS(NS(J)+1,J)
            IF (MICRO.EQ.3) THEN
               CALL TUBE1(IQUAD,RGAR,SIGT(I),SMALL,PII,PIS,PSS)
            ELSE IF (MICRO.EQ.4) THEN
               CALL SHELL1(IQUAD,RGAR,SIGT(I),SMALL,PII,PIS,PSS)
            ENDIF
            CHORD=4.0D0*RGAR(2)/REAL(MICRO-1)
            FACT=FRT/CHORD
            DP0=DP0+FACT
            DP1=DP1+FACT/PIS
         ENDIF
   20    CONTINUE
         SIG2(I)=DP1
         RRRR(I)=DP1/DP0
      ENDIF
   30 CONTINUE
      RETURN
      END
