*DECK LIBBAS
      SUBROUTINE LIBBAS(NISO,AT,AKT,AMT,T,IX,V,DV,NDTE,P,XS,X,E)
C
C-----------------------------------------------------------------------
C
C SCATTERING KERNEL BASED ON THE FREE GAS MODEL OF BROWN AND ST. JOHN.
C A. HEBERT 1990/01/01
C
C INPUT PARAMETERS:
C  NISO  : NUMBER OF TERMS IN THE MODEL.
C          NISO=1 : PURE FREE GAS MODEL;
C          NISO=2 : BROWN AND ST. JOHN MODEL.
C  AT    : POTENTIAL MICROSCOPIC CROSS SECTION. DIMENSION AT(NISO)
C  AKT   : EXPONENTIAL CONSTANT IN THE MODEL. EQUAL TO ZERO FOR THE
C          PURE FREE GAS MODEL. DIMENSION AKT(NISO)
C  AMT   : ISOTOPE MASS DIVIDED BY NEUTRON MASS. DIMENSION AMT(NISO)
C  T     : ABSOLUTE TEMPERATURE DIVIDED BY 293.6K.
C  IX    : NUMBER OF THERMAL GROUPS.
C  V     : NEUTRON VELOCITIES. DIMENSION V(IX)
C  DV    : USED TO TRANSFORM VELOCITY TO ENERGY. DIMENSION DV(IX)
C  NDTE  : FIRST DIMENSION OF MATRIX P.
*
C OUTPUT PARAMETERS:
C  P     : SCATTERING KERNEL. THE FIRST INDEX IS FOR SECONDARY NEUTRONS.
C          DIMENSION P(NDTE,IX)
C  XS    : SCATTERING MICROSCOPIC CROSS SECTION. DIMENSION XS(IX)
C
C SCRATCH STORAGE:
C  X     : DIMENSION X(IX)
C  E     : DIMENSION E(IX)
C
C REFERENCE:
C  H. C. HONECK, 'THE DISTRIBUTION OF THERMAL NEUTRONS IN SPACE AND
C  ENERGY IN REACTOR LATTICES. PART 1: THEORY', NUCL. SCI. ENG., 8,
C  193 (1960).
C
C-----------------------------------------------------------------------
C
      IMPLICIT NONE
C----
C LOCAL VARIABLES
C----
      INTEGER  NDTE,I,IX,N,NISO,J
      REAL     AM,BETS,T,BET,TAUS,TAU,ALPHA,THETA,ZETA,OMEG,CONST,
     1         CONST1,TEM,WA,WB,EA,EB,AFERF
      REAL     AT(*),AKT(*),AMT(*),V(*),DV(*),P(NDTE,*),XS(*),X(*),E(*)
C
      DO 15 I=1,IX
      XS(I)=0.0
      DO 15 J=1,IX
   15 P(I,J)=0.0
  100 DO 500 N=1,NISO
      IF(AT(N).EQ.0.0) GO TO 500
      AM=AMT(N)
      BETS=AM/T
      BET=SQRT(BETS)
      DO 120 I=1,IX
      X(I)=BET*V(I)
  120 E(I)=X(I)*X(I)
      TAUS=BETS/(BETS+AKT(N))
      TAU=SQRT(TAUS)
      ALPHA=AKT(N)*TAUS/BETS
      THETA=(AM+1.0)/(2.0*AM*TAU)
      ZETA=TAU-THETA
      OMEG=TAUS*(BETS+(AM+1.0)*AKT(N))/BETS
      CONST=(AT(N)*TAU*TAUS*(AM+1.0)*(AM+1.0))/(4.0*AM*OMEG)
  200 DO 400 I=1,IX
      DO 300 J=I,IX
      CONST1=CONST*X(I)/X(J)
      WA=ALPHA*E(J)
      TEM=0.0
      IF(WA.LT.50.0) THEN
        EA=AFERF((THETA*X(I))+(ZETA*X(J)))+
     >     AFERF((THETA*X(I))-(ZETA*X(J)))
        TEM=CONST1*EA*EXP(-WA)
      ENDIF
      WB=(OMEG*E(I)-E(J))/AM
      IF(WB.LT.50.0) THEN
        EB=AFERF((THETA*X(J))-(ZETA*X(I)))-
     >  AFERF((ZETA*X(I))+(THETA*X(J)))
        TEM=TEM+CONST1*EB*EXP(-WB)
      ENDIF
      IF(TEM.LE.1.E-15) GO TO 350
      P(I,J)=TEM
  300 CONTINUE
  350 EA=TAU*X(I)
      WA=ALPHA*E(I)
      IF(WA.GE.50.0) THEN 
        WA=0.0
      ELSE
        WA=EXP(-WA)
      ENDIF
      EB=WA*AFERF(EA)*(EA+(0.5/EA))
      IF(E(I).GE.50.0) THEN
        WB=0.0
      ELSE
        WB=EXP(-E(I))
      ENDIF
      XS(I)=XS(I)+(AT(N)*TAUS*TAUS/BET)*(EB+0.5641896*WB)
  400 CONTINUE
  500 CONTINUE
      DO 610 I=1,IX
      E(I)=0.0
      WA=(V(I)*V(I))/T
      IF(WA.LT.50.0) THEN
        E(I)=V(I)*V(I)*EXP(-WA)
      ENDIF
  610 CONTINUE
      DO 620 I=1,IX
      DO 620 J=I,IX
      IF(E(I).GT.1.E-20) THEN 
        P(J,I)=P(I,J)*E(J)/E(I)
      ENDIF
  620 CONTINUE
      DO 650 J=1,IX
      TEM=0.0
      DO 640 I=1,IX
  640 TEM=TEM+P(I,J)*DV(I)
  650 P(J,J)=((XS(J)-TEM)/DV(J))+P(J,J)
      DO 680 I=1,IX
      XS(I)=XS(I)/V(I)
      DO 680 J=1,IX
  680 P(I,J)=P(I,J)*DV(J)/V(I)
      RETURN
      END
