*DECK LIBCMB
      SUBROUTINE LIBCMB(MAXMIX,MAXISO,NBISO,NEWISO,NNMIX,MIXCMB,
     >                  VOLTOT,VOLFRA,DENMIX,ISONAM,ISONRF,ISOMIX,
     >                  IHLIB,DENISO,TMPISO,AWRISO,LSHI,SNISO,
     >                  NTFG,MASKI,NELSN,NGRCD,DILSN,DILSB)
C
C------------------------------  LIBCMB  ------------------------------
C
C  PROGRAMME STATISTICS:
C     NAME     : LIBCMB
C     ENTRY    : LIBCMB
C     USE      : COMBINE MIXTURES BY VOLUME FRACTION
C     MODIFIED : 97-02-04
C     AUTHOR   : G. MARLEAU
C
C  ROUTINE PARAMETERS:
C   INPUT/OUTPUT
C     MAXMIX  : MAXIMUM VALUE OF NBMIX               I
C     MAXISO  : MAXIMUM NB ISOTOPES PERMITTED        I
C     NBISO   : NB ISOTOPES BEFORE COMBINATION       I
C     NEWISO  : NB ISOTOPES AFTER COMBINATION        I
C     NNMIX   : NEW MIXTURE TO CREATE OR MODIFY      I
C     MIXCMB  : MIXTURE TO ADD                       I
C     VOLTOT  : TOTAL VOLUME FRACTION TO DATE        R
C     VOLFRA  : VOLUME FRACTION OF CURRENT MIXTURE   R
C     DENMIX  : DENSITY OF EACH MIXTURE.             R(MAXMIX)
C     ISONAM  : NAME OF ISOTOPES                     I(3,MAXISO)
C     ISONRF  : REFERENCE NAME OF ISOTOPES           I(3,MAXISO)
C     ISOMIX  : MIX NUMBER OF EACH ISOTOPE           I(MAXISO)
C     IHLIB   : ISOTOPE OPTIONS                      I(2,MAXISO,5)
C     DENISO  : DENSITY OF ISOTOPES                  R(MAXISO)
C     TMPISO  : TEMPERATURE OF ISOTOPES              R(MAXISO)
C     AWRISO  : ATOMIC WEIGHT OF ISOTOPES            R(MAXISO)
C     LSHI    : SELF-SHIELDING FLAG                  I(MAXISO)
C     SNISO   : DILUTION CROSS SECTION               R(MAXISO)
C     NTFG    : NB THERMAL INELASTIC GROUPS          I(MAXISO)
C     MASKI   : TREAT ISOTOPE LOGICAL                L(MAXISO)
C     NELSN   : TOTAL LENGTH OF DILSN,DILSB          I
C     NGRCD   : FIRST DIMENSION OF DILSN AND DILSB   I
C               VECTOR
C     DILSN   : STANDARD DILUTION                    R(NGRCD,MAXISO)
C     DILSB   : L-J DILUTION                         R(NGRCD,MAXISO)
C
C------------------------------  LIBCMB  ------------------------------
C
      IMPLICIT  NONE
C-----
C LOCAL VARIABLES
C-----
      INTEGER          MAXMIX,MAXISO,NBISO,NEWISO,NNMIX,NELSN,NGRCD,
     1                 ISO,IGRP,JSO,ITC,ILC
C
      INTEGER          MIXCMB,ISONAM(3,MAXISO),ISONRF(3,MAXISO),
     1                 ISOMIX(MAXISO),IHLIB(2,MAXISO,5),LSHI(MAXISO),
     2                 NTFG(MAXISO)
C
      REAL             VOLTOT,VOLFRA,CMBVOL,RMAS1,RMAS2,CMBMAS,TMPDEN
C
      REAL             DENMIX(MAXMIX),DENISO(MAXISO),TMPISO(MAXISO),
     1                 AWRISO(MAXISO),SNISO(MAXISO),DILSN(NGRCD,MAXISO),
     2                 DILSB(NGRCD,MAXISO)
      LOGICAL          MASKI(MAXISO)
C----
      CMBVOL=VOLTOT+VOLFRA
      RMAS1=1.0
      RMAS2=1.0
      IF(MIXCMB .EQ. 0) THEN
C----
C  MIXTURE TO ADD IS VOID
C----
        IF(DENMIX(NNMIX) .EQ. -1.0) THEN
C----
C  REDUCE ATOMIC DENSITY
C----
          RMAS1=VOLTOT/CMBVOL
        ELSE
C----
C  REDUCE MIXTURE DENSITY BUT NOT WEIGHT PERCENT
C----
          DENMIX(NNMIX)=DENMIX(NNMIX)*VOLTOT/CMBVOL
        ENDIF
      ELSE
C----
C  MIXTURE TO ADD IS NOT VOID
C----
        IF(DENMIX(NNMIX) .EQ. -1.0) THEN
          IF(DENMIX(MIXCMB) .EQ. -1.0) THEN
C----
C  REDUCE ATOMIC DENSITY
C----
            RMAS1=VOLTOT/CMBVOL
            RMAS2=VOLFRA/CMBVOL
          ELSE
            IF(VOLTOT .GT. 0.0)
     >        CALL XABORT('LIBCMB: CANT COMBINE MIXTURE WITH '//
     >                    ' WEIGHT PERCENT AND ATOM CONTENTS')
C----
C  TRANSFER MIXTURE DENSITY WITH INITIAL WEIGHT PERCENT TO NEWISO
C----
            DENMIX(NNMIX)=DENMIX(MIXCMB)
          ENDIF
        ELSE
          IF(DENMIX(MIXCMB) .EQ. -1.0)
     >      CALL XABORT('LIBCMB: CANT COMBINE MIXTURE WITH '//
     >                  ' WEIGHT PERCENT AND ATOM CONTENTS')
C----
C  REDUCE MIXTURE DENSITY AND WEIGHT PERCENT FOR OLD ISO
C  TRANSFER MIXTURE DENSITY WITH REDUCED WEIGHT PERCENT TO NEWISO
C----
          RMAS1=VOLTOT*DENMIX(NNMIX)
          RMAS2=VOLFRA*DENMIX(MIXCMB)
          CMBMAS=RMAS1+RMAS2
          RMAS1=RMAS1/CMBMAS
          RMAS2=RMAS2/CMBMAS
          DENMIX(NNMIX)=CMBMAS/CMBVOL
        ENDIF
      ENDIF
      NEWISO=NBISO
C----
C  RESET OLD DENSITIES
C----
      DO 100 ISO=1,NBISO
        IF(ISOMIX(ISO).EQ.NNMIX) THEN
          DENISO(ISO)=DENISO(ISO)*RMAS1
          IF(NELSN .GT. 0) THEN
            DO 101 IGRP=1,NGRCD
              DILSN(IGRP,ISO)=DILSN(IGRP,ISO)*DENISO(ISO)
              DILSB(IGRP,ISO)=DILSB(IGRP,ISO)*DENISO(ISO)
 101        CONTINUE
          ENDIF
        ENDIF
 100  CONTINUE
      DO 110 ISO=1,NBISO
        IF(ISOMIX(ISO).EQ.MIXCMB) THEN
C----
C  SCAN ISO IN NNMIX TO IDENTIFY IDENTICAL ISOTOPES
C----
          DO 111 JSO=1,NBISO
            IF(ISOMIX(JSO).EQ.NNMIX) THEN
              IF(ISONAM(1,JSO).EQ.ISONAM(1,ISO).AND.
     >           ISONAM(2,JSO).EQ.ISONAM(2,ISO).AND.
     >           ISONRF(1,JSO).EQ.ISONRF(1,ISO).AND.
     >           ISONRF(2,JSO).EQ.ISONRF(2,ISO).AND.
     >           TMPISO(JSO)  .EQ.TMPISO(ISO)  .AND.
     >           LSHI(JSO)    .EQ.LSHI(ISO)    .AND.
     >           SNISO(JSO)   .EQ.SNISO(ISO)        ) THEN
                TMPDEN=DENISO(ISO)*RMAS2
                DENISO(JSO)=DENISO(JSO)+TMPDEN
                IF(NELSN .GT. 0) THEN
                  IF(DENISO(JSO).GT.0.0) THEN
                    DO 112 IGRP=1,NGRCD
                      DILSN(IGRP,JSO)=(DILSN(IGRP,JSO)
     >                  +DILSN(IGRP,ISO)*TMPDEN)/DENISO(JSO)
                      DILSB(IGRP,JSO)=(DILSB(IGRP,JSO)
     >                  +DILSB(IGRP,ISO)*TMPDEN)/DENISO(JSO)
 112                CONTINUE
                  ELSE
                    DO 113 IGRP=1,NGRCD
                      DILSN(IGRP,JSO)=1.0E10
                      DILSB(IGRP,JSO)=1.0E10
 113                CONTINUE
                  ENDIF
                ENDIF
                GO TO 115
              ENDIF
            ENDIF
 111      CONTINUE
          NEWISO=NEWISO+1
          ISOMIX(NEWISO)=NNMIX
          DENISO(NEWISO)=DENISO(ISO)*RMAS2
          IF(NELSN .GT. 0) THEN
            DO 114 IGRP=1,NGRCD
              DILSN(IGRP,NEWISO)=DILSN(IGRP,ISO)
              DILSB(IGRP,NEWISO)=DILSB(IGRP,ISO)
 114        CONTINUE
          ENDIF
          TMPISO(NEWISO)=TMPISO(ISO)
          AWRISO(NEWISO)=AWRISO(ISO)
          NTFG(NEWISO)=NTFG(ISO)
          SNISO(NEWISO)=SNISO(ISO)
          LSHI(NEWISO)=LSHI(ISO)
          MASKI(NEWISO)=.TRUE.
          DO 120 ITC=1,2
            ISONAM(ITC,NEWISO)=ISONAM(ITC,ISO)
            ISONRF(ITC,NEWISO)=ISONRF(ITC,ISO)
 120      CONTINUE
          ISONRF(3,NEWISO)=ISONRF(3,ISO)
          DO 130 ILC=1,5
            DO 131 ITC=1,2
              IHLIB(ITC,NEWISO,ILC)=IHLIB(ITC,ISO,ILC)
 131        CONTINUE
 130      CONTINUE
        ENDIF
 115    CONTINUE
 110  CONTINUE
      NBISO=NEWISO
      VOLTOT=CMBVOL
      RETURN
      END
