*DECK LIBDRC
      SUBROUTINE LIBDRC(NGRO,LBIN,NL,NPROC,INDPRO,ITYPRO,TERPM,XSREC,
     >                  SCAT,BIN,XSREC2,SCAT2,BIN2)
C
C-----------------------------------------------------------------------
C
C INTERPOLATE IN TEMPERATURE ONE ISOTOPE IN DRAGLIB FORMAT
C
C INPUT PARAMETERS:
C  NGRO    : NUMBER OF ENERGY GROUPS.
C  LBIN    : NUMBER OF FINE GROUPS.
C  NL      : NUMBER OF LEGENDRE ORDERS REQUIRED IN THE CALCULATION.
C            NL=1 OR HIGHER.
C  NPROC   : NUMBER OF CROSS SECTION TYPE PROCESSED
C  INDPRO  : FLAG FOR CROSS SECTION TYPE TO PROCESS INDPRO(NPROC)
C  ITYPRO  : FLAG FOR CROSS SECTION TYPE PROCESSED ITYPRO(NPROC)
C  TERPM   : INTERPOLATION FACTOR DOUBLE PRECISION TERPM
C INPUT/OUTPUTPARAMETERS:
C  XSREC   : VECTOR XS RECORD XSREC(NGRO,NPROC)
C  SCAT    : SCATTERING MATRIX SCAT(NGRO,NGRO,NL)
C  BIN     : FINE GROUP CROSS SECTIONS BIN(LBIN,3)
C SCRATCH STORAGE:
C  XSREC2  : DIMENSION XSREC2(NGRO,NPROC)
C  SCAT2   : DIMENSION SCAT2(NGRO,NGRO,NL)
C  BIN2    : DIMENSION BIN2(LBIN,3)
C
C----------------------------------- AUTHOR:   A. HEBERT  29/01/90 ----
C----------------------------------- MODIFIED: G. MARLEAU 97/07/03 ----
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER       NDPROC
      PARAMETER    (NDPROC=20)
C----
C DIMENSIONS
C----
      INTEGER       NGRO,LBIN,NL,NPROC
C----
C LOCAL VARIABLES
C----
      INTEGER       IG1,IG2,ITBIN,IGBIN,IXSR,IL
      INTEGER       INDPRO(NPROC),ITYPRO(NPROC)
      REAL          XSREC(NGRO,NPROC),SCAT(NGRO,NGRO,NL),BIN(LBIN,3),
     1              XSREC2(NGRO,NPROC),SCAT2(NGRO,NGRO,NL),
     2              BIN2(LBIN,3)
C
      DOUBLE PRECISION TERPM
C----
C  INTERPOLATE STANDARD CROSS SECTIONS
C----
      DO 100 IXSR=1,NDPROC
        IF(INDPRO(IXSR).EQ.1.AND.ITYPRO(IXSR).EQ.1) THEN
          DO 110 IG1=1,NGRO
            XSREC(IG1,IXSR)=XSREC(IG1,IXSR)+TERPM*XSREC2(IG1,IXSR)
 110      CONTINUE
        ENDIF
 100  CONTINUE
*----
*  INTERPOLATE SCATTERING CROSS SECTIONS
*----
      DO 120 IL=1,NL
        IXSR=NDPROC+IL
        IF(INDPRO(IXSR).EQ.1.AND.ITYPRO(IXSR).EQ.1) THEN
          DO 130 IG1=1,NGRO
            XSREC(IG1,IXSR)=XSREC(IG1,IXSR)+TERPM*XSREC2(IG1,IXSR)
            DO 140 IG2=1,NGRO
               SCAT(IG2,IG1,IL)=SCAT(IG2,IG1,IL)+TERPM*
     >                          SCAT2(IG2,IG1,IL)
 140        CONTINUE
 130      CONTINUE
        ENDIF
 120  CONTINUE
      DO 150 ITBIN=1,3
        DO 160 IGBIN=1,LBIN
          BIN(IGBIN,ITBIN)=BIN(IGBIN,ITBIN)+TERPM*BIN2(IGBIN,ITBIN)
 160    CONTINUE
 150  CONTINUE
      RETURN
      END
