*DECK LIBEDR
      SUBROUTINE LIBEDR(MAXR  ,NAXPAR,NDEPL ,NFISS ,NSUPF ,
     >                  DCR   ,KPAR  ,BPAR  ,KFISS ,YIELD ,QF    ,
     >                  KPAX  ,BPAX  )
C
C-------------------------- LIBEDR   ----------------------------------
C
C  PROGRAMME STATISTICS:
C     NAME     : LIBEDR
C     ENTRY    : LIBEDR
C     USE      : CREATE DRAGON RECORDS FROM DRAGLIB DATA
C     MODIFIED : 97-01-08
C     AUTHOR   : G. MARLEAU
C
C  ROUTINE PARAMETERS:
C   INPUT
C     MAXR   : NUMBER OF REACTION TYPES             I
C     NAXPAR : MAX NUMBER OF REACTION SAVED         I
C     NDEPL  : NUMBER OF DEPLETING ISOTOPES         I
C     NFISS  : NUMBER OF FISSILE ISOTOPES           I
C     NSUPF  : NUMBER OF FISSILE PRODUCTS           I
C     DCR    : ISOTOPE DECAY RATE                   R(NDEPL)
C     KPAR   : REACTION TYPE                        I(NDEPL,NAXPAR)
C     BPAR   : REACTION YIELD                       R(NDEPL,NAXPAR)
C     KFISS  : FISSILE ISOTOPES                     I(NFISS)
C     YIELD  : FISSILE YIELD                        R(NFISS,NSUPF)
C     QF     : FISSION ENERGY                       R(NFISS)
C   OUTPUT
C     KPAX   : COMPLETE REACTION TYPE MATRIX        I(NDEPL+MAXR,NDEPL)
C     BPAX   : COMPLETE BRANCHING RATIO MATRIX      R(NDEPL+MAXR,NDEPL)
C
C--------------------------  LIBEDR  ----------------------------------
C
      IMPLICIT NONE
C----
C DIMENSIONS
C----
      INTEGER      MAXR,NDEPL,NFISS,NSUPF,NAXPAR
C----
C LOCAL VARIABLES
C----
      INTEGER      NSUPL,IDEPL,IRT,IREA,JDEPL,IFISS,JEL,IEL,KREAC,
     1             IDFI
C
      INTEGER      KPAR(NDEPL,NAXPAR),
     1             KFISS(NFISS),KPAX(NDEPL+MAXR,NDEPL)
C
      REAL         YELD,SUMFI
C
      REAL         DCR(NDEPL),BPAR(NDEPL,NAXPAR),YIELD(NFISS,NSUPF),
     1             QF(NFISS),BPAX(NDEPL+MAXR,NDEPL)
C
      CHARACTER    HMSG*131
C
      NSUPL=NDEPL-NSUPF
      DO 100 IDEPL=1,NDEPL
        DO 110 IRT=1,NAXPAR
          IF(KPAR(IDEPL,IRT).NE.0) THEN
            IREA=MOD(KPAR(IDEPL,IRT),10)
            JDEPL=KPAR(IDEPL,IRT)/10
            IF(IREA.EQ.0) THEN
C----
C  RADIOACTIVE DECAY
C----
              KPAX(IDEPL,JDEPL)=1
              BPAX(IDEPL,JDEPL)=BPAR(IDEPL,IRT)
            ELSE IF(IREA.EQ.1) THEN
C----
C  FISSION PRODUCT
C----
              DO 120 IFISS=1,NFISS
                JDEPL=KFISS(IFISS)
                YELD=YIELD(IFISS,IDEPL-NSUPL)
                IF(YELD.GT.0.0) THEN
                  KPAX(IDEPL,JDEPL)=2
                  BPAX(IDEPL,JDEPL)=YELD
                ENDIF
 120          CONTINUE
            ELSE IF(IREA.EQ.2) THEN
C----
C  RADIOACTIVE CAPTURE
C----
              KPAX(IDEPL,JDEPL)=3
              BPAX(IDEPL,JDEPL)=BPAR(IDEPL,IRT)
            ELSE IF(IREA.EQ.3) THEN
C----
C  N2N REACTION
C----
              KPAX(IDEPL,JDEPL)=4
              BPAX(IDEPL,JDEPL)=BPAR(IDEPL,IRT)
              KPAX(NDEPL+2,JDEPL)=4
              BPAX(NDEPL+2,JDEPL)=1.0
            ELSE IF(IREA.EQ.4) THEN
C----
C  N3N REACTION
C----
              KPAX(IDEPL,JDEPL)=5
              BPAX(IDEPL,JDEPL)=BPAR(IDEPL,IRT)
              KPAX(NDEPL+3,JDEPL)=5
              BPAX(NDEPL+3,JDEPL)=1.0
            ELSE
C----
C  INVALID REACTION
C----
              WRITE(HMSG,9000) KPAR(IDEPL,IRT),IDEPL,IRT
            ENDIF
          ENDIF
 110    CONTINUE
        IF(DCR(IDEPL).GT.0.0) THEN
          KPAX(NDEPL+4,IDEPL)=1
          BPAX(NDEPL+4,IDEPL)=DCR(IDEPL)*1.0E8
        ENDIF
        KPAX(NDEPL+1,IDEPL)=3
        BPAX(NDEPL+1,IDEPL)=1.0
 100  CONTINUE
      DO 130 IFISS=1,NFISS
        IDEPL=KFISS(IFISS)
        KPAX(NDEPL+5,IDEPL)=6
        BPAX(NDEPL+5,IDEPL)=QF(IFISS)
 130  CONTINUE
C----
C  DEPLETION CHAIN COMPLETED
C  CHECK IF IT IS COHERENT
C  FIND FISSILE ISOTOPE AND FISSION PRODUCT
C  RENUMBER FISSILE ISOTOPE AND FISSION PRODUCT
C  DELETE ISOTOPES WITH NO REACTION
C----
      DO 200 IEL=1,NDEPL
        DO 210 JEL=1,NDEPL
          KREAC=KPAX(JEL,IEL)
          IF(KREAC.EQ.1) THEN
            IF(KPAX(NDEPL+4,IEL).NE.KREAC)
     >        CALL XABORT('LIBEDR: PARENT DOES NOT DEPLETE '//
     >                    'VIA REACTION : DECAY')
          ELSE IF(KREAC.EQ.2) THEN
            IF(KPAX(NDEPL+5,JEL).EQ.0) THEN
              KPAX(NDEPL+5,JEL)=2
            ELSE IF(KPAX(NDEPL+5,JEL).NE.2) THEN
              CALL XABORT('LIBEDR: SON NOT A FISSION PRODUCT')
            ENDIF
            IF(KPAX(NDEPL+5,IEL).NE.6)
     >        CALL XABORT('LIBEDR: PARENT NOT FISSILE')
          ELSE IF(KREAC.EQ.3) THEN
            IF(KPAX(NDEPL+1,IEL).NE.KREAC)
     >        CALL XABORT('LIBEDR: PARENT DOES NOT DEPLETE '//
     >                    'VIA REACTION : CAPTURE')
          ELSE IF(KREAC.EQ.4) THEN
            IF(KPAX(NDEPL+2,IEL).NE.KREAC)
     >        CALL XABORT('LIBEDR: PARENT DOES NOT DEPLETE '//
     >                    'VIA REACTION : N2N')
          ELSE IF(KREAC.EQ.5) THEN
            IF(KPAX(NDEPL+3,IEL).NE.KREAC)
     >        CALL XABORT('LIBEDR: PARENT DOES NOT DEPLETE '//
     >                    'VIA REACTION : N3N')
          ELSE IF(KREAC.EQ.6) THEN
            IF(KPAX(NDEPL+5,IEL).NE.KREAC)
     >        CALL XABORT('LIBEDR: PARENT DOES NOT DEPLETE '//
     >                    'VIA REACTION : FISSION')
          ENDIF
 210    CONTINUE
 200  CONTINUE
C----
C  FIND FISSILE ISOTOPE NON ZERO YIELD
C----
      IDFI=0
      DO 220 IEL=1,NDEPL
        IF(KPAX(NDEPL+5,IEL).EQ.6) THEN
          SUMFI=0.0
          DO 221 JEL=1,NDEPL
            IF(KPAX(JEL,IEL).EQ.2) THEN
              SUMFI=SUMFI+BPAX(JEL,IEL)
            ENDIF
 221      CONTINUE
          IF(SUMFI.GT.0.0) THEN
            IDFI=IDFI+10
            KPAX(NDEPL+5,IEL)=KPAX(NDEPL+5,IEL)+IDFI
          ENDIF
        ENDIF
 220  CONTINUE
C----
C  RETURN
C----
      RETURN
C----
C  ABORT FORMAT
C----
 9000 FORMAT('LIBEDR: INVALID REACTION = ',I10,
     >       ' FOR ISOTOPE = ',I10,' REACTION NUMBER = ',I5)
      END
