*DECK LIBEPR
      SUBROUTINE LIBEPR(IMPX,MAXR,NDEPL,NDFI,NDFP,NBXR,HNADPL,IDR,RER,
     >                  KPAR,BPAR,YIELD)
C
C-------------------------- LIBEPR   ----------------------------------
C
C  PROGRAMME STATISTICS:
C     NAME     : LIBEPR
C     ENTRY    : LIBEPR
C     USE      : PRINT ISOTOPIC DEPLETION CHAIN
C     MODIFIED : 97-01-08
C     AUTHOR   : G. MARLEAU
C
C  ROUTINE PARAMETERS:
C   INPUT
C     IMPX   : PRINTING LEVEL                       I
C     MAXR   : NUMBER OF REACTION TYPES             I
C     NDEPL  : NUMBER OF DEPLETING ISOTOPES         I
C     NDFI   : NUMBER OF DIRECT FISSILE ISOTOPES    I
C     NDFP   : NUMBER OF DIRECT FISSION PRODUCT     I
C     NBXR   : MAXIMIM NUMBER OF PARENT ISOPOPES    I
C              FROM DECAY AND CAPTURE
C     HNADPL : REACTIVE ISOTOPE NAMES IN  CHAIN     I(2,NDEPL)
C              -> READ(NAME,'(2A4)')
C                 HNAME(1,I),HNAME(2,I)
C     IDR    : REACTION IDENTIFIER                  I(MAXR,NDEPL)
C     RER    : REACTION CONSTANT                    R(MAXR,NDEPL)
C     KPAR   : REDUCED REACTION TYPE MATRIX         I(NBXR,NDEPL)
C     BPAR   : REDUCED BRANCHING RATIO MATRIX       R(NBXR,NDEPL)
C     YIELD  : FISSION PRODUCT YIELD                R(NDFI,NDFP)
C   INTERNAL PARAMETERS
C     IOUT   : OUTPUT UNIT = 6                      I
C     KCAPTU : DRAGON CAPTURE FLAG = 3              I
C     KDECAY : DRAGON DECAY FLAG = 1                I
C     KFISSP : DRAGON FISSION PRODUCT FLAG = 2      I
C     KFISSI : DRAGON FISSILE ISOTOPE FLAG = 6      I
C     KN2N   : DRAGON N2N FLAG = 4                  I
C     KN3N   : DRAGON N3N FLAG = 5                  I
C
C--------------------------  LIBEPR  ----------------------------------
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER      IMPX
      INTEGER      IOUT,KCAPTU,KDECAY,KFISSP,KFISSI,KN2N,KN3N
      REAL         CNVDAY
      PARAMETER   (IOUT=6,KCAPTU=3,KDECAY=1,KFISSP=2,KFISSI=6,KN2N=4,
     1             KN3N=5,CNVDAY=1.0E+8/86400.0)
C----
C LOCAL VARIABLES
C----
      INTEGER      MAXR,NDEPL,NDFI,NDFP,NBXR,ISO,KRC,KRN2N,KRN3N,KRD,
     1             KRF,IPF,ISOF,NBPROD,IPAR,JSO,KREAC,ISOFP,KREF
      INTEGER      HNADPL(2,NDEPL),IDR(MAXR,NDEPL),KPAR(NBXR,NDEPL)
C
      REAL         RATF,RATFS,RATD,RREAC
      REAL         RER(MAXR,NDEPL),BPAR(NBXR,NDEPL),YIELD(NDFI,NDFP)
      CHARACTER    NAMREA(0:6)*8,NONOFF(2)*3,LINE*72,LINE2*72
      CHARACTER    NAMISC*10,NAMISP*10
      INTEGER      IC,KRTOT
C----
C DATA
C----
      DATA         NAMREA/'        ','   DECAY',' FISSION',
     1                    '      NG','     N2N','     N3N',
     2                    ' FISSION'/
      DATA         NONOFF/' NO','YES'/
C
      SAVE         NAMREA,NONOFF
C----
C  PRINT THE DEPLETION CHAIN.
C---- 
      IF(IMPX .GE. 20) THEN 
      WRITE(IOUT,6000)
      DO 110 ISO=1,NDEPL
        KRC=1
        IF(MOD(IDR(1,ISO),10).EQ.KCAPTU) KRC=2
        KRN2N=1
        IF(MOD(IDR(2,ISO),10).EQ.KN2N) KRN2N=2
        KRN3N=1
        IF(MOD(IDR(3,ISO),10).EQ.KN3N) KRN3N=2
        KRD=1
        RATD=0.0
        IF(MOD(IDR(4,ISO),10).EQ.KDECAY) THEN
          RATD=CNVDAY/RER(4,ISO)
          KRD=2
        ENDIF
        KRF=1
        RATF=0.0
        RATFS=0.0
        IF(MOD(IDR(5,ISO),10).EQ.KFISSI) THEN
          KRF=2
          RATF=RER(5,ISO)
          ISOF=IDR(5,ISO)/10
          IF(ISOF.NE.0) THEN
            DO 111 IPF=1,NDFP
              RATFS=RATFS+YIELD(ISOF,IPF)
 111        CONTINUE
          ENDIF
        ENDIF
        LINE=' '
C----
C  WRITE ISOTOPE PROPERTIES
C----
        IF(KRF.EQ.2) THEN
          IF(KRD.EQ.2) THEN
            WRITE(LINE,6010) ISO,HNADPL(1,ISO),HNADPL(2,ISO),
     >        NONOFF(KRC),NONOFF(KRN2N),NONOFF(KRN3N),
     >        NONOFF(KRD),RATD,NONOFF(KRF),RATF,RATFS
          ELSE
            WRITE(LINE,6011) ISO,HNADPL(1,ISO),HNADPL(2,ISO),
     >        NONOFF(KRC),NONOFF(KRN2N),NONOFF(KRN3N),
     >        NONOFF(KRD),NONOFF(KRF),RATF,RATFS
          ENDIF
        ELSE
          IF(KRD.EQ.2) THEN
            WRITE(LINE,6010) ISO,HNADPL(1,ISO),HNADPL(2,ISO),
     >        NONOFF(KRC),NONOFF(KRN2N),NONOFF(KRN3N),
     >        NONOFF(KRD),RATD,NONOFF(KRF)
          ELSE
            WRITE(LINE,6011) ISO,HNADPL(1,ISO),HNADPL(2,ISO),
     >        NONOFF(KRC),NONOFF(KRN2N),NONOFF(KRN3N),
     >        NONOFF(KRD),NONOFF(KRF)
          ENDIF
        ENDIF
C----
C  WRITE PARENTS FROM ALL REACTION EXCEPT FISSION
C----
        NBPROD=0
        DO 120 IPAR=1,NBXR
          JSO=KPAR(IPAR,ISO)/10
          KREAC=MOD(KPAR(IPAR,ISO),10)
          RREAC=BPAR(IPAR,ISO)
          IF(KREAC.NE.KFISSP) THEN
            IF(JSO.GT.0) THEN
              NBPROD=NBPROD+1
              IF(NBPROD.GT.1) THEN
                WRITE(IOUT,6012) NAMREA(KREAC),
     >            JSO,HNADPL(1,JSO),HNADPL(2,JSO),RREAC
              ELSE
                WRITE(IOUT,6013) LINE,NAMREA(KREAC),
     >            JSO,HNADPL(1,JSO),HNADPL(2,JSO),RREAC
              ENDIF
            ENDIF
          ENDIF
 120    CONTINUE
C----
C  WRITE PARENTS FROM FISSION IF REQUIRED
C----
        KREAC=MOD(IDR(5,ISO),10)
        ISOFP=IDR(5,ISO)/10
        IF(KREAC.EQ.KFISSP.AND.ISOFP.GT.0) THEN
          IF(ISOFP.GT.NDFP)
     >      CALL XABORT('LIBEPR: INVALID FISSPROD NUMBER')
          DO 130 JSO=1,NDEPL
            KREF=MOD(IDR(5,JSO),10)
            ISOF=IDR(5,JSO)/10
            IF(KREF.EQ.KFISSI.AND.ISOF.NE.0) THEN
              IF(ISOF.GT.NDFI)
     >          CALL XABORT('LIBEPR: INVALID FISSILE NUMBER')
              RREAC=YIELD(ISOF,ISOFP)
              IF(RREAC.GT.0) THEN
                NBPROD=NBPROD+1
                IF(NBPROD.GT.1) THEN
                  WRITE(IOUT,6012) NAMREA(KREAC),
     >              JSO,HNADPL(1,JSO),HNADPL(2,JSO),RREAC
                ELSE
                  WRITE(IOUT,6013) LINE,NAMREA(KREAC),
     >              JSO,HNADPL(1,JSO),HNADPL(2,JSO),RREAC
                ENDIF
              ENDIF
            ENDIF
 130      CONTINUE
        ENDIF
        IF(NBPROD.EQ.0) THEN
          WRITE(IOUT,6014) LINE
        ENDIF
 110  CONTINUE
      ENDIF
*----
*  CHAIN IN DRAGON INPUT FORMAT
*----
      IF(IMPX .GE. 30) THEN 
      WRITE(IOUT,7000) NDEPL
      DO 200 ISO=1,NDEPL
        KRC=1
        IF(MOD(IDR(1,ISO),10).EQ.KCAPTU) KRC=2
        KRN2N=1
        IF(MOD(IDR(2,ISO),10).EQ.KN2N) KRN2N=2
        KRN3N=1
        IF(MOD(IDR(3,ISO),10).EQ.KN3N) KRN3N=2
        KRD=1
        RATD=0.0
        IF(MOD(IDR(4,ISO),10).EQ.KDECAY) THEN
          RATD=RER(4,ISO)
          KRD=2
        ENDIF
        KRF=1
        RATF=0.0
        RATFS=0.0
        IF(MOD(IDR(5,ISO),10).EQ.KFISSI) THEN
          KRF=2
          RATF=RER(5,ISO)
          ISOF=IDR(5,ISO)/10
        ENDIF
        WRITE(NAMISC,7100) HNADPL(1,ISO),HNADPL(2,ISO)
        DO 210 IC=9,2,-1
          IF(NAMISC(IC:IC) .NE. ' ') GO TO 215
          NAMISC(IC:IC)=NAMISC(IC+1:IC+1)
          NAMISC(IC+1:IC+1)=' '
 210    CONTINUE
 215    CONTINUE
        LINE=' '
        LINE2=' '
        WRITE(LINE,7010) NAMISC,ISO
        KRTOT=KRC+KRN2N+KRN3N+KRD+KRF
        IF(KRTOT .EQ. 5) THEN 
          WRITE(LINE(21:30),7011) '  STABLE'
        ELSE
          IF(KRD .EQ. 2)   WRITE(LINE2(11:35),7012) NAMREA(1),RATD
          IF(KRF .EQ. 2)   WRITE(LINE2(40:65),7012) NAMREA(2),RATF
          IF(KRC .EQ. 2)   WRITE(LINE(21:30),7011) NAMREA(3)
          IF(KRN2N .EQ. 2) WRITE(LINE(31:40),7011) NAMREA(4)
          IF(KRN3N .EQ. 2) WRITE(LINE(41:50),7011) NAMREA(5)
        ENDIF
        WRITE(IOUT,'(A72)') LINE
        IF(KRD+KRF .GT. 2) THEN
          WRITE(IOUT,'(A72)') LINE2
        ENDIF          
C----
C  WRITE PARENTS FROM ALL REACTION EXCEPT FISSION
C----
        NBPROD=0
        DO 220 IPAR=1,NBXR
          JSO=KPAR(IPAR,ISO)/10
          KREAC=MOD(KPAR(IPAR,ISO),10)
          RREAC=BPAR(IPAR,ISO)
          IF(KREAC.NE.KFISSP) THEN
            IF(JSO.GT.0) THEN
              NBPROD=NBPROD+1
              WRITE(NAMISP,7100) HNADPL(1,JSO),HNADPL(2,JSO)
              DO 221 IC=9,2,-1
                IF(NAMISP(IC:IC) .NE. ' ') GO TO 225
                NAMISP(IC:IC)=NAMISP(IC+1:IC+1)
                NAMISP(IC+1:IC+1)=' '
 221          CONTINUE
 225          CONTINUE
              IF(NBPROD.EQ.1) WRITE(IOUT,7020)
              WRITE(IOUT,7021) NAMREA(KREAC),RREAC,NAMISP
            ENDIF
          ENDIF
 220    CONTINUE
C----
C  WRITE PARENTS FROM FISSION IF REQUIRED
C----
        KREAC=MOD(IDR(5,ISO),10)
        ISOFP=IDR(5,ISO)/10
        IF(KREAC.EQ.KFISSP.AND.ISOFP.GT.0) THEN
          IF(ISOFP.GT.NDFP)
     >      CALL XABORT('LIBEPR: INVALID FISSPROD NUMBER')
          DO 230 JSO=1,NDEPL
            KREF=MOD(IDR(5,JSO),10)
            ISOF=IDR(5,JSO)/10
            IF(KREF.EQ.KFISSI.AND.ISOF.NE.0) THEN
              IF(ISOF.GT.NDFI)
     >          CALL XABORT('LIBEPR: INVALID FISSILE NUMBER')
              RREAC=YIELD(ISOF,ISOFP)
              IF(RREAC.GT.0) THEN
                WRITE(NAMISP,7100) HNADPL(1,JSO),HNADPL(2,JSO)
                DO 231 IC=9,2,-1
                  IF(NAMISP(IC:IC) .NE. ' ') GO TO 235
                  NAMISP(IC:IC)=NAMISP(IC+1:IC+1)
                  NAMISP(IC+1:IC+1)=' '
 231            CONTINUE
 235            CONTINUE
                NBPROD=NBPROD+1
                IF(NBPROD.EQ.1) WRITE(IOUT,7020)
                WRITE(IOUT,7021) NAMREA(KREAC),RREAC,NAMISP
              ENDIF
            ENDIF
 230      CONTINUE
        ENDIF
 200  CONTINUE
      WRITE(IOUT,7001) 
      ENDIF
C----
C  RETURN FROM LIBEPR
C----
      RETURN
C----
C  FORMAT
C----
 6000 FORMAT(' DEPLETION CHAIN DATA'/' --------------------'//
     >       '                                   DEPLETION ',
     >       'REACTIONS                                ',
     >       'PRODUCTION REACTIONS'/
     >       '                -----------------------------',
     >       '----------------------------   ----------',
     >       '-----------------------------'/
     >       '   ISOTOPE      NG  N2N N3N DECAY            ',
     >       ' FISSION                       REACTION  ',
     >       '   ISOTOPE              YIELD'/
     >       '   NB. NAME                     MEANLIFE(DAYS)',
     >       '    ENERGY(MEV) TOTAL YIELD              ',
     >       '  NB. NAME    ')
 6010 FORMAT(1P,I5,1X,2A4,1X,4(A3,1X),E13.6,1X,A3,1X,E11.4,1X,E11.4)
 6011 FORMAT(1P,I5,1X,2A4,1X,4(A3,1X),14X,A3,1X,E11.4,1X,E11.4)
 6012 FORMAT(76X,A8,3X,I5,1X,2A4,1X,1P,E13.6)
 6013 FORMAT(1X,A72,3X,A8,3X,I5,1X,2A4,1X,1P,E13.6)
 6014 FORMAT(1X,A72)
 7000 FORMAT('DEPL',2X,I10/'CHAIN ')
 7001 FORMAT('ENDCHAIN ')
 7010 FORMAT(A10,2X,I8)
 7011 FORMAT(2X,A8)
 7012 FORMAT(2X,A8,2X,1P,E13.6)
 7020 FORMAT(10X,'FROM')
 7021 FORMAT(15X,A8,5X,1P,E13.6,5X,A10)
 7100 FORMAT(1H',2A4,1H')
 
      END
