*DECK LIBEWR
      SUBROUTINE LIBEWR(CFILNA,MAXR,NEL,ITNAM,KPAX,BPAX)
C
C-------------------------- LIBEWR   ----------------------------------
C
C  PROGRAMME STATISTICS:
C     NAME     : LIBEWR
C     ENTRY    : LIBEWR
C     USE      : READ DEPLETION DATA ON WIMS-AECL FORMAT LIBRARY
C     MODIFIED : 97-01-08
C     AUTHOR   : G. MARLEAU
C
C  ROUTINE PARAMETERS:
C   INPUT
C     CFILNA : WIMS FILE NAME                       C*8
C     MAXR   : NUMBER OF REACTION TYPES             I
C     NEL    : NUMBER OF ISOTOPES ON LIBRARY        I
C   OUTPUT
C     ITNAM  : REACTIVE ISOTOPE NAMES IN  CHAIN     I(2,NEL)
C              -> READ(NAME,'(2A4)')
C                 HNAME(1,I),HNAME(2,I)
C     KPAX   : COMPLETE REACTION TYPE MATRIX        I(NEL+MAXR,NEL)
C     BPAX   : COMPLETE BRANCHING RATIO MATRIX      R(NEL+MAXR,NEL)
C   WORK
C   INTERNAL PARAMETERS
C     KCAPTU : DRAGON CAPTURE FLAG = 3              I
C     KDECAY : DRAGON DECAY FLAG = 1                I
C     KFISSP : DRAGON FISSION PRODUCT FLAG = 2      I
C     KFISSI : DRAGON FISSILE ISOTOPE FLAG = 6      I
C     KN2N   : DRAGON N2N FLAG = 4                  I
C     KN3N   : DRAGON N3N FLAG = 5                  I
C     CONVE  : ENERGY CONVERSION FACTOR             R
C              FROM JOULES/(MOLES*10**-24) TO
C              MEV/NUCLIDE = 1.03643526E+13
C     CONVD  : DECAY CONSTANT CONVERSION FACTOR     R
C              FROM S**(-1)) TO 10**(-8)*S**(-1))
C              = 1.0+8
C   WIMS-AECL LIBRARY PARAMETERS
C     IUTYPE : TYPE OF FILE = 4 (DA)                I
C     LRIND  : LENGHT RECORD ON DA FILE = 256       I
C     IACTO  : OPEN ACTION = 2 (READ ONLY)          I
C     IACTC  : CLOSE ACTION = 2 (KEEP)              I
C     MAXISO : MAX. NB. OF ISO = 246                I
C     MLDEP  : MAXIMUM NUMBER OF REACTION PER       I
C              ISOTOPE IN WIMS-AECL = MAXISO+4
C     LPZ    : LENGTH OF WIMS PARAMETER ARRAY = 9   I
C     LMASTB : LENGTH OF MST TAB = MAXISO+9         I
C     LMASIN : LENGTH OF MST IDX = LMASTB-4         I
C     LGENTB : LENGTH OF GEN TAB = 6                I
C     LGENIN : LENGTH OF GEN IDX = LGENTB           I
C     LSUBTB : LENGTH OF SUB TAB = 6*MAXTEM+21-5+12 I
C     LSUBIN : LENGTH OF SUB IDX = LSUBTB-12        I
C     ICAPTU : WIMS-AECL CAPTURE FLAG = 1           I
C     IDECAY : WIMS-AECL DECAY FLAG = 2             I
C     IFISSP : WIMS-AECL FISSION PRODUCT FLAG = 3   I
C     IFISSI : WIMS-AECL FISSILE ISOTOPE FLAG = 4   I
C     IN2N   : WIMS-AECL N2N FLAG = 5               I
C     IN3N   : WIMS-AECL N3N FLAG = 6               I
C     MASTER : MASTER INDEX ARRAY                   I(LMASTB)
C     GENINX : GENERAL INDEX ARRAY                  I(LGENTB)
C     SUBINX : SUB INDEX ARRAY                      I(LSUBTB)
C     NPZ    : LIST OF MAIN PARAMETERS              I(LPZ)
C     IWISO  : ID OF ISOTOPE                        I(2*MAXISO)
C     IBURN  : INTEGER BURNUP PARAMETERS            I(2,MLDEP)
C     RBURN  : REAL BURNUP PARAMETERS               R(2,MLDEP)
C
C--------------------------  LIBEWR  ----------------------------------
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER      KCAPTU,KDECAY,KFISSP,KFISSI,KN2N,KN3N
      REAL         CONVE,CONVD
      PARAMETER   (KCAPTU=3,KDECAY=1,KFISSP=2,KFISSI=6,KN2N=4,
     1             KN3N=5,CONVE=1.03643526E+13,CONVD=1.0E+8)
C----
C  WIMS-AECL LIBRARY PARAMETERS
C----
      INTEGER      IUTYPE,LRIND,IACTO,IACTC,MAXISO,MLDEP,LPZ,
     1             MAXTEM,LMASTB,LMASIN,LGENTB,LGENIN,LSUBTB,
     2             LSUBIN,ICAPTU,IDECAY,IFISSP,IFISSI,IN2N,IN3N
      PARAMETER   (IUTYPE=4,LRIND=256,IACTO=2,IACTC=1,MAXISO=246,
     1             MLDEP=MAXISO+4,LPZ=9,MAXTEM=20,LMASTB=MAXISO+9,
     2             LMASIN=LMASTB-4,LGENTB=6,LGENIN=LGENTB,
     3             LSUBTB=6*MAXTEM+28,LSUBIN=LSUBTB-12,ICAPTU=1,
     4             IDECAY=2,IFISSP=3,IFISSI=4,IN2N=5,IN3N=6)
      INTEGER      MASTER(LMASTB),GENINX(LGENTB),SUBINX(LSUBTB),
     1             NPZ(LPZ),IWISO(2*MAXISO),IBURN(2,MLDEP)
C----
C INTERFACE VARIABLES
C----
      INTEGER      MAXR,NEL
      INTEGER      ITNAM(2,NEL),KPAX(NEL+MAXR,NEL)
      REAL         BPAX(NEL+MAXR,NEL)
      CHARACTER    CFILNA*8
C----
C EXTERNAL FUNCTIONS
C----
      INTEGER      KDROPN,LIBWID
C----
C LOCAL VARIABLES
C----
      INTEGER      IUNIT,IEL2,IEL,ISO,NBURN,NMIN,IHANDL,NFP,JBRN,
     1             KTYPE,JSO
      REAL         RBURN(2,MLDEP)
C
      CHARACTER    TEXT8*8
C----
C  OPEN WIMS-AECL LIBRARY
C  READ INDEX AND GENERAL DIMENSIONING NPZ
C  READ ISOTOPE NAME AND ID NUMBER
C----
      IUNIT=KDROPN(CFILNA,IACTO,IUTYPE,LRIND,IHANDL)
      IF(IUNIT.LE.0) CALL XABORT('LIBEWR: WIMS-AECL LIBRARY '//
     >    CFILNA//' CANNOT BE OPENED FOR DEPLETION')
      CALL OPNIND(IUNIT,MASTER,LMASTB)
      CALL REDIND(IUNIT,MASTER,LMASIN,GENINX,LGENTB,1)
      CALL REDIND(IUNIT,GENINX,LGENIN,NPZ,LPZ,1)
      IF(NPZ(1).NE.NEL) CALL XABORT('LIBEWR: TOO MANY ISOTOPES '//
     >    'ON WIMS-AECL LIBRARY'//CFILNA)
      CALL REDIND(IUNIT,GENINX,LGENIN,IWISO,2*NEL,3)
      IEL2=1
      DO 10 IEL=1,NEL
        CALL UPCKIC(IWISO(IEL2),TEXT8,1)
        READ(TEXT8,'(2A4)') ITNAM(1,IEL),ITNAM(2,IEL)
        IEL2=IEL2+2
 10   CONTINUE
      CALL REDIND(IUNIT,GENINX,LGENIN,IWISO,NEL,2)
C----
C  READ DEPLETION CHAIN FOR EACH ISOTOPES
C----
      DO 100 ISO=1,NEL
C----
C  READ SUB INDEX ASSOCIATED WITH ISOTOPE
C----
        CALL REDIND(IUNIT,MASTER,LMASIN,SUBINX,LSUBTB,ISO+4)
        NBURN=SUBINX(LSUBIN+1)
        IF(NBURN.GT.MAXISO) THEN
          CALL XABORT('LIBEWR: NBURN LARGER THAN MAXISO')
        ENDIF
        NMIN=2*MAX0(NBURN,1)
        CALL REDIND(IUNIT,SUBINX,LSUBIN,GENINX,4,1)
        CALL REDIND(IUNIT,GENINX,4,IBURN,NMIN,1)
        CALL REDIND(IUNIT,GENINX,4,RBURN,NMIN,2)
C----
C  STORE REACTION TYPES AND RATES IN KPAX AND BPAX STARTING
C  WITH HEAVIER ISOTOPES
C----
        NFP=0
        DO 101 JBRN=1,NBURN
          IF(IBURN(2,JBRN).EQ.ICAPTU) THEN
            KTYPE=KCAPTU
            JSO=LIBWID(NEL,IWISO,IBURN(1,JBRN))
            IF(JSO.GT.0) THEN
              KPAX(JSO,ISO)=KTYPE
              BPAX(JSO,ISO)=RBURN(1,JBRN)
              KTYPE=10*JSO+KTYPE
            ENDIF
            KPAX(NEL+1,ISO)=KTYPE
            BPAX(NEL+1,ISO)=1.0
          ELSE IF(IBURN(2,JBRN).EQ.IN2N) THEN
            KTYPE=KN2N
            JSO=LIBWID(NEL,IWISO,IBURN(1,JBRN))
            IF(JSO.GT.0) THEN
              KPAX(JSO,ISO)=KTYPE
              BPAX(JSO,ISO)=RBURN(1,JBRN)
              KTYPE=10*JSO+KTYPE
            ENDIF
            KPAX(NEL+2,ISO)=KTYPE
            BPAX(NEL+2,ISO)=1.0
          ELSE IF(IBURN(2,JBRN).EQ.IN3N) THEN
            KTYPE=KN3N
            JSO=LIBWID(NEL,IWISO,IBURN(1,JBRN))
            IF(JSO.GT.0) THEN
              KPAX(JSO,ISO)=KTYPE
              BPAX(JSO,ISO)=RBURN(1,JBRN)
              KTYPE=10*JSO+KTYPE
            ENDIF
            KPAX(NEL+3,ISO)=KTYPE
            BPAX(NEL+3,ISO)=1.0
          ELSE IF(IBURN(2,JBRN).EQ.IDECAY.AND.
     >            RBURN(1,JBRN).GT.0.0) THEN
            KTYPE=KDECAY
            JSO=LIBWID(NEL,IWISO,IBURN(1,JBRN))
            IF(JSO.GT.0) THEN
              KPAX(JSO,ISO)=KTYPE
              BPAX(JSO,ISO)=1.0
              KTYPE=10*JSO+KTYPE
            ENDIF
            IF(KPAX(NEL+4,ISO).EQ.0) THEN
              KPAX(NEL+4,ISO)=KTYPE
              BPAX(NEL+4,ISO)=RBURN(1,JBRN)*CONVD
            ENDIF
          ELSE IF(IBURN(2,JBRN).EQ.IFISSI.AND.
     >            IBURN(1,JBRN).GT.0) THEN
            KTYPE=KFISSI
            KPAX(NEL+5,ISO)=KTYPE
            BPAX(NEL+5,ISO)=RBURN(1,JBRN)*CONVE
          ELSE IF(IBURN(2,JBRN).EQ.IFISSP.AND.
     >            RBURN(1,JBRN).GT.0.0) THEN
            KTYPE=KFISSP
            JSO=LIBWID(NEL,IWISO,IBURN(1,JBRN))
            IF(JSO.GT.0) THEN
              NFP=NFP+1
              KPAX(JSO,ISO)=KFISSP
              BPAX(JSO,ISO)=RBURN(1,JBRN)
            ENDIF
          ENDIF
 101    CONTINUE
        IF(NFP.GT.0) THEN
          IF(KPAX(NEL+5,ISO).EQ.KFISSI) THEN
            KPAX(NEL+5,ISO)=10*NFP+KPAX(NEL+5,ISO)
          ELSE
            CALL XABORT('LIBEWR: THERE ARE FISSION PRODUCT '//
     >        'ASSOCIATED WITH A NON FISSILE ISOTOPE')
          ENDIF
        ENDIF
 100  CONTINUE
C----
C  CLOSE WIMS-AECL LIBRARY
C----
      CALL CLSIND(IUNIT)
C----
C  RETURN
C----
      RETURN
      END
