*DECK LIBINP
      SUBROUTINE LIBINP(IPLIB ,INDREC,IPRINT,MAXMIX,MAXED ,MAXISO,
     >                  NBISO ,NGROUP,NGT   ,NL    ,ITRANC,ITIME ,
     >                  NLIB  ,NDEPL ,NCOMB ,NEDMAC,NBMIX ,NRES  ,
     >                  NGF   ,NGFR  ,ISOADD,HVECT ,
     >                  ISONAM,ISONRF,ISOMIX,DENISO,TMPISO,AWRISO,
     >                  NTFG  ,LSHI  ,SNISO ,MASKI ,MASK  ,DENMIX,
     >                  TMPMIX,IHLIB ,ITISOM)
C
C-----------------------------------------------------------------------
C
C READ THE INFORMATION RELATED TO MICROSCOPIC CROSS SECTION LIBRARY.
C
C INPUT/OUTPUT PARAMETERS:
C  IPLIB   : POINTER TO THE LATTICE MICROSCOPIC CROSS SECTION LIBRARY
C            (L_LIBRARY SIGNATURE).
C  INDREC  : =1 A NEW LIBRARY IS CREATED; =2 THE LIBRARY IS UPDATED;
C            =3 A READ-ONLY MACROLIB IS COPIED IN A NEW LIBRARY.
C  IPRINT  : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
C  MAXMIX  : MAXIMUM VALUE OF NBMIX.
C  MAXED   : MAXIMUM VALUE OF NEDMAC.
C  MAXISO  : MAXIMUM NUMBER OF ISOTOPES PERMITTED
C  NBISO   : NUMBER OF ISOTOPES PRESENT IN THE CALCULATION DOMAIN.
C  NGROUP  : NUMBER OF ENERGY GROUPS.
C  NGT     : NUMBER OF ENERGY GROUPS TO TEST.
C  NL      : NUMBER OF LEGENDRE ORDERS REQUIRED IN THE CALCULATION.
C            NL=1 (FOR ISOTROPIC SCATTERING) OR HIGHER.
C  ITRANC  : TYPE OF TRANSPORT CORRECTIONS. =0: NO TRANSPORT CORRECTION;
C            = 0: NO CORRECTION; >0: WIMS TYPE
C            = 1: APOLLO TYPE TRANSPORT CORRECTION;
C            = 2: WIMS TYPE CORRECTION
C  ITIME   : =1 : STEADY-STATE / =2 : PROMPT.
C  NLIB    : NUMBER OF INDEPENDENT LIBRARY
C  NDEPL   : NUMBER OF DEPLETING ISOTOPES (USED BY EVO:).
C  NCOMB   : NUMBER OF DEPLETING MIXTURES (USED BY EVO:).
C  NEDMAC  : NUMBER OF EXTRA VECTOR EDITS FROM MATXS.
C  NBMIX   : NUMBER OF MIXTURES DEFINED IN THE LIBRARY.
C  NRES    : NUMBER OF RESONANT MIXTURES (USED BY SHI:).
C  NGF     : NUMBER OF FAST GROUPS
C  ISOADD  : FLAG TO COMPLETE MIXTURE DEFINITION
C            IF DEPLETION CHAIN PRESENT
C            =-1 Partial completion (warning if missing information)
C            = 0 COMPLETE MIXTURES
C            = 1 NO COMPLETION
C  NGFR    : NUMBER OF FAST + RESONANCE GROUPS
C  HVECT   : CHARACTER*6 MATXS NAMES OF THE EXTRA VECTOR EDITS.
C            HVECT(MAXED)
C  ISONAM  : NAME OF EACH ISOTOPE. ISONAM(NTC,MAXISO)
C  ISONRF  : REFERENCE NAME OF EACH ISOTOPE. ISONRF(NTC,MAXISO)
C  ISOMIX  : MIX NUMBER OF EACH ISOTOPE. ISOMIX(MAXISO)
C  DENISO  : DENSITY OF EACH ISOTOPE. DENISO(MAXISO)
C  TMPISO  : TEMPERATURE OF EACH ISOTOPE. TMPISO(MAXISO)
C  AWRISO  : ATOMIC WEIGHT OF EACH ISOTOPE. AWRISO(MAXISO)
C  NTFG    : NUMBER OF THERMAL GROUPS WHERE THE THERMAL INELASTIC
C            CORRECTION IS APPLIED. NTFG(MAXISO)
C  LSHI    : RESONANT REGION NUMBER ASSOCIATED WITH ISOTOPE I IN SHIBA.
C            0 -> INFINITE DILUTION. LSHI(MAXISO)
C  SNISO   : DILUTION CROSS SECTION OF EACH GROUP AND EACH ISOTOPE.
C            1.0E10 -> INFINITE DILUTION. SNISO(MAXISO)
C WORK PARAMETERS:
C  MASKI   : ISOTOPE UPDATE LOGICAL. MASKI(MAXISO)
C  MASK    : MIXTURE UPDATE LOGICAL. MASK(MAXMIX)
C  DENMIX  : DENSITY OF EACH MIXTURE. DENMIX(MAXMIX)
C  TMPMIX  : TEMPERATURE OF EACH MIXTURE. TMPMIX(MAXMIX)
C  IHLIB   : ISOTOPE OPTIONS. IHLIB(NTC-1,MAXISO,5)
C
C--------------------------------------- AUTHOR: A. HEBERT 12/04/88 ----
C
      IMPLICIT     NONE
      INTEGER      IOUT,NTC,NHOBL,MAXR,NSTATE,NDPROC
      CHARACTER    NAMSBR*6
      PARAMETER   (IOUT=6,NTC=3,NHOBL=3,MAXR=5,NSTATE=40,NDPROC=20,
     >             NAMSBR='LIBINP')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER      IPLIB,INDREC,IPRINT,MAXMIX,MAXED,MAXISO,NBISO,
     >             NGROUP,NGT,NL,ITRANC,ITIME,NDEPL,NCOMB,
     >             NEDMAC,NBMIX,NRES,NGF,NGFR,ISOADD
      CHARACTER    HVECT(MAXED)*6
      INTEGER      ISONAM(NTC,MAXISO),ISONRF(NTC,MAXISO),
     >             ISOMIX(MAXISO),NTFG(MAXISO),LSHI(MAXISO),
     >             IHLIB(NTC-1,MAXISO,5),ITISOM(MAXISO)
      LOGICAL      MASKI(MAXISO),MASK(MAXMIX)
      REAL         DENISO(MAXISO),TMPISO(MAXISO),AWRISO(MAXISO),
     >             SNISO(MAXISO),DENMIX(MAXMIX),TMPMIX(MAXMIX)
C----
C  MEMORY ALLOCATION PARAMETERS
C----
      INTEGER      IBASE(1)
      LOGICAL      LBASE(1)
      REAL         RBASE
      COMMON       RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1),LBASE(1))
      INTEGER      IMILVO,IKPAR,IDR,INAM,ISOEVO,IGSB,IGSN,IJCEDM,
     >             IDELTA,IMASKL,JDEPR,NIFIS,ISONFI,IEMEV
C----
C  INPUT
C----
      INTEGER      ITYPLU,INTLIR
      CHARACTER    CARLIR*12
      REAL         REALIR
      DOUBLE PRECISION DBLLIR
C----
C  LOCAL PARAMETERS
C----
      CHARACTER    TEXT4*4,CFILNA*8,HHLIB*8
      LOGICAL      LNEW
      INTEGER      KCHAR(NTC),ISTATE(NSTATE),NGRCD,NGIS,NLIB,NNMIX,
     >             ILCMLN,ILCMTY,ITC,IIISO,IMIX,MIXCMB,NEWISO,
     >             ILIB,IED,NDEPLR,NBXR,JJISO,ITH,
     >             NELSN,ILOCSN,ILOCSB,NELHL,IGR,ITSTMP,IDEB,
     >             ITEXT4,NNFI
      REAL         TMPDAY(3),VOLTOT,VOLFRA
C----
C  DATA
C----
      CHARACTER    HOBL(NHOBL)*6
      SAVE         HOBL
      DATA         HOBL
     >           /'NHEAT','NG','N2N'/
C----
C  ALLOCATE REQUIRED MEMORY FOR ISOTOPE VECTORS
C----
      TEXT4='    '
      READ(TEXT4,'(A4)') ITEXT4
      NNMIX=0
      IF(INDREC .EQ. 2) THEN
C----
C  READ OLD LIBRARY INFORMATION
C----
        NNMIX=NBMIX
        CALL LCMGET(IPLIB,'ISOTOPESUSED',ISONAM)
        CALL LCMGET(IPLIB,'ISOTOPERNAME',ISONRF)
        CALL LCMGET(IPLIB,'ILIBRARYTYPE',IHLIB(1,1,1))
        CALL LCMGET(IPLIB,'ILIBRARYNAME',IHLIB(1,1,2))
        CALL LCMGET(IPLIB,'ISOTOPESCOH ',IHLIB(1,1,3))
        CALL LCMGET(IPLIB,'ISOTOPESINC ',IHLIB(1,1,4))
        CALL LCMGET(IPLIB,'ISOTOPESHIN ',IHLIB(1,1,5))
        CALL LCMGET(IPLIB,'ISOTOPESNTFG',NTFG)
        CALL LCMGET(IPLIB,'ISOTOPESMIX ',ISOMIX)
        CALL LCMGET(IPLIB,'ISOTOPESDENS',DENISO)
        CALL LCMGET(IPLIB,'ISOTOPESTEMP',TMPISO)
        CALL LCMGET(IPLIB,'ISOTOPESAWR ',AWRISO)
        CALL LCMGET(IPLIB,'ISOTOPESSHI ',LSHI)
        DO 10 IIISO=1,NBISO
          MASKI(IIISO)=.FALSE.
 10     CONTINUE
        DO 20 IMIX=1,MAXMIX
          DO 21 IIISO=1,NBISO
            IF(ISOMIX(IIISO) .EQ. IMIX) THEN
              TMPMIX(IMIX)=TMPISO(IIISO)
              GO TO 25
            ENDIF
 21       CONTINUE
 25       CONTINUE
 20     CONTINUE
      ENDIF
      CALL LCMLEN(IPLIB,'ISOTOPESDSN ',ILCMLN,ILCMTY)
      NELSN=ILCMLN
      IF(NELSN .GT. 0 ) THEN
        NGRCD=NGROUP
        NGIS=NGRCD*MAXISO
C----
C  READ OLD DILUTIONS IF PRESENT
C----
        CALL SETARA(RBASE,NGIS,IGSN)
        CALL SETARA(RBASE,NGIS,IGSB)
        CALL XDRSET(RBASE(IGSN),NGIS,1.0E10)
        CALL XDRSET(RBASE(IGSB),NGIS,1.0E10)
        CALL LCMGET(IPLIB,'ISOTOPESDSN ',RBASE(IGSN))
        CALL LCMGET(IPLIB,'ISOTOPESDSB ',RBASE(IGSB))
      ELSE
        NGRCD=1
        NGIS=NGRCD*MAXISO
        CALL SETARA(RBASE,NGIS,IGSN)
        CALL SETARA(RBASE,NGIS,IGSB)
      ENDIF
C----
C   READ THE SPECIFICATION FOR EACH MIXTURE
C----
      CARLIR='MIXS'
      ITYPLU=3
 50   CONTINUE
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': CHARACTER DATA EXPECTED.')
      IF(CARLIR(1:4) .EQ. 'MIXS') THEN
C----
C  GET LIBRARY TYPE AND NAME
C----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3 .OR. CARLIR(1:4) .NE. 'LIB:')
     >    CALL XABORT(NAMSBR//': KEYWORD LIB: EXPECTED')
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3)
     >    CALL XABORT(NAMSBR//': CHARACTER LIBRARY NAME REQUIRED.')
        HHLIB=CARLIR
        IF(HHLIB(1:4) .EQ. 'WIMS') THEN
          IF(HHLIB(5:8) .EQ. 'AECL' .OR.
     >       HHLIB(5:8) .EQ. '    ') THEN
            HHLIB='WIMSAECL'
          ELSE IF(HHLIB(5:8) .NE. 'D4  ') THEN
            CALL XABORT(NAMSBR//' INVALID LIBRARY TYPE '//HHLIB)
          ENDIF
        ELSE IF(HHLIB .NE. 'MATXS   ' .AND.
     >          HHLIB .NE. 'MATXS2  ' .AND.
     >          HHLIB .NE. 'APLIB1  ' .AND.
     >          HHLIB .NE. 'DRAGON  '  ) THEN
          CALL XABORT(NAMSBR//' INVALID LIBRARY TYPE '//HHLIB)
        ENDIF
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF( ITYPLU .NE. 3 .OR. CARLIR(1:4) .NE. 'FIL:')
     >    CALL XABORT(NAMSBR//': FIL: EXPECTED.')
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': CHARACTER DATA EXPECTED.')
        CFILNA=CARLIR(1:8)
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        NLIB=NLIB+1
        CALL LIBNRG(IPLIB,HHLIB,CFILNA,NGROUP,NGF,NGFR,NGT)
        GO TO 50
      ELSE IF(CARLIR(1:4) .EQ. ';') THEN
C----
C  READING FINISHED EXIT
C----
        GO TO 55
      ELSE IF(CARLIR(1:4) .EQ. 'MIX ') THEN
C----
C  READ MIXTURE INFORMATION
C----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .EQ. 1) THEN
          NNMIX=INTLIR
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        ELSE
          NNMIX=NNMIX+1
        ENDIF
        IF(NNMIX .GT. MAXMIX) THEN
          CALL XABORT(NAMSBR//': MIX NUMBER LARGER THAN MAXMIX.')
        ELSE IF(NNMIX .LE. 0) THEN
          CALL XABORT(NAMSBR//': MIX NUMBER .LE. 0.')
        ENDIF
        NBMIX=MAX(NNMIX,NBMIX)
        IF(ITYPLU .EQ. 3) THEN
          IF(CARLIR(1:4) .NE. 'COMB') CALL XABORT(NAMSBR//
     >    ': ONLY COMB KEYWORD CAN FOLLOW MIXTURE NUMBER')
C----
C  THIS MIXTURE IS A COMBINATION OF OTHER MIXTURE
C----
          VOLTOT=0.0
 200      CONTINUE
          VOLFRA=0.0
          MIXCMB=0
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .EQ. 3) THEN
            IF(VOLTOT .EQ. 0.0) CALL XABORT(NAMSBR//
     >      ': TOTAL VOLUME FRACTION OF 0.0 IS ILLEGAL')
            GO TO 50
          ENDIF
          MIXCMB=INTLIR
          IF(ITYPLU .EQ. 2) CALL XABORT(NAMSBR//
     >    ': MIXTURE NUMBER MISSING FOR COMBINATION')
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .EQ. 1 .OR. ITYPLU .EQ. 3)  CALL XABORT(NAMSBR//
     >    ': VOLUME FRACTION MISSING FOR COMBINATION')
          VOLFRA=REALIR
          IF(VOLFRA .EQ. 0.0) CALL XABORT(NAMSBR//
     >    ': INDIVIDUAL VOLUME FRACTION OF 0.0 IS ILLEGAL')
          CALL LIBCMB(MAXMIX,MAXISO,NBISO,NEWISO,NNMIX,MIXCMB,
     >                VOLTOT,VOLFRA,DENMIX,ISONAM,ISONRF,ISOMIX,
     >                IHLIB,DENISO,TMPISO,AWRISO,LSHI,SNISO,
     >                NTFG,MASKI,NELSN,NGRCD,RBASE(IGSN),
     >                RBASE(IGSB))
          GO TO 200
        ELSE
          TMPMIX(NNMIX)=REALIR
C----
C  RESET TEMPERATURE FOR ISOTOPE IF REQUIRED
C----
          DO 60 IIISO=1,NBISO
            IF(ISOMIX(IIISO) .EQ. NNMIX) THEN
              TMPISO(IIISO)=TMPMIX(NNMIX)
              MASKI(IIISO)=.TRUE.
            ENDIF
   60     CONTINUE
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .EQ. 2) THEN
C----
C  RESET DENSITY FOR ISOTOPE IF REQUIRED
C----
            IF(DENMIX(NNMIX) .EQ. -1.0) THEN
              CALL LIBCON(NNMIX,NBISO,ISOMIX,DENISO,AWRISO,
     >                    DENMIX(NNMIX),2)
            ENDIF
            DENMIX(NNMIX)=REALIR
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          ENDIF
        ENDIF
C----
C  MAIN MIXTURE INFORMATION TERMINATED LOOP ON 50
C----
        GO TO 50
      ENDIF
C----
C  IF NOT 'MIXS',  ';' OR  'MIX' READ ASSUME IT IS AN ISOTOPE NAME
C  TEST IF ISOTOPE ALIAS KNOWN AND UPDATE
C----
      READ(CARLIR,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
      DO 70 IIISO=1,NBISO
        IF(KCHAR(1) .EQ. ISONAM(1,IIISO) .AND.
     >     KCHAR(2) .EQ. ISONAM(2,IIISO) .AND.
     >     KCHAR(3) .EQ. ISONAM(3,IIISO) .AND.
     >     NNMIX    .EQ. ISOMIX(IIISO)   ) THEN
          NEWISO=IIISO
          LNEW=.FALSE.
          GO TO 75
        ENDIF
 70   CONTINUE
      LNEW=.TRUE.
      NBISO=NBISO+1
      NEWISO=NBISO
      IF(NBISO .GT. MAXISO) CALL XABORT(NAMSBR//
     >': MAXISO TOO SMALL.')
      DO 71 ITC=1,NTC
        ISONAM(ITC,NBISO)=KCHAR(ITC)
        ISONRF(ITC,NBISO)=KCHAR(ITC)
 71   CONTINUE
 75   CONTINUE
      ISOMIX(NEWISO)=NNMIX
      MASKI(NEWISO)=.TRUE.
      READ(HHLIB,'(2A4)') (IHLIB(ITC,NEWISO,1),ITC=1,NTC-1)
      READ(CFILNA,'(2A4)')(IHLIB(ITC,NEWISO,2),ITC=1,NTC-1)
      TMPISO(NEWISO)=TMPMIX(NNMIX)
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
      IF(ITYPLU .EQ. 3 .AND. CARLIR(1:1) .EQ.'=') THEN
        IF(CARLIR(2:2) .EQ. ' ') THEN
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .EQ. 3) THEN
            READ(CARLIR,'(3A4)') (ISONRF(ITC,NEWISO),ITC=1,3)
          ELSE
            CALL XABORT(NAMSBR//
     >      ': LIBRARY ISOTOPE NAME MISSING AFTER =')
          ENDIF
        ELSE
          CARLIR=CARLIR(2:12)//' '
          READ(CARLIR,'(3A4)') (ISONRF(ITC,NEWISO),ITC=1,3)
        ENDIF
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
      ENDIF
      IF(ITYPLU .EQ. 3) THEN
C----
C  USE THE CURRENT NAME AS ALIAS
C----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        GO TO 50
      ELSE IF(ITYPLU .NE. 2) THEN
        CALL XABORT(NAMSBR//
     >  ': ISOTOPIC DENSITY OR WEIGHT PERCENT EXPECTED.')
      ENDIF
      IF(.NOT.LNEW .AND. DENMIX(NNMIX).NE.-1.0 .AND.
     >    ABS(DENISO(NEWISO)-REALIR) .GT. 1.0E-5)
     >  CALL XABORT(NAMSBR//
     >  ': PERTURBATION OF THE WEIGHT PERCENTS IS FORBIDDEN.')
        DENISO(NEWISO)=REALIR
 80   CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
      IF(ITYPLU .EQ. 2) THEN
        IF(REALIR .LT. 0.0) CALL XABORT(NAMSBR//
     >  ': DILUTION MUST BE POSITIVE')
        SNISO(NEWISO)=REALIR
      ELSE IF(ITYPLU .EQ. 1) THEN
        LSHI(NEWISO)=INTLIR
        NRES=MAX(NRES,INTLIR)
      ELSE IF(CARLIR(1:4) .EQ. 'INF') THEN
        SNISO(NEWISO)=1.0E10
      ELSE IF(CARLIR(1:4) .EQ. 'SHIB') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': CHARACTER DATA EXPECTED.')
        READ(CARLIR,'(2A4)') (IHLIB(ITC,NEWISO,5),ITC=1,NTC-1)
      ELSE IF(CARLIR(1:4) .EQ. 'THER') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': NUMBER OF THERMALIZED GROUPS REQUIRED.')
        NTFG(NEWISO)=INTLIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': CHARACTER DATA EXPECTED.')
        READ(CARLIR,'(2A4)') (IHLIB(ITC,NEWISO,4),ITC=1,NTC-1)
      ELSE IF(CARLIR(1:4) .EQ. 'TCOH') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': CHARACTER DATA EXPECTED.')
        READ(CARLIR,'(2A4)') (IHLIB(ITC,NEWISO,3),ITC=1,NTC-1)
      ELSE IF(CARLIR(1:4) .EQ. 'DBYE') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': REAL DATA EXPECTED.')
        TMPISO(NEWISO)=REALIR
      ELSE
        GO TO 50
      ENDIF
      GO TO 80
C----
C  INCLUDE SOME DEFAULT EXTRA EDITS.
C----
 55   CONTINUE
      IF(NGROUP .EQ. 0 .OR. NGT .EQ .0) CALL XABORT(NAMSBR//
     >': NUMBER OF GROUPS REQUIRED')
      DO 90 ILIB=1,NHOBL
        DO 91 IED=1,NEDMAC
          IF(HVECT(IED) .EQ. HOBL(ILIB)) GO TO 95
 91     CONTINUE
        NEDMAC=NEDMAC+1
        IF(NEDMAC .GT. MAXED) CALL XABORT(NAMSBR//
     >  ': TOO MANY EXTRA EDITS REQUESTED.')
        HVECT(IED)=HOBL(ILIB)
 95     CONTINUE
 90   CONTINUE
C----
C  ADD THE MISSING ISOTOPES IN ISONAM FROM THE DEPLETION CHAIN.
C  AJOUT: Modification de la condition sur ISOADD
C----
      IF(NDEPL .NE. 0 .AND. ISOADD .LE. 0) THEN
        CALL LCMSIX(IPLIB,'DEPL-CHAIN',1)
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        NDEPLR=ISTATE(1)
        IF(NDEPLR .NE. NDEPL) CALL XABORT(NAMSBR//
     >  ': INCONSISTENT DEPL-CHAIN')
        NBXR=ISTATE(7)
        CALL SETARA(IBASE,MAXISO,ISOEVO)
        CALL SETARA(IBASE,(NTC-1)*NDEPL,INAM)
        CALL SETARA(IBASE,MAXR*NDEPL,IDR)
        CALL SETARA(IBASE,NBXR*NDEPL,IKPAR)
        CALL SETARA(IBASE,MAXMIX,IMILVO)
        CALL XDRSET(IBASE(ISOEVO),MAXISO,0)
        CALL LCMGET(IPLIB,'ISOTOPESUSED',IBASE(INAM))
        CALL LCMGET(IPLIB,'DEPLETE-REAC',IBASE(IDR))
        CALL LCMGET(IPLIB,'PRODUCE-REAC',IBASE(IKPAR))
        CALL LCMSIX(IPLIB,' ',2)
        CALL XDRSET(IBASE(IMILVO),MAXMIX,0)
        CALL LIBEAD(MAXISO,MAXMIX,NDEPL,MAXR,NBXR,NCOMB,NBISO,
     >              ISONAM,ISONRF,IHLIB,ISOMIX,DENISO,TMPISO,
     >              MASKI,IBASE(ISOEVO),IBASE(INAM),
     >              IBASE(IDR),IBASE(IKPAR),IBASE(IMILVO),
     >              ITISOM)
        CALL RLSARA(IBASE(IMILVO))
        CALL RLSARA(IBASE(IKPAR))
        CALL RLSARA(IBASE(IDR))
        CALL RLSARA(IBASE(INAM))
        CALL RLSARA(IBASE(ISOEVO))
      ENDIF
      IF(ISOADD .LT. 0) THEN
        WRITE(IOUT,9002) NAMSBR
      ENDIF
C----
C  SET THE MIXTURE MASKS.
C----
      DO 100 IMIX=1,NBMIX
        MASK(IMIX)=.FALSE.
        DO 101 IIISO=1,NBISO
          IF(ISOMIX(IIISO) .EQ. IMIX .AND. MASKI(IIISO)) THEN
            MASK(IMIX)=.TRUE.
            GO TO 105
          ENDIF
 101    CONTINUE
 105    CONTINUE
 100  CONTINUE
C----
C  FIND AND NAME DISTINCT ISOTOPES.
C----
      DO 110 IIISO=1,NBISO
        IF(MASKI(IIISO)) THEN
          DO 111 JJISO=1,IIISO-1
            DO 112 ITC=1,NTC-1
              IF(ISONAM(ITC,IIISO) .NE. ISONAM(ITC,JJISO))
     >        GO TO 116
              IF(ISONRF(ITC,IIISO) .NE. ISONRF(ITC,JJISO))
     >        GO TO 116
 112        CONTINUE
            DO 113 ITH=1,5
              DO 114 ITC=1,NTC-1
                IF(IHLIB(ITC,IIISO,ITH) .NE. IHLIB(ITC,JJISO,ITH))
     >          GO TO 116
 114          CONTINUE
 113        CONTINUE
            IF(TMPISO(IIISO) .EQ. TMPISO(JJISO) .AND.
     >         SNISO(IIISO)  .EQ. SNISO(JJISO)  .AND.
     >         NTFG(IIISO)   .EQ. NTFG(JJISO)   .AND.
     >         LSHI(IIISO)   .EQ. 0              .AND.
     >         LSHI(JJISO)   .EQ. 0              ) THEN
              MASKI(IIISO)=.FALSE.
              ITISOM(IIISO)=0
              IMIX=ISOMIX(JJISO)
              GO TO 115
            ENDIF
 116        CONTINUE
 111      CONTINUE
          IMIX=ISOMIX(IIISO)
 115      CONTINUE
          IF(ISONAM(NTC,IIISO) .NE. ITEXT4) THEN
            WRITE(IOUT,360) NAMSBR,(ISONAM(ITC,IIISO),ITC=1,NTC)
          ELSE
            WRITE(TEXT4,'(I4)') IMIX
            READ(TEXT4,'(A4)') ISONAM(NTC,IIISO)
          ENDIF
        ENDIF
 110  CONTINUE
C----
C PRINT IF REQUIRED
C----
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6000) MAXMIX,NBISO,NGROUP,NL,ITRANC,
     >                   ITIME,NLIB,NGF,NGFR,NDEPL,NCOMB,NEDMAC,
     >                   NBMIX,NRES
        IF(NEDMAC .GT. 0)
     >   WRITE(IOUT,310) (IED,HVECT(IED),IED=1,NEDMAC)
        IF(IPRINT .GE. 5) THEN
          WRITE(IOUT,320)
          DO 120 IIISO=1,NBISO
            IF(DENMIX(ISOMIX(IIISO)) .EQ. -1.0) THEN
              WRITE(IOUT,330) IIISO,
     >         (ISONAM(ITC,IIISO),ITC=1,NTC),
     >         (ISONRF(ITC,IIISO),ITC=1,NTC),
     >         (IHLIB(ITC,IIISO,1),ITC=1,NTC-1),
     >         (IHLIB(ITC,IIISO,2),ITC=1,NTC-1),
     >         ISOMIX(IIISO),DENISO(IIISO),
     >         TMPISO(IIISO),SNISO(IIISO),LSHI(IIISO),
     >         (IHLIB(ITC,IIISO,5),ITC=1,NTC-1),NTFG(IIISO),
     >         (IHLIB(ITC,IIISO,4),ITC=1,NTC-1),
     >         (IHLIB(ITC,IIISO,3),ITC=1,NTC-1)
            ELSE
              WRITE(IOUT,340) IIISO,
     >         (ISONAM(ITC,IIISO),ITC=1,NTC),
     >         (ISONRF(ITC,IIISO),ITC=1,NTC),
     >         (IHLIB(ITC,IIISO,1),ITC=1,NTC-1),
     >         (IHLIB(ITC,IIISO,2),ITC=1,NTC-1),
     >         ISOMIX(IIISO),DENMIX(ISOMIX(IIISO)),DENISO(IIISO),
     >         TMPISO(IIISO),SNISO(IIISO),LSHI(IIISO),
     >         (IHLIB(ITC,IIISO,5),ITC=1,NTC-1),NTFG(IIISO),
     >         (IHLIB(ITC,IIISO,4),ITC=1,NTC-1),
     >         (IHLIB(ITC,IIISO,3),ITC=1,NTC-1)
            ENDIF
  120     CONTINUE
        ENDIF
      ENDIF
      NGIS=NGROUP*NBISO
      IF(NELSN .EQ. 0) THEN
        CALL RLSARA(RBASE(IGSB))
        CALL RLSARA(RBASE(IGSN))
        NGIS=NGROUP*NBISO
C----
C  READ OLD DILUTIONS IF PRESENT
C----
        CALL SETARA(RBASE,NGIS,IGSN)
        CALL SETARA(RBASE,NGIS,IGSB)
        CALL XDRSET(RBASE(IGSN),NGIS,1.0E10)
        CALL XDRSET(RBASE(IGSB),NGIS,1.0E10)
      ENDIF
      ILOCSN=IGSN
      ILOCSB=IGSB
      DO 130 IIISO=1,NBISO
        IF(SNISO(IIISO) .GT. 0.0) THEN
          CALL XDRSET(RBASE(ILOCSN),NGROUP,SNISO(IIISO))
          CALL XDRSET(RBASE(ILOCSN),NGROUP,SNISO(IIISO))
        ENDIF
        ILOCSN=ILOCSN+NGROUP
        ILOCSB=ILOCSB+NGROUP
 130  CONTINUE
C----
C   SAVE THE INFORMATION.
C----
      NBMIX=0
      DO 140 IIISO=1,NBISO
        NBMIX=MAX(NBMIX,ISOMIX(IIISO))
 140  CONTINUE
      IF(NBMIX .GT. MAXMIX) CALL XABORT(NAMSBR//
     >': MAXMIX TOO SMALL.')
      CARLIR='L_LIBRARY'
      READ(CARLIR,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
      CALL LCMPUT(IPLIB,'SIGNATURE',3,3,KCHAR)
      CALL XDRSET(ISTATE,NSTATE,0)
      ISTATE(1)=MAXMIX
      ISTATE(2)=NBISO
      ISTATE(3)=NGROUP
      ISTATE(4)=NL
      ISTATE(5)=ITRANC
      ISTATE(6)=ISOADD
      ISTATE(7)=ITIME
      ISTATE(8)=NLIB
      ISTATE(9)=NGF
      ISTATE(10)=NGFR
      ISTATE(11)=NDEPL
      ISTATE(12)=NCOMB
      ISTATE(13)=NEDMAC
      ISTATE(14)=NBMIX
      ISTATE(15)=NRES
      ISTATE(16)=MAXISO/MAXMIX+1
      ISTATE(17)=NDPROC
      CALL LCMPUT(IPLIB,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL LCMPUT(IPLIB,'ISOTOPESUSED',NTC*NBISO,3,ISONAM)
      CALL LCMPUT(IPLIB,'ISOTOPERNAME',NTC*NBISO,3,ISONRF)
      CALL LCMPUT(IPLIB,'ISOTOPESMIX ',NBISO,1,ISOMIX)
      NELHL=(NTC-1)*NBISO
      IF(NLIB .GT. 0) THEN
        CALL LCMPUT(IPLIB,'ILIBRARYTYPE',NELHL,3,IHLIB(1,1,1))
        CALL LCMPUT(IPLIB,'ILIBRARYNAME',NELHL,3,IHLIB(1,1,2))
        CALL LCMPUT(IPLIB,'ISOTOPESCOH ',NELHL,3,IHLIB(1,1,3))
        CALL LCMPUT(IPLIB,'ISOTOPESINC ',NELHL,3,IHLIB(1,1,4))
        CALL LCMPUT(IPLIB,'ISOTOPESNTFG',NBISO,1,NTFG)
        IF(NRES .GT. 0) THEN
          CALL LCMPUT(IPLIB,'ISOTOPESHIN ',NELHL,3,IHLIB(1,1,5))
          CALL LCMPUT(IPLIB,'ISOTOPESDSN ',NGIS,2,RBASE(IGSN))
          CALL LCMPUT(IPLIB,'ISOTOPESDSB ',NGIS,2,RBASE(IGSB))
          CALL LCMPUT(IPLIB,'ISOTOPESSHI ',NBISO,1,LSHI)
        ENDIF
      ENDIF
      IF(NEDMAC .GT. 0) THEN
        CALL SETARA(IBASE,(NTC-1)*NEDMAC,IJCEDM)
        IDEB=IJCEDM
        DO 141 IED=1,NEDMAC
          READ(HVECT(IED),'(A4,A2)')
     >    (IBASE(IIISO),IIISO=IDEB,IDEB+NTC-2)
          IDEB=IDEB+NTC-1
 141    CONTINUE
        CALL LCMPUT(IPLIB,'HVECT',(NTC-1)*NEDMAC,3,IBASE(IJCEDM))
        CALL RLSARA(IBASE(IJCEDM))
      ENDIF
C----
C   READ AND INTERPOLATE IN THE MICROSCOPIC X-SECTIONS LIBRARIES.
C----
      IF(NGROUP .EQ. 0) CALL XABORT(NAMSBR//
     >': NUMBER OF GROUPS NOT DEFINED.')
      CALL SETARA(RBASE,NGROUP,IDELTA)
      CALL LIBLIB(IPLIB,IPRINT,MAXISO,NBISO,NGROUP,ISONAM,ISONRF,
     >            IHLIB,NTFG,TMPISO,AWRISO,RBASE(IGSN),RBASE(IGSB),
     >            MASKI,NL,ITIME,NEDMAC,HVECT,ITRANC,ITISOM,
     >            RBASE(IDELTA),NGFR,ISOADD)
      CALL RLSARA(RBASE(IDELTA))
      CALL RLSARA(RBASE(IGSB))
      CALL RLSARA(RBASE(IGSN))
      DO 142 IIISO=1,NBISO
        IF(AWRISO(IIISO) .EQ. 0.0 .AND.  .NOT.MASKI(IIISO)) THEN
          DO 143 JJISO=1,IIISO-1
            IF(ISONAM(1,JJISO) .EQ. ISONAM(1,IIISO) .AND.
     >         ISONAM(2,JJISO) .EQ. ISONAM(2,IIISO) .AND.
     >         ISONAM(3,JJISO) .EQ. ISONAM(3,IIISO)     ) THEN
              AWRISO(IIISO)=AWRISO(JJISO)
            ENDIF
 143      CONTINUE
        ENDIF
 142  CONTINUE
C----
C  COMPUTE AND STORE THE EFFECTIVE DENSITY FROM AWR
C   AND MATERIAL DENSITY
C----
      DO 150 IMIX=1,NBMIX
        IF(MASK(IMIX) .AND. DENMIX(IMIX) .GE. 0.0) THEN
          CALL LIBCON(IMIX,NBISO,ISOMIX,DENISO,AWRISO,
     >                DENMIX(IMIX),1)
        ENDIF
 150  CONTINUE
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,370)
        WRITE(IOUT,320)
        DO 160 IIISO=1,NBISO
          IF(MASK(ISOMIX(IIISO))) THEN
            WRITE(IOUT,330) IIISO,
     >       (ISONAM(ITC,IIISO),ITC=1,NTC),
     >       (ISONRF(ITC,IIISO),ITC=1,NTC),
     >       (IHLIB(ITC,IIISO,1),ITC=1,NTC-1),
     >       (IHLIB(ITC,IIISO,2),ITC=1,NTC-1),
     >       ISOMIX(IIISO),DENISO(IIISO),
     >       TMPISO(IIISO),SNISO(IIISO),LSHI(IIISO),
     >       (IHLIB(ITC,IIISO,5),ITC=1,NTC-1),NTFG(IIISO),
     >       (IHLIB(ITC,IIISO,4),ITC=1,NTC-1),
     >       (IHLIB(ITC,IIISO,3),ITC=1,NTC-1)
           ENDIF
  160   CONTINUE
      ENDIF
      CALL LCMPUT(IPLIB,'ISOTOPESAWR ',NBISO,2,AWRISO)
      CALL LCMPUT(IPLIB,'ISOTOPESDENS',NBISO,2,DENISO)
      IF(NLIB .GT. 0) THEN
        CALL LCMPUT(IPLIB,'ISOTOPESTEMP',NBISO,2,TMPISO)
      ENDIF
C----
C  COMPUTE THE MACROSCOPIC X-SECTIONS
C----
      CALL SETARA(LBASE,NGROUP,IMASKL)
      DO 170 IGR=1,NGROUP
        LBASE(IMASKL+IGR-1)=.TRUE.
 170  CONTINUE
      ITSTMP=0
      TMPDAY(1)=0.0
      TMPDAY(2)=0.0
      TMPDAY(3)=0.0
      CALL LIBMIX(IPLIB,NGROUP,NBISO,ISONAM,ISOMIX,DENISO,MASK,
     >            LBASE(IMASKL),ITSTMP,TMPDAY)
      IF(NDEPL .GT. 0) THEN
        CALL LCMSIX(IPLIB,'DEPL-CHAIN',1)
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        NDEPLR=ISTATE(1)
        IF(NDEPLR .NE. NDEPL) CALL XABORT(NAMSBR//
     >  ': INCONSISTENT DEPL-CHAIN')
        CALL SETARA(IBASE,(NTC-1)*NDEPL,INAM)
        CALL SETARA(IBASE,MAXR*NDEPL,IDR)
        CALL SETARA(RBASE,MAXR*NDEPL,JDEPR)
        CALL LCMGET(IPLIB,'ISOTOPESUSED',IBASE(INAM))
        CALL LCMGET(IPLIB,'DEPLETE-REAC',IBASE(IDR))
        CALL LCMGET(IPLIB,'DEPLETE-RATE',RBASE(JDEPR))
        CALL LCMSIX(IPLIB,' ',2)
        CALL LCMSIX(IPLIB,'MACROLIB',1)
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        NNFI=ISTATE(4)
        IF(NNFI .GT. 0) THEN
          CALL SETARA(IBASE,NBMIX*NNFI,NIFIS)
          CALL SETARA(IBASE,(NTC-1)*NNFI,ISONFI)
          CALL SETARA(RBASE,NBMIX*NNFI,IEMEV)
          CALL LCMGET(IPLIB,'FISSIONNB   ',IBASE(NIFIS))
          CALL LCMGET(IPLIB,'FISSIONNAMES',IBASE(ISONFI))
          CALL XDRSET(RBASE(IEMEV),NBMIX*NNFI,0.0)
          CALL LIBMFE(NDEPL ,MAXR  ,NBISO ,NBMIX ,NNFI  ,ISONAM,
     >                ISONRF,ISOMIX,
     >                IBASE(INAM)  ,IBASE(IDR)   ,RBASE(JDEPR) ,
     >                IBASE(ISONFI),IBASE(NIFIS) ,RBASE(IEMEV) )
          CALL LCMPUT(IPLIB,'EFISS       ',NBMIX*NNFI,
     >                2,RBASE(IEMEV))
          CALL RLSARA(RBASE(IEMEV))
          CALL RLSARA(IBASE(ISONFI))
          CALL RLSARA(IBASE(NIFIS))
        ENDIF
        CALL LCMSIX(IPLIB,'MACROLIB',2)
        CALL RLSARA(RBASE(JDEPR))
        CALL RLSARA(IBASE(IDR))
        CALL RLSARA(IBASE(INAM))
      ELSE
        CALL LCMSIX(IPLIB,'MACROLIB',1)
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        NNFI=ISTATE(4)
        IF(NNFI .GT. 0) THEN
          CALL SETARA(RBASE,NBMIX*NNFI,IEMEV)
          CALL XDRSET(RBASE(IEMEV),NBMIX*NNFI,0.0)
          CALL LCMPUT(IPLIB,'EFISS       ',NBMIX*NNFI,
     >                2,RBASE(IEMEV))
          CALL RLSARA(RBASE(IEMEV))
        ENDIF
        CALL LCMSIX(IPLIB,'MACROLIB',2)
      ENDIF
      CALL RLSARA(LBASE(IMASKL))
      IF(IPRINT .GE. 100) CALL LCMLIB(IPLIB)
      RETURN
*
 6000 FORMAT(/' STATE-VECTOR FOR MICROLIB'/' -------'/
     >  ' MAXMIX ',I6,' (MAXIMUM NUMBER OF MIXTURES)'/
     >  ' NBISO  ',I6,' (NUMBER OF ISOTOPES OR MATERIALS)'/
     >  ' NGROUP ',I6,' (NUMBER OF ENERGY GROUPS)'/
     >  ' NL     ',I6,' (NUMBER OF LEGENDRE ORDERS)'/
     >  ' ITRANC ',I6,' (TYPE OF TRANSPORT CORRECTION)'/
     >  ' ITIME  ',I6,' (1=STEADY-STATE/2=PROMPT)'/
     >  ' NLIB   ',I6,' (NUMBER OF SETS OF LIBRARIES)'/
     >  ' NGF    ',I6,' (NUMBER OF FAST ENERGY GROUPS)'/
     >  ' NGFR   ',I6,' (NUMBER OF FAST+RESONANCE ENERGY GROUPS)'/
     >  ' NDEPL  ',I6,' (NUMBER OF DEPLETING ISOTOPES)'/
     >  ' NCOMB  ',I6,' (NUMBER OF DEPLETING MIXTURES)'/
     >  ' NEDMAC ',I6,' (NUMBER OF EXTRA EDITS)'/
     >  ' NBMIX  ',I6,' (NUMBER OF MIXTURES)'/
     >  ' NRES   ',I6,' (NUMBER OF SETS OF RESONANT MIXTURES)'/)
  310 FORMAT(/' MATXS EDIT NAME (LCM DIRECTORY NAME)'/1X,36(' ')/
     1 (1X,I3,2X,A6))
  320 FORMAT(/' SPEC LOCAL NAME   ISOTOPE      FROM LIBRARY     ',
     >        '   MIX     DENSITY     WEIGHT%    TEMP(K)',
     >        '   SIGZERO   SELF-SHIEL    THERMAL CORRECTION'/
     >        ' ---- ------------ ------------ -----------------',
     >        '   ---     -------     -------  ---------',
     >        '  --------   ----------    ------------------')
  330 FORMAT(1X,I4,1X,3A4,1X,3A4,1X,2A4,1X,2A4,1X,I5,1P,E12.4,12X,
     1 E11.3,E10.2,I4,1X,2A4,I5,1X,4A4)
  340 FORMAT(1X,I4,1X,3A4,1X,3A4,1X,2A4,1X,2A4,1X,I5,1P,2E12.4,
     1 E11.3,E10.2,I4,1X,2A4,I5,1X,4A4)
  350 FORMAT(/' TIME SPENT TO MIX THE MACROSCOPIC CROSS SECTIONS =',
     1 F8.1,' SECONDS.'/)
  360 FORMAT(/1X,A6,': UNABLE TO RENAME ISOTOPE ',3A4)
  370 FORMAT(/60X,'NUMBER')
 9002 FORMAT(' Message from ',A6,' : Option PDEPLCH activated')
      END
