*DECK LIBLAG
      SUBROUTINE LIBLAG(NEF,XE,GE,XI,GS)
C
C-----------------------------------------------------------------------
C
C LAGRANGE INTERPOLATION IN A TABLE OF POINTS.
C
C INPUT PARAMETERS:
C  NEF     : NUMBER OF POINTS.
C  XE      : X-VALUES. DIMENSION XE(NEF)
C  GE      : F(X)-VALUES. DIMENSION GE(NEF)
C  XI      : INTERPOLATING X-VALUE.
C
C OUTPUT PARAMETER:
C  GS      : INTERPOLATED VALUE F(XI).
C
C-----------------------------------------------------------------------
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER   NINT
      PARAMETER(NINT=3)
C----
C INTERFACE VARIABLES
C----
      INTEGER  NEF
      REAL     XI,XE(*),GE(*),GS
C----
C LOCAL VARIABLES
C----
      INTEGER   I,IORD,IXP,IMIN,IMAX,I0,I1,J
      REAL      PP,YMIN,YMAX,WEIGHT(NINT)
C----
C     CONSTANT FUNCTION DEFINED BY A SINGLE POINT
C----
      IF(NEF.EQ.1) THEN
         GS=GE(1)
         RETURN
      ENDIF
*
      IORD=MIN(NINT,NEF)
*
*     LOCATE FIRST POINT TO THE RIGHT OF XI (PT IXP)
*
      DO 30 IXP=1,NEF
       IF(ABS(XE(IXP)-XI).LE.1.0E-5*ABS(XI)) THEN
          GS=GE(IXP)
          RETURN
       ELSE IF(XE(IXP).GT.XI) THEN
          IMIN=IXP-(IORD+1)/2
          IMAX=IMIN+IORD-1
          GO TO 40
       ENDIF
   30 CONTINUE
*
*     XI IS TO THE RIGHT OF EVERY POINT XE
*
      IMAX=NEF
      IMIN=NEF-IORD+1
      GO TO 70
*
   40 IF(IMIN.LT.1) THEN
         IMIN=1
         IMAX=IORD
      ELSE IF(IMAX.GT.NEF) THEN
         IMIN=NEF-IORD+1
         IMAX=NEF
      ENDIF
*
   70 I0=IMIN-1
      DO 90 I=IMIN,IMAX
      PP=1.0
      DO 80 J=IMIN,IMAX
       IF(I.NE.J) PP=PP*((XI-XE(J))/(XE(I)-XE(J)))
   80 CONTINUE
   90 WEIGHT(I+1-IMIN)=PP
*
      GS=0.0
      DO 110 I=1,IORD
  110 GS=GS+WEIGHT(I)*GE(I+I0)
      DO 120 I=1,IORD
      I1=I+I0
      IF(XE(I1).GT.XI) THEN
         IF(I1-1.GT.0) THEN
            YMIN=MIN(GE(I1-1),GE(I1))
            YMAX=MAX(GE(I1-1),GE(I1))
            IF((GS.GT.YMAX).OR.(GS.LT.YMIN)) THEN
               GS=GE(I1-1)+(GE(I1)-GE(I1-1))*
     1         (XI-XE(I1-1))/(XE(I1)-XE(I1-1))
            ENDIF
         ELSE
            GS=GE(1)+(GE(2)-GE(1))*(XI-XE(1))/(XE(2)-XE(1))
            IF(GS.LE.0.) GS=GE(1)
         ENDIF
         RETURN
      ENDIF
  120 CONTINUE
      RETURN
      END
