*DECK LIBTR1
      SUBROUTINE LIBTR1(IPLIB,IMPX,CFILNA,NGRO,NBISO,NL,ISONAM,ISONRF,
     >                  ICOHNA,IINCNA,NTFG,TN,AW,SN,SB,ILIB,ITISOM,
     >                  DELTA,NGFR,ITRANC,NED,HVECT,IVECT,ITIME,NPROC,
     >                  INDPRO,ITYPRO,XSREC,SCAT,IPR,GAR,NJJ,IJJ,CNORM)
C
C-----------------------------------------------------------------------
C
C TRANSCRIPTION OF THE USEFULL INTERPOLATED MICROSCOPIC CROSS SECTION
C DATA FROM MATXS TO LCM. USE MATXS FORMAT FROM NJOY-II OR NJOY89.
C
C INPUT PARAMETERS:
C  IPLIB   : POINTER TO THE LATTICE MICROSCOPIC CROSS SECTION LIBRARY
C            (L_LIBRARY SIGNATURE).
C  IMPX    : PRINT FLAG.
C  CFILNA  : CHARACTER*8 NAME OF THE LIBRARY FILE.
C  NGRO    : NUMBER OF ENERGY GROUPS.
C  NBISO   : NUMBER OF ISOTOPES PRESENT IN THE CALCULATION DOMAIN.
C  NL      : NUMBER OF LEGENDRE ORDERS REQUIRED IN THE CALCULATION.
C            NL=1 OR HIGHER.
C  ISONAM  : ALIAS NAME OF ISOTOPES. ISONAM(3,NBISO)
C  ISONRF  : REFERENCE NAME OF ISOTOPES. ISONRF(3,NBISO)
C  ICOHNA  : HCOH NAME. ICOHNA(2,NBISO)
C  IINCNA  : HINC NAME. IINCNA(2,NBISO)
C  NTFG    : NUMBER OF THERMAL GROUPS WHERE THE THERMAL INELASTIC
C            CORRECTION IS APPLIED. NTFG(MAXISO)
C  TN      : TEMPERATURE OF EACH ISOTOPE. DIMENSION TN(NBISO)
C  AW      : ATOMIC WEIGHT. AW(NBISO)
C  SN      : DILUTION CROSS SECTION IN EACH ENERGY GROUP OF EACH
C            ISOTOPE. SN(NGRO,NBISO)
C  SB      : DILUTION CROSS SECTION AS USED BY LIVOLANT AND JEANPIERRE
C            NORMALIZATION. SB(NGRO,NBISO)
C  ILIB    : LIBRARY TO PROCESS
C  ITISOM  : ISOTOPE WITH INDEX ILIB IS PROCESSED. ITISOM(NBISO)
C  DELTA   : LETHARGY LIMITS OF GROUPS. DELTA(NGRO)
C  NGFR    : NUMBER OF FAST+RESONANCE GROUPS.
C  ITRANC  : TRANSPORT CORRECTION
C            0 -> NONE ; 1 -> APOL; 2 -> WIMS
C  NED     : NUMBER OF EXTRA VECTOR EDITS FROM MATXS.
C  HVECT   : CHARACTER*6 MATXS NAMES OF THE EXTRA VECTOR EDITS.
C            DIMENSION HVECT(NED)
C            MATXS RESERVED NAMES:
C            NWT0/NWT1   : P0/P1 LIBRARY WEIGHT FUNCTION.
C            NTOT0/NTOT1 : P0/P1 NEUTRON TOTAL CROSS SECTIONS.
C            NELAS : NEUTRON ELASTIC SCATTERING CROSS SECTION.
C            NINEL : NEUTRON INELASTIC SCATTERING CROSS SECTION.
C            NG    : RADIATIVE CAPTURE CROSS SECTION.
C            NFTOT : TOTAL FISSION CROSS SECTION.
C            NUDEL : NUMBER OF DELAYED SECONDARY NEUTRONS (NU-D).
C            NFSLO : NU C SLOW FISSION CROSS SECTION.
C            CHIS/CHID : SLOW/DELAYED FISSION SPECTRUM.
C            NF/NNF/N2NF/N3NF : NU C PARTIAL FISSION CROSS SECTIONS.
C            N2N/N3N/N4N : (N,2N),(N,3N),(N,4N) CROSS SECTIONS.
C  IVECT   : POSITION OF EXTRA EDIT IN XSREC VECTOR IVECT(NED)
C  ITIME   : =1 : STEADY-STATE / =2 : PROMPT.
C  NPROC   : NUMBER OF CROSS SECTION TYPE PROCESSED
C  INDPRO  : FLAG FOR CROSS SECTION TYPE TO PROCESSED INDPRO(NPROC)
C SCRATCH STORAGE:
C  ITYPRO  : FLAG FOR CROSS SECTION TYPE PROCESSED ITYPRO(NPROC)
C  XSREC   : DIMENSION XSREC(NGRO,NPROC)
C  SCAT    : DIMENSION SCAT(NGRO,NGRO,NL)
C  IPR     : DIMENSION IPR(2,NBISO)
C  GAR     : DIMENSION GAR(NGRO*NGRO)
C  NJJ     : DIMENSION NJJ(NGRO)
C  IJJ     : DIMENSION IJJ(NGRO)
C  CNORM   : DIMENSION CNORM(NBISO,3)
C
C SUBROUTINES CALLED BY LIBTR1:
C LIBTER, XDREED, LCM---.
C
C REFERENCE:
C R. E. MACFARLANE, TRANSX-CTR: A CODE FOR INTERFACING MATXS CROSS-
C SECTION LIBRARIES TO NUCLEAR TRANSPORT CODES FOR FUSION SYSTEMS
C ANALYSIS, LOS ALAMOS NATIONAL LABORATORY, REPORT LA-9863-MS,
C NEW MEXICO, FEBRUARY 1984.
C
C-------------------------------------- AUTHOR:  A. HEBERT 07/04/88 ----
C-------------------------------------MODIFIED: G. MARLEAU 97/06/27 ----
C
      IMPLICIT NONE
C----
C PARAMETESR
C----
      INTEGER       IOUT,NDPROC,MULT,MAXA
      CHARACTER     FORM*4
      PARAMETER    (IOUT=6,NDPROC=20,MULT=2,MAXA=1000,FORM='(A6)')
C----
C INTEFAFCE VARIABLES
C----
      INTEGER       IPLIB,IMPX,NGRO,NBISO,NL,ILIB,NGFR,ITRANC,NED
      INTEGER       ISONAM(3,NBISO),ISONRF(3,NBISO),ICOHNA(2,NBISO),
     1              IINCNA(2,NBISO),NTFG(NBISO),ITISOM(NBISO),
     3              IVECT(NED),ITIME,NPROC,INDPRO(NPROC),ITYPRO(NPROC),
     4              IPR(2,NBISO),NJJ(NGRO),IJJ(NGRO)
      REAL          TN(NBISO),AW(NBISO),SN(NGRO,NBISO),
     1              SB(NGRO,NBISO),DELTA(NGRO),
     2              XSREC(NGRO,NPROC),SCAT(NGRO,NGRO,NL),
     3              GAR(NGRO*NGRO),CNORM(NBISO,3)
      CHARACTER     CFILNA*8,HVECT(NED)*6
C----
C MEMORY ALLOCATION
C----
      INTEGER       IBASE(1)
      REAL          RBASE(1)
      COMMON        RBASE
      EQUIVALENCE  (RBASE(1),IBASE(1))
      INTEGER       IXTERP,ITERP,ITEMP,ISIGZ
C----
C EXTERNAL FUNCTIONS
C----
      INTEGER       KDROPN,KDRCLS
C----
C  LOCAL VARIABLES
C----
      INTEGER       NIN,IREC,NWDS,IVER,NPART,NTYPE,NHOLL,I,NWC,L2,L2H,
     1              NEX1,NG,IG4EV,IRZT,IT,ITYPE,NDEX,NMAT,NOUTP,LOCT,
     2              LMC,LMCH,IRZM,IM,IMX,ITC,NINP,NING,NOUTG,NSBLK,IMT,
     3              LOC,NSUBM,L3,L3H,LOCM,ISUBM,LOCS,IK,N1DR,N1DB,N2DB,
     4              JREC,NEX2,IR,NEX3,IB,IBLK,IJ,IXS,JJ,K,NK,IXSR,
     5              LORD,LONE,LN,LG,IFISN,INC,NP,IFTOT,L5,J,IL,ILNOW,
     6              IP,IER,IG1,IPRO,JJP,IG2,NISOT
      INTEGER       IA(MAXA),IHGAR(22)
C
      REAL          TMAT,SMAT,TERPZ,ADD,ADDD,XSNOW,SUM,XSEC
      REAL          RA(MAXA)
      CHARACTER     HSMG*131,HNAMIS*12,HNISOR*12,README*88,
     1              HN*6,HU*6,HS*6,HPRT*6,HTYPE*6,HMAT*6,HVPS*6,
     2              HMTX*6,HGAR(18)*6,HINC*6,HCOH*6
      LOGICAL       LSUBM1,LTHERM,LTIME,LTERP
      DOUBLE PRECISION DA(MAXA/2)
      EQUIVALENCE  (RA(1),IA(1),DA(1))
C----
C  INITIALIZE IPR AND OPEN MATXS FILE
C  INITIALIZE LIBRARY
C----
      CALL XDRSET(IPR,2*NBISO,0)
      IF(IMPX.GT.0) WRITE(IOUT,890) CFILNA
      NIN=KDROPN(CFILNA,2,2,0,0)
      IF(NIN.LE.0) THEN
        WRITE(HSMG,9000) CFILNA
        CALL XABORT(HSMG)
      ENDIF
      IREC=1
      NWDS=1+3*MULT
*-------FILE IDENTIFICATION--------
      CALL XDREED(NIN,IREC,RA,NWDS)
*----------------------------------
      WRITE(HN,FORM) DA(1)
      WRITE(HU,FORM) DA(2)
      WRITE(HS,FORM) DA(3)
      IVER=IA(1+3*MULT)
      IF(IMPX.GT.0) WRITE(IOUT,935) HN,HU,HS,IVER
      IREC=2
      NWDS=3
*-------FILE CONTROL---------------
      CALL XDREED(NIN,IREC,RA,NWDS)
*----------------------------------
      NPART=IA(1)
      NTYPE=IA(2)
      NHOLL=IA(3)
      IREC=3
      NWDS=NHOLL*MULT
      IF(NWDS.GT.MAXA) CALL XABORT
     >  ('LIBTR1: LENGTH OF RECORD 3 > MAXA ')
*-------HOLLERITH IDENTIFICATION---
      CALL XDREED(NIN,IREC,RA,NWDS)
*----------------------------------
      WRITE(README(9:),'(6H FROM ,12A6)') (DA(I),I=1,MIN(NHOLL,12))
      IF(IMPX.GT.0) WRITE(IOUT,'(1X,12A6)') (DA(I),I=1,MIN(NHOLL,12))
      IREC=4
      NWDS=(NPART+NTYPE)*MULT+6*NTYPE+NPART
      IF(NWDS.GT.MAXA) CALL XABORT
     >  ('LIBTR1: LENGTH OF RECORD 4 > MAXA ')
*-------FILE DATA------------------
      CALL XDREED(NIN,IREC,RA,NWDS)
*----------------------------------
      NWC=NPART+NTYPE
      IF((NWDS/2)*2.NE.NWDS) NWDS=NWDS+1
      L2=1+NWDS
      L2H=(L2-1)/MULT+1
*----
*  CHECK GROUP STRUCTURES
*----
      NEX1=(NPART+NTYPE)*MULT+6*NTYPE
      DO 100 I=1,NPART
        WRITE(HPRT,FORM) DA(I)
        NG=IA(NEX1+I)
        IF(((HPRT.EQ.'NEUT'.OR.HPRT.EQ.'neut').OR.
     >      (HPRT.EQ.'N'.OR.HPRT.EQ.'n')).AND.(NG.NE.NGRO))
     >    CALL XABORT('LIBTR1: INCONSISTENT GROUP STRUCTURES.')
        IREC=IREC+1
  100 CONTINUE
      IG4EV=0
      IF(ITRANC.GE.2) THEN
        IG4EV=NGFR
      ENDIF
      IRZT=5+NPART
      CALL XDRSET(CNORM,3*NBISO,0.0)
*----
*  DATA TYPE LOOP
*----
      DO 110 IT=1,NTYPE
        WRITE(HTYPE,FORM) DA(NPART+IT)
        CALL XDRCAS('LOWTOUP',HTYPE)
        IF(HTYPE.EQ.'NSCAT') THEN
           ITYPE=1
        ELSE IF(HTYPE.EQ.'NTHERM') THEN
           ITYPE=2
        ELSE
           GO TO 115
        ENDIF
        NDEX=(NPART+NTYPE)*MULT+IT
        NMAT=IA(NDEX)
        NDEX=NDEX+NTYPE
        NINP=IA(NDEX)
        NDEX=NDEX+NTYPE
        NING=IA(NDEX)
        NDEX=NDEX+NTYPE
        NOUTP=IA(NDEX)
        NDEX=NDEX+NTYPE
        NOUTG=IA(NDEX)
        NDEX=NDEX+NTYPE
        LOCT=IA(NDEX)
*----
*  DATA TYPE CONTROL
*----
        IREC=LOCT+IRZT
        NWDS=(2+MULT)*NMAT+NINP+NOUTP+1
        IF(L2+NWDS-1.GT.MAXA)  CALL XABORT
     >    ('LIBTR1: LENGTH OF CURRENT RECORD > MAXA ')
*--------------------------------------
      CALL XDREED(NIN,IREC,RA(L2),NWDS)
*--------------------------------------
        IF((NWDS/2)*2.NE.NWDS) NWDS=NWDS+1
        LMC=L2+NWDS
        LMCH=L2H+NWDS/MULT
        NSBLK=IA(L2+NMAT*(MULT+2)+NINP+NOUTP)
        IRZM=IREC+1
*----
*  READ THROUGH MATXS FILE AND ACCUMULATE CROSS SECTIONS
*  FOR THIS RANGE OF MATS, LEGENDRE ORDERS, AND GROUPS.
*----
        DO 120 IM=1,NMAT
          WRITE(HMAT,FORM) DA(L2H-1+IM)
*----
*  MATERIAL/ISOTOPE LOOP
*----
 130      CONTINUE
          DO 140 IMX=1,NBISO
            IF(ITISOM(IMX).EQ.ILIB) THEN
              IMT=IMX
              WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,IMT),ITC=1,3)
              WRITE(HNISOR,'(3A4)') (ISONRF(ITC,IMT),ITC=1,3)
              WRITE(HCOH,'(A4,A2)') (ICOHNA(ITC,IMT),ITC=1,2)
              WRITE(HINC,'(A4,A2)') (IINCNA(ITC,IMT),ITC=1,2)
              IF(NTFG(IMX).EQ.0) IPR(2,IMX)=1
              IF((HMAT.EQ.HNISOR(:6)).AND.
     >           (IPR(ITYPE,IMX).EQ.0)) GO TO 145
            ENDIF
 140      CONTINUE
          GO TO 125
*----
*  MATERIAL CONTROL
*----
 145      CONTINUE
          IPR(ITYPE,IMT)=1
          CALL LCMSIX(IPLIB,HNAMIS,1)
          IF(ITYPE.EQ.2) THEN
            IF(IPR(1,IMT).EQ.0) CALL XABORT
     >        ('LIBTR1: NSCAT DATA IS MISSING.')
            CALL XDRLGS(IPLIB,-1,NPROC,0,NL-1,1,NGRO,INDPRO,ITYPRO,
     >                  XSREC,SCAT,GAR,NJJ,IJJ)
          ELSE
            CALL XDRSET(XSREC,NGRO*NPROC,0.0)
            CALL XDRSET(SCAT,NGRO*NGRO*NL,0.0)
          ENDIF
*----
*  MASS RATIO OF EACH MATERIAL/ISOTOPE IN THE CALCULATION DOMAIN:
*----
          LOC=L2-1+MULT*NMAT+IM
          NSUBM=IA(LOC)
          LOCM=IA(LOC+NMAT)
          IREC=LOCM+IRZM
          NWDS=MULT+1+6*NSUBM
          IF(LMC+NWDS-1.GT.MAXA) CALL XABORT
     >      ('LIBTR1: LENGTH OF CURRENT RECORD > MAXA ')
*-------------------------------------------
          CALL XDREED(NIN,IREC,RA(LMC),NWDS)
*-------------------------------------------
          AW(IMT)=RA(LMC+MULT)
          NWDS=NWDS+MULT-1
          L3=LMC+NWDS
          L3H=LMCH+NWDS/MULT
          CALL SETARA(RBASE,NSUBM*NGRO,IXTERP)
          ITERP=IXTERP
          CALL SETARA(RBASE,NSUBM,ITEMP)
          CALL SETARA(RBASE,NSUBM,ISIGZ)
          DO 160 ISUBM=1,NSUBM
            RBASE(ITEMP+ISUBM-1)=RA(LMC+MULT+6*(ISUBM-1)+1)
            RBASE(ISIGZ+ISUBM-1)=RA(LMC+MULT+6*(ISUBM-1)+2)
 160      CONTINUE
          CALL LIBTER(NGRO,NSUBM,RBASE(ITEMP),RBASE(ISIGZ),
     >                TN(IMT),SN(1,IMT),RBASE(ITERP))
          CALL RLSARA(RBASE(ISIGZ))
          CALL RLSARA(RBASE(ITEMP))
*----
*  TEMPERATURE AND BACKGROUND LOOP
*----
          DO 170 ISUBM=1,NSUBM
            LOC=LMC+MULT+6*(ISUBM-1)
            TMAT=RA(LOC+1)
            SMAT=RA(LOC+2)
            LOCS=IA(LOC+6)
            LSUBM1=(ISUBM.EQ.1)
            IF(.NOT.LSUBM1) THEN
              LTERP=.TRUE.
              DO 171 IK=1,NGRO
                LTERP=LTERP.AND.
     >               (RBASE(ITERP+NGRO*(ISUBM-1)+IK-1).EQ.0.0)
 171          CONTINUE
              IF(LTERP) GO TO 175
            ENDIF
*----
*  PROCESS THIS SUBMATERIAL
*----
            LOC=LMC+MULT+6*(ISUBM-1)
            N1DR=IA(LOC+3)
            N1DB=IA(LOC+4)
            N2DB=IA(LOC+5)
            JREC=IREC+LOCS
            IF(N1DR.EQ.0) GO TO 180
            NWDS=(3+MULT)*N1DR
            JREC=JREC+1
            IF(L3+NWDS-1.GT.MAXA) CALL XABORT
     >        ('LIBTR1: LENGTH OF CURRENT RECORD > MAXA ')
*-------------VECTOR CONTROL-----------------
            CALL XDREED(NIN,JREC,RA(L3),NWDS)
*--------------------------------------------
            NEX1=L3-1+MULT*N1DR
            NEX2=NEX1+N1DR
            NEX3=NEX2+N1DR
            IF(LSUBM1.AND.(IMPX.GE.10)) THEN
              WRITE(IOUT,870) HTYPE,HMAT,(DA(L3H+IR-1),IR=1,N1DR)
            ENDIF
*----
*  VECTOR PARTIALS
*----
            IF(LSUBM1) THEN
              IFTOT=0
*----
*  IF NF IS PRESENT, SET IFTOT=1 AND USE NF+NNF+N2NF+N3NF
*----
              DO 190 IR=1,N1DR
                WRITE(HVPS,FORM) DA(L3H-1+IR)
                CALL XDRCAS('LOWTOUP',HVPS)
                IF(HVPS.EQ.'NF') IFTOT=1
 190          CONTINUE
              CALL XDRSET(XSREC(1,NDPROC-1),2*NGRO,0.0)
            ENDIF
*----
*  LOOP OVER REACTIONS
*----
            IB=0
            DO 200 IR=1,N1DR
              IBLK=IA(NEX1+IR)
              IF(IBLK.GT.IB) THEN
                NWDS=0
*-----
*  MANY VECTORS (REACTIONS) ARE STORED IN BLOCK IBLK.
*-----
                DO 210 IJ=1,N1DR
                  IF(IA(NEX1+IJ).EQ.IBLK) THEN
                    NWDS=NWDS+IA(NEX3+IJ)-IA(NEX2+IJ)+1
                  ENDIF
 210            CONTINUE
                JREC=JREC+1
                CALL SETARA(RBASE,NWDS,IXS)
*-----------------VECTOR BLOCK-----------------------
                CALL XDREED(NIN,JREC,RBASE(IXS),NWDS)
*----------------------------------------------------
                IB=IBLK
                L5=IXS-1
              ENDIF
              WRITE(HVPS,FORM) DA(L3H-1+IR)
              CALL XDRCAS('LOWTOUP',HVPS)
              NK=IA(NEX3+IR)-IA(NEX2+IR)+1
*----
*  PROCESS REQUIRED EXTRA EDIT.
*----
              DO 220 I=1,NED
                IF(HVPS.EQ.HVECT(I).AND.IVECT(I).NE.0) THEN
                  IXSR=IVECT(I)
                  DO 221 IK=1,NK
                    IF(RBASE(L5+IK).NE.0.0) THEN
                      JJ=IA(NEX2+IR)+IK-1
                      TERPZ=1.0
                      IF(.NOT.LSUBM1)
     >                  TERPZ=RBASE(ITERP+NGRO*(ISUBM-1)+JJ-1)
                      XSREC(JJ,IXSR)=XSREC(JJ,IXSR)+TERPZ*RBASE(L5+IK)
                    ENDIF
 221              CONTINUE
                  GO TO 225
                ENDIF
 220          CONTINUE
*----
*  SAVE MODEL WEIGHT FUNCTIONS
*----
 225          CONTINUE
              IF((HTYPE.EQ.'NSCAT').AND.(HVPS.EQ.'NWT0').AND.
     >           LSUBM1) THEN
                DO 230 IK=1,NK
                  JJ=IA(NEX2+IR)+IK-1
                  XSREC(JJ,16)=RBASE(L5+IK)
 230            CONTINUE
                GO TO 205
              ENDIF
              IF((HTYPE.EQ.'NTHERM').AND.
     >           (HVPS.NE.HINC).AND.(HVPS.NE.HCOH)) GO TO 205
*----
*  LOOP OVER GROUPS
*----
              DO 240 IK=1,NK
                IF(RBASE(L5+IK).EQ.0.0) GO TO 245
                JJ=IA(NEX2+IR)+IK-1
                LTHERM=(JJ.GE.NGRO-NTFG(IMT)+1)
                LTIME=(ITIME.EQ.1)
*----
*  INTERPOLATION FACTOR
*----
                TERPZ=1.0
                IF(.NOT.LSUBM1)
     >            TERPZ=RBASE(ITERP+NGRO*(ISUBM-1)+JJ-1)
                IF(ABS(TERPZ).LT.1.0E-3) GO TO 245
                ADD=TERPZ*RBASE(L5+IK)
                IF(HVPS.EQ.'NTOT0') THEN
*----
*  TOTAL XSEC
*----
                  XSREC(JJ,1)=XSREC(JJ,1)+ADD
                ELSE IF((.NOT.LSUBM1).AND.(HVPS.EQ.'NFTOT')) THEN
*----
*  FISSION CROSS SECTION
*----
                  XSREC(JJ,3)=XSREC(JJ,3)+ADD*XSREC(JJ,19)
                ELSE IF(LSUBM1.AND.(HVPS.EQ.'NFTOT')) THEN
*----
*  TOTAL FISSION
*----
                  XSREC(JJ,20)=XSREC(JJ,20)+ADD
                ELSE IF(LSUBM1.AND.(HVPS.EQ.'NFSLO')) THEN
*----
*  SLOW FISSION
*----
                  XSREC(JJ,3)=XSREC(JJ,3)+ADD
                  XSREC(JJ,19)=XSREC(JJ,19)+ADD
                  IF(IK.EQ.1) CNORM(IMT,2)=0.0
                  CNORM(IMT,2)=CNORM(IMT,2)+ADD*XSREC(JJ,16)
                ELSE IF(LSUBM1.AND.(HVPS.EQ.'CHIS')) THEN
*----
*  SLOW FISSION
*----
                  IF(CNORM(IMT,2).EQ.0.0) THEN
                    WRITE(HSMG,1050) HMAT
                    CALL XABORT(HSMG)
                  ENDIF
                  ADDD=CNORM(IMT,2)*RBASE(L5+IK)
                  CNORM(IMT,1)=CNORM(IMT,1)+ADDD
                  XSREC(JJ,5)=XSREC(JJ,5)+ADDD
                ELSE IF(LSUBM1.AND.LTIME.AND.(HVPS.EQ.'NUDEL')) THEN
*----
*  DELAYED FISSION
*----
                  XSREC(JJ,3)=XSREC(JJ,3)+ADD*XSREC(JJ,20)
                  XSREC(JJ,19)=XSREC(JJ,19)+XSREC(JJ,20)*ADD
                  IF(IK.EQ.1) CNORM(IMT,3)=0.0
                  CNORM(IMT,3)=CNORM(IMT,3)
     >                        +ADD*XSREC(JJ,20)*XSREC(JJ,16)
                ELSE IF(LSUBM1.AND.(HVPS.EQ.'CHID')) THEN
*----
*  DELAYED FISSION
*----
                  IF(CNORM(IMT,3).EQ.0.0) THEN
                    WRITE(HSMG,1060) HMAT
                    CALL XABORT(HSMG)
                  ENDIF
                  ADDD=CNORM(IMT,3)*RBASE(L5+IK)
                  CNORM(IMT,1)=CNORM(IMT,1)+ADDD
                  XSREC(JJ,5)=XSREC(JJ,5)+ADDD
                ELSE IF(LTHERM.AND.(HVPS.EQ.'NELAS')) THEN
*----
*  THERMAL CORRECTION TO TOTAL CROSS SECTION
*----
                  XSREC(JJ,1)=XSREC(JJ,1)-ADD
                ELSE IF(LTHERM.AND.(HVPS.EQ.HINC)) THEN
*----
*  THERMAL CORRECTION TO TOTAL CROSS SECTION
*----
                  XSREC(JJ,1)=XSREC(JJ,1)+ADD
                ELSE IF(LTHERM.AND.(HVPS.EQ.HCOH)) THEN
*----
*  THERMAL CORRECTION TO TOTAL CROSS SECTION
*----
                  XSREC(JJ,1)=XSREC(JJ,1)+ADD
                ENDIF
 245            CONTINUE
 240          CONTINUE
*----
*  END OF REACTION LOOP
*----
 205          CONTINUE
              L5=L5+NK
              IF(L5.EQ.IXS+NWDS-1) THEN
                CALL RLSARA(RBASE(IXS))
              ENDIF
 200        CONTINUE
*----
*  SCATTERING MATRIX CONTROL
*----
 180        CONTINUE
            IF(N2DB.EQ.0) GO TO 175
            DO 250 K=1,N2DB
              NWDS=MULT+2+2*NOUTG
              IF(L3+NWDS-1.GT.MAXA) CALL XABORT
     >          ('LIBTR1: LENGTH OF CURRENT RECORD > MAXA ')
              JREC=JREC+1
*---------------MATRIX CONTROL-----------------
              CALL XDREED(NIN,JREC,RA(L3),NWDS)
*----------------------------------------------
              LORD=IA(L3+MULT+1)
              IF(LORD.EQ.0) GO TO 255
              WRITE(HMTX,FORM) DA(L3H)
              CALL XDRCAS('LOWTOUP',HMTX)
              LONE=IA(L3+MULT)
              LN=L3+MULT+1
              LG=LN+NOUTG
              IFISN=0
              IF((HTYPE.EQ.'NSCAT').AND.
     >          (HMTX.EQ.'NF'.OR.HMTX.EQ.'NNF'.OR.
     >           HMTX.EQ.'N2NF'.OR.HMTX.EQ.'N3NF')) IFISN=1
              IF((HTYPE.EQ.'NSCAT').AND.(HMTX.EQ.'NFTOT')) IFISN=2
*----
*  SCATTERING SUB-BLOCKS
*----
              INC=(NOUTG-1)/NSBLK+1
              DO 260 J=1,NSBLK
                NWDS=0
                DO 270 JJ=(J-1)*INC+1,MIN(J*INC,NOUTG)
                  NWDS=NWDS+IA(LN+JJ)
 270            CONTINUE
                IF(NWDS.EQ.0) GO TO 265
                NWDS=NWDS*LORD
                CALL SETARA(RBASE,NWDS,IXS)
                JREC=JREC+1
*-----------------MATRIX SUB-BLOCK-------------------
                CALL XDREED(NIN,JREC,RBASE(IXS),NWDS)
*----------------------------------------------------
                IF(IFTOT.EQ.1.AND.IFISN.EQ.2) THEN
                  CALL RLSARA(RBASE(IXS))
                  GO TO 265
                ENDIF
*----
*  STORE DESIRED CROSS SECTIONS
*----
                IF((HTYPE.EQ.'NTHERM').AND.
     >              (HMTX.NE.HINC).AND.(HMTX.NE.HCOH)) GO TO 310
                L5=IXS-1
*----
*  LOOP OVER SINK, ORDER, SOURCE
*----
                DO 320 JJ=(J-1)*INC+1,MIN(J*INC,NOUTG)
                  NP=IA(LN+JJ)
                  IF(NP.EQ.0) GO TO 325
                  DO 330 IL=1,LORD
                    ILNOW=IL+LONE
                    IF(ILNOW.GT.NL) GO TO 335
                    DO 340 IP=1,NP
                      XSNOW=RBASE(L5+IP+NP*(IL-1))
                      IF(XSNOW.EQ.0.) GO TO 345
                      JJP=IA(LG+JJ)-IP+1
*----
*  THERMAL CORRECTION TO SCATTERING MATRIX
*----
                      IF((HMTX.EQ.'NELAS').AND.
     >                   (JJP.GE.NGRO-NTFG(IMT)+1)) GO TO 345
                      IF(((HMTX.EQ.HINC).OR.(HMTX.EQ.HCOH)).AND.
     >                   (JJP.LT.NGRO-NTFG(IMT)+1)) GO TO 345
*----
*  INTERPOLATION FACTOR
*----
                      TERPZ=1.0
                      IF(.NOT.LSUBM1)
     >                  TERPZ=RBASE(ITERP+NGRO*(ISUBM-1)+JJP-1)
                      IF(ABS(TERPZ).LT.1.0E-3) GO TO 345
                      XSEC=TERPZ*XSNOW
*----
*  CHECK FOR FISSION MATRICES
*----
                      IF(IFISN.EQ.0) THEN
*----
*  SCATTERING MATRIX AND TOTAL SCATTERING VECTOR
*  SCAT(SECONDARY,PRIMARY,ORDER+1)
*----
                        SCAT(JJ,JJP,ILNOW)=SCAT(JJ,JJP,ILNOW)+XSEC
                        XSREC(JJP,NDPROC+ILNOW)=
     >                    XSREC(JJP,NDPROC+ILNOW)+XSEC
*----
* FISSION VECTORS
*----
                      ENDIF
                      IF(ILNOW.NE.1) GO TO 345
                      IF(IFTOT.EQ.1.AND.IFISN.NE.1) GO TO 345
                      IF(IFTOT.EQ.0.AND.IFISN.NE.2) GO TO 345
                      XSREC(JJP,3)=XSREC(JJP,3)+XSEC
                      CNORM(IMT,1)=CNORM(IMT,1)+XSEC*XSREC(JJP,16)
                      XSREC(JJ,5)=XSREC(JJ,5)+XSEC*XSREC(JJP,16)
 345                  CONTINUE
 340                CONTINUE
 335                CONTINUE
 330              CONTINUE
                  L5=L5+NP*LORD
 325              CONTINUE
 320            CONTINUE
*----
* ACCUMULATE FISSION NUBAR
*----
 310            CONTINUE
                IF(LSUBM1.AND.(HTYPE.EQ.'NSCAT')) THEN
                  IF(IFTOT.EQ.1.AND.IFISN.NE.1) GO TO 350
                  IF(IFTOT.EQ.0.AND.IFISN.NE.2) GO TO 350
                  L5=IXS-1
                  DO 360 JJ=(J-1)*INC+1,MIN(J*INC,NOUTG)
                    NP=IA(LN+JJ)
                    IF(NP.NE.0) THEN
                      DO 361 IP=1,NP
                        JJP=IA(LG+JJ)-IP+1
                        XSREC(JJP,19)=XSREC(JJP,19)+RBASE(L5+IP)
 361                  CONTINUE
                    ENDIF
                    L5=L5+NP*LORD
 360             CONTINUE
                ENDIF
 350            CONTINUE
                CALL RLSARA(RBASE(IXS))
 265            CONTINUE
 260          CONTINUE
              HGAR(MOD(K-1,18)+1)=HMTX
              IF((K.EQ.1).AND.LSUBM1.AND.(IMPX.GE.10)) THEN
                WRITE(IOUT,880) HTYPE,HMAT
              ENDIF
              IF((MOD(K-1,18).EQ.17).AND.LSUBM1.AND.(IMPX.GE.10)) THEN
                WRITE(IOUT,885) (HGAR(I)//' ',I=1,18)
              ELSE IF((K.EQ.N2DB).AND.LSUBM1.AND.(IMPX.GE.10)) THEN
                WRITE(IOUT,885) (HGAR(I)//' ',I=1,MOD(N2DB-1,18)+1)
              ENDIF
 255          CONTINUE
 250        CONTINUE
*----
*  SAVE FISSION NU FOR SHIELDING TERMS
*----
            IF(LSUBM1.AND.(HTYPE.EQ.'NSCAT')) THEN
              DO 370 JJ=1,NGRO
                IF(XSREC(JJ,20).NE.0) THEN
                  XSREC(JJ,19)=XSREC(JJ,19)/XSREC(JJ,20)
                ENDIF
 370          CONTINUE
            ENDIF
*----
*  END OF SUBMATERIAL LOOP
*----
 175        CONTINUE
 170      CONTINUE
          CALL RLSARA(RBASE(IXTERP))
*----
*   SAVE MATRICES ON LCM
*----
          CALL XDRLGS(IPLIB,1,NPROC,0,NL-1,1,NGRO,INDPRO,ITYPRO,
     >                XSREC,SCAT,GAR,NJJ,IJJ)
          CALL LCMSIX(IPLIB,' ',2)
          GO TO 130
*----
*  END OF MATERIAL AND DATA TYPE LOOPS
*----
 125      CONTINUE
 120    CONTINUE
 115    CONTINUE
 110  CONTINUE
*----
*  CLOSE MATXS FILE.
*----
      IER=KDRCLS(NIN,1)
      IF(IER.LT.0) THEN
        WRITE(HSMG,9001) CFILNA
        CALL XABORT(HSMG)
      ENDIF
*----
*  CHECK IF ALL NBISO ISOTOPES HAVE BEEN PROCESSED.
*----
      NISOT=0
      DO 380 I=1,NBISO
        IF(ITISOM(I).EQ.ILIB) THEN
          IF((IPR(1,I).EQ.0).OR.(IPR(2,I).EQ.0)) THEN
            WRITE(IOUT,910) (ISONAM(ITC,I),ITC=1,3),CFILNA
            NISOT=NISOT+1
          ENDIF
        ENDIF
  380 CONTINUE
      IF(NISOT.GT.0) CALL XABORT('LIBTR1: MISSING ISOTOPES')
*----
*  PERFORM LIVOLANT-JEANPIERRE NORMALIZATION AND SAVE CROSS SECTION
*  INFORMATION ON LCM.
*----
      DO 390 IMT=1,NBISO
        IF(ITISOM(IMT).EQ.ILIB) THEN
          WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,IMT),ITC=1,3)
          CALL LCMSIX(IPLIB,HNAMIS,1)
*----
*  READ CROSS SECTION
*----
          CALL XDRLGS(IPLIB,-1,NPROC,0,NL-1,1,NGRO,INDPRO,ITYPRO,
     >                XSREC,SCAT,GAR,NJJ,IJJ)
*----
*  COMPUTE NORMALIZATION FUNCTION
*----
          DO 400 IG1=1,NGRO
            IF((SN(IG1,IMT).NE.SB(IG1,IMT)).AND.
     >         (SN(IG1,IMT).LT.1.0E10)) THEN
              GAR(IG1)=1.0/(1.0+(XSREC(IG1,1)-XSREC(IG1,NDPROC+1))
     >               *(1.0/SN(IG1,IMT)-1.0/SB(IG1,IMT)))
            ELSE
              GAR(IG1)=1.0
            ENDIF
            XSREC(IG1,17)=XSREC(IG1,16)
            IF(SN(IG1,IMT).LT.1.0E10) THEN
              XSREC(IG1,16)=SN(IG1,IMT)/(SN(IG1,IMT)+XSREC(IG1,1)
     >                     -XSREC(IG1,NDPROC+1))/GAR(IG1)
            ELSE
              XSREC(IG1,16)=1.0
            ENDIF
 400      CONTINUE
          IF(IMPX.GE.10) THEN
            WRITE(IOUT,920) HNAMIS
            WRITE(IOUT,928) (GAR(IG1),IG1=1,NGRO)
            SUM=0.0
            DO 410 IG1=1,NGRO
              SUM=SUM+XSREC(IG1,17)
 410        CONTINUE
            WRITE(IOUT,927) HNAMIS,SUM
            WRITE(IOUT,928) (XSREC(IG1,17),IG1=1,NGRO)
          ENDIF
*----
*  NORMALIZE VECTOR CROSS SECTION
*----
          DO 420 IPRO=1,NPROC
            IF(IPRO.NE.5.AND.IPRO.NE.16) THEN
              IF(INDPRO(IPRO).EQ.1.AND.ITYPRO(IPRO).EQ.1) THEN
                DO 430 IG1=1,NGRO
                  XSREC(IG1,IPRO)=XSREC(IG1,IPRO)*GAR(IG1)
 430            CONTINUE
              ENDIF
            ENDIF
 420      CONTINUE
          IF(CNORM(IMT,1).NE.0.0.AND.INDPRO(5).EQ.1) THEN
*----
*  FISSION SOURCE (CHI) NORMALIZATION
*----
            DO 440 IG1=1,NGRO
              XSREC(IG1,5)=XSREC(IG1,5)/CNORM(IMT,1)
 440        CONTINUE
          ENDIF
*----
*  NORMALIZE SCATTERING CROSS SECTION
*----
          DO 450 IL=1,NL
            IF(INDPRO(NDPROC+IL).EQ.1) THEN
              DO 451 IG1=1,NGRO
                DO 452 IG2=1,NGRO
                  SCAT(IG1,IG2,IL)=SCAT(IG1,IG2,IL)*GAR(IG2)
 452            CONTINUE
 451          CONTINUE
            ENDIF
 450      CONTINUE
*----
*  COMPUTE TRANSPORT CORRECTION
*----
          IF(NL.GE.2.AND.ITRANC.GE.1) THEN
            DO 460 IG1=1,IG4EV
              XSREC(IG1,2)=0.0
              DO 461 IG2=1,NGRO
                XSREC(IG1,2)=XSREC(IG1,2)
     >            +SCAT(IG1,IG2,2)*DELTA(IG2)/DELTA(IG1)
 461          CONTINUE
 460        CONTINUE
            DO 470 IG1=IG4EV+1,NGRO
              XSREC(IG1,2)=XSREC(IG1,NDPROC+2)
 470        CONTINUE
          ENDIF
*----
*  COMPUTE NG CROSS SECTION
*----
          IF(INDPRO(7).EQ.1.AND.ITYPRO(7).EQ.0) THEN
            DO 480 IG1=1,NGRO
              XSREC(IG1,7)=XSREC(IG1,1)-XSREC(IG1,NDPROC+1)
     >                    -XSREC(IG1,4)
 480        CONTINUE
          ENDIF
*----
*  SAVE CROSS SECTION
*----
          CALL XDRLGS(IPLIB,1,NPROC,0,NL-1,1,NGRO,INDPRO,ITYPRO,
     >                XSREC,SCAT,GAR,NJJ,IJJ)
          WRITE(README(:8),'(A8)') HNAMIS(1:8)
          READ(README,'(22A4)') (IHGAR(I),I=1,22)
          CALL LCMPUT(IPLIB,'README',22,3,IHGAR)
          CALL LCMSIX(IPLIB,' ',2)
        ENDIF
  390 CONTINUE
      RETURN
*----
*  PRINT FORMATS
*----
  870 FORMAT(/52H AVAILABLE IDENTIFIERS OF REACTION VECTORS FOR TYPE ,
     1 A6,14H AND MATERIAL ,A6,1H:/(1X,18A7))
  880 FORMAT(/53H AVAILABLE IDENTIFIERS OF REACTION MATRICES FOR TYPE ,
     1 A6,14H AND MATERIAL ,A6,1H:)
  885 FORMAT(1X,18A7)
  890 FORMAT(/32H PROCESSING MATXS LIBRARY NAMED ,A8,1H.)
  910 FORMAT(/27H LIBTR1: MATERIAL/ISOTOPE ',3A4,16H' IS MISSING ON ,
     1 16HMATXS FILE NAME ,A8,1H.)
  920 FORMAT(/40H L-J NORMALIZATION FACTORS FOR MATERIAL ,A12)
  927 FORMAT(/19H FLUX FOR MATERIAL ,A12,7H   SUM=,1P,E12.5)
  928 FORMAT(1X,1P,10E12.4)
  930 FORMAT(1X,15I4)
  935 FORMAT(/16H MATXS FILE ID: ,3A6,6H VERS ,I2)
 1050 FORMAT(35HLIBTR1: SNORM MISSING FOR MATERIAL ,A6,1H.)
 1060 FORMAT(35HLIBTR1: DNORM MISSING FOR MATERIAL ,A6,1H.)
*----
*  ABORT FORMATS
*----
 9000 FORMAT('LIBTR1: UNABLE TO OPEN MATXS LIBRARY FILE ',A8)
 9001 FORMAT('LIBTR1: UNABLE TO CLOSE MATXS LIBRARY FILE ',A8)
      END
