*DECK LIBTR2
      SUBROUTINE LIBTR2(IPLIB,IMPX,CFILNA,NGRO,NBISO,NL,ISONAM,ISONRF,
     >                  ICOHNA,IINCNA,NTFG,TN,AW,SN,SB,ILIB,ITISOM,
     >                  DELTA,NGFR,ITRANC,NED,HVECT,IVECT,ITIME,NPROC,
     >                  INDPRO,ITYPRO,XSREC,SCAT,IPR,GAR,NJJ,IJJ,XSMAT)
C
C-----------------------------------------------------------------------
C
C TRANSCRIPTION OF THE USEFULL INTERPOLATED MICROSCOPIC CROSS SECTION
C DATA FROM MATXS TO LCM. USE MATXS FORMAT FROM NJOY-91.
C
C INPUT PARAMETERS:
C  IPLIB   : POINTER TO THE LATTICE MICROSCOPIC CROSS SECTION LIBRARY
C            (L_LIBRARY SIGNATURE).
C  IMPX    : PRINT FLAG.
C  CFILNA  : CHARACTER*8 NAME OF THE LIBRARY FILE.
C  NGRO    : NUMBER OF ENERGY GROUPS.
C  NBISO   : NUMBER OF ISOTOPES PRESENT IN THE CALCULATION DOMAIN.
C  NL      : NUMBER OF LEGENDRE ORDERS REQUIRED IN THE CALCULATION.
C            NL=1 OR HIGHER.
C  ISONAM  : ALIAS NAME OF ISOTOPES. ISONAM(3,NBISO)
C  ISONRF  : REFERENCE NAME OF ISOTOPES. ISONRF(3,NBISO)
C  ICOHNA  : HCOH NAME. ICOHNA(2,NBISO)
C  IINCNA  : HINC NAME. IINCNA(2,NBISO)
C  NTFG    : NUMBER OF THERMAL GROUPS WHERE THE THERMAL INELASTIC
C            CORRECTION IS APPLIED. NTFG(NBISO)
C  TN      : TEMPERATURE OF EACH ISOTOPE. DIMENSION TN(NBISO)
C  AW      : ATOMIC WEIGHT. AW(NBISO)
C  SN      : DILUTION CROSS SECTION IN EACH ENERGY GROUP OF EACH
C            ISOTOPE. SN(NGRO,NBISO)
C  SB      : DILUTION CROSS SECTION AS USED BY LIVOLANT AND JEANPIERRE
C            NORMALIZATION. SB(NGRO,NBISO)
C  ILIB    : LIBRARY TO PROCESS
C  ITISOM  : ISOTOPE WITH INDEX ILIB IS PROCESSED. ITISOM(NBISO)
C  DELTA   : LETHARGY. DELTA(NGRO)
C  NGFR    : NUMBER OF FAST+RESONANCE GROUPS.
C  ITRANC  : TRANSPORT CORRECTION
C            0 -> NONE ; 1 -> APOL; 2 -> WIMS
C  NED     : NUMBER OF EXTRA VECTOR EDITS FROM MATXS.
C  HVECT   : NAMES OF THE EXTRA VECTOR EDITS.
C            DIMENSION CHARACTER HVECT(NED)*6
C            MATXS RESERVED NAMES:
C            NWT0/NWT1   : P0/P1 LIBRARY WEIGHT FUNCTION.
C            NTOT0/NTOT1 : P0/P1 NEUTRON TOTAL CROSS SECTIONS.
C            NELAS : NEUTRON ELASTIC SCATTERING CROSS SECTION.
C            NINEL : NEUTRON INELASTIC SCATTERING CROSS SECTION.
C            NG    : RADIATIVE CAPTURE CROSS SECTION.
C            NFTOT : TOTAL FISSION CROSS SECTION.
C            NUDEL : NUMBER OF DELAYED SECONDARY NEUTRONS (NU-D).
C            NFSLO : NU * SLOW FISSION CROSS SECTION.
C            CHIS/CHID : SLOW/DELAYED FISSION SPECTRUM.
C            NF/NNF/N2NF/N3NF : NU * PARTIAL FISSION CROSS SECTIONS.
C            N2N/N3N/N4N : (N,2N),(N,3N),(N,4N) CROSS SECTIONS.
C  IVECT   : POSITION OF EXTRA EDIT IN XSREC VECTOR IVECT(NED)
C  ITIME   : =1 : STEADY-STATE / =2 : PROMPT.
C  NPROC   : NUMBER OF CROSS SECTION TYPE PROCESSED
C  INDPRO  : FLAG FOR CROSS SECTION TYPE TO PROCESS INDPRO(NPROC)
C SCRATCH STORAGE:
C  ITYPRO  : FLAG FOR CROSS SECTION TYPE PROCESSED ITYPRO(NPROC)
C  XSREC   : DIMENSION XSREC(NGRO,NPROC)
C  SCAT    : DIMENSION SCAT(NGRO,NGRO,NL)
C  IPR     : DIMENSION IPR(NBISO)
C  GAR     : DIMENSION GAR(NGRO*NGRO)
C  NJJ     : DIMENSION NJJ(NGRO)
C  IJJ     : DIMENSION IJJ(NGRO)
C  XSMAT   : DIMENSION XSMAT(NGRO,NGRO,NL)
C
C SUBROUTINES CALLED BY LIBTR2:
C LIBTE2, XDREED, LCM---.
C
C REFERENCE:
C R. E. MACFARLANE, TRANSX 2: A CODE FOR INTERFACING MATXS CROSS-
C SECTION LIBRARIES TO NUCLEAR TRANSPORT CODES, LOS ALAMOS NATIONAL
C LABORATORY, REPORT LA-12312-MS, NEW MEXICO, JULY 1992.
C
C-------------------------------------- AUTHOR:  A. HEBERT 05/12/91 ----
C-------------------------------------MODIFIED: A. ARONSON 97/02/01 ----
C-------------------------------------MODIFIED: G. MARLEAU 97/06/27 ----
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER       IOUT,NDPROC,MULT,MAXA
      CHARACTER     FORM*4
      PARAMETER    (IOUT=6,NDPROC=20,MULT=2,MAXA=1000,FORM='(A6)')
C----
C INTEFAFCE VARIABLES
C----
      INTEGER       IPLIB,IMPX,NGRO,NBISO,NL,ISONAM(3,NBISO),
     1              ISONRF(3,NBISO),ICOHNA(2,NBISO),IINCNA(2,NBISO),
     2              NTFG(NBISO),ILIB,ITISOM(NBISO),NGFR,ITRANC,NED,
     3              IVECT(NED),ITIME,NPROC,INDPRO(NPROC),ITYPRO(NPROC),
     4              IPR(NBISO),NJJ(NGRO),IJJ(NGRO)
      REAL          TN(NBISO),AW(NBISO),SN(NGRO,NBISO),
     1              SB(NGRO,NBISO),DELTA(NGRO),
     2              XSREC(NGRO,NPROC),SCAT(NGRO,NGRO,NL),
     3              GAR(NGRO*NGRO),XSMAT(NGRO,NGRO,NL)
      CHARACTER     CFILNA*8,HVECT(NED)*6
C----
C MEMORY ALLOCATION
C----
      INTEGER       IBASE(1)
      REAL          RBASE(1)
      COMMON        RBASE
      EQUIVALENCE  (RBASE(1),IBASE(1))
      INTEGER       IXTERP,ITEMP,ISIGZ,IXS,IWORK
C----
C EXTERNAL FUNCTIONS
C----
      INTEGER       KDROPN,KDRCLS
C----
C  LOCAL VARIABLES
C----
      INTEGER       NIN,IREC,NWDS,IVER,NPART,NTYPE,NHOLL,I,L2,L2H,
     1              NMAT,MAXW,NEX1,NG,IG4EV,IRZM,IM,IMX,ITC,IMT,LOC,
     2              NSUB,LOCM,L3,L3H,ITERP,ISUBM,NSUB0,IJ,IR,IRMAX,
     3              NOUTG,IK,NING,LOCG,LOCS,IOLDTY,N1D,N2D,JOUTP,JREC,
     4              NEX2,IJ0,NW,L5,NK,IXSR,JJ,ITYPE,N0,JINP,K,LORD,LN,
     5              LG,IFISN,JCONST,NOUMAX,NOUMIN,NP,IL,IP,JJP,JJP0,IG1,
     6              IG2,IER,NISOT,IPRO
      INTEGER       IA(MAXA),IHGAR(22)
C
      REAL          SMAT,TMAT,ADD,DNORM,TERPZ,ADDD,CNORM,SPEC,XSNOW,
     1              SUM,XSEC
      REAL          RA(MAXA)
      CHARACTER     HSMG*131,HNAMIS*12,HNISOR*12,README*88,
     1              HN*6,HU*6,HS*6,HPRT*6,HTYPE*6,HMAT*6,HVPS*6,
     2              HMTX*6,HGAR(18)*6,HINC*6,HCOH*6
      LOGICAL       LSUBM1,LTHERM,LTIME,LTERP
      DOUBLE PRECISION DA(MAXA/2)
      EQUIVALENCE  (RA(1),IA(1),DA(1))
C----
C  INITIALIZE IPR AND OPEN MATXS2 FILE
C  INITIALIZE LIBRARY
C----
      CALL XDRSET(IPR,NBISO,0)
      IF(IMPX.GT.0) WRITE(IOUT,890) CFILNA
      NIN=KDROPN(CFILNA,2,2,0,0)
      IF(NIN.LE.0) THEN
        WRITE(HSMG,9000) CFILNA
        CALL XABORT(HSMG)
      ENDIF
      IREC=1
      NWDS=1+3*MULT
*-------FILE IDENTIFICATION--------
      CALL XDREED(NIN,IREC,RA,NWDS)
*----------------------------------
      WRITE(HN,FORM) DA(1)
      WRITE(HU,FORM) DA(2)
      WRITE(HS,FORM) DA(3)
      IVER=IA(1+3*MULT)
      IF(IMPX.GT.0) WRITE(IOUT,935) HN,HU,HS,IVER
      IREC=2
      NWDS=6
*-------FILE CONTROL---------------
      CALL XDREED(NIN,IREC,RA,NWDS)
*----------------------------------
      NPART=IA(1)
      NTYPE=IA(2)
      NHOLL=IA(3)
      NMAT=IA(4)
      MAXW=IA(5)
      IREC=3
      NWDS=NHOLL*MULT
      IF(NWDS.GT.MAXA) CALL XABORT
     >  ('LIBTR2: LENGTH OF RECORD 3 > MAXA ')
*-------HOLLERITH IDENTIFICATION---
      CALL XDREED(NIN,IREC,RA,NWDS)
*----------------------------------
      WRITE(README(9:),'(6H FROM ,9A8)') (DA(I),I=1,MIN(NHOLL,9))
      IF(IMPX.GT.0) WRITE(IOUT,'(1X,9A8)') (DA(I),I=1,MIN(NHOLL,9))
      IREC=4
      NWDS=(NPART+NTYPE+NMAT)*MULT+2*NTYPE+NPART+2*NMAT
      IF(NWDS.GT.MAXA) CALL XABORT
     >  ('LIBTR2: LENGTH OF RECORD 4 > MAXA ')
*-------FILE DATA------------------
      CALL XDREED(NIN,IREC,RA,NWDS)
*----------------------------------
      IF((NWDS/2)*2.NE.NWDS) NWDS=NWDS+1
      L2=1+NWDS
      L2H=1+NWDS/MULT
*----
*  CHECK GROUP STRUCTURES
*----
      NEX1=(NPART+NTYPE+NMAT)*MULT
      DO 100 I=1,NPART
        WRITE(HPRT,FORM) DA(I)
        CALL XDRCAS('LOWTOUP',HPRT)
        NG=IA(NEX1+I)
        IF(HPRT.EQ.'N'.AND.NG.NE.NGRO)
     >    CALL XABORT('LIBTR2: INCONSISTENT GROUP STRUCTURES.')
        IREC=IREC+1
 100  CONTINUE
      IG4EV=0
      IF(ITRANC.GE.2) THEN
        IG4EV=NGFR
      ENDIF
*----
*  READ THROUGH MATXS2 FILE AND ACCUMULATE CROSS SECTIONS
*  FOR THIS RANGE OF MATS, LEGENDRE ORDERS, AND GROUPS.
*----
      IRZM=IREC+1
      DO 120 IM=1,NMAT
*----
*  MATERIAL/ISOTOPE LOOP
*----
 130    CONTINUE
        CNORM=0.0
        WRITE(HMAT,FORM) DA(NPART+NTYPE+IM)
        DO 140 IMX=1,NBISO
          IF(ITISOM(IMX).EQ.ILIB) THEN
            IMT=IMX
            WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,IMT),ITC=1,3)
            WRITE(HNISOR,'(3A4)') (ISONRF(ITC,IMT),ITC=1,3)
            WRITE(HCOH,'(A4,A2)') (ICOHNA(ITC,IMT),ITC=1,2)
            WRITE(HINC,'(A4,A2)') (IINCNA(ITC,IMT),ITC=1,2)
            IF((HMAT.EQ.HNISOR(1:6)).AND.
     >         (IPR(IMX).EQ.0)) GO TO 145
          ENDIF
 140    CONTINUE
        GO TO 125
*----
* MATERIAL CONTROL
*----
 145    CONTINUE
        CALL XDRSET(ITYPRO,NPROC,0)
        CALL XDRSET(XSREC,NGRO*NPROC,0.0)
        CALL XDRSET(SCAT,NGRO*NGRO*NL,0.0)
        IPR(IMT)=1
        CALL LCMSIX(IPLIB,HNAMIS,1)
        LOC=(NPART+NTYPE+NMAT)*MULT+NPART+2*NTYPE+IM
        NSUB=IA(LOC)
        LOCM=IA(LOC+NMAT)
        IREC=LOCM+IRZM
        NWDS=MULT+1+6*NSUB
        IF(L2+NWDS-1.GT.MAXA) CALL XABORT
     >    ('LIBTR2: LENGTH OF CURRENT RECORD > MAXA ')
*---------MATERIAL CONTROL---------------
        CALL XDREED(NIN,IREC,RA(L2),NWDS)
*----------------------------------------
        AW(IMT)=RA(L2+MULT)
        IF((NWDS/2)*2.NE.NWDS) NWDS=NWDS+1
        L3=L2+NWDS
        L3H=L2H+NWDS/MULT
        CALL SETARA(RBASE,NSUB*NGRO,IXTERP)
        ITERP=IXTERP
        CALL SETARA(RBASE,NSUB,ITEMP)
        CALL SETARA(RBASE,NSUB,ISIGZ)
        DO 160 ISUBM=1,NSUB
          RBASE(ITEMP+ISUBM-1)=RA(L2+MULT+6*(ISUBM-1)+1)
          RBASE(ISIGZ+ISUBM-1)=RA(L2+MULT+6*(ISUBM-1)+2)
 160    CONTINUE
        NSUB0=0
        DO 161 ITYPE=1,NTYPE
          N0=0
          DO 162 ISUBM=1,NSUB
            IF(IA(L2+MULT+6*(ISUBM-1)+3).EQ.ITYPE) N0=N0+1
 162      CONTINUE
          CALL SETARA(RBASE,N0,IWORK)
          CALL LIBTE2(NGRO,N0,RBASE(ITEMP+NSUB0),RBASE(ISIGZ+NSUB0),
     >                TN(IMT),SN(1,IMT),RBASE(ITERP+NSUB0*NGRO),
     >                RBASE(IWORK))
          CALL RLSARA(RBASE(IWORK))
          NSUB0=NSUB0+N0
 161    CONTINUE
        IF(NSUB0.NE.NSUB) CALL XABORT('LIBTR2: DATA TYPE FAILURE.')
        CALL RLSARA(RBASE(ISIGZ))
        CALL RLSARA(RBASE(ITEMP))
*----
*  TEMPERATURE AND BACKGROUND LOOP
*----
        IOLDTY=0
        DO 170 ISUBM=1,NSUB
          LOC=L2+MULT+6*(ISUBM-1)
          TMAT=RA(LOC+1)
          SMAT=RA(LOC+2)
          ITYPE=IA(LOC+3)
          LSUBM1=(ITYPE.NE.IOLDTY)
          IOLDTY=ITYPE
          N1D=IA(LOC+4)
          N2D=IA(LOC+5)
          LOCS=IA(LOC+6)
          LOCG=(NPART+NTYPE+NMAT)*MULT
          JINP=IA(LOCG+NPART+ITYPE)
          NING=IA(LOCG+JINP)
          JOUTP=IA(LOCG+NPART+NTYPE+ITYPE)
          NOUTG=IA(LOCG+JOUTP)
          WRITE(HPRT,FORM) DA(JINP)
          WRITE(HTYPE,FORM) DA(NPART+ITYPE)
          CALL XDRCAS('LOWTOUP',HTYPE)
          IF(IMPX.GE.10) WRITE(IOUT,860) ISUBM,HPRT,HTYPE,HMAT
          IF(HTYPE.NE.'NSCAT'.AND.HTYPE.NE.'NTHERM') GO TO 175
          IF(.NOT.LSUBM1) THEN
            LTERP=.TRUE.
            DO 171 IK=1,NGRO
              LTERP=LTERP.AND.
     >             (RBASE(ITERP+NGRO*(ISUBM-1)+IK-1).EQ.0.0)
 171        CONTINUE
            IF(LTERP) GO TO 175
          ENDIF
*----
*  PROCESS THIS SUBMATERIAL
*----
          JREC=IREC+LOCS
          IF(N1D.EQ.0) GO TO 180
          NWDS=(2+MULT)*N1D
          JREC=JREC+1
          IF(L3+NWDS-1.GT.MAXA) CALL XABORT
     >      ('LIBTR2: LENGTH OF CURRENT RECORD > MAXA ')
*-----------VECTOR CONTROL-----------------
          CALL XDREED(NIN,JREC,RA(L3),NWDS)
*------------------------------------------
          NEX1=L3-1+MULT*N1D
          NEX2=NEX1+N1D
          IF(LSUBM1.AND.(IMPX.GE.10)) THEN
            WRITE(IOUT,870) HTYPE,HMAT,(DA(L3H+IR-1),IR=1,N1D)
          ENDIF
*----
*  LOOP OVER REACTIONS
*----
          IRMAX=0
          DO 200 IR=1,N1D
            IF(IR.GT.IRMAX) THEN
              NWDS=0
              IJ0=IRMAX+1
              DO 210 IJ=IJ0,N1D
                NW=IA(NEX2+IJ)-IA(NEX1+IJ)+1
                IF(NWDS+NW.GE.MAXW) GO TO 215
                IRMAX=IRMAX+1
                NWDS=NWDS+NW
 210          CONTINUE
 215          CONTINUE
              IF(NWDS.EQ.0) CALL XABORT
     >          ('LIBTR2: LENGTH OF CURRENT RECORD = 0 ')
              CALL SETARA(RBASE,NWDS,IXS)
              JREC=JREC+1
*---------------VECTOR BLOCK-----------------------
              CALL XDREED(NIN,JREC,RBASE(IXS),NWDS)
*--------------------------------------------------
              L5=IXS-1
            ENDIF
            WRITE(HVPS,FORM) DA(L3H-1+IR)
            CALL XDRCAS('LOWTOUP',HVPS)
            NK=IA(NEX2+IR)-IA(NEX1+IR)+1
*----
*  SAVE REQUIRED EXTRA EDIT.
*----
            DO 220 I=1,NED
              IF(HVPS.EQ.HVECT(I).AND.IVECT(I).NE.0) THEN
                IXSR=IVECT(I)
                ITYPRO(IXSR)=1
                DO 221 IK=1,NK
                  IF(RBASE(L5+IK).NE.0.0) THEN
                    JJ=IA(NEX1+IR)+IK-1
                    TERPZ=1.0
                    IF(.NOT.LSUBM1)
     >                TERPZ=RBASE(ITERP+NGRO*(ISUBM-1)+JJ-1)
                    XSREC(JJ,IXSR)=XSREC(JJ,IXSR)+TERPZ*RBASE(L5+IK)
                  ENDIF
 221            CONTINUE
                GO TO 225
              ENDIF
 220        CONTINUE
*----
*  SAVE MODEL WEIGHT FUNCTIONS
*----
 225        CONTINUE
            IF(HTYPE.EQ.'NSCAT'.AND.HVPS.EQ.'NWT0'.AND.LSUBM1) THEN
              ITYPRO(16)=1
              DO 230 IK=1,NK
                JJ=IA(NEX1+IR)+IK-1
                XSREC(JJ,16)=RBASE(L5+IK)
 230          CONTINUE
              GO TO 205
            ENDIF
            IF(HTYPE.EQ.'NTHERM'.AND.
     >         (HVPS.NE.HINC).AND.(HVPS.NE.HCOH)) GO TO 205
*----
*  LOOP OVER GROUPS
*----
            DO 240 IK=1,NK
              IF(RBASE(L5+IK).EQ.0.0) GO TO 245
              JJ=IA(NEX1+IR)+IK-1
              LTHERM=(JJ.GE.NGRO-NTFG(IMT)+1)
              LTIME=(ITIME.EQ.1)
*----
*  INTERPOLATION FACTOR
*----
              TERPZ=1.0
              IF(.NOT.LSUBM1)
     >          TERPZ=RBASE(ITERP+NGRO*(ISUBM-1)+JJ-1)
              IF(ABS(TERPZ).LT.1.0E-3) GO TO 245
              ADD=TERPZ*RBASE(L5+IK)
              IF(HVPS.EQ.'NTOT0') THEN
*----
*  TOTAL XSEC
*----
                XSREC(JJ,1)=XSREC(JJ,1)+ADD
              ELSE IF((.NOT.LSUBM1).AND.HVPS.EQ.'NFTOT') THEN
*----
*  FISSION CROSS SECTION
*----
                XSREC(JJ,3)=XSREC(JJ,3)+ADD*XSREC(JJ,19)
              ELSE IF(LSUBM1.AND.HVPS.EQ.'NFTOT') THEN
*----
*  TOTAL FISSION
*----
                XSREC(JJ,20)=XSREC(JJ,20)+ADD
              ELSE IF(LSUBM1.AND.LTIME.AND.HVPS.EQ.'NUDEL') THEN
*----
*  DELAYED FISSION
*----
                XSREC(JJ,3)=XSREC(JJ,3)+ADD*XSREC(JJ,20)
                XSREC(JJ,19)=XSREC(JJ,19)+XSREC(JJ,20)*ADD
                IF(IK.EQ.1) DNORM=0.0
                DNORM=DNORM+ADD*XSREC(JJ,20)*XSREC(JJ,16)
              ELSE IF(LSUBM1.AND.HVPS.EQ.'CHID') THEN
*----
*  DELAYED FISSION
*----
                IF(DNORM.EQ.0.0) THEN
                  WRITE(HSMG,1060) HMAT
                  CALL XABORT(HSMG)
                ENDIF
                ADDD=DNORM*RBASE(L5+IK)
                CNORM=CNORM+ADDD
                XSREC(JJ,5)=XSREC(JJ,5)+ADDD
              ELSE IF(LTHERM.AND.HVPS.EQ.'NELAS') THEN
*----
*  THERMAL CORRECTION TO TOTAL CROSS SECTION
*----
                XSREC(JJ,1)=XSREC(JJ,1)-ADD
              ELSE IF(LTHERM.AND.(HVPS.EQ.HINC)) THEN
*----
*  THERMAL CORRECTION TO TOTAL CROSS SECTION
*----
                XSREC(JJ,1)=XSREC(JJ,1)+ADD
              ELSE IF(LTHERM.AND.(HVPS.EQ.HCOH)) THEN
*----
*  THERMAL CORRECTION TO TOTAL CROSS SECTION
*----
                XSREC(JJ,1)=XSREC(JJ,1)+ADD
              ENDIF
 245          CONTINUE
 240        CONTINUE
*----
*  END OF REACTION LOOP
*----
 205        CONTINUE
            L5=L5+NK
            IF(L5.EQ.IXS+NWDS-1) CALL RLSARA(RBASE(IXS))
 200      CONTINUE
*----
* SCATTERING MATRIX CONTROL
*----
 180      CONTINUE
          IF(N2D.EQ.0) GO TO 175
          DO 250 K=1,N2D
            NWDS=MULT+2+2*NOUTG
            IF(L3+NWDS-1.GT.MAXA) CALL XABORT
     >        ('LIBTR2: LENGTH OF CURRENT RECORD > MAXA ')
            JREC=JREC+1
*-------------MATRIX CONTROL-----------------
            CALL XDREED(NIN,JREC,RA(L3),NWDS)
*--------------------------------------------
            LORD=IA(L3+MULT)
            IF(LORD.EQ.0) GO TO 255
            WRITE(HMTX,FORM) DA(L3H)
            CALL XDRCAS('LOWTOUP',HMTX)
            LN=L3+MULT+1
            LG=LN+NOUTG
            IFISN=0
            IF(HTYPE.EQ.'NSCAT'.AND.
     >         (HMTX.EQ.'NF'.OR.HMTX.EQ.'NNF'.OR.
     >          HMTX.EQ.'N2NF'.OR.HMTX.EQ.'N3NF')) IFISN=1
            IF(HTYPE.EQ.'NSCAT'.AND.HMTX.EQ.'NFTOT') IFISN=2
            JCONST=IA(L3+MULT+1)
*----
* SCATTERING SUB-BLOCKS
*----
            IF(NING.NE.NOUTG) CALL XABORT
     >        ('LIBTR2: MATRIX REACTION MUST BE SQUARE ')
            CALL XDRSET(XSMAT,NGRO*NGRO*NL,0.0)
            NOUMAX=0
 260        CONTINUE
            NWDS=0
            NOUMIN=NOUMAX+1
            DO 270 JJ=NOUMIN,NOUTG
              NW=IA(LN+JJ)*LORD
              IF(NWDS+NW.GE.MAXW) GO TO 275
              NOUMAX=NOUMAX+1
              NWDS=NWDS+NW
 270        CONTINUE
 275        CONTINUE
            IF(NWDS.GT.0) THEN
              CALL SETARA(RBASE,NWDS,IXS)
              JREC=JREC+1
*---------------MATRIX SUB-BLOCK-------------------
              CALL XDREED(NIN,JREC,RBASE(IXS),NWDS)
*--------------------------------------------------
              L5=IXS-1
              DO 280 JJ=NOUMIN,NOUMAX
                NP=IA(LN+JJ)
                IF(NP.NE.0) THEN
                  DO 281 IL=1,LORD
                    IF(IL.LE.NL) THEN
                      DO 282 IP=1,NP
                        JJP=IA(LG+JJ)-IP+1
                        XSMAT(JJP,JJ,IL)=RBASE(L5+IP+NP*(IL-1))
 282                  CONTINUE
                    ENDIF
 281              CONTINUE
                ENDIF
                L5=L5+NP*LORD
 280          CONTINUE
              CALL RLSARA(RBASE(IXS))
            ENDIF
            IF(NOUMAX.LT.NOUTG) GO TO 260
            IF(JCONST.NE.0) THEN
              IF(LORD.GT.1) CALL XABORT
     >          ('LIBTR2: INVALID VALUE FOR LORD ON MATXS2')
              NWDS=NOUTG+JCONST
              CALL SETARA(RBASE,NWDS,IXS)
              JREC=JREC+1
*---------------CONSTANT SUB-BLOCK-----------------
              CALL XDREED(NIN,JREC,RBASE(IXS),NWDS)
*--------------------------------------------------
              L5=IXS-1
              DO 290 JJ=1,NOUTG
                SPEC=RBASE(L5+JJ)
                JJP0=NING-JCONST+1
                DO 291 JJP=JJP0,NING
                  XSMAT(JJP,JJ,1)=XSMAT(JJP,JJ,1)
     >                           +SPEC*RBASE(L5+NOUTG+JJP-JJP0+1)
 291            CONTINUE
 290          CONTINUE
              CALL RLSARA(RBASE(IXS))
            ENDIF
*----
* STORE DESIRED CROSS SECTIONS
*----
            IF(HTYPE.EQ.'NTHERM'.AND.
     >          HMTX.NE.HINC.AND.HMTX.NE.HCOH) GO TO 310
*----
* LOOP OVER SINK, ORDER, SOURCE
*----
            DO 320 JJ=1,NOUTG
              DO 330 IL=1,LORD
                IF(IL.GT.NL) GO TO 335
                DO 340 JJP=1,NING
                  XSNOW=XSMAT(JJP,JJ,IL)
                  IF(XSNOW.EQ.0.) GO TO 345
*----
*  THERMAL CORRECTION TO SCATTERING MATRIX
*----
                  IF((HMTX.EQ.'NELAS').AND.
     >               (JJP.GE.NGRO-NTFG(IMT)+1)) GO TO 345
                  IF(((HMTX.EQ.HINC).OR.(HMTX.EQ.HCOH)).AND.
     >                (JJP.LT.NGRO-NTFG(IMT)+1)) GO TO 345
*----
*  INTERPOLATION FACTOR
*----
                  TERPZ=1.0
                  IF(.NOT.LSUBM1)
     >              TERPZ=RBASE(ITERP+NGRO*(ISUBM-1)+JJP-1)
                  IF(ABS(TERPZ).LT.1.0E-3) GO TO 345
                  XSEC=TERPZ*XSNOW
*----
*  CHECK FOR FISSION MATRICES
*----
                  IF(IFISN.EQ.0) THEN
*----
*  SCATTERING MATRIX AND TOTAL SCATTERING VECTOR
*  SCAT(SECONDARY,PRIMARY,ORDER+1)
*----
                    SCAT(JJ,JJP,IL)=SCAT(JJ,JJP,IL)+XSEC
                    XSREC(JJP,NDPROC+IL)=XSREC(JJP,NDPROC+IL)+XSEC
                  ELSE IF((IL.EQ.1).AND.(IFISN.NE.0).AND.
     >                    (HTYPE.EQ.'NSCAT')) THEN
*----
*  FISSION VECTORS
*----
                    XSREC(JJP,3)=XSREC(JJP,3)+XSEC
                    CNORM=CNORM+XSEC*XSREC(JJP,16)
                    XSREC(JJ,5)=XSREC(JJ,5)+XSEC*XSREC(JJP,16)
                  ENDIF
 345              CONTINUE
 340            CONTINUE
 335            CONTINUE
 330          CONTINUE
 320        CONTINUE
*----
*  ACCUMULATE FISSION NUBAR
*----
 310        CONTINUE
            IF(LSUBM1.AND.(IFISN.NE.0).AND.(HTYPE.EQ.'NSCAT')) THEN
              DO 360 JJ=1,NOUTG
                DO 361 JJP=1,NING
                  XSREC(JJP,19)=XSREC(JJP,19)+XSMAT(JJP,JJ,1)
 361            CONTINUE
 360          CONTINUE
            ENDIF
            HGAR(MOD(K-1,18)+1)=HMTX
            IF((K.EQ.1).AND.LSUBM1.AND.(IMPX.GE.10)) THEN
              WRITE(IOUT,880) HTYPE,HMAT
            ENDIF
            IF((MOD(K-1,18).EQ.17).AND.LSUBM1.AND.(IMPX.GE.10)) THEN
              WRITE(IOUT,885) (HGAR(I)//' ',I=1,18)
            ELSE IF((K.EQ.N2D).AND.LSUBM1.AND.(IMPX.GE.10)) THEN
              WRITE(IOUT,885) (HGAR(I)//' ',I=1,MOD(N2D-1,18)+1)
            ENDIF
 255        CONTINUE
 250      CONTINUE
*----
*  SAVE FISSION NU FOR SHIELDING TERMS
*----
          IF(LSUBM1.AND.(HTYPE.EQ.'NSCAT')) THEN
            DO 370 JJ=1,NGRO
              IF(XSREC(JJ,20).NE.0) THEN
                XSREC(JJ,19)=XSREC(JJ,19)/XSREC(JJ,20)
              ENDIF
 370        CONTINUE
          ENDIF
*----
*  END OF SUBMATERIAL LOOP
*----
 175      CONTINUE
 170    CONTINUE
        CALL RLSARA(RBASE(IXTERP))
*----
*  PERFORM LIVOLANT-JEANPIERRE NORMALIZATION
*  AND SAVE CROSS SECTION INFORMATION ON LCM
*----
        DO 400 IG1=1,NGRO
          IF((SN(IG1,IMT).NE.SB(IG1,IMT)).AND.
     >       (SN(IG1,IMT).LT.1.0E10)) THEN
            GAR(IG1)=1.0/(1.0+(XSREC(IG1,1)-XSREC(IG1,NDPROC+1))
     >      *(1.0/SN(IG1,IMT)-1.0/SB(IG1,IMT)))
          ELSE
            GAR(IG1)=1.0
          ENDIF
          XSREC(IG1,17)=XSREC(IG1,16)
          IF(SN(IG1,IMT).LT.1.0E10) THEN
            XSREC(IG1,16)=SN(IG1,IMT)/(SN(IG1,IMT)+XSREC(IG1,1)
     >                   -XSREC(IG1,NDPROC+1))/GAR(IG1)
          ELSE
            XSREC(IG1,16)=1.0
          ENDIF
 400    CONTINUE
        IF(IMPX.GE.10) THEN
          WRITE(IOUT,920) HNAMIS
          WRITE(IOUT,928) (GAR(IG1),IG1=1,NGRO)
          SUM=0.0
          DO 410 IG1=1,NGRO
            SUM=SUM+XSREC(IG1,17)
 410      CONTINUE
          WRITE(IOUT,927) HNAMIS,SUM
          WRITE(IOUT,928) (XSREC(IG1,17),IG1=1,NGRO)
        ENDIF
*----
*  NORMALIZE VECTOR CROSS SECTION
*----
        DO 420 IPRO=1,NPROC
          IF(IPRO.NE.5.AND.IPRO.NE.16) THEN
            IF(INDPRO(IPRO).EQ.1) THEN
              DO 430 IG1=1,NGRO
                XSREC(IG1,IPRO)=XSREC(IG1,IPRO)*GAR(IG1)
 430          CONTINUE
            ENDIF
          ENDIF
 420    CONTINUE
        IF(CNORM.NE.0.0.AND.INDPRO(5).EQ.1) THEN
*----
*  FISSION SOURCE (CHI) NORMALIZATION
*----
          DO 440 IG1=1,NGRO
            XSREC(IG1,5)=XSREC(IG1,5)/CNORM
 440      CONTINUE
        ENDIF
*----
*  NORMALIZE SCATTERING CROSS SECTION
*----
        DO 450 IL=1,NL
          IF(INDPRO(NDPROC+IL).EQ.1) THEN
            DO 451 IG1=1,NGRO
              DO 452 IG2=1,NGRO
                SCAT(IG1,IG2,IL)=SCAT(IG1,IG2,IL)*GAR(IG2)
 452          CONTINUE
 451        CONTINUE
          ENDIF
 450    CONTINUE
*----
*  COMPUTE TRANSPORT CORRECTION
*----
        IF(NL.GE.2.AND.ITRANC.GE.1) THEN
          DO 460 IG1=1,IG4EV
            XSREC(IG1,2)=0.0
            DO 461 IG2=1,NGRO
              XSREC(IG1,2)=XSREC(IG1,2)
     >          +SCAT(IG1,IG2,2)*DELTA(IG2)/DELTA(IG1)
 461        CONTINUE
 460      CONTINUE
          DO 470 IG1=IG4EV+1,NGRO
            XSREC(IG1,2)=XSREC(IG1,NDPROC+2)
 470      CONTINUE
        ENDIF
*----
*  COMPUTE NG CROSS SECTION
*----
        IF(INDPRO(7).EQ.1.AND.ITYPRO(7).EQ.0) THEN
          DO 480 IG1=1,NGRO
            XSREC(IG1,7)=XSREC(IG1,1)-XSREC(IG1,NDPROC+1)
     >                  -XSREC(IG1,4)
 480      CONTINUE
        ENDIF
*----
*  SAVE CROSS SECTION
*----
        CALL XDRLGS(IPLIB,1,NPROC,0,NL-1,1,NGRO,INDPRO,ITYPRO,
     >              XSREC,SCAT,GAR,NJJ,IJJ)
        WRITE(README(:8),'(A8)') HNAMIS(1:8)
        READ(README,'(22A4)') (IHGAR(I),I=1,22)
        CALL LCMPUT(IPLIB,'README',22,3,IHGAR)
        CALL LCMSIX(IPLIB,' ',2)
        GO TO 130
*----
*  END OF MATERIAL/ISOTOPE LOOP.
*----
 125    CONTINUE
 120  CONTINUE
*----
*  CLOSE MATXS2 FILE.
*----
      IER=KDRCLS(NIN,1)
      IF(IER.LT.0) THEN
        WRITE(HSMG,9001) CFILNA
        CALL XABORT(HSMG)
      ENDIF
*----
*  CHECK IF ALL NBISO ISOTOPES HAVE BEEN PROCESSED.
*----
      NISOT=0
      DO 380 IMT=1,NBISO
        IF(ITISOM(IMT).EQ.ILIB) THEN
          IF(IPR(IMT).EQ.0) THEN
            WRITE(IOUT,910) (ISONAM(ITC,IMT),ITC=1,3),CFILNA
            NISOT=NISOT+1
          ENDIF
        ENDIF
 380  CONTINUE
      IF(NISOT.GT.0) CALL XABORT('LIBTR2: MISSING ISOTOPES')
      RETURN
*----
*  PRINT FORMATS
*----
 860  FORMAT(/31H LIBTR2: PROCESSING SUBMATERIAL,I5,5X,12HINCIDENT PAR,
     > 6HTICLE=,A6,5X,10HDATA TYPE=,A6,5X,9HMATERIAL=,A6)
 870  FORMAT(/52H AVAILABLE IDENTIFIERS OF REACTION VECTORS FOR TYPE ,
     > A6,14H AND MATERIAL ,A6,1H:/(1X,18A7))
 880  FORMAT(/53H AVAILABLE IDENTIFIERS OF REACTION MATRICES FOR TYPE ,
     > A6,14H AND MATERIAL ,A6,1H:)
 885  FORMAT(1X,18A7)
 890  FORMAT(/33H PROCESSING MATXS2 LIBRARY NAMED ,A8,1H.)
 910  FORMAT(/27H LIBTR2: MATERIAL/ISOTOPE ',3A4,16H' IS MISSING ON ,
     > 16HMATXS FILE NAME ,A8,1H.)
 920  FORMAT(/40H L-J NORMALIZATION FACTORS FOR MATERIAL ,A12)
 927  FORMAT(/19H FLUX FOR MATERIAL ,A12,7H   SUM=,1P,E12.5)
 928  FORMAT(1X,1P,10E12.4)
 930  FORMAT(1X,15I4)
 935  FORMAT(/17H MATXS2 FILE ID: ,3A6,6H VERS ,I2)
 1060 FORMAT(35HLIBTR2: DNORM MISSING FOR MATERIAL ,A6,1H.)
*----
*  ABORT FORMATS
*----
 9000 FORMAT('LIBTR2: UNABLE TO OPEN MATX2 LIBRARY FILE ',A8)
 9001 FORMAT('LIBTR2: UNABLE TO CLOSE MATX2 LIBRARY FILE ',A8)
      END
