*DECK LIBWED
      SUBROUTINE LIBWED(MAXR,NEL,NDEPL,NDFI,NDFP,NHEAVY,NLIGHT,
     >                  NBXR,ITNAM,ITZEA,MATNO,KPAX,BPAX,
     >                  HNADPL,IZEA,IDR,RER,KPAR,BPAR,YIELD)
C
C-------------------------- LIBWED   ----------------------------------
C
C  PROGRAMME STATISTICS:
C     NAME     : LIBWED
C     ENTRY    : LIBWED
C     USE      : CREATE DRAGON RECORDS FROM WIMS-NEA AND WIMS-AECL
C                DATA
C     MODIFIED : 97-01-08
C     AUTHOR   : G. MARLEAU
C
C  ROUTINE PARAMETERS:
C   INPUT
C     MAXR   : NUMBER OF REACTION TYPES             I
C     NEL    : NUMBER OF ISOTOPES ON LIBRARY        I
C     NDEPL  : NUMBER OF DEPLETING ISOTOPES         I
C     NDFI   : NUMBER OF DIRECT FISSILE ISOTOPES    I
C     NDFP   : NUMBER OF DIRECT FISSION PRODUCT     I
C     NHEAVY : NUMBER OF HEAVY ISOTOPES             I
C              FISSILE ISOTOPES + DECAY + CAPTURE
C              ISOTOPES
C     NLIGHT : NUMBER OF LIGHT ISOTOPES             I
C              FISSION PRODUCT + DECAY + CAPTURE
C              ISOTOPES
C     NBXR   : MAXIMIM NUMBER OF PARENT ISOPOPES    I
C              FROM DECAY AND CAPTURE
C     ITNAM  : REACTIVE ISOTOPE NAMES IN  CHAIN     I(2,NEL)
C              -> READ(NAME,'(2A4)')
C                 HNAME(1,I),HNAME(2,I)
C     ITZEA  : 6-DIGIT NUCLIDE IDENTIFIER           I(NEL)
C              ATOMIC NUMBER Z*10000 (DIGITS) +
C              MASS NUMBER A*10 +
C              ENERGY STATE (0 = GROUND STATE,
C              1 = FOR FIRST STATE, ETC.)
C     MATNO  : REACTION MATERIAL NUMBER             I(NEL)
C     KPAX   : COMPLETE REACTION TYPE MATRIX        I(NEL+MAXR,NEL)
C     BPAX   : COMPLETE BRANCHING RATIO MATRIX      R(NEL+MAXR,NEL)
C   OUTPUT
C     HNADPL : REACTIVE ISOTOPE NAMES IN  CHAIN     I(2,NDEPL)
C              -> READ(NAME,'(2A4)')
C                 HNAME(1,I),HNAME(2,I)
C     IZEA   : 6-DIGIT NUCLIDE IDENTIFIER           I(NDEPL)
C              ATOMIC NUMBER Z*10000 (DIGITS) +
C              MASS NUMBER A*10 +
C              ENERGY STATE (0 = GROUND STATE,
C              1 = FOR FIRST STATE, ETC.)
C     IDR    : REACTION IDENTIFIER                  I(MAXR,NDEPL)
C     RER    : REACTION CONSTANT                    R(MAXR,NDEPL)
C     KPAR   : REDUCED REACTION TYPE MATRIX         I(NBXR,NDEPL)
C     BPAR   : REDUCED BRANCHING RATIO MATRIX       R(NBXR,NDEPL)
C     YIELD  : FISSION PRODUCT YIELD                R(NDFI,NDFP)
C   INTERNAL PARAMETERS
C     KCAPTU : DRAGON CAPTURE FLAG = 3              I
C     KDECAY : DRAGON DECAY FLAG = 1                I
C     KFISSP : DRAGON FISSION PRODUCT FLAG = 2      I
C     KFISSI : DRAGON FISSILE ISOTOPE FLAG = 6      I
C     KN2N   : DRAGON N2N FLAG = 4                  I
C     KN3N   : DRAGON N3N FLAG = 5                  I
C
C--------------------------  LIBWED  ----------------------------------
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER      KCAPTU,KDECAY,KFISSP,KFISSI,KN2N,KN3N
      PARAMETER   (KCAPTU=3,KDECAY=1,KFISSP=2,KFISSI=6,KN2N=4,KN3N=5)
C----
C INTERFACE PARAMETERS
C----
      INTEGER      MAXR,NEL,NDEPL,NDFI,NDFP,NHEAVY,NLIGHT,NBXR
      INTEGER      ITNAM(2,NEL),ITZEA(NEL),MATNO(NEL),
     1             KPAX(NEL+MAXR,NEL),HNADPL(2,NDEPL),IZEA(NDEPL),
     2             IDR(MAXR,NDEPL),KPAR(MAXR,NDEPL)
      REAL         BPAX(NEL+MAXR,NEL),RER(MAXR,NDEPL),
     1             BPAR(MAXR,NDEPL),YIELD(NDFI,NDFP)
C----
C LOCAL VARIABLES
C----
      INTEGER      ISO,ISOHEA,ISOLIG,ISOOTH,ISONUM,IT,IEL,NREAC,JEL,
     1             JSO,IREAC,ITYF,IFI,ITYP,IFP
C----
C  RENUMBER ISOTOPES AND SAVE IDR AND RER
C----
      ISOHEA=0
      ISOLIG=ISOHEA+NHEAVY
      ISOOTH=ISOLIG+NLIGHT
      DO 100 ISO=NEL,1,-1
        ISONUM=0
        IF(MATNO(ISO).EQ.-KFISSI) THEN
          ISOHEA=ISOHEA+1
          ISONUM=ISOHEA
        ELSE IF(MATNO(ISO).EQ.-KFISSP) THEN
          ISOLIG=ISOLIG+1
          ISONUM=ISOLIG
        ELSE IF(MATNO(ISO).EQ.-KDECAY) THEN
          ISOOTH=ISOOTH+1
          ISONUM=ISOOTH
        ENDIF
        IF(ISONUM.GT.0) THEN
          MATNO(ISO)=ISONUM
          HNADPL(1,ISONUM)=ITNAM(1,ISO)
          HNADPL(2,ISONUM)=ITNAM(2,ISO)
          IZEA(ISONUM)=ITZEA(ISO)
          DO 101 IT=1,4
            IDR(IT,ISONUM)=MOD(KPAX(NEL+IT,ISO),10)
            RER(IT,ISONUM)=BPAX(NEL+IT,ISO)
 101      CONTINUE
          IDR(5,ISONUM)=KPAX(NEL+5,ISO)
          RER(5,ISONUM)=BPAX(NEL+5,ISO)
        ENDIF
 100  CONTINUE
C----
C  CREATE KPAR AND BPAR MATRIX
C----
      DO 110 IEL=1,NEL
        ISO=MATNO(IEL)
        IF(ISO.GT.0) THEN
          NREAC=0
          DO 111 JEL=1,NEL
            JSO=MATNO(JEL)
            IF(JSO.GT.0) THEN
              IF(KPAX(IEL,JEL).EQ.KCAPTU.OR.
     >           KPAX(IEL,JEL).EQ.KN2N.OR.
     >           KPAX(IEL,JEL).EQ.KN3N.OR.
     >           KPAX(IEL,JEL).EQ.KDECAY) THEN
                NREAC=NREAC+1
                IF(NREAC.GT.NBXR)
     >            CALL XABORT('LIBWED: TOO MANY DECAY PARENTS')
                KPAR(NREAC,ISO)=JSO*10+KPAX(IEL,JEL)
                BPAR(NREAC,ISO)=BPAX(IEL,JEL)
              ELSE IF(KPAX(IEL,JEL).EQ.KFISSP.AND.
     >                BPAX(IEL,JEL).GT.0.0         ) THEN
                DO 112 IREAC=1,NREAC
                  IF(MOD(KPAR(IREAC,ISO),10).EQ.KFISSP) THEN
                    GO TO 115
                  ENDIF
 112            CONTINUE
                NREAC=NREAC+1
                IF(NREAC.GT.NBXR)
     >            CALL XABORT('LIBWED: TOO MANY DECAY PARENTS')
                IF(ISO.LE.NHEAVY) THEN
                  KPAR(NREAC,ISO)=10+KFISSP
                ELSE
                  KPAR(NREAC,ISO)=20+KFISSP
                ENDIF
 115            CONTINUE
              ENDIF
            ENDIF
 111      CONTINUE
        ENDIF
 110  CONTINUE
C----
C  CREATE YIELD MATRIX
C----
      DO 120 IEL=1,NEL
        ISO=MATNO(IEL)
        IF(ISO.GT.0) THEN
          ITYF=MOD(IDR(5,ISO),10)
          IFI=IDR(5,ISO)/10
          IF(ITYF.EQ.KFISSI.AND.IFI.GT.0) THEN
            IF(IFI.GT.NDFI)
     >        CALL XABORT('LIBWDE: INVALID FISSILE ISOTOPE NUMBER')
            DO 121 JEL=1,NEL
              JSO=MATNO(JEL)
              IF(JSO.GT.0) THEN
                ITYP=MOD(IDR(5,JSO),10)
                IFP=IDR(5,JSO)/10
                IF(ITYP.EQ.KFISSP.AND.IFP.GT.0) THEN
                  IF(KPAX(JEL,IEL) .EQ. 2) THEN
                    IF(IFP.GT.NDFP)
     >                CALL XABORT('LIBWDE: INVALID FISSPROD NUMBER')
                    YIELD(IFI,IFP)=BPAX(JEL,IEL)
                  ENDIF
                ENDIF
              ENDIF
 121        CONTINUE
          ENDIF
        ENDIF
 120  CONTINUE
C----
C  RETURN
C----
      RETURN
      END
