*DECK LIBWET
      SUBROUTINE LIBWET(MAXR,NEL,NSTATE,ISTATE,MATNO,KPAX,BPAX)
C
C-------------------------- LIBWET   ----------------------------------
C
C  PROGRAMME STATISTICS:
C     NAME     : LIBWET
C     ENTRY    : LIBWET
C     USE      : IDENTIFY WIMS-NEA AND WIMS-AECL ISOTOPES
C                BY TYPE AND NAME IN DEPLETION CHAIN
C     MODIFIED : 97-01-08
C     AUTHOR   : G. MARLEAU
C
C  ROUTINE PARAMETERS:
C   INPUT
C     MAXR   : NUMBER OF REACTION TYPES             I
C     NEL    : NUMBER OF ISOTOPES ON LIBRARY        I
C     NSTATE : NUMBER OF PARAMETERS                 I
C   OUTPUT
C     ISTATE : LIBRARY PARAMETERS VECTOR            I(NSTATE)
C     MATNO  : REACTION MATERIAL NUMBER             I(NEL)
C   INPUT/OUTPUT
C     KPAX   : COMPLETE REACTION TYPE MATRIX        I(NEL+MAXR,NEL)
C     BPAX   : COMPLETE BRANCHING RATIO MATRIX      R(NEL+MAXR,NEL)
C   INTERNAL PARAMETERS
C     KCAPTU : DRAGON CAPTURE FLAG = 3              I
C     KDECAY : DRAGON DECAY FLAG = 1                I
C     KFISSP : DRAGON FISSION PRODUCT FLAG = 2      I
C     KFISSI : DRAGON FISSILE ISOTOPE FLAG = 6      I
C     KN2N   : DRAGON N2N FLAG = 4                  I
C     KN3N   : DRAGON N3N FLAG = 5                  I
C
C--------------------------  LIBWET  ----------------------------------
C
      IMPLICIT NONE
C----
C PARAMTERS
C----
      INTEGER      KCAPTU,KDECAY,KFISSP,KFISSI,KN2N,KN3N
      PARAMETER   (KCAPTU=3,KDECAY=1,KFISSP=2,KFISSI=6,KN2N=4,KN3N=5)
C----
C INTERFACE VARIABLES
C----
      INTEGER      MAXR,NEL,NSTATE
      INTEGER      ISTATE(NSTATE),MATNO(NEL),KPAX(NEL+MAXR,NEL)
      REAL         BPAX(NEL+MAXR,NEL)
C----
C LOCAL VARIABLES
C----
      INTEGER      NDFP,JSO,KTYP,KNUM,ISO,ITR,NUNTIL,NHEAVY,
     1             NDFI,IUNTIL,NEW,NLIGHT,NOTHER,NIFP,NDEPL
C----
C  COMPUTE NUMBER OF DIRECT FISSION PRODUCT
C  NUMBER DIRECT FISSION PRODUCT
C----
      NDFI=0
      NDFP=0
      DO 100 JSO=NEL,1,-1
        KTYP=MOD(KPAX(NEL+5,JSO),10)
        KNUM=KPAX(NEL+5,JSO)/10
        IF(KTYP.EQ.KFISSI.AND.KNUM.GT.0) THEN
          NDFI=NDFI+1
          KPAX(NEL+5,JSO)=KFISSI+10*NDFI
        ELSE
          NIFP=0
          DO 101 ISO=NEL,1,-1
            IF(KPAX(JSO,ISO).EQ.KFISSP.AND.
     >         BPAX(JSO,ISO).GT.0.0) THEN
              NIFP=NIFP+1
            ENDIF
 101      CONTINUE
          IF(NIFP.GT.0) THEN
            NDFP=NDFP+1
            KPAX(NEL+5,JSO)=KFISSP+10*NDFP
          ENDIF
        ENDIF
 100  CONTINUE
C----
C  SET MATNO TO HEAVY (-KFISSI) FOR FISSILE ISOTOPES
C  SET MATNO TO LIGHT (-KFISSP) FOR FISSION PRODUCT
C  SET MATNO TO OTHER (-KDECAY) FOR DECAY/CAPTURE/N2N/N3N ISOTOPES
C----
      DO 110 ISO=1,NEL
        IF(MOD(KPAX(NEL+5,ISO),10).EQ.KFISSI) THEN
          MATNO(ISO)=-KFISSI
        ELSE IF(MOD(KPAX(NEL+5,ISO),10).EQ.KFISSP) THEN
          MATNO(ISO)=-KFISSP
        ENDIF
 110  CONTINUE
      DO 112 ISO=1,NEL
        IF(MATNO(ISO).EQ.0) THEN
          DO 113 ITR=1,MAXR
            IF(KPAX(NEL+ITR,ISO).NE.0) THEN
              MATNO(ISO)=-KDECAY
              GO TO 115
            ENDIF
 113      CONTINUE
        ENDIF
 115    CONTINUE
 112  CONTINUE
C----
C  SET MATNO TO HEAVY (-KFISSI) FOR ISOTOPES RESULTING FROM DECAY
C  OR CAPTURE OF HEAVY ISOTOPE UNTIL ALL HEAVY ISOTOPES IDENTIFIED
C----
      NUNTIL=NEL
      NHEAVY=0
      DO 120 IUNTIL=1,NUNTIL
        NEW=0
        DO 121 ISO=NEL,1,-1
          IF(MATNO(ISO).EQ.-KFISSI) THEN
            NHEAVY=NHEAVY+1
            DO 122 JSO=NEL,1,-1
              IF(MATNO(JSO).NE.-KFISSI) THEN
                IF(KPAX(JSO,ISO).EQ.KDECAY.OR.
     >             KPAX(JSO,ISO).EQ.KCAPTU.OR.
     >             KPAX(JSO,ISO).EQ.KN2N.OR.
     >             KPAX(JSO,ISO).EQ.KN3N) THEN
                  NEW=NEW+1
                  MATNO(JSO)=-KFISSI
                ENDIF
              ENDIF
 122        CONTINUE
          ENDIF
 121    CONTINUE
        IF(NEW.EQ.0) GO TO 125
        NHEAVY=0
 120  CONTINUE
 125  CONTINUE
C----
C  SET MATNO TO LIGHT (-KFISSP) FOR ISOTOPES RESULTING FROM DECAY
C  OR CAPTURE OF LIGHT ISOTOPE UNTIL ALL LIGHT ISOTOPES IDENTIFIED
C----
      NUNTIL=NUNTIL-NHEAVY
      NLIGHT=0
      DO 130 IUNTIL=1,NUNTIL
        NEW=0
        DO 131 ISO=NEL,1,-1
          IF(MATNO(ISO).EQ.-KFISSP) THEN
            NLIGHT=NLIGHT+1
            DO 132 JSO=NEL,1,-1
              IF(MATNO(JSO).NE.-KFISSI.AND.
     >           MATNO(JSO).NE.-KFISSP) THEN
                IF(KPAX(JSO,ISO).EQ.KDECAY.OR.
     >             KPAX(JSO,ISO).EQ.KCAPTU.OR.
     >             KPAX(JSO,ISO).EQ.KN2N.OR.
     >             KPAX(JSO,ISO).EQ.KN3N) THEN
                  NEW=NEW+1
                  MATNO(JSO)=-KFISSP
                ENDIF
              ENDIF
 132        CONTINUE
          ENDIF
 131    CONTINUE
        IF(NEW.EQ.0) GO TO 135
        NLIGHT=0
 130  CONTINUE
 135  CONTINUE
C----
C  SET MATNO TO OTHER (-KDECAY) FOR ISOTOPES RESULTING FROM DECAY
C  OR CAPTURE OF OTHER ISOTOPE UNTIL ALL OTHER ISOTOPES IDENTIFIED
C----
      NUNTIL=NUNTIL-NLIGHT
      NOTHER=0
      DO 140 IUNTIL=1,NUNTIL
        NEW=0
        DO 141 ISO=NEL,1,-1
          IF(MATNO(ISO).EQ.-KDECAY) THEN
            NOTHER=NOTHER+1
            DO 142 JSO=NEL,1,-1
              IF(MATNO(JSO).NE.-KFISSI.AND.
     >           MATNO(JSO).NE.-KFISSP.AND.
     >           MATNO(JSO).NE.-KDECAY) THEN
                IF(KPAX(JSO,ISO).EQ.KDECAY.OR.
     >             KPAX(JSO,ISO).EQ.KCAPTU.OR.
     >             KPAX(JSO,ISO).EQ.KN2N.OR.
     >             KPAX(JSO,ISO).EQ.KN3N) THEN
                  NEW=NEW+1
                  MATNO(JSO)=-KDECAY
                ENDIF
              ENDIF
 142        CONTINUE
          ENDIF
 141    CONTINUE
        IF(NEW.EQ.0) GO TO 145
        NOTHER=0
 140  CONTINUE
 145  CONTINUE
      NDEPL=NHEAVY+NLIGHT+NOTHER
      CALL XDRSET(ISTATE,NSTATE,0)
      ISTATE(1)=NDEPL
      ISTATE(2)=NDFI
      ISTATE(3)=NDFP
      ISTATE(4)=NHEAVY
      ISTATE(5)=NLIGHT
      ISTATE(6)=NOTHER
      ISTATE(7)=MAXR
C----
C  RETURN
C----
      RETURN
      END
