*DECK MACIXS
      SUBROUTINE MACIXS(IPMACR,NBMIX ,NGROUP,NANISO,NIFISS,MAXFIS,
     >                  LOLDXS,
     >                  XSTOTL,XSTRAN,XSFISS,XSFIXE,XSSCAT,XSWORK,
     >                  INGSCT,IFGSCT,ISCATA,XEFISS,
     >                  NALBP ,NALBR ,ALBP )
C---------------------------  MACIXS  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : MACIXS
C       USE      : CROSS SECTIONS INITIALIZATION
C       MODIFIED : 2000-01-20 (T. COURAU)
C                  DEAL WITH ADJOINT AND GENERALIZED ADJOINT
C                  FIXED SOURCES: 'FIXE' , 'FIXA' , 'FIXG'
C                  91-01-24
C       AUTHOR   : G.MARLEAU
C
C   2- ROUTINE PARAMETERS:
C    INPUT
C       IPMACR   : POINTER TO THE MACROLIB            I
C       NBMIX    : MAXIMUM NUMBER OF MIXTURES         I
C       NGROUP   : NUMBER OF ENERGY GROUPS            I
C       NANISO   : MAXIMUM LEGENDRE ORDER             I
C                  = 1 ISOTROPIC COLLISION
C                  = 2 LINEARLY ANISOTROPIC COLLISION
C       NIFISS   : NUMBER OF FISSILE ISOTOPES         I
C       MAXFIS   : MAX(1,NIFISS)                      I
C       LOLDXS   : CHECK IF CROSS SECTION TYPE IS     L(NBXS)
C                  ALREADY PRESENT ON THE MACROLIB.
C       XSTOTL   : TOTAL CROSS SECTION OF MIXTURE     R(NBMIX,NGROUP)
C       XSTRAN   : TRANSPORT CORRECTION XS OF MIXTURE R(NBMIX,NGROUP)
C       XSFISS   : FISSION CROSS SECTION OF MIXTURE   R(NBMIX,
C                                                    MAXFIS,NGROUP,3)
C       XSFIXE   : FIXE SOURCES                       R(NBMIX,
C                                                       NGROUP,3)
C       XSSCAT   : SCATT X-SECTION OF MIXTURE/GROUP   R(NBMIX,NANISO,
C                                                      NGROUP,NGROUP)
C       XSWORK   : WORK CROSS-SECTION VECTOR          R(NBMIX*NGROUP)
C       INGSCT   : NB OF SCATTERING GROUP FOR  XS     I(NBMIX)
C       IFGSCT   : FIRST SCATTERING GROUP FOR  XS     I(NBMIX)
C       ISCATA   : CHECK FOR SCATTERING ANISOTROPY    I(NANISO)
C       XEFISS   : AVERAGE ENERGY PER FISSION         R(NBMIX,MAXFIS)
C       NALBP    : Number of physical albedos         I
C       NALBR    : Physical albedos processing        I
C       ALBP     : Physical albedos                   R(0:NALBP,NGROUP)
C
C---------------------------   MACIXS  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN,NBXS
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NBXS=10,NAMSBR='MACIXS')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPMACR,NBMIX,NGROUP,NANISO,NIFISS,MAXFIS
      INTEGER     INGSCT(NBMIX),IFGSCT(NBMIX),ISCATA(NANISO)
      LOGICAL     LOLDXS(NBXS)
      REAL        XSTOTL(NBMIX,NGROUP),XSTRAN(NBMIX,NGROUP),
     >            XSFISS(NBMIX,MAXFIS,NGROUP,3),
     >            XSFIXE(NBMIX,NGROUP,3),
     >            XSSCAT(NGROUP,NBMIX,NANISO,NGROUP),
     >            XSWORK(NBMIX*NGROUP),
     >            XEFISS(NBMIX,MAXFIS)
      INTEGER     NALBP,NALBR
      REAL        ALBP(0:NALBP,NGROUP)
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CGRPNM*12,CANISO*2,NAMREC*12
      INTEGER     IGROUP,JGROUP,IANIS,ILCMLN,ILCMTY,ICMATR,
     >            IMATER,IPWRK,IGD,IGF
      INTEGER     IALB
C----
C  READ/INITIALIZE MACROLIB CROSS SECTION DATA
C----
      CALL XDRSET(XSTOTL,NBMIX*NGROUP,0.0)
      CALL XDRSET(XSFISS,NBMIX*NIFISS*NGROUP*3,0.0)
      CALL XDRSET(XSFIXE,NBMIX*NGROUP*3,0.0)
      CALL XDRSET(XSSCAT,NBMIX*NGROUP*NGROUP*NANISO,0.0)
      CALL XDRSET(XSTRAN,NBMIX*NGROUP,0.0)
      CALL XDRSET(XEFISS,NBMIX       ,0.0)
      CALL LCMLEN(IPMACR,'EFISS',ILCMLN,ILCMTY)
      CALL XDRSET(ALBP,(NALBP+1)*NGROUP,0.0)
      NALBR=0
      IF(ILCMLN .GT. 0) THEN
        LOLDXS(10)=.TRUE.
        CALL LCMGET(IPMACR,'EFISS',XEFISS)
      ENDIF
      DO 200 IGROUP=1,NGROUP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGROUP,NGROUP
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
C----
C  READ OR INITIALISE NUSIGF NFTOT AND CHI
C----
        CALL LCMLEN(IPMACR,'NUSIGF',ILCMLN,ILCMTY)
        IF(ILCMLN .GT. 0) THEN
          LOLDXS(2)=.TRUE.
          CALL LCMGET(IPMACR,'NUSIGF',XSFISS(1,1,IGROUP,1))
        ENDIF
        CALL LCMLEN(IPMACR,'NFTOT',ILCMLN,ILCMTY)
        IF(ILCMLN .GT. 0) THEN
          LOLDXS(9)=.TRUE.
          CALL LCMGET(IPMACR,'NFTOT',XSFISS(1,1,IGROUP,2))
        ENDIF
        CALL LCMLEN(IPMACR,'CHI',ILCMLN,ILCMTY)
        IF(ILCMLN .GT. 0) THEN
          LOLDXS(4)=.TRUE.
          CALL LCMGET(IPMACR,'CHI',XSFISS(1,1,IGROUP,3))
        ENDIF
C----
C  READ OR INITIALISE TOTAL XS AND FIXED SOURCES
C----
        CALL LCMLEN(IPMACR,'TOTAL',ILCMLN,ILCMTY)
        IF(ILCMLN .GT. 0) THEN
          LOLDXS(1)=.TRUE.
          CALL LCMGET(IPMACR,'TOTAL',XSTOTL(1,IGROUP))
        ENDIF
        CALL LCMLEN(IPMACR,'TRANC',ILCMLN,ILCMTY)
        IF(ILCMLN .GT. 0) THEN
          LOLDXS(6)=.TRUE.
          CALL LCMGET(IPMACR,'TRANC',XSTRAN(1,IGROUP))
        ENDIF
C----
C  READ FIXED SOURCES DIRECT, ADJOINT OR
C  GENERALIZED ADJOINT
C----
        CALL LCMLEN(IPMACR,'FIXE',ILCMLN,ILCMTY)
        IF(ILCMLN .GT. 0) THEN
          LOLDXS(3)=.TRUE.
          CALL LCMGET(IPMACR,'FIXE',XSFIXE(1,IGROUP,1))
        ENDIF
        CALL LCMLEN(IPMACR,'FIXA',ILCMLN,ILCMTY)
        IF(ILCMLN .GT. 0) THEN
          LOLDXS(7)=.TRUE.
          CALL LCMGET(IPMACR,'FIXA',XSFIXE(1,IGROUP,2))
        ENDIF
        CALL LCMLEN(IPMACR,'FIXG',ILCMLN,ILCMTY)
        IF(ILCMLN .GT. 0) THEN
          LOLDXS(8)=.TRUE.
          CALL LCMGET(IPMACR,'FIXG',XSFIXE(1,IGROUP,3))
        ENDIF
C----
C  READ OR INITIALISE SCATTERING CROSS SECTIONS
C----
        DO 203 IANIS=1,NANISO
          WRITE(CANISO,'(I2)') IANIS-1
          NAMREC='NJJD'//CANISO
          CALL LCMLEN(IPMACR,NAMREC,ILCMLN,ILCMTY)
          ICMATR=ILCMLN
          IF(ICMATR .GT. 0) THEN
            LOLDXS(5)=.TRUE.
            ISCATA(IANIS)=1
C----
C  READ COMPRESS SCATTERING XS PLUS INFORMATION TO EXPAND XS
C----
            CALL LCMGET(IPMACR,NAMREC,INGSCT)
            NAMREC='SCAD'//CANISO
            CALL LCMGET(IPMACR,NAMREC,XSWORK)
            NAMREC='IJJD'//CANISO
            CALL LCMGET(IPMACR,NAMREC,IFGSCT)
C----
C  EXPAND SCATTERING XS TO XSSCAT(JGROUP,IMATER,IANIS,IGROUP)
C  WHERE IGROUP IS THE SECONDARY GROUP.
C----
            IPWRK=1
            DO 204 IMATER=1,ICMATR
              IF(INGSCT(IMATER) .GT. 0) THEN
                IGD=IFGSCT(IMATER)
                IGF=IGD-INGSCT(IMATER)+1
                DO 205 JGROUP=IGD,IGF,-1
                  XSSCAT(JGROUP,IMATER,IANIS,IGROUP)=XSWORK(IPWRK)
                  IPWRK=IPWRK+1
 205            CONTINUE
              ENDIF
 204        CONTINUE
          ENDIF
 203    CONTINUE
C----
C  INITIALISE PHYSICAL ALBEDOS
C----
        CALL LCMLEN(IPMACR,'PHYSALBEDO',ILCMLN,ILCMTY)
        IF(ILCMLN .GT. 0) THEN
          NALBR=-1
          CALL LCMGET(IPMACR,'PHYSALBEDO',ALBP(1,IGROUP))
        ENDIF
        CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 200  CONTINUE      
      RETURN
      END
