************************************************************************
      SUBROUTINE MCUTRK(IPGEOM,  IPTRK, GEONAM, 
     +                  IPRT,   NDIM,  ISPEC,       
     +                  NANGL,  ISYMM, DENUSR, RCUTOF,  MXSEG,
     +                  ALBEDO,  ICODE,   NORE, 
     +                  NGRT, MAXMIX, XSSIGT, XSSIGW,
     +                  TITREC, MAXPTS, MATCOD, VOLUME, KEYFLX,
     +                  LEAKSW,  IUSED, 
     +                  IPFLUX, IPMACR,RB2, NIFIS, NANIS, NL,NUN, 
     +                  NBN,NINTMX, AIEPS , NEXTMX, AEEPS ,
     +                  IFRITR, IACITR, ITYPEC, ILEAK , 
     +                  XSOLD , XSCHI , XSNUF , XSCAT , VSCAT ,
     +                  XSTK  , ILK   , ICAC  , ITRANC, ITRB  ,
     +                  TITRE , ITCRMX, ATCRMX, LJACC , LITLM ,
     +                  LINIF ,LETAB , MGTRK , MAXSCR,NSUR, NVOL)
************************************************************************
*                                                                      *
*     NAME: MCUTRK                                                     *
*     COMPONENT: MCU                                                   *
*     AUTHOR: M. DAHMANI                                               *
*                                                                      *
*     SUBROUTINE: THIS ROUTINE IS USED TO TREAT CARTESIAN ASSEMBLIES   *
*                 OF CELLS; THIS IS A TWO-STEP PROCESS:                *
*                 1) STUDY THE GEOMETRY TO GET VOLUMES AND MATERIALS   *
*                 2) CONNECT TO THE 3D CHARACTERISTICS SOLVER          *      
*                                                                      *
*--------+-------------- V A R I A B L E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                 /IO/MOD(DIMENS)*
*--------+----------------------------------------------+--+-----------*
* IPGEOM / POINTER TO THE GEOMETRY (L_GEOM)             /I./INT        *
* IPTRK  / POINTER TO THE EXCELL TRACKING (L_EXCELL)    /IO/INT        *
* GEONAM / GEOMETRY NAME                                /I./CAR*12     *
* IPRT   / GEOMETRY PRINT LEVEL                         /I./INT        *
* NDIM   / # OF DIMENSIONS (2.OR.3)                     /I./INT        *
* ISPEC  / KIND OF TRACKING (0:ISOTROPIC;1:SPECULAR)    /IO/INT        *
* NRMV   / VOLUME FACTORS (0:NO;1:YES)                  /I./INT        *
* NV     / # OF ZONES IN THE ASSEMBLY.                  /.O/INT        *
* NS     / # OF SURFACES IN THE ASSEMBLY.               /.O/INT        *
* NANGL  / # OF ANGLES USED IN TEMPORARY TRACKING FILE. /IO/INT        *
* ISYMM  / SYMMETRY FACTOR.                             /IO/INT        *
* DENUSR / DENSITY OF TRACKS IN THE PLANE PERPENDICULAR /IO/REL        *
*        /      TO THE TRACKING ANGLES.                 /  /           *
* RCUTOF / CUTOF FOR CORNER TRACKING( 0.25 SUGGESTED )  /I./REL        *
* MXSEG  / MAXIMUM # OF SEGMENTS IN A SINGLE TRACK.     /.O/INT        *
* ICODE  / INDEX FOR BOUNDARY CONDITIONS.               /.O/INT(6)     *
* ALBEDO / GEOMETRIC ALBEDOS.                           /.O/REL(6)     *
* NORE   / TRACK NORMALIZATION (0:YES 1:NO)             /I./INT        *
* NGRT   / NUMBER OF GROUPS                             /I./INT        *
* MAXMIX / # OF MIXTURES IN THE MACROLIB                /I./INT        *
* XSSIGT / TOTAL XS FOR MIXTURES IN THE MACROLIB        /I./REL(MAXMIX *
*        /                                              /I./  +1,NGRT) *
* XSSIGW / SCATT XS FOR MIXTURES IN THE MACROLIB        /I./REL(MAXMIX *
*        /                                              /I./  +1,NGRT) *
* TITREC / TITLE FOR THIS CASE                          /I./C*72       *
* MAXPTS / # OF ZONES ACCORDING TO USER                 /I./INT        *
* MATCOD / MATERIAL #S FOR ZONES IN THE SUPERCELL       /.O/INT(MAXPTS)*
* VOLUME / VOLUMES OF ZONES IN THE SUPERCELL            /.O/REL(MAXPTS)*
* KEYFLX / ZONE KEY FOR THE UNKNOWN VECTORS (FLUXES...) /.O/REL(MAXPTS)*
* LEAKSW / LEAKAGE SWITCH                               /.O/LOG        *
* IUSED  / INDEX FOR BOUNDARY CONDITIONS USED (0/1)     /.O/INT(6)     *
*--------+---------------- R O U T I N E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                                *
*--------+-------------------------------------------------------------*
* XELBIN / TO IDENTIFY ALL POSSIBLE TYPES AND ALL INTERFACES           *
* XELDCL / TO DECLARE VARIABLES FOR DIMENSIONING IN CELL GEOMETRY      *
* XELEDC / TO DECLARE VARIABLES FOR DIMENSIONING IN EXACT GEOMETRY     *
* XELETR / TO PREPARE TRACKING FOR EXACT GEOMETRY TREATMENT            *
* XELMRG / TO PREPARE MERGING OF ZONES AND SURFACES WITH RESPECT TO BC.*
* XELPRP / TO GET MAIN VALUES FOR GEOMETRY DIMENSIONING                *
* XELTRP / TO PREPARE CELL TRACKING BY NUMBERING                       *
* MCU3DL / DRIVER FOR CHARACTERISTICS SOLVER                           *
************************************************************************
C
      IMPLICIT        NONE
      INTEGER         IOUT,NALB,NSTATE,ICPALL,ICPEND
      PARAMETER      (IOUT=6,NALB=6,NSTATE=40,ICPALL=4,ICPEND=3)
C----
C  ROUTINE PARAMETERS
C----
      INTEGER         IPSYS ,IPGEOM,IPTRK ,IPRT  ,NDIM  ,IPFLUX,
     >                ISPEC ,NVO   ,NSO   ,NV    ,NS    ,NANGL ,
     >                ISYMM ,MXSEG ,NORE  ,NGRT  ,MAXMIX,MAXPTS
      CHARACTER       GEONAM*12,TITREC*72
      REAL            DENUSR,RCUTOF,AIEPS, AEEPS
      INTEGER         ICODE(NALB), IUSED(NALB)
      INTEGER         MATCOD(MAXPTS),KEYFLX(MAXPTS)
CMD      INTEGER         MATCOD(NREG),KEYFLX(NREG)
      REAL            ALBEDO(NALB),XSSIGT(0:MAXMIX,NGRT),
     >                XSSIGW(0:MAXMIX,NGRT),VOLUME(MAXPTS)
      LOGICAL         LEAKSW
      INTEGER         MXANGL
C----
C  PARAMETERS FOR MEMORY ALLOCATION
C----
      REAL            RD
      INTEGER         ID(1)
      COMMON          RD(1)
      EQUIVALENCE    (RD(1),ID(1))
C----
C  INTEGER ARRAYS
C----
      INTEGER         LGRP  ,MINDIM,MAXDIM,ICORD ,INDEL ,KEYMRG,
     >                MATALB,INAM  ,MATMRG,ICUR  ,INCR  
     >                IDL   ,MATRT
C----
C  REAL AND DOUBLE PRECISION ARRAYS
C----
      INTEGER         REMESH,VOLSUR,VOLMRG,INVVOL,
     >                CONV  ,TRKBEG,TRKDIR,ANGLES,DENSTY,
     >                STAYIN,GOSOUT,PRODUC,DSCBEG,DSCEND,
     >                IWEIG ,ICHI  ,IWFS  ,IWPAD ,IRI   ,IRBAR ,
     >                IGBAR ,IBILAN,ICIJ  ,IWSP  , IWORK ,JPSST ,
     >                JPSVT ,IDD   ,IPIS  , IARRAY, VOLTRK,NBN
C----
C  LOCAL PARAMETERS
C----
      INTEGER         ISTATE(NSTATE)
      INTEGER         NCODE(NALB),MAXGRI(3),LCLSYM(3)
      INTEGER         NUNKMR,NREG  ,NPIJ  ,NALBG ,NC    ,NTR   ,
     >                NTZ   ,ICL   ,NSOUT ,ITGEO ,NGRP  ,IENDGR,
     >                NSPOS ,IR    ,ICMAX ,IBEGGR,LENGHT,IPRTT ,
     >                NRMV  ,IRMV  ,NTY   ,INDATA,NSUBG ,IG    ,
     >                NRENOR,JG    ,NANGLE,LTRK  ,LINMAX,NTOTG ,
     >                INDLEC,NUNK  ,MAXR  ,NEXTGE,NTX   ,NCOR  ,
     >                ISCATR,IPHASE,NSUR  ,NTOTCL,NVOL
      LOGICAL         SWZERO,SWVOID,SWNZBC,LHALT ,LALBS ,
     >                LNORM ,LTAKE
      REAL            ALBOLD(NALB)
      REAL            CUTOFX, REDATA
      DOUBLE PRECISION DBLINP
CMD
      INTEGER   IUNOUT, ILCMUP, ILCMDN, NDIMO, NANIS, NIFIS, NUN,
     >          IPMACR, IREGIO, ITYLCM, IMAT, IMIX, ITYPEC,
     >          IFIS, JGR,  NTRK
      REAL      EPSR, RB2, ATCRMX
      PARAMETER (IUNOUT=6,ILCMUP=1,ILCMDN=2)
      PARAMETER (NDIMO=3, EPSR=1.E-6)
      LOGICAL     ILK, LJACC, LITLM, LINIF, LETAB, L2LNG
      CHARACTER   TITRE*72,CGRPNM*12
      INTEGER     NREGIO,ITCRMX,MGTRK,NINTMX,
     >            ITRANC,ITRB,IGR,ILCMLN
      REAL        VOL(MAXPTS), XSOLD(0:MAXMIX,NGRT),
     >            XSCAT(NGRT,0:MAXMIX,0:NANIS,NGRT),VSCAT(*),
     >            XSNUF(0:MAXMIX,NIFIS,NGRT),XSCHI(0:MAXMIX,NIFIS,NGRT),
     >            XSTK(MAXMIX,NIFIS)
CD
      INTEGER     NJJ, IJJ, IPOS, NMAT, NUNKNO, INCR, NR, NRNL, NL
      INTEGER     NEXTMX, IFRITR, IACITR, ILEAK, ICAC, MAXSCR, NMOD
C
      CHARACTER       TEDATA*8
      CHARACTER*8     CTISO, CTSPC, CCORN, CHALT, CSYMM, CMEDI, CPNOR,
     >                CSKIP, CSUBG, CTAKE, CLIST, CALBS, CNORM, CBLAN
      SAVE            CTISO, CTSPC, CCORN, CHALT, CSYMM, CMEDI, CPNOR,
     >                CSKIP, CSUBG, CTAKE, CLIST, CALBS, CNORM, CBLAN
      DATA            CTISO, CTSPC, CCORN, CHALT, CSYMM, CMEDI, CPNOR,
     >                CSKIP, CSUBG, CTAKE, CLIST, CALBS, CNORM, CBLAN
     >              / 'TISO','TSPC','CORN','HALT','SYMM','MEDI','PNOR',
     >                'SKIP','SUBG','TAKE','LIST','ALBS','NORM','    ' /
CMD
      NMAT=MAXMIX
C
      LTAKE= .FALSE.
      LHALT= .FALSE.
      LNORM= .FALSE.
      LALBS= .FALSE.
      SWZERO=.TRUE.
      LTRK  = 0
      NANGLE= 0
C      LSYMXY= .FALSE.
      DENUSR= 0.0
      RCUTOF= 0.0
      LINMAX= 0
      CUTOFX= 0.0
      TEDATA= CBLAN
      ISYMM=1
C
C. 1) READ ALL USER INPUT.
C
C     READ TRACKING PARAMETERS  LTRK= 0 : NO  TRACKING
C                               LTRK= 1 : ISOTROPIC TRACKING   (TISO)
C                               LTRK= 2 : SPECULAR TRACKING    (TSPC)
         CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
         NTOTG= 1
         NSUBG= NGRT
         IF( TEDATA(1:4).EQ.CSUBG(1:4) )THEN
            NSUBG= 0
            CALL REDGET( INDLEC, NSUBG, DENUSR, TEDATA,DBLINP)
            IF( NSUBG.LE.0.OR.NSUBG.GT.NGRT )THEN
               CALL XABORT('MCUTRK: INVALID *SUBG* PARAMETER')
            ENDIF
            NTOTG= NGRT/NSUBG
            IF( MOD(NGRT,NSUBG).NE.0 ) NTOTG= NTOTG+1
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
         ELSEIF( TEDATA(1:4).EQ.CTAKE(1:4) )THEN
            NSUBG= 0
            CALL REDGET( INDLEC, NSUBG, DENUSR, TEDATA,DBLINP)
            IF( NSUBG.LE.0.OR.NSUBG.GT.NGRT )THEN
               CALL XABORT('MCUTRK: INVALID *TAKE* PARAMETER')
            ENDIF
            LTAKE=.TRUE.
         ENDIF
         CALL SETARA(ID, NSUBG, LGRP)
         IF( LTAKE )THEN
            CALL REDGET( INDLEC, INDATA, DENUSR, TEDATA,DBLINP)
            IF( INDLEC.NE.3.OR.TEDATA(1:4).NE.CLIST(1:4) )THEN
               CALL XABORT('MCUTRK: *LIST* KEYWORD EXPECTED')
            ENDIF
            DO 110 IG= 1, NSUBG
               CALL REDGET( INDLEC, INDATA, DENUSR, TEDATA,DBLINP)
               IF( INDLEC.NE.1 )THEN
                  CALL XABORT('MCUTRK: INTEGER GROUP NUMBER EXPECTED')
               ELSEIF( INDATA.LE.0.OR.INDATA.GT.NGRT )THEN
                  CALL XABORT('MCUTRK: BAD GROUP NUMBER')
               ENDIF
               DO 111 JG= 1, IG-1
                 IF( INDATA.EQ.ID(LGRP+JG-1) )THEN
                   CALL XABORT('MCUTRK:  2 TIMES SAME GROUP')
                 ENDIF
 111           CONTINUE
               ID(LGRP+IG-1)= INDATA
 110        CONTINUE
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
         ENDIF
         NRENOR=0
         IF( TEDATA(1:4).EQ.CNORM(1:4) )THEN
            LNORM= .TRUE.
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
         ELSEIF( TEDATA(1:4).EQ.CALBS(1:4) )THEN
            LALBS= .TRUE.
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
         ENDIF
         IPHASE=0
         ISCATR=0
         IF( TEDATA(1:4).EQ.CSKIP(1:4) )THEN
            ISCATR=1
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
         ENDIF
         IF( INDLEC.NE.3 )THEN
            LTRK  = 1
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
            IF( INDLEC.EQ.3 )THEN
            CALL XABORT('MCUTRK: *TISO* ASSUMED, PUT NANGLE & DENSTY' )
            ENDIF
            CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
         ELSE
            IF( TEDATA(1:4).EQ.CTISO(1:4) )THEN
               LTRK  = 1
            ELSEIF( TEDATA(1:4).EQ.CTSPC(1:4) )THEN
               LTRK  = 2
            ENDIF
            IF( LTRK.GT.0 )THEN
               CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
               IF( LTRK.EQ.2.AND.TEDATA(1:4).EQ.CMEDI(1:4) )THEN
                  SWZERO= .FALSE.
                  CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
               ENDIF
               CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
               CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
            ELSE
               CALL XABORT( 'MCUTRK: *TISO* OR *TSPC* HERE')
            ENDIF
         ENDIF
         IF( TEDATA(1:4).EQ.CHALT(1:4) )THEN
            LHALT= .TRUE.
            CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
         ENDIF
         IF( INDLEC.NE.3 )THEN
            CALL XABORT( 'MCUTRK: ; OR *CORN* IS SUPPOSED TO BE HERE' )
         ELSEIF( TEDATA(1:4).EQ.CCORN(1:4) )THEN
            CALL REDGET( INDLEC, INDATA, RCUTOF, TEDATA,DBLINP)
            IF(INDLEC.NE.2) CALL XABORT('MCUTRK: REAL DATA EXPECTED')
            CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
         ENDIF
         IF( INDLEC.NE.3 )THEN
            CALL XABORT( 'MCUTRK: ; OR *SYMM* IS SUPPOSED TO BE HERE' )
         ELSEIF( TEDATA(1:4) .EQ. 'NOSY'   )THEN
            ISYMM=0
            CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
         ELSEIF( TEDATA(1:4).EQ.CSYMM(1:4) )THEN
            CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
            IF(INDLEC.NE.1) CALL XABORT('MCUTRK: INTEGER DATA EXPECTED')
            ISYMM=INDATA
            CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
         ENDIF
         IF( INDLEC.NE.3.OR.TEDATA(1:4).NE.';' )THEN
            CALL XABORT( 'MCUTRK: ; IS SUPPOSED TO BE HERE' )
         ENDIF
         IF( LTRK.GT.0.AND.NANGLE.LE.0 )THEN
         CALL XABORT( 'MCUTRK: INVALID NUMBER OF ANGLES (NANGLE < 0)' )
         ENDIF
         IF( LTRK.GT.0.AND.DENUSR.LE.0.0 )THEN
         CALL XABORT( 'MCUTRK: INVALID DENSITY (DENSTY < 0.0) ' )
         ENDIF
   10 CONTINUE
C----
C  PROCESS THE GEOMETRY
C----
      IPRT=0
      CALL AXGXEL(IPGEOM,IPTRK ,IPRT  ,GEONAM)
C----
C  READ THE GEOMETRY INFORMATION STORED ON IPTRK
C----
      CALL LCMSIX(IPTRK,'EXCELL      ',1)
      CALL XDRSET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      NDIM     =ISTATE(1)
      NSUR    =-ISTATE(2)
      NVOL     =ISTATE(3)
      NTOTCL   =ISTATE(4)
      MAXR     =ISTATE(5)
      NUNK     =ISTATE(6)
      NEXTGE   =ISTATE(7)
C----
C  Intrinsic symmetries used in geometry
C  Use these to simplify tracking unless
C  NOSYMM tracking option activated
C----
      LCLSYM(1) =ISTATE(8)
      LCLSYM(2) =ISTATE(9)
      LCLSYM(3) =ISTATE(10)
      IF(ISYMM .NE. 0) THEN
        ISYMM=0
        IF(NDIM .EQ. 2) THEN
          IF(LCLSYM(1) .NE. 0) THEN
C----
C  X SYMMETRY
C----
            ISYMM=2
          ENDIF
          IF(LCLSYM(2) .NE. 0) THEN
            IF(ISYMM .EQ. 0) THEN
C----
C  Y SYMMETRY
C----
              ISYMM=4
            ELSE
C----
C  X AND Y SYMMETRY
C----
              ISYMM=8
            ENDIF
          ENDIF
CC          IF(ISTATE(11) .NE. 0) THEN
CCC----
CCC  X-Y DIAGONAL SYMMETRY
CCC----
CC            IF(ISYMM .EQ. 0) THEN
CC              ISYMM=10
CC            ELSE
CC              ISYMM=12
CC            ENDIF
CC          ENDIF
        ELSE
          IF(LCLSYM(1) .NE. 0) THEN
C----
C  X SYMMETRY
C----
            ISYMM=2
          ENDIF
          IF(LCLSYM(2) .NE. 0) THEN
            IF(ISYMM .EQ. 0) THEN
C----
C  Y SYMMETRY
C----
              ISYMM=4
            ELSE
C----
C  X AND Y SYMMETRY
C----
              ISYMM=8
            ENDIF
          ENDIF
          IF(LCLSYM(3) .NE. 0) THEN
C----
C  Z SYMMETRY
C----
            ISYMM=ISYMM+16
          ENDIF
        ENDIF
        IF(ISYMM .EQ. 0) ISYMM=1
      ENDIF
C
      CALL SETARA(ID,NTOTCL,MINDIM)
      CALL SETARA(ID,NTOTCL,MAXDIM)
      CALL SETARA(ID,NTOTCL,ICORD )
      CALL SETARA(ID,4*NUNK,INDEL )
      CALL SETARA(RD,MAXR  ,REMESH)
      CALL SETARA(ID,NUNK  ,KEYMRG)
      CALL SETARA(ID,NUNK  ,MATALB)
      CALL SETARA(RD,NUNK  ,VOLSUR)
      CALL LCMGET(IPTRK,'MINDIM      ',ID(MINDIM))
      CALL LCMGET(IPTRK,'MAXDIM      ',ID(MAXDIM))
      CALL LCMGET(IPTRK,'ICORD       ',ID(ICORD) )
      CALL LCMGET(IPTRK,'INDEX       ',ID(INDEL) )
      CALL LCMGET(IPTRK,'REMESH      ',RD(REMESH))
      CALL LCMGET(IPTRK,'KEYMRG      ',ID(KEYMRG))
      CALL LCMGET(IPTRK,'MATALB      ',ID(MATALB))
      CALL LCMGET(IPTRK,'VOLSUR      ',RD(VOLSUR))
      CALL LCMSIX(IPTRK,'EXCELL      ',2)
      CALL LCMGET(IPTRK,'ICODE       ',ICODE )
      CALL LCMGET(IPTRK,'NCODE       ',NCODE )
      CALL LCMGET(IPTRK,'ALBEDO      ',ALBOLD)
C
C----
C  VERIFY SYMMETRY AND
C  STUDY TRACKING PARAMETERS. ARE THEY BASICALLY POSSIBLE ?
C----
      IF(LTRK .EQ. 1)THEN
        NCOR= 1
        IF(NDIM .EQ. 2) THEN
          MXANGL=NANGLE
CC          IF(ISYMM .EQ. 12) THEN
CC            NANGL = NANGLE/4
          IF(ISYMM .GE. 2) THEN
            NANGL = NANGLE/2
          ELSE
            NANGL = NANGLE
          ENDIF
          IF( RCUTOF.GT.0.0 ) NCOR= 2
        ELSE IF(NDIM .EQ. 3) THEN
          IF(MOD(NANGLE,2) .EQ. 1)THEN
            NANGLE=NANGLE+1
            WRITE(IOUT,'(/31H MESS = ONLY EVEN # EQN ANGLES )')
          ENDIF
          IF(NANGLE .GT. 16)THEN
            NANGLE=16
            WRITE(IOUT,'(/31H MESS = 16 IS MAX # EQN ANGLES )')
          ENDIF
          MXANGL=(NANGLE * (NANGLE+2)) / 2
          IF(NEXTGE .EQ. 1) THEN
            NANGL = (NANGLE * (NANGLE+2)) / 8
          ELSE
            IF(ISYMM .EQ. 8 .OR. ISYMM .EQ. 24) THEN
              NANGL = (NANGLE * (NANGLE+2)) / 8
            ELSE IF(ISYMM .EQ. 2  .OR. ISYMM .EQ. 4 .OR.
     >              ISYMM .EQ. 18 .OR. ISYMM .EQ. 20 ) THEN
              NANGL = (NANGLE * (NANGLE+2)) / 4
            ELSE
              NANGL = (NANGLE * (NANGLE+2)) / 2
            ENDIF
          ENDIF
          IF(RCUTOF .GT. 0.0) NCOR= 4
        ENDIF
      ELSEIF( LTRK.EQ.2 )THEN
        NCOR  = 1
        IF( NDIM.EQ.2 )THEN
           NANGL = NANGLE
        ELSEIF( NDIM.EQ.3 )THEN
           CALL XABORT('MCUTRK: *TSPC* NOT AVAILABLE FOR 3-D GEOMETRY')
        ENDIF
        CUTOFX= RCUTOF
      ENDIF
      IF( IPRT.GE.1 )THEN
        WRITE(IOUT,'(1H )')
        WRITE(IOUT,'(26H >>> GEOMETRY TREATED:    ,A12)') GEONAM
        WRITE(IOUT,'(26H >>> EXCELL TREATMENT <<< )')
        WRITE(IOUT,'(1H )')
      ENDIF
      IF( IPRT.GT.1 .AND. NEXTGE.EQ.0 )THEN
C
C        IF PRINT REQUIRED AND OVERALL CARTESIAN GEOMETRY
C        PRINT CARTESIAN REGION MAP
         NTX= ID(MAXDIM  )-ID(MINDIM  )
         NTY= ID(MAXDIM+1)-ID(MINDIM+1)
         NTZ= ID(MAXDIM+2)-ID(MINDIM+2)
         NTR=0
         DO 103 ICL=3,NTOTCL-1
            NTR= MAX(NTR,ID(MAXDIM+ICL)-ID(MINDIM+ICL)+1)
  103    CONTINUE
         CALL SETARA(ID,4*(NTR+1)*(NTX+2)*(NTY+2)*(NTZ+2),INAM)
         CALL XELGPR(NDIM,NTX,NTY,NTZ,NTR,ISYMM,
     >               NSUR,NVOL,NTOTCL,ID(MINDIM),ID(MAXDIM),
     >               ID(KEYMRG),ID(INDEL),ID(MATALB),ID(INAM))
         CALL RLSARA(ID(INAM))
      ENDIF
      CALL SETARA( RD, 2*(NANGL+1)*NUNK, VOLTRK)
C
      NV=  NVOL
      NS= -NSUR
      NREG = NVOL
      NSOUT = NS 
      NUNKNO= NREG+NSOUT+1
      CALL SETARA( RD,     NUNK, VOLMRG)
      CALL SETARA( ID,     NUNK, MATMRG)
      CALL SETARA( RD,     NUNKNO, INVVOL)
      ITGEO=3
      CALL XELCMP(   NS,    NV, RD(VOLSUR), ID(MATALB), ID(KEYMRG),
     >            NSOUT,  NREG, RD(VOLMRG), ID(MATMRG), ITGEO,ICODE)
C
C. 3) DO THE TRACKING OF THE EXACT GEOMETRY FOR *NEWT* OPTION.
      IF( LTRK.NE.0 )THEN
         NC= NTOTCL - 3
         IF( IPRT.GE.1 )THEN
            WRITE(IOUT,'(1H )')
            IF    ( NC.EQ.0 )THEN
               WRITE(IOUT,'(/21H NOW, TRACKING   >>> ,A12,
     >                   13H GEOMETRY <<<,
     >                   13H    (WITH  NO,11H CYLINDER ) /)')
     >                                            GEONAM
            ELSEIF( NC.EQ.1 )THEN
               WRITE(IOUT,'(/21H NOW, TRACKING   >>> ,A12,
     >                   13H GEOMETRY <<<,
     >                   13H    (WITH ONE,11H CYLINDER ) /)')
     >                                            GEONAM
            ELSE
               WRITE(IOUT,'(/21H NOW, TRACKING   >>> ,A12,
     >                   13H GEOMETRY <<<,
     >                   10H    (WITH ,I3,11H CYLINDERS) /)')
     >                                            GEONAM, NC
            ENDIF
         ENDIF
         CALL SETARA( ID,  NTOTCL,   ICUR )
         CALL SETARA( ID,  NTOTCL,   INCR )
         CALL SETARA( RD,  NTOTCL,   CONV )
         CALL SETARA( RD,  NTOTCL, TRKBEG )
         CALL SETARA( RD,  NTOTCL, TRKDIR )
C
C  3.0)  WRITE FIRST RECORDS OF THE UNNORMALIZED TRACKING FILE
         IF( LTRK.EQ.1 )THEN
            LINMAX= 2*NVOL + 10
         ELSE
            LINMAX= 8*NANGL*(2*NVOL + 8)
         ENDIF
         ISPEC = LTRK-1
         NALBG = 6
         CALL SETARA( RD,3*NANGL , ANGLES )
         CALL SETARA( RD,  NANGL , DENSTY )
         MXSEG= LINMAX
         CALL SETARA(ID,MAXMIX,NJJ)
         CALL SETARA(ID,MAXMIX,IJJ)
         CALL SETARA(ID,MAXMIX,IPOS)
C
      NGRP=NGRT
      MXSEG= LINMAX
      NR=NREG
      NRNL=(NL+1)**2
C
      ICMAX= 0
      DO 60 IR= 1, 6
        ALBEDO(IR)=1.0
        IUSED(IR)= 0
        ICMAX= MAX(ICMAX,ICODE(IR))
        IF( ICODE(IR).LT.0 ) ALBEDO(IR)= ALBOLD(-ICODE(IR))
   60 CONTINUE
      IF( ICMAX.GT.0 )THEN
         CALL XABORT('MCUTRK: PHYSICAL ALBEDOS NOT IMPLEMENTED')
      ENDIF
      LEAKSW=.TRUE.
      DO 80 IR= -NSOUT, -1
         IUSED(-ID(MATMRG+IR+NSOUT))= 1
         LEAKSW= LEAKSW .AND. ALBEDO(-ID(MATMRG+IR+NSOUT)).EQ.1.0
   80 CONTINUE
      LEAKSW=.NOT.LEAKSW
      IF( LALBS.AND.(.NOT.LEAKSW) )
     > CALL XABORT('MCUTRK: INVALID BOUNDARY CONDITIONS. THE ALSB '
     >      //'OPTION REQUIRES SOME BOUNDARY LEAKAGE.')
      IF( NREG.GT.MAXPTS ) THEN
       WRITE(IOUT,'(28H ****** MCUTRK ERROR  ******,/
     >   28H NUMBER OF REGION COMPUTED =,I10/
     >   28H MAXIMUM NUMBER OF REGION  =,I10)') NREG,MAXPTS
        CALL XABORT('MCUTRK: MAXR TOO SMALL')
      ENDIF
CMD
         IBEGGR= 1
         IPRTT= IPRT
C
C-------------------THE DRIVER FOR 3D CHARACTERISTICS SOLVER--------------
C
            CALL MCU3DL( IPRTT, NANGLE,  DENUSR, ISYMM,  RD(ANGLES),
     +             RD(DENSTY), NTOTCL,MAXMIX,NEXTGE, MAXR,RD(REMESH),
     +             LINMAX,MAXPTS, RCUTOF,NSUR,NVOL,ID(INDEL),ID(MINDIM),
     +             NANGL, ID(MAXDIM), ID(ICORD), ID(INCR), ID(ICUR),
     +             RD(TRKBEG), RD(CONV), RD(TRKDIR),RD(INVVOL), 
     +             NGRP, SWVOID, ALBEDO, ISPEC,
     +             NORE,MATCOD,ID(MATMRG),RD(VOLTRK),ID(KEYMRG),NSOUT,  
     +             NREG, RD(VOLMRG), NUNK, NUNKNO, VOLUME,
     +             IPFLUX, IPMACR, RB2, NIFIS, NANIS, NL,  
     +             NBN, NINTMX, AIEPS , NEXTMX, AEEPS , IFRITR,
     +             IACITR, ITYPEC, ILEAK , XSOLD , XSCHI , XSNUF,
     +             XSCAT, VSCAT, XSTK,NMOD, LEAKSW, ILK, ICAC,
     +             ITRANC, ITRB,TITRE, ITCRMX, ATCRMX, LJACC, LITLM,
     +             LINIF , LETAB , MGTRK , MAXSCR, 
     +             RD(VOLSUR), ID(MATALB), ID(NJJ), ID(IJJ),ID(IPOS) )
C-------------------------------------------------------------------------     
C      ENDIF
         CALL RLSARA( RD(DENSTY) )
         CALL RLSARA( RD(ANGLES) )
         CALL RLSARA( RD(TRKDIR) )
         CALL RLSARA( RD(TRKBEG) )
         CALL RLSARA( RD(  CONV) )
         CALL RLSARA( ID(  INCR) )
         CALL RLSARA( ID(  ICUR) )
         CALL RLSARA( ID(NJJ) )
         CALL RLSARA( ID(IJJ) )
         CALL RLSARA( ID(IPOS) )
      ENDIF
C
      CALL RLSARA( ID( INDEL) )
      CALL RLSARA( ID( ICORD) )
      CALL RLSARA( ID(MAXDIM) )
      CALL RLSARA( ID(MINDIM) )
      CALL RLSARA( RD(REMESH) )
      CALL RLSARA( RD(VOLSUR) )
      CALL RLSARA( RD(VOLTRK) )
      CALL RLSARA( ID(KEYMRG) )
      CALL RLSARA( ID(MATALB) )
CMD
      CALL RLSARA( RD(VOLMRG) )
      CALL RLSARA( ID(MATMRG) )
      CALL RLSARA( ID(  LGRP) )
      CALL RLSARA( RD(INVVOL) )     
      NV=NREG
      NS=NSOUT
C
      RETURN
      END
