*DECK MOC2AJ 
      SUBROUTINE MOC2AJ(IPFLUX,IPTRK,IFTRAK,NGRP,NREG,NSOUT, 
     >              NANIS,NMAT,NGSS ,MXSEG ,NTRK,NPHI,NFUNL,NMOD,KEYANI,
     >              MATALB,VOL,VOLINV,WGSS,XGSS,YGSS,PHIANG,SEGLEN,
     >              NRSEG,EXPT,EXP2,CYCLE,FLM,FLP,CYP,CYM,ALPHA,DFLUX,
     >              DWEIG,FMEAN,SIGANG,NALB,ALBEDO,XSTOT,XSTRC,
     >              XTINV,XSDIA,XSCAT,NIFIS,XSNUF,XSCHI,ITRB,
     >              FLUX,ADJFLX,ADJFLO,DIFFB1,ISGNR,RHARM,LEAKSW,LREBAL,
     >              NINTMX,AIEPSF,NEXTMX,AEEPSF,IPRT,ITYPEC,ILEAK,
     >              NL,NBN,RKEFF,RB2,TRHAR,REBAL,IFRITR,IACITR,PHI,
     >              FLUN,PNL,XCSOU,XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,
     >              XCABS,DIFF,DB2,BETA,LEXIN,NOVOID,REFKEF,ADJSOU,
     >              ISADJ,IPMACR,ADJFXS,GMXUPS,LMODE,CADJFL,FLUXO,
     >              IPTRK2,NTRK2,NANGL2,IFTRK2,LCACA,NGSS2,XMTOT,XMTRC,
     >              XMCAT,XMDIA,ANGPHI,ANGADJ   )
*  
*---------------------------  MOC2AJ  ---------------------------------
* 1- PROGRAMME STATISTICS:
*     NAME     :  MOC2AJ
*     USE      :  ADJOINT FLUX CALCULATION MODULE
*                 FOR CASE (ITYPEC = 1 OR 6) AND (ISADJ = 1, 2 OR 3)
*     AUTHOR   :  MONCHAI A. (22/03/07)
*
* 2- ROUTINE PARAMETERS:
*   INPUT
*     ADJFLX   : ADJOINT FLUX ARRAY                      D(NFUNL,NREG,
*                                                          NGRP,8)
*     ADJSOU   : POINTER FOR EXTERNAL ADJOINT SOURCE     R(NREG,NGRP)
*     FLUXO    : GLOBAL DIRECT FLUX ARRAY                R(NUN,NFUNL,
*                FOR MODULE DATA EXCHANGE                  NGRP)
*     ADJFLO   : GLOBAL ADJOINT FLUX ARRAY               R(NUN,NFUNL,
*                FOR MODULE DATA EXCHANGE                  NGRP)
*     CDIRFL   : FLUX DIRECTORY NAME                     C*12
*     CADJFL   : ADJOINT FLUX DIRECTORY NAME             C(2)*12
*                FOR STANDARD AND GENERALIZED ADJOINT
*                PROBLEM
*     CSTDAJ   : STANDARD ADJOINT FLUX DIRECTORY NAME    C*12
*     CSTDDR   : STANDARD DIRECT FLUX DIRECTORY NAME     C*12
*     GMXUPS   : MAX.ENERGY GROUP CONTAINGING            I
*                UPSCATTERING XS(EX. GROUP 27)
*     GAJSCH   : GENERALIZED ADJOINT DECONTAMINATION     I
*     IPFLUX   : POINTER TO THE FLUX                     I
*     IPTRK    : POINTER TO THE TRACKING                 I
*     IPMACR   : POINTER TO THE MACROLIB                 I
*     IFTRAK   : FILE UNIT NUMBER FOR TRACKS.            I
*     IPRT     : PRINT OPTION FOR CHARACTERISTIC SOLVER  I
*     ITYPEC   : type of flux calculation                I
*        ITYPEC= 0 to skip the flux calculation; 
*        ITYPEC= 1 to compute the direct problem 
*                  solution with fission sources 
*                  and Keff convergence
*        ITYPEC= 2 to compute the direct problem 
*                  solution with fission sources,
*                  Keff convergence and imposed 
*                  leakage
*        ITYPEC= 3 to compute the direct problem 
*                  solution with fission sources,
*                  Keff convergence and imposed 
*                  buckling
*        ITYPEC= 4 to compute the direct problem 
*                  solution with fission sources,
*                  imposed Keff and convergence 
*                  on buckling
*        ITYPEC= 5 direct problem with leakage 
*                  sources and convergence on 
*                  buckling
*        ITYPEC= 6 direct problem fixed sources.
*     ISADJ  : FLAG FOR TYPE OF ADJOINT CALCULATION      I
*        ISADJ = 1 ADJOINT FLUX CALCULATION WITHOUT-- 
*                  EXTERNAL SOURCE FOR ADJOINT KEFF PROBLEM
*        ISADJ = 2 ADJOINT FLUX CALCULATION WITH--
*                  EXTERNAL SOURCE FOR ADJOINT FIXED SOURCE
*                  PROBLEM
*        ISADJ = 3 GENERALIZED ADJOINT FLUX CALCULATION WITH--
*                  EXTERNAL GENERALIZED ADJOINT SOURCE 
*     IGDEB    : MULTIGROUP REDUCTION IN FORWARD         I
*                DIRECTION
*     JGDEB    : MULTIGROUP REDUCTION IN BACKWARD        I
*                DIRECTION
*     LMODE    : CONTROL FLAG FOR CALCULATION MODE       L(20)
*     LSPLIT   : CONTROL FLAG FOR ITERATION WITH GROUP   L
*                SPLITTING
*     NGRP     : NUMBER OF GROUPS TREATED                I
*     NMAT     : NUMBER OF MIXTURES CONSIDERED           I
*     NREG     : NUMBER OF REGIONS CONSIDERED            I
*     VOL      : VOLUME OF REGION                        R(NREG)
*     VOLINV   : INVERSE OF VOLUMES                      R(NREG)
*     LEAKSW   : LEAKAGE SWITCH                          L
*     TITRE    : EXECUTION TITLE                         C*72
*     REFKEF   : KEFF VALUE FOR B2 SEARCH                D
*     RKEFF    : REFERENCE KEFF VALUE FOR EIGENVALUE     R 
*                PROBLEM 
*
*   WORK VECTOR
*     ADJFXS   : ADJFIX SOURCE BY MIXTURE/NGRP           R(0:NMAT,NGRP)
*     XSTOT    : GROUP TOTAL CROSS SECTION               R(NMAT+1)
*     XSTRC    : GROUP TRANSPORT-CORRECTED TOTAL         R(NMAT+1)
*                CROSS SECTION       
*     XSDIA    : WITHIN GROUP SCATTERING                 R(NMAT+1)
*                CROSS SECTION
*     XSCAT    : GROUP SCATTERING CROSS SECTION          R(NMAT+1)
*     XSNUF    : FISSION NEUTRON PRODUCTION              R(0:NMAT,NIFIS,
*                CROSS SECTION                             NGRP)
*     XSCHI    : FISSION NEUTRON DISTRIBUTION            R(0:NMAT,NIFIS,
*                                                          NGRP)
*     XMTOT    : REFERENCE TOTAL CROSS SECTION           R(NMAT+1)
*     XMTRC    : REFERENCE TRANSPORT-CORRECTED TOTAL     R(NMAT+1)
*                CROSS SECTION       
*     XMDIA    : REFERENCE WITHIN GROUP SCATTERING       R(NMAT+1)
*                CROSS SECTION
*     XMCAT    : REFERENCE SCATTERING CROSS SECTION      R(NMAT+1)
*     LACCOU   : CONTROL FLAG FOR OUTER ACCELERATION     L
*     LACCIN   : CONTROL FLAG FOR INNER ACCELERATION     L
*
* 3- INTERNAL PARAMETERS:
*     IOUT     : OUTPUT UNIT = 6                         I
*     ILCMUP   : GO UP LCM INDEX = 1                     I
*     ILCMDN   : GO DOWN LCM INDEX = 2                   I
*
*---------------------------   MOC2AJ  --------------------------------
      IMPLICIT NONE
      REAL        A
      COMMON      A(1)
      INTEGER     I(1)
      EQUIVALENCE (A(1),I(1))
      INTEGER NGRP,NREG,NANIS,NMAT,NGSS,MXSEG,NTRK,NSOUT,MXGAUS,
     >        IPRT,ITYPEC,IFTRAK,NPHI,NFUNL,NMOD,IPTRK,IPTRK2,
     >        NL,NBN,IPFLUX,ITRB,NALB,IPMACR,NITR,NALGIT,NITRLT,
     >        IFRITR,IACITR,CTOT,CPTM,MAXITR,IOUT,ILCMUP,ILCMDN,
     >        IR,IG,JG,IA,IT,JT,IS,ITERF,IGDEB,IGDEBO,JGDEBO,JGDEB,
     >        JGCOUN,IDUM
      INTEGER MATALB(-NSOUT:NREG),NRSEG(MXSEG),KEYANI(NFUNL),
     >        ISGNR(NFUNL,NMOD)
      INTEGER NEXTMX,NINTMX,NINTMO,IEXTF,ILEAK,NIFIS
      PARAMETER(IOUT=6,MXGAUS=64,ILCMUP=1,ILCMDN=2)
      INTEGER  GMXUPS,ILPCN0,ILPCN1,ILPCN2,ILOOP,NSTART, NSTOP, NSTEP,
     >         NTRK2,NANGL2,IFTRK2,LCACA,NGSS2 ,NTRACK,IFFIL,NMODE,
     >         NGSSCL,IANG,IE,JM,IDCNTM,ISADJ,ILCMLN,ILCMTY,
     >         IRESD,IPVEC,IXVEC,IYVEC,IZVEC,ISVEC,IPIJ,IQSOU,
     >         IXVEC1,IYVEC1,IZVEC1,ISVEC1,IDUMMY,AFLXDU,IGDUM,ICP,
     >         IPIJNW,IPNEW,IPOLD,IDUMNN,IDUMZ,GAJSCH,INGR,NINGR
      REAL    SEGLEN(MXSEG),ALBEDO(NALB),RKEFF,RB2,ALPHA(NGSS),
     >        ADJFLO(NREG,NFUNL,NGRP),ADJSOU(NREG,NGRP),DFLUX(NGSS),
     >        FLUXO(NREG,NFUNL,NGRP),DWEIG(NGSS),FMEAN(NGSS),
     >        ADJFXS(0:NMAT,NGRP),EXPT(NGSS,MXSEG),EXP2(NGSS,MXSEG),
     >        CYCLE(NGSS),SIGANG(NGSS,-NALB:NMAT,NGRP),
     >        XSTOT(0:NMAT,NGRP),XSTRC(0:NMAT,NGRP),XSDIA(0:NMAT,NGRP),
     >        XSCAT(NGRP,0:NMAT,0:NANIS,NGRP),XTINV(0:NMAT,NGRP),
     >        WGSS(MXGAUS),XGSS(MXGAUS),YGSS(MXGAUS),PHIANG(NPHI),
     >        XSNUF(0:NMAT,NIFIS,NGRP),XSCHI(0:NMAT,NIFIS,NGRP),
     >        XMTOT(0:NMAT,NGRP),XMTRC(0:NMAT,NGRP),
     >        XMCAT(NGRP,0:NMAT,0:NANIS,NGRP),XMDIA(0:NMAT,NGRP) 
      REAL    VOL(NREG),VOLINV(NREG),DIFFB1(NGRP),RHARM(NGSS,*),
     >        TRHAR(NFUNL,NGSS,*),EINNO,EEXT,EEMX,
     >        EBMX,MU,AIEPSF,AEEPSF,FACT,ONE,ZERO,BETASC,SCALE
      DOUBLE PRECISION EIMXO,EIMX,EINN,FINN,GINN
      DOUBLE PRECISION ADJFLX(NFUNL,NREG,NGRP,8),
     >                 FLUX(NFUNL,NREG,NGRP,8),
     >                 ANGPHI(NREG,-NPHI:NPHI,NGSS,NGRP),
     >                 ANGADJ(NREG,-NPHI:NPHI,NGSS,NGRP),
     >                 FLM(NGSS,MXSEG),FLP(NGSS,MXSEG),
     >                 CYM(NGSS,MXSEG),CYP(NGSS,MXSEG)
      DOUBLE PRECISION PHI(NGRP),XCSOU(NGRP),XCTOT(NGRP),XCABS(NGRP),
     >        XCCAT(NGRP,0:NANIS,NGRP),DIFF(NGRP),DB2(NGRP),PNL(NGRP),
     >        XCNUF(NGRP),XCCHI(NGRP),XCTRC(NGRP),XCDIA(NGRP),AKEEP(8)
      DOUBLE PRECISION AFLNOR,FISOUR,AKEFF,AKEFFO,OLDBIL,DTHIRD,REFKEF,
     >                 B2VAL,B2VALO,BFLNOR,AKFRB,OOL,DONE,DZERO,
     >                 REBAL(NGRP,NGRP+1),BETA(NGRP),FLUN(NGRP),
     >                 PRODFS,OPRODF,AKEFF1,ANZFAC
      LOGICAL LACCOU,LACCIN,LAMH,LPRODF,LANZFA,LSPLIT,LORTHO,LREBAO,
     >        LDCOUT,LDCNTM,L2SECF,L2SECN,LMODE(20),LFANG,LOWTRK,
     >        LBETAS,LPJC,LPJCO,
     >        LEAKSW,LREBAL,NOVOID,LEXACT,LEXIN,
     >        LXSCAT,LEXTSO,LCWDR1,LCWAJ2
      CHARACTER*12 NAME,NAMFLX,CADJFL(2),CSTDDR,CSTDAJ,CADJCP,CADJFO,
     >             CCPDIR,DIRPIS,CANGAJ,CSAV
      CHARACTER*5  MESSIN, MESSOU
      PARAMETER (  ZERO=0.0,    ONE=1.0,
     >            DZERO=0.0D0, DONE=1.0D0, DTHIRD=DONE/3.0D0 )
      IF (ITYPEC .EQ. 0)  RETURN 
      IF ((ITYPEC.NE.6).AND.(ITYPEC.NE.1))  THEN
        CALL XABORT('MOC2AJ: INVALID ITYPEC VALUE FOR ADJOINT MODULE')
      ENDIF
      IF ((ISADJ.LT.1).OR.(ISADJ.GT.3))  THEN
        CALL XABORT('MOC2AJ: INVALID ISADJ VALUE FOR ADJOINT MODULE')
      ENDIF
      IF ((ISADJ.EQ.3).AND.(ITYPEC .NE.1))  THEN
        CALL XABORT('MOC2AJ: INVALID ITYPEC VALUE FOR GENERALIZED'// 
     >              'ADJOINT CALCULATION')
      ENDIF
      CALL MOCCPY((NMAT+1)*NGRP ,XSTOT  ,XMTOT )
      CALL MOCCPY((NMAT+1)*NGRP ,XSTRC  ,XMTRC )
      CALL MOCCPY((NMAT+1)*NGRP ,XSDIA  ,XMDIA )
      CALL MOCCPY((NMAT+1)*NGRP*NGRP*(NANIS+1) ,XSCAT ,XMCAT )
      OLDBIL = DZERO
      LEXACT = LEXIN
      LREBAO = LREBAL      
      CSTDDR = 'FLUXDIRECT  '
      CSTDAJ = 'FLUXADJOINT '
      CADJCP = 'PSEUDOADJ   '
      CANGAJ = 'ANGULAR ADJ'
      CCPDIR = 'P_JI DIR    '
      DIRPIS = 'PIS DIR     '
      OOL= DZERO
      OPRODF= DZERO
      EINNO = 1.0E0
      EBMX= 1.E-6
      ANZFAC = 1.0
      B2VAL = DZERO
      LSPLIT = LMODE(1)  
      LPJC  = LMODE(2) 
      LPJCO = LPJC
      BETASC = 0.0000000000
      LBETAS = LMODE(3)
      LFANG = LMODE(6)
      IF (LPJC) THEN
        LBETAS = .FALSE.
      ENDIF
      IF (LPJC .AND. LFANG) CALL XABORT('MOC2AJ: ANGULAR FLUX/ADJOINT 
     > CALCULATIONS IN MOCC RESPONSE MATRIX MODE  N/A')
      IF (IFTRK2 .GT. 0) THEN
        LOWTRK = .TRUE. 
      ELSE
        LOWTRK = .FALSE.
      ENDIF
      IF (LPJC)  LOWTRK = .FALSE.
      LOWTRK = .FALSE. 
      IF(LSPLIT) THEN
        NMODE = 1 
      ELSE
        NMODE = 0 
      ENDIF
      LCWDR1 = .TRUE.
      LCWAJ2 = .FALSE.
      AFLXDU = 0
      CALL SETARA(A,2*(2*NPHI)*NREG*NGSS,AFLXDU )
      CALL SETARA(A,2*NGRP*NREG,IPVEC)
      CALL XDRSET(A(IPVEC),2*NGRP*NREG,0.0)
      CALL SETARA(A,2*NGRP*NREG,IXVEC)
      CALL XDRSET(A(IXVEC),2*NGRP*NREG,0.0)
      CALL SETARA(A,2*NGRP*NREG,IYVEC)
      CALL XDRSET(A(IYVEC),2*NGRP*NREG,0.0)
      CALL SETARA(A,2*NGRP*NREG,IZVEC)
      CALL XDRSET(A(IZVEC),2*NGRP*NREG,0.0)
      CALL SETARA(A,2*NGRP*NREG,ISVEC)
      CALL XDRSET(A(ISVEC),2*NGRP*NREG,0.0)
      CALL SETARA(A,2*NGRP*NREG,IXVEC1 )
      CALL XDRSET(A(IXVEC1),2*NGRP*NREG,0.0)
      CALL SETARA(A,2*NGRP*NREG,IYVEC1 )
      CALL XDRSET(A(IYVEC1),2*NGRP*NREG,0.0)
      CALL SETARA(A,2*NGRP*NREG,IZVEC1 )
      CALL XDRSET(A(IZVEC1),2*NGRP*NREG,0.0)
      CALL SETARA(A,2*NGRP*NREG,ISVEC1 )
      CALL XDRSET(A(ISVEC1),2*NGRP*NREG,0.0)
      CALL SETARA(A       , 2* NGRP*NREG  , IRESD )
      CALL XDRSET(A(IRESD), 2* NGRP*NREG  , 0.0)
      IDCNTM = 0
      LORTHO = .TRUE.
      SCALE  = 1.0E0
      GAJSCH = 1
        IF(GAJSCH .EQ. 1) THEN
          LDCOUT = .FALSE. 
          L2SECF = .TRUE.
        ELSEIF(GAJSCH .EQ. 2) THEN
          LDCOUT = .TRUE. 
          L2SECF = .FALSE.
        ELSEIF(GAJSCH .EQ. 3) THEN
          LDCOUT = .FALSE. 
          L2SECF = .FALSE.
        ELSEIF(GAJSCH .EQ. 4) THEN
          LDCOUT = .TRUE. 
          L2SECF = .TRUE.
        ELSE 
          CALL XABORT('MOC2AJ:INVALID GAJSCH NUMBER : '//CHAR(GAJSCH))
        ENDIF
        LDCNTM = .NOT.LDCOUT
        L2SECN = .NOT.L2SECF 
        IF(LDCOUT.AND. L2SECN ) L2SECN = .FALSE.  
        IF(LDCNTM.AND. L2SECF ) L2SECF = .FALSE. 
        IF(IPRT .GT. 0) THEN 
        IF(LDCOUT) THEN
         IF(L2SECF) THEN
          WRITE(IOUT,'(1X,A55,/,A55,/,A55)') 
     >      '2 STEP (EACH OUTER LOOP) DECONTAMINATION SCHEME : ON',
     >      '  WITH 1nd STEP DECONTAM. AT EACH OUTER LOOP    : ON',
     >      '  WITH 2nd STEP DECONTAM. AT EACH OUTER LOOP    : OFF'
         ELSE
          WRITE(IOUT,'(1X,A55)') 
     >      '1 STEP DECONTAMINATION SCHEME AT EACH OUTER LOOP: ON'
         ENDIF
        ENDIF
        IF(LDCNTM) THEN 
         IF(L2SECN) THEN 
          WRITE(IOUT,'(1X,A55,/,A55)') 
     >     '2 STEP DECONTAMINATION SCHEME                    : ON',
     >     ' WITH 2nd STEP DECONTAMINATION AT EACH OUTER LOOP: ON'
          ELSE
          WRITE(IOUT,'(1X,A55,/,A55)') 
     >     '2 STEP DECONTAMINATION SCHEME                    : ON',
     >     ' WITH 2nd STEP DECONTAMINATION AT EACH OUTER LOOP: OFF'
          ENDIF
        ENDIF
      ENDIF
      AKEEP(1)=DONE
      AKEEP(2)=DONE
      AKEEP(3)=DONE
      AKEEP(5)=DONE
      AKEEP(6)=DONE
      AKEEP(7)=DONE
      IF( IPRT.GT.1 )THEN
         WRITE(IOUT,999)
      ENDIF
      IF (ISADJ.EQ.3 .AND. ITYPEC .EQ.1) THEN
        CALL LCMGET(IPFLUX,'K-EFFECTIVE',RKEFF)
      ENDIF
       LEXACT = .TRUE.
       LANZFA = .FALSE.
       LAMH   = .FALSE.
       LPRODF = .TRUE. 
       IF (LEAKSW .AND. .NOT.LPRODF) 
     >    CALL XABORT('MOC2AJ:LPRODF MODE NOT SELECT WHEN LEAKSW ON')
       ILOOP  =  0
       ILPCN0 =  0
       ILPCN1 =  0
       ILPCN2 =  0
       JM=0
       IF (LAMH) JM=JM+1
       IF (LPRODF) JM=JM+1
       IF (LANZFA) JM=JM+1
       IF(JM.GT.1.OR.JM.EQ.0)CALL XABORT('MOC2AJ:INVALID NORMLZ SELECT')
        AKEFF = RKEFF
        LACCOU= .TRUE.
        LACCIN= .TRUE.
        DO 1003 IG=1,NGRP
         PNL(IG)= DONE  
1003    CONTINUE
        DO 30 JG=1,NGRP
         DO 20 IG=1,NGSS
         DO 13 IR=-NALB,-1
            SIGANG(IG,IR,JG)= ONE-ALBEDO(-IR)
   13    CONTINUE
         SIGANG(IG,0,JG) = ZERO
         DO 10 IR=1,NMAT
            SIGANG(IG,IR,JG) = XSTRC(IR,JG) * YGSS(IG)
   10    CONTINUE
   20    CONTINUE
   30   CONTINUE   
        CALL LCMLEN(IPFLUX,CADJFL(1),ILCMLN,ILCMTY)
        CADJFO = CADJFL(1)
        IF (ILCMLN .NE. -1) THEN
         IF (CADJFL(1) .EQ. CSTDAJ) THEN
          CALL LCMLEN(IPFLUX,CADJCP,ILCMLN,ILCMTY)
          IF (ILCMLN.EQ. -1) THEN
            CADJFO = CADJCP
          ELSE
            CALL XABORT('MOC2AJ: '//CADJFL(1)//' AND '//CADJCP
     >       //' DIRECTORIES DO NOT EXIST')  
          ENDIF
         ELSE
          CALL XABORT('MOC2AJ:'//CADJFL(1)//' DIRECTORY DOES NOT EXIST')             
         ENDIF
        ENDIF
        CALL LCMSIX(IPFLUX,CADJFL(1),ILCMUP)
          CALL MOCAJS(IPFLUX,IPMACR,ISADJ ,NMAT ,NSOUT,NGRP,NREG,
     >                ADJSOU,ADJFXS,MATALB,LMODE,VOL)
        CALL LCMSIX(IPFLUX,CADJFL(1),ILCMDN) 
        CALL LCMSIX(IPFLUX,CADJFO,ILCMUP)
        DO 22 IG= 1, NGRP
         XCSOU(IG)= DZERO 
         WRITE(NAMFLX,'(4HFLUX,I3,1X)') IG
         CALL LCMLEN(IPFLUX,NAMFLX,ILCMLN,ILCMTY)
         IF(ILCMLN.EQ.NREG) THEN
          CALL LCMGET(IPFLUX,NAMFLX,ADJFLO(1,1,IG))
         ELSE
          CALL XABORT('MOC2AJ:INVALID DATA LENGTH OF ADJFLUX IN IPFLUX')
         ENDIF
         DO 12 IR= 1, NREG
            ADJFLX(1,IR,IG,2) = ADJFLO(IR,1,IG)
            IF (ITYPEC.EQ.6) THEN 
                 IF (ISADJ.EQ.2) THEN
                   ADJFLX(1,IR,IG,4) = ADJSOU(IR,IG)
                   XCSOU(IG)= XCSOU(IG) + ADJFLX(1,IR,IG,4)*VOL(IR)
                 ELSE
                   CALL XABORT('MOC2AJ: INVALID ISADJ FOR ITYPEC=6')
                 ENDIF
            ELSE
                 ADJFLX(1,IR,IG,4) = DZERO
            ENDIF
   12    CONTINUE   
   22   CONTINUE  
        CALL LCMSIX(IPFLUX,CADJFO,ILCMDN)
        IF (ISADJ.EQ.3 .AND. ITYPEC .EQ.1) THEN
         CALL LCMSIX(IPFLUX,CSTDDR,ILCMUP)
         DO 301 IG= 1, NGRP
         WRITE(NAMFLX,'(4HFLUX,I3,1X)') IG
         CALL LCMLEN(IPFLUX,NAMFLX,ILCMLN,ILCMTY)
         IF(ILCMLN.EQ.NREG) THEN
          CALL LCMGET(IPFLUX,NAMFLX,FLUXO(1,1,IG))
         ELSE
          CALL XABORT('MOC2AJ:INVALID DATA LENGTH OF '//
     >                'DIREC FLUX IN IPFLUX')
         ENDIF
301      CONTINUE  
         CALL LCMSIX(IPFLUX,CSTDDR,ILCMDN)      
         CALL LCMSIX(IPFLUX,CSTDAJ,ILCMUP)
         DO 302 IG= 1, NGRP
          WRITE(NAMFLX,'(4HFLUX,I3,1X)') IG
          CALL LCMLEN(IPFLUX,NAMFLX,ILCMLN,ILCMTY)
          IF(ILCMLN.EQ.NREG) THEN
           CALL LCMGET(IPFLUX,NAMFLX,ADJFLO(1,1,IG))
          ELSE
           CALL XABORT('MOC2AJ:INVALID DATA LENGTH OF STANDARD '//
     >                'ADJFLUX IN IPFLUX')
          ENDIF
302      CONTINUE  
         CALL LCMSIX(IPFLUX,CSTDAJ,ILCMDN)   
         CALL MOCSOT(IPRT,NREG,NGRP,NFUNL,FLUXO,ADJSOU,VOL)
        ENDIF
        IF( ITYPEC .EQ. 6 )THEN
         AKEFFO= DONE
         AKEFF=  DONE
         AFLNOR= DONE    
         AKEEP(1)= AKEFF
         AKEEP(2)= AKEFF
         AKEEP(3)= AKEFF
         AKEEP(4)= AKEFF
         AKEEP(5)= AKEFF
         AKEEP(6)= AKEFF   
         AKEEP(7)= AKEFF
         AKEEP(8)= AKEFF
        ELSE
         IF ((ITYPEC.EQ.1).AND.(ISADJ.EQ.1)) THEN
          CALL MOC2HM(NGRP,NREG,NANIS,NMAT,NFUNL,NIFIS,KEYANI,MATALB(1),
     >               VOL,ADJFLX(1,1,1,2),
     >               XSTOT,XSTRC,XSDIA,XSCAT,XSNUF,XSCHI,
     >               PHI,
     >               XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,IPRT,
     >               LCWAJ2,
     >               IPFLUX,DIRPIS,LEAKSW,.FALSE.,A(IXVEC),
     >               ADJFLX(1,1,1,4),XCSOU)
          PRODFS=0.0D0
          CALL MOCAKF( NGRP, NANIS, PHI, DB2,AKEFF,AFLNOR,LEAKSW,
     >                   XCTOT,XCDIA,XCCAT,XCNUF,XCCHI, IPRT,PRODFS,
     >                   OLDBIL)
          IF (OPRODF.EQ.DZERO)  OPRODF=PRODFS
          IF (LAMH) THEN
           CALL MOCAMH(1,NBN,NGRP,NANIS,           
     >             PHI,XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,
     >             DIFF,BETA,REBAL,FLUN,AKFRB,B2VAL,OOL,EBMX,1,
     >             AKEFF1)
           AKEFF=AKEFF1
           AFLNOR=1.0D0/AKEFF
          ENDIF
           DO 2200 IG= 1, NGRP
              DO 2210 IR= 1, NREG
                 DO 2220 IS= 1, NFUNL
C         --- IF MULTIGROUP NORMALIZATION IS SELLECTED ---
          IF (LAMH)  ADJFLX(IS,IR,IG,2)= ADJFLX(IS,IR,IG,2)
     >                                  * FLUN(IG)
          IF(LPRODF) ADJFLX(IS,IR,IG,2)=ADJFLX(IS,IR,IG,2)/PRODFS
2220            CONTINUE
2210         CONTINUE 
2200       CONTINUE
         ENDIF
         AKEFFO= AKEFF
         AKEEP(1)= AKEFF
         AKEEP(2)= AKEFF
         AKEEP(3)= AKEFF
         AKEEP(4)= AKEFF
         AKEEP(5)= AKEFF
         AKEEP(6)= AKEFF   
         AKEEP(7)= AKEFF
         AKEEP(8)= AKEFF
         AFLNOR= 1.0/AKEFF
        ENDIF
      CTOT = IACITR+IFRITR
      IF( IACITR.EQ.0 )THEN
         CPTM=CTOT+1
      ELSE
         CPTM= IACITR
      ENDIF
      MAXITR= 0
      NITR   = 0
      NITRLT = 0
      NALGIT= 0
      EIMX   = DBLE(AIEPSF)
      EIMXO  = EIMX
      NINTMO = NINTMX
      EEMX=   AEEPSF
      MESSOU= '     '
      IF ( ISADJ .EQ.3 .AND. ITYPEC.EQ.1 ) THEN
       CALL MOCSCL(NANIS ,NREG  ,NGRP ,NFUNL ,NIFIS ,NMAT ,NSOUT,
     >             ADJFLO,XSCHI ,XSNUF,ADJSOU,MATALB,XSCAT,XSDIA,
     >             SCALE ,IPRT  ,RKEFF)
       CALL MOCGAD(IPFLUX,NREG ,NGRP  ,NFUNL ,NIFIS ,NMAT,NSOUT,
     >             ADJFLO,FLUXO,XSCHI ,XSNUF ,ADJFLX(1,1,1,2),
     >             MATALB,VOL  ,SCALE ,LORTHO,IPRT)
      ENDIF
      CALL MOCRWD( IFTRAK )
      IF (IFTRK2.GT.0)  CALL MOCRWD( IFTRK2 )
      IF (LEAKSW) THEN 
          IGDEB = 1
          JGDEB = 0
          LOWTRK = .FALSE.
            CALL MOCPIS(IPFLUX,DIRPIS,NREG ,NGRP ,NL,NMAT,NANIS,NSOUT,
     >                  NIFIS ,NGSS  ,NALB ,MXGAUS,IFTRK2,IFTRAK,NTRK2,
     >                  NTRK  ,MXSEG,MATALB,NRSEG ,SIGANG,XSDIA ,XSCAT,
     >                  XTINV,XSTRC,SEGLEN,ALBEDO,EXPT,EXP2,WGSS,YGSS,
     >                  NGSS2,FLM,FLP,CYM,CYP,LOWTRK,IPRT,VOL,VOLINV,
     >                  MAXITR,IGDEB ,JGDEB,A(IXVEC),A(IYVEC),A(IZVEC),
     >                  A(ISVEC) )
      ENDIF
C     BEGIN  OUTER LOOP 
      DO 210 IT=1,NEXTMX
         MESSIN='     '
         IGDEB = 1
         JGDEB = 0
         IF (ITYPEC.LT.6) THEN
            DO 92 IG= 1, NGRP
               DO 91 IR= 1,NREG
                  ADJFLX(1,IR,IG,4)= DZERO
                  IF(ISADJ.EQ.2 .OR. ISADJ.EQ.3) THEN 
                  ADJFLX(1,IR,IG,4)= ADJSOU(IR,IG)
                  ENDIF
   91          CONTINUE
   92       CONTINUE
            DO 96 IR= 1,NREG
               DO 95 IS= 1,NIFIS
                  FISOUR= DZERO
                  DO 93 IG= 1, NGRP
                     FISOUR= FISOUR+
     >                       XSCHI(MATALB(IR),IS,IG)*ADJFLX(1,IR,IG,2)
   93             CONTINUE
                  FISOUR= FISOUR*AFLNOR
                  DO 94 IG= 1,NGRP
                     ADJFLX(1,IR,IG,4)= ADJFLX(1,IR,IG,4)+
     >                              XSNUF(MATALB(IR),IS,IG)*FISOUR
   94             CONTINUE
   95          CONTINUE
   96       CONTINUE
            DO 98 IG= 1, NGRP
               XCSOU(IG)= DZERO
               DO 97 IR= 1,NREG
                  XCSOU(IG)= XCSOU(IG)+ADJFLX(1,IR,IG,4)*VOL(IR)
   97          CONTINUE
   98       CONTINUE
         ENDIF
         DO 783 IG= 1, NGRP
            DO 782 IR= 1, NREG
               DO 781 IS= 1, NFUNL
                  ADJFLX(IS,IR,IG,6)= ADJFLX(IS,IR,IG,2)
                  ADJFLX(IS,IR,IG,7)= ADJFLX(IS,IR,IG,2)
  781          CONTINUE
  782       CONTINUE
  783    CONTINUE
11111    CONTINUE 
         IF (LSPLIT) THEN
           IF (ILOOP.EQ.0.AND.NGRP.GE.GMXUPS.AND.(GMXUPS.GT.1))THEN
             ILOOP = 1
             IGDEB = GMXUPS
             JGDEB = 0
           ELSEIF (ILOOP.EQ.1.AND.NGRP.GE.GMXUPS.AND.(GMXUPS.GT.1))THEN
             ILOOP = 2
             IGDEB = 1
             JGDEB = NGRP - ( GMXUPS-1 )
           ENDIF
         ENDIF
         IF (LBETAS) THEN
          BETASC = 0.10000000000 
         ELSE 
          BETASC = 0.00000000000  
         ENDIF 
         IF (LBETAS .OR. LOWTRK) THEN
          EIMX   = 1.0D-02 
         ELSE
          EIMX =    EIMXO 
         ENDIF 
            CALL MOCSIG(NGSS,NGSS2,NALB,NMAT,NGRP,NANIS,NSOUT,MXGAUS,
     >                  ALBEDO,XSTOT,XSTRC,XSCAT,XSDIA,XTINV,XGSS,YGSS,
     >                  WGSS,XMTOT,XMTRC,XMCAT,XMDIA,LCACA,
     >                  SIGANG,BETASC,IFTRK2,LOWTRK)
         IF (IT.EQ.1 .AND. ILOOP.EQ.NMODE .AND. LPJC) THEN
          IF (LOWTRK) 
     >    CALL XABORT('MOC2AJ:LOWTRK ARE NOT APPLICABLE FOR CP EVAL.')
          IPIJ  = 0
          IQSOU = 0
          CALL SETARA(A,2*NREG*NREG,IPIJ  )
          CALL SETARA(A,2*NREG*NREG,IQSOU )
          IF (IPIJ .EQ. 0 .OR. IQSOU .EQ. 0)  THEN
            CALL XABORT('MOC2AJ: INSUFFICIENT MEM FOR PIJ OR QSOU ')
          ENDIF
            CALL MOCCP2(IPFLUX,CCPDIR,NREG,NGRP,NL,NMAT,NANIS,NSOUT,
     >                  NIFIS,NGSS,NALB,MXGAUS,IFTRK2,IFTRAK,NTRK2,NTRK,
     >                  MXSEG,MATALB,NRSEG,SIGANG,XSDIA,XSCAT,XTINV,
     >                  XSTRC,SEGLEN,ALBEDO,EXPT,EXP2,WGSS,YGSS,NGSS2,
     >                  FLM,FLP,CYM,CYP,LOWTRK,IPRT,VOL,VOLINV,MAXITR,
     >                  IGDEB,JGDEB,A(IXVEC),A(IYVEC),A(IZVEC),A(ISVEC),
     >                  A(IPIJ))
            CALL RLSARA(A(IPIJ))
            CALL RLSARA(A(IQSOU))
         ENDIF
         NINTMX = NINTMO
C     BEGIN  INNER LOOP 
      DO 500 JT= 1, NINTMX
             IF (ILOOP.EQ.1  ) THEN
               NSTEP  =  -1 
             ELSE IF (ILOOP.EQ.2 .OR. ILOOP.EQ.0) THEN
               NSTEP =   -1
             ENDIF
             IF(NSTEP .EQ. +1) THEN
                 NSTART =  IGDEB
                 NSTOP  =  (NGRP-JGDEB)
             ELSEIF(NSTEP .EQ. -1) THEN
                 NSTART =  (NGRP-JGDEB)
                 NSTOP  =  IGDEB
             ELSE
               CALL XABORT('MOC2AJ: INVALID NSTEP VALUES')
             ENDIF
          DO 45 IG= NSTART, NSTOP , NSTEP 
               IF (LOWTRK .AND. IFTRK2.GT.0 ) THEN
                  IFFIL   = IFTRK2
                  NTRACK  = NTRK2
                  NGSSCL  = NGSS2
               ELSE
                  IFFIL   = IFTRAK
                  NTRACK  = NTRK
                  NGSSCL  = NGSS
               ENDIF
               IF (IG .LE. ( GMXUPS-1 ) ) THEN
                NINGR  =   1
               ELSE
                 NINGR  =   1 
               ENDIF
           DO 451 INGR= 1, NINGR
            IF( NL.EQ.0 )THEN
             LXSCAT = .TRUE. 
             LEXTSO = .TRUE.
             IF (LPJC) THEN
               CALL SETARA(A      ,NREG*NREG  , ICP )
               CALL MOCCWP(IPFLUX,CCPDIR,NREG,NGRP,NMAT ,NANIS ,NSOUT,
     >                  MATALB,XSDIA ,XSCAT,LXSCAT,LEXTSO,
     >                  IG ,NGRP-IG , LCWAJ2 ,ADJFLX(1,1,1,7),
     >                  ADJFLX(1,1,1,4),ADJFLX(1,1,1,7),A(ISVEC),A(ICP))
               CALL RLSARA( A( ICP  ) )
             ELSE
              CALL MOCSWX(IPFLUX,IPRT,NREG,NGRP,NL,NMAT,NANIS,NSOUT,
     >                  NIFIS,NGSS,NPHI,NALB,MXGAUS,IFTRK2,IFTRAK,NTRK2,
     >                  NTRK ,MXSEG,IPTRK,IPTRK2,NPHI,NANGL2,
     >                  MATALB,NRSEG ,SIGANG ,XSDIA ,XSCAT ,XTINV,
     >                  SEGLEN,ALBEDO,EXPT   ,EXP2  ,WGSS  ,YGSS,NGSS2,
     >                  FLM   ,FLP   ,CYM    ,CYP   ,LXSCAT,LOWTRK,
     >                  VOLINV,LEXTSO,IG   ,NGRP-IG ,IDUMMY,LCWAJ2,
     >                  ADJFLX(1,1,1,7),ADJFLX(1,1,1,4),ADJFLX(1,1,1,7),
     >                  A(ISVEC),ANGADJ,LMODE )
             ENDIF
            ELSE
               CALL XABORT('MOC2AJ: ANISOTROPIC OPTION *N/A* YET')
            ENDIF
            CALL MOCRWD( IFTRAK )
            IF (IFTRK2.GT.0.AND.LOWTRK)  CALL MOCRWD( IFTRK2 )
            MAXITR= MAXITR+1
            IF (LOWTRK) THEN 
              NITRLT = NITRLT+1
            ELSE
              NITR= NITR+1       
            ENDIF
            EINN= DZERO
            GINN= DZERO
            FINN= DZERO
            IF (NINGR.GT.1) THEN
             IF( LREBAL )THEN                                                     
              CALL MOC2HM(NGRP,NREG,NANIS,NMAT,NFUNL,NIFIS,KEYANI,
     >               MATALB(1),VOL,ADJFLX(1,1,1,7),
     >               XSTOT,XSTRC,XSDIA,XSCAT,XSNUF,XSCHI,
     >               PHI,
     >               XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,IPRT,
     >               LCWAJ2,
     >               IPFLUX,DIRPIS,LEAKSW, .TRUE. ,A(IXVEC),
     >               ADJFLX(1,1,1,4),XCSOU)
             CALL MOC2BL(NGRP,IGDEB,NANIS,PHI,XCSOU,XCTRC,XCDIA,XCCAT,
     >                   DB2,REBAL,FLUN,IPRT ,JGDEB,LCWAJ2)
             DO 1301 IGDUM= IGDEB, (NGRP-JGDEB)
             DO 1311 IR= 1, NREG                                               
               DO 1321 IS= 1, NFUNL                                          
                ADJFLX(IS,IR,IGDUM,7)=ADJFLX(IS,IR,IGDUM,7)*FLUN(IGDUM) 
 1321          CONTINUE
 1311        CONTINUE
 1301        CONTINUE
             ENDIF
             DO 2221 IR= 1, NREG
               DO 2211 IA= 1, NFUNL
               GINN=MAX(GINN,ABS(ADJFLX(IA,IR,IG,6)-ADJFLX(IA,IR,IG,7)))
                FINN= MAX(FINN,ABS(ADJFLX(IA,IR,IG,7)))
                ADJFLX(IA,IR,IG,5) = ADJFLX(IA,IR,IG,6)
                ADJFLX(IA,IR,IG,6)= ADJFLX(IA,IR,IG,7)
 2211          CONTINUE
 2221        CONTINUE
             IF (FINN .GT. DZERO) THEN
               GINN= GINN/FINN
             ELSE
               GINN= 1.0D+01
             ENDIF
             EINN=MAX(EINN,GINN)
            ENDIF
            IF( EINN.LT.EIMX )  GO TO 45
  451      CONTINUE
   45     CONTINUE
          IF( LREBAL )THEN                                                     
           CALL MOC2HM(NGRP,NREG,NANIS,NMAT,NFUNL,NIFIS,KEYANI,
     >                 MATALB(1),VOL,ADJFLX(1,1,1,7),XSTOT,XSTRC,XSDIA,
     >                 XSCAT,XSNUF,XSCHI,PHI,XCTOT,XCTRC,XCDIA,XCCAT,
     >                 XCNUF,XCCHI,XCABS,IPRT,LCWAJ2,IPFLUX,DIRPIS,
     >                 LEAKSW,.TRUE.,A(IXVEC),ADJFLX(1,1,1,4),XCSOU )
           CALL MOC2BL(NGRP,IGDEB,NANIS,PHI,XCSOU,XCTRC,XCDIA,XCCAT,DB2,
     >                 REBAL,FLUN,IPRT ,JGDEB,LCWAJ2)
           DO 1300 IG= IGDEB, (NGRP-JGDEB)
            DO 1310 IR= 1, NREG                                               
               DO 1320 IS= 1, NFUNL                                          
                ADJFLX(IS,IR,IG,7)= ADJFLX(IS,IR,IG,7) * FLUN(IG)       
 1320          CONTINUE
 1310       CONTINUE
 1300      CONTINUE
         ENDIF
         IF (LACCIN) THEN                                     
           IF( MOD(JT-1,CTOT).GE.CPTM )THEN                                     
            CALL MOC2AC( NGRP, NFUNL, NREG, ADJFLX(1,1,1,5), AKEEP(5),
     >                   IGDEB, MU  , JGDEB )
           ELSE
            MU= ONE
           ENDIF
         ENDIF
         EINN= DZERO
         IGDEBO = IGDEB
         JGDEBO = JGDEB  
         JGCOUN = 0
         DO 220 IG= IGDEBO, (NGRP-JGDEBO)
           GINN= DZERO
           FINN= DZERO
           DO 222 IR= 1, NREG
             DO 221 IA= 1, NFUNL
              GINN= MAX(GINN,ABS(ADJFLX(IA,IR,IG,6)-ADJFLX(IA,IR,IG,7)))
              FINN= MAX(FINN,ABS(ADJFLX(IA,IR,IG,7)))
              ADJFLX(IA,IR,IG,5) = ADJFLX(IA,IR,IG,6)
              ADJFLX(IA,IR,IG,6)= ADJFLX(IA,IR,IG,7)
  221        CONTINUE
  222      CONTINUE
           IF (FINN .GT. DZERO) THEN
               GINN= GINN/FINN
           ELSE
               GINN= 1.0D+01
           ENDIF
           IF( GINN.LT.EIMX.AND.IGDEB.EQ.IG ) THEN
CCC             IGDEB= IGDEB+1
           ENDIF
           IF( GINN.LT.EIMX  ) THEN
             JGCOUN = JGCOUN + 1
           ELSE
             JGCOUN = 0
           ENDIF
           EINN=MAX(EINN,GINN)
  220    CONTINUE
         JGDEB = JGDEBO + JGCOUN
         ITERF= JT
         IF (ILOOP.EQ.0)  ILPCN0 = ILPCN0 + 1
         IF (ILOOP.EQ.1)  ILPCN1 = ILPCN1 + 1
         IF (ILOOP.EQ.2)  ILPCN2 = ILPCN2 + 1
         IF( IPRT.GT.0 )THEN
           WRITE(IOUT,2062)
           WRITE(IOUT,2064) JT,EINN,EIMX,IGDEB,MU
         ENDIF
         IF( EINN.LT.EIMX )  GO TO 750
  500  CONTINUE
       MESSIN='*NOT*'
  750  CONTINUE
       EIMX   = EIMXO
       IF( IPRT.GE.1 )  WRITE(IOUT,1060) 'INTERNAL',MESSIN,ITERF
       IF (ILOOP .EQ. 1) THEN
         GOTO 11111
       ENDIF
       IF (ILOOP .EQ. 2) THEN
         ILOOP = 0
       ENDIF
       DO 763 IG= 1, NGRP
          DO 762 IR= 1, NREG
             DO 761 IS= 1, NFUNL
                ADJFLX(IS,IR,IG,3)= ADJFLX(IS,IR,IG,7)
  761        CONTINUE
  762     CONTINUE
  763  CONTINUE
       IF ( (ITYPEC.EQ.1).AND.(ISADJ.EQ.1) ) THEN
         CALL MOC2HM(NGRP,NREG,NANIS,NMAT,NFUNL,NIFIS,KEYANI,MATALB(1),
     >               VOL,ADJFLX(1,1,1,3),XSTOT,XSTRC,XSDIA,XSCAT,XSNUF,
     >               XSCHI,PHI,XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,
     >               XCABS,IPRT,LCWAJ2,IPFLUX,DIRPIS,LEAKSW,.FALSE.,
     >               A(IXVEC),ADJFLX(1,1,1,4),XCSOU)
            CALL MOCAKF( NGRP, NANIS, PHI, DB2,AKEFF,AFLNOR,LEAKSW,
     >                   XCTOT,XCDIA,XCCAT,XCNUF,XCCHI, IPRT,PRODFS,
     >                   OLDBIL)
        IF (LAMH) THEN
         CALL MOCAMH(ITYPEC,NBN,NGRP,NANIS,           
     >       PHI,XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,
     >       DIFF,BETA,REBAL,FLUN,AKFRB,B2VAL,OOL,EBMX,1,AKEFF1)
         AKEFF=AKEFF1
         AFLNOR=1.0D0/AKEFF
        ENDIF 
        DO 12000 IG= 1, NGRP
          DO 12100 IR= 1, NREG                       
          DO 12200 IS= 1, NFUNL
          IF(LAMH) ADJFLX(IS,IR,IG,3)=ADJFLX(IS,IR,IG,3)*FLUN(IG)
          IF(LPRODF) ADJFLX(IS,IR,IG,3)=ADJFLX(IS,IR,IG,3)/PRODFS
12200     CONTINUE
12100     CONTINUE
12000   CONTINUE
        IF (LANZFA)  THEN 
         CALL MOCANZ(NGRP,NREG,NFUNL,ADJFLX(1,1,1,3),ANZFAC)
        ENDIF
       ENDIF
       IEXTF= IT
       IF (ITYPEC.EQ.1 .AND. ISADJ.EQ.1) THEN
         EEXT= ABS(AKEFF-AKEFFO)/AKEFF
       ELSE
         EEXT= DZERO
       ENDIF
       AKEEP(3)= AKEFF
       IF (LACCOU) THEN  
         IF( MOD(IT-1,CTOT).GE.CPTM )THEN
           CALL MOC2AC(NGRP,NFUNL,NREG,ADJFLX(1,1,1,1),AKEEP(1),1,MU,0)
         ELSE
            MU= ONE 
         ENDIF
       ENDIF
       IF (LREBAL.AND.ISADJ.EQ.3.AND.ITYPEC.EQ.1) THEN
        DO 13000 IG= 1, NGRP
        DO 13100 IR= 1, NREG                       
        DO 13200 IS= 1, NFUNL
        IF(ADJFLX(IS,IR,IG,3) .LT. DZERO) THEN
         WRITE(IOUT,'(1X,A22,I4,2X,A4,I4,A2,2X,E14.6)') 
     >    'NEGATIVE BIASED T*(IR=',IR,',IG=',IG,')=',ADJFLX(IS,IR,IG,3)
        ENDIF
13200   CONTINUE
13100   CONTINUE
13000   CONTINUE
       ENDIF
      IF (ISADJ.EQ.3.AND.ITYPEC.EQ.1 
     >     .AND.(LDCOUT.OR.IDCNTM.EQ.1)) THEN
       CALL MOCGAD(IPFLUX,NREG ,NGRP  ,NFUNL ,NIFIS ,NMAT,NSOUT,
     >             ADJFLO,FLUXO,XSCHI ,XSNUF ,ADJFLX(1,1,1,3),
     >             MATALB,VOL  ,SCALE ,LORTHO,IPRT)
       IDCNTM = 2
      ENDIF
       IF(ITYPEC.LT.6)THEN
          IF(IPRT.GT.0) WRITE(IOUT,2065) IT,EEXT,EEMX,AKEFF,B2VAL
          EINN= DZERO
          IF(EEXT.LT.1.0E-0)THEN
              EINN= DZERO
              DO 322 IG= 1, NGRP
               GINN= DZERO
               FINN= DZERO
               DO 321 IR= 1, NREG
                DO 320 IA= 1, NFUNL
              GINN=MAX(GINN,ABS(ADJFLX(IA,IR,IG,2)-ADJFLX(IA,IR,IG,3)))
                  FINN= MAX(FINN,ABS(ADJFLX(IA,IR,IG,3)))
                  ADJFLX(IA,IR,IG,1)= ADJFLX(IA,IR,IG,2)
                  ADJFLX(IA,IR,IG,2)= ADJFLX(IA,IR,IG,3)
  320           CONTINUE
  321          CONTINUE
               IF (FINN .GT. DZERO) THEN
                 GINN= GINN/FINN
               ELSE
                 CALL XABORT('MOC2AJ: OUTER FINN =0 ')        
               ENDIF
               EINN= MAX(EINN,GINN)
  322         CONTINUE
              IF(IPRT.GT.0)  WRITE(IOUT,2066) IT,EINN,EIMX,AFLNOR,MU
          ELSE
            DO 325 IG= 1, NGRP
            DO 324 IR= 1, NREG
            DO 323 IA= 1, NFUNL
                ADJFLX(IA,IR,IG,1)= ADJFLX(IA,IR,IG,2)
                ADJFLX(IA,IR,IG,2)= ADJFLX(IA,IR,IG,3)
  323       CONTINUE
  324       CONTINUE
  325       CONTINUE
            IF( IPRT.GT.0 )  WRITE(IOUT,2067) IT,AFLNOR,MU
          ENDIF
       ELSE
         DO 425 IG= 1, NGRP
         DO 424 IR= 1, NREG
         DO 423 IA= 1, NFUNL
             ADJFLX(IA,IR,IG,1)= ADJFLX(IA,IR,IG,2)
             ADJFLX(IA,IR,IG,2)= ADJFLX(IA,IR,IG,3)
  423    CONTINUE
  424    CONTINUE
  425    CONTINUE
       ENDIF
        AKEEP(1)= AKEEP(2)
        AKEEP(2)= AKEEP(3)
        AFLNOR= (AKEFF/AKEEP(3))*AFLNOR
        AKEFFO= AKEFF
        IF (EEXT.LT. 1.0E-02.AND.EINN.LT. 1.0D-02.AND.LBETAS)THEN
         LBETAS = .FALSE.
        ENDIF
        IF (EEXT.LT.1.0E-02.AND.EINN.LT.1.0D-02.AND.LOWTRK)THEN
         LOWTRK = .FALSE.
        ENDIF
        IF((EEXT.LT.EEMX).AND.(EINN.LT.EIMX))THEN
          IF ( ISADJ.EQ.3 .AND. ITYPEC  .EQ.1 .AND. 
     >       ( LDCNTM   .OR.  L2SECN  .OR.  L2SECF    )  ) THEN
             IF (L2SECN ) LDCOUT = .TRUE.
             IF (L2SECF ) LDCOUT = .FALSE.
             IF (LDCNTM ) IDCNTM = 1
             LREBAL = .FALSE.
             SCALE  = 0.0E0
             LDCNTM = .FALSE.
             L2SECN = .FALSE.
             L2SECF = .FALSE.
             CALL MOCGAD(IPFLUX,NREG ,NGRP  ,NFUNL ,NIFIS ,NMAT,NSOUT,
     >                   ADJFLO,FLUXO,XSCHI ,XSNUF ,ADJFLX(1,1,1,2),
     >                   MATALB,VOL  ,SCALE ,LORTHO,IPRT)
            WRITE(IOUT,'(1X,A42)') 
     >         'TURN ON DECONTAMINATION AND TURN OFF REBAL'  
            WRITE(IOUT,'(1X,A7,E14.6)') 'SCALE =',SCALE
            WRITE(IOUT,'(1X,A7,I4,2X,A7,I4)') 
     >         'IDCNTM=',IDCNTM,'LREBAL=',LREBAL
            GO TO 209
          ENDIF
          IF(LEXACT) GO TO 240
 209      CONTINUE
          IGDEB = 1
          JGDEB = 0
          LEXACT= .TRUE.
          IF(IPRT.GT.1) WRITE(IOUT,7000)
        ENDIF
        EINNO = EINN
 210  CONTINUE
      IF( IPRT.GT.0 )THEN
         WRITE(IOUT,7001)
      ENDIF
      MESSOU= '*NOT*'
 240  CONTINUE
      IF (ISADJ .EQ.3 .AND. ITYPEC .EQ.1) THEN
       SCALE = 0.0E0
       LORTHO= .TRUE.
       CALL MOCGAD(IPFLUX,NREG ,NGRP  ,NFUNL ,NIFIS ,NMAT,NSOUT,
     >             ADJFLO,FLUXO,XSCHI ,XSNUF ,ADJFLX(1,1,1,3),
     >             MATALB,VOL  ,SCALE ,LORTHO,IPRT)
       DO 672 IG= 1, NGRP
       DO 671 IR= 1, NREG
            ADJFLX(1,IR,IG,1)= ADJFLX(1,IR,IG,3)
 671   CONTINUE
 672   CONTINUE
       SCALE = 0.0E0
       LORTHO= .FALSE.
       CALL MOCGAD(IPFLUX,NREG ,NGRP  ,NFUNL ,NIFIS ,NMAT,NSOUT,
     >             ADJFLO,FLUXO,XSCHI ,XSNUF ,ADJFLX(1,1,1,3),
     >             MATALB,VOL  ,SCALE ,LORTHO,IPRT)
      ENDIF
      CALL MOCFIM(NREG  ,NGRP  ,NL,NMAT,NANIS ,NIFIS,NFUNL,
     >            AKEFF ,ISADJ ,ITYPEC ,NSOUT ,MATALB,XSDIA,
     >            XSCAT ,ADJSOU,XSNUF  ,XSCHI ,ADJFLX)
       IF( IPRT.GT.0 )THEN
         WRITE(IOUT,1060) 'EXTERNAL',MESSOU,IEXTF
         WRITE(IOUT,1061) MAXITR,AKEFF,B2VAL,EEXT
         WRITE(IOUT,2070) ILPCN0
         WRITE(IOUT,2071) ILPCN1
         WRITE(IOUT,2072) ILPCN2
       ENDIF
      RKEFF=AKEFF
      LREBAL=LREBAO 
      LPJC=LPJCO
      IF(IPRT.GE.2)THEN
         WRITE(IOUT,1030) (IR,IR=1,NREG)
         DO 161 IG=1,NGRP
            WRITE(IOUT,1090) IG
            WRITE(IOUT,1040) (ADJFLX(1,IR,IG,3)*AFLNOR,IR=1,NREG)
            DO 166 IA= 2, NFUNL
               WRITE(IOUT,1045) IA,
     >         (ADJFLX(IA,IR,IG,3)*AFLNOR,IR=1,NREG)
  166       CONTINUE 
  161    CONTINUE
      ENDIF
      IF (ISADJ.EQ.3.AND.ITYPEC.EQ.1) THEN
       DO 6721 IG=1,NGRP
       DO 6711 IR=1,NREG
          ADJSOU(IR,IG)=ADJFLX(1,IR,IG,1)
6711   CONTINUE
6721   CONTINUE
       NAME(1:1) = 'O'
       NAME(2:4) = CADJFL(1)(1:3)
       NAME(5:12)= CADJFL(1)(5:12)
       CALL LCMSIX(IPFLUX,NAME,ILCMUP)
       CALL MOCSAJ(IPFLUX,NGRP,NREG,ADJSOU )
       CALL LCMSIX(IPFLUX,NAME,ILCMDN)
      ENDIF
      WRITE(IOUT,'(1X,A46,2X,I4)') 
     >  'MAX NO OF GRP ITER. EXCLD CG ALGO(LOW ORDER) =', NITRLT
      WRITE(IOUT,'(1X,A46,2X,I4)') 
     >  'MAX NO OF GRP ITER. EXCLD CG ALGO(HIGH ORDER)=', NITR
      WRITE(IOUT,'(1X,A30,2X,I4)')  
     >  'MAX NO OF BICGSTAB ALGO ITER.=',NALGIT
      CALL RLSARA(A(IPVEC))
      CALL RLSARA(A(IXVEC))
      CALL RLSARA(A(IYVEC))
      CALL RLSARA(A(IZVEC))
      CALL RLSARA(A(ISVEC))
      CALL RLSARA(A(IXVEC1))
      CALL RLSARA(A(IYVEC1))
      CALL RLSARA(A(IZVEC1))
      CALL RLSARA(A(ISVEC1))
      CALL RLSARA(A(IRESD))
       DO 773 IG= 1, NGRP
       DO 772 IR= 1, NREG
       DO 771 IS= 1, NFUNL
                ADJFLO(IR,IS,IG)= ADJFLX(IS,IR,IG,3)
  771  CONTINUE
  772  CONTINUE
  773  CONTINUE
       CALL LCMSIX(IPFLUX,CADJFL(1),ILCMUP)
       CALL MOCSAJ(IPFLUX,NGRP,NREG,ADJFLO)
       CALL LCMPUT(IPFLUX,'ADJ K-EFF   ',1,2,RKEFF)
       CALL LCMSIX(IPFLUX,CADJFL(1),ILCMDN)
       DO 776 IG= 1, NGRP
       DO 775 IR= 1, NREG
       DO 774 IS= 1, NFUNL
           ADJFLO(IR,IS,IG)= ADJFLX(IS,IR,IG,4)
  774  CONTINUE
  775  CONTINUE
  776  CONTINUE
       CALL LCMSIX(IPFLUX,CADJFL(2),ILCMUP)
       CALL MOCSAJ(IPFLUX,NGRP,NREG,ADJFLO)
       CALL LCMPUT(IPFLUX,'ADJ K-EFF   ',1,2,RKEFF)
       CALL LCMSIX(IPFLUX,CADJFL(2),ILCMDN)
       CALL LCMSIX(IPFLUX,CADJFL(1),ILCMUP)
       IF (LFANG) THEN 
         CALL MOCSAC(IPFLUX,IPRT,CANGAJ,NGRP,NPHI,NGSS,NREG,
     >               ANGADJ,A(AFLXDU))
       ENDIF
      CALL LCMSIX(IPFLUX,CADJFL(1),ILCMDN)
      CALL RLSARA(A(AFLXDU) )
      RETURN
  999 FORMAT (/38H SOLVING ADJOINT TRANSPORT EQUATION   //)
 1010 FORMAT (/28H PROPERTIES OF THE GEOMETRY:/
     1 (9X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,
     2 I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,
     3 2H/=,I4))
 1020 FORMAT (1H ,A6,2H: ,1P,10E12.5/(9X,10E12.5))
 1025 FORMAT (1H ,A2,1H(,I2,3H): ,1P,10E12.5/(9X,10E12.5))
 1030 FORMAT (/36H ADJOINT FLUXES AVERAGED           :/
     1 (9X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,
     2 I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,
     3 2H/=,I4))
 1035 FORMAT (/28H CURRENT AVERAGED          :/
     1 (9X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,
     2 I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,
     3 2H/=,I4))
 1040 FORMAT (15H ADJOINT FLUX  ,2H: ,1P,10E12.5/(9X,10E12.5))
 1045 FORMAT (5H CUR ,I2,2H: ,1P,10E12.5/(9X,10E12.5))
 1055 FORMAT (8H SOURCE ,2H: ,1P,10E12.5/(9X,10E12.5))
 1060 FORMAT ( 1X,A8,
     1            13H CONVERGENCE ,A5,14H REACHED AFTER ,I6,
     2            12H ITERATIONS.  )
 1061 FORMAT (/1X,18H TRACKING CALLED= ,I6,6H TIMES ,
     1            13H FINAL KEFF= ,1P,D15.7,5H, B2=,E15.6,
     1            11H PRECISION=,E9.2)
 1067 FORMAT ( 10X,14H  GROUP CONV= ,100I1)
 1070 FORMAT (/21H FINAL VALUE OF KEFF=  ,1P,D15.7 )
 1080 FORMAT (11H ITERATION ,I4)
 1090 FORMAT (/14H ENERGY GROUP ,I6)
 1100 FORMAT (23H ADJOINT FLUX AVERAGED ,2H: ,1P,10E12.5/(9X,10E12.5))
 2062 FORMAT (     1X,24HADJOINT FLUX CALCULATION )
 2064 FORMAT (     10X,3HIN(,I3,6H) FLX:,
     1             5H PRC=,1P,E9.2,
     2             5H TAR=,   E9.2,
     3             7H IGDEB=, I13,
     4             6H ACCE=,0P,F12.5)
 2065 FORMAT (     5H OUT(,I3,6H) EIG:,
     1             5H PRC=,1P,E9.2,
     2             5H TAR=,   E9.2,
     3             6H KEFF=,D15.7,
     4             6H BUCK=,E12.5)
 2066 FORMAT (     5H OUT(,I3,6H) FLX:,
     1             5H PRC=,1P,E9.2,
     2             5H TAR=,   E9.2,
     3             6H FNOR=,   E13.6,
     4             6H ACCE=,0P,F12.5)
 2067 FORMAT (     5H OUT(,I3,6H) FLX:,
     1             28X,
     3             6H FNOR=,1P,E13.6,
     4             6H ACCE=,0P,F12.5)
 2070 FORMAT (34H TOTAL NO. OF INNER ITERATIONS FOR,
     1        25H NON-GROUPSPLITING =     ,I6)
 2071 FORMAT (34H TOTAL NO. OF INNER ITERATIONS FOR,
     1        25H GROUPSPLITTING SET #1 = ,I6)
 2072 FORMAT (34H TOTAL NO. OF INNER ITERATIONS FOR,
     1        25H GROUPSPLITTING SET #2 = ,I6)
 7000 FORMAT(1X,'<<< FROM NOW ON, USING EXACT EXP(-X) CALLS >>>')
 7001 FORMAT(1X,3('*** MOC2AJ: CONVERGENCE NOT REACHED ***'/))
      END