*DECK MOC2MH
      SUBROUTINE MOC2MH(ITYPEC,NBN,NGRP,NANIS,PHI, XCTOT,XCTRC,XCDIA,
     >                  XCCAT,XCNUF,XCCHI,XCABS,DIFF,BETA,REBAL,FLUN,
     >                  REFKEF,AKFRB,B2VAL,OOL,EBMX,IPRT,PRODFS,LEAKSW)
*---------------------------  MOC2MH  ---------------------------------
*
*   1- PROGRAMME STATISTICS:
*       NAME     :  MOC2MH
*       USE      :  COMPUTE BN AND FLUX NORMALIZED FACTOR
*       AUTHOR   :  R. ROY (01/01/98)
*       MODIFIED :  MONCHAI A. (01/01/05)
*
*   2- ROUTINE PARAMETERS:
*    INPUT
*       DB2      : D x B^2  COEFFICIENT               D(NGRP)
*       DIFF     : REGION-INTEGRATED DIFFUSION        D(NGRP)
*                  COEFFICIENT
*       LEAKSW   : FLAG INDICATING PRESENCE (.TRUE.)  L
*                  OR ABSENCE (.FALSE.) OF LEAKAGE.  
*       NGRP     : NUMBER OF ENERGY GROUPS            I
*       PHI      : DOMAIN-INTEGRATED GROUP FLUXES     D(NGRP)
*       PRODFS   : TOTAL NEUTRON PRODUCTION RATE      D
*       REFKEF   : REFERENCE K-EFFECTIVE              D(NGRP)
*       XCABS    : HOMOGENIZED ABSORPTION             D(NGRP)
*                  CROSS SECTION
*       XCCHI    : HOMOGENIZED CHI                    D(NGRP)
*       XCDIA    : HOMOGENIZED WITHIN-GROUP           D(NGRP)
*                  SCATTERING CROSS SECTION
*       XCCAT    : HOMOGENIZED GROUP                  D(NGRP,0:NANIS,NGRP)
*                  SCATTERING CROSS SECTION
*       XCNUF    : HOMOGENIZED NUSIGMA-F              D(NGRP)
*                  CROSS SECTION
*       XCSOU    : DOMAIN-INTEGRATED GROUP TOTAL      D(NGRP)
*                  FISSION + SCATTERING SOURCES
*       XCTOT    : HOMOGENIZED GROUP TOTAL            D(NGRP)
*                  CROSS SECTION
*       XCTRC    : HOMOGENIZED GROUP TRANSPORT-       D(NGRP)
*                  CORRECTED TOTAL CROSS SECTION
*
*    WORKING
*       REBAL    : DUMMY MATRIX FOR SOLUTION OF       D(NGRP,NGRP+1)
*                  LINEAR EQUATION
*       BETA     : DUMMY VARIABLE                     D
*    OUTPUT
*       B2VAL    : B2 VALUE                           D
*       FLUN     : GROUP REBALANCE FACTOR             D(NGRP)
*
*
*---------------------------   MOC2MH  --------------------------------
      IMPLICIT NONE
      INTEGER ITYPEC,NBN,NGRP,NANIS,IOUT
      INTEGER IGRP,IPRT, JGRP, IS, IPRTB
      INTEGER NSRC, IER
      DOUBLE PRECISION XCTOT(NGRP),XCABS(NGRP),XCCAT(NGRP,0:NANIS,NGRP),
     >        XCNUF(NGRP),XCCHI(NGRP),XCTRC(NGRP),XCDIA(NGRP),
     >        PHI(NGRP),FLUN(NGRP),DIFF(NGRP),BETA(NGRP),
     >        B2VAL,B2
      DOUBLE PRECISION REBAL(NGRP,NGRP+1),SSRC,REFKEF
      DOUBLE PRECISION AKFRB,AKOLD,LOSS,OLOSS,OOL,AFLNOR,DZERO,DONE
      REAL    EBMX
      LOGICAL LEAKSW

      DOUBLE PRECISION PRODFS,SUMCHI,FISOUR

      PARAMETER (IOUT=6,DZERO= 0.0D0, DONE=1.0D0 )
      LOGICAL TKEFF

      IF( ITYPEC .EQ. 4 )THEN
          IPRTB= IPRT
          CALL MOC2BN(NGRP,NANIS,NBN,PHI,
     >                XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,
     >                DIFF,BETA,REBAL,FLUN,REFKEF,EBMX,B2,IPRTB )
          B2VAL= B2
          RETURN
      ENDIF
C
      FISOUR= DZERO
      SUMCHI= DZERO
      DO 101 IGRP=1,NGRP
         FISOUR= FISOUR + XCNUF(IGRP)*PHI(IGRP)
         SUMCHI= SUMCHI + XCCHI(IGRP)
 101  CONTINUE
      PRODFS= SUMCHI*FISOUR
C
      RETURN
 1015 FORMAT (5X,7A12)
 1010 FORMAT (5X,I12,1P,3E12.5,E13.6)
 1040 FORMAT (7H FLUX  ,2H: ,1P,10E12.5/(9X,10E12.5))
 6000 FORMAT(/' F L U X E S   A N D   H O M O G E N I Z E D   X - S'/
     > 1X,51(1H-))
 6021 FORMAT(1X,'GROUP',4X,'AVERAGE',8X,'TOTAL',7X,
     >'ABSORPTION',5X,'NUSIGF',8X,'FISSION',10X,'SCATTERING X-S'/11X,
     >'FLUX',11X,'X-S',11X,'X-S',10X,'X-S',
     >10X,'SPECTRUM',3X,'WITHIN GROUP',2X,'OUT OF GROUP')
 6023 FORMAT(1X,I4,1P,8E14.5)
 6028 FORMAT(3H I=,I4,2H: ,1P,10E12.4/(9X,10E12.4))
 6029 FORMAT(/' ORDER OF LEGENDRE: ',I2
     > /' SCATTERING TRANSFER X-S (I TOWARD J) IN REGION',I5,1H:
     > //(11X,2HJ=,I4,:,6X,2HJ=,I4,:,6X,2HJ=,I4,:,6X,2HJ=,I4,:,6X,2HJ=,
     > I4,:,6X,2HJ=,I4,:,6X,2HJ=,I4,:,6X,2HJ=,I4,:,6X,2HJ=,I4,:,6X,
     > 2HJ=,I4))
 7000 FORMAT(1X,A6,': *** WARNING *** REBAL <= 0')
      END
