*DECK MOC3I0
      SUBROUTINE MOC3I0(LMODE,NPHI,IFTRAK, NG,MT, NR, NGSS, NS,MXS,NT,
     >                  MATALB, WG,YG,SEGLEN ,NRSEG,EXPT, EXP2,FLM,FLP,
     >                  CYM,CYP,NA,SIGANG,FLMADJ,FLPADJ,SR,FLUX,IG0,
     >                  LDRFLX,LDAJCB,ADJSR,ADJFLX,NE,ANGPHI,ANGADJ)
*  
*---------------------------   MOC3I0  --------------------------------
* 1- PROGRAMME STATISTICS:
*     NAME     :  MOC3I0
*     USE      :  COMBINING SPATIAL ITERATION LOOP 
*     AUTHOR   :  ROY R.  
*     MODIFIED :  MONCHAI A. (22/03/07)
*
* 2- ROUTINE PARAMETERS:
*   INPUT
*     ADJFLX   : ADJOINT FLUX ARRAY                      D(NF,NR,NG)
*     ADJSR    : REGION TOTAL ADJOINT SOURCES            D(NF,NR,NG)
*     ANGPHI   : ANGULAR FLUX                            D(NR,-NPHI:NPHI
*                                                        ,NGSS,NG)
*     ANGADJ   : ANGULAR ADJOINT FLUX                    D(NR,-NPHI:NPHI
*                                                        ,NGSS,NG)
*     FLUX     : FLUX ARRAY                              D(NF,NR,NG)
*     IFTRAK   : FILE UNIT NUMBER FOR TRACKS.            I
*     IPRT     : PRINT OPTION FOR CHARACTERISTIC SOLVER  I
*     IG0      : MULTIGROUP REDUCTION IN FORWARD         I
*                DIRECTION
*     JGDEB    : MULTIGROUP REDUCTION IN BACKWARD        I
*                DIRECTION
*     LDAJCB   : FLAG TO TURN ON DIRECT AND ADJOINT FLUX L
*                CALCULATIONS
*     LMODE    : CONTROL FLAG FOR CALCULATION MODE       L(20)
*     MATALB   : MATERIAL CODE IN REGIONS.               I(-NS:NR)
*     MT       : NUMBER OF MIXTURES CONSIDERED           I
*     MXE      : FIXED MAXIMUM NUMBER OF SEGMENTS ON     I
*                EACH TRACK (FIXED = 64) (MXE >= MXS)
*     MXS      : MAXIMUM NUMBER OF SEGMENTS ON EACH      I
*                TRACK (VARIABLE ACC. TO TRACKING LINE)
*     NA       : NUMBER OF SURFACE ALBEDO                I 
*     NF       : NUMBER OF LEGENDRE ORDER FOR FLUX       I
*     NG       : NUMBER OF GROUPS TREATED                I
*     NGSS     : MAXIMUM NUMBER OF POLAR QUADRATURE      I
*     NE       : NUMBER OF POLAR QUADRATURE              I
*                (LOW OR HIGH ORDER TRACKING LINE)
*     NR       : NUMBER OF REGIONS CONSIDERED            I
*     NRSEG    : KEY INDEX FOR REGIONAL NUMBER SEGMENT   I(MXS)
*     NS       : NUMBER OF SURFACES CONSIDERED           I
*     SEGLEN   : LENGTH OF EACH SEGMENT                  R(MXS)  
*     SR       : REGIONAL TOTAL SOURCES                  D(NF,NR,NG)
*     TITRE    : EXECUTION TITLE                         C*72
*
*   WORK VECTOR
*     FLM      : DUMMY VARIABLE                          D(NGSS,MXS)
*     FLP      : DUMMY VARIABLE                          D(NGSS,MXS) 
*     CYM      : DUMMY VARIABLE                          D(NGSS,MXS)
*     CYP      : DUMMY VARIABLE                          D(NGSS,MXS)
*
*---------------------------   MOC3I0  --------------------------------
      IMPLICIT  NONE
      INTEGER   NF,MXE
      PARAMETER (NF=1,MXE=64)
      INTEGER   IFTRAK, NG, MT, NR, NS, NE, MXS, NT, NA,NGSS,
     >          NPHI  , IG,     IR,     IE,      IT, IG0,
     >          NOIL, MTIL, LINE, IANG, IL, JL, NUMOLD
      INTEGER   NRSEG(MXS), MATALB(-NS:NR)
      REAL      SIGANG(NGSS,-NA:MT,NG),
     >          EXPT(NGSS,MXS), EXP2(NGSS,MXS), SEGLEN(MXS)
      REAL      DWEIG(MXE), WG(MXE), YG(MXE)
      DOUBLE PRECISION SR(NF,NR,NG), FLUX(NF,NR,NG), CY1(MXE),
     >          FL0(MXE),FL0ADJ(MXE), CY0(MXE), FL1(MXE),FL1ADJ(MXE),
     >          FLM(NGSS,MXS),FLP(NGSS,MXS),CYM(NGSS,MXS),CYP(NGSS,MXS),
     >          FLMADJ(NGSS,MXS), FLPADJ(NGSS,MXS),
     >          ADJSR(NF,NR,NG), ADJFLX(NF,NR,NG)
      DOUBLE PRECISION ANGPHI(NR,-NPHI:NPHI,NGSS,NG),
     >                 ANGADJ(NR,-NPHI:NPHI,NGSS,NG)
      LOGICAL   LDAJCB,LDRFLX,LANGUL,LMODE(20) 
      REAL      WEIGHT,ONE,ZERO
      DOUBLE PRECISION    TAU,CUTEXP,DONE,DZERO,HALF,SIXT
      PARAMETER ( ONE=1.0E0,ZERO=0.0E0,
     >            DONE=1.00000000D0, DZERO=0.0000000D0, 
     >            CUTEXP=2.00000000D-02,HALF=0.50000000D0,
     >            SIXT = HALF/3.000000000D0 ) 
      LANGUL = LMODE(6)
      IF(.NOT.LDRFLX .AND. .NOT.LDAJCB) GO TO 100
      DO 1000 IG= IG0,IG0
         DO 1100 IR=1,NR
           IF (LDRFLX)  FLUX(NF,IR,IG)= DZERO
           IF (LDAJCB)  ADJFLX(NF,IR,IG)= DZERO
           IF (LANGUL) THEN 
            DO 1101 IANG= -NPHI,NPHI
            DO 1102 IE= 1,NGSS
              IF (LDRFLX)  ANGPHI(IR,IANG,IE,IG) = DZERO
              IF (LDAJCB)  ANGADJ(IR,IANG,IE,IG) = DZERO
 1102       CONTINUE
 1101       CONTINUE
           ENDIF
 1100    CONTINUE
 1000 CONTINUE
      DO 4000 IT= 1, NT
           READ(IFTRAK)  IANG,LINE,WEIGHT,
     >       (NRSEG(IL),IL= 1,LINE),(SEGLEN(IL),IL= 1,LINE)
           DO 4100 IE=1,NE
              DWEIG(IE)= ONE
 4100      CONTINUE
           IF (LINE .GT. MXS) CALL XABORT('MOC3I0: LINE>MXS ') 
           DO 4200 IG= IG0,IG0
              DO 4210 IE=1,NE
              IF (LDRFLX) THEN
                 FL0(IE)= DZERO
                 FL1(IE)= DZERO
              ENDIF
              IF (LDAJCB) THEN
                 FL0ADJ(IE)= DZERO
                 FL1ADJ(IE)= DZERO
              ENDIF
                 CY0(IE)= DONE
                 CY1(IE)= DONE
 4210         CONTINUE
              NUMOLD= NRSEG(1)
              DO 4220 IL= 1, LINE
                 NOIL  = NRSEG(IL)
                 MTIL  = MATALB(NOIL)
                 IF( MTIL.LT.0 )THEN
                    IF( NUMOLD.NE.NOIL )THEN
                       DO 4221 IE=1,NE
                          IF (LDRFLX) THEN
                           FLM(IE,IL)   = DZERO
                           FLP(IE,IL)   = DZERO
                          ENDIF
                          IF (LDAJCB) THEN
                           FLMADJ(IE,IL)= DZERO
                           FLPADJ(IE,IL)= DZERO
                          ENDIF
                          EXP2(IE,IL)= SIGANG(IE,MTIL,IG)
                          EXPT(IE,IL)= DONE
 4221                  CONTINUE
                    ELSE
                       DO 4229 IE=1,NE
                          IF (LDRFLX) THEN
                           FLM(IE,IL)   = DZERO
                           FLP(IE,IL)   = DZERO
                          ENDIF
                          IF (LDAJCB) THEN
                           FLMADJ(IE,IL)= DZERO
                           FLPADJ(IE,IL)= DZERO
                          ENDIF
                          EXP2(IE,IL)= DZERO
                          EXPT(IE,IL)= DZERO
 4229                  CONTINUE
                    ENDIF
                 ELSE
                    DO 4222 IE=1,NE
                       TAU =        SIGANG(IE,MTIL,IG)*SEGLEN(IL)
                       IF( TAU.EQ.DZERO )THEN
                          EXP2(IE,IL)= SEGLEN(IL)*YG(IE)
                          EXPT(IE,IL)= DZERO
                       ELSEIF (SIGANG(IE,MTIL,IG).LT.CUTEXP) THEN
                          EXP2(IE,IL)= TAU - (HALF-SIXT*TAU )* TAU**2
                          EXPT(IE,IL)= DONE
                       ELSE
                          EXP2(IE,IL)= DONE-EXP(-TAU)
                          EXPT(IE,IL)= DONE
                       ENDIF
                       IF (LDRFLX) THEN
                        FLM (IE,IL)= DWEIG(IE)*SR(NF,NOIL,IG)
                        FLP (IE,IL)= DWEIG(IE)*SR(NF,NOIL,IG)
                       ENDIF
                       IF (LDAJCB) THEN
                        FLMADJ(IE,IL)= DWEIG(IE)*ADJSR(NF,NOIL,IG)
                        FLPADJ(IE,IL)= DWEIG(IE)*ADJSR(NF,NOIL,IG)
                       ENDIF
 4222               CONTINUE
                 ENDIF
                 NUMOLD= NOIL
 4220         CONTINUE
              DO 4226 IL= 1, LINE
                 JL= LINE + 1 - IL
                 DO 4225 IE=1,NE
                   IF (LDRFLX) THEN
                    FLM(IE,IL) =(FLM(IE,IL)+FL0(IE))*EXP2(IE,IL)
                    FLP(IE,JL) =(FLP(IE,JL)+FL1(IE))*EXP2(IE,JL)
                    FL0(IE)    = FL0(IE)-FLM(IE,IL)*EXPT(IE,IL)
                    FL1(IE)    = FL1(IE)-FLP(IE,JL)*EXPT(IE,JL)
                   ENDIF
                   CYM(IE,IL) =  CY0(IE)*EXP2(IE,IL)
                   CYP(IE,JL) =  CY1(IE)*EXP2(IE,JL)
                   IF (LDAJCB) THEN
                    FLMADJ(IE,IL)=(FLMADJ(IE,IL)+FL0ADJ(IE))*EXP2(IE,IL)
                    FLPADJ(IE,JL)=(FLPADJ(IE,JL)+FL1ADJ(IE))*EXP2(IE,JL)
                    FL0ADJ(IE) = FL0ADJ(IE)-FLMADJ(IE,IL)*EXPT(IE,IL)
                    FL1ADJ(IE) = FL1ADJ(IE)-FLPADJ(IE,JL)*EXPT(IE,JL)
                   ENDIF
                   CY0(IE)    = CY0(IE)-CYM(IE,IL)*EXPT(IE,IL)
                   CY1(IE)    = CY1(IE)-CYP(IE,JL)*EXPT(IE,JL)
 4225            CONTINUE
 4226         CONTINUE
              DO 4230 IE=1,NE
                 IF (LDRFLX) THEN
                  FL0(IE)=  FL0(IE)/(DONE-CY0(IE))
                  FL1(IE)=  FL1(IE)/(DONE-CY1(IE))
                 ENDIF
                 IF (LDAJCB) THEN
                  FL0ADJ(IE)=  FL0ADJ(IE)/(DONE-CY0(IE))
                  FL1ADJ(IE)=  FL1ADJ(IE)/(DONE-CY1(IE))
                 ENDIF
 4230         CONTINUE
              DO 4240 IL= 1, LINE
                 NOIL  = NRSEG(IL)
                 IF( NOIL.GT.0 )THEN
                 DO 4241 IE=1,NE
                 IF (LDRFLX) THEN
                       FLUX(NF,NOIL,IG)=FLUX(NF,NOIL,IG)
     >                 +WEIGHT*WG(IE)*(FLM(IE,IL)+FL0(IE)*CYM(IE,IL))
     >                 +WEIGHT*WG(IE)*(FLP(IE,IL)+FL1(IE)*CYP(IE,IL))
                  IF(LANGUL) THEN
                   ANGPHI(NOIL,IANG,IE,IG)=ANGPHI(NOIL,IANG,IE,IG)-
     >             WEIGHT*(FLM(IE,IL)+FL0(IE)*CYM(IE,IL))
                   ANGPHI(NOIL,-IANG,IE,IG)=ANGPHI(NOIL,-IANG,IE,IG)-
     >             WEIGHT*(FLP(IE,IL)+FL1(IE)*CYP(IE,IL))
                  ENDIF
                 ENDIF
                 IF (LDAJCB) THEN
                   ADJFLX(NF,NOIL,IG)= ADJFLX(NF,NOIL,IG)
     >             +WEIGHT*WG(IE)*(FLMADJ(IE,IL)+FL0ADJ(IE)*CYM(IE,IL))
     >             +WEIGHT*WG(IE)*(FLPADJ(IE,IL)+FL1ADJ(IE)*CYP(IE,IL))
                  IF(LANGUL) THEN
                   ANGADJ(NOIL,IANG,IE,IG)=ANGADJ(NOIL,IANG,IE,IG)-
     >             WEIGHT*(FLMADJ(IE,IL)+FL0ADJ(IE)*CYM(IE,IL))
                   ANGADJ(NOIL,-IANG,IE,IG)=ANGADJ(NOIL,-IANG,IE,IG)-
     >             WEIGHT*(FLPADJ(IE,IL)+FL1ADJ(IE)*CYP(IE,IL))
                  ENDIF
                 ENDIF
 4241            CONTINUE
                 ENDIF
 4240         CONTINUE
 4200       CONTINUE
 4000 CONTINUE
  100 CONTINUE
      RETURN
      END