*DECK MOCAKF
      SUBROUTINE MOCAKF( NGRP, NANIS, PHI, DB2,AKEFF,AFLNOR,LEAKSW,
     >                   XCTOT,XCDIA,XCCAT,XCNUF,XCCHI, IPRT,PRODFS,
     >                   OLDBIL)
    
* 
* ---------------------------  MOCAKF  ---------------------------------
* 
*  1- PROGRAMME STATISTICS:
* 
*      NAME     :  MOCAKF
*      USE      :  K-EFFECTIVE EVALUATION FOR ADJOINT MOCC SOLVER 
*      MODIFIED BY :  MONCHAI A. (10/04/04)   FROM MOC2KF
* 
*  2- ROUTINE PARAMETERS:
*    INPUT
*      ADJFLX   : ADJOINT FLUX ARRAY                      D(NFUNL,NREG,
*                                                           NGRP,8)
*      IPRT     : PRINT OPTION FOR CHARACTERISTIC SOLVER    I
*      LEAKSW   : LOGICAL SCALAR VARIABLE DESCRIBING        L
*                 THE PRESENCE (.TRUE.) OR THE ABSENCE
*                 (.FALSE.) OF LEAKAGE.
*      NANIS    : NUMBER OF LEGENDRE ORDER FOR SCATTERING   I
*      NFUNL    : NUMBER OF LEGENDRE ORDER FOR THE FLUX     I
*                 (NF=1 FOR CURRENT VERSION)
*      NGRP     : NUMBER OF GROUPS TREATED                  I
*      NMAT     : NUMBER OF MIXTURES CONSIDERED             I
*      NREG     : NUMBER OF REGIONS CONSIDERED              I
*      TITRE    : EXECUTION TITLE                           C*72
* 
*    OUTPUT  
*      AFLNOR   : INVERSE OF K-EFFECTIVE                    D
*      AKEFF    : K-EFFECTIVE                               D
* 
*      
*      FULL DOMIAN HOMOGENIZED XS'S
*      XCCHI    : CHI                                       D(NGRP)
*      XCDIA    : TRANSPORT CORRECTED WITHINGROUP           D(NGRP)
*                 SCATT. XS                               
*      XCCAT    : SCATTERING XS                             D(NGRP,0:
*                                                           NANIS,NGRP)
*      XCNUF    : NU * FISSION XS                           D(NGRP)
*      XCTOT    : TOTAL XS                                  D(NGRP)
*      XCABS    : ABSORPTION XS                             D(NGRP)  
*      XCTRC    : TRANSPORT CORRECTED TOTAL XS              D(NGRP)
*      PHI      : ADJOINT FLUX(IREG,NGRP)*VOL(IREG)         D(NGRP)
* 
* 
*  3- INTERNAL PARAMETERS:
*      IOUT     : OUTPUT UNIT = 6                         I
* 
* 
*---------------------------  MOCAKF  ---------------------------------
      IMPLICIT  NONE
      INTEGER   NGRP, NANIS, IPRT, IGRP, JGRP
      INTEGER   IOUT
      DOUBLE PRECISION PHI(NGRP),DB2(NGRP)
      DOUBLE PRECISION XCTOT(NGRP),XCCAT(NGRP,0:NANIS,NGRP),
     >                 XCNUF(NGRP),XCCHI(NGRP),XCDIA(NGRP)
      DOUBLE PRECISION PRODFS,PRODSC,LOSS,FISOUR,
     >                 AKEFF,AFLNOR,DZERO,DONE,OLDBIL
      LOGICAL LEAKSW
      PARAMETER (IOUT=6,DZERO=0.0D0, DONE=1.0D0 ) 
      FISOUR= DZERO
      DO 10 IGRP=1,NGRP
         FISOUR= FISOUR + XCCHI(IGRP)*PHI(IGRP)
10    CONTINUE
      PRODFS=DZERO  
      DO 11 IGRP=1,NGRP
         PRODFS = PRODFS+ XCNUF(IGRP)*FISOUR
11    CONTINUE
      PRODSC=DZERO
      DO 12 IGRP=1,NGRP
        DO 13 JGRP=1,NGRP
         PRODSC = PRODSC+XCCAT(IGRP,0,JGRP)*PHI(JGRP)
13      CONTINUE
12    CONTINUE
      LOSS=DZERO
      DO 20 IGRP=1,NGRP
         LOSS  = LOSS+ XCTOT(IGRP)*PHI(IGRP)
20    CONTINUE
      LOSS = LOSS - PRODSC
      IF( PRODFS.LT.DZERO )THEN
        CALL XABORT('MOCAKF: ADJOINT FISSION PRODUCTION IS NEGATIVE')
      ENDIF
      IF( LOSS.LT.DZERO )THEN
        CALL XABORT('MOCAKF: ADJOINT LOSS IS NEGATIVE')
      ENDIF
      IF( LEAKSW )THEN
         IF( OLDBIL.GT.DZERO )THEN
            AKEFF= PRODFS/OLDBIL
            AFLNOR= DONE/AKEFF
         ELSE
            AKEFF= PRODFS
            AFLNOR= DONE/AKEFF
         ENDIF
      ELSE
         AKEFF=  PRODFS/LOSS
         AFLNOR= 1.0D0/AKEFF
      ENDIF
         OLDBIL=  AFLNOR
      IF( IPRT.GT.2 )THEN
         WRITE(IOUT,6000) PRODFS,LOSS,AFLNOR,AKEFF
      ENDIF
      RETURN
 6000 FORMAT(/' ************ PRODFS=',D20.14
     >       /' ************ LOSS  =',D20.14
     >       /' ************ AFLNOR=',D20.14
     >       /' ************ AKEFF =',D20.14/)
      END