*DECK MOCAMH
      SUBROUTINE MOCAMH(ITYPEC,NBN,NGRP,NANIS,PHI, XCTOT,XCTRC,XCDIA,
     >                  XCCAT,XCNUF,XCCHI,XCABS,DIFF,BETA,REBAL,FLUN,
     >                  AKFRB,B2VAL,OOL,EBMX,IPRT,AKEFF)
*
*---------------------------  MOCAMH  ---------------------------------
*
*   1- PROGRAMME STATISTICS:
*       NAME     :  MOCAMH
*       USE      :  MULTIGROUP ADJOINT FLUX NORMALIZATION
*       AUTHOR   :  R. ROY (01/01/98) FROM MOC2MH
*       MODIFIED :  MONCHAI A. (01/01/05)  
*
*   2- ROUTINE PARAMETERS:
*    INPUT
*       DIFF     : REGION-INTEGRATED DIFFUSION        D(NGRP)
*                  COEFFICIENT
*       IG0      : STARTING GROUP INDEX               I
*       JGDEB    : ENDING GROUP INDEX                 I
*       LEAKSW   : FLAG INDICATING PRESENCE (.TRUE.)  L
*                  OR ABSENCE (.FALSE.) OF LEAKAGE.  
*       NGRP     : NUMBER OF ENERGY GROUPS            I
*       PHI      : DOMAIN-INTEGRATED GROUP FLUXES     D(NGRP)
*       PRODFS   : TOTAL NEUTRON PRODUCTION RATE      D
*       REFKEF   : REFERENCE K-EFFECTIVE              D(NGRP)
*       XCABS    : HOMOGENIZED ABSORPTION             D(NGRP)
*                  CROSS SECTION
*       XCCHI    : HOMOGENIZED CHI                    D(NGRP)
*       XCDIA    : HOMOGENIZED WITHIN-GROUP           D(NGRP)
*                  SCATTERING CROSS SECTION
*       XCCAT    : HOMOGENIZED GROUP                  D(NGRP,0:NANIS,NGRP)
*                  SCATTERING CROSS SECTION
*       XCNUF    : HOMOGENIZED NUSIGMA-F              D(NGRP)
*                  CROSS SECTION
*       XCSOU    : DOMAIN-INTEGRATED GROUP TOTAL      D(NGRP)
*                  FISSION + SCATTERING SOURCES
*       XCTOT    : HOMOGENIZED GROUP TOTAL            D(NGRP)
*                  CROSS SECTION
*       XCTRC    : HOMOGENIZED GROUP TRANSPORT-       D(NGRP)
*                  CORRECTED TOTAL CROSS SECTION
*
*    WORKING
*       REBAL    : DUMMY MATRIX FOR SOLUTION OF       D(NGRP,NGRP+1)
*                  LINEAR EQUATION
*       BETA     : DUMMY VARIABLE                     D
*    OUTPUT
*       B2VAL    : B2 VALUE                           D
*       FLUN     : DUMMY VARIABLE                     D(NGRP)
*
*---------------------------   MOCAMH  --------------------------------
*
      IMPLICIT NONE  
      INTEGER ITYPEC,NBN,NGRP,NANIS,IOUT
      INTEGER IGRP,IPRT, JGRP, IS, IPRTB
      INTEGER NSRC, IER
      DOUBLE PRECISION XCTOT(NGRP),XCABS(NGRP),XCCAT(NGRP,0:NANIS,NGRP),
     >        XCNUF(NGRP),XCCHI(NGRP),XCTRC(NGRP),XCDIA(NGRP),
     >        PHI(NGRP),FLUN(NGRP),DIFF(NGRP),BETA(NGRP),
     >        B2VAL,B2,PROD,FISOU
      DOUBLE PRECISION REBAL(NGRP,NGRP+1),SUMNUF,AKEFF
      DOUBLE PRECISION AKFRB,AKOLD,LOSS,OLOSS,OOL,AFLNOR,DZERO,DONE
      REAL    EBMX 
      PARAMETER (IOUT=6,DZERO= 0.0D0, DONE=1.0D0 )
      LOGICAL TKEFF
      NSRC=  NGRP+1
      TKEFF= .TRUE.
      SUMNUF= DZERO
      DO 110 IGRP=1,NGRP
        SUMNUF= SUMNUF + XCNUF(IGRP) 
110   CONTINUE
C     --- FILL IN REBAL MATRIX ----
      DO 190  IGRP= 1, NGRP
        DO 193 JGRP= 1, NGRP
            REBAL(IGRP,JGRP)= (-XCCAT(IGRP,0,JGRP)*PHI(JGRP))
193     CONTINUE
         REBAL(IGRP,NSRC)= XCNUF(IGRP)/SUMNUF
         REBAL(IGRP,IGRP)= (XCTRC(IGRP)-XCDIA(IGRP))*PHI(IGRP)
  190 CONTINUE
      CALL ALSBD(NGRP,1,REBAL,IER,NGRP)
      IF( IER.NE.0 ) CALL XABORT(' MOCAMH: SINGULAR REBALANCING MATRIX')
      LOSS=   DZERO
      OLOSS=  DZERO
      AKFRB=  DZERO
      AKOLD=  DZERO
      PROD= DZERO
      FISOU= DZERO
      DO 10 IGRP=1,NGRP
         FISOU = FISOU   + XCCHI(IGRP)*REBAL(IGRP,NSRC)*PHI(IGRP)
10    CONTINUE
      DO 191 IGRP= 1, NGRP
         AKOLD=  AKOLD + XCCHI(IGRP)*PHI(IGRP)
         AKFRB=  AKFRB + XCCHI(IGRP)*REBAL(IGRP,NSRC)
         TKEFF=  TKEFF.AND.(REBAL(IGRP,NSRC).GT.DZERO)
         PROD =  PROD+ XCNUF(IGRP)* FISOU
         IF( REBAL(IGRP,NSRC).LE.DZERO )THEN
            WRITE(IOUT,7000) 'MOCAMH'
         ENDIF
         LOSS =  LOSS  + XCTOT(IGRP)*REBAL(IGRP,NSRC)*PHI(IGRP)
         OLOSS = OLOSS + XCTOT(IGRP)*PHI(IGRP)
         DO 195 JGRP= 1, NGRP
           LOSS=  LOSS  - XCCAT(IGRP,0,JGRP)*REBAL(JGRP,NSRC)*PHI(JGRP)
           OLOSS= OLOSS - XCCAT(IGRP,0,JGRP)*PHI(JGRP)
  195    CONTINUE
  191 CONTINUE
C     --- USING OOL/LOSS TO IMPROVE ADJOINT FLUX NORMALIZATION ----
      IF( OOL.EQ.DZERO )THEN
         AFLNOR= OLOSS/LOSS
         OOL=OLOSS
      ELSE
         AFLNOR= OOL/LOSS
      ENDIF
C     ---CALCULATE NORMALIZED FACTOR---
      DO 196 IGRP= 1, NGRP
        FLUN(IGRP)= REBAL(IGRP,NSRC)* AFLNOR
196   CONTINUE
C     --- THERE ARE 2 MODULES TO CALCULTE K-EFF: MOCAKF AND MOCAMH---
      AKEFF=PROD/LOSS
      RETURN
 1015 FORMAT (5X,7A12)
 1010 FORMAT (5X,I12,1P,3E12.5,E13.6)
 1040 FORMAT (7H FLUX  ,2H: ,1P,10E12.5/(9X,10E12.5))
 6000 FORMAT(/' F L U X E S   A N D   H O M O G E N I Z E D   X - S'/
     > 1X,51(1H-))
 6021 FORMAT(1X,'GROUP',4X,'AVERAGE',8X,'TOTAL',7X,
     >'ABSORPTION',5X,'NUSIGF',8X,'FISSION',10X,'SCATTERING X-S'/11X,
     >'FLUX',11X,'X-S',11X,'X-S',10X,'X-S',
     >10X,'SPECTRUM',3X,'WITHIN GROUP',2X,'OUT OF GROUP')
 6023 FORMAT(1X,I4,1P,8E14.5)
 6028 FORMAT(3H I=,I4,2H: ,1P,10E12.4/(9X,10E12.4))
 6029 FORMAT(/' ORDER OF LEGENDRE: ',I2
     > /' SCATTERING TRANSFER X-S (I TOWARD J) IN REGION',I5,1H:
     > //(11X,2HJ=,I4,:,6X,2HJ=,I4,:,6X,2HJ=,I4,:,6X,2HJ=,I4,:,6X,2HJ=,
     > I4,:,6X,2HJ=,I4,:,6X,2HJ=,I4,:,6X,2HJ=,I4,:,6X,2HJ=,I4,:,6X,
     > 2HJ=,I4))
 7000 FORMAT(1X,A6,': *** WARNING *** REBAL <= 0')
      END
