*DECK MOCCP2
      SUBROUTINE MOCCP2(IPFLUX,CCPDIR,NREG,NGRP,NL,NMAT,NANIS,NSOUT,
     >                  NIFIS,NGSS,NALB,MXGAUS,IFTRK2,IFTRAK,NTRK2 ,
     >                  NTRK,MXSEG,MATALB,NRSEG,SIGANG,XSDIA,XSCAT ,
     >                  XTINV,XSTRC,SEGLEN,ALBEDO,EXPT,EXP2,WGSS,YGSS,
     >                  NGSS2,FLM,FLP,CYM,CYP,LOWTRK,IPRT,VOL,VOLINV,
     >                  MAXITR,IGDBST,JGDBST,X1,Y1,Z1,S1,PMC)
*
*---------------------------  MOCCP2  ---------------------------------
*
* 1- PROGRAMME STATISTICS:
*
*     NAME     :  MOCCP2
*     USE      :  MOCC RESPONSE MATRIX CALCULATION
*     BY       :  MONCHAI A. (24/02/05)   
*
* 2- ROUTINE PARAMETERS:
*   INPUT
*     IFTRAK   : FILE UNIT NUMBER FOR TRACKS             I
*     IPFLUX   : FILE UNIT NUMBER FOR FLUX               I
*     IPRT     : PRINT OPTION FOR CHARACTERISTIC SOLVER  I
*     IGDEB    : MULTIGROUP REDUCTION IN FORWARD         I
*                DIRECTION
*     JGDEB    : MULTIGROUP REDUCTION IN BACKWARD        I
*                DIRECTION
*     LMODE    : CONTROL FLAG FOR CALCULATION MODE       L(20)
*     LDAJCB   : FLAG TO TURN ON DIRECT AND ADJOINT FLUX L
*                CALCULATIONS
*     MATALB   : MATERIAL CODE IN REGIONS.               I(-NSOUT:NREG)
*     NMAT     : NUMBER OF MIXTURES CONSIDERED           I
*     MXSEG    : MAXIMUM NUMBER OF SEGMENTS ON EACH      I
*                TRACK (VARIABLE ACC. TO TRACKING LINE)
*     NALB     : NUMBER OF SURFACE ALBEDO                I 
*     NGRP     : NUMBER OF GROUPS TREATED                I
*     NGSS     : MAXIMUM NUMBER OF POLAR QUADRATURE      I
*     NE       : NUMBER OF POLAR QUADRATURE              I
*                (LOW OR HIGH ORDER TRACKING LINE)
*     NFUNL    : NUMBER OF LEGENDRE ORDER FOR FLUX       I
*     NIFIS    : NUMBER OF FISSILE ISOTOPES              I
*     NREG     : NUMBER OF REGIONS CONSIDERED            I
*     NRSEG    : KEY INDEX FOR REGIONAL NUMBER SEGMENT   I(MXSEG)
*     NSOUT    : NUMBER OF SURFACES CONSIDERED           I
*     SEGLEN   : LENGTH OF EACH SEGMENT                  R(MXSEG)  
*     TITRE    : EXECUTION TITLE                         C*72
*     VOL      : VOLUME OF REGION                        R(NREG)
*     VOLINV   : INVERSE OF VOLUMES                      R(NREG)
*     X1       : FLUXES                                  D(NREG,NGRP)
*     Y1       : FIXED EXTERNAL INPUT                    D(NREG,NGRP)
*
*   WORK VECTOR
*     FLM      : DUMMY VARIABLE                          D(NGSS,MXSEG)
*     FLP      : DUMMY VARIABLE                          D(NGSS,MXSEG) 
*     CYM      : DUMMY VARIABLE                          D(NGSS,MXSEG)
*     CYP      : DUMMY VARIABLE                          D(NGSS,MXSEG)
*      S1      : DUMMY VARIABLE                          D(NREG,NGRP)
*   OUTPUT
*      Z1      : NEW FLUXES                              D(NREG,NGRP)
*--------------------------   MOCCP2    ------------------------------
*
      IMPLICIT NONE
      REAL        A
      COMMON      A(1)
      INTEGER     I(1)
      EQUIVALENCE (A(1),I(1))
      INTEGER      IPFLUX,MAXITR,IGDB1,JGDB1,IGDBST,JGDBST
      INTEGER      NREG,NGRP,NL,NMAT,NANIS,NSOUT,NIFIS,NGSS,NGSS2,
     >             NALB,MXGAUS,IFTRK2,IFTRAK,NTRK2,NTRK,MXSEG,
     >             MATALB(-NSOUT:NREG),NRSEG(MXSEG)
      INTEGER      IG,JG,IREG,JREG,IS,IOUT,IPRT,IFFIL,NTRACK 
      REAL         SIGANG(NGSS,-NALB:NMAT,NGRP),XSDIA(0:NMAT,NGRP),
     >             XSCAT(NGRP,0:NMAT,0:NANIS,NGRP),XTINV(0:NMAT,NGRP),
     >             SEGLEN(MXSEG),ALBEDO(NALB),XSTRC(0:NMAT,NGRP), 
     >             EXPT(NGSS,MXSEG),EXP2(NGSS,MXSEG),
     >             WGSS(MXGAUS),YGSS(MXGAUS),VOLINV(NREG),VOL(NREG)
      DOUBLE PRECISION  FLM(NGSS,MXSEG),FLP(NGSS,MXSEG),
     >                  CYM(NGSS,MXSEG),CYP(NGSS,MXSEG)
      DOUBLE PRECISION X1(NREG,NGRP),Y1(NREG,NGRP),Z1(NREG,NGRP),
     >                 S1(NREG,NGRP)
      DOUBLE PRECISION PMC(NREG,NREG)
      DOUBLE PRECISION  DZERO,DONE,DSUM
      REAL     ZERO
      LOGICAL  LEXTSO,LXSCAT,LOWTRK,LCWDR1,LCWAJ2
      INTEGER  ILCMLN,ILCMTY,ILCMUP,ILCMDN,IMC 
      PARAMETER(IOUT=6,ILCMUP=1,ILCMDN=2)
      CHARACTER*12 NAMFLX,CCPDIR
      DZERO = 0.00000000D+0
      DONE  = 1.00000000D+0
      ZERO  = 0.00000000E+0
      LCWDR1 = .TRUE.
      LCWAJ2 = .FALSE.
      CALL LCMLEN(IPFLUX,CCPDIR,ILCMLN,ILCMTY)
      IF (ILCMLN .EQ. -1) THEN
      CALL LCMSIX(IPFLUX,CCPDIR,ILCMUP)
       DO 101 IG= IGDBST, NGRP-JGDBST
         WRITE(NAMFLX,'(4HP_JI,I3,1X)') IG 
         CALL LCMLEN(IPFLUX,NAMFLX,ILCMLN,ILCMTY)
         IF (ILCMLN.NE.NREG*NREG.OR.ILCMTY.NE.2) GO TO 1000
101    CONTINUE
       CALL LCMSIX(IPFLUX,CCPDIR,ILCMDN) 
       RETURN
      ENDIF
1000  CONTINUE
      CALL SETARA(A,NREG*NREG,IMC)
      CALL XDRSET(A(IMC),NREG*NREG,0.0)
      CALL XDRSET(X1,2*NGRP*NREG,0.0)
      CALL XDRSET(S1,2*NGRP*NREG,0.0)
      DO 100 IG= IGDBST, NGRP-JGDBST
        DO 200 IREG=1,NREG
         S1(IREG,IG)=DONE
         IGDB1=IG
         JGDB1=NGRP-IG
         LXSCAT=.FALSE. 
         LEXTSO=.TRUE.
            CALL MOCSWP(NREG  ,NGRP  ,NL,NMAT,NANIS ,NSOUT ,NIFIS,NGSS,
     >                  NALB  ,MXGAUS,IFTRK2 ,IFTRAK,NTRK2 ,NTRK,MXSEG,
     >                  MATALB,NRSEG ,SIGANG ,XSDIA ,XSCAT ,XTINV,
     >                  SEGLEN,ALBEDO,EXPT   ,EXP2  ,WGSS ,YGSS,NGSS2,
     >                  FLM   ,FLP   ,CYM    ,CYP   ,LXSCAT,LOWTRK,
     >                  VOLINV,LEXTSO,IGDB1  ,JGDB1 ,MAXITR,LCWAJ2,
     >                  X1,S1,Z1,S1)
        DO 300 JREG=1,NREG
         PMC(JREG,IREG)=Z1(JREG,IG)
300     CONTINUE
        S1(IREG,IG)=DZERO
200    CONTINUE
       CALL LCMSIX(IPFLUX,CCPDIR,ILCMUP)
        DO 202 IREG=1,NREG
         DO 201 JREG=1,NREG
           A(IMC-1+IREG+(JREG-1)*NREG)=PMC(IREG,JREG) 
201      CONTINUE
202     CONTINUE
       WRITE(NAMFLX,'(4HP_JI,I3,1X)') IG 
       CALL LCMPUT(IPFLUX,NAMFLX, NREG*NREG,2,A(IMC) )
       CALL LCMSIX(IPFLUX,CCPDIR,ILCMDN) 
100   CONTINUE
      CALL RLSARA(A(IMC))
      RETURN
      END
