*DECK MOCFIM
      SUBROUTINE MOCFIM(NREG  ,NGRP  ,NL,NMAT,NANIS ,NIFIS,NFUNL,
     >                  AKEFF ,ISADJ ,ITYPEC ,NSOUT ,MATALB,XSDIA,
     >                  XSCAT ,ADJSOU,XSNUF  ,XSCHI ,ADJFLX)
* 
* ---------------------------  MOCFIM  ---------------------------------
* 
*  1- PROGRAMME STATISTICS:
*      NAME     :  MOCFIM
*      USE      :  COMPUTE STANDARD OR GENERALIZED FLUX IMPORTANCE
*      MODIFIED BY :  MONCHAI A. (24/02/05)  
* 
*  2- ROUTINE PARAMETERS:
*    INPUT
*      AKEFF    : K-EFFECTIVE                                  D
*      ADJFLX   : ADJOINT FLUX ARRAY                           D(NFUNL,
*                                                                NREG,
*                                                                NGRP,8)
*      ADJSOU   : POINTER INDEX FOR EXTERNAL ADJOINT SOURCE    I
*      ITYPEC   : TYPE OF FLUX CALCULATION                     I
*         ITYPEC= 0 TO SKIP THE FLUX CALCULATION; 
*         ITYPEC= 1 TO COMPUTE THE DIRECT PROBLEM 
*                   SOLUTION WITH FISSION SOURCES 
*                   AND KEFF CONVERGENCE
*         ITYPEC= 2 TO COMPUTE THE DIRECT PROBLEM 
*                   SOLUTION WITH FISSION SOURCES,
*                   KEFF CONVERGENCE AND IMPOSED 
*                   LEAKAGE
*         ITYPEC= 3 TO COMPUTE THE DIRECT PROBLEM 
*                   SOLUTION WITH FISSION SOURCES,
*                   KEFF CONVERGENCE AND IMPOSED 
*                   BUCKLING
*         ITYPEC= 4 TO COMPUTE THE DIRECT PROBLEM 
*                   SOLUTION WITH FISSION SOURCES,
*                   IMPOSED KEFF AND CONVERGENCE 
*                   ON BUCKLING
*         ITYPEC= 5 DIRECT PROBLEM WITH LEAKAGE 
*                   SOURCES AND CONVERGENCE ON 
*                   BUCKLING
*         ITYPEC= 6 DIRECT PROBLEM FIXED SOURCES.
*
*     ISADJ  : FLAG FOR TYPE OF ADJOINT CALCULATION           I
*        ISADJ = 1 ADJOINT FLUX CALCULATION W/O 
*                  EXTERNAL SOURCE FOR ADJOINT KEFF 
*                  PROBLEM
*        ISADJ = 2 ADJOINT FLUX CALCULATION W/
*                  EXTERNAL SOURCE FOR ADJOINT FIXED 
*                  SOURCE PROBLEM
*        ISADJ = 3 GENERALIZED ADJOINT FLUX CALCULATION W/
*                  EXTERNAL GENERALIZED ADJOINT SOURCE 
*      MATALB   : MATERIAL CODE IN REGIONS.                    I(-NSOUT:
*                                                                NREG)
*      NFUNL    : NUMBER OF LEGENDRE ORDER FOR FLUX            I
*      NIFIS    : NUMBER OF FISSILE ISOTOPES                   I
*      NGRP     : NUMBER OF GROUPS TREATED                     I
*      NMAT     : NUMBER OF MIXTURES CONSIDERED                I
*      NREG     : NUMBER OF REGIONS CONSIDERED                 I
*      XSDIA    : WITHIN GROUP SCATTERING                      R(NMAT+1)
*                 CROSS SECTION
*      XSCAT    : GROUP SCATTERING CROSS SECTION               R(NMAT+1)
*      XSCHI    : CHI                                          R(0:NMAT,
*                                                                NIFIS,
*                                                                NGRP)
* --------------------------   MOCFIM    ------------------------------
*
      IMPLICIT NONE
      INTEGER      NREG,NGRP,NMAT,NANIS,NIFIS,NFUNL,ISADJ,ITYPEC,NL,
     >             NSOUT,MATALB(-NSOUT:NREG)
      INTEGER      IG,JG,IR,IS,IOUT,IPRT
      REAL         XSDIA(0:NMAT,NGRP),XSCAT(NGRP,0:NMAT,0:NANIS,NGRP)
      REAL         ADJSOU(NREG,NGRP),XSNUF(0:NMAT,NIFIS,NGRP),
     >             XSCHI(0:NMAT,NIFIS,NGRP) 
      DOUBLE PRECISION ADJFLX(NFUNL,NREG,NGRP,8),AKEFF,FISOUR,DZERO
      IOUT =6
      DZERO=0.00000000000D+0
       IF (ITYPEC.EQ.6) THEN
         DO 22 IG=1,NGRP
            DO 12 IR=1,NREG
             IF (ISADJ.EQ.2) THEN
                ADJFLX(1,IR,IG,4)=ADJSOU(IR,IG)
             ELSE
                CALL XABORT('MOCFIM: INVALID ISADJ FOR ITYPEC=6')
             ENDIF
   12       CONTINUE   
   22    CONTINUE  
       ELSEIF (ITYPEC.LT.6) THEN
        DO 92 IG= 1,NGRP
        DO 91 IR= 1,NREG
          ADJFLX(1,IR,IG,4)=DZERO
          IF(ISADJ.EQ.2.OR.ISADJ.EQ.3) THEN 
           ADJFLX(1,IR,IG,4)=ADJSOU(IR,IG)
          ENDIF
   91   CONTINUE
   92   CONTINUE
        DO 96 IR=1,NREG
        DO 95 IS=1,NIFIS
          FISOUR=DZERO
          DO 93 IG=1,NGRP
           FISOUR=FISOUR+XSCHI(MATALB(IR),IS,IG)*ADJFLX(1,IR,IG,3)
   93     CONTINUE
          FISOUR=FISOUR/AKEFF
          DO 94 IG=1,NGRP
           ADJFLX(1,IR,IG,4)=ADJFLX(1,IR,IG,4)+
     >      XSNUF(MATALB(IR),IS,IG)*FISOUR
   94     CONTINUE
   95   CONTINUE
   96   CONTINUE
       ENDIF
       DO 45 IG= 1,NGRP
       DO 65 IR= 1,NREG
       DO 35 JG= 1,NGRP
         IF( IG.EQ.JG )THEN
          ADJFLX(1,IR,IG,4)= ADJFLX(1,IR,IG,4)+
     >     XSDIA(MATALB(IR),JG)* ADJFLX(1,IR,JG,3)
         ELSE
          ADJFLX(1,IR,IG,4)= ADJFLX(1,IR,IG,4)+
     >     XSCAT(IG,MATALB(IR),0,JG)*ADJFLX(1,IR,JG,3)
         ENDIF
   35  CONTINUE
   65  CONTINUE
   45  CONTINUE
      RETURN
      END