*DECK RECT1
      SUBROUTINE RECT1 (NA,A,B,SIGT,TRONC,PII,PVS,PSS,ALPA,PWA)
*
*-----------------------------------------------------------------------
*
* INTEGRATION OF THE COLLISION PROBABILITIES OF AN HOMOGENEOUS 2-D
* RECTANGLE (DP-0 SURFACE ANGULAR FLUX APPROXIMATION).
*
* INPUT PARAMETERS:
*   NA      : NUMBER OF GAUSS-LEGENDRE BASE POINTS.
*   A,B     : X- AND Y-THICKNESS OF THE RECTANGLE.
*   SIGT    : CROSS SECTION.
*   TRONC   : VOIDED BLOCK CRITERION.
*   ALPA    : GAUSS-LEGENDRE BASE POINTS. DIMENSION ALPA(NA)
*   PWA     : GAUSS-LEGENDRE WEIGHTS. DIMENSION PWA(NA)
*
* OUTPUT PARAMETERS:
*   PII     : VOLUME TO VOLUME REDUCED PROBABILITY.
*   PVS     : VOLUME TO SURFACE PROBABILITIES.
*             XINF: SURFACE 1   XSUP: SURFACE 2
*             YINF: SURFACE 3   YSUP: SURFACE 4
*             DIMENSION PVS(4,NZ)
*   PSS     : SURFACE TO SURFACE PROBABILITIES IN THE FOLLOWING ORDER:
*             PSS(I,J) IS THE PROBABILITY FROM SURFACE I TO SURFACE J.
*             DIMENSION PSS(4,4)
*
*---------------------------------------------- A. HEBERT 10/06/87 -----
*
      PARAMETER (MKI3=600,MKI4=600)
      PARAMETER (PI=3.141592654,ZI40=0.6666666667)
      DOUBLE PRECISION Y1,Y2,AOB,BOA,R,PHI,PHJ,RM
      DIMENSION PVS(*),PSS(4,*),ALPA(*),PWA(*),GAR(4),QUOT(2),PHIB(2),
     1 ABSC(2),PBB(5),ISL(4,4)
      COMMON /BICKL3/BI3(0:MKI3,3),PAS3,XLIM3,L3
      COMMON /BICKL4/BI4(0:MKI4,3),PAS4,XLIM4,L4
      SAVE ISL
      DATA ISL/5,4,1,1,
     1         4,5,1,1,
     2         3,3,5,2,
     3         3,3,2,5/
*
      AOB=A/B
      BOA=B/A
      X=2.0*A*B/(A+B)
      IF (X*SIGT.LE.TRONC) GO TO 120
      SIG=1.0/SIGT
      IF (A.EQ.B) GO TO 55
*
* RECTANGULAR CELL.
      QUOT(1)=BOA
      QUOT(2)=AOB
      PHIB(1)=ATAN(AOB)
      PHIB(2)=ATAN(BOA)
      ABSC(1)=B
      ABSC(2)=A
      DO 10 I=1,4
10    GAR(I)=0.0
      DO 50 IL=1,NA
      X=0.5*(ALPA(IL)+1.0)
      WA=PWA(IL)
      L=0
      DO 50 IZ=1,2
      PHI=PHIB(IZ)*X
      CO=COS(PHI)
      SI=SIN(PHI)
      Z1=PHIB(IZ)*WA*CO*SI/(PI*A)
      Z2=PHIB(IZ)*WA*(CO-QUOT(IZ)*SI)*2.0/PI
      POP=SIGT*ABSC(IZ)/CO
      ZI3=0.0
      ZI4=0.0
      IF (POP.GE.XLIM3) GO TO 40
      K=NINT(POP*PAS3)
      ZI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
      ZI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
40    GAR(L+1)=GAR(L+1)+(ZI40-ZI4)*Z1
      GAR(L+2)=GAR(L+2)+ZI3*Z2
50    L=L+2
      PBB(1)=(GAR(1)+GAR(3))*SIG
      PBB(2)=GAR(2)
      PBB(4)=GAR(4)
      GO TO 110
*
* SQUARE CELL
55    PHI1=0.25*PI
      GAR(1)=0.0
      GAR(2)=0.0
      DO 100 IL=1,NA
      X=0.5*(ALPA(IL)+1.0)
      WA=PWA(IL)
      PHI=PHI1*X
      CO=COS(PHI)
      SI=SIN(PHI)
      Z1=0.25*WA*CO*SI/A
      Z2=0.5*WA*(CO-SI)
      POP=SIGT*A/CO
      ZI3=0.0
      ZI4=0.0
      IF (POP.GE.XLIM3) GO TO 90
      K=NINT(POP*PAS3)
      ZI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
      ZI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
90    GAR(1)=GAR(1)+(ZI40-ZI4)*Z1
100   GAR(2)=GAR(2)+ZI3*Z2
      PBB(1)=2.0*GAR(1)*SIG
      PBB(2)=GAR(2)
      PBB(4)=PBB(2)
*
110   PBB(3)=PBB(1)*AOB
      PVS(3)=0.25*SIG*(1.0-2.0*PBB(1)-PBB(2))/B
      PVS(1)=0.25*SIG*(1.0-2.0*PBB(3)-PBB(4))/A
      PII=SIG*(1.0-2.0*(PVS(3)+PVS(1)))
      GO TO 130
*
* VOIDED CELL.
120   IF (A.GT.1.0E2*B) THEN
         PHI=0.5D0*PI
         R=A*(1.0D0+BOA*BOA*(0.5D0-0.125D0*BOA*BOA))
         RM=BOA*(0.5D0-0.125D0*BOA*BOA)
         Y1=LOG(DBLE((R+A)/B))
         PBB(3)=0.5D0*(1.0D0-RM)+SIGT*B*LOG(B/R)/PI
         PBB(1)=PBB(3)*BOA
         PBB(2)=(R-B)/A-4.0D0*SIGT*(B*PHI+B*BOA*LOG(B/R))/PI
     1   +SIGT*SIGT*B*B*(Y1-(R-B)/A)
         PBB(4)=RM
         PVS(3)=(PHI+0.5D0*BOA*LOG(B/R))/PI-0.25D0*SIGT*(B*Y1-BOA*(R-B))
         PVS(1)=-0.5D0*BOA*LOG(B/R)/PI
         PII=0.5D0*B*Y1-0.5D0*BOA*(R-B)
      ELSE IF (B.GT.1.0E2*A) THEN
         PHJ=0.5D0*PI
         R=B*(1.0D0+AOB*AOB*(0.5D0-0.125D0*AOB*AOB))
         RM=AOB*(0.5D0-0.125D0*AOB*AOB)
         Y2=LOG(DBLE((R+B)/A))
         PBB(1)=0.5D0*(1.0D0-RM)+SIGT*A*LOG(A/R)/PI
         PBB(3)=PBB(1)*AOB
         PBB(2)=RM
         PBB(4)=(R-A)/B-4.0D0*SIGT*(A*PHJ+A*AOB*LOG(A/R))/PI
     1   +SIGT*SIGT*A*A*(Y2-(R-A)/B)
         PVS(3)=-0.5D0*AOB*LOG(A/R)/PI
         PVS(1)=(PHJ+0.5D0*AOB*LOG(A/R))/PI-0.25D0*SIGT*(A*Y2-AOB*(R-A))
         PII=0.5D0*A*Y2-0.5D0*AOB*(R-A)
      ELSE
         PHI=ATAN(AOB)
         PHJ=ATAN(BOA)
         R=SQRT(A*A+B*B)
         Y1=LOG(DBLE((R+A)/B))
         Y2=LOG(DBLE((R+B)/A))
         PBB(1)=(A+B-R)/(A+A)+SIGT*(A*LOG(A/R)+B*BOA*LOG(B/R))/PI
     1   +SIGT*SIGT*(R*R*R-A*A*A-B*B*B)/(6.0D0*A)
         PBB(3)=PBB(1)*AOB
         PBB(2)=(R-B)/A-4.0D0*SIGT*(B*PHI+B*BOA*LOG(B/R))/PI
     1   +SIGT*SIGT*B*B*(Y1-(R-B)/A)
         PBB(4)=(R-A)/B-4.0D0*SIGT*(A*PHJ+A*AOB*LOG(A/R))/PI
     1   +SIGT*SIGT*A*A*(Y2-(R-A)/B)
         PVS(3)=(0.5D0*BOA*LOG(B/R)-0.5D0*AOB*LOG(A/R)+PHI)/PI
     1   -0.25D0*SIGT*((R*R*R-A*A*A-B*B*B)/(3.0D0*A*B)-(R-B)*BOA+B*Y1)
         PVS(1)=(0.5D0*AOB*LOG(A/R)-0.5D0*BOA*LOG(B/R)+PHJ)/PI
     1   -0.25D0*SIGT*((R*R*R-A*A*A-B*B*B)/(3.0D0*A*B)-(R-A)*AOB+A*Y2)
         PII=(A*A*A+B*B*B-R*R*R)/(6.0D0*A*B)+0.5D0*B*Y1+0.5D0*A*Y2
      ENDIF
  130 PVS(2)=PVS(1)
      PVS(4)=PVS(3)
      PBB(5)=0.0
      DO 140 JC=1,4
      DO 140 IC=1,4
  140 PSS(IC,JC)=PBB(ISL(IC,JC))
      RETURN
      END
