*DECK SHI
      SUBROUTINE SHI(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
C
C-----------------------------------------------------------------------
C
C INTERPOLATION OF NUCLEAR PROPERTIES IN A LATTICE CODE LIBRARY.
C
C INPUT/OUTPUT PARAMETERS:
C  NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.
C  HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
C  IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
C           =4 SEQUENTIAL ASCII FILE.
C  JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
C           =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
C           =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
C  KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.
C           DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
C           KENTRY(NENTRY)
C
C LINKED LIST / XSM FILE:  (ORDER IS ARBITRARY FOR 2,3,4)
C  HENTRY(1) : CREATION OR MODIFICATION TYPE(L_LIBRARY).
C  HENTRY(2) : OPTIONAL READ-ONLY TYPE(L_LIBRARY).
C  HENTRY(3) : READ-ONLY LINKED LIST TYPE(L_TRACK)
C  HENTRY(4) : OPTIONAL READ-ONLY SEQUENTIAL BINARY TRACKING FILE.
C
C-------------------------------------- AUTHOR: A. HEBERT ; 01/11/94 ---
C
      IMPLICIT         NONE
      INTEGER          IOUT,MAXED,NSTATE,ILCMUP,ILCMDN
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,MAXED=50,NSTATE=40,ILCMUP=1,ILCMDN=2,
     >                 NAMSBR='SHI   ')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER          NENTRY
      CHARACTER        HENTRY(NENTRY)*12
      INTEGER          IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
C----
C  MEMORY ALLOCATION
C----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          IMAT,IVOL,IIDL,JJMIX,JJDEN,JJAWR,JJTN,
     >                 ISNGAR,ISBGAR,ILSHI,JJNAM,JJNRF,JJNTFG,
     >                 JHLIB,IJCEDM,LMASKM,LMASKG,JSIGT1,JSIGT2,
     >                 JSIGT3,JNOCON,JSIGE
C----
C  INPUT PARAMETERS
C----
      INTEGER          ITYPLU,INTLIR
      CHARACTER        CARLIR*12
      REAL             REALIR
      DOUBLE PRECISION DBLLIR
C----
C  LOCAL PARAMETERS
C----
      CHARACTER        HSMG*131,TEXT12*12,HSIGN*12,
     >                 CDOOR*12,TITLE*72,HVECT(MAXED)*6
      INTEGER          ISTATE(NSTATE),KCHAR(18),ITC,IKLIB,IPLIB,IEN,
     >                 IPLIBO,IKLIBO,IPTRK,IKTRK,NREG,NGMIX,IFTRAK,
     >                 ILCMLN,ILCMTY,NBISO,NGRO,NL,ITRANC,ITIME,NLIB,
     >                 NGF,NGFR,NEDMAC,NBMIX,NRES,IDEB,IED,IPRINT,
     >                 IGRMIN,IGRMAX,MAXX0,IBIEFF,IGC,ITRANZ
      REAL             EPS
      LOGICAL          LEAKSW
C----
C  PARAMETER VALIDATION.
C----
      IF(NENTRY .LT. 2) CALL XABORT(NAMSBR//
     >': AT LEAST TWO DATA STRUCTURES REQUIRED')
      IF(IENTRY(1) .NE. 1 .AND.
     >   IENTRY(1) .NE. 2) CALL XABORT(NAMSBR//
     >': FIRST DATA STRUCTURE NOT A LINKED LIST OR XSM FILE')
C----
C  CHECK IF FIRST ENTRY IS LIBRARY OR A NEW FILE
C----
      IKLIB=1
      IPLIB=KENTRY(IKLIB)
      IF(JENTRY(IKLIB) .EQ. 0) THEN
C----
C  FOR A NEW LIB FIND OLD LIB IN READ ONLY MODE
C----
        IPLIBO=0
        IKLIBO=0
        DO 100 IEN=2,4
          IF(IENTRY(IEN) .EQ. 1 .OR. IENTRY(IEN) .EQ. 2) THEN
            CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
            WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
            IF(HSIGN .EQ. 'L_LIBRARY' .AND. IKLIBO .EQ. 0) THEN
              IPLIBO=KENTRY(IEN)
              IKLIBO=IEN
              GO TO 105
            ENDIF
          ENDIF
 100    CONTINUE
        CALL XABORT(NAMSBR//
     >  ': NO DATA STRUCTURE WITH SIGNATURE L_LIBRARY FOUND.')
 105    CONTINUE
        TEXT12=HENTRY(IKLIBO)
        IF(JENTRY(IKLIBO) .NE. 2) CALL XABORT(NAMSBR//
     >  ': DATA STRUCTURE '//TEXT12//'  NOT IN READ-ONLY MODE')
        CALL LCMEQU(KENTRY(IKLIBO),KENTRY(IKLIB))
      ELSE IF(JENTRY(IKLIB) .EQ. 1) THEN
        CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
        IF(HSIGN.NE.'L_LIBRARY') THEN
          TEXT12=HENTRY(IKLIB)
          CALL XABORT(NAMSBR//
     >    ': DATA STRUCTURE '//TEXT12//' HAS SIGNATURE '//HSIGN//
     >    '. L_LIBRARY EXPECTED.')
        ENDIF
      ELSE
        TEXT12=HENTRY(IKLIB)
        CALL XABORT(NAMSBR//
     >  ': DATA STRUCTURE '//TEXT12//' NOT IN CREATE OR MODIFY MODE')
      ENDIF
C----
C  FIND TRACKING DATA STRUCTURE
C----
      IPTRK=0
      IKTRK=0
      DO 110 IEN=2,4
        IF(IENTRY(IEN) .EQ. 1 .OR. IENTRY(IEN) .EQ. 2) THEN
          CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
          WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
          IF(HSIGN .EQ. 'L_TRACK' .AND. IKTRK .EQ. 0) THEN
            IPTRK=KENTRY(IEN)
            IKTRK=IEN
            GO TO 115
          ENDIF
        ENDIF
 110  CONTINUE
      CALL XABORT(NAMSBR//
     >': NO DATA STRUCTURE WITH SIGNATURE L_TRACK FOUND.')
 115  CONTINUE
      TEXT12=HENTRY(IKTRK)
      IF(JENTRY(IKTRK) .NE. 2) CALL XABORT(NAMSBR//
     >': DATA STRUCTURE '//TEXT12//' NOT IN READ-ONLY MODE')
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      CALL LCMGET(IPTRK,'TRACK-TYPE',KCHAR)
      WRITE(CDOOR,'(3A4)') (KCHAR(ITC),ITC=1,3)
      IF(CDOOR .NE. 'JPM'    .AND.
     >   CDOOR .NE. 'EXCELL' .AND.
     >   CDOOR .NE. 'SYBIL' ) CALL XABORT(NAMSBR//
     >  ': ILLEGAL TRACKING FORMAT '//CDOOR)
      NREG=ISTATE(1)
      LEAKSW=ISTATE(3).EQ.0
      NGMIX=ISTATE(4)
C----
C  FIND TRACKING FLIE
C----
      IFTRAK=0
      DO 120 IEN=2,4
        IF(IENTRY(IEN) .EQ. 3) THEN
          IFTRAK=KENTRY(IEN)
          TEXT12=HENTRY(IEN)
          IF(JENTRY(IEN).NE.2) CALL XABORT(NAMSBR//
     >    ': TRACKING FILE '//TEXT12//' NOT IN READ-ONLY MODE')
          GO TO 125
        ENDIF
 120  CONTINUE
 125  CONTINUE
C----
C  RECOVER GENERAL TRACKING INFORMATION.
C----
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(RBASE,NREG,IVOL)
      CALL SETARA(IBASE,NREG,IIDL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',RBASE(IVOL))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IIDL))
      CALL LCMLEN(IPTRK,'TITLE',ILCMLN,ILCMTY)
      IF(ILCMLN .GT. 0) THEN
         CALL LCMGET(IPTRK,'TITLE',KCHAR)
         WRITE(TITLE,'(18A4)') (KCHAR(ITC),ITC=1,18)
      ELSE
         TITLE='*** NO TITLE PROVIDED ***'
      ENDIF
C----
C  RECOVER GENERAL INTERNAL LIBRARY INFORMATION.
C----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
      NBISO=ISTATE(2)
      NGRO=ISTATE(3)
      NL=ISTATE(4)
      ITRANC=ISTATE(5)
      ITIME=ISTATE(7)
      NLIB=ISTATE(8)
      NGF=ISTATE(9)
      NGFR=ISTATE(10)
      NEDMAC=ISTATE(13)
      NBMIX=ISTATE(14)
      NRES=ISTATE(15)
      IF(NGMIX .GT. NBMIX) THEN
        WRITE(HSMG,9000) NAMSBR,NGMIX,NBMIX
        CALL XABORT(HSMG)
      ENDIF
      IF(NLIB .EQ. 0 .OR. NRES .EQ. 0)
     >  CALL XABORT(NAMSBR//': THERE IS NO RESONANT ISOTOPES.')
C----
C  RECOVER INFORMATION ON LCM
C----
      CALL SETARA(IBASE,NBISO,JJMIX)
      CALL SETARA(RBASE,NBISO,JJDEN)
      CALL SETARA(RBASE,NBISO,JJAWR)
      CALL SETARA(RBASE,NBISO,JJTN)
      CALL SETARA(RBASE,NGRO*NBISO,ISNGAR)
      CALL SETARA(RBASE,NGRO*NBISO,ISBGAR)
      CALL SETARA(IBASE,NBISO,ILSHI)
      CALL SETARA(IBASE,3*NBISO,JJNAM)
      CALL SETARA(IBASE,3*NBISO,JJNRF)
      CALL SETARA(IBASE,NBISO,JJNTFG)
      CALL SETARA(IBASE,2*NBISO*5,JHLIB)
      CALL SETARA(IBASE,2*NEDMAC,IJCEDM)
      CALL LCMGET(IPLIB,'ISOTOPESMIX ',IBASE(JJMIX))
      CALL LCMGET(IPLIB,'ISOTOPESDENS',RBASE(JJDEN))
      CALL LCMGET(IPLIB,'ISOTOPESAWR ',RBASE(JJAWR))
      CALL LCMGET(IPLIB,'ISOTOPESTEMP',RBASE(JJTN))
      CALL LCMGET(IPLIB,'ISOTOPESDSB ',RBASE(ISBGAR))
      CALL LCMGET(IPLIB,'ISOTOPESDSN ',RBASE(ISNGAR))
      CALL LCMGET(IPLIB,'ISOTOPESSHI ',IBASE(ILSHI))
      CALL LCMGET(IPLIB,'ISOTOPESUSED',IBASE(JJNAM))
      CALL LCMGET(IPLIB,'ISOTOPERNAME',IBASE(JJNRF))
      CALL LCMGET(IPLIB,'ILIBRARYTYPE',IBASE(JHLIB))
      CALL LCMGET(IPLIB,'ILIBRARYNAME',IBASE(JHLIB+2*NBISO))
      CALL LCMGET(IPLIB,'ISOTOPESCOH ',IBASE(JHLIB+4*NBISO))
      CALL LCMGET(IPLIB,'ISOTOPESINC ',IBASE(JHLIB+6*NBISO))
      CALL LCMGET(IPLIB,'ISOTOPESHIN ',IBASE(JHLIB+8*NBISO))
      CALL LCMGET(IPLIB,'ISOTOPESNTFG',IBASE(JJNTFG))
      CALL LCMGET(IPLIB,'HVECT       ',IBASE(IJCEDM))
      IDEB=IJCEDM
      DO 130 IED=1,NEDMAC
        WRITE(HVECT(IED),'(A4,A2)') IBASE(IDEB),IBASE(IDEB+1)
        IDEB=IDEB+2
 130  CONTINUE
      CALL RLSARA(IBASE(IJCEDM))
C----
C  READ THE LIBRARY SELF-SHIELDING PARAMETERS
C----
      IPRINT=1
      CALL LCMLEN(IPLIB,'SHIBA',ILCMLN,ILCMTY)
      IF(ILCMLN .EQ. -1) THEN
        CALL LCMSIX(IPLIB,'SHIBA',ILCMUP)
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        CALL LCMGET(IPLIB,'EPS-SHIBA',EPS)
        CALL LCMSIX(IPLIB,'SHIBA',ILCMDN)
        IGRMIN=ISTATE(1)
        IGRMAX=ISTATE(2)
        MAXX0=ISTATE(3)
        IBIEFF=ISTATE(4)
        IGC=ISTATE(5)
        ITRANZ=ISTATE(6)
      ELSE
        IGRMIN=NGF+1
        IGRMAX=NGFR
        MAXX0=20
        IBIEFF=0
        IGC=1
        ITRANZ=ITRANC
        EPS=1.0E-4
      ENDIF
C----
C   READ THE SELF-SHIELDING INPUT DATA.
C----
 140  CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
      IF(ITYPLU .EQ. 10) THEN
        GO TO 145
      ELSE IF(ITYPLU .NE. 3) THEN
        CALL XABORT(NAMSBR//': CHARACTER DATA EXPECTED.')
      ENDIF
      IF(CARLIR .EQ. ';') THEN
        GO TO 145
      ELSE IF(CARLIR(1:4) .EQ. 'EDIT') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': INTEGER PRINT LEVEL EXPECTED')
        IPRINT=INTLIR
      ELSE IF(CARLIR(1:4) .EQ. 'GRMI') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': INTEGER GRMIN EXPECTED')
        IGRMIN=INTLIR
      ELSE IF(CARLIR(1:4) .EQ. 'GRMA') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': INTEGER GRMAX EXPECTED')
        IGRMAX=INTLIR
        IF(IGRMAX .GT. NGRO) IGRMAX=NGRO
      ELSE IF(CARLIR(1:4) .EQ. 'MXIT') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': INTEGER MXIT EXPECTED')
        MAXX0=INTLIR
      ELSE IF(CARLIR(1:3) .EQ. 'EPS') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': REAL EPS EXPECTED')
        EPS=REALIR
      ELSE IF(CARLIR(1:4) .EQ. 'NOLJ') THEN
        IBIEFF=0
      ELSE IF(CARLIR(1:2) .EQ. 'LJ') THEN
        IBIEFF=1
      ELSE IF(CARLIR(1:4) .EQ. 'NOGC') THEN
        IGC=0
      ELSE IF(CARLIR(1:2) .EQ. 'GC') THEN
        IGC=1
      ELSE IF(CARLIR(1:4) .EQ.'NOTR') THEN
        ITRANZ=0
      ELSE IF(CARLIR(1:4) .EQ. 'PIJ') THEN
        IF(CDOOR .EQ. 'JPM') CDOOR='JPMC'
      ELSE
        CALL XABORT(NAMSBR//': '//CARLIR//' IS AN INVALID KEY-WORD.')
      ENDIF
      GO TO 140
C----
C  INPUT DATA READ COMPLETE
C----
 145  CONTINUE
      IF(NBMIX .EQ. 0) CALL XABORT(NAMSBR//
     >': NBMIX NOT YET DEFINED.')
      IF(NRES .EQ. 0) CALL XABORT(NAMSBR//
     >': THERE IS NO RESONANT ISOTOPES.')
      IF(NREG .EQ. 0) CALL XABORT(NAMSBR//
     >': NREG = 0')
      CALL SETARA(IBASE,NBMIX,LMASKM)
      CALL SETARA(IBASE,NGRO,LMASKG)
      CALL SETARA(RBASE,NBMIX*NGRO,JSIGT1)
      CALL SETARA(RBASE,NBMIX*NGRO,JSIGT2)
      CALL SETARA(RBASE,NBMIX*NGRO,JSIGT3)
      CALL SETARA(RBASE,NBMIX*NGRO,JNOCON)
      CALL SETARA(RBASE,NRES*NGRO,JSIGE)
      CALL SHIDRV(IPLIB ,IPTRK ,IFTRAK,NGRO  ,NBISO ,NBMIX ,
     >            NREG  ,CDOOR ,NRES  ,IPRINT,LEAKSW,TITLE ,
     >            IGRMIN,IGRMAX,MAXX0 ,IBIEFF,IGC   ,ITRANZ,
     >            EPS   ,NEDMAC,HVECT ,NL    ,ITIME ,ITRANC,
     >            IBASE(JJNAM) ,IBASE(JJMIX) ,RBASE(JJDEN) ,
     >            RBASE(JJTN)  ,RBASE(ISNGAR),RBASE(ISBGAR),
     >            IBASE(ILSHI) ,IBASE(IMAT)  ,RBASE(IVOL)  ,
     >            IBASE(LMASKM),IBASE(LMASKG),RBASE(JSIGT1),
     >            RBASE(JSIGT2),RBASE(JSIGT3),RBASE(JNOCON),
     >            RBASE(JSIGE) ,IBASE(JHLIB) ,IBASE(JJNTFG),
     >            IBASE(JJNRF) ,RBASE(JJAWR) )
      CALL RLSARA(RBASE(JSIGE))
      CALL RLSARA(RBASE(JNOCON))
      CALL RLSARA(RBASE(JSIGT3))
      CALL RLSARA(RBASE(JSIGT2))
      CALL RLSARA(RBASE(JSIGT1))
      CALL RLSARA(IBASE(LMASKG))
      CALL RLSARA(IBASE(LMASKM))
C----
C  RELEASE GENERAL INTERNAL LIBRARY INFORMATION.
C----
      CALL RLSARA(IBASE(JHLIB))
      CALL RLSARA(IBASE(JJNTFG))
      CALL RLSARA(IBASE(JJNRF))
      CALL RLSARA(IBASE(JJNAM))
      CALL RLSARA(IBASE(ILSHI))
      CALL RLSARA(RBASE(ISBGAR))
      CALL RLSARA(RBASE(ISNGAR))
      CALL RLSARA(RBASE(JJTN))
      CALL RLSARA(RBASE(JJAWR))
      CALL RLSARA(RBASE(JJDEN))
      CALL RLSARA(IBASE(JJMIX))
C----
C  RELEASE GENERAL TRACKING INFORMATION.
C----
      CALL RLSARA(IBASE(IIDL))
      CALL RLSARA(RBASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      RETURN
C----
C  FORMAT
C----
 9000 FORMAT(A6,': NUMBER OF MIXTURES IN THE TRACKING FILE (',
     >       I5,') > NUMBER OF MIXTURES IN LIBRARY =(',I5,')')
      END
