*DECK SHIDRV
      SUBROUTINE SHIDRV(IPLIB ,IPTRK ,IFTRAK,NGRO  ,NBISO ,NBMIX ,
     >                  NREG  ,CDOOR ,NRES  ,IPRINT ,LEAKSW,TITR  ,
     >                  IGRMIN,IGRMAX,MAXX0 ,IBIEFF,IGC   ,ITRANZ,
     >                  EPS   ,NEDMAC,HVECT ,NL    ,ITIME ,ITRANC,
     >                  ISONAM,MIX   ,DEN   ,TN    ,SN    ,SB    ,
     >                  LSHI  ,MAT   ,VOL   ,MASK  ,MASKL ,SIGT1 ,
     >                  SIGT2 ,SIGT3 ,NOCONV,SIGE  ,IHLIB ,NTFG  ,
     >                  ISONRF,AWR   )
C
C-----------------------------------------------------------------------
C
C PERFORM A MULTIDIMENSIONAL SELF-SHIELDING CALCULATION IN ORDER TO
C COMPUTE THE DILUTION CROSS SECTION OF EACH RESONANT ISOTOPE PRESENT
C IN THE DOMAIN.
C
C  IPLIB  : POINTER TO THE INTERNAL MICROSCOPIC CROSS SECTION LIBRARY
C           (L_LIBRARY SIGNATURE).
C  IPTRK  : POINTER TO THE TRACKING. (L_TRACK SIGNATURE).
C  IFTRAK : UNIT NUMBER OF THE SEQUENTIAL BINARY TRACKING FILE.
C  NGRO   : NUMBER OF ENERGY GROUPS.
C  NBISO  : NUMBER OF ISOTOPES PRESENT IN THE CALCULATION DOMAIN.
C  NBMIX  : NUMBER OF MIXTURES IN THE MACROLIB.
C  NREG   : NUMBER OF REGIONS.
C  CDOOR  : CHARACTER*12 NAME OF THE GEOMETRY/SOLUTION MODULE.
C           IF CDOOR='JPM' SHIBA WILL PERFORM A 'FAST' SELF-SHIELDING
C           CALCULATION.
C  NRES   : NUMBER OF RESONANT MIXTURES.
C  IPRINT  : PRINT FLAG.
C  LEAKSW : LEAKSW=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
C           IS PRESENT.
C  TITR   : TITLE.
C  IGRMIN : FIRST GROUP WHERE THE SELF-SHIELDING IS APPLIED.
C  IGRMAX : MOST THERMAL GROUP WHERE THE SELF-SHIELDING IS APPLIED.
C  MAXX0  : MAXIMUM NUMBER OF SELF-SHIELDING ITERATIONS.
C  IBIEFF : =1: WILL ACTIVATE A LIVOLANT-JEANPIERRE EQUIVALENCE
C           SIMILAR TO THE TECHNIQUE USED UN THE BIEFF SUBROUTINE OF
C           THE APOLLO-1 CODE.
C  IGC    : =1: WILL ACTIVATE A GOLDSTEIN-COHEN APPROXIMATION IN CASES
C           WHERE GOLDSTEIN-COHEN ARE AVAILABLE IN THE INTERNAL LIBRARY.
C  ITRANZ : TYPE OF TRANSPORT CORRECTION USED IN THE SELF-SHIELDING
C           CALCULATIONS.
C  EPS    : CONVERGENCE CRITERION FOR THE SELF-SHIELDING ITERATIONS.
C  NEDMAC :
C  HVECT  : DIMENSION HVECT(NEDMAC)*6
C  NL     : SCATTERING LEGENDRE ORDER
C  ITIME  : TIME STEP
C  ITRANC : TRANSPORT CORRECTION
C  ISONAM : NAME OF ISOTOPE. ISONAM(3,NBISO)
C  MIX    : MIX NUMBER OF EACH ISOTOPE. CAN BE ZERO.
C           DIMENSION MIX(NBISO)
C  DEN    : DENSITY OF EACH ISOTOPE. DIMENSION DEN(NBISO)
C  TN     : TEMPERATURE OF EACH ISOTOPE. DIMENSION TN(NBISO)
C  SN     : ESTIMATE (INPUT) OR COMPUTED (OUTPUT) DILUTION CROSS
C           SECTION IN EACH ENERGY GROUP FOR EACH ISOTOPE
C           DIMENSION SN(NGRO,NBISO)
C  SB     : COMPUTED (OUTPUT) DILUTION CROSS SECTION AS USED
C           IN LIVOLANT AND JEANPIERRE NORMALIZATION
C           DIMENSION SB(NGRO,NBISO)
C  LSHI   : RESONANT REGION NUMBER ASSOCIATED WITH ISOTOPE I.
C           INFINITE DILUTION WILL BE ASSUMED IF LSHI(I)=0.
C           DIMENSION LSHI(NBISO)
C           Isotope not in geometry if LSHI(I)<0.
C  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
C           DIMENSION MAT(NREG)
C  VOL    : REGION VOLUME DIMENSION VOL(NREG)
C  MASK   : MIXTURE MASK MASK(NBMIX)
C  MASKL  : GROUP MASK MASKL(NGRO)
C  SIGT1  : WORK CROSS SECTION SIGT1(NBMIX,NGRO)
C  SIGT2  : WORK CROSS SECTION SIGT2(NBMIX,NGRO)
C  SIGT3  : WORK CROSS SECTION SIGT3(NBMIX,NGRO)
C  NOCONV : CONVERGENCE VECTOR NOCONV(NBMIX,NGRO)
C  SIGE   : WORK DILUTION VECTOR SIGE(NRES,NGRO)
C  IHLIB  : LIBRARY NAMES IHLIB(2,NBISO,5)
C  NTFG   : NUMBER OF THERMAL GROUP FOR INCOHERENT SCATTERING
C            NTFG(NBISO)
C  ISONRF : REFERENCE ISOTOPES NAMES ISONRF(3,NBISO)
C  AWR    : ISOTOPIC ATOMIC WEIGHT AWR(NBISO)
C
C--------------------------------------- AUTHOR: A. HEBERT 26/05/88 ----
C
      IMPLICIT    NONE
      INTEGER     IOUT,NSTATE,NDPROC,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NSTATE=40,NDPROC=20,ILCMUP=1,ILCMDN=2,
     >            NAMSBR='SHIDRV')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPLIB,IPTRK,IFTRAK,NGRO,NBISO,NBMIX,NREG,
     >            NRES,IPRINT,IGRMIN,IGRMAX,MAXX0,IBIEFF,IGC,ITRANZ,
     >            NEDMAC,NL,ITIME,ITRANC
      CHARACTER   CDOOR*12,TITR*72,HVECT(NEDMAC)*6
      LOGICAL     LEAKSW,MASK(NBMIX),MASKL(NGRO),NOCONV(NBMIX,NGRO)
      INTEGER     ISONAM(3,NBISO),MIX(NBISO),LSHI(NBISO),MAT(NREG),
     >            IHLIB(2,NBISO,5),NTFG(NBISO),ISONRF(3,NBISO)
      REAL        EPS,DEN(NBISO),TN(NBISO),SN(NGRO,NBISO),
     >            SB(NGRO,NBISO),VOL(NREG),SIGT1(NBMIX,NGRO),
     >            SIGT2(NBMIX,NGRO),SIGT3(NBMIX,NGRO),
     >            SIGE(NBMIX,NGRO),AWR(NBISO)
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     IIRES,ISIG0,ISIG3,ITOTAL,ISIGOL,IGAR,IMIX2,
     >            IIRNBM,ISIGRE,IMASKI,ITISOM,IDELTA,ITYPRO
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CGRPNM*12,ERRMSG*130
      INTEGER     ISTATE(NSTATE),IREG,ISO,IGR,IBM,NBRMIX,NITER,
     >            NPROC,LNGRO,ICOUNT,ILCMLN,ILCMTY,ITSTMP
      LOGICAL     START,BIEFF,LGC
      REAL        TMPDAY(3),ZZMAX,ZNORM,YMAX,ZMAX
*
* READ SELF SHIELDING DATA.
      IF(LEAKSW) CALL XABORT(NAMSBR//
     >': NEUTRON LEAKAGE IS FORBIDDEN.')
      IF(CDOOR .EQ. ' ') CALL XABORT(NAMSBR//
     >': THE GEOMETRY IS NOT YET DEFINED.')
      BIEFF=(IBIEFF .EQ. 1)
      LGC=(IGC .EQ. 1)
*
      IF(IPRINT .GT. 0) THEN
        WRITE(IOUT,400) TITR,CDOOR
        WRITE(IOUT,405) IGRMIN,IGRMAX,MAXX0,IBIEFF,IGC,ITRANZ,EPS
      ENDIF
      IF(NRES .EQ. 0) THEN
        WRITE(IOUT,410) NAMSBR
        RETURN
      ENDIF
      DO 30 IREG=1,NREG
        IF(MAT(IREG) .GT. NBMIX) THEN
          WRITE(ERRMSG,380) NAMSBR,NBMIX
          CALL XABORT(ERRMSG)
        ENDIF
   30 CONTINUE
      IGRMAX=MIN(IGRMAX,NGRO)
      DO 40 IGR=1,NGRO
        DO 41 ISO=1,NBISO
          SB(IGR,ISO)=SN(IGR,ISO)
 41     CONTINUE
        DO 42 IBM=1,NBMIX
          NOCONV(IBM,IGR)=.FALSE.
 42     CONTINUE
 40   CONTINUE
      CALL LCMSIX(IPLIB,'MACROLIB',ILCMUP)
      DO 50 IGR=IGRMIN,IGRMAX
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRO
        CALL LCMSIX(IPLIB,CGRPNM,ILCMUP)
        CALL LCMGET(IPLIB,'TOTAL',SIGT2(1,IGR))
        DO 51 IBM=1,NBMIX
          SIGT3(IBM,IGR)=0.0
 51     CONTINUE
        IF(ITRANZ .GE. 1) THEN
*       READ TRANSPORT CORRECTION
          CALL LCMLEN(IPLIB,'TRANC',ILCMLN,ILCMTY)
          IF(ILCMLN .EQ. NBMIX)
     >    CALL LCMGET(IPLIB,'TRANC',SIGT3(1,IGR))
        ENDIF
        CALL LCMSIX(IPLIB,CGRPNM,ILCMDN)
        DO 52 IBM=1,NBMIX
          NOCONV(IBM,IGR)=.TRUE.
 52     CONTINUE
 50   CONTINUE
      CALL LCMSIX(IPLIB,'MACROLIB',ILCMDN)
      IF(IPRINT .GE. 5) THEN
        WRITE(IOUT,'(/20H SHIBA INPUT VALUES:/)')
        DO 80 IGR=IGRMIN,IGRMAX
          WRITE(IOUT,420) IGR
          WRITE(IOUT,470) (SN(IGR,ISO),ISO=1,NBISO)
          WRITE(IOUT,480) (SIGT2(IBM,IGR),IBM=1,NBMIX)
   80   CONTINUE
        WRITE(IOUT,490)
      ENDIF
*----
*  Eliminate isotope absent from geometry
*----
      DO IBM=1,NBMIX
        DO IREG=1,NREG
          IF(MAT(IREG) .EQ. IBM) GO TO 1001
        ENDDO
        DO ISO=1,NBISO
          IF(MIX(ISO) .EQ. IBM) LSHI(ISO)=-ABS(LSHI(ISO))
        ENDDO
1001    CONTINUE
      ENDDO
*
* DETERMINE THE AMOUNT OF SCRATCH STORAGE REQUIRED.
      NBRMIX=0
      DO 90 ISO=1,NBISO
        IF(LSHI(ISO) .GT. 0) NBRMIX=NBRMIX+1
   90 CONTINUE
*
* ITERATION LOOP.
      NITER=0
 100  CONTINUE
      NITER=NITER+1
      START=(NITER .EQ. 1)
      IF(IPRINT .GE. 5) WRITE(IOUT,430) NITER
      DO 110 IGR=IGRMIN,IGRMAX
        DO 110 IBM=1,NBMIX
          SIGT1(IBM,IGR)=SIGT2(IBM,IGR)
 111    CONTINUE
 110  CONTINUE
      NPROC=NDPROC+NL
      CALL SETARA(IBASE,NBMIX,IIRES)
      CALL SETARA(RBASE,NBMIX,ISIG0)
      CALL SETARA(RBASE,NBMIX,ISIG3)
      CALL SETARA(RBASE,NGRO*NBRMIX,ITOTAL)
      CALL SETARA(RBASE,NGRO*NBRMIX,ISIGOL)
      CALL SETARA(RBASE,NGRO,IGAR)
      CALL SETARA(IBASE,NBISO,IMIX2)
      CALL SETARA(IBASE,NBMIX,IIRNBM)
      CALL SETARA(RBASE,NBMIX,ISIGRE)
      CALL SETARA(IBASE,NBISO,IMASKI)
      CALL SETARA(IBASE,NBISO,ITISOM)
      CALL SETARA(RBASE,NGRO,IDELTA)
      CALL SETARA(IBASE,NPROC,ITYPRO)
      CALL SHISN2(IPLIB ,IPTRK ,IFTRAK,NGRO  ,NBISO ,NBMIX ,
     >            NREG  ,CDOOR ,NRES  ,IPRINT,ISONAM,MIX   ,
     >            DEN   ,TN    ,SN    ,SB    ,LSHI  ,MAT   ,
     >            VOL   ,LEAKSW,TITR  ,START ,SIGT2 ,SIGT3 ,
     >            NOCONV,BIEFF ,LGC   ,SIGE  ,IHLIB ,NTFG  ,
     >            ISONRF,AWR   ,NEDMAC,HVECT ,NL    ,ITIME ,
     >            ITRANC,NPROC ,NBRMIX,
     >            IBASE(IIRES) ,RBASE(ISIG0) ,RBASE(ISIG3) ,
     >            RBASE(ITOTAL),RBASE(ISIGOL),RBASE(IGAR)  ,
     >            IBASE(IMIX2) ,IBASE(IIRNBM),RBASE(ISIGRE),
     >            IBASE(IMASKI),IBASE(ITISOM),RBASE(IDELTA),
     >            IBASE(ITYPRO))
      CALL RLSARA(IBASE(ITYPRO))
      CALL RLSARA(RBASE(IDELTA))
      CALL RLSARA(IBASE(ITISOM))
      CALL RLSARA(IBASE(IMASKI))
      CALL RLSARA(RBASE(ISIGRE))
      CALL RLSARA(IBASE(IIRNBM))
      CALL RLSARA(IBASE(IMIX2))
      CALL RLSARA(RBASE(IGAR))
      CALL RLSARA(RBASE(ISIGOL))
      CALL RLSARA(RBASE(ITOTAL))
      CALL RLSARA(RBASE(ISIG3))
      CALL RLSARA(RBASE(ISIG0))
      CALL RLSARA(IBASE(IIRES))
      ZZMAX=0.0
      LNGRO=0
      ICOUNT=0
      DO 140 IGR=IGRMIN,IGRMAX
        ZNORM=0.0
        DO 141 IBM=1,NBMIX
          ZNORM=MAX(ZNORM,ABS(SIGT2(IBM,IGR)))
 141    CONTINUE
        ZMAX=0.0
        MASKL(IGR)=.FALSE.
        DO 142 IBM=1,NBMIX
          YMAX=ABS(SIGT1(IBM,IGR)-SIGT2(IBM,IGR))/ZNORM
          ZMAX=MAX(ZMAX,YMAX)
          NOCONV(IBM,IGR)=NOCONV(IBM,IGR) .AND. (YMAX .GT. EPS)
          MASKL(IGR)=MASKL(IGR) .OR. NOCONV(IBM,IGR)
 142    CONTINUE
        IF(MASKL(IGR)) ICOUNT=ICOUNT+1
        IF(ZMAX.GT.ZZMAX) THEN
          ZZMAX=ZMAX
          LNGRO=IGR
        ENDIF
 140  CONTINUE
      IF(IPRINT .GE. 5)
     >  WRITE(IOUT,440) NITER,ICOUNT,ZZMAX,LNGRO
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,450) (IGR,MASKL(IGR),IGR=IGRMIN,IGRMAX)
        WRITE(IOUT,'(/31H INPUT MACROSCOPIC X-S IN GROUP,I4,1H:)')
     >    LNGRO
        WRITE(IOUT,'(1X,1P,10E12.4)')
     >    (SIGT1(IBM,LNGRO),IBM=1,NBMIX)
        WRITE(IOUT,'(/32H OUTPUT MACROSCOPIC X-S IN GROUP,I4,1H:)')
     >    LNGRO
        WRITE(IOUT,'(1X,1P,10E12.4)')
     >    (SIGT2(IBM,LNGRO),IBM=1,NBMIX)
        WRITE(IOUT,'(/29H OUTPUT DILUTION X-S IN GROUP,I4,1H:)')
     >    LNGRO
        WRITE(IOUT,'(1X,1P,10E12.4)')
     >    (SN(LNGRO,ISO),ISO=1,NBISO)
      ENDIF
      IF((NITER.GE.MAXX0).AND.(ICOUNT.GT.0)) THEN
         WRITE(IOUT,390) NAMSBR
        GO TO 150
      ELSE IF(ICOUNT.GT.0) THEN
        GO TO 100
      ENDIF
*
* CONVERGENCE IS OBTAINED.
  150 CONTINUE
      IF(IPRINT .GE. 5) THEN
        WRITE(IOUT,'(/21H SHIBA OUTPUT VALUES:/)')
        DO 160 IGR=IGRMIN,IGRMAX
          WRITE(IOUT,420) IGR
          WRITE(IOUT,460) (SIGE(ISO,IGR),ISO=1,NRES)
          WRITE(IOUT,470) (SN(IGR,ISO),ISO=1,NBISO)
          IF(BIEFF) WRITE(IOUT,475) (SB(IGR,ISO),ISO=1,NBISO)
          WRITE(IOUT,480) (SIGT2(IBM,IGR),IBM=1,NBMIX)
  160   CONTINUE
        WRITE(IOUT,490)
      ENDIF
*
* COMPUTE THE NEW SELF-SHIELDED MACROSCOPIC CROSS SECTIONS.
      DO 170 IGR=1,NGRO
        MASKL(IGR)=.FALSE.
 170  CONTINUE
      DO 180 IGR=IGRMIN,IGRMAX
        MASKL(IGR)=.TRUE.
 180  CONTINUE
      DO 200 IBM=1,NBMIX
        DO 190 ISO=1,NBISO
          MASK(IBM)=(MIX(ISO) .EQ. IBM) .AND. (LSHI(ISO) .GT. 0)
          IF(MASK(IBM)) GO TO 195
  190   CONTINUE
  195   CONTINUE
  200 CONTINUE
      ITSTMP=0
      TMPDAY(1)=0.0
      TMPDAY(2)=0.0
      TMPDAY(3)=0.0
      CALL LIBMIX(IPLIB ,NGRO  ,NBISO ,ISONAM,MIX   ,DEN   ,
     >            MASK  ,MASKL ,ITSTMP,TMPDAY)
      IF(IPRINT .GT. 0) WRITE(IOUT,500) NITER,ZZMAX
*
* STORE THE GENERAL SHIBA PARAMETERS.
      CALL XDRSET(ISTATE,NSTATE,0)
      ISTATE(1)=IGRMIN
      ISTATE(2)=IGRMAX
      ISTATE(3)=MAXX0
      ISTATE(4)=IBIEFF
      ISTATE(5)=IGC
      ISTATE(6)=ITRANZ
      CALL LCMSIX(IPLIB,'SHIBA',ILCMUP)
      CALL LCMPUT(IPLIB,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL LCMPUT(IPLIB,'EPS-SHIBA',1,2,EPS)
      CALL LCMSIX(IPLIB,'SHIBA',ILCMDN)
*
* SAVE THE GROUP- AND ISOTOPE-DEPENDENT DILUTIONS.
      CALL LCMPUT(IPLIB,'ISOTOPESDSB ',NBISO*NGRO,2,SB)
      CALL LCMPUT(IPLIB,'ISOTOPESDSN ',NBISO*NGRO,2,SN)
      RETURN
*
  380 FORMAT(A6,': INVALID VALUE OF NBMIX (',I5,')')
  390 FORMAT(/1X,61('*')/1X,A6,
     >': MAXIMUM NUMBER OF SELF-SHIELDING HITERATIONS EXCEEDED'/
     >1X,61('*')/)
  400 FORMAT(
     > 1X,'SHIBA MULTIDIMENSIONAL SELF-SHIELDING CALCULATION',
     > 1X,'-> A. HEBERT'/
     > 1X,A72/
     > 1X,'COLLISION PROBABILITY MODULE: ',A12/
     > 1X,'USING STAMMLER APPROXIMATION')
  405 FORMAT(1X,' OPTIONS'/1X,8('-')/
     >1X,'IGRMIN',  I8,' (FIRST GROUP TO PROCESS)'/
     >1X,'IGRMAX',  I8,' (MOST THERMAL GROUP TO PROCESS)'/
     >1X,'MAXX0 ',  I8,' (MAXIMUM NUMBER OF ITERATIONS)'/
     >1X,'IBIEFF',  I8,' (=1: LIVOLANT-JEANPIERRE NORMALIZATION)'/
     >1X,'IGC   ',  I8,' (=1: GOLDSTEIN-COHEN APPROXIMATION)'/
     >1X,'ITRANZ',  I8,' (TYPE OF TRANSPORT CORRECTION)'/
     >1X,'EPS   ',1P,E8.1,' (STOPING CRITERION)'/)
  410 FORMAT(1X,A6,/
     >': THERE IS NO REQUEST TO PROCESS ANY RESONANT ISOTOPE'/)
  420 FORMAT(/1X,131('-')//1X,' GROUP = ',I4/)
  430 FORMAT(/1X,'PERFORMING SELF-SHIELDING ITERATION NB.',I5)
  440 FORMAT(/1X,'SELF-SHIELDING ITERATION = ',I4,5X,
     >'NUMBER OF NON CONVERGED GROUPS =',I4,5X,
     >'ERROR =',1P,E13.4,0P,' IN GROUP ',I4)
  450 FORMAT(9(1X,'MASKL(',I3,')=',L1,:))
  460 FORMAT(' MACROSCOPIC DILUTION CROSS SECTIONS: '/(9X,1P,11E11.3))
  470 FORMAT(' MICROSCOPIC DILUTION CROSS SECTIONS: '/(9X,1P,11E11.3))
  475 FORMAT(' LIVOLANT AND JEANPIERRE MICROSCOPIC DILUTION CROSS XS: '/
     >(9X,1P,11E11.3))
  480 FORMAT(' MACROSCOPIC TOTAL CROSS SECTIONS: '/(9X,1P,11E11.3))
  490 FORMAT(/1X,131('-')/)
  500 FORMAT(/1x,'CONVERGENCE REACHED AT SHIBA ITERATION =',I4,5X,
     >'ERROR =',1P,E11.3)
      END
