*DECK SHIJPM
      SUBROUTINE SHIJPM (IPSYS ,IPTRK ,IFTRAK,IPRINT,NBMIX ,NREG  ,
     >                   MAT   ,VOL   ,LEAKSW,NRES  ,IRES  ,SIG0  ,
     >                   SIG1  ,SIG2  ,DIAG  ,PSTAR ,
     >                   SIGT0 ,SIGT1 ,SIGT2 ,MASK2 ,IRES2 )
C
C-----------------------------------------------------------------------
C
C CALCULATION OF THE ESCAPE MATRIX USING JPM SINGLE BLOCK INFORMATION.
C
C INPUT PARAMETERS:
C  IPSYS  : POINTER TO THE PIJ (L_PIJ SIGNATURE).
C  IPTRK  : POINTER TO THE TRACKING. (L_JPM SIGNATURE).
C  IFTRAK : UNIT NUMBER OF THE SEQUENTIAL BINARY TRACKING FILE.
C  IPRINT : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
C  NBMIX  : TOTAL NUMBER OF MIXTURES
C  NREG   : TOTAL NUMBER OF MERGED BLOCKS FOR WHICH SPECIFIC VALUES
C           OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
C           MANY BLOCKS WITH DIFFERENT POSITION IN THE DOMAIN CAN
C           BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
C           OWN THE SAME GENERATING BLOCK. THIS ALLOWS SOME
C           REDUCTION IN CPU TIME AND MEMORY. NREG.LE.NMBLK
C  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
C           DIMENSION MAT(NREG)
C  VOL    : VOLUMES. DIMENSION VOL(NREG)
C  LEAKSW : LEAKSW=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
C           IS PRESENT.
C  NRES   : NUMBER OF RESONANT MIXTURES.
C  IRES   : RESONANT MIXTURE NUMBER ASSIGNED TO EACH MIXTURE.
C           DIMENSION IRES(NBMIX)
C  SIG0   : TOTAL MACROSCOPIC CROSS SECTIONS OF THE RESONANT MATERIALS
C           IN EACH MIXTURE
C           DIMENSION SIG0(NBMIX)
C  SIG1   : TOTAL MACROSCOPIC CROSS SECTIONS OF THE LIGHT MATERIALS IN
C           EACH MIXTURE
C           DIMENSION SIG1(NBMIX)
C  SIG2   : TRANSPORT CORRECTION IN EACH MIXTURE
C           DIMENSION SIG2(NBMIX)
C  DIAG   : IF DIAG=.TRUE., COMPUTE A DIAGONAL APPROXIMATION OF PSTAR.
C  PSTAR  : ESCAPE MATRIX. DIMENSION PSTAR(NRES,NRES)
C  SIGT0  : DIMENSION SIGT0(NREG)
C  SIGT1  : DIMENSION SIGT1(NREG)
C  SIGT2  : DIMENSION SIGT2(NREG)
C  MASK2  : DIMENSION MASK2(NREG)
C  IRES2  : DIMENSION IRES2(NREG)
C
C---------------------------------------------- A. HEBERT 30/05/88 -----
C
      IMPLICIT    NONE
      INTEGER     IOUT,NSTATE
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NSTATE=40,NAMSBR='SHIJPM')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPSYS,IPTRK,IFTRAK,IPRINT,NBMIX,NREG,NRES
      LOGICAL     LEAKSW,DIAG,MASK2(NREG)
      INTEGER     MAT(NREG),IRES(NBMIX),IRES2(NREG)
      REAL        VOL(NREG),SIG0(NBMIX),SIG1(NBMIX),SIG2(NBMIX),
     >            PSTAR(NRES,NRES),SIGT0(NREG),SIGT1(NREG),SIGT2(NREG)
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     ISIG,IPII,IPIS,IPSS,IPSSB,IIGAT,IIGAU,IIGAU2,
     >            IOPTIC,IPST,IVST,IWP,IGAR,ISSSS,
     >            IIFR,IALB,IINU,IMIX,IDVX,IIGE,IISU,ICHO,IMU1,IIMA
C----
C  FUNCTIONS
C----
      INTEGER     LCMIOF
C----
C  LOCAL PARAMETERS
C----
      LOGICAL     MASKK,USEOLD
      CHARACTER   HSMG*131
      INTEGER     ISTATE(NSTATE),ITG,NMBLK,NGEN,IJAT,NPIS,NPSS,
     >            IKG,IKK,IBM,ILCMLN,ILCML2,ILCMTY
*
* RECOVER JPM SPECIFIC PARAMETERS.
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      ITG=ISTATE(7)
      NMBLK=ISTATE(11)
      NGEN=ISTATE(12)
      IJAT=ISTATE(13)
      USEOLD=ISTATE(10).EQ.1
      NPIS=ISTATE(5)
      NPSS=ISTATE(16)
      IIFR=LCMIOF(IPTRK,'IFR')
      IALB=LCMIOF(IPTRK,'ALB')
      IINU=LCMIOF(IPTRK,'INUM')
      IMIX=LCMIOF(IPTRK,'MIX')
      IDVX=LCMIOF(IPTRK,'DVX')
      IIGE=LCMIOF(IPTRK,'IGEN')
      IISU=LCMIOF(IPTRK,'ISURF')
      ICHO=LCMIOF(IPTRK,'CHORD')
      IMU1=LCMIOF(IPTRK,'MU1')
      IIMA=LCMIOF(IPTRK,'IMA')
*
* CHECK FOR CONSISTENCY OF THE GENERATING BLOCK ALLOCATION.
      IF(NGEN .GT. NREG) THEN
        WRITE(HSMG,70) NAMSBR,NGEN
        CALL XABORT(HSMG)
      ENDIF
      IF(IPRINT .GE. 10) THEN
        DO 10 IKG=1,NGEN
          IRES2(IKG)=0
 10     CONTINUE
        DO 20 IKK=1,NREG
          IKG=IBASE(IIGE+IKK-1)
          IF(IRES2(IKG) .NE. 0) THEN
             IF(IRES2(IKG) .NE. MAT(IKK)) THEN
               WRITE(HSMG,80) NAMSBR,IKK,MAT(IKK),IKG
               CALL XABORT(HSMG)
             ENDIF
          ELSE
            IRES2(IKG)=MAT(IKK)
          ENDIF
   20   CONTINUE
      ENDIF
*
* COMPUTE THE INDIVIDUAL GENERATING BLOCK COLLISION PROBABILITIES.
      DO 30 IKK=1,NREG
        IKG=IBASE(IIGE+IKK-1)
        MASK2(IKG)=.TRUE.
        IBM=MAT(IKK)
        IF(IBM .NE. 0) THEN
          SIGT0(IKG)=SIG0(IBM)+SIG1(IBM)-SIG2(IBM)
        ELSE
          SIGT0(IKG)=0.0
        ENDIF
   30 CONTINUE
      CALL SETARA(RBASE,NGEN,ISIG)
      CALL SETARA(RBASE,NGEN,IPII)
      CALL SETARA(RBASE,NPIS,IPIS)
      CALL SETARA(RBASE,NPSS,IPSS)
      CALL LCMLEN(IPSYS,'DRAGONJPMTXS',ILCMLN,ILCMTY)
      CALL LCMLEN(IPSYS,'DRAGONJPMPIS',ILCML2,ILCMTY)
      IF(USEOLD .AND.
     >   ILCMLN .EQ. NGEN .AND.
     >   ILCML2 .EQ. NPIS) THEN
        CALL LCMGET(IPSYS,'DRAGONJPMTXS',RBASE(ISIG))
        MASKK=.TRUE.
        DO 50 IKK=1,NREG
          IKG=IBASE(IIGE+IKK-1)
          MASK2(IKG)=RBASE(ISIG+IKG-1) .NE. SIGT0(IKG)
          MASKK=MASKK .AND. MASK2(IKG)
 50     CONTINUE
        IF(.NOT. MASKK) THEN
          CALL LCMGET(IPSYS,'DRAGONJPMPII',RBASE(IPII))
          CALL LCMGET(IPSYS,'DRAGONJPMPIS',RBASE(IPIS))
          CALL LCMGET(IPSYS,'DRAGONJPMPSS',RBASE(IPSS))
        ENDIF
      ENDIF
      CALL JPMDIS(IPTRK ,ITG   ,IPRINT,NGEN  ,IFTRAK,
     >            IBASE(IISU)  ,RBASE(ICHO)  ,SIGT0 ,MASK2 ,
     >            RBASE(IPII)  ,RBASE(IPIS)  ,RBASE(IPSS)  )
      IF(.NOT. USEOLD .OR.
     >   ILCMLN .NE. NGEN .OR.
     >   ILCML2 .NE. NPIS  ) THEN
        CALL LCMPUT(IPSYS,'DRAGONJPMTXS',NGEN,2,SIGT0)
        CALL LCMPUT(IPSYS,'DRAGONJPMPII',NGEN,2,RBASE(IPII))
        CALL LCMPUT(IPSYS,'DRAGONJPMPIS',NPIS,2,RBASE(IPIS))
        CALL LCMPUT(IPSYS,'DRAGONJPMPSS',NPSS,2,RBASE(IPSS))
      ENDIF
*
* BUILD THE ESCAPE MATRIX.
      DO 60 IKK=1,NREG
        IKG=IBASE(IIGE+IKK-1)
        IBM=MAT(IKK)
        IF(IBM.NE.0) THEN
          SIGT0(IKG)=SIG0(IBM)
          SIGT1(IKG)=SIG1(IBM)
          SIGT2(IKG)=SIG2(IBM)
          IRES2(IKG)=IRES(IBM)
        ELSE
          SIGT0(IKG)=0.0
          SIGT1(IKG)=0.0
          SIGT2(IKG)=0.0
          IRES2(IKG)=0
        ENDIF
   60 CONTINUE
      CALL SETARA(RBASE,IBASE(IIMA+IJAT-1),IPSSB)
      CALL SETARA(IBASE,NREG,IIGAT)
      CALL SETARA(IBASE,NGEN,IIGAU)
      CALL SETARA(IBASE,NGEN,IIGAU2)
      CALL SETARA(RBASE,NGEN,IOPTIC)
      CALL SETARA(RBASE,2*NRES*NRES,IPST)
      CALL SETARA(RBASE,NRES,IVST)
      CALL SETARA(RBASE,IJAT*NRES,IWP)
      CALL SETARA(RBASE,IJAT,IGAR)
      CALL SETARA(RBASE,NRES,ISSSS)
      CALL SHIDAN(IPRINT,DIAG,NMBLK,IBASE(IIFR),RBASE(IALB),NREG,
     >            IJAT,IBASE(IINU),VOL,IBASE(IMIX),RBASE(IDVX),
     >            IBASE(IMU1),IBASE(IIMA),NGEN,IBASE(IIGE),
     >            IBASE(IISU),RBASE(ICHO),SIGT0,SIGT1,SIGT2,
     >            RBASE(IPII),RBASE(IPIS),RBASE(IPSS),NRES,
     >            IRES2,PSTAR,RBASE(IPSSB),IBASE(IIGAT),IBASE(IIGAU),
     >            IBASE(IIGAU2),RBASE(IOPTIC),RBASE(IPST),RBASE(IVST),
     >            RBASE(IWP),RBASE(IGAR),RBASE(ISSSS))
      CALL RLSARA(RBASE(ISSSS))
      CALL RLSARA(RBASE(IGAR))
      CALL RLSARA(RBASE(IWP))
      CALL RLSARA(RBASE(IVST))
      CALL RLSARA(RBASE(IPST))
      CALL RLSARA(RBASE(IOPTIC))
      CALL RLSARA(IBASE(IIGAU2))
      CALL RLSARA(IBASE(IIGAU))
      CALL RLSARA(IBASE(IIGAT))
      CALL RLSARA(RBASE(IPSSB))
*
      CALL RLSARA(RBASE(IPSS))
      CALL RLSARA(RBASE(IPIS))
      CALL RLSARA(RBASE(IPII))
      CALL RLSARA(RBASE(ISIG))
      RETURN
*
   70 FORMAT(A6,': TOO MANY GENERATING BLOCKS (',I5,')')
   80 FORMAT(A6,': BLOCK NUMBER',I5,' MADE OF MIXTURE ',I5,
     >' HAS INVALID GENERATING NUMBER (',I5,')')
      END
