*DECK SHIPIJ
      SUBROUTINE SHIPIJ(IPSYS ,IPTRK ,IFTRAK,CDOOR ,IPRINT,NBMIX ,
     >                  NREG  ,MAT   ,VOL   ,LEAKSW,NRES  ,IRES  ,
     >                  SIG0  ,SIG1  ,SIG2  ,DIAG  ,TITR  ,PSTAR ,
     >                  PSGAR ,PIJ   ,VST   ,SIGT  ,PIJSC )
C
C-----------------------------------------------------------------------
C
C CALCULATION OF THE ESCAPE MATRIX USING STANDARD COLLISION
C PROBABILITIES.
C
C INPUT PARAMETERS:
C  IPSYS  : POINTER TO THE PIJ (L_PIJ SIGNATURE).
C  IPTRK  : POINTER TO THE TRACKING. (L_JPM SIGNATURE).
C  IFTRAK : UNIT NUMBER OF THE SEQUENTIAL BINARY TRACKING FILE.
C  CDOOR  : CHARACTER*12 NAME OF THE GEOMETRY/SOLUTION MODULE.
C  IPRINT : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
C  NBMIX  : NUMBER OF MIXTURES.
C  NREG   : TOTAL NUMBER OF REGIONS.
C  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
C           DIMENSION MAT(NREG)
C  VOL    : VOLUMES. DIMENSION VOL(NREG)
C  LEAKSW : LEAKSW=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
C           IS PRESENT.
C  NRES   : NUMBER OF RESONANT MIXTURES.
C  IRES   : RESONANT MIXTURE NUMBER ASSIGNED TO EACH MIXTURE.
C           DIMENSION IRES(NBMIX)
C  SIG0   : TOTAL MACROSCOPIC CROSS SECTIONS OF THE RESONANT MATERIALS
C           IN EACH MIXTURE. DIMENSION SIG0(NBMIX)
C  SIG1   : TOTAL MACROSCOPIC CROSS SECTIONS OF THE LIGHT MATERIALS IN
C           EACH MIXTURE. DIMENSION SIG1(NBMIX)
C  SIG2   : TRANSPORT CORRECTION IN EACH MIXTURE. DIMENSION SIG2(NBMIX)
C  DIAG   : IF DIAG=.TRUE., COMPUTE A DIAGONAL APPROXIMATION OF PSTAR.
C  TITR   : TITLE
C  PSTAR  : ESCAPE MATRIX. DIMENSION PSTAR(NRES,NRES)
C  PSGAR  : DIMENSION PSGAR(NRES,2*NRES)
C  PIJ    : DIMENSION PIJ(NREG*(NREG+1)/2)
C  VST    : DIMENSION VST(NRES)
C  SIGT   : DIMENSION SIGT(NBMIX+1)
C  PIJSC  : DIMENSION PIJSC(NREG,2*NREG)
C
C---------------------------------------------- A. HEBERT 30/05/88 -----
C
      IMPLICIT    NONE
      INTEGER     IOUT
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NAMSBR='SHIPIJ')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPSYS,IPTRK,IFTRAK,IPRINT,NBMIX,NREG,NRES
      CHARACTER   CDOOR*12,TITR*72
      LOGICAL     LEAKSW,DIAG
      INTEGER     MAT(NREG),IRES(NBMIX)
      REAL        VOL(NREG),SIG0(NBMIX),SIG1(NBMIX),SIG2(NBMIX),
     >            PSTAR(NRES,NRES),PSGAR(NRES,2*NRES),
     >            PIJ(NREG*(NREG+1)/2),VST(NRES),SIGT(0:NBMIX),
     >            PIJSC(NREG,2*NREG)
C----
C  LOCAL PARAMETERS
C----
      LOGICAL     LTRAN
      INTEGER     IPIJK,NELPIJ,KNORM,INDPIJ,IBM,IREG,JREG,
     >            IER,JRES,KRES,IND,ILIMIT,JBM,JND
      REAL        SUM,PIJG
*----
*  Variables to implement modifications in EXCELP
*----
      INTEGER     NSBG,NPST,JGL,NGROUP,IFUITE
      REAL        PROBKS
      INTEGER     NALBP
      REAL        ALBP
C----
C  LOCAL FUNCTION
C----
      INTEGER     INDPOS
      INDPOS(IREG,JREG)=MAX(IREG,JREG)*(MAX(IREG,JREG)-1)/2
     >                 +MIN(IREG,JREG)
*
      IPIJK=1
      NELPIJ=NREG*(NREG+1)/2
      SIGT(0)=0.0
      LTRAN=.FALSE.
      DO 100 IBM=1,NBMIX
        LTRAN=LTRAN .OR. (SIG2(IBM) .NE. 0.0)
        SIGT(IBM)=SIG0(IBM)+SIG1(IBM)-SIG2(IBM)
 100  CONTINUE
      IF(CDOOR .EQ. 'EXCELL') THEN
        NSBG=1
        NPST=1
        JGL=0
        IFUITE=-1
        NGROUP=NSBG
        PROBKS=1.0
        KNORM=0
        NALBP=0
        CALL EXCELP(IPSYS ,IPTRK ,IFTRAK,IPRINT,NREG  ,NBMIX,
     >              MAT   ,VOL   ,KNORM ,NELPIJ,IPIJK ,LEAKSW,
     >              TITR  ,SIGT  ,PIJ   ,NSBG  ,NPST  ,PROBKS,
     >              JGL   ,NGROUP,IFUITE,
     >              NALBP ,ALBP  )
      ELSE IF(CDOOR.EQ.'JPMC') THEN
        CALL JPMP  (IPSYS ,IPTRK ,IFTRAK,IPRINT,NREG  ,NBMIX,
     >              MAT   ,VOL   ,KNORM ,NELPIJ,IPIJK ,LEAKSW,
     >              TITR  ,SIGT  ,PIJ   )
      ELSE IF(CDOOR.EQ.'SYBIL') THEN
        CALL SYBILP(IPSYS ,IPTRK ,IFTRAK,IPRINT,NREG  ,NBMIX,
     >              MAT   ,VOL   ,KNORM ,NELPIJ,IPIJK ,LEAKSW,
     >              TITR  ,SIGT  ,PIJ   )
      ENDIF
* PERFORM TRANSPORT CORRECTION.
      IF(LTRAN) THEN
        DO 110 IREG=1,NREG
          DO 111 JREG=1,NREG
            INDPIJ=INDPOS(IREG,JREG)
            IBM=MAT(JREG)
            IF(IBM .EQ. 0) THEN
              PIJSC(IREG,JREG)=0.0
            ELSE
              PIJSC(IREG,JREG)=SIG2(IBM)*PIJ(INDPIJ)
            ENDIF
            PIJSC(IREG,NREG+JREG)=PIJ(INDPIJ)
 111      CONTINUE
          PIJSC(IREG,IREG)=VOL(IREG)+PIJSC(IREG,IREG)
 110    CONTINUE
        CALL ALSB(NREG,NREG,PIJSC,IER,NREG)
        IF(IER .NE. 0) CALL XABORT(NAMSBR//': SINGULAR MATRIX.')
        DO 120 IREG=1,NREG
          DO 121 JREG=IREG,NREG
            INDPIJ=INDPOS(IREG,JREG)
            PIJ(INDPIJ)=PIJSC(IREG,NREG+JREG)*VOL(IREG)
 121      CONTINUE
 120    CONTINUE
      ENDIF
*
      DO 130 JRES=1,NRES
        VST(JRES)=0.0
        DO 131 KRES=1,2*NRES
          PSGAR(JRES,KRES)=0.0
 131    CONTINUE
        PSGAR(JRES,NRES+JRES)=1.0
 130  CONTINUE
      DO 140 IREG=1,NREG
        IF(MAT(IREG) .GT. 0) THEN
          IND=IRES(MAT(IREG))
          IF(IND .GT. 0) THEN
            VST(IND)=VST(IND)+VOL(IREG)
          ENDIF
        ENDIF
 140  CONTINUE
*
      ILIMIT=0
      DO 150 IBM=1,NBMIX
        IF(SIG0(IBM) .GE. 1.0E10) THEN
          ILIMIT=ILIMIT+1
          IND=IBM
        ENDIF
 150  CONTINUE
      IF(ILIMIT .EQ. 0) THEN
        DO 160 IREG=1,NREG
          IF(MAT(IREG) .NE. 0) THEN
            IND=IRES(MAT(IREG))
            IF(IND .NE. 0) THEN
              DO 161 JREG=1,NREG
                JBM=MAT(JREG)
                IF(JBM .NE. 0) THEN
                  JND=IRES(JBM)
                  IF(JND .NE. 0) THEN
                    IF(.NOT.DIAG .OR. JND .EQ. IND) THEN
                      INDPIJ=INDPOS(IREG,JREG)
                      PIJG=PIJ(INDPIJ)/VOL(IREG)
                      PSGAR(IND,JND)=PSGAR(IND,JND)
     >                              +PIJG*VOL(IREG)/VST(IND)
                      PSGAR(IND,JND+NRES)=PSGAR(IND,JND+NRES)
     >                             -PIJG*SIG0(JBM)*VOL(IREG)/VST(IND)
                    ENDIF
                  ENDIF
                ENDIF
 161          CONTINUE
            ENDIF
          ENDIF
 160    CONTINUE
        IF(NRES .EQ. 1) THEN
          PSTAR(1,1)=PSGAR(1,2)/PSGAR(1,1)
        ELSE
          CALL ALSB(NRES,NRES,PSGAR,IER,NRES)
          IF(IER .NE. 0) CALL XABORT(NAMSBR//
     >    ': UNABLE TO SOLVE THE LINEAR SYSTEM.')
          DO 170 JRES=1,NRES
            DO 171 KRES=1,NRES
              PSTAR(JRES,KRES)=PSGAR(JRES,NRES+KRES)
 171        CONTINUE
 170      CONTINUE
        ENDIF
      ELSE IF(ILIMIT .EQ. 1) THEN
*        COMPUTE PSTAR DIRECTLY FROM THE FUEL TO NON-FUEL CP.
        IF(NRES .GT. 1) CALL XABORT(NAMSBR//
     >  ': THE NUMBER OF RESONANT REGIONS IS GREATER THAN ONE.')
        SUM=0.0
        DO 90 IREG=1,NREG
          IBM=MAT(IREG)
          IF(IBM .NE. 0) THEN
            IF(IRES(IBM) .EQ. 0) THEN
              DO 80 JREG=1,NREG
                INDPIJ=INDPOS(IREG,JREG)
                JBM=MAT(JREG)
                IF(JBM .NE. 0) THEN
                  IF(IRES(JBM) .NE. 0) THEN
                    SUM=SUM+(SIG1(IBM)-SIG2(IBM))*PIJ(INDPIJ)
                  ENDIF
                ENDIF
   80         CONTINUE
            ENDIF
          ENDIF
   90   CONTINUE
        PSTAR(1,1)=SUM*SIGT(IND)/VST(1)
      ELSE
        CALL XABORT(NAMSBR//
     >  ': MORE THAN ONE MIXTURE HAS AN INFINITE TOTAL CROSS SECTION.')
      ENDIF
      RETURN
      END
