*DECK SHIPST
      SUBROUTINE SHIPST(IPSYS ,IPTRK ,IFTRAK,CDOOR ,IPRINT,NBMIX ,
     >                  NREG  ,MAT   ,VOL   ,LEAKSW,NRES  ,IRES  ,
     >                  SIG0  ,SIG1  ,SIG2  ,DIAG  ,TITR  ,PSTAR )
C
C-----------------------------------------------------------------------
C
C CALCULATION OF THE ESCAPE MATRIX FOR SHIBA.
C
C INPUT PARAMETERS:
C  IPSYS  : POINTER TO THE PIJ (L_PIJ SIGNATURE).
C  IPTRK  : POINTER TO THE TRACKING. (L_JPM, L_SYBIL OR L_EXCELL
C           SIGNATURE).
C  IFTRAK : UNIT NUMBER OF THE SEQUENTIAL BINARY TRACKING FILE.
C  CDOOR  : CHARACTER*12 NAME OF THE GEOMETRY/SOLUTION MODULE.
C           IF CDOOR='JPM' SHIPST WILL PERFORM A 'FAST' ESCAPE MATRIX
C           CALCULATION.
C  IPRINT : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
C  NBMIX  : NUMBER OF MIXTURES.
C  NREG   : TOTAL NUMBER OF MERGED BLOCKS FOR WHICH SPECIFIC VALUES
C           OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
C  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
C           DIMENSION MAT(NREG)
C  VOL    : VOLUMES. DIMENSION VOL(NREG)
C  LEAKSW : LEAKSW=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
C           IS PRESENT.
C  NRES   : NUMBER OF RESONANT MIXTURES.
C  IRES   : RESONANT MIXTURE NUMBER ASSIGNED TO EACH MIXTURE.
C           DIMENSION IRES(NBMIX)
C  SIG0   : TOTAL MACROSCOPIC CROSS SECTIONS OF THE RESONANT  MATERIALS
C           IN EACH MIXTURE. DIMENSION SIG0(NBMIX)
C  SIG1   : TOTAL MACROSCOPIC CROSS SECTIONS OF THE LIGHT MATERIALS IN
C           EACH MIXTURE. DIMENSION SIG1(NBMIX)
C  SIG2   : TRANSPORT CORRECTION IN EACH MIXTURE. DIMENSION SIG2(NBMIX)
C  DIAG   : IF DIAG=.TRUE., COMPUTE A DIAGONAL APPROXIMATION OF PSTAR.
C  TITR   : TITLE
C
C OUTPUT PARAMETER:
C  PSTAR  : ESCAPE MATRIX. DIMENSION PSTAR(NRES,NRES)
C
C---------------------------------------------- A. HEBERT 30/05/88 -----
C
      IMPLICIT    NONE
      INTEGER     IOUT
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NAMSBR='SHIPST')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPSYS,IPTRK,IFTRAK,IPRINT,NBMIX,NREG,NRES
      CHARACTER   CDOOR*12,TITR*72
      LOGICAL     LEAKSW,DIAG
      INTEGER     MAT(NREG),IRES(NBMIX)
      REAL        VOL(NREG),SIG0(NBMIX),SIG1(NBMIX),SIG2(NBMIX),
     >            PSTAR(NRES,NRES)
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     ISIGT0,ISIGT1,ISIGT2,IMASK2,IIRES2,
     >            IPSGAR,IPIJ,IVST,ISIGT,IPIJSC
C----
C  LOCAL PARAMETERS
C----
      INTEGER    IPRNT2,I,J
*
      IPRNT2=MAX(0,IPRINT-10)
      IF(CDOOR .EQ. 'JPM') THEN
        CALL SETARA(RBASE,NREG,ISIGT0)
        CALL SETARA(RBASE,NREG,ISIGT1)
        CALL SETARA(RBASE,NREG,ISIGT2)
        CALL SETARA(IBASE,NREG,IMASK2)
        CALL SETARA(IBASE,NREG,IIRES2)
        CALL SHIJPM(IPSYS ,IPTRK ,IFTRAK,IPRNT2,NBMIX ,NREG  ,
     >              MAT   ,VOL   ,LEAKSW,NRES  ,IRES  ,SIG0  ,
     >              SIG1  ,SIG2  ,DIAG  ,PSTAR ,
     >              RBASE(ISIGT0),RBASE(ISIGT1),RBASE(ISIGT2),
     >              IBASE(IMASK2),IBASE(IIRES2))
        CALL RLSARA(IBASE(IIRES2))
        CALL RLSARA(IBASE(IMASK2))
        CALL RLSARA(RBASE(ISIGT2))
        CALL RLSARA(RBASE(ISIGT1))
        CALL RLSARA(RBASE(ISIGT0))
      ELSE
        CALL SETARA(RBASE,2*NRES*NRES,IPSGAR)
        CALL SETARA(RBASE,NREG*(NREG+1)/2,IPIJ)
        CALL SETARA(RBASE,NRES,IVST)
        CALL SETARA(RBASE,NBMIX+1,ISIGT)
        CALL SETARA(RBASE,2*NREG*NREG,IPIJSC)
        CALL SHIPIJ(IPSYS ,IPTRK ,IFTRAK,CDOOR ,IPRNT2,NBMIX ,
     >              NREG  ,MAT   ,VOL   ,LEAKSW,NRES  ,IRES  ,
     >              SIG0  ,SIG1  ,SIG2  ,DIAG  ,TITR  ,PSTAR ,
     >              RBASE(IPSGAR),RBASE(IPIJ)  ,RBASE(IVST)  ,
     >              RBASE(ISIGT) ,RBASE(IPIJSC))
        CALL RLSARA(RBASE(IPIJSC))
        CALL RLSARA(RBASE(ISIGT))
        CALL RLSARA(RBASE(IVST))
        CALL RLSARA(RBASE(IPIJ))
        CALL RLSARA(RBASE(IPSGAR))
      ENDIF
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,'(24H SHIPST: ESCAPE MATRIX :/)')
        WRITE(IOUT,20) (J,J=1,NRES)
        DO 10 I=1,NRES
          WRITE(IOUT,30) I,(PSTAR(I,J),J=1,NRES)
   10   CONTINUE
        WRITE(IOUT,'(//)')
      ENDIF
      RETURN
   20 FORMAT(6X,11(5X,'J=',I4,:))
   30 FORMAT(' I=',I4,': ',1P,11E11.3/(9X,11E11.3))
      END
