*DECK SHIRAT
      SUBROUTINE SHIRAT(IPRINT,NRAT  ,SIGX  ,DILUT ,IGRP  ,SA    ,
     >                  COEF  ,DENOM )
C
C-----------------------------------------------------------------------
C
C CALCULATION OF THE NRAT-TERMS RATIONAL APPROXIMATION COEFFICIENTS FOR
C THE SIGX-DEPENDANT FUEL-TO-FUEL REDUCED COLLISION PROBABILITY IN A
C CLOSED CELL.
C
C INPUT PARAMETERS:
C  IPRINT : PRINT FLAG. NO PRINT IF IPRINT.LT.10.
C  NRAT   : NUMBER OF TERMS IN THE PIJ RATIONAL APPROXIMATION.
C  SIGX   : INTERPOLATION VALUES FOR THE RESONANT CROSS SECTION OF
C           THE HEAVY NUCLIDE. DIMENSION SIGX(2*NRAT-1)
C  DILUT  : INTERPOLATED MACROSCOPIC ESCAPE CROSS SECTIONS CORRESPON-
C           DING TO SIGX VALUES. DIMENSION DILUT(2*NRAT-1)
C  IGRP   : GROUP INDEX.
C
C OUTPUT PARAMETERS:
C  SA     : ASYMPTOTIC MACROSCOPIC ESCAPE CROSS SECTION.
C  COEF   : NUMERATOR COEFFICIENTS FOR THE RATIONAL APPROXIMATION
C           OF FUEL-TO-FUEL REDUCED COLLISION PROBABILITY.
C  DENOM  : DENOMINATOR COEFFICIENTS FOR THE RATIONAL APPROXIMATION
C           OF FUEL-TO-FUEL REDUCED COLLISION PROBABILITY.
C           COMPLEX COEF(NRAT),DENOM(NRAT)
C
C---------------------------------------------- A. HEBERT 10/10/89 -----
C  MODIFICATIONS:  A. HEBERT 98/04/24 + G. MARLEAU
C    PASS GROUP NUMBER TO ROUTINE
C    REPLACE STANDARD PADE APPROXIMATION  (CALL ALPADE)
C    BY LEAST SQUARE PADE APPROXIMATION   (CALL ALPLSF)
C
C
      IMPLICIT         NONE
      INTEGER          IOUT,NMAX,NORIN
      DOUBLE PRECISION EPSRID
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NMAX=11,NORIN=(NMAX-1)/2,EPSRID=1.0D-5,
     >                 NAMSBR='SHIRAT')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER          IPRINT,NRAT,IGRP
      REAL             SIGX(2*NRAT-1),DILUT(2*NRAT-1),SA
      COMPLEX          COEF(NRAT),DENOM(NRAT)
C----
C  LOCAL PARAMETERS
C----
      INTEGER          I,J,NOR,NORP1,NALPHA
      COMPLEX          CTEMP,EAV
      DOUBLE PRECISION A(0:NORIN),B(0:NORIN),C(0:NORIN+1),
     >                 ROOTS(2,NORIN+1),PREC,SGN,X,Z1,Z2
      REAL             EPS
*
      IF(2*NRAT-1 .GT. NMAX) CALL XABORT(NAMSBR//': INCREASE NMAX.')
      DO 10 I=2,NRAT
        COEF(I)=0.0
        DENOM(I)=1.0
   10 CONTINUE
      EPS=0.0
      DO 15 I=1,2*NRAT-1,2
        EPS=MAX(EPS,ABS(DILUT(I)-DILUT(NRAT))/ABS(DILUT(NRAT)))
   15 CONTINUE
      IF(EPS .LT. 5.0E-4) THEN
        SA=DILUT(NRAT)
        COEF(1)=1.0
        DENOM(1)=DILUT(NRAT)
        IF(IPRINT .GE. 10) WRITE(IOUT,6110) SA
        RETURN
      ENDIF
C----
C  OLD PADE APPROXIMATION
C      CALL ALPADE(NRAT-1,SIGX,DILUT,EPSRID,NOR,A,B,IERR)
C  NEW PADE APPROXIMATION
C     CALL ALPLSF(NRAT-1,SIGX,DILUT,EPSRID,.FALSE.,NOR,A,B,PREC)
C----
      CALL ALPLSF(NRAT-1,SIGX,DILUT,EPSRID,.FALSE.,NOR,A,B,PREC)
      SA=A(NOR)
      NORP1=NOR+1
      SGN=1.0D0
      C(0)=A(0)
      DO 100 I=1,NOR
        SGN=-SGN
        C(I)=SGN*(B(I-1)+A(I))
 100  CONTINUE
      C(NORP1)=-SGN
      IF(NOR .LT. 0) THEN
        CALL XABORT(NAMSBR//': PADE COLLOCATION FAILURE.')
      ENDIF
      CALL ALROOT(C,NORP1,ROOTS)
      CALL ALPOLY(B,NOR,ROOTS,DENOM,COEF)
      CTEMP=DENOM(1)
      DO 110 I=2,NORP1
        CTEMP=CTEMP*DENOM(I)
 110  CONTINUE
      IF(ABS(REAL(CTEMP)-REAL(A(0))) .GT. 1.0E-3*ABS(REAL(A(0)))) THEN
        WRITE(IOUT,9100) REAL(CTEMP)
        WRITE(IOUT,9101) (A(I),I=0,NOR)
        WRITE(IOUT,9102)
        CALL XABORT(NAMSBR//': INTERPOLATION ALGORITHM FAILURE 1')
      ENDIF
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6080) IGRP,(COEF(I),I=1,NOR+1)
        WRITE(IOUT,6090) (DENOM(I),I=1,NOR+1)
        WRITE(IOUT,6100)
        X=1.0D0
        DO 70 I=1,2*NRAT-1
          Z1=0.0D0
          Z2=0.0D0
          DO 60 J=0,NOR
            Z1=Z1+A(J)*X
            Z2=Z2+B(J)*X
            X=X*SIGX(I)
   60     CONTINUE
          WRITE(IOUT,'(1X,I5,1P,3E13.5)') I,SIGX(I),DILUT(I),Z1/Z2
   70   CONTINUE
        WRITE(IOUT,6110) SA
      ENDIF
      EAV=0.0
      DO 75 I=1,NRAT
        EAV=EAV+COEF(I)*SQRT(DENOM(I))
   75 CONTINUE
      EAV=EAV*EAV
      IF(REAL(EAV) .LT. 0.0) THEN
        NALPHA=2*NRAT-1
        WRITE(IOUT,9200) EAV,IGRP
        WRITE(IOUT,6120) (SIGX(I),I=1,NALPHA)
        WRITE(IOUT,6130) (DILUT(I),I=1,NALPHA)
        WRITE(IOUT,6080) IGRP,(COEF(I),I=1,NRAT)
        WRITE(IOUT,6090) (DENOM(I),I=1,NRAT)
        CALL XABORT(NAMSBR//': RATIONAL EXPANSION FAILURE')
      ELSE IF(ABS(AIMAG(EAV)) .GT. 5.0E-3*REAL(EAV)) THEN
        NALPHA=2*NRAT-1
        WRITE(IOUT,9000) EAV,IGRP
        WRITE(IOUT,6120) (SIGX(I),I=1,NALPHA)
        WRITE(IOUT,6130) (DILUT(I),I=1,NALPHA)
        WRITE(IOUT,6080) IGRP,(COEF(I),I=1,NRAT)
        WRITE(IOUT,6090) (DENOM(I),I=1,NRAT)
      ENDIF
      RETURN
*
 6080 FORMAT(//' RATIONAL APPROXIMATION COEFFICIENTS FOR FUEL-TO-FUEL',
     >         ' REDUCED COLLISION PROBABILITIES OF THE CLOSED CELL',
     >         ' (GROUP = ',I5,')'//
     >  1P,  9X,'NUMERATOR '  ,3(2H (,E15.7,1H,,E15.7,1H),:)/
     >      19X               ,3(2H (,E15.7,1H,,E15.7,1H),:))
 6090 FORMAT(7X,'DENOMINATOR ',3(2H (,E15.7,1H,,E15.7,1H),:)/
     >      19X               ,3(2H (,E15.7,1H,,E15.7,1H),:))
 6100 FORMAT(/5X,'I',9X,'SIGX',8X,'DILUT',10X,'FIT')
 6110 FORMAT(11X,'INFINITE',   1P,E15.7)
 6120 FORMAT(//'  SIGX:',1P,7E15.7)
 6130 FORMAT(  ' DILUT:',7E15.7/)
 9000 FORMAT(' SHIRAT: *** WARNING ***'/
     >       ' EAV HAS LARGE IMAGINARY PART IN RATIONAL EXPANSION'/
     >       ' EAV = ',2E15.7,5X,'IN GROUP = ',I10/
     >       ' ***********************')
 9100 FORMAT(' SHIRAT: *** ERROR ***'/
     >       ' PRODUCT OF ROOTS = ',1P,E15.7/
     >       ' POLYNOMIAL COEFFICIENTS FOLLOWS : ')
 9101 FORMAT(1P,4E15.7)
 9102 FORMAT(' ***********************')
 9200 FORMAT(' SHIRAT: *** ERROR ***'/
     >       ' RATIONAL EXPANSION FAILURE '/
     >       ' EAV HAS NEGATIVE REAL PART '/
     >       ' EAV = ',2E17.7,5X,'IN GROUP = ',I10/
     >       ' ***********************')
      END
