*DECK SLAB
      SUBROUTINE SLAB (IAPP,XX,SIGT,TRONC,PVV,PVS,PSS)
*
*-----------------------------------------------------------------------
*
* INTEGRATION OF THE COLLISION PROBABILITIES OF AN HOMOGENEOUS 1-D
* SLAB (DP-0 OR DP-1 SURFACE ANGULAR FLUX APPROXIMATION).
*
* INPUT PARAMETERS.
*   IAPP    : =1 : DP-0   ;   =3 : DP-1 .
*   XX      : THICKNESS OF THE SLAB.
*   SIGT    : CROSS SECTION.
*   TRONC   : VOIDED CRITERIA.
*
* OUTPUT PARAMETERS.
*   PVV     : VOLUME TO VOLUME REDUCED PROBABILITY.
*   PVS     : VOLUME TO SURFACE PROBABILITIES.
*             DIMENSION PVS(2) FOR DP-0 APPROXIMATION
*             DIMENSION PVS(4) FOR DP-1 APPROXIMATION
*   PSS     : SURFACE TO SURFACE PROBABILITIES IF THE FOLLOWING ORDER:
*             DP-0 APPROXIMATION:
*                1 TO 1, 2 TO 1, 1 TO 2, 2 TO 2.
*                WHERE 1 IS THE LEFT SURFACE;
*                      2 IS THE RIGHT SURFACE.
*                DIMENSION PSS(4).
*             DP-1 APPROXIMATION:
*                1 TO 1, 2 TO 1, 3 TO 1, 4 TO 1, 1 TO 2, 2 TO 2, ETC.
*                WHERE 1 IS THE DP-0 COMPONENT OF THE LEFT SURFACE;
*                      2 IS THE DP-1 COMPONENT OF THE LEFT SURFACE;
*                      3 IS THE DP-0 COMPONENT OF THE RIGHT SURFACE;
*                      4 IS THE DP-1 COMPONENT OF THE RIGHT SURFACE.
*                DIMENSION PSS(16).
*
*---------------------------------------------- A. HEBERT 28/05/87 -----
*
      DOUBLE PRECISION GAMMA,CHO,ZLOG,SQ
      PARAMETER (GAMMA=0.57721566490153286D0)
      DIMENSION PVS(*),PSS(*)
*
      CHO=SIGT*XX
      IF (IAPP.EQ.1) THEN
         PSS(1)=0.0
         PSS(4)=0.0
         IF (2.0D0*CHO.GT.TRONC) THEN
            PSS(2)=2.0*TABEN(3,CHO)
            PVS(1)=(1.0-PSS(2))/(4.0*CHO)
            PVV=(1.0-2.0*PVS(1))/SIGT
         ELSE
            ZLOG=-1.0D10
            IF (CHO.GT.0.0D0) ZLOG=LOG(CHO)+GAMMA
            PSS(2)=1.0D0+CHO*(-2.0D0+CHO*(1.5D0-ZLOG))
            PVS(1)=0.5D0+CHO*(-0.375D0+0.25D0*ZLOG)
            PVV=0.5D0*XX*(1.5D0-ZLOG)
         ENDIF
         PSS(3)=PSS(2)
         PVS(2)=PVS(1)
      ELSE IF (IAPP.EQ.3) THEN
         DO 20 I=1,16
   20    PSS(I)=0.0
         DO 30 I=1,4
   30    PVS(I)=0.0
         SQ=SQRT(2.0D0)
         IF (2.0D0*CHO.GT.TRONC) THEN
            GAR=3.0*TABEN(4,CHO)
            PSS(3)=2.0*TABEN(3,CHO)
            PSS(4)=2.0*SQ*(GAR-PSS(3))
            PSS(8)=4.0*(9.0*TABEN(5,CHO)-4.0*GAR+2.0*PSS(3))
            PVS(1)=(1.0-PSS(3))/(4.0*CHO)
            PVS(2)=-PSS(4)/(4.0*CHO)
            PVV=(1.0-2.0*PVS(1))/SIGT
         ELSE
            ZLOG=-1.0D10
            IF (CHO.GT.0.0D0) ZLOG=LOG(CHO)+GAMMA
            PSS(3)=1.0D0+CHO*(-2.0D0+CHO*(1.5D0-ZLOG))
            PSS(4)=SQ*CHO*(1.0D0+2.0D0*CHO*ZLOG)
            PSS(8)=1.0D0+CHO*(-4.0D0+CHO*(-3.0D0-8.0D0*ZLOG))
            PVS(1)=0.5D0+CHO*(-0.375D0+0.25D0*ZLOG)
            PVS(2)=-0.5D0*(1.0D0+2.0D0*CHO*ZLOG)/SQ
            PVV=0.5D0*XX*(1.5D0-ZLOG)
         ENDIF
         PSS(7)=PSS(4)
         PSS(9)=PSS(3)
         PSS(10)=PSS(4)
         PSS(13)=PSS(4)
         PSS(14)=PSS(8)
         PVS(3)=PVS(1)
         PVS(4)=PVS(2)
      ENDIF
      RETURN
      END
