*DECK SPHDRV
      SUBROUTINE SPHDRV(IPEDIT,IPGEO1,IPGEO2,IPFLUX,IPMACR,IPSYS ,
     >                  MAXPTS,CDOOR ,NDOOR ,IPRINT,ITRANC,NGROUP,
     >                  NGCOND,NBMIX ,NREGIO,NMERGE,NIFISS,ILK    ,
     >                  MATCOD,VOLUME,KEYFLX,IGCOND,IMERGE,FLUXES,
     >                  NSPH  ,KSPH  ,CURNAM,CTITRE,IPTRK2,IFTRK2,
     >                  SPH   )
*
*-----------------------------------------------------------------------
*
*  DRIVER FOR THE CALCULATION OF THE SPH FACTORS. THESE FACTORS ARE
*  USED TO MULTIPLY THE CROSS SECTIONS AND TO DIVIDE THE FLUXES. THE
*  SPH FACTORS CALCULATION IS GENERALLY APPLICATION DEPENDENT. NEW
*  ALGORITHMS SHOULD BE IMPLEMENTED IN THIS DRIVER.
*
* INPUT PARAMETERS:
*   IPEDIT : POINTER TO EDIT INFORMATION (L_EDIT SIGNATURE).
*   IPGEO1 : POINTER TO THE ORIGINAL GEOMETRY (L_GEOM SIGNATURE).
*   IPGEO2 : POINTER TO THE MACRO-GEOMETRY (L_GEOM SIGNATURE). EQUAL TO
*            ZERO IF THE MACRO-GEOMETRY IS NOT YET DEFINED.
*   IPFLUX : POINTER TO THE REFERENCE SOLUTION (L_FLUX SIGNATURE).
*   IPMACR : POINTER TO THE REFERENCE MACROLIB (L_MACROLIB SIGNATURE).
*   IPSYS  : POINTER TO THE REFERENCE PIJ (L_PIJ SIGNATURE).
*   MAXPTS : MAXIMUM NUMBER OF MACRO-REGIONS.
*   CDOOR  : TRACKING MODULE PREVIOUSLY USED IN ASM:.
*   NDOOR  : TRACKING MODULE USED FOR THE MACRO-CALCULATION.
*   IPRINT : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*   ITRANC : TYPE OF TRANSPORT CORRECTION.
*   NGROUP : NUMBER OF GROUPS.
*   NGCOND : NUMBER OF GROUPS CONDENSED.
*   NBMIX  : NUMBER OF MIXTURES.
*   NREGIO : NUMBER OF REGIONS.
*   NMERGE : NUMBER OF REGIONS MERGED.
*   NIFISS : NUMBER OF FISSILE ISOTOPES.
*   ILK    : ILK=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
*            IS PRESENT.
*   MATCOD : MIXTURE INDEX PER REGION. DIMENSION MATCOD(NREGIO)
*   VOLUME : VOLUME OF REGIONS. DIMENSION VOLUME(NREGIO)
*   KEYFLX : POSITION OF AVERAGE FLUXES. DIMENSION KEYFLX(NREGIO)
*   IGCOND : LIMIT CONDENSED GROUPS. DIMENSION IGCOND(NGCOND)
*   IMERGE : POSITION OF MERGED REGION. DIMENSION IMERGE(NREGIO)
*   FLUXES : FLUXES. DIMENSION FLUXES(NREGIO,NGROUP)
*   NSPH   : =1 THE SPH FACTORS ARE READ FROM LCM.
*            =2 HOMOGENEOUS MACRO-CALCULATION (NON ITERATIVE PROCEDURE
*               OR HEBERT-BENOIST SPH-5 PROCEDURE).
*            =3 ANY TYPE OF PIJ MACRO-CALCULATION WITH GEOMETRY
*               PROVIDED AS INPUT.
*            =4 ANY TYPE OF DIFFUSION MACRO-CALCULATION.
*            =5 ANY TYPE OF PIJ MACRO-CALCULATION WITH TRACKING
*               PROVIDED AS INPUT.
*   KSPH   : =1 AVERAGE FLUX NORMALIZATION.
*            =2 SELENGUT NORMALIZATION.
*            =3 SELENGUT NORMALIZATION WITH SURFACE LEAKAGE.
*   CURNAM : CHARACTER*12 NAME OF THE LCM DIRECTORY WHERE THE SPH
*            FACTORS ARE STORED.
*   CTITRE : TITLE.
*   IPTRK2 : Tracking data structure for SPH geometry.
*   IFTRK2 : Tracking file for SPH geometry.
*
* OUTPUT PARAMETERS:
*   SPH    : SPH HOMOGENIZATION FACTORS. DIMENSION SPH(NMERGE,NGCOND)
*
*------------------------------------- AUTHOR: A. HEBERT ; 09/11/88 ----
*
      IMPLICIT    NONE
      INTEGER     IOUT,NSTATE,ILCMUP,ILCMDN,MAXT
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NSTATE=40,ILCMUP=1,ILCMDN=2,
     >            MAXT=3,NAMSBR='SPHDRV')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER   IPEDIT,IPGEO1,IPGEO2,IPFLUX,IPMACR,IPSYS,
     >          MAXPTS,IPRINT,ITRANC,NGROUP,NGCOND,NBMIX,NREGIO,
     >          NMERGE,NIFISS,NSPH,KSPH
      INTEGER   IPTRK2,IFTRK2
      LOGICAL   ILK
      CHARACTER CDOOR*12,NDOOR*12,CURNAM*12,CTITRE*72
      INTEGER   MATCOD(NREGIO),KEYFLX(NREGIO),IGCOND(NGROUP),
     >          IMERGE(NREGIO)
      REAL      VOLUME(NREGIO),FLUXES(NREGIO,NGROUP),
     >          SPH(NMERGE,NGCOND)
C----
C  MEMORY ALLOCATION PARAMETERS
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     IVOL2,IMAT2,IKEY2,IMERG2,IXX,IYY,IINUM,IIGEN,
     >            IIORI,IIMAT,INMC,ISIGMA,ISIGMS,ICOURI,IVOLME,
     >            ISIGT,IFLXME,ISUNME,IFUNKN,IFACTO,ISOURC,ICURH
C----
C  INPUT PARAMETERS
C----
      INTEGER     ITYPLU,INTLIR
      CHARACTER   CARLIR*12
      REAL        REALIR
      DOUBLE PRECISION DBLLIR
C----
C  FUNCTIONS
C----
      INTEGER     KDRCLS,KDROPN
C----
C  LOCAL PARAMETERS
C----
      LOGICAL     LNEWDR,LNEWGE,LCM
      CHARACTER   GEONAM*12,TEXT12*12,MACGEO*12,NAMT*12,
     >            HENTRY(MAXT)*12
      INTEGER     ISTATE(NSTATE),KCHAR(18),IENTRY(MAXT),
     >            JENTRY(MAXT),KENTRY(MAXT),ITC,ILCMLN,ILCMTY,
     >            IFTRAK,IER,ISIGW,IDIFF,NUN2,NBMIX2,NREG2,IREG,
     >            NENTRY,IMACG,MAXMER,MAXGEO
*
* TRACKING OF THE MACRO-GEOMETRY.
      IF(NSPH .NE. 5) THEN 
        IF(IPGEO1.EQ.0) CALL XABORT(NAMSBR//
     >  ': NO REFERENCE GEOMETRY AVAILABLE.')
        NAMT=' '
        CALL LCMNXT(IPGEO1,NAMT,GEONAM,TEXT12,LCM)
        IF(IPRINT.GT.0) WRITE(IOUT,100) NAMSBR,GEONAM,CDOOR
        CALL LCMGET(IPGEO1,'SIGNATURE',KCHAR)
        WRITE(TEXT12,'(3A4)') (KCHAR(ITC),ITC=1,3)
        IF(TEXT12.NE.'L_GEOM') THEN
           CALL XABORT(NAMSBR//'
     >    : SIGNATURE OF '//GEONAM//' IS '//TEXT12//' L_GEOM EXPECTED')
        ENDIF
        CALL LCMGET(IPGEO1,'STATE-VECTOR',ISTATE)
        MAXGEO=MAX(MAXPTS,ISTATE(6))
      ELSE
        CALL LCMGET(IPTRK2,'SIGNATURE',KCHAR)
        WRITE(TEXT12,'(3A4)') (KCHAR(ITC),ITC=1,3)
        IF(TEXT12.NE.'L_TRACK') THEN
           CALL XABORT(NAMSBR//
     >     ': Second tracking data structure expected')
        ENDIF
        IFTRAK=IFTRK2
        CALL LCMGET(IPTRK2,'STATE-VECTOR',ISTATE)
        MAXGEO=MAX(MAXPTS,ISTATE(1))
        NREG2=ISTATE(1)
        NUN2=ISTATE(2)
        ILK=ISTATE(3).EQ.0
      ENDIF
      LNEWDR=.FALSE.
      LNEWGE=.FALSE.
      IF((NSPH.EQ.1).OR.(NSPH.EQ.2)) THEN
*        DO-IT-YOURSELF EQUIVALENCE OR HOMOGENEOUS MACRO-CALCULATION.
         IF((NSPH.EQ.2).AND.(NMERGE.NE.1)) CALL XABORT(NAMSBR//
     >   ': NMERGE SHOULD BE EQUAL TO 1')
         NDOOR=CDOOR
         NREG2=NREGIO
         NUN2=NREGIO
         CALL SETARA(RBASE,NREG2,IVOL2)
         CALL SETARA(IBASE,NREG2,IMAT2)
         CALL SETARA(IBASE,NREG2,IKEY2)
         CALL SETARA(IBASE,NREG2,IMERG2)
         NBMIX2=0
         DO 10 IREG=1,NREG2
         RBASE(IVOL2+IREG-1)=VOLUME(IREG)
         IBASE(IMAT2+IREG-1)=MATCOD(IREG)
         NBMIX2=MAX(NBMIX2,IBASE(IMAT2+IREG-1))
         IBASE(IKEY2+IREG-1)=IREG
   10    IBASE(IMERG2+IREG-1)=1
         IF(KSPH.GT.1) ILK=.TRUE.
      ELSE IF((NSPH.EQ.3).OR.(NSPH.EQ.4)) THEN
*        ANY TYPE OF MACRO-CALCULATION.
         LNEWDR=.TRUE.
         IF(IPGEO2.EQ.0) THEN
*           MAKE THE MACRO-GEOMETRY 'SPH-GEO' FROM GEOMETRY GEONAM.
            LNEWGE=.TRUE.
            MACGEO='SPH-GEO'
            IF(IPRINT.GT.0) WRITE(IOUT,110) NAMSBR,MACGEO,NDOOR
            CALL LCMOP(IPGEO2,'SPH-GEO',0,1,0)
            IMACG=1
            MAXMER=MIN(NMERGE,MAXGEO)
            CALL SETARA(RBASE,MAXGEO,IXX)
            CALL SETARA(RBASE,MAXGEO,IYY)
            CALL SETARA(RBASE,2*MAXGEO,IINUM)
            CALL SETARA(RBASE,MAXGEO,IIGEN)
            CALL SETARA(RBASE,2*MAXGEO,IIORI)
            CALL SETARA(RBASE,2*MAXGEO,IIMAT)
            CALL SETARA(RBASE,MAXGEO+1,INMC)
            CALL SPHGEO(MAXGEO,MAXMER,IPGEO1,IPGEO2,IPRINT,NREGIO,
     1      NMERGE,IMERGE,RBASE(IXX),RBASE(IYY),RBASE(IINUM),
     2      RBASE(IIGEN),RBASE(IIORI),RBASE(IIMAT),RBASE(INMC))
            CALL RLSARA(RBASE(INMC))
            CALL RLSARA(RBASE(IIMAT))
            CALL RLSARA(RBASE(IIORI))
            CALL RLSARA(RBASE(IIGEN))
            CALL RLSARA(RBASE(IINUM))
            CALL RLSARA(RBASE(IYY))
            CALL RLSARA(RBASE(IXX))
         ELSE
*           USE AN EXISTING MACRO-GEOMETRY NAMED MACGEO.
            NAMT=' '
            CALL LCMNXT(IPGEO2,NAMT,MACGEO,TEXT12,LCM)
            IMACG=2
            IF(LCM) IMACG=1
            IF(IPRINT.GT.0) WRITE(IOUT,120) NAMSBR,MACGEO,NDOOR
            CALL LCMGET(IPGEO2,'SIGNATURE',KCHAR)
            WRITE(TEXT12,'(3A4)') (KCHAR(ITC),ITC=1,3)
            IF(TEXT12.NE.'L_GEOM') THEN
               CALL XABORT(NAMSBR//
     >         ': SIGNATURE OF '//MACGEO//' IS '//
     1         TEXT12//'. L_GEOM EXPECTED')
            ENDIF
            CALL XDRSET(ISTATE,NSTATE,0)
            CALL LCMGET(IPGEO2,'STATE-VECTOR',ISTATE)
            IF(ISTATE(7) .NE. NMERGE) CALL XABORT(NAMSBR//
     >      ': MACRO-GEOMETRY '//MACGEO//
     >      ' NOT COMPATIBLE WITH USER-SUPPLIED MERGE INDICES.')
         ENDIF
         CALL XDRSET(SPH,NMERGE*NGCOND,1.0)
*
*        BUILD THE CALLING PARAMETERS FOR THE TRACKING MODULE.
         HENTRY(1)='SPH-TRK'
         CALL LCMOP(IPTRK2,HENTRY(1),0,1,0)
         IENTRY(1)=1
         JENTRY(1)=0
         KENTRY(1)=IPTRK2
         NENTRY=2
         IFTRAK=0
         IF((NDOOR.EQ.'EXCELL').OR.(NDOOR.EQ.'NXT').OR.
     >      (NDOOR.EQ.'JPM')) THEN
            IFTRAK=KDROPN('SPH-FIL',0,2,0,0)
            HENTRY(2)='SPH-FIL'
            IENTRY(2)=3
            JENTRY(2)=0
            KENTRY(2)=IFTRAK
            NENTRY=3
         ENDIF
         HENTRY(NENTRY)=MACGEO
         IENTRY(NENTRY)=IMACG
         JENTRY(NENTRY)=2
         KENTRY(NENTRY)=IPGEO2
         CALL LCMCL(IPGEO2,1)
         CALL LCMOP(IPGEO2,MACGEO,2,IMACG,0)
*
*        TRACKING OF THE MACRO-GEOMETRY.
         IF(NDOOR.EQ.'EXCELL') THEN
            CALL EXCELT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
         ELSE IF(NDOOR.EQ.'NXT') THEN
            CALL NXT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
         ELSE IF(NDOOR.EQ.'JPM') THEN
            CALL JPMT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
         ELSE IF(NDOOR.EQ.'SYBIL') THEN
            CALL SYBILT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
         ELSE IF(NDOOR.EQ.'BIVAC') THEN
            CALL BIVACT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
         ELSE
            CALL XABORT(NAMSBR//
     >      ': '//NDOOR//' IS AN INVALID TRACKING MODULE NAME.')
         ENDIF
         CALL LCMGET(IPTRK2,'STATE-VECTOR',ISTATE)
         NREG2=ISTATE(1)
         NUN2=ISTATE(2)
         ILK=ISTATE(3).EQ.0
         CALL SETARA(RBASE,NREG2,IVOL2)
         CALL SETARA(IBASE,NREG2,IMAT2)
         CALL SETARA(IBASE,NREG2,IKEY2)
         CALL SETARA(IBASE,NREG2,IMERG2)
         CALL LCMGET(IPTRK2,'VOLUME',RBASE(IVOL2))
         CALL LCMGET(IPTRK2,'MATCOD',IBASE(IMAT2))
         CALL LCMGET(IPTRK2,'KEYFLX',IBASE(IKEY2))
         CALL LCMLEN(IPGEO2,'TITLE',ILCMLN,ILCMTY)
         IF(ILCMLN.GT.0) THEN
            CALL LCMGET(IPGEO2,'TITLE',KCHAR)
            WRITE(CTITRE,'(18A4)') (KCHAR(ITC),ITC=1,18)
         ENDIF
         NBMIX2=0
         DO 20 IREG=1,NREG2
         NBMIX2=MAX(NBMIX2,IBASE(IMAT2+IREG-1))
   20    IBASE(IMERG2+IREG-1)=IBASE(IMAT2+IREG-1)
         IF(NBMIX2.NE.NMERGE) CALL XABORT(NAMSBR//
     >   ': TRACKING FAILURE.')
         CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
         IF(ITYPLU .NE. 3 .OR. CARLIR .NE. ';') CALL XABORT(NAMSBR//
     >    ': ; KEY-WORD MISSING')
      ELSE IF(NSPH .EQ. 5) THEN
         CALL SETARA(RBASE,NREG2,IVOL2)
         CALL SETARA(IBASE,NREG2,IMAT2)
         CALL SETARA(IBASE,NREG2,IKEY2)
         CALL SETARA(IBASE,NREG2,IMERG2)
         CALL LCMGET(IPTRK2,'VOLUME',RBASE(IVOL2))
         CALL LCMGET(IPTRK2,'MATCOD',IBASE(IMAT2))
         CALL LCMGET(IPTRK2,'KEYFLX',IBASE(IKEY2))
         NBMIX2=0
         DO 21 IREG=1,NREG2
         NBMIX2=MAX(NBMIX2,IBASE(IMAT2+IREG-1))
   21    IBASE(IMERG2+IREG-1)=IBASE(IMAT2+IREG-1)
         IF(NBMIX2.NE.NMERGE) CALL XABORT(NAMSBR//
     >   ': TRACKING FAILURE.')
      ENDIF
      IF(KSPH.GT.1) THEN
         IF(.NOT.ILK) CALL XABORT(NAMSBR//
     >': SOME LEAKAGE SHOULD BE PRESENT WITH SELENGUT NORMALIZATION'//
     >' THIS LEAKAGE WILL BE CANCELLED BY THE SPH PROCEDURE')
         ILK=.FALSE.
      ENDIF
      IF((IPRINT.GT.3).AND.ILK) THEN
         WRITE(IOUT,130) NAMSBR
      ELSE IF(IPRINT.GT.3) THEN
         WRITE(IOUT,140) NAMSBR
      ENDIF
      IF(LNEWGE) THEN
*        DESTROY THE MACRO-GEOMETRY.
         CALL LCMCL(IPGEO2,1)
         TEXT12='SPH-GEO'
         CALL LCMOP(IPGEO2,TEXT12,1,1,0)
         CALL LCMCL(IPGEO2,2)
      ENDIF
*
      IF(NSPH.EQ.1) THEN
*       READ SPH FACTORS FROM THE EDITION MACROLIB.
        CALL LCMSIX(IPEDIT,CURNAM,ILCMUP)
        CALL LCMGET(IPEDIT,'STATE-VECTOR',ISTATE)
        CALL LCMSIX(IPEDIT,'MACROLIB',ILCMUP)
        CALL LCMLEN(IPEDIT,'SPH-FACTORS',ILCMLN,ILCMTY)
        IF(ISTATE(7) .GT. 0 .AND. ILCMLN .EQ. NMERGE*NGCOND) THEN
          CALL LCMGET(IPEDIT,'SPH-FACTORS',SPH)
        ELSE
          CALL XDRSET(SPH,NMERGE*NGCOND,1.0)
        ENDIF
        CALL LCMSIX(IPEDIT,'MACROLIB',ILCMDN)
        CALL LCMSIX(IPEDIT,CURNAM,ILCMDN)
      ELSE IF((NSPH.EQ.2).OR.(NSPH.EQ.3).OR.
     >        (NSPH.EQ.4).OR.(NSPH.EQ.5)) THEN
*       GENERAL PROCEDURE FOR COMPUTING THE SPH FACTORS.
        CALL SETARA(RBASE,NBMIX2+1,ISIGMA)
        CALL SETARA(RBASE,NBMIX2+1,ISIGMS)
        CALL SETARA(RBASE,NGCOND,ICOURI)
        CALL SETARA(RBASE,NMERGE,IVOLME)
        CALL SETARA(RBASE,NMERGE*NGCOND,ISIGT)
        CALL SETARA(RBASE,NMERGE*NGCOND,ISIGW)
        CALL SETARA(RBASE,NMERGE*NGCOND,IDIFF)
        CALL SETARA(RBASE,NMERGE*NGCOND,IFLXME)
        CALL SETARA(RBASE,NMERGE*NGCOND**2,ISUNME)
        CALL SETARA(RBASE,NUN2*NGCOND,IFUNKN)
        CALL SETARA(RBASE,NMERGE,IFACTO)
        CALL SETARA(RBASE,NREG2,ISOURC)
        CALL SETARA(RBASE,NGROUP*NREGIO*3,ICURH)
        CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMUP)
        CALL SPHEQU(IPTRK2,IFTRAK,IPFLUX,IPMACR,IPSYS ,NBMIX2,
     >              NDOOR ,NSPH  ,KSPH  ,IPRINT,ITRANC,NGROUP,
     >              NGCOND,NBMIX ,NREGIO,NMERGE,NIFISS,MATCOD,
     >              VOLUME,KEYFLX,IGCOND,IMERGE,FLUXES,NREG2 ,
     >              NUN2  ,ILK   ,CTITRE,SPH   ,
     >              IBASE(IMAT2) ,RBASE(IVOL2) ,IBASE(IKEY2) ,
     >              IBASE(IMERG2),RBASE(ISIGMA),RBASE(ISIGMS),
     >              RBASE(ICOURI),RBASE(IVOLME),RBASE(ISIGT) ,
     >              RBASE(ISIGW) ,RBASE(IDIFF) ,RBASE(IFLXME),
     >              RBASE(ISUNME),RBASE(IFUNKN),RBASE(IFACTO),
     >              RBASE(ISOURC),RBASE(ICURH))
        CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMDN)  
        CALL RLSARA(RBASE(ICURH))
        CALL RLSARA(RBASE(ISOURC))
        CALL RLSARA(RBASE(IFACTO))
        CALL RLSARA(RBASE(IFUNKN))
        CALL RLSARA(RBASE(ISUNME))
        CALL RLSARA(RBASE(IFLXME))
        CALL RLSARA(RBASE(IDIFF))
        CALL RLSARA(RBASE(ISIGW))
        CALL RLSARA(RBASE(ISIGT))
        CALL RLSARA(RBASE(IVOLME))
        CALL RLSARA(RBASE(ICOURI))
        CALL RLSARA(RBASE(ISIGMS))
        CALL RLSARA(RBASE(ISIGMA))
      ELSE
        CALL XABORT(NAMSBR//': UNKNOWN TYPE OF EQUIVALENCE.')
      ENDIF
C----
C  SAVE SPH FACTORS
C----
      CALL LCMSIX(IPEDIT,CURNAM,ILCMUP)
      CALL LCMSIX(IPEDIT,'MACROLIB',ILCMUP)
      CALL LCMPUT(IPEDIT,'SPH-FACTORS',NMERGE*NGCOND,2,SPH)
      CALL LCMSIX(IPEDIT,'MACROLIB',ILCMDN)
      CALL LCMSIX(IPEDIT,CURNAM,ILCMDN)
      IF(LNEWDR) THEN
         IF((NDOOR.EQ.'EXCELL').OR.(NDOOR.EQ.'NXT').OR.
     >      (NDOOR.EQ.'JPMT')) THEN
            IER=KDRCLS(IFTRAK,2)
            IF(IER.NE.0) CALL XABORT(NAMSBR//': KDRCLS FAILURE.')
         ENDIF
         CALL LCMCL(IPTRK2,2)
      ENDIF
      CALL RLSARA(IBASE(IMERG2))
      CALL RLSARA(IBASE(IKEY2))
      CALL RLSARA(IBASE(IMAT2))
      CALL RLSARA(RBASE(IVOL2))
      RETURN
 100  FORMAT(/1X,A6,': GEOMETRY ',A12,' WAS PREVIOUSLY TRACKED ',
     > 'BY MODULE ',A12)
 110  FORMAT(/1X,A6,': MACRO-GEOMETRY ',A12,' WILL BE TRACKED ',
     > 'BY MODULE ',A12)
 120  FORMAT(/1X,A6,': USER-SUPPLIED MACRO-GEOMETRY ',A12,
     > ' WILL BE TRACKED BY MODULE ',A12)
 130  FORMAT(/1X,A6,': NEUTRON LEAKAGE THROUGH BOUNDARY')
 140  FORMAT(/1X,A6,': NO NEUTRON LEAKAGE THROUGH BOUNDARY')
      END
