*DECK SUS
      SUBROUTINE SUS(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* To prepare information for the SUS3D code.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Update(s): 
* None.
*
*Reference: 
*
*Parameters: input
* NENTRY  number of data structures transfered to this module.
* HENTRY  name of the data structures.
* IENTRY  data structure type where:
*         \begin{itemize} 
*         \item IENTRY=1 for LCM memory object; 
*         \item IENTRY=2 for XSM file; 
*         \item IENTRY=3 for sequential binary file;
*         \item IENTRY=4 for sequential ASCII file.  
*         \end{itemize}
*JENTRY   access permission for the data structure where:
*         \begin{itemize} 
*         \item JENTRY=0 for a data structure in creation mode;
*         \item JENTRY=1 for a data structure in modifications mode;
*         \item JENTRY=2 for a data structure in read-only mode.
*         \end{itemize} 
* KENTRY  data structure pointer.
* 
*Comments:
* Instructions for the use of the SUS: module:
*   DirOut FLXOUT ADJOUT := SUS: [ BINOUT ] VOLTRK FLUX :: (SUSget) ;
* or
*   ASCOUT   := SUS: [ ASCOUT ] VOLTRK FLUX :: (SUSget) ;
* where
*     DirOut   : sequential ASCII SUS3D direction and weight file
*     FLXOUT   : sequential binary/ASCII SUS3D flux file
*     ADJOUT   : sequential binary/ASCII SUS3D adjoint file
*     VOLTRK   : tracking data structure
*                (signature L_TRACK) 
*     FLUX     : flux data structure
*                (signature L_FLUX) 
*     (SUSget) : Processing options
*                (read from input using the SUSGET routine).
*
*-----------------------------------------------------------------------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          NENTRY                  	
      CHARACTER        HENTRY(NENTRY)*12
      INTEGER          IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='SUS   ')
      INTEGER          ILCMUP,ILCMDN
      PARAMETER       (ILCMUP=1,ILCMDN=2)
      INTEGER          NSTATE,NTC,MAXENT
      PARAMETER       (NSTATE=40,NTC=3)
*----
*  Memory allocation
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          XPOL,WPOL,XDAZI,WDAZI,XAZI,WAZI,
     >                 FLUX,AFLUX,TFLUX,VOLUME
*----
*  Local variables
*----
      INTEGER          IPSUS(3),IKSUS(3),IPTRK,IKTRK,IPFLU,IKFLU
      INTEGER          IEN,ITC,KCHAR(NTC)
      CHARACTER        HSIGN*12
      INTEGER          ISTATE(NSTATE)
      INTEGER          IPRINT,IFORM
      INTEGER          NTREG,NTUNK,NTANI,ITROP,LTRK,
     >                 AZMOAQ,AZMQUA,POLQUA,POLOAQ
      INTEGER          NDIM
      INTEGER          ISADJ,NGROUP,NREG,NBAQU
      INTEGER          LCMLN,LCMTY
*----
*  Validate entry parameters
*----
      IF(NENTRY .NE. 5) CALL XABORT(NAMSBR//
     >  ': Five data structures required for this module.')
*----
*  Scan data structure to determine type and mode
*----
      DO IEN=1,3
        IKSUS(IEN)=0
        IPSUS(IEN)=0
      ENDDO
      IKTRK=0
      IPTRK=0
      IKFLU=0
      IPFLU=0
*----
*  Output structure
*----
      IEN=1
      IF(IENTRY(IEN) .EQ. 4) THEN
        IF(JENTRY(IEN) .EQ. 2) CALL XABORT(NAMSBR//
     >  ': Data structure not in update or creation mode.')
        IKSUS(IEN)=IEN
        IPSUS(IEN)=KENTRY(IEN)
        IF(JENTRY(IEN) .EQ. 1) REWIND(IPSUS(IEN))
      ELSE
        CALL XABORT(NAMSBR//
     >  ': Invalid data structure format for '//HENTRY(IEN)//'.')
      ENDIF
      IEN=2
      IF(IENTRY(IEN) .EQ. 3 .OR. IENTRY(IEN) .EQ. 4) THEN
        IF(JENTRY(IEN) .EQ. 2) CALL XABORT(NAMSBR//
     >  ': Data structure not in update or creation mode.')
        IKSUS(IEN)=IEN
        IPSUS(IEN)=KENTRY(IEN)
        IF(IENTRY(IEN) .EQ. 3) IKSUS(IEN)=-IEN
        IF(JENTRY(IEN) .EQ. 1) REWIND(IPSUS(IEN))
      ELSE
        CALL XABORT(NAMSBR//
     >  ': Invalid data structure format for '//HENTRY(IEN)//'.')
      ENDIF
      IEN=3
      IF(IENTRY(IEN) .EQ. 3 .OR. IENTRY(IEN) .EQ. 4) THEN
        IF(JENTRY(IEN) .EQ. 2) CALL XABORT(NAMSBR//
     >  ': Data structure not in update or creation mode.')
        IKSUS(IEN)=IEN
        IPSUS(IEN)=KENTRY(IEN)
        IF(IENTRY(IEN) .EQ. 3) IKSUS(IEN)=-IEN
        IF(JENTRY(IEN) .EQ. 1) REWIND(IPSUS(IEN))
      ELSE
        CALL XABORT(NAMSBR//
     >  ': Invalid data structure format for '//HENTRY(IEN)//'.')
      ENDIF
*----
*  Input structure
*----
      DO IEN=4,NENTRY
        IF(IENTRY(IEN) .EQ. 1 .OR. IENTRY(IEN) .EQ. 2) THEN
          IF(JENTRY(IEN) .NE. 2) CALL XABORT(NAMSBR//
     >    ': Data structure not in read-only mode.')
            CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
          WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
          IF(HSIGN .EQ. 'L_FLUX') THEN
            IKFLU=IEN
            IPFLU=KENTRY(IEN)
          ELSE IF(HSIGN .EQ. 'L_TRACK') THEN
            IKTRK=IEN
            IPTRK=KENTRY(IEN)
            CALL LCMGET(KENTRY(IEN),'TRACK-TYPE',KCHAR)
            WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
            IF(HSIGN .NE. 'EXCELL') CALL XABORT(NAMSBR//
     >      ': Only EXCELL type tracking valid.')
          ELSE
            CALL XABORT(NAMSBR//
     >      ': Invalid signature for '//HENTRY(IEN)//'.')
          ENDIF
        ELSE
          CALL XABORT(NAMSBR//
     >    ': Invalid data structure format for '//HENTRY(IEN)//'.')
        ENDIF
      ENDDO
*----
*  Recover processing option
*----
      CALL SUSGET(IPRINT,IFORM)
*----
*  Get STATE-VECTOR from FLUX data structure
*----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPFLU,'STATE-VECTOR',ISTATE)
      ISADJ=ISTATE(8)
      NGROUP=ISTATE(9)
      NREG=ISTATE(10)
*----
*  Get STATE-VECTOR from VOLTRK data structure
*----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      NTREG=ISTATE(1)
      NTUNK=ISTATE(2)
      NTANI=ISTATE(6)
      ITROP=ISTATE(7)
      LTRK=ISTATE(9)
      AZMOAQ=ISTATE(11)
      POLQUA=ISTATE(13)
      POLOAQ=ISTATE(14)
      AZMQUA=ISTATE(15)
      IF(NTREG .NE. NTUNK) CALL XABORT(NAMSBR//
     >': Inconsistent number of regions and unknowns.')
      IF(NTREG .NE. NREG) CALL XABORT(NAMSBR//
     >': Number of unknowns in VOLTRK and FLUX inconsistent.')
      IF(ITROP .NE. 4) CALL XABORT(NAMSBR//
     >': Only NXT: tracking permitted.')
      IF(POLQUA .LE. -1) CALL XABORT(NAMSBR//
     >': No polar quadrature provided.')
      IF(AZMQUA .LE.  0) CALL XABORT(NAMSBR//
     >': No azimuthal quadrature provided.')
      CALL SETARA(RBASE,NTREG,VOLUME)
      CALL LCMGET(IPTRK,'VOLUME      ',RBASE(VOLUME))
      CALL LCMSIX(IPTRK,'NXTRecords  ',ILCMUP)
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'G00000001DIM',ISTATE)
      NDIM=ISTATE(1)
*----
*  Allocate memory for direction and weights and read them
*----
      CALL LCMLEN(IPTRK,'TrackingTrkW',LCMLN,LCMTY)
      AZMOAQ=LCMLN/2
      CALL SETARA(RBASE,2*POLOAQ,XPOL)
      CALL SETARA(RBASE,POLOAQ,WPOL)
      CALL SETARA(RBASE,2*NDIM*AZMOAQ,XDAZI)
      CALL SETARA(RBASE,2*AZMOAQ,WDAZI)
      CALL LCMGET(IPTRK,'TrackingDirc',RBASE(XDAZI))
      CALL LCMGET(IPTRK,'TrackingTrkW',RBASE(WDAZI))
      CALL LCMGET(IPTRK,'POLAR MU    ',RBASE(XPOL))
      CALL LCMGET(IPTRK,'POLAR WEIGHT',RBASE(WPOL))
      CALL LCMSIX(IPTRK,'NXTRecords  ',ILCMDN)
      CALL SETARA(RBASE,NDIM*AZMOAQ,XAZI)
      CALL SETARA(RBASE,AZMOAQ,WAZI)
      CALL XDRSDB(NDIM*AZMOAQ,RBASE(XAZI),RBASE(XDAZI),1)
      CALL XDRSDB(AZMOAQ,RBASE(WAZI),RBASE(WDAZI),1)
      CALL RLSARA(RBASE(WDAZI))
      CALL RLSARA(RBASE(XDAZI))
*----
*  Allocate memory for integrated and angular flux
*---
      NBAQU=POLOAQ*AZMOAQ*2
      CALL SETARA(RBASE,NREG*2*NGROUP*2,FLUX)
      CALL SETARA(RBASE,2*NREG*NBAQU*2*NGROUP,AFLUX)
      CALL SETARA(RBASE,(NREG+1)*NBAQU*2,TFLUX)
*----
*  Process information
*----
      CALL SUSDRV(IPFLU ,IPSUS ,IKSUS ,IPRINT,IFORM ,
     >            NREG  ,NGROUP,NDIM  ,POLOAQ,AZMOAQ,
     >            RBASE(VOLUME),RBASE(XPOL)  ,RBASE(WPOL)  ,
     >            RBASE(XAZI)  ,RBASE(WAZI)  ,
     >            RBASE(FLUX)  ,RBASE(AFLUX) ,RBASE(TFLUX))
*----
*  Release memory and return
*----
      CALL RLSARA(RBASE(TFLUX))
      CALL RLSARA(RBASE(AFLUX))
      CALL RLSARA(RBASE(FLUX))
      CALL RLSARA(RBASE(WAZI))
      CALL RLSARA(RBASE(XAZI))
      CALL RLSARA(RBASE(WPOL))
      CALL RLSARA(RBASE(XPOL))
      CALL RLSARA(RBASE(VOLUME))
      RETURN
*----
*  Warning formats
*----
      END
