*DECK SUSDRV
      SUBROUTINE SUSDRV(IPFLU ,IPSUS ,IKSUS ,IPRINT,IFORM ,
     >                  NREG  ,NGROUP,NDIM  ,POLOAQ,AZMOAQ,
     >                  VOLUME,XPOL  ,WPOL  ,XAZI  ,WAZI  ,
     >                  FLUX  ,AFLUX ,TFLUX )
*
*-----------------------------------------------------------------------
*
*Purpose:
* To process the angular fluxes and generate the SUS3D file.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Parameters: input
* IPFLU   : pointer to the FLUX data structure
* IPSUS   : pointer to the SUS3D files
* IKSUS   : tupe of SUS3D files
* IPRINT  : print level
* IFORM   : SUSD3D flux format
* NREG    : number of regions for problem
* NGROUP  : number of groups for problem
* NDIM    : number of dimensions of problem
* POLOAQ  : polar quadrature order
* AZMOAQ  : azimuthal quadrature order
* VOLUME  : regional volumes
* XPOL    : polar quadrature points
* WPOL    : polar quadrature weights
* XAZI    : azimuthal quadrature points
* WAZI    : azimuthal quadrature weights
*Parameters: output
* FLUX    : direct and adjoint flux.
* AFLUX   : angular components of the
*           direct and adjoint flux.
* TFLUX   : temporary flux vector.
*
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPFLU,IPSUS(3),IKSUS(3)
      INTEGER          IPRINT,IFORM,NREG,NGROUP,NDIM,POLOAQ,AZMOAQ
      REAL             VOLUME(NREG),XPOL(POLOAQ,2),WPOL(POLOAQ),
     >                 XAZI(NDIM,AZMOAQ),WAZI(AZMOAQ)
      REAL             FLUX(NREG,2,NGROUP,2)
      DOUBLE PRECISION AFLUX(NREG,POLOAQ,AZMOAQ*2,2,NGROUP)
      DOUBLE PRECISION TFLUX(NREG+1,POLOAQ,AZMOAQ*2)
      REAL             ETA
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='SUSDRV')
      INTEGER          ILCMUP,ILCMDN
      PARAMETER       (ILCMUP=1,ILCMDN=2)
*----
*  Local variables
*----
      INTEGER          IGROUP,IPU,IKU,IAPU,IAKU,NAZ,IA,IP,IR,IT
      CHARACTER*12     NAMREC
      REAL             RELERR,ERRMAX
*----
*  Initialize FLUX vectors
*----
      NAZ=AZMOAQ
      DO IP=1,POLOAQ
        XPOL(IP,2)=SQRT(1.-XPOL(IP,1)*XPOL(IP,1))  
      ENDDO
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6000) NAMSBR,NGROUP,NDIM,2*AZMOAQ,POLOAQ,NREG
        WRITE(IOUT,6001)
        WRITE(IOUT,6011) 
     >  (XAZI(1,IA),XAZI(2,IA),WAZI(IA),IA=1,AZMOAQ)
        WRITE(IOUT,6002)
        WRITE(IOUT,6011) 
     >  (XPOL(IP,1),XPOL(IP,2),WPOL(IP),IP=1,POLOAQ)
      ENDIF
      CALL XDRSET(FLUX,NREG*2*NGROUP*2,0.0)
      CALL XDDSET(AFLUX,NREG*POLOAQ*AZMOAQ*2*2*NGROUP,0.0D0)
*----
*  Get information from FLUX data structure.
*  1. Flux
*  2. Angular flux
*  3. Adjoint 
*  4. Angular adjoint
*----
      CALL LCMSIX(IPFLU,'FLUXDIRECT  ',ILCMUP)
      DO IGROUP=1,NGROUP
        WRITE(NAMREC,'(A4,I3,5X)') 'FLUX',IGROUP
        CALL LCMGET(IPFLU,NAMREC,FLUX(1,1,IGROUP,1))
      ENDDO
      CALL LCMSIX(IPFLU,'ANGULAR DIR ',ILCMUP)
      DO IGROUP=1,NGROUP
        WRITE(NAMREC,'(A5,I3,4X)') 'AFLUX',IGROUP
        CALL LCMGET(IPFLU,NAMREC,AFLUX(1,1,1,1,IGROUP))
      ENDDO
      CALL LCMSIX(IPFLU,'ANGULAR DIR ',ILCMDN)
      CALL LCMSIX(IPFLU,'FLUXDIRECT  ',ILCMDN)
      CALL LCMSIX(IPFLU,'FLUXADJOINT ',ILCMUP)
      DO IGROUP=1,NGROUP
        WRITE(NAMREC,'(A4,I3,5X)') 'FLUX',IGROUP
        CALL LCMGET(IPFLU,NAMREC,FLUX(1,2,IGROUP,1))
      ENDDO
      CALL LCMSIX(IPFLU,'ANGULAR ADJ ',ILCMUP)
      DO IGROUP=1,NGROUP
        WRITE(NAMREC,'(A5,I3,4X)') 'AFLUX',IGROUP
        CALL LCMGET(IPFLU,NAMREC,AFLUX(1,1,1,2,IGROUP))
      ENDDO
      CALL LCMSIX(IPFLU,'ANGULAR ADJ ',ILCMDN)
      CALL LCMSIX(IPFLU,'FLUXADJOINT ',ILCMDN)
*----
*  Create first SUS file
*  Volume, and tracking directions
*----
      IPU=IPSUS(1)
      WRITE(IPU,1000) NREG
      WRITE(IPU,1001) (VOLUME(IR),IR=1,NREG)
      WRITE(IPU,1010) 2*AZMOAQ*POLOAQ
*----
*  -\Omega (\varphi+\pi,\pi-\theta)
*----
      DO IA=1,AZMOAQ
        ETA=XAZI(2,NAZ+1-IA)
        IF(ETA .EQ. 0.0) THEN
          ETA=1.0E-10
        ENDIF
        DO IP=1,POLOAQ
          WRITE(IPU,1001) 
     >   -XPOL(IP,2)*XAZI(1,NAZ+1-IA),
     >   -XPOL(IP,2)*ETA,
     >   -XPOL(IP,1),
     >    WPOL(IP)/WAZI(NAZ+1-IA)
        ENDDO
      ENDDO
*----
*  +\Omega (\varphi,\theta)
*----
      DO IA=1,AZMOAQ
        ETA=XAZI(2,IA)
        IF(ETA .EQ. 0.0) THEN
          ETA=1.0E-10
        ENDIF
        DO IP=1,POLOAQ
          WRITE(IPU,1001) 
     >    XPOL(IP,2)*XAZI(1,IA),
     >    XPOL(IP,2)*ETA,
     >    XPOL(IP,1),
     >    WPOL(IP)/WAZI(IA)
        ENDDO
      ENDDO
*----
*  +\Omega (\varphi,\theta)
*----
*      DO IA=1,AZMOAQ
*        DO IP=1,POLOAQ
*          WRITE(IPU,1001) 
*     >    XPOL(IP,2)*XAZI(1,IA),
*     >    XPOL(IP,2)*XAZI(2,IA),
*     >    XPOL(IP,1),
*     >    WPOL(IP)/WAZI(IA)
*        ENDDO
*      ENDDO
*----
*  Print angular flux
*----
      IPU=IPSUS(2)
      IKU=IKSUS(2)
      ERRMAX=0.0
      IF(IKU .LT. 0) THEN
        DO IGROUP=1,NGROUP 
          WRITE(IPU) 'Group',IGROUP
        ENDDO
        DO IGROUP=NGROUP+1,NGROUP+6
          WRITE(IPU) 'Comments'
        ENDDO
      ELSE IF(IKU .GT. 0) THEN
        DO IGROUP=1,NGROUP 
          WRITE(IPU,'(A8,4X,I8)') 'Groups =',IGROUP
        ENDDO
        DO IGROUP=NGROUP+1,NGROUP+6
          WRITE(IPU,'(A8)') 'Comments'
        ENDDO
      ENDIF
      DO IGROUP=1,NGROUP
        DO IA=1,AZMOAQ*2
          DO IP=1,POLOAQ
            IF(IFORM .EQ. 1) THEN
              DO IR=1,NREG
                TFLUX(IR,IP,IA)=AFLUX(IR,IP,IA,1,IGROUP)
                IF(IA .LE. AZMOAQ) THEN
                  FLUX(IR,1,IGROUP,2)=FLUX(IR,1,IGROUP,2)
     >           +AFLUX(IR,IP,IA,1,IGROUP)*WPOL(IP)/WAZI(NAZ+1-IA)
                ELSE
                  FLUX(IR,1,IGROUP,2)=FLUX(IR,1,IGROUP,2)
     >           +AFLUX(IR,IP,IA,1,IGROUP)*WPOL(IP)/WAZI(IA-NAZ)
                ENDIF
              ENDDO
              IF(IPRINT .GE. 100) THEN
                WRITE(IOUT,6003) IP,IA,IGROUP
                WRITE(IOUT,6010) (TFLUX(IR,IP,IA),IR=1,NREG)
              ENDIF
            ELSE
              IR=1
              TFLUX(IR,IP,IA)=AFLUX(IR,IP,IA,1,IGROUP)
              DO IR=1,NREG
                TFLUX(IR+1,IP,IA)=
     >            2.0D0*AFLUX(IR,IP,IA,1,IGROUP)-TFLUX(IR,IP,IA)
                IF(IA .LE. AZMOAQ) THEN
                  FLUX(IR,1,IGROUP,2)=FLUX(IR,1,IGROUP,2)
     >           +AFLUX(IR,IP,IA,1,IGROUP)*WPOL(IP)/WAZI(NAZ+1-IA)
                ELSE
                  FLUX(IR,1,IGROUP,2)=FLUX(IR,1,IGROUP,2)
     >           +AFLUX(IR,IP,IA,1,IGROUP)*WPOL(IP)/WAZI(IA-NAZ)
                ENDIF
              ENDDO
              IF(IPRINT .GE. 100) THEN
                WRITE(IOUT,6003) IP,IA,IGROUP
                WRITE(IOUT,6010) 
     >          ((TFLUX(IR,IP,IA)+TFLUX(IR+1,IP,IA))/2.0,IR=1,NREG)
              ENDIF
            ENDIF
          ENDDO
        ENDDO
        IF(IFORM .EQ. 1) THEN
          IF(IKU .LT. 0) THEN
            WRITE(IPU) (((TFLUX(IR,IP,IA),IR=1,NREG),
     >                   IP=1,POLOAQ),IA=1,AZMOAQ)
            WRITE(IPU) (((TFLUX(IR,IP,IA),IR=1,NREG),
     >                   IP=1,POLOAQ),IA=AZMOAQ*2,AZMOAQ+1,-1)
          ELSE IF(IKU .GT. 0) THEN
            WRITE(IPU,1001) (((TFLUX(IR,IP,IA),IR=1,NREG),
     >                   IP=1,POLOAQ),IA=1,AZMOAQ)
            WRITE(IPU,1001) (((TFLUX(IR,IP,IA),IR=1,NREG),
     >                   IP=1,POLOAQ),IA=AZMOAQ*2,AZMOAQ+1,-1)
          ENDIF
        ELSE
          IF(IKU .LT. 0) THEN
            WRITE(IPU) (((TFLUX(IR,IP,IA),IR=1,NREG+1),
     >                   IP=1,POLOAQ),IA=1,AZMOAQ)
            WRITE(IPU) (((TFLUX(IR,IP,IA),IR=1,NREG+1),
     >                   IP=1,POLOAQ),IA=AZMOAQ*2,AZMOAQ+1,-1)
          ELSE IF(IKU .GT. 0) THEN
            WRITE(IPU,1001) (((TFLUX(IR,IP,IA),IR=1,NREG+1),
     >                   IP=1,POLOAQ),IA=1,AZMOAQ)
            WRITE(IPU,1001) (((TFLUX(IR,IP,IA),IR=1,NREG+1),
     >                   IP=1,POLOAQ),IA=AZMOAQ*2,AZMOAQ+1,-1)
          ENDIF
        ENDIF
        IF(IPRINT .GE. 20) THEN
          WRITE(6,6030) IGROUP
        ENDIF          
        DO IR=1,NREG
          RELERR=100.0*(FLUX(IR,1,IGROUP,2)-FLUX(IR,1,IGROUP,1))
     >          /FLUX(IR,1,IGROUP,2)
          ERRMAX=MAX(ERRMAX,ABS(RELERR))
          IF(IPRINT .GE. 20) THEN
            WRITE(6,6031) IR,FLUX(IR,1,IGROUP,1),FLUX(IR,1,IGROUP,2),
     >                RELERR
          ENDIF          
        ENDDO
      ENDDO      
      IF(IPRINT .GE. 10) THEN
        WRITE(6,6020) ERRMAX
      ENDIF
*----
*  Print angular adjoint
*----
      IAPU=IPSUS(3)
      IAKU=IKSUS(3)
      ERRMAX=0.0
      IF(IAKU .LT. 0) THEN
        DO IGROUP=NGROUP,1,-1 
          WRITE(IAPU) 'Group',IGROUP
        ENDDO
        DO IGROUP=NGROUP+1,NGROUP+6
          WRITE(IAPU) 'Comments'
        ENDDO
      ELSE IF(IAKU .GT. 0) THEN
        DO IGROUP=NGROUP,1,-1 
          WRITE(IAPU,'(A8,4X,I8)') 'Groups =',IGROUP
        ENDDO
        DO IGROUP=NGROUP+1,NGROUP+6
          WRITE(IAPU,'(A8)') 'Comments'
        ENDDO
      ENDIF
      DO IGROUP=NGROUP,1,-1
        DO IA=1,AZMOAQ*2
          DO IP=1,POLOAQ
            IF(IFORM .EQ. 1) THEN
              DO IR=1,NREG
                TFLUX(IR,IP,IA)=AFLUX(IR,IP,IA,2,IGROUP)
                IF(IA .LE. AZMOAQ) THEN
                  FLUX(IR,2,IGROUP,2)=FLUX(IR,2,IGROUP,2)
     >           +AFLUX(IR,IP,IA,2,IGROUP)*WPOL(IP)/WAZI(NAZ+1-IA)
                ELSE
                  FLUX(IR,2,IGROUP,2)=FLUX(IR,2,IGROUP,2)
     >           +AFLUX(IR,IP,IA,2,IGROUP)*WPOL(IP)/WAZI(IA-NAZ)
                ENDIF
              ENDDO
              IF(IPRINT .GE. 100) THEN
                WRITE(IOUT,6004) IP,IA,IGROUP
                WRITE(IOUT,6010) (TFLUX(IR,IP,IA),IR=1,NREG)
              ENDIF
            ELSE
              IR=1
              TFLUX(IR,IP,IA)=AFLUX(IR,IP,IA,2,IGROUP)
              DO IR=1,NREG
                TFLUX(IR+1,IP,IA)=
     >            2.0D0*AFLUX(IR,IP,IA,2,IGROUP)-TFLUX(IR,IP,IA)
                IF(IA .LE. AZMOAQ) THEN
                  FLUX(IR,2,IGROUP,2)=FLUX(IR,2,IGROUP,2)
     >           +AFLUX(IR,IP,IA,2,IGROUP)*WPOL(IP)/WAZI(NAZ+1-IA)
                ELSE
                  FLUX(IR,2,IGROUP,2)=FLUX(IR,2,IGROUP,2)
     >           +AFLUX(IR,IP,IA,2,IGROUP)*WPOL(IP)/WAZI(IA-NAZ)
                ENDIF
              ENDDO
              IF(IPRINT .GE. 100) THEN
                WRITE(IOUT,6004) IP,IA,IGROUP
                WRITE(IOUT,6010) 
     >          ((TFLUX(IR,IP,IA)+TFLUX(IR+1,IP,IA))/2.0,IR=1,NREG)
              ENDIF
            ENDIF
          ENDDO
        ENDDO
        IF(IFORM .EQ. 1) THEN
          IF(IAKU .LT. 0) THEN
            WRITE(IAPU) (((TFLUX(IR,IP,IA),IR=1,NREG),
     >                   IP=1,POLOAQ),IA=1,AZMOAQ)
            WRITE(IAPU) (((TFLUX(IR,IP,IA),IR=1,NREG),
     >                   IP=1,POLOAQ),IA=AZMOAQ*2,AZMOAQ+1,-1)
          ELSE IF(IAKU .GT. 0) THEN
            WRITE(IAPU,1001) (((TFLUX(IR,IP,IA),IR=1,NREG),
     >                   IP=1,POLOAQ),IA=1,AZMOAQ)
            WRITE(IAPU,1001) (((TFLUX(IR,IP,IA),IR=1,NREG),
     >                   IP=1,POLOAQ),IA=AZMOAQ*2,AZMOAQ+1,-1)
          ENDIF
        ELSE
          IF(IAKU .LT. 0) THEN
            WRITE(IAPU) (((TFLUX(IR,IP,IA),IR=1,NREG+1),
     >                   IP=1,POLOAQ),IA=1,AZMOAQ)
            WRITE(IAPU) (((TFLUX(IR,IP,IA),IR=1,NREG+1),
     >                   IP=1,POLOAQ),IA=AZMOAQ*2,AZMOAQ+1,-1)
          ELSE IF(IAKU .GT. 0) THEN
            WRITE(IAPU,1001) (((TFLUX(IR,IP,IA),IR=1,NREG+1),
     >                   IP=1,POLOAQ),IA=1,AZMOAQ)
            WRITE(IAPU,1001) (((TFLUX(IR,IP,IA),IR=1,NREG+1),
     >                   IP=1,POLOAQ),IA=AZMOAQ*2,AZMOAQ+1,-1)
          ENDIF
        ENDIF
        IF(IPRINT .GE. 20) THEN
          WRITE(6,6032) IGROUP
        ENDIF          
        DO IR=1,NREG
          RELERR=100.0*(FLUX(IR,2,IGROUP,2)-FLUX(IR,2,IGROUP,1))
     >          /FLUX(IR,2,IGROUP,2)
          ERRMAX=MAX(ERRMAX,ABS(RELERR))
          IF(IPRINT .GE. 20) THEN
            WRITE(6,6031) IR,FLUX(IR,2,IGROUP,1),FLUX(IR,2,IGROUP,2),
     >                RELERR
          ENDIF          
        ENDDO
      ENDDO      
      IF(IPRINT .GE. 10) THEN
        WRITE(6,6021) ERRMAX
      ENDIF
*----
*  Processing finished, return
*----
      RETURN 
*----
*  Formats
*----
 1000 FORMAT('Number of regions =',I5/'Regional volumes')
 1001 FORMAT(1P,5E20.10)
 1010 FORMAT('Nomber of angular quadrature points ',I5/
     >       'Angular quadrature points')
 6000 FORMAT('Output from routine ',A6/
     >       'Number of groups  =',I5/
     >       'Number of dimens  =',I5/
     >       'Number of azimuth =',I5/
     >       'Number of polar   =',I5/
     >       'Number of regions =',I5)
 6001 FORMAT('Azimuthal quadrature')
 6002 FORMAT('Polar quadrature')
 6003 FORMAT('Polar =',I5,2X,'Azim  = ',I5,2X,'Group = ',I5/
     >       'Direct angular flux per region ')
 6004 FORMAT('Polar =',I5,2X,'Azim  = ',I5,2X,'Group = ',I5/
     >       'Adjoint angular flux per region')
 6010 FORMAT(1P,5E20.10)
 6011 FORMAT(1P,3E20.10)
 6020 FORMAT('Maximum relative on flux    (%) = ',F15.7)
 6021 FORMAT('Maximum relative on adjoint (%) = ',F15.7)
 6030 FORMAT(' Flux for group    ',I5)
 6031 FORMAT(' Region ',I5,1P,2E20.10,5X,0P,F20.10)
 6032 FORMAT(' Adjoint for group ',I5)
      END 
