*DECK SYBALC
      SUBROUTINE SYBALC (NPIJ,MAXPTS,RAYRE,SIGT,PIJ,MR,ALBEDO,AUXI)
*
*----------------------------------------------------------------------
*
* PIJ CALCULATION USING THE METHOD OF KAVENOKY IN ANNULAR GEOMETRY.
*
* INPUT PARAMETERS:
*    NPIJ   : NUMBER OF REGIONS.
*    MAXPTS : FIRST DIMENSION OF MATRIX PIJ.
*    RAYRE  : RADIUS OF REGIONS ARRAY. DIMENSION RAYRE(NPIJ+1)
*    SIGT   : TOTAL CROSS SECTION ARRAY. DIMENSION SIGT(NPIJ)
*    MR     : NUMBER OF GAUSS POINTS.
*    ALBEDO : OUTSIDE ALBEDO.
*    AUXI   : DUMMY WORK ARRAY. DIMENSION AUXI(4*NPIJ)
*
* OUTPUT PARAMETERS:
*    PIJ    : REDUCED COLLISION PROBABILITIES ARRAY.
*             DIMENSION  PIJ(MAXPTS,NPIJ)
*
* REFERENCE:
* A. KAVENOKY, 'CALCUL ET UTILISATION DES PROBABILITES DE PREMIERE
* COLLISION POUR LES MILIEUX HETEROGENES A UNE DIMENSION: LES PROGRAMMES
* ALCOLL ET CORTINA', NOTE CEA-N-1077, COMMISSARIAT A L'ENERGIE
* ATOMIQUE, MARS 1969.
*
*----------------------------------------------------------------------
*
      PARAMETER (PI=3.1415926535,TRONC=1.0E-3)
      DIMENSION RAYRE(*),SIGT(*),PIJ(*),AUXI(*),DCB(64),ALPHA(64)
      DOUBLE PRECISION DZ1,DZ2,DZ22
*
      CUTOFF=1.0-TRONC
      TSUR=2.0*PI*RAYRE(NPIJ+1)
      TSUR4=4.0/TSUR
*
*------    START PIJ CALCULATION
*
      NVID=0
      NPIJ2=2*NPIJ
      NPIJ3=3*NPIJ
      DO 1 I=1,NPIJ*NPIJ
    1 PIJ(I)=0.0
      DO 120 I=1,NPIJ
      AUXI(NPIJ+I)=PI*(RAYRE(I+1)-RAYRE(I))*(RAYRE(I+1)+RAYRE(I))
      IND=SIGT(I)+CUTOFF
      IF (IND.GT.1) THEN
        IND=1
      ENDIF
      AUXI(NPIJ2+I)=IND
      IF (IND.LE. 0) THEN
        NVID=NVID+1
        AUXI(NPIJ3+NVID)=FLOAT(I)+0.5
      ENDIF
  120 CONTINUE
*
      CALL ALGPT(MR,-1.0,1.0,ALPHA,DCB)
      DO 2 IM=1,NPIJ
      IIM=(NPIJ+1)*IM-NPIJ
      RD=SQRT(RAYRE(IM+1)-RAYRE(IM))
*
      DO 3 M=1,MR
      II=IIM
C------
C  REPLACE GAUSS BY ALGPT - THUS YM=RD*(ALPHA(M)+0.5)
C  BECOMES YM=RD*0.5*(ALPHA(M)+1.0) SINCE ALPHA IN GAUSS ARE
C  0.5 TIME THOSE IN ALGPT
C------
      YM=RD*0.5*(ALPHA(M)+1.0)
      H=RAYRE(IM+1)-YM**2
      PW=RD*DCB(M)
      PW1=PW*YM
*------
      DO 4 I=IM,NPIJ
    4 AUXI(I)=SQRT((RAYRE(I+1)-H)*(RAYRE(I+1)+H))
      IF (IM.EQ.NPIJ) GO TO 100
      IF (IM.EQ.NPIJ-1) THEN
         DZ22=0.0
         GO TO 101
      ENDIF
      NN2=NPIJ-2
*------
      DZ2=SIGT(IM)*AUXI(IM)*2.0D0
      DZ22=DZ2
      DO 5 I=IM,NN2
      XI=AUXI(NPIJ2+I)
      XI1=AUXI(NPIJ2+I+1)
      JJ=II+NPIJ+1
      A=1.0
      IF (I.EQ.IM) GO TO 150
      A=2.0
*------
      DZ22=DZ22+2.0D0*(AUXI(I)-AUXI(I-1))*SIGT(I)
*------    EVALUATE ZT2 USING TABKI
  150 ZT2=PW1*TABKI(3,REAL(DZ22))
      DZ2=DZ22
      PIJ(II)=PIJ(II)-(A*PW1*PI/4.0-ZT2)*XI
      PIJ(JJ)=PIJ(JJ)+ZT2*XI1
      PIJ(JJ-1)=PIJ(JJ-1)+(PW1*PI/4.0-ZT2)*XI*XI1
      IJ=II+NPIJ
      IK=IJ+NPIJ
*------
      DZ1=(AUXI(I+1)-AUXI(I))*SIGT(I+1)
      DZ2=DZ2+DZ1
*------    EVALUATE ZT1,ZT2 USING TABKI
      ZT1=PW1*TABKI(3,REAL(DZ1))
      ZT2=PW1*TABKI(3,REAL(DZ2))
      XJ=XI1
      XJ1=AUXI(NPIJ2+I+2)
      PIJ(IK)=PIJ(IK)+(ZT1-ZT2)*XI*XJ1
      PIJ(JJ)=PIJ(JJ)+2.*(ZT1-ZT2)*XJ
      PIJ(IK+1)=PIJ(IK+1)+(ZT2-ZT1)*XJ*XJ1
      PIJ(IJ)=PIJ(IJ)+(ZT2-ZT1)*XI*XJ
      I2=I+2
      IJ=IK
      DO 6 J=I2,NPIJ
      XJ=AUXI(NPIJ2+J)
      XJ1=AUXI(NPIJ2+J+1)
      IK=IJ+NPIJ
*------
      U=(AUXI(J)-AUXI(J-1))*SIGT(J)
      DZ1=DZ1+U
      DZ2=DZ2+U
*------    EVALUATE ZT1,ZT2 USING TABKI
      ZT1=TABKI(3,REAL(DZ1))*PW1
      ZT2=TABKI(3,REAL(DZ2))*PW1
      PIJ(IJ+1)=PIJ(IJ+1)+(ZT1-ZT2)*XI1*XJ
      PIJ(IJ)=PIJ(IJ)+(ZT2-ZT1)*XI*XJ
      IF (J.EQ.NPIJ) GO TO 6
      PIJ(IK)=PIJ(IK)+(ZT1-ZT2)*XI*XJ1
      PIJ(IK+1)=PIJ(IK+1)+(ZT2-ZT1)*XI1*XJ1
      IJ=IK
    6 CONTINUE
      II=JJ
    5 CONTINUE
  101 CONTINUE
*------
      DZ1=(AUXI(NPIJ)-AUXI(NPIJ-1))*SIGT(NPIJ)
      JJ=II+NPIJ+1
      DZ2=2.0D0*AUXI(NPIJ-1)*SIGT(NPIJ-1)
      IF(IM.NE.(NPIJ-1))THEN
        DZ2=DZ2-2.0D0*AUXI(NPIJ-2)*SIGT(NPIJ-1)
      ENDIF
      DZ22=DZ22+DZ2
      DZ2=DZ22
*------    EVALUATE ZT1,ZT2 USING TABKI
      ZT1=YM*PW*TABKI(3,REAL(DZ1))
      ZT2=YM*PW*TABKI(3,REAL(DZ2))
      A=2.0
      IF (IM.EQ.NPIJ-1) A=1.0
      XJ=AUXI(NPIJ2+NPIJ-1)
      XJ1=AUXI(NPIJ2+NPIJ)
      ZT=PW1*PI/4.0
      PIJ(II)=PIJ(II)+XJ*(ZT2-A*ZT)
      PIJ(JJ-1)=PIJ(JJ-1)+XJ*XJ1*(ZT-ZT1-ZT2)
      PIJ(JJ)=PIJ(JJ)+(2.*(ZT1-ZT)+ZT2)*XJ1
*------
      ZZ2=DZ2+(AUXI(NPIJ)-AUXI(NPIJ-1))*SIGT(NPIJ)
      ZZ1=ZZ2+(AUXI(NPIJ)-AUXI(NPIJ-1))*SIGT(NPIJ)
      DZ2=ZZ1
*------    EVALUATE ZT1,ZT2 USING TABKI
      ZT1=YM*PW*TABKI(3,ZZ1)
      ZT2=YM*PW*TABKI(3,ZZ2)
      PIJ(JJ-1)=PIJ(JJ-1)+ZT2*AUXI(NPIJ2+NPIJ-1)*AUXI(NPIJ2+NPIJ)
      PIJ(JJ)=PIJ(JJ)-2.*ZT2*AUXI(NPIJ2+NPIJ)+AUXI(NPIJ2+NPIJ)*ZT1
      GO TO 1000
*------
  100 ZZ1=2.*AUXI(NPIJ)*SIGT(NPIJ)
*------    EVALUATE ZT1 USING TABKI
      ZT1=YM*PW*(TABKI(3,ZZ1)-PI/4.0)
      PIJ(II)=PIJ(II)+AUXI(NPIJ2+NPIJ)*ZT1
 1000 IF (NVID.EQ.0) GO TO 3
      INDV=AUXI(NPIJ3+NVID)
      IF (IM.GT.INDV) GO TO 3
      DO 12 K=1,NVID
      I=AUXI(NPIJ3+K)
      DZ1=0.0
      DZ2=0.0
      IF(I.LT.IM) THEN
        GO TO 12
      ELSE IF(I.EQ.IM) THEN
        GO TO 140
      ELSE 
        GO TO 230
      ENDIF
  230 DO 220 J=IM,I-1
*
*------    MODIFIED BY G.M. TO INCLUDE VOIDS
*------
      IF(J.EQ.IM) THEN
        U=AUXI(J)*SIGT(J)
      ELSE
        U=(AUXI(J)-AUXI(J-1))*SIGT(J)
      ENDIF
  220 DZ2=DZ2+U
      DZ1=DZ2
      IJ=NPIJ*(I-1)+IM
      DO 14 J=IM,I-1
*
*------    MODIFIED BY G.M. TO INCLUDE VOIDS
*
      IF(J.EQ.IM) THEN
        U=AUXI(J)*SIGT(J)
      ELSE
        U=(AUXI(J)-AUXI(J-1))*SIGT(J)
      ENDIF
      DZ1=DZ1-U
      DZ2=DZ2+U
      ZA1=YM*TABKI(2,REAL(DZ1))*PW
      ZA2=YM*TABKI(2,REAL(DZ2))*PW
*------
      PIJ(IJ)=PIJ(IJ)+(AUXI(I)-AUXI(I-1))*(ZA1-ZA2)*AUXI(NPIJ2+J)
      IF (J.EQ.I-1) GO TO 14
      PIJ(IJ+1)=PIJ(IJ+1)-(AUXI(I)-AUXI(I-1))*(ZA1-ZA2)*AUXI(NPIJ2+J+1)
      IJ=IJ+1
   14 CONTINUE
  140 IF (I.EQ.NPIJ) GO TO 12
*OLD  IJ=(NPIJ+1)*I-NPIJ
*MODRR
      IJ=(NPIJ+1)*I
       IF(IJ.GT.NPIJ*NPIJ) CALL XABORT('SYBALC: A')
*------
      IF(I.EQ.IM) THEN
        V=AUXI(I)
      ELSE
        V=AUXI(I)-AUXI(I-1)
      ENDIF
*OLD  PIJ(IJ)=PIJ(IJ)+V*YM*PW*(1.+TABKI(1,REAL(DZ2)))*AUXI(NPIJ2+I+1)
*MODRR
      PIJ(IJ)=PIJ(IJ)+V*YM*PW*(1.+TABKI(2,REAL(DZ2)))*AUXI(NPIJ2+I+1)
      DO 15 J=I+1,NPIJ
      IK=IJ+NPIJ
*------  THERE IS A BUG BELOW THIS LINE
      U=(AUXI(J)-AUXI(J-1))*SIGT(J)
      DZ1=DZ1+U
      DZ2=DZ2+U
      ZA1=YM*TABKI(2,REAL(DZ1))*PW
      ZA2=YM*TABKI(2,REAL(DZ2))*PW
*OLD  IF (J.GT.I+1) PIJ(IJ)=PIJ(IJ)-V*(ZA1+ZA2)*AUXI(NPIJ2+J)
*MODRR
      PIJ(IJ)=PIJ(IJ)-V*(ZA1+ZA2)*AUXI(NPIJ2+J)
      IF (J.EQ.NPIJ) GO TO 15
      PIJ(IK)=PIJ(IK)+V*(ZA1+ZA2)*AUXI(NPIJ2+J+1)
      IJ=IK
   15 CONTINUE
   12 CONTINUE
    3 CONTINUE
    2 CONTINUE
      DO 17 I=1,NPIJ
      INDI=AUXI(NPIJ2+I)+0.5
      II=(NPIJ+1)*I-NPIJ
      IJ=II
      JI=IJ
      AI=SIGT(I)
      IF (INDI.EQ.0) AI=1.
      IF (INDI.EQ.0) GO TO 200
*------
      PIJ(II)=1.+2.*PIJ(II)/SIGT(I)/AUXI(NPIJ+I)
      PIJ(II)=PIJ(II)/SIGT(I)
  200 IF (I.EQ.NPIJ) GO TO 17
      DO 16 J=I+1,NPIJ
      INDJ=AUXI(NPIJ2+J)+0.5
      IJ=IJ+NPIJ
      JI=JI+1
      AJ=SIGT(J)
      IF (INDJ.EQ.0) AJ=1.
*------
      PIJ(IJ)=2.*PIJ(IJ)/AI /AUXI(NPIJ+I)/AJ
      PIJ(JI)=PIJ(IJ)*AUXI(NPIJ+I)/AUXI(NPIJ+J)
   16 CONTINUE
   17 CONTINUE
*
*------    INCLUDE ALBEDO.
      IF(ALBEDO.GT.0.0) THEN
        DO 300 I=1,NPIJ
        AUXI(I)=0.0
        IJ=I
        DO 310 J=1,NPIJ
        AUXI(I)=AUXI(I)+PIJ(IJ)*SIGT(J)
  310   IJ=IJ+NPIJ
  300   AUXI(I)=1.0-AUXI(I)
        PSS=0.0
        DO 320 I=1,NPIJ
  320   PSS=PSS+TSUR4*AUXI(NPIJ+I)*AUXI(I)*SIGT(I)
        PSS=1.0-PSS
        PSSM=ALBEDO/(1.0-ALBEDO*PSS)
        IJ=0
        DO 130 J=1,NPIJ
        PSVJ=4.0*AUXI(J)*AUXI(NPIJ+J)/TSUR
        DO 130 I=1,NPIJ
        IJ=IJ+1
  130   PIJ(IJ)=PIJ(IJ)+AUXI(I)*PSSM*PSVJ
      ENDIF
*
*------    REBUILD THE PIJ MATRIX
*
      DO 3000 I=1,NPIJ
      IN=NPIJ-I+1
      IK=NPIJ*IN+1
      IL=MAXPTS*(IN-1)+NPIJ+1
      DO 2100 J=1,NPIJ
      ILJ=IL-J
      IKJ=IK-J
 2100 PIJ(ILJ)=PIJ(IKJ)
 3000 CONTINUE
*
      RETURN
      END
