*DECK SYBALS
      SUBROUTINE SYBALS (N,MAXPTS,R,SIGT,PIJ,MR,ALBG,S)
*
*----------------------------------------------------------------------
*
* PIJ CALCULATION USING THE METHOD OF KAVENOKY IN SPHERICAL GEOMETRY.
*
* INPUT PARAMETERS:
*    N      : NUMBER OF REGIONS.
*    MAXPTS : FIRST DIMENSION OF MATRIX PIJ.
*    R      : RADIUS OF REGIONS ARRAY. DIMENSION R(N+1)
*    SIGT   : TOTAL CROSS SECTION ARRAY. DIMENSION SIGT(N)
*    MR     : NUMBER OF GAUSS POINTS.
*    ALBG   : OUTSIDE ALBEDO.
*    S      : DUMMY WORK ARRAY. DIMENSION S(4*N)
*
* OUTPUT PARAMETERS:
*    PIJ    : REDUCED COLLISION PROBABILITIES ARRAY.
*             DIMENSION  PIJ(MAXPTS,N)
*
* REFERENCE:
* A. KAVENOKY, 'CALCUL ET UTILISATION DES PROBABILITES DE PREMIERE
* COLLISION POUR LES MILIEUX HETEROGENES A UNE DIMENSION: LES PROGRAMMES
* ALCOLL ET CORTINA', NOTE CEA-N-1077, COMMISSARIAT A L'ENERGIE
* ATOMIQUE, MARS 1969.
*
*----------------------------------------------------------------------
*
      DIMENSION R(*),SIGT(*),PIJ(MAXPTS,*),S(*),VIDE(3),X(200),IND(100),
     1 INDV(20),ALPHA(64),DCB(64),PIJN(100)
      EQUIVALENCE (VIDE(3),X(1))
      REAL IND1(100)
C
      IF (N.GT.100)
     > CALL XABORT('SYBALS: NUMBER OF SHELLS GREATER THAN 100.')
      DO 1 I=1,N
      X(I)=0.
      DO 1J=1,N
    1 PIJ(I,J)=0.
      DO 130 I=1,N
  130 S(I)=4./3.*(R(I+1)-R(I))*(R(I+1)**2+R(I+1)*R(I)+R(I)*R(I))
      KMAX=0
      DO 120 IM=1,N
      IND(IM)=SIGT(IM)+0.999999
      SIGT(IM)=-SIGT(IM)
      IF (IND(IM).GT.1) IND(IM)=1
      IND1(IM)=IND(IM)
      IF(IND(IM).LE.0) THEN 
        KMAX=KMAX+1
        INDV(KMAX)=IM
      ENDIF
  120 CONTINUE
      IF (KMAX.GT.20)
     > CALL XABORT('SYBALS: NUMBER OF VOID REGIONS GREATER THAN 20.')
C
      CALL ALGPT(MR,-1.0,1.0,ALPHA,DCB)
      DO 2 IM=1,N
      RD=SQRT(R(IM+1)-R(IM))
      DO 3 M=1,MR
      Y=RD*0.5*(ALPHA(M)+1.0)
      H=R(IM+1)-Y*Y
      PW1=RD*DCB(M)*Y*H
C
      K=N+IM
      DO 4 I=IM,N
      X(I)=SQRT(R(I+1)**2-H**2)
      XX=X(I)
      IF(I.NE.IM) XX=X(I)-X(I-1)
      XX=XX*SIGT(I)
      IF (XX.GT.-150.0) THEN
         X(K)=EXP(XX)
      ELSE
         X(K)=0.0
      ENDIF
    4 K=K+1
C
      IF(IM.EQ.N)GO TO 100
      IF (IM-N+1.NE. 0) THEN 
        ZT22=PW1
        X(N-2)=0.
        GO TO 101
      ENDIF
  110 N2=N-2
      ZT22=PW1*X(N+IM)**2
      DO 5 I=IM,N2
      NI=N+I
      A=1.
      IF (I.EQ.IM) GO TO 150
      A=2.
      ZT22=ZT22*X(NI)**2
  150 ZT2=ZT22
      ZT1=PW1
      PIJ(I,I)=PIJ(I,I)-(A*ZT1-ZT2)*IND1(I)
      PIJ(I+1,I+1)=PIJ(I+1,I+1)+ ZT2*IND1(I+1)
      PIJ(I,I+1)=PIJ(I,I+1)+(ZT1-ZT2)*IND1(I)*IND1(I+1)
      ZT2=ZT2*X(NI+1)
      ZT1=ZT1*X(NI+1)
      PIJ(I,I+1)=PIJ(I,I+1)+(ZT2-ZT1)*IND1(I)*IND1(I+1)
      PIJ(I,I+2)=PIJ(I,I+2)+(ZT1-ZT2)*IND1(I)*IND1(I+2)
      PIJ(I+1,I+1)=PIJ(I+1,I+1)+2.*(ZT1-ZT2)*IND1(I+1)
      PIJ(I+1,I+2)=PIJ(I+1,I+2)+(ZT2-ZT1)*IND1(I+1)*IND1(I+2)
      I2=I+2
      DO 6 J=I2,N
      NJ=N+J
      ZT1=ZT1*X(NJ)
      ZT2=ZT2*X(NJ)
      PIJ(I+1,J)=PIJ(I+1,J)+(ZT1-ZT2)*IND1(I+1)*IND1(J)
      PIJ(I,J)=PIJ(I,J)+(ZT2-ZT1)*IND1(I)*IND1(J)
      IF(J.EQ.N)GO TO 6
      PIJ(I,J+1)=PIJ(I,J+1)+(ZT1-ZT2)*IND1(I)*IND1(J+1)
      PIJ(I+1,J+1)=PIJ(I+1,J+1)+(ZT2-ZT1)*IND1(I+1)*IND1(J+1)
    6 CONTINUE
    5 CONTINUE
  101 ZT1=PW1*X(2*N)
      A=2.
      IF (IM.EQ.N-1) A=1.
      ZT2=PW1
      PIJ(N-1,N-1)=PIJ(N-1,N-1)-A *IND1(N-1)*ZT2
      PIJ(N-1,N)=PIJ(N-1,N)+IND1(N)*IND1(N-1)*(ZT2-ZT1)
      PIJ(N,N)=PIJ(N,N)+2.*(ZT1-ZT2)*IND1(N)
      ZT2=ZT22*X(2*N-1)**2
      PIJ(N-1,N-1)=PIJ(N-1,N-1)+IND1(N-1)*ZT2
      PIJ(N-1,N)=PIJ(N-1,N)-IND1(N-1)*IND1(N)*ZT2
      PIJ(N,N)=PIJ(N,N)+IND1(N)*ZT2
      ZT2=ZT2*X(2*N)
      PIJ(N-1,N)=PIJ(N-1,N)+ZT2*IND1(N-1)*IND1(N)
      PIJ(N,N)=PIJ(N,N)-2.*ZT2*IND1(N)
      ZT2=ZT2*X(2*N)
      PIJ(N,N)=PIJ(N,N)+IND1(N)*ZT2
      GO TO 1000
C
  100 ZT=PW1*(X(2*N)**2-1.)*IND1(N)
      PIJ(N,N)=PIJ(N,N)+ZT
 1000 IF (KMAX.EQ.0) GO TO 3
      IF (IM.GT.INDV(KMAX)) GO TO 3
      DO 12 K=1,KMAX
      I=INDV(K)
      ZA1=PW1
      ZA2=PW1
      X(IM-1)=0.
      IF (I-IM.LT.0) THEN 
        GO TO 12
      ELSE IF (I-IM.EQ.0) THEN 
        GO TO 140
      ENDIF
  230 I1=I-1
      DO 220 J=IM,I1
      U=X(N+J)
      ZA2=ZA2*X(N+J)
  220 CONTINUE
      ZA1=ZA2
      DO 14 J=IM,I1
      ZA1=ZA1/X(N+J)
      ZA2=ZA2*X(N+J)
      PIJ(J,I)=PIJ(J,I)+(X(I)-X(I-1))*(ZA1-ZA2)*IND1(J)
      IF (J.EQ.I1) GO TO 14
      PIJ(J+1,I)=PIJ(J+1,I)-(X(I)-X(I-1))*(ZA1-ZA2)*IND1(J+1)
   14 CONTINUE
  140 V=X(I)-X(I-1)
      IF (I.EQ.N) GO TO 12
      PIJ(I,I+1)=PIJ(I,I+1)+V*(PW1+ZA2)*IND1(I+1)
      I1=I+1
      DO15 J=I1,N
      ZA1=ZA1*X(N+J)
      ZA2=ZA2*X(N+J)
      PIJ(I,J)=PIJ(I,J)-V*(ZA1+ZA2)*IND1(J)
      IF(J.EQ.N)GO TO 15
      PIJ(I,J+1)=PIJ(I,J+1)+V*(ZA1+ZA2)*IND1(J+1)
   15 CONTINUE
   12 CONTINUE
    3 CONTINUE
    2 CONTINUE
      DO 1001 I=1,N
 1001 SIGT(I)=-SIGT(I)
      DO 17 I=1,N
      I1=I+1
      PIJ(I,I)=2./S(I)*PIJ(I,I)
      IF (IND(I).NE.0)PIJ(I,I)=1./SIGT(I)+1./SIGT(I)/SIGT(I)*PIJ(I,I)
      AUXI=1.
      IF (IND(I).NE.0) AUXI=1./SIGT(I)
      IF (I.EQ.N) GO TO 17
      DO 16 J=I1,N
      AUXJ=1.
      IF (IND(J).NE.0) AUXJ=1./SIGT(J)
      POIDS=AUXI*AUXJ
      PIJ(I,J)=2./S(I)*PIJ(I,J)*POIDS
      PIJ(J,I)=S(I)/S(J)*PIJ(I,J)
   16 CONTINUE
   17 CONTINUE
*
      DO 235 I=1,N
  235 S(I)=(R(I+1)-R(I))*(R(I+1)**2+R(I)*R(I+1)+R(I)*R(I))
      SURF=3.*R(N+1)**2
      DO 250 I=1,N
      PIJN(I)=0.
      DO 240 J=1,N
  240 PIJN(I)=PIJN(I)+PIJ(I,J)*SIGT(J)
  250 PIJN(I)=1.-PIJN(I)
      PSS=0.
      DO 260 I=1,N
  260 PSS=PSS+4.*PIJN(I)*S(I)*SIGT(I)/SURF
      PSS=1.-PSS
      DO 270 J=1,N
      PSJ=4./SURF*PIJN(J)*S(J)
      DO 270 I=1,N
  270 PIJ(I,J)=PIJ(I,J)+ALBG*PIJN(I)*PSJ/(1.-ALBG*PSS)
      RETURN
      END
