*DECK SYBEUR
      SUBROUTINE SYBEUR (MAXPTS,MAXCEL,MAXJ,MAXZ,IPGEOM,NREG,IR,MAT,
     1 VOL,ILK,IMPX,IHEX,NCOUR,LMAIL,NMCEL,NMERGE,NGEN,IJAT,MULTC,IWIGN,
     2 IHALT,ILIGN,INORM,IRECT,IQW,IQUAD,XX,YY,NMC,RAYRE,MAIL,IZMAIL,
     3 RZMAIL,IFR,ALB,INUM,MIX,DVX,IGEN,POURCE,IORI)
*
*-----------------------------------------------------------------------
*
* RECOVERY AND ANALYSIS OF THE GEOMETRY FOR THE DESCRIPTION OF A 2-D
* ASSEMBLY (EURYDICE-2) FOR THE INTERFACE CURRENT METHOD.
*
* GENERAL PARAMETERS:
*   MAXPTS : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION NREG.
*   MAXCEL : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION NMCEL, NMERGE OR
*            NGEN.
*   MAXJ   : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION IJATIJAS, IJAZ OR
*            IJAT.
*   MAXZ   : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION LMAIL.
*   IPGEOM : POINTER TO THE GEOMETRY (L_GEOM SIGNATURE).
*   NREG   : TOTAL NUMBER OF VOLUMES.
*   IR     : NUMBER OF MIXTURES.
*   MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*            DIMENSION MAT(NREG)
*   VOL    : VOLUMES. DIMENSION VOL(NREG)
*   ILK    : ILK=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
*            IS PRESENT.
*   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*   IHEX   : =0 : CARTESIAN LATTICE ; .NE.0 : HEXAGONAL LATTICE.
*   NCOUR  : NUMBER OF OUT-CURRENTS PER CELL.
*   LMAIL  : SPACE REQUIRED TO STORE THE TRACKING INFORMATION.
*
* TRACKING OPTIONS:
*   MULTC  : TYPE OF MULTICELL APPROXIMATION.
*   IWIGN  : =1/2/3: ASKEW/WIGNER/SANCHEZ CYLINDERIZATION.
*   IHALT  : =1 TO STOP AFTER CALCULATION OF THE MAX TRACKING STORAGE.
*   ILIGN  : =1 TO PRINT THE TRACKING.
*   INORM  : =1 TO AVOID TRACK NORMALIZATION.
*   IRECT  : =1 TO AVOID CONSIDERING SYMMETRIES OF SQUARE CELLS.
*   IQW    : =1 TO USE EQUAL WEIGHT QUADRATURES IN ANGLE AND SPACE.
*   IQUAD  : QUADRATURE PARAMETERS. DIMENSION IQUAD(4)
*
* PARAMETERS RELATED TO CELLS DEFINITION:
*   NMCEL  : TOTAL NUMBER OF CELLS IN THE DOMAIN.
*   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAS) WHERE
*            IJAS IS THE TOTAL NUMBER OF OUT-CURRENTS SURROUNDING ALL
*            THE CELLS.
*   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
*            DIMENSION ALB(IJAS)
*
* PARAMETERS RELATED TO THE MERGED CELLS DEFINITION:
*   NMERGE : TOTAL NUMBER OF MERGED CELLS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*            MANY CELLS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*            OWN THE SAME GENERATING CELL. THIS ALLOWS SOME REDUCTION
*            IN CPU TIME AND MEMORY. NMERGE.LE.NMCEL
*   IJAT   : TOTAL NUMBER OF DISTINCT OUT-CURRENTS.
*   INUM   : INDEX-NUMBER OF THE MERGED CELL ASSOCIATED TO EACH CELL.
*            DIMENSION INUM(NMCEL)
*   MIX    : INDEX-NUMBER OF OUT-CURRENTS. DIMENSION MIX(IJAZ) WHERE
*            IJAZ IS THE TOTAL NUMBER OF OUT-CURRENTS SURROUNDING ALL
*            THE MERGED CELLS.
*   DVX    : WEIGHT ASSOCIATED WITH EACH OUT-CURRENT.
*            DIMENSION DVX(IJAZ)
*
*            REMARK:
*            IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
*            GEOMETRICAL 'A' MATRIX.
*
* PARAMETERS RELATED TO THE GENERATING CELLS DEFINITION:
*   NGEN   : TOTAL NUMBER OF GENERATING CELLS. A GENERATING CELL IS
*            DEFINED BY ITS MIXTURE AND DIMENSIONS IRRESPECTIVE OF
*            ITS POSITION IN THE DOMAIN. NGEN.LE.NMERGE
*   XX,YY  : X AND Y-THICKNESS OF THE GENERATING CELLS.
*            DIMENSION XX(NGEN),YY(NGEN)
*   NMC    : OFFSET OF THE FIRST VOLUME IN EACH GENERATING CELL.
*            DIMENSION NMC(NGEN+1)
*   RAYRE  : RADIUS OF THE TUBES IN EACH GENERATING CELL.
*            DIMENSION RAYRE(NMC(NGEN+1))
*   MAIL   : OFFSET OF THE FIRST TRACKING INFORMATION IN EACH GENERATING
*            CELL. DIMENSION MAIL(NGEN)
*   IZMAIL : INTEGER TRACKING INFORMATION. DIMENSION IZMAIL(LMAIL)
*   RZMAIL : REAL TRACKING INFORMATION. DIMENSION RZMAIL(LMAIL)
*   IGEN   : INDEX-NUMBER OF THE GENERATING CELL ASSOCIATED WITH EACH
*            MERGED CELL. DIMENSION IGEN(NMERGE)
*
* SCRATCH STORAGE:
*   POURCE : DIMENSION POURCE(MAXCEL)
*   IORI   : DIMENSION IORI(MAXCEL)
*
*---------------------------------------- AUTHOR: A. HEBERT 22/05/87 ---
*
      PARAMETER (PI=3.141592654)
      LOGICAL ILK,LHEX
      CHARACTER HSMG*131,HDOM*8
      DIMENSION MAT(*),VOL(*),IQUAD(*),XX(*),YY(*),NMC(*),RAYRE(*),
     1 MAIL(*),IZMAIL(*),RZMAIL(*),IFR(*),ALB(*),INUM(*),MIX(*),DVX(*),
     2 IGEN(*),POURCE(*),IORI(*),NCODE(6),ZCODE(6),IIT(15),ISMR(6,8),
     3 ISMH(8,12)
      COMMON BASE(1)
      SAVE ISMR,ISMH,IIT
      DATA ISMR/1,2,3,4,0,0,3,4,2,1,0,0,2,1,4,3,0,0,4,3,1,2,0,0,
     1          2,1,3,4,0,0,3,4,1,2,0,0,1,2,4,3,0,0,4,3,2,1,0,0/
      DATA ISMH/1,2,3,4,5,6,0,0,6,1,2,3,4,5,0,0,5,6,1,2,3,4,0,0,
     1          4,5,6,1,2,3,0,0,3,4,5,6,1,2,0,0,2,3,4,5,6,1,0,0,
     2          1,6,5,4,3,2,0,0,2,1,6,5,4,3,0,0,3,2,1,6,5,4,0,0,
     3          4,3,2,1,6,5,0,0,5,4,3,2,1,6,0,0,6,5,4,3,2,1,0,0/
      DATA IIT/1,2,3,4,5,6,7,8,9,10,12,16,20,32,64/
*
      CALL SETARA(BASE,MAXPTS,IMIL)
      CALL SETARA(BASE,MAXPTS,INBREG)
      CALL SETARA(BASE,MAXPTS,IMILIE)
      CALL SETARA(BASE,MAXPTS,IRAYON)
      CALL SETARA(BASE,3*MAXPTS,ICELL)
      CALL SETARA(BASE,MAXPTS+1,IXXX)
      CALL SETARA(BASE,MAXPTS+1,IYYY)
      CALL READEU(MAXPTS,IPGEOM,IR,MAT,ILK,NMCEL,NMERGE,NGEN,INUM,
     1 IGEN,NREG,LX,LY,XX,YY,RAYRE,NMC,IORI,NCODE,ZCODE,IHEX,IMPX,
     2 BASE(IMIL),BASE(INBREG),BASE(IMILIE),BASE(IRAYON),BASE(ICELL),
     3 BASE(IXXX),BASE(IYYY))
      LHEX=IHEX.NE.0
      CALL RLSARA(BASE(IYYY))
      CALL RLSARA(BASE(IXXX))
      CALL RLSARA(BASE(ICELL))
      CALL RLSARA(BASE(IRAYON))
      CALL RLSARA(BASE(IMILIE))
      CALL RLSARA(BASE(INBREG))
      CALL RLSARA(BASE(IMIL))
      IF (.NOT.LHEX) THEN
         A=0.0
         B=0.0
         DO 50 IKG=1,NGEN
         IF ((A.NE.0.0).AND.(A.NE.XX(IKG)).AND.(IWIGN.EQ.2).AND.(MULTC
     1   .LE.2)) CALL XABORT('SYBEUR: THE OPTION ASKE IS REQUIRED.')
         A=XX(IKG)
         IF ((B.NE.0.0).AND.(B.NE.YY(IKG)).AND.(IWIGN.EQ.2).AND.(MULTC
     1   .LE.2)) CALL XABORT('SYBEUR: THE OPTION ASKE IS REQUIRED.')
50       B=YY(IKG)
      ENDIF
*
      IF (MULTC.EQ.1) THEN
         IF (IMPX.GE.1) WRITE (6,'(/20H ROTH APPROXIMATION.)')
         IJAS=NMCEL
      ELSE IF (LHEX.AND.(MULTC.EQ.2)) THEN
         IF (IMPX.GE.1) WRITE (6,'(/24H ROTH X 6 APPROXIMATION.)')
         IJAS=6*NMCEL
      ELSE IF (MULTC.EQ.2) THEN
         IF (IMPX.GE.1) WRITE (6,'(/24H ROTH X 4 APPROXIMATION.)')
         IJAS=4*NMCEL
      ELSE IF (LHEX.AND.(MULTC.EQ.3)) THEN
         IF (IMPX.GE.1) WRITE (6,'(/27H DP-0 APPROXIMATION AROUND ,
     1   13HEACH HEXAGON.)')
         IJAS=6*NMCEL
      ELSE IF (MULTC.EQ.3) THEN
         IF (IMPX.GE.1) WRITE (6,'(/27H DP-0 APPROXIMATION AROUND ,
     1   20HEACH CARTESIAN CELL.)')
         IJAS=4*NMCEL
      ELSE IF (LHEX.AND.(MULTC.EQ.4)) THEN
         IF (IMPX.GE.1) WRITE (6,'(/27H DP-1 APPROXIMATION AROUND ,
     1   13HEACH HEXAGON.)')
         IJAS=18*NMCEL
      ELSE IF (MULTC.EQ.4) THEN
         IF (IMPX.GE.1) WRITE (6,'(/27H DP-1 APPROXIMATION AROUND ,
     1   20HEACH CARTESIAN CELL.)')
         IJAS=12*NMCEL
      ENDIF
      IF (IJAS.GT.MAXJ) THEN
         WRITE (HSMG,660) IJAS
         CALL XABORT(HSMG)
      ENDIF
      IF (IMPX.GE.1) THEN
         IF ((MULTC.LE.2).AND.(IWIGN.EQ.1)) THEN
            WRITE (6,'(23H ASKEW CYLINDERIZATION./)')
         ELSE IF ((MULTC.LE.2).AND.(IWIGN.EQ.2)) THEN
            WRITE (6,'(24H WIGNER CYLINDERIZATION./)')
         ELSE IF ((MULTC.LE.2).AND.(IWIGN.EQ.3)) THEN
            WRITE (6,'(25H SANCHEZ CYLINDERIZATION./)')
         ENDIF
         IF (INORM.EQ.1) WRITE (6,670)
         IF (IRECT.EQ.1) WRITE (6,'(/30H SYMMETRIES OF SQUARE CELLS AR,
     1   17HE NOT CONSIDERED.)')
         IF (IQW.EQ.1) WRITE (6,'(/32H USE EQUAL WEIGHT QUADRATURES IN,
     1   17H ANGLE AND SPACE.)')
         HDOM='(0,PI/4)'
         IF (LHEX) HDOM='(0,PI/6)'
         WRITE (6,690) IQUAD(1),HDOM,IQUAD(2),IQUAD(3),IQUAD(4)
      ENDIF
      DO 100 IQQ=1,4
      DO 60 I=1,15
      IF (IQUAD(IQQ).EQ.IIT(I)) GO TO 100
60    CONTINUE
      CALL XABORT('SYBEUR: INVALID QUADRATURE PARAMETER.')
100   CONTINUE
*
      NCODE(5)=0
      NCODE(6)=0
      CALL SETARA(BASE,NGEN,IZZ)
      DO 120 IKG=1,NGEN
120   BASE(IZZ+IKG-1)=1.0
      IF (LHEX) THEN
         NCOUR=6
         CALL SETARA(BASE,8*NMCEL,IJF2)
         CALL SETARA(BASE,NMCEL,IGG3)
         CALL NUMERH(NCOUR,MULTC,NCODE,ZCODE,IHEX,LX,1,IORI,ISMH,POURCE,
     1   IMPX,NMCEL,IFR,ALB,NMERGE,INUM,MIX,DVX,IGEN,XX,BASE(IZZ),
     2   BASE(IJF2),BASE(IGG3))
      ELSE
         NCOUR=4
         CALL SETARA(BASE,6*NMCEL,IJF2)
         CALL SETARA(BASE,NMCEL,IGG3)
         CALL NUMER3(NCOUR,MULTC,NCODE,ZCODE,LX,LY,1,IORI,ISMR,POURCE,
     1   IMPX,NMCEL,IFR,ALB,NMERGE,INUM,MIX,DVX,IGEN,XX,YY,BASE(IZZ),
     2   BASE(IJF2),BASE(IGG3))
      ENDIF
      CALL RLSARA(BASE(IGG3))
      CALL RLSARA(BASE(IJF2))
      CALL RLSARA(BASE(IZZ))
*
* COMPUTE THE NUMBER OF DISTINCT OUT CURRENTS BETWEEN THE CELLS.
      IJAT=0
      DO 130 I=1,NCOUR*NMERGE
130   IJAT=MAX(IJAT,MIX(I))
      IF (IMPX.GE.1) WRITE (6,760) NMCEL,NMERGE,NGEN,MAXCEL
      IF (NMCEL.GT.MAXCEL) CALL XABORT('SYBEUR: INSUFFICIENT MAXCEL.')
      IF (IMPX.GE.1) WRITE (6,770) IJAT,IJAS,MAXJ
      IF (IJAT.GT.MAXJ) CALL XABORT('SYBEUR: INSUFFICIENT MAXJ.')
*
* ESTIMATION OF THE MEMORY REQUIRED TO STORE THE TRACKING INFORMATION.
      NA=IQUAD(1)
      IF (.NOT.LHEX) NA=2*NA
      NX=IQUAD(2)
      LMAIL=0
      IF (MULTC.GE.3) THEN
         DO 150 IKG=1,NGEN
         J2=NMC(IKG+1)-NMC(IKG)
         IF (J2.EQ.1) GO TO 150
         IF (LHEX) THEN
            LMAIL=LMAIL+6+NA*(3*J2+5+NX*(9+J2*3*(J2+1)))
         ELSE IF ((XX(IKG).EQ.YY(IKG)).AND.(IRECT.NE.1)) THEN
            LMAIL=LMAIL+6+NA*(5+NX*(J2+1)*(2*J2+1))
         ELSE
            LMAIL=LMAIL+6+NA*(7+NX*(2*J2+1)*(2*J2+1))
         ENDIF
150      CONTINUE
         IF ((IMPX.GE.1).OR.(LMAIL.GT.MAXZ)) WRITE (6,680) LMAIL,MAXZ
         IF (LMAIL.GT.MAXZ) WRITE (6,'(/27H SYBEUR: *** WARNING *** MA,
     1   46HXZ IS LESS THAN THE ESTIMATED TRACKING LENGTH.)')
         DO 155 I=1,LMAIL
         IZMAIL(I)=0
155      RZMAIL(I)=0.0
      ENDIF
*
* CALCULATION OF THE VOLUMES (WEIGHTED BY POURCE).
      I1=0
      DO 170 IKK=1,NMERGE
      IKG=IGEN(IKK)
      J1=NMC(IKG)
      I2=NMC(IKG+1)-J1
      RJ=0.0
      DO 160 I=1,I2-1
      RJ1=RAYRE(J1+I+1)**2
      VOL(I1+I)=PI*(RJ1-RJ)*POURCE(IKK)
160   RJ=RJ1
      IF (LHEX) THEN
         VOL(I1+I2)=(1.5*SQRT(3.0)*XX(IKG)**2-PI*RJ)*POURCE(IKK)
      ELSE
         VOL(I1+I2)=(XX(IKG)*YY(IKG)-PI*RJ)*POURCE(IKK)
      ENDIF
170   I1=I1+I2
      IF (I1.NE.NREG) CALL XABORT('SYBEUR: WRONG NUMBER OF REGIONS.')
      IF (IMPX.GE.1) WRITE (6,780) NREG,MAXPTS
      IF (NREG.GT.MAXPTS) CALL XABORT('SYBEUR: INSUFFICIENT MAXPTS.')
*
* COMPUTE THE TRACKING INFORMATION.
      IF (IHALT.EQ.1) CALL XABORT('SYBEUR: STOP REQUESTED.')
      IF (MULTC.GE.3) THEN
         LMAIL=0
         PRECC=0.0
         DO 180 IKG=1,NGEN
         MAIL(IKG)=0
         J1=NMC(IKG)
         J2=NMC(IKG+1)-J1
         IF (J2.EQ.1) GO TO 180
         IF (LHEX) THEN
           LZI=3*NA+2
           JJ2=3*((J2*(J2+1))/2)+J2
           CALL SETARA(BASE,JJ2,IVAP)
           IRR=(J2-1)**2
           CALL SETARA(BASE,2*IRR,IAR)
           CALL SYBHTK(NA,NX,J2,RAYRE(J1+2),RZMAIL(LMAIL+LZI+1),
     1     IZMAIL(LMAIL+1),INORM,PREC,ILIGN,IQW,XX(IKG),LL,BASE(IVAP),
     2     BASE(J2+IVAP),BASE(IAR),BASE(IAR+IRR))
           CALL RLSARA(BASE(IVAP))
           CALL RLSARA(BASE(IAR))
           L=LL+LZI
         ELSE
           CALL SETARA(BASE,J2,IVAP)
           CALL SYBRTK(NA,NX,J2,XX(IKG),YY(IKG),RAYRE(J1+1),ILIGN,INORM,
     1     IRECT,IQW,L,RZMAIL(LMAIL+1),IZMAIL(LMAIL+1),PREC,BASE(IVAP))
           CALL RLSARA(BASE(IVAP))
         ENDIF
         PRECC=AMAX1(PRECC,PREC)
         MAIL(IKG)=LMAIL+1
         LMAIL=LMAIL+L
180      CONTINUE
         IF (IMPX.GE.1) WRITE (6,710) LMAIL,PRECC
         IF (LMAIL.GT.MAXZ) CALL XABORT('SYBEUR: INSUFFICIENT MAXZ.')
      ENDIF
      RETURN
*
660   FORMAT (42HSYBEUR: MAXJ SHOULD BE GREATER OR EQUAL TO,I5,1H.)
670   FORMAT (52H THE TRACKS ARE NOT NORMALIZED AND THE GEOMETRIC ACC,
     1 43HURACY OF THE TRACKING MESH IS NOT COMPUTED./)
680   FORMAT (/41H MAXIMUM STORAGE REQUIRED BY THE TRACKS =,I7,8X,
     1 24HAVAILABLE STORAGE MAXZ =,I7/)
690   FORMAT (/23H QUADRATURE PARAMETERS:/24H HETEROGENEOUS CELLS   -,
     1 17H    ANGLES   NA =,I3,4H IN ,A8/24X,17H  SEGMENTS   NX =,
     2 I3//41H HOMOGENEOUS CELLS     -    ANGLES   NH =,I3//
     3 41H CYLINDRICAL CELLS     -  SEGMENTS   MR =,I3//)
710   FORMAT (/39H EXACT STORAGE REQUIRED BY THE TRACKS =,I7,8X,
     1 10HACCURACY =,1P,E11.3/)
760   FORMAT (/38H TOTAL NUMBER OF CELLS IN THE DOMAIN =,I5/
     1         31H TOTAL NUMBER OF MERGED CELLS =,I5/
     2         35H TOTAL NUMBER OF GENERATING CELLS =,I5,5X,
     3         26HAVAILABLE STORAGE MAXCEL =,I5/)
770   FORMAT (/40H TOTAL NUMBER OF DISTINCT OUT-CURRENTS =,I5/
     1 57H TOTAL NUMBER OF OUT-CURRENTS SURROUNDING ALL THE CELLS =,I5,
     2 5X,24HAVAILABLE STORAGE MAXJ =,I5/)
780   FORMAT (/20H NUMBER OF VOLUMES =,I5,5X,22HAVAILABLE STORAGE MAXB,
     1 4HLK =,I5/)
      END
