*DECK SYBHTK
      SUBROUTINE SYBHTK(NA,NX,NR,R,Z,ZI,INORM,PREC,ILIGN,IQW,COTE,L,
     +                  R2,VOL,RCO,RSQ)
C
C---------------------------  SYBHTK  ---------------------------------
C
C   1-  PROGRAMME STATISTICS:
C
C          FUNCTION   :  COMPUTE THE TRACKING INFORMATION RELATED TO A
C                        HEXAGONAL HETEROGENEOUS CELL IN DP1 CASE.
C          REVISED    :  01-11-1990
C          AUTHOR     :  M.OUISLOUMEN
C
C   2-  PROGRAMME PARAMETERS:
C
C INPUT PARAMETERS.
C   NA      : NUMBER OF ANGLES IN (0,PI/6)               I
C   NX      : NUMBER OF TRACKS IN EACH SUB DOMAIN FOR A GIVEN ANGLE  I
C   NR      : NUMBER OF REGIONS IN THE CELL              I
C   SIGT    : TOTAL CROSS SECTION.                       R
C   R       : RADIUS OF CYLINDERS
C   COTE    : LENGHT OF ONE OF THE HEXAGONE
C   ILIGN   : IF ILIGN=1 THE TRACKING INFORMATION IS PRINTED
C   IQW     : IF IQW=1 EQUAL WEIGHT QUADRATURE IS USED.
C
C OUTPUT PARAMETERS.
C   Z,ZI    : INTEGRATION MESH.                          R,I
C             THE MAXIMUM DIMENSION OF Z IS
C                  NA*(3*NR+2+NX*(NR*(3*NR+3)+9))+4
C             THE MAXIMUM DIMENSION OF ZI IS
C                  3*NA+2
C             Z(1) TO Z(4) CONTAINS THE ORTHONORMALIZATION FACTORS
C   PREC    : ACCURACY OBTAINED IF THE NORMALIZED TRACKS ARE USED TO
C             INTEGRATE THE VOLUMES.
C   L       : EXACT DIMENSION OF Z
C
C SCRATCH STORAGE
C   R2      : DIMENSION NR
C   VOL     : DIMENSION 3*NR*(NR+1)/2
C   RCO     : DIMENSION (NR-1)**2
C   RSQ     : DIMENSION (NR-1)**2
C
C                        SIDE 2
C                       XXXXXXXX
C                      X        X
C             SIDE 3  X          X  SIDE 1
C                    X            X
C                     X          X
C             SIDE 4   X        X SIDE 6
C                       XXXXXXXX
C                        SIDE 5
C
C
C---------------------------   SYBHTK  --------------------------------
C
      PARAMETER (IOUT=6,PI=3.141592653589793,PI12=PI/12.,
     +           SQRT3=1.732050807568877)
C
      REAL       WA(64),ZA(64),WX(64),ZX(64),R(NR),Z(*),R2(NR)
      REAL       VOL(3,NR*(NR+1)/2),RCO(NR-1,NR-1),RSQ(NR-1,NR-1)
      INTEGER    ZI(*)
      LOGICAL    LGZON6,LGZON5,LGG,LGNORM
C
      IF (NX.GT.64) CALL XABORT('SYBHTK: NX IS GREATER THAN 64.')
      IF (NA.GT.64) CALL XABORT('SYBHTK: NA IS GREATER THAN 64.')
      IF (2.0*R(NR-1).GT.SQRT3*COTE) THEN
        WRITE(IOUT,9000) NR-1,R(NR-1),COTE,COTE*SQRT3/2.0
        CALL XABORT('SYBHTK: A RADIUS > SQRT(3.0)*SIDE/2.')
      ENDIF
      IF (IQW.EQ.0) THEN
C        GAUSS-LEGENDRE INTEGRATION POINTS.
         CALL ALGPT(NX,-1.,1.,ZX,WX)
         CALL ALGPT(NA,-1.,1.,ZA,WA)
      ELSE
C        EQUAL WEIGHT INTEGRATION POINTS.
         DO 51 I=1,NX
         ZX(I)=(2.0*REAL(I)-1.0)/REAL(NX)-1.0
   51    WX(I)=2.0/REAL(NX)
         DO 61 I=1,NA
         ZA(I)=(2.0*REAL(I)-1.0)/REAL(NA)-1.0
   61    WA(I)=2.0/REAL(NA)
      ENDIF
C
      LGNORM=INORM.EQ.0
      A2=COTE*.5
      DO 1 I=1,NR-1
       R2(I)=R(I)**2
    1 CONTINUE
      R2(NR)=COTE*COTE
      PREC=0.
      IZ1=2
      ZI(1)=NA
      ZI(2)=NX
      IZ=4
      NVOL=(NR*(NR+1))/2
      QAUX=COTE*SQRT3
      IF(LGNORM) THEN
       V4=.5*R2(NR)*SQRT3
       SUM4=.5*R2(1)*PI
       AIRI=1./(3.*V4)
       DO 33 I=1,NR-1
       DO 33 J=I,NR-1
          RCO(I,J)=R2(J)*ACOS(R(I)/R(J))
          RSQ(I,J)=R(I)*SQRT(R2(J)-R2(I))
   33  CONTINUE
      ENDIF
      ORTH1=0.
      ORTH2=0.
      ORTH3=0.
C
C       ANGULAR INTEGRATION
C
          DO 150 IA=1,NA
C
      FI=PI12*(1.+ZA(IA))
      SINFI=SIN(FI)
      COSFI=COS(FI)
      AUX=SQRT3*COSFI
      AUY= SQRT3*SINFI
      COSA=AUX-SINFI
      SINA=AUY+COSFI
      COSB=AUX+SINFI
      SINB=COSFI-AUY
      W611=SINA*COSFI
      W511=COSA*SINFI
      SIN2=SINFI*SINFI
C
C       ORTHONORMALIZATION FACTORS
C
      ORTH1=ORTH1+WA(IA)*(SINFI+.5*COSB*SQRT3)
      ORTH2=ORTH2+WA(IA)*(1.+.25*SINA*SINA)
      ORTH3=ORTH3+WA(IA)*(SIN2*SINFI+.125*SINA*SINA*SINA
     +                    +COSFI*COSFI*COSFI)
C
C       FOR SURFACE 6:  DMIN6<X<A*SINFI; FOR SURFACE 5:  DMIN5<X<DMAX5
C
      DMIN6=A2*COSA
      DMAX5=A2*SINA
      DMIN5=A2*SINB
      DX0=A2*COSB
C
C     INITIALIZE VOLUME PER ANGLE
C
      DO 101 I=1,NVOL
       VOL(1,I)=0.
       VOL(2,I)=0.
       VOL(3,I)=0.
 101  CONTINUE
C
C        NUMBER OF REGION CROSSED FOR DIFFERENT EXIT SURFACES
C        SURFACE 6
C
      LGZON6=.FALSE.
      DO 2 IM=1,NR-1
       IF(R(IM).GT.DMIN6) THEN
       IMIN6=IM
       NR6=NR-IMIN6+1
       GOTO 3
       ENDIF
    2 CONTINUE
      LGZON6=.TRUE.
      IMIN6=NR
      NR6=1
    3 IZ1=IZ1+1
      ZI(IZ1)=IMIN6
C
C       SURFACES 4 ET 5
C
      LGZON5=.FALSE.
      DO 4 IM=1,NR-1
       IF(R(IM).GT.DMIN5) THEN
       IMIN=IM
       GOTO 5
       ENDIF
    4 CONTINUE
      IMIN=NR
      LGZON5=.TRUE.
      IMAX=NR
      GOTO 8
    5 DO 6 IM=IMIN,NR-1
       IF(R(IM).GT.DMAX5) THEN
       IMAX=IM
       GOTO 7
       ENDIF
    6 CONTINUE
      IMAX=NR
    7 IF(IMIN.EQ.IMAX) LGZON5=.TRUE.
    8 IZ1=IZ1+2
      ZI(IZ1-1)=IMIN
      NR5=IMAX-IMIN+1
      ZI(IZ1)=NR5
C
C       INTEGRATION WHEIGHTS FOR SURFACES 6,5 ET 4 RESPECTIVELY
C       LGZON6=.TRUE. => NO CYLINDER CROSSED GOING TO SURFACE 6
C              IDEM FOR SURFACES 5 AND 4
C
      IZ=IZ+1
      ITAB6=IZ
      IF(LGZON6)THEN
       Z(IZ)=-DMIN6+DX0
      ELSE
       Z(IZ)=-R(NR-1)+DX0
       IZ=IZ+1
       DO 10 I=NR-1,IMIN6+1,-1
       Z(IZ)=R(I)-R(I-1)
       IZ=IZ+1
   10  CONTINUE
       Z(IZ)=R(IMIN6)-DMIN6
      ENDIF
C
C       SURFACE 5
C
      IZ=IZ+1
      ITAB5=IZ
      IF(LGZON5)THEN
       Z(IZ)=DMAX5-DMIN5
      ELSE
       Z(IZ)=DMAX5-R(IMAX-1)
       IZ=IZ+1
       DO 12 I=IMAX-1,IMIN+1,-1
       Z(IZ)=R(I)-R(I-1)
       IZ=IZ+1
   12  CONTINUE
       Z(IZ)=R(IMIN)-DMIN5
      ENDIF
C
C       SURFACE 4.
C       FOR SYMMETRY REASONS, STORE ONLY HALF THE WEIGHTS.
C
      IZ=IZ+1
      ITAB4=IZ
      IF(IMIN.EQ.1) THEN
       Z(IZ)=2.*DMIN5
      ELSE
       Z(IZ)=2.*(DMIN5-R(IMIN-1))
       IZ=IZ+IMIN-1
       IZ4=IZ
       ZAUX=0.
       DO 14 I=1,IMIN-1
       Z(IZ4)=2.*(R(I)-ZAUX)
       ZAUX=R(I)
       IZ4=IZ4-1
   14  CONTINUE
      ENDIF
C
C      INTEGRATION OVER THE SPACE VARIABLE.
C
      RAUX=SQRT3/(COSA*SINFI)
      TG=SINFI/COSFI
      TAUX=DMIN6+COTE*TG*AUY
      R04=QAUX/COSFI
      W1=WA(IA)
      SQRS=SQRT3/SINA
      R00=SQRS*COTE
      SQRS=SQRS/COSFI
      A2SIN=A2*SINFI
C
C       INTEGRATION WEIGHTS W600,W610,W611,W511,W420,W411
C
           DO 100 IX=1,NX
C
      ZXX=ZX(IX)+1.
      WW=W1*WX(IX)
C
C        DP0 CASE
C
      IZ=IZ+9
      Z(IZ)=WW
C
C         DP1 CASE
C
      Z(IZ-1)=WW*SIN2
      Z(IZ-2)=WW*COSFI
      Z(IZ-3)=WW*W511
      Z(IZ-4)=WW*W611
      Z(IZ-5)=WW*COSB
C
C        JPM CASE
C
      WW5=WW*.5
      Z(IZ-6)=WW*SINFI
      Z(IZ-7)=WW5*SINA
      Z(IZ-8)=WW5*COSA
C
C      LOCATE CYLINDRICAL REGIONS CROSSED WHEN TRYING TO REACH
C         SURFACE 6
C
      X1=0.
      ITT6=ITAB6
      IXX=0
C
      DO 16 I=NR,IMIN6+1,-1
C
C       LOCATE MOST INTERNAL CYLINDER CROSSED BY LINE
C
       X2=DX0-R(I-1)
       X=.5*(X2-X1)*ZXX+X1
       D=DX0-X
       D2=D*D
       R0=X*RAUX
       AUX=0.
       IZ=IZ+1
       LGG=.FALSE.
       WVOL=Z(ITT6)*WX(IX)
       ITT6=ITT6+1
C
C      MEASURE DISTANCE TRAVEL IN EACH CYLINDER
C
       DO 15 J=I,NR-1
        RJ=2.*SQRT(R2(J)-D2)
        RR=RJ-AUX
        Z(IZ)=RR
        IXX=IXX+1
        VOL(1,IXX)=VOL(1,IXX)+WVOL*RR
        AUX=RJ
        LGG=.TRUE.
        IZ=IZ+1
   15  CONTINUE
C
C       MEASURE DISTANCE TRAVELLED OUTSIDE CYLINDERS
C
       IZ=IZ+1
       IXX=IXX+1
       IF(LGG) THEN
        RR=R0-AUX
        Z(IZ-1)=RR
        Z(IZ)=-DMIN5+X/TG-.5*AUX
        VOL(1,IXX)=VOL(1,IXX)+WVOL*RR
       ELSE
        Z(IZ-1)=R0
        Z(IZ)=R0
        VOL(1,IXX)=VOL(1,IXX)+WVOL*R0
       ENDIF
       X1=X2
   16 CONTINUE
C
C       TREATMENT OF LAST REGION OR HOMOGENEOUS REGION
C
      WVOL=Z(ITT6)*WX(IX)
      X=(A2SIN-.5*X1)*ZXX+X1
      D=DX0-X
      D2=D*D
      R0=X*RAUX
      AUX=0.
      IZ=IZ+1
      LGG=.FALSE.
C
      DO 17 J=IMIN6,NR-1
       RJ=2.*SQRT(R2(J)-D2)
       RR=RJ-AUX
       Z(IZ)=RR
       IXX=IXX+1
       VOL(1,IXX)=VOL(1,IXX)+WVOL*RR
       AUX=RJ
       LGG=.TRUE.
       IZ=IZ+1
   17 CONTINUE
      IZ=IZ+1
      IXX=IXX+1
      INOR6=IXX
      IF(LGG) THEN
       RR=R0-AUX
       Z(IZ-1)=RR
       Z(IZ)=-DMIN5+X/TG-.5*AUX
       VOL(1,IXX)=VOL(1,IXX)+WVOL*RR
      ELSE
       Z(IZ-1)=R0
       Z(IZ)=R0
       VOL(1,IXX)=VOL(1,IXX)+WVOL*R0
      ENDIF
C
C      LOCATE CYLINDRICAL REGIONS CROSSED WHEN TRYING TO REACH
C       SURFACE 5
C
      X1=0.
      ITT5=ITAB5
      IXX=0
C
C       LOCATE MOST INTERNAL CYLINDER CROSSED BY LINE
C
      DO 19 I=IMAX-1,IMIN,-1
       WVOL=Z(ITT5)*WX(IX)
       ITT5=ITT5+1
       X2=DMAX5-R(I)
       X=.5*(X2-X1)*ZXX+X1
       D=DMAX5-X
       D2=D*D
       R0=R00+X*SQRS
       AUX=0.
       IZ=IZ+1
       LGG=.FALSE.
C
C      MEASURE DISTANCE TRAVEL IN EACH CYLINDER
C
       DO 18 J=I+1,NR-1
        RJ=2.*SQRT(R2(J)-D2)
        RR=RJ-AUX
        Z(IZ)=RR
        IXX=IXX+1
        VOL(2,IXX)=VOL(2,IXX)+WVOL*RR
        IZ=IZ+1
        LGG=.TRUE.
        AUX=RJ
   18  CONTINUE
C
C       MEASURE DISTANCE TRAVELLED OUTSIDE CYLINDERS
C
       IZ=IZ+1
       IXX=IXX+1
       IF(LGG) THEN
        RR=R0-AUX
        Z(IZ-1)=RR
        Z(IZ)=DMIN6+X*TG-.5*AUX
        VOL(2,IXX)=VOL(2,IXX)+WVOL*RR
       ELSE
        Z(IZ-1)=R0
        Z(IZ)=R0
        VOL(2,IXX)=VOL(2,IXX)+WVOL*R0
       ENDIF
       X1=X2
   19 CONTINUE
C
C       TREATMENT OF LAST REGION OR HOMOGENEOUS REGION
C
      WVOL=Z(ITT5)*WX(IX)
      ITT5=ITT5+1
      X=(A2SIN*SQRT3-.5*X1)*ZXX+X1
      D=DMAX5-X
      D2=D*D
      R0=R00+X*SQRS
      AUX=0.
      IZ=IZ+1
      LGG=.FALSE.
      DO 20 J=IMIN,NR-1
       RJ=2.*SQRT(R2(J)-D2)
       RR=RJ-AUX
       Z(IZ)=RR
       IXX=IXX+1
       VOL(2,IXX)=VOL(2,IXX)+WVOL*RR
       IZ=IZ+1
       LGG=.TRUE.
       AUX=RJ
   20 CONTINUE
      IZ=IZ+1
      IXX=IXX+1
      INOR5=IXX
      IF(LGG) THEN
       RR=R0-AUX
       Z(IZ-1)=RR
       Z(IZ)=DMIN6+X*TG-.5*AUX
       VOL(2,IXX)=VOL(2,IXX)+WVOL*RR
      ELSE
       Z(IZ-1)=R0
       Z(IZ)=R0
       VOL(2,IXX)=VOL(2,IXX)+WVOL*R0
      ENDIF
C
C      LOCATE CYLINDRICAL REGIONS CROSSED WHEN TRYING TO REACH
C       SURFACE 4
C
      X1=0.
      ITT4=ITAB4
      IXX=0
      DO 22 I=IMIN-1,1,-1
C
C       LOCATE MOST INTERNAL CYLINDER CROSSED BY LINE
C
       WVOL=Z(ITT4)*.5*WX(IX)
       ITT4=ITT4+1
       X2=DMIN5-R(I)
       X=.5*(X2-X1)*ZXX+X1
       D=DMIN5-X
       D2=D*D
       AUX=0.
       IZ=IZ+1
       LGG=.FALSE.
C
C      MEASURE DISTANCE TRAVEL IN EACH CYLINDER
C
       DO 21 J=I+1,NR-1
        RJ=2.*SQRT(R2(J)-D2)
        RR=RJ-AUX
        Z(IZ)=RR
        IXX=IXX+1
        VOL(3,IXX)=VOL(3,IXX)+WVOL*RR
        IZ=IZ+1
        LGG=.TRUE.
        AUX=RJ
   21  CONTINUE
C
C       MEASURE DISTANCE TRAVELLED OUTSIDE CYLINDERS
C
       IZ=IZ+1
       IXX=IXX+1
       IF(LGG) THEN
        RR=R04-AUX
        Z(IZ-1)=RR
        Z(IZ)=TAUX+X*TG-.5*AUX
        VOL(3,IXX)=VOL(3,IXX)+WVOL*RR
       ELSE
        Z(IZ-1)=R04
        Z(IZ)=R04
        VOL(3,IXX)=VOL(3,IXX)+WVOL*R04
       ENDIF
       X1=X2
   22 CONTINUE
C
C       TREATMENT OF CENTRAL CYLINDER
C
      WVOL=Z(ITT4)*.5*WX(IX)
      X=.5*(DMIN5-X1)*ZXX+X1
      D=DMIN5-X
      D2=D*D
      AUX=0.
      IZ=IZ+1
      LGG=.FALSE.
      DO 23 J=1,NR-1
       RJ=2.*SQRT(R2(J)-D2)
       RR=RJ-AUX
       Z(IZ)=RR
       IXX=IXX+1
       VOL(3,IXX)=VOL(3,IXX)+WVOL*RR
       IZ=IZ+1
       LGG=.TRUE.
       AUX=RJ
   23 CONTINUE
      IZ=IZ+1
      IXX=IXX+1
      IVOL=IZ
      INOR4=IXX
      IF(LGG) THEN
       RR=R04-AUX
       Z(IZ-1)=RR
       Z(IZ)=TAUX+X*TG-.5*AUX
       VOL(3,IXX)=VOL(3,IXX)+WVOL*RR
      ELSE
       Z(IZ-1)=R04
       Z(IZ)=R04
       VOL(3,IXX)=VOL(3,IXX)+WVOL*R04
      ENDIF
C
  100      CONTINUE
C
            IF(LGNORM) THEN
C
C       NORMALIZATION OF THE TRACKS
C
      XM4=-DMIN5+R(1)
      VAUX4=R04*(SQRT3*A2SIN-XM4)
      VV4=1.5*R2(NR)*TG
      V5=(VV4+.5*V4)
      A1=R00*SINFI
      B=A2-A1
      VAUX5=R00*A1*.5*SINB+SQRT3*B*B
      V6=V4*SINFI/COSA
      XMM=DX0
      VOL45=A2*(COSA+TG*SINA)
      VOL6=A2*(-SINB+COSB/TG)
C
C       COMPARE EXACT VOLUME FOR EACH ANGLE WITH COMPUTED ONE
C       FOR SURFACE 4
C
         DO 160 I=1,NX
C
      V=V4
      INOR=INOR4
      DM=0.
         IF(IMIN.GT.1) THEN
      IVOL=IVOL-NR-1
      INOR=INOR-NR
      IV=IVOL+1
      IN=INOR+1
      SUM=SUM4
      SUMA=.5*VOL(3,IN)
      RAP=SUM/SUMA
      PREC=MAX(PREC,ABS(1.-RAP)*SUM*AIRI)
      Z(IV)=Z(IV)*RAP
      DO 140 L=2,NR-1
       AUX=.5*R2(L)*PI-RCO(1,L)+RSQ(1,L)-SUM
       SUM=SUM+AUX
       IV=IV+1
       IN=IN+1
       SUMA=SUMA+.5*VOL(3,IN)
       RAP=AUX/(.5*VOL(3,IN))
       PREC=MAX(PREC,ABS(1.-RAP)*AIRI*AUX)
       Z(IV)=Z(IV)*RAP
 140  CONTINUE
      XM=XM4
      VAUX=VAUX4
      IV=IV+2
      IN=IN+1
      VTEX=R(1)*(VOL45-.5*TG*R(1))
      FF1=.5*SUM
      FF2=.5*SUMA
      F1=(VTEX-FF1)/(VTEX-FF2)
      F2=(V-VAUX-VTEX-FF1)/(V-VAUX-VTEX-FF2)
      PREC=MAX(PREC,ABS(1.-F1)*(VTEX-FF1)*AIRI)
      PREC=MAX(PREC,ABS(1.-F2)*(V-VAUX-VTEX-FF1)*AIRI)
      Z(IV-1)=Z(IV-1)*F2+Z(IV)*(F1-F2)
      Z(IV)=Z(IV)*F1
      V=VAUX
      DO 146 K=2,IMIN-1
        IVOL=IVOL-NR-2+K
        INOR=INOR-NR-1+K
        IV=IVOL+1
        IN=INOR+1
        SUM=RCO(K-1,K)-RSQ(K-1,K)
        SUMA=.5*VOL(3,IN)
        VAP=SUM/SUMA
        PREC=MAX(PREC,ABS(1.-VAP)*SUM*AIRI)
        Z(IV)=Z(IV)*VAP
       DO 145 L=K+1,NR-1
        AUX=RCO(K-1,L)-RSQ(K-1,L)-RCO(K,L)+RSQ(K,L)-SUM
        SUM=SUM+AUX
        IV=IV+1
        IN=IN+1
        SUMA=SUMA+.5*VOL(3,IN)
        VAP=AUX/(.5*VOL(3,IN))
        PREC=MAX(PREC,ABS(1.-VAP)*AUX*AIRI)
        Z(IV)=Z(IV)*VAP
 145   CONTINUE
        XM=A2*SINB-R(K)
        VAUX=R04*(SQRT3*A2SIN+XM)
        IV=IV+2
        IN=IN+1
        VTEX=(R(K)-R(K-1))*(VOL45-.5*TG*(R(K)+R(K-1)))
        FF1=.5*SUM
        FF2=.5*SUMA
        F1=(VTEX-FF1)/(VTEX-FF2)
        F2=(V-VAUX-VTEX-FF1)/(V-VAUX-VTEX-FF2)
        PREC=MAX(PREC,ABS(1.-F1)*(VTEX-FF1)*AIRI)
        PREC=MAX(PREC,ABS(1.-F2)*(V-VAUX-VTEX-FF1)*AIRI)
        Z(IV-1)=Z(IV-1)*F2+Z(IV)*(F1-F2)
        Z(IV)=Z(IV)*F1
        V=VAUX
 146  CONTINUE
      DM=R(IMIN-1)
         ENDIF
      SUM=0.
      SUMA=0.
      IVOL=IVOL-2-NR+IMIN
      INOR=INOR-NR+IMIN-1
      IV=IVOL
      IN=INOR
      DO 147 K2=IMIN,NR-1
       AUX=R2(K2)*ACOS(DM/R(K2))-DM*SQRT(R2(K2)-DM*DM)
     +     -R2(K2)*ACOS(DMIN5/R(K2))+DMIN5*SQRT(R2(K2)-DMIN5*DMIN5)-SUM
       SUM=SUM+AUX
       IV=IV+1
       IN=IN+1
       SUMA=SUMA+.5*VOL(3,IN)
       VAP=AUX/(.5*VOL(3,IN))
       Z(IV)=Z(IV)*VAP
       PREC=MAX(PREC,ABS(1.-VAP)*AUX*AIRI)
 147  CONTINUE
      IV=IV+2
      IN=IN+1
      VTEX=(DMIN5-DM)*(VOL45-.5*TG*(DMIN5+DM))
      FF1=.5*SUM
      FF2=.5*SUMA
      F1=(VTEX-FF1)/(VTEX-FF2)
      F2=(V-VV4-VTEX-FF1)/(V-VV4-VTEX-FF2)
      PREC=MAX(PREC,ABS(1.-F1)*(VTEX-FF1)*AIRI)
      PREC=MAX(PREC,ABS(1.-F2)*(V-VV4-VTEX-FF1)*AIRI)
      Z(IV-1)=Z(IV-1)*F2+Z(IV)*(F1-F2)
      Z(IV)=Z(IV)*F1
C
C       FOR SURFACE 5
C
      DM=DMIN5
      V=V5
      INOR=INOR5
      DO 135 K=IMIN,IMAX-1
        IVOL=IVOL-NR-2+K
        INOR=INOR-NR-1+K
        IV=IVOL+1
        IN=INOR+1
        SUM=R2(K)*ACOS(DM/R(K))-DM*SQRT(R2(K)-DM*DM)
        SUMA=.5*VOL(2,IN)
        VAP=SUM/(.5*VOL(2,IN))
        Z(IV)=Z(IV)*VAP
        PREC=MAX(PREC,ABS(1.-VAP)*SUM*AIRI)
       DO 130 L=K+1,NR-1
        AUX=R2(L)*ACOS(DM/R(L))-DM*SQRT(R2(L)-DM*DM)
     +      -RCO(K,L)+RSQ(K,L)-SUM
        SUM=SUM+AUX
        IV=IV+1
        IN=IN+1
        SUMA=SUMA+.5*VOL(2,IN)
        VAP=AUX/(.5*VOL(2,IN))
        Z(IV)=Z(IV)*VAP
        PREC=MAX(PREC,ABS(1.-VAP)*AUX*AIRI)
 130   CONTINUE
        XM=(DMAX5-R(K))/COSFI
        RO=SQRT3*(COTE+XM)/SINA
        A1=RO*SINFI
        A3=A1-XM
        B=A2-A3
        VAUX=.5*COSFI*RO*A1+SQRT3*(B*B-.5*A3*A3)
        IV=IV+2
        IN=IN+1
        VTEX=(R(K)-DM)*(VOL45-.5*TG*(R(K)+DM))
        FF1=.5*SUM
        FF2=.5*SUMA
        F1=(VTEX-FF1)/(VTEX-FF2)
        F2=(V-VAUX-VTEX-FF1)/(V-VAUX-VTEX-FF2)
        PREC=MAX(PREC,ABS(1.-F1)*(VTEX-FF1)*AIRI)
        PREC=MAX(PREC,ABS(1.-F2)*(V-VAUX-VTEX-FF1)*AIRI)
        Z(IV-1)=Z(IV-1)*F2+Z(IV)*(F1-F2)
        Z(IV)=Z(IV)*F1
        V=VAUX
        DM=R(K)
 135  CONTINUE
      SUM=0.
      SUMA=0.
      IVOL=IVOL-2-NR+IMAX
      INOR=INOR-NR+IMAX-1
      IV=IVOL
      IN=INOR
      DO 136 K2=IMAX,NR-1
       AUX=R2(K2)*ACOS(DM/R(K2))-DM*SQRT(R2(K2)-DM*DM)
     +     -R2(K2)*ACOS(DMAX5/R(K2))+DMAX5*SQRT(R2(K2)-DMAX5*DMAX5)-SUM
       SUM=SUM+AUX
       IV=IV+1
       IN=IN+1
       SUMA=SUMA+.5*VOL(2,IN)
       VAP=AUX/(.5*VOL(2,IN))
       Z(IV)=Z(IV)*VAP
       PREC=MAX(PREC,ABS(1.-VAP)*AUX*AIRI)
 136  CONTINUE
      IV=IV+1
      IN=IN+1
      VTEX=(DMAX5-DM)*(VOL45-.5*TG*(DMAX5+DM))
      FF1=.5*SUM
      FF2=.5*SUMA
      F1=(VTEX-FF1)/(VTEX-FF2)
      F2=(V-VAUX5-VTEX-FF1)/(V-VAUX5-VTEX-FF2)
      PREC=MAX(PREC,ABS(1.-F1)*(VTEX-FF1)*AIRI)
      PREC=MAX(PREC,ABS(1.-F2)*(V-VAUX5-VTEX-FF1)*AIRI)
      Z(IV-1)=Z(IV-1)*F2+Z(IV)*(F1-F2)
      Z(IV)=Z(IV)*F1
C
C       FOR SURFACE 6
C
      DM=DMIN6
      V=V6
      INOR=INOR6
      DO 125 K=IMIN6,NR-1
        IVOL=IVOL-NR-2+K
        INOR=INOR-NR-1+K
        IV=IVOL+1
        IN=INOR+1
        SUM=R2(K)*ACOS(DM/R(K))-DM*SQRT(R2(K)-DM*DM)
        SUMA=.5*VOL(1,IN)
        VAP=SUM/SUMA
        Z(IV)=Z(IV)*VAP
        PREC=MAX(PREC,ABS(1.-VAP)*SUM*AIRI)
       DO 120 L=K+1,NR-1
        AUX=R2(L)*ACOS(DM/R(L))-DM*SQRT(R2(L)-DM*DM)
     +     -RCO(K,L)+RSQ(K,L)-SUM
        SUM=SUM+AUX
        IV=IV+1
        IN=IN+1
        SUMA=SUMA+.5*VOL(1,IN)
        VAP=AUX/(.5*VOL(1,IN))
        Z(IV)=Z(IV)*VAP
        PREC=MAX(PREC,ABS(1.-VAP)*AUX*AIRI)
 120   CONTINUE
        XM=(XMM-R(K))/SINFI
        VAUX=.5*XM*XM*SQRT3*SINFI/COSA
        IV=IV+2
        IN=IN+1
        VTEX=(R(K)-DM)*(VOL6-.5*(R(K)+DM)/TG)
        FF1=.5*SUM
        FF2=.5*SUMA
        F1=(VTEX-FF1)/(VTEX-FF2)
        F2=(V-VAUX-VTEX-FF1)/(V-VAUX-VTEX-FF2)
        PREC=MAX(PREC,ABS(1.-F1)*(VTEX-FF1)*AIRI)
        PREC=MAX(PREC,ABS(1.-F2)*(V-VAUX-VTEX-FF1)*AIRI)
        Z(IV-1)=Z(IV-1)*F2+Z(IV)*(F1-F2)
        Z(IV)=Z(IV)*F1
        V=VAUX
        DM=R(K)
 125  CONTINUE
      IVOL=IVOL-2
      VAP=V/(.5*VOL(1,INOR))
      PREC=MAX(PREC,ABS(1.-VAP)*V*AIRI)
      Z(IVOL+1)=Z(IVOL+1)*VAP
      Z(IVOL+2)=Z(IVOL+2)*VAP
      IVOL=IVOL-9
  160    CONTINUE
            ENDIF
C
  150      CONTINUE
      L=IZ
C
C   STORAGE OF THE ORTHONORMALIZATION FACTORS
C
      ORT=PI12
      ORTH1=ORTH1*ORT
      ORTH2=ORTH2*ORT
      ORTH3=ORTH3*ORT
      Z(1)=1.0/SQRT(ORTH1)
      Z(2)=1.0/SQRT(0.75*ORTH3-0.7205061948*ORTH2*ORTH2/ORTH1)
      Z(3)=Z(2)*0.8488263632*ORTH2/ORTH1
      Z(4)=2.0/SQRT(3.0*(ORTH1-ORTH3))
C
C   TRACKING INFORMATION OUTPUT
C
      IF(ILIGN.NE.1) RETURN
      WRITE(IOUT,500) (Z(I),I=1,4)
      IZ0=2
      IZ=4
      L2=0
      DO 250 IA=1,NA
       IZ0=IZ0+3
       NR5=ZI(IZ0)
       IMIN=ZI(IZ0-1)
       IMIN6=ZI(IZ0-2)
       IMAX=NR5+IMIN-1
       NR6=NR-IMIN6+1
       IZ=IZ+NR5+NR6+IMIN
       DO 250 IX=1,NX
        IZ=IZ+9
        L2=L2+1
        IF(IX.EQ.1) WRITE(IOUT,520) Z(IZ-2)/Z(IZ)
        WRITE(IOUT,510) L2
        DO 180 I=NR-1,IMIN6-1,-1
        MZ=NR-I
        IF(MZ.GT.1) THEN
            Z3=Z(IZ+MZ+1)
            WRITE(IOUT,550) MZ,Z3,(Z(IZ+J)*.5,J=NR-I-1,2,-1),
     +        Z(IZ+1),(Z(IZ+J),J=2,NR-I-1),Z(IZ+MZ)-Z3
        ELSE
            WRITE(IOUT,550) MZ,Z(IZ+1)
        ENDIF
        IZ=IZ+NR-I+1
 180    CONTINUE
        DO 185 I=IMAX-1,IMIN-1,-1
        MZ=NR-I
        IF(MZ.GT.1) THEN
            Z3=Z(IZ+MZ+1)
            WRITE(IOUT,550) MZ,Z3,(Z(IZ+J)*.5,J=NR-I-1,2,-1),
     +        Z(IZ+1),(Z(IZ+J),J=2,NR-I-1),Z(IZ+MZ)-Z3
        ELSE
            WRITE(IOUT,550) MZ,Z(IZ+1)
        ENDIF
        IZ=IZ+NR-I+1
 185    CONTINUE
        DO 190 I=IMIN-1,0,-1
        IF(MZ.GT.1) THEN
            Z3=Z(IZ+MZ+1)
            WRITE(IOUT,550) MZ,Z3,(Z(IZ+J)*.5,J=NR-I-1,2,-1),
     +        Z(IZ+1),(Z(IZ+J),J=2,NR-I-1),Z(IZ+MZ)-Z3
        ELSE
            WRITE(IOUT,550) MZ,Z(IZ+1)
        ENDIF
        IZ=IZ+NR-I+1
 190    CONTINUE
 250  CONTINUE
      RETURN
C
 9000 FORMAT(' SYBHTK: ERROR'/
     +       '                RADIAL REGION NUMBER = ',I5/
     +       '                WITH RADIUS          = ',1P,E15.7/
     +       ' IS NOT INCLUDED IN HEXAGONE OF SIDE = ',E15.7/
     +       '            MAXIMUM RADIUS PERMITTED = ',E15.7)
 500  FORMAT(//' TRACKING INFORMATION LISTING.'//
     +       ' NUMERICAL OHRTHONORMALIZATION FACTORS =',
     +       1P,4E12.4//' TRACK')
 520  FORMAT(6X,'  COS =',1P,E10.3)
 510  FORMAT(1X,I5)
 550  FORMAT(42X,'  MZ =',I3,'  SEGMENTS =',1P,5E10.3/(63X,5E10.3))
      END
