*DECK SYBRX4
      SUBROUTINE SYBRX4 (IPAS,SIGT,P,IMPX,NCOUR,NMCEL,NMERGE,NGEN,IJAT,
     1 IQUAD,XX,YY,NMC,RAYRE,MAIL,IZMAIL,RZMAIL,IFR,ALB,INUM,MIX,DVX,
     2 IGEN,PIS,PSS,PSSB,SIGT2)
*
*-----------------------------------------------------------------------
*
* COMPUTE THE REDUCED COLLISION PROBABILITIES IN A 2-D CARTESIAN OR
* HEXAGONAL ASSEMBLY USING THE INTERFACE CURRENT METHOD WITH DP-0
* APPROXIMATION.
*
* GENERAL PARAMETERS:
*   IPAS   : TOTAL NUMBER OF VOLUMES.
*   SIGT   : TOTAL MACROSCOPIC CROSS SECTIONS. DIMENSION SIGT(IPAS)
*   P      : REDUCED COLLISION PROBABILITIES. DIMENSION P(IPAS,IPAS)
*   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*   NCOUR  : =4 : CARTESIAN LATTICE ; =6 : HEXAGONAL LATTICE.
*   IQUAD  : QUADRATURE PARAMETERS. DIMENSION IQUAD(4)
*
*   PARAMETERS RELATED TO CELLS DEFINITION:
*   NMCEL  : TOTAL NUMBER OF CELLS IN THE DOMAIN.
*   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAS) WHERE
*            IJAS IS THE TOTAL NUMBER OF SURFACES SURROUNDING ALL THE
*            CELLS.
*   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
*            DIMENSION ALB(IJAS)
*
*   PARAMETERS RELATED TO THE MERGED CELLS DEFINITION:
*   NMERGE : TOTAL NUMBER OF MERGED CELLS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*            MANY CELLS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*            OWN THE SAME GENERATING CELL. THIS ALLOWS SOME REDUCTION
*            IN CPU TIME AND MEMORY. NMERGE.LE.NMCEL
*   IJAT   : TOTAL NUMBER OF DISTINCT OUT-CURRENTS.
*   INUM   : INDEX-NUMBER OF THE MERGED CELL ASSOCIATED TO EACH CELL.
*            DIMENSION INUM(NMCEL)
*   MIX    : INDEX-NUMBER OF OUT-CURRENTS. DIMENSION MIX(IJAZ) WHERE
*            IJAZ IS THE TOTAL NUMBER OF SURFACES SURROUNDING ALL THE
*            MERGED CELLS.
*   DVX    : WEIGHT ASSOCIATED WITH EACH OUT-CURRENT.
*            DIMENSION DVX(IJAZ)
*
*            REMARK:
*            IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
*            GEOMETRICAL 'A' MATRIX.
*
*   PARAMETERS RELATED TO THE GENERATING CELLS DEFINITION:
*   NGEN   : TOTAL NUMBER OF GENERATING CELLS. A GENERATING CELL IS
*            DEFINED BY ITS MATERIAL AND DIMENSIONS IRRESPECTIVE OF
*            ITS POSITION IN THE DOMAIN. NGEN.LE.NMERGE
*   XX,YY  : X AND Y-THICKNESS OF THE GENERATING CELLS.
*            DIMENSION XX(NGEN),YY(NGEN)
*   NMC    : OFFSET OF THE FIRST VOLUME IN EACH GENERATING CELL.
*            DIMENSION NMC(NGEN+1)
*   RAYRE  : RADIUS OF THE TUBES IN EACH GENERATING CELL.
*            DIMENSION RAYRE(NMC(NGEN+1))
*   MAIL   : OFFSET OF THE FIRST TRACKING INFORMATION IN EACH GENERATING
*            CELL. DIMENSION MAIL(NGEN)
*   IZMAIL : INTEGER TRACKING INFORMATION. DIMENSION IZMAIL(LMAIL) WHERE
*            LMAIL IS THE SPACE REQUIRED TO STORE THE TRACKING
*            INFORMATION.
*   RZMAIL : REAL TRACKING INFORMATION. DIMENSION RZMAIL(LMAIL)
*   IGEN   : INDEX-NUMBER OF THE GENERATING CELL ASSOCIATED WITH EACH
*            MERGED CELL. DIMENSION IGEN(NMERGE)
*
* SCRATCH STORAGE:
*   PIS    : DIMENSION PIS(NCOUR*IPAS)
*   PSS    : DIMENSION PSS(NCOUR*NCOUR*NGEN)
*   PSSB   : DIMENSION PSSB(2*IJAT**2)
*   SIGT2  : DIMENSION SIGT2(IPAS)
*
*---------------------------------------- AUTHOR: A. HEBERT 22/05/87 ---
*
      PARAMETER (PI=3.141592654,SMALL=5.0E-3,SQRT3=1.732050807568877)
      CHARACTER HSMG*131
      DIMENSION SIGT(*),P(IPAS,*),IQUAD(*),XX(*),YY(*),NMC(*),RAYRE(*),
     1 MAIL(*),IZMAIL(*),RZMAIL(*),IFR(*),ALB(*),INUM(*),MIX(*),DVX(*),
     2 IGEN(*),PIS(NCOUR,*),PSS(NCOUR,NCOUR,*),PSSB(IJAT,*),SIGT2(*),
     3 SURFA(6),ALPA(64),PWA(64)
      COMMON BASE(1)
*
      DO 10 I=1,IPAS
      DO 10 J=1,IPAS
10    P(I,J)=0.0
      I1=0
      DO 30 IKK=1,NMERGE
      IKG=IGEN(IKK)
      J1=NMC(IKG)
      I2=NMC(IKG+1)-J1
      DO 20 I=1,I2
20    SIGT2(J1+I)=SIGT(I1+I)
30    I1=I1+I2
*
      DO 100 JKG=1,NGEN
      J1=NMC(JKG)
      J2=NMC(JKG+1)-J1
      A=XX(JKG)
      B=YY(JKG)
      CALL SETARA(BASE,J2*J2,IP)
      IF ((NCOUR.EQ.4).AND.(J2.EQ.1)) THEN
         CALL ALGPT(IQUAD(3),-1.0,1.0,ALPA,PWA)
         CALL RECT1(IQUAD(3),A,B,SIGT2(J1+1),SMALL,BASE(IP),PIS(1,J1+1),
     1   PSS(1,1,JKG),ALPA,PWA)
      ELSE IF (J2.EQ.1) THEN
         CALL ALGPT(IQUAD(3),-1.0,1.0,ALPA,PWA)
         CALL XHX2D0(IQUAD(3),ALPA,PWA,A,SIGT2(J1+1),SMALL,BASE(IP),
     1   PIS(1,J1+1),PSS(1,1,JKG))
      ELSE
         IB=MAIL(JKG)
         IF (NCOUR.EQ.4) THEN
            CALL SYBRN1(J2,A,B,RZMAIL(IB),IZMAIL(IB),RAYRE(J1+1),
     1      SIGT2(J1+1),SMALL,PIS(1,J1+1),PSS(1,1,JKG))
            X=2.0*A*B/(A+B)
         ELSE
            LZI=3*IQUAD(1)+2
            CALL SETARA(BASE,J2,IPA)
            CALL SYBHN1(J2,RZMAIL(IB+LZI),IZMAIL(IB),A,SIGT2(J1+1),
     1      SMALL,RAYRE(J1+2),BASE(IPA),PIS(1,J1+1),PSS(1,1,JKG))
            CALL RLSARA(BASE(IPA))
            X=SQRT3*A
         ENDIF
         SIGTOT=SIGT2(J1+J2)
         IF (X*SIGTOT.LE.SMALL) THEN
            WRITE (HSMG,260) JKG
            CALL XABORT(HSMG)
         ENDIF
         JJJ=J2-1
         CALL SETARA(BASE,4*JJJ,IX)
         CALL SYBALC(JJJ,J2,RAYRE(J1+1),SIGT2(J1+1),BASE(IP),IQUAD(4),
     1   0.0,BASE(IX))
         CALL RLSARA(BASE(IX))
         W=1.0
         DO 40 IC=1,NCOUR
40       W=W-PIS(IC,J1+J2)
         VOL=PI*RAYRE(J1+J2)**2
         IF (NCOUR.EQ.4) THEN
            Y=A*B-VOL
         ELSE
            Y=1.5*SQRT3*A*A-VOL
         ENDIF
         DO 70 I=1,JJJ
         III=J2-I
         R2=PI*RAYRE(J1+III)**2
         X=1.0
         DO 50 IC=1,NCOUR
50       X=X-PIS(IC,J1+III)
         DO 60 J=1,JJJ
60       X=X-SIGT2(J1+J)*BASE(IP+(J-1)*J2+III-1)
         BASE(IP+(J2-1)*J2+III-1)=X/SIGTOT
         BASE(IP+III*J2-1)=(VOL-R2)*BASE(IP+(J2-1)*J2+III-1)/Y
         W=W-SIGT2(J1+III)*BASE(IP+III*J2-1)
70       VOL=R2
         BASE(IP+J2*J2-1)=W/SIGTOT
      ENDIF
      IF (IMPX.GE.8) CALL SYBPRX (NCOUR,J2,JKG,SIGT2(J1+1),BASE(IP),
     1 PIS(1,J1+1),PSS(1,1,JKG))
      I1=0
      DO 90 IKK=1,NMERGE
      IKG=IGEN(IKK)
      I2=NMC(IKG+1)-NMC(IKG)
      IF (IKG.EQ.JKG) THEN
         DO 80 I=1,J2
         DO 80 J=1,J2
80       P(I1+I,I1+J)=BASE(IP+(J-1)*J2+I-1)
      ENDIF
90    I1=I1+I2
      CALL RLSARA(BASE(IP))
100   CONTINUE
*
* COMPUTATION OF PSSB=A*(I-PSS*A)**-1
      DO 150 I=1,IJAT
      DO 140 J=1,IJAT
      PSSB(I,J)=0.0
140   PSSB(I,IJAT+J)=0.0
150   PSSB(I,I)=1.0
      DO 155 ICEL=1,NMCEL
      IKK=INUM(ICEL)
      IT=NCOUR*(IKK-1)
      IS=NCOUR*(ICEL-1)
      IKG=IGEN(IKK)
      DO 155 JC=1,NCOUR
      J1=IFR(IS+JC)
      J2=MIX(IT+JC)
      ALBEDO=ALB(IS+JC)
      PSSB(J1,IJAT+J2)=PSSB(J1,IJAT+J2)+ALBEDO*DVX(IT+JC)
      DO 155 IC=1,NCOUR
      J2=MIX(IT+IC)
155   PSSB(J1,J2)=PSSB(J1,J2)-PSS(IC,JC,IKG)*ALBEDO*DVX(IT+IC)
      CALL ALSB (IJAT,IJAT,PSSB,IER,IJAT)
      IF (IER.NE.0) CALL XABORT('SYBRX4: SINGULAR MATRIX.')
*
* COMPUTATION OF PIS*PSSB*PSJ
      I1=0
      DO 240 IKK=1,NMERGE
      IKG=IGEN(IKK)
      I1P=NMC(IKG)
      I2=NMC(IKG+1)-I1P
      IT=NCOUR*(IKK-1)
      DO 230 I=1,I2
      DO 230 IC=1,NCOUR
      ICC=MIX(IT+IC)
      ZZZ=PIS(IC,I1P+I)
      J1=0
      DO 230 JKK=1,NMERGE
      JKG=IGEN(JKK)
      J1P=NMC(JKG)
      J2=NMC(JKG+1)-J1P
      JT=NCOUR*(JKK-1)
      RJ1=0.0
      DO 220 J=1,J2
      IF ((NCOUR.EQ.4).AND.(J.EQ.J2)) THEN
         RJ2=XX(JKG)*YY(JKG)
      ELSE IF (J.EQ.J2) THEN
         RJ2=1.5*SQRT3*XX(JKG)*XX(JKG)
      ELSE
         RJ2=PI*RAYRE(J1P+J+1)**2
      ENDIF
      IF (NCOUR.EQ.4) THEN
         SURFA(1)=0.25*YY(JKG)
         SURFA(2)=0.25*YY(JKG)
         SURFA(3)=0.25*XX(JKG)
         SURFA(4)=0.25*XX(JKG)
      ELSE
         DO 200 JC=1,NCOUR
200      SURFA(JC)=0.25*XX(JKG)
      ENDIF
      DO 210 JC=1,NCOUR
      JCC=MIX(JT+JC)
      PBJ=PIS(JC,J1P+J)*(RJ2-RJ1)/SURFA(JC)
210   P(I1+I,J1+J)=P(I1+I,J1+J)+ZZZ*DVX(JT+JC)*PSSB(JCC,IJAT+ICC)*PBJ
220   RJ1=RJ2
230   J1=J1+J2
240   I1=I1+I2
      RETURN
260   FORMAT (30HSYBRX4: GENERATING CELL NUMBER,I6,14H HAS A VOIDED ,
     1 6HCROWN.)
      END
