*DECK SYBRXE
      SUBROUTINE SYBRXE(IPAS,NSUPCE,RAYRE,SIGT,P,IQUAD,ISTAT,NMC,PROCEL,
     1 SURFA,POURCE,IMPX,IAPPAR,PIB,PBI,PSS,PBB)
*
*-----------------------------------------------------------------------
*
* CALCULATION OF THE REDUCED COLLISION PROBABILITIES FOR THE
* 'DO-IT-YOURSELF' APPROACH.
*
* REFERENCE: A. HOFFMANN, F. JEANPIERRE, A. KAVENOKY, M. LIVOLANT
* AND H. LORAIN, 'APOLLO, CODE MULTIGROUPE DE RESOLUTION DE L'EQUATION
* DU TRANSPORT POUR LES NEUTRONS RAPIDES ET THERMIQUES', REPORT
* CEA-N-1610, COMMISSARIAT A L'ENERGIE ATOMIQUE, P. 13 (1973).
*
* GENERAL PARAMETERS:
*   IPAS   : NUMBER OF VOLUMES.
*   NSUPCE : NUMBER OF CELLS.
*   RAYRE  : RADIUS OF THE TUBES IN EACH CELL.
*            DIMENSION RAYRE(NSUPCE+IPAS)
*   SIGT   : TOTAL MACROSCOPIC CROSS SECTIONS. DIMENSION SIGT(IPAS)
*   P      : REDUCED COLLISION PROBABILITIES. DIMENSION P(IPAS,IPAS)
*   IQUAD  : QUADRATURE PARAMETER.
*   ISTAT  : ISTAT=1 FOR THE STATISTICAL APPROXIMATION.
*   NMC    : OFFSET OF THE FIRST VOLUME IN EACH CELL.
*            DIMENSION NMC(NSUPCE+1)
*   PROCEL : USER SUPPLIED GEOMETRICAL MATRIX.
*            DIMENSION PROCEL(NSUPCE**2)
*   SURFA  : DIMENSION SURFA(NSUPCE)
*   POURCE : DIMENSION POURCE(NSUPCE)
*   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*
* SCRATCH STORAGE:
*   IAPPAR : DIMENSION IAPPAR(IPAS)
*   PIB    : DIMENSION PIB(IPAS)
*   PBI    : DIMENSION PBI(IPAS)
*   PSS    : DIMENSION PSS(NSUPCE,2*NSUPCE)
*   PBB    : DIMENSION PBB(NSUPCE)
*
*---------------------------------------- AUTHOR: A. HEBERT 21/05/87 ---
*
      PARAMETER (PI=3.141592654)
      DIMENSION RAYRE(*),SIGT(*),P(IPAS,*),NMC(*),PROCEL(NSUPCE,*),
     1 SURFA(*),POURCE(*),IAPPAR(*),PIB(*),PBI(*),PSS(NSUPCE,*),PBB(*)
      COMMON BASE(1)
      DO 2 IKK=1,NSUPCE
      J1=NMC(IKK)
      J2=NMC(IKK+1)-J1
      CALL SETARA(BASE,J2**2,IPP)
      CALL SETARA(BASE,4*J2,IX)
      CALL SYBALC(J2,J2,RAYRE(J1+IKK),SIGT(J1+1),BASE(IPP),IQUAD,0.0,
     1 BASE(IX))
      CALL RLSARA(BASE(IX))
      PBB(IKK)=0.0
      RJN=0.0
      DO 20 I=1,J2
      IAPPAR(I+J1)=IKK
      PIB(I+J1)=0.0
      DO 21 J=1,J2
      P(I+J1,J+J1)=BASE(IPP+(J-1)*J2+I-1)
   21 PIB(I+J1)=PIB(I+J1)+BASE(IPP+(J-1)*J2+I-1)*SIGT(J+J1)
      PIB(I+J1)=1.-PIB(I+J1)
      RJN1=RAYRE(I+J1+IKK)**2
      VOLT=PI*(RJN1-RJN)*POURCE(IKK)
      PBI(I+J1)=4.0*VOLT*PIB(I+J1)/SURFA(IKK)
      PBB(IKK)=PBB(IKK)+PBI(I+J1)*SIGT(I+J1)
   20 RJN=RJN1
      PBB(IKK)=1.-PBB(IKK)
      IF (IMPX.GE.8) CALL SYBPRX (1,J2,IKK,SIGT(J1+1),BASE(IPP),
     1 PIB(J1+1),PBB(IKK))
      CALL RLSARA(BASE(IPP))
    2 CONTINUE
*
* COMPUTATION OF QIJ MATRIX.
      IF (ISTAT.EQ.0) THEN
         DO 22 I=1,NSUPCE
         DO 23 J=1,NSUPCE
         PSS(I,J)=-PROCEL(I,J)*PBB(J)
   23    PSS(I,NSUPCE+J)=PROCEL(I,J)
   22    PSS(I,I)=1.0+PSS(I,I)
         CALL ALSB (NSUPCE,NSUPCE,PSS,IER,NSUPCE)
         IF (IER.NE.0) CALL XABORT('SYBRXE: SINGULAR MATRIX.')
*        COMPUTATION OF PIJ MATRIX.
         DO 26 I=1,IPAS
         K=IAPPAR(I)
         DO 26 J=1,IPAS
         L=IAPPAR(J)
         XX=PIB(I)*PSS(K,NSUPCE+L)*PBI(J)
         IF (L.EQ.K) THEN
            P(I,J)=P(I,J)+XX
         ELSE
            P(I,J)=XX
         ENDIF
   26    CONTINUE
      ELSE
         X1=0.
         DO 47 I=1,NSUPCE
   47    X1=X1+PBB(I)*PROCEL(1,I)
         X1=1./(1.-X1)
         DO 46 J=1,IPAS
         L=IAPPAR(J)
         ZZZ=PROCEL(1,L)*X1*PBI(J)
         DO 46 I=1,IPAS
         K=IAPPAR(I)
         XX=PIB(I)*ZZZ
         IF (L.EQ.K) THEN
            P(I,J)=P(I,J)+XX
         ELSE
            P(I,J)=XX
         ENDIF
   46    CONTINUE
      ENDIF
      RETURN
      END
