*DECK TUBE2
      SUBROUTINE TUBE2 (NGPT,RAYRE,SIGT,TRONC,PVV,PVS,PSS)
*
*-----------------------------------------------------------------------
*
* INTEGRATION OF THE COLLISION PROBABILITIES OF AN HOMOGENEOUS 1-D
* CYLINDER OR TUBE (DP-1 SURFACE ANGULAR FLUX APPROXIMATION).
*
* INPUT PARAMETERS.
*   NGPT    : NUMBER OF GAUSS-JACOBI INTEGRATION POINTS.
*   RAYRE   : RADIUS OF TUBE. DIMENSION RAYRE(2)
*   SIGT    : CROSS SECTION.
*   TRONC   : VOIDED BLOCK CRITERION.
*
* OUTPUT PARAMETERS.
*   PVV     : VOLUME TO VOLUME REDUCED PROBABILITY.
*   PVS     : VOLUME TO SURFACE PROBABILITIES.
*             DIMENSION PVS(2) (CYLINDER) OR PVS(4) (TUBE)
*   PSS     : SURFACE TO SURFACE PROBABILITIES IN THE FOLLOWING ORDER:
*             1 TO 1, 2 TO 1, 3 TO 1, 4 TO 1, 1 TO 2, 2 TO 2, ETC.
*             WHERE 1 AND 2 ARE THE INTERNAL SURFACES;
*                   3 AND 4 ARE THE EXTERNAL SURFACES.
*             DIMENSION PSS(4) (CYLINDER) OR PSS(16) (TUBE).
*
*----------------------------------------------------- M-L VERGAIN -----
*
      PARAMETER (MKI3=600,MKI4=600,MKI5=600)
      PARAMETER (PI=3.1415926535,MGPT=64)
      COMMON /BICKL3/BI3(0:MKI3,3),PAS3,XLIM3,L3
      COMMON /BICKL4/BI4(0:MKI4,3),PAS4,XLIM4,L4
      COMMON /BICKL5/BI5(0:MKI5,3),PAS5,XLIM5,L5
      LOGICAL FILLED
      DIMENSION RAYRE(2),PVS(*),PSS(*),ZJKSI(MGPT),WJKSI(MGPT),
     >          SEGL(6*MGPT),AEGL(2*MGPT),POSE(7)
*
      CALL ALGJP(NGPT,ZJKSI,WJKSI)
      RAI=RAYRE(1)
      RAA=RAYRE(2)
      FILLED=(RAI.EQ.0.0)
      PI8=8.0/PI
      PI4=4.0/PI
      PI2=2.0/PI
      OPI=1.0/PI8
      QPI=1.0/PI4
      DPI=1.0/PI2
      SQ=SQRT(2.0)
      PVOL=RAA*RAA-RAI*RAI
      SS1=RAI/RAA
      SS2=SS1*SS1
      FP1=1.0-SS1
      SP1=SQRT(FP1)
      IOS=1
      IL=1
      IF (FILLED) THEN
         DO 10 I=1,2
   10    PVS(I)=0.0
         DO 20 I=1,4
   20    PSS(I)=0.0
         DO 30 J=1,NGPT
         XZ2=ZJKSI(J)*ZJKSI(J)
         AEGL(IL)=2.0*RAA*ZJKSI(J)*SQRT(2.0-XZ2)
         SEGL(IOS)=PI8*WJKSI(J)
         SEGL(IOS+1)=PI8*WJKSI(J)*ZJKSI(J)*SQRT(2.0-XZ2)
         SEGL(IOS+2)=PI8*WJKSI(J)*XZ2*(2.0-XZ2)
         IL=IL+1
   30    IOS=IOS+3
      ELSE
         DO 40 I=1,4
   40    PVS(I)=0.0
         DO 50 I=1,16
   50    PSS(I)=0.0
         DO 120 J=1,NGPT
         XZ2=ZJKSI(J)*ZJKSI(J)
         AEGL(IL)=2.0*RAA*SP1*ZJKSI(J)*SQRT(2.0-FP1*XZ2)
         AEGL(IL+1)=RAA*(SQRT(1.0-SS2*(1.0-XZ2)**2)
     >              -SS1*ZJKSI(J)*SQRT(2.0-XZ2))
         SEGL(IOS)=PI8*WJKSI(J)
         SEGL(IOS+1)=PI8*WJKSI(J)*SQRT(1.0-SS2*(1.0-XZ2)**2)
         SEGL(IOS+2)=PI8*WJKSI(J)*ZJKSI(J)*SQRT(2.0-XZ2)
         SEGL(IOS+3)=PI8*WJKSI(J)*ZJKSI(J)*SQRT(2.0-XZ2)
     >               *SQRT(1.0-SS2*(1.0-XZ2)**2)
         SEGL(IOS+4)=PI8*WJKSI(J)*ZJKSI(J)*SQRT(2.0-XZ2*FP1)*FP1*SP1
         SEGL(IOS+5)=PI8*WJKSI(J)*XZ2*(2.0-XZ2*FP1)*(FP1**2)
         IL=IL+2
  120    IOS=IOS+6
      ENDIF
*
      DO 130 IN=1,7
  130 POSE(IN)=0.0
      SIGR=SIGT*RAA
      SIGXX=2.0*(RAA-RAI)*SIGT
*
      IF (SIGXX.GE.TRONC) THEN
*         NON-VOID ANNULAR REGION
          IL=1
          IOS=1
          IF (FILLED) THEN
*            INTEGRATE CYLINDER PSS
             DO 210 IPT=1,NGPT
             XE=AEGL(IL)*SIGT
             ZI3=0.0
             IF (XE.LT.XLIM3) THEN
                K=NINT(XE*PAS3)
                ZI3=BI3(K,1)+XE*(BI3(K,2)+XE*BI3(K,3))
             ENDIF
             ZI4=0.0
             IF (XE.LT.XLIM4) THEN
                 K=NINT(XE*PAS4)
                 ZI4=BI4(K,1)+XE*(BI4(K,2)+XE*BI4(K,3))
             ENDIF
             ZI5=0.0
             IF (XE.LT.XLIM5) THEN
                 K=NINT(XE*PAS5)
                 ZI5=BI5(K,1)+XE*(BI5(K,2)+XE*BI5(K,3))
             ENDIF
             POSE(1)=POSE(1)+SEGL(IOS)*ZI3
             POSE(2)=POSE(2)+SEGL(IOS+1)*ZI4
             POSE(3)=POSE(3)+SEGL(IOS+2)*ZI5
             IL=IL+1
  210        IOS=IOS+3
*
*------      COMPUTE PVS,PVV CYLINDER
*
             PSS(1)=POSE(1)
             PSS(2)=3.0*SQ*POSE(2) - 2.0*SQ*POSE(1)
             PSS(4)=18.0*POSE(3) - 24.0*POSE(2) + 8.0*POSE(1)
             PSS(3)=PSS(2)
             PVS(1)=(1.0-PSS(1))/(2.0*SIGR)
             PVS(2)=-PSS(2)/(2.0*SIGR)
             PVV=(1.0-PVS(1))/SIGT
          ELSE
*           INTEGRATE TUBE PROBABILITIES
            DO 220 IPT=1,NGPT
            XE=AEGL(IL+1)*SIGT
            ZI3=0.0
            IF (XE.LT.XLIM3) THEN
                K=NINT(XE*PAS3)
                ZI3=BI3(K,1)+XE*(BI3(K,2)+XE*BI3(K,3))
            ENDIF
            ZI4=0.0
            IF (XE.LT.XLIM4) THEN
                K=NINT(XE*PAS4)
                ZI4=BI4(K,1)+XE*(BI4(K,2)+XE*BI4(K,3))
            ENDIF
            ZI5=0.0
            IF (XE.LT.XLIM5) THEN
                K=NINT(XE*PAS5)
                ZI5=BI5(K,1)+XE*(BI5(K,2)+XE*BI5(K,3))
            ENDIF
            POSE(1)=POSE(1)+SEGL(IOS)*ZI3
            POSE(2)=POSE(2)+SEGL(IOS+1)*ZI4
            POSE(3)=POSE(3)+SEGL(IOS+2)*ZI4
            POSE(4)=POSE(4)+SEGL(IOS+3)*ZI5
            XI=AEGL(IL)*SIGT
            ZI3=0.0
            IF (XI.LT.XLIM3) THEN
                K=NINT(XI*PAS3)
                ZI3=BI3(K,1)+XI*(BI3(K,2)+XI*BI3(K,3))
            ENDIF
            ZI4=0.0
            IF (XI.LT.XLIM4) THEN
                K=NINT(XI*PAS4)
                ZI4=BI4(K,1)+XI*(BI4(K,2)+XI*BI4(K,3))
            ENDIF
            ZI5=0.0
            IF (XI.LT.XLIM5) THEN
                K=NINT(XI*PAS5)
                ZI5=BI5(K,1)+XI*(BI5(K,2)+XI*BI5(K,3))
            ENDIF
            POSE(5)=POSE(5)+SEGL(IOS)*FP1*ZI3
            POSE(6)=POSE(6)+SEGL(IOS+4)*ZI4
            POSE(7)=POSE(7)+SEGL(IOS+5)*ZI5
            IL=IL+2
  220       IOS=IOS+6
*
*------     COMPUTE PVS,PVV TUBE
*
            SV1=RAI/(PVOL*SIGT*2.0)
            SV2=SV1*RAA/RAI
            PSS(9)=POSE(1)
            PSS(13)=3.0*SQ*POSE(2) - 2.0*SQ*POSE(1)
            PSS(10)=3.0*SQ*POSE(3) - 2.0*SQ*POSE(1)
            PSS(14)=18.0*POSE(4)-12.0*(POSE(2)+POSE(3))+8.0*POSE(1)
            PSS(11)=POSE(5)
            PSS(15)=3.0*SQ*POSE(6) - 2.0*SQ*POSE(5)
            PSS(16)=18.0*POSE(7) - 24.0*POSE(6) + 8.0*POSE(5)
            PSS(12)=PSS(15)
            PSS(3)=SS1*PSS(9)
            PSS(4)=SS1*PSS(13)
            PSS(7)=SS1*PSS(10)
            PSS(8)=SS1*PSS(14)
            PVS(3)=(1.0-PSS(11)-PSS(3))*SV2
            PVS(1)=(1.0-PSS(9))*SV1
            PVS(2)=-SV1*PSS(10)
            PVS(4)=-SV2*(PSS(4)+PSS(15))
            PVV=(1.0-PVS(3)-PVS(1))/SIGT
         ENDIF
      ELSE
*        COMPUTE PVV FROM SERIES EXPANSION OF KI3
         SGR2=SIGR*SIGR
         IF (FILLED) THEN
*           TREAT FILLED ANNULUS
            PSS(1)=1.0-2.0*SIGR+8.0*SGR2/3.0
            PSS(2)=-5.0*SQ*SGR2/6.0
            PSS(4)=1.0-2.0*SIGR+68.0*SGR2/15.0
            PSS(3)=PSS(2)
            PVS(2)=5.0*SQ*SIGR/12.0
            PVS(1)=1.0-4.0*SIGR/3.0
            PVV=4.0*RAA/3.0
         ELSE
*           TREAT TUBES
            IOS=1
            IL=2
            DO 230 I=1,NGPT
            XCI=AEGL(IL)
            POSE(1)=POSE(1)+SEGL(IOS)*XCI*XCI
            POSE(2)=POSE(2)+SEGL(IOS+1)*XCI
            POSE(3)=POSE(3)+SEGL(IOS+2)*XCI
            POSE(4)=POSE(4)+SEGL(IOS+3)*XCI*XCI
            POSE(5)=POSE(5)+SEGL(IOS+3)
            IL=IL+2
  230       IOS=IOS+6
*
*------     COMPUTE PVS,PVV
*
            SS3=SS1**3
            SS4=SS1**4
            SS5=SS1**5
            RSV=RAA/(2.0*RAA*(1.0-SS2))
            ASV=PI4*(SS1*SQRT(1.0-SS2)+ASIN(SS1))
            AVV=(2.0-3.0*SS1+SS3)*2.0/3.0
            ATV=3.0-1.5*ASV+(SS3-SS1)*SQRT(1.0-SS2)*PI4
            AUV=8.0-15.0*SS1+10.0*SS3-3.0*SS5
            AXX=(-ASV+2.0)/3.0-SIGR*AVV+SGR2*ATV*0.5
            AYY=AVV*3.0/8.0-SIGR*ATV/3.0+SGR2*AUV*2.0/15.0
            PSS(11)=(1.0-SS1)+SIGR*(ASV-2.0)+SGR2*AVV*2.0
            PSS(15)=3.0*SQ*AXX-2.0*SQ*PSS(11)
            PSS(16)=18.0*AYY-24.0*AXX+8.0*PSS(11)
*--------
            PSI=QPI*POSE(1)/(RAA*RAA)
            PSE=QPI*POSE(2)/RAA
            PSO=QPI*POSE(3)/RAA
            PSU=OPI*POSE(4)/(RAA*RAA)
            PSA=OPI*1.5*POSE(5)
            TIV=(PI8*(1.0-SS2)**1.5)/SS1
            BXX=ASV/(3.0*SS1)-SIGR*PSE+SGR2*(2.0*ASV*SS1
     >          + TIV*SS1+ASV/SS1-8.0*SS1+2.0*SS3)/8.0
            BYY=2.0/3.0-SIGR*PSO+SGR2*(4.0+2.0*SS2-4.0*ASV
     >          -TIV+ASV/SS2)/8.0
            BZZ=PSA-SIGR*(8.0-2.0*SS2-4.0*ASV-TIV+ASV/SS2)/12.0
     >          + SGR2*PSU
            PSS(9)=1.0+SIGR*(SS2-ASV*0.5)/SS1+SGR2*PSI
            PSS(13)=3.0*SQ*BXX-2.0*SQ*PSS(9)
            PSS(10)=3.0*SQ*BYY-2.0*SQ*PSS(9)
            PSS(14)=18.0*BZZ-12.0*BXX-12.0*BYY+8.0*PSS(9)
            PSS(12)=PSS(15)
            PSS(3)=SS1*PSS(9)
            PSS(4)=SS1*PSS(13)
            PSS(7)=SS1*PSS(10)
            PSS(8)=SS1*PSS(14)
            PVS(3)=-(ASV*0.5-2.0+SS2+SIGR*(2.0*AVV+SS1*PSI))
     >             /(2.0*(1.0-SS2))
            PVS(1)=-(SS2-ASV*0.5+SIGR*PSI*SS1)/(2.0*(1.0-SS2))
            PVS(2)=(3.0*PSO*SS1+2.0*SS2-ASV-SIGR*((4.0*SS1
     >         +2.0*SS3+ASV/SS1-TIV*SS1-4.0*SS1*ASV)*3.0/8.0
     >         -2.0*PSI*SS1))/(SQ*(1.0-SS2))
            PVS(4)=(3.0*PSE*SS1+2.0*SS2+ASV+3.0*AVV-4.0-SIGR*
     >        ((2.0*ASV*SS2+TIV*SS2+ASV-8.0*SS2+2.0*SS4)*3.0/8.0
     >        -2.0*SS1*PSI+1.5*ATV-4.0*AVV))/(SQ*(1.0-SS2))
            PVV=RAA*(AVV+SS1*PSI)/(1.0-SS2)
         ENDIF
      ENDIF
      RETURN
      END
