*DECK TUBE3D
      SUBROUTINE TUBE3D(NGPT,RAYRE,HAUT,SIGT,TRONC,PVV,PVS,PSS)
C
C------------------------  TUBE3D  ------------------------------------
C
C 1- PROGRAMME STATISTICS:
C
C   NAME       :  TUBE3D
C   FUNCTION   :  INTEGRATION OF THE COLLISION PROBABILITIES
C                 OF AN HOMOGENEOUS 3-D CYLINDER OR TUBE.
C   VERSION    :  2.0
C   DATE       :  01-01-89
C   REVISED    :  04-01-90 (G. MARLEAU)
C   AUTHOR     :  R.ROY
C
C 2- PROGRAMME PARAMETERS:
C
C   INPUT PARAMETERS:
C     NGPT     :  NUMBER OF GAUSS-LEGENDRE INTEGRATION POINTS.   I
C     RAYRE    :  RADIUS OF TUBE                                 R(2)
C                 RAYRE(1)---> INTERNAL RADIUS
C                 RAYRE(2)---> EXTERNAL RADIUS
C     HAUT     :  HEIGHT OF TUBE.                                R
C     SIGT     :  REGION CROSS SECTION.                          R
C     TRONC    :  VOIDED BLOCK EXPANSION CRITERION.              R
C   OUTPUT PARAMETERS:                                           R
C     PVV      :  VOLUME TO VOLUME REDUCED PROBABILITY.          R
C     PVS      :  VOLUME TO SURFACE PROBABILITIES.            R(IDIM)
C                 PVS(KK)= PROBABILITY LEAKAGE FROM VOLUME TO
C                 SURFACE (KK=1,IDIM)
C     PSS      :  SURFACE TO SURFACE PROBABILITIES            R(IDIM,
C                                                              IDIM)
C                 PSS(JJ,KK)= PROBABILITY OF TRANSMISSION FROM
C                 SURFACE (JJ=1,IDIM) TO SURFACE (KK=1,IDIM)
C   WHERE WE USED:
C     IDIM=3   FOR RAYRE(1) = 0
C     IDIM=4   FOR RAYRE(1) > 0
C   AND THE SURFACES ARE IDENTIFIED AS FOLLOWS
C     IDIM = 3 -> OUTER SURFACE  = 1
C                 BOTTOM SURFACE = 2
C                 TOP SURFACE    = 3
C     IDIM = 4 -> INNER SURFACE  = 1
C                 OUTER SURFACE  = 2
C                 BOTTOM SURFACE = 3
C                 TOP SURFACE    = 4
C
C 3- ROUTINES AND FUNCTIONS CALLED:
C
C   ROUTINES CALLED
C     TABEN   :    EXPONENTIAL INTEGRAL OF ORDER N                   F
C     ALGPT   :    GAUSS-LEGENDRE INTEGRATION POINTS AND WEIGHTS     S
C   INTERNAL FUNCTIONS
C     SEXP    :    SAFE EXPONENTIAL                                  F
C
C 4- PARAMETER AND SAVED DATA
C
C   PARAMETERS
C     PI      : VALUE OF PI                                          R
C     PIO4    : VALUE OF PI/4                                        R
C     QOPI    : VALUE OF 4/PI                                        R
C     MGPT    : MAXIMUM NUMBER OF GAUSS POINTS AVAILABLES            I
C   SAVED
C     IVS  : INDICATOR FOR FILLING PVS MATRIX                I( 4,2)
C     ISS  : INDICATOR FOR FILLING PSS MATRIX                I(16,2)
C
C------------------------  TUBE3D  ------------------------------------
C
      SAVE       IVS,ISS
      PARAMETER (PI=3.1415926535,MGPT=64,PIO4=0.25*PI,QOPI=4.0/PI)
      INTEGER    NGPT,ITY,IVS(4,2),ISS(16,2)
      REAL       RAYRE(2),HAUT,SIGT,TRONC,PVV,PVS(*),PSS(*),
     >           XGAUS(MGPT),WGAUS(MGPT),RPHH1(3),RPHH2(3),RPHH3(3),
     >           RPHH4,RPHI1(3),RPHI2(3),RPVH1(2),RPVH2(2),
     >           RPVH3(2),RPVH4,RPVI1(2),RPVI2(2),RPVO1(2),RPVO3(2)
      DATA       IVS /3,1,2,3,1,2,3,4/
      DATA       ISS /9,9,9,9,9,1,2,3,9, 4, 5, 6, 9, 7, 8, 9,
     >                1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16/
C----
C  INTRINSIC FUNCTIONS - NO UNDERFLOW SAFE EXPONENTIAL
C----
      SEXP(X)=EXP(MAX(-50.0,X))
C----
C  INITIALIZE INTEGRATION POINTS AND WEIGHTS
C----
      XINF=0.0
      XSUP=1.0
      CALL ALGPT(NGPT,XINF,XSUP,XGAUS,WGAUS)
      ALPHA=RAYRE(2)+RAYRE(1)
      DELTA=RAYRE(2)-RAYRE(1)
      CDELTA=DELTA*DELTA
      VEE=ALPHA+DELTA
      VEEC=VEE*VEE
      QR2R1=4.0*RAYRE(2)*RAYRE(1)
      TWOR1=2.0*RAYRE(1)
      SURI=2.0*PI*RAYRE(1)*HAUT
      IF(RAYRE(1).GT.0.0) THEN
        OSURI=1.0/SURI
        ITY=2
      ELSE
        OSURI=1.0
        ITY=1
      ENDIF
      SIGCUT = 2.*SIGT*DELTA*HAUT/(DELTA+HAUT)
      UT1C=ALPHA*DELTA
      UT1=SQRT(UT1C)
      UT2C=2.0*RAYRE(1)*DELTA
      UT2=SQRT(UT2C)
      UT3C=4.0*UT1C
      UT3=2.0*UT1
      U3LB=SQRT(VEE-UT3)
      U3W=SQRT(VEE)-U3LB
      CHAUT=HAUT*HAUT
      SURO   = 2.0*PI*RAYRE(2)*HAUT
      SURD   = PI*(RAYRE(2)*RAYRE(2)-RAYRE(1)*RAYRE(1))
      VOL    = SURD*HAUT
      QSIGV  = 4.0*SIGT*VOL
      GZERO  = TABEN(3,0.0)
      GH2    = TABEN(3,SIGT*HAUT)
      FH2    = GH2/CHAUT
      DO 60 IEXP=1,3
        RPHH1(IEXP)= 0.0
        RPHH2(IEXP)= 0.0
        RPHH3(IEXP)= 0.0
        RPHI1(IEXP)= 0.0
        RPHI2(IEXP)= 0.0
 60   CONTINUE
      RPHH4= 0.0
      DO 61 IEXP=1,2
        RPVH1(IEXP)= 0.0
        RPVH2(IEXP)= 0.0
        RPVH3(IEXP)= 0.0
        RPVI1(IEXP)= 0.0
        RPVI2(IEXP)= 0.0
        RPVO1(IEXP)= 0.0
        RPVO3(IEXP)= 0.0
 61   CONTINUE
      RPVH4= 0.0
      IF (SIGCUT.GE.TRONC) THEN
C----
C  NON-VOID TUBES
C----
          SIGTOP = 1.0/(SIGT*(UT1C+CHAUT))
C----
C  FIRST CONSIDER TERMS THAT CONTRIBUTE TO BOTH HOLLOW AND FILLED TUBES
C----
          DO 70 IGPT=1,NGPT
C----
C  TERM 3: INTEGRAL OVER T FROM 0 TO UT3=2.0*SQRT(ALPHA*DELTA)
C  CHANGE OF VARIABLE  U**2=(V-T)
C  PHH3=RPHH3(1)*UT3
C  PVH3=RPVH3(1)*UT3
C  HOLLOW TUBE -> CONTRIBUTIONS TO PHH, PVH
C  FILLED TUBE -> CONTRIBUTIONS TO PHH, PVH
C----
            XE=XGAUS(IGPT)*U3W+U3LB
            CXE=XE*XE
            XIFAC=WGAUS(IGPT)*CXE*SQRT(2*VEE-CXE)
            ZE=VEE-CXE
            CZE=ZE*ZE
            CT2H2=CHAUT+CZE
            GT2   = TABEN(3,SIGT*ZE)
            GT2H2 = TABEN(3,SIGT*SQRT(CT2H2))
            RPHH3(1)=RPHH3(1)+XIFAC*(FH2-GT2H2/CT2H2)
            RPVH3(1)=RPVH3(1)+XIFAC*(GZERO-GH2-GT2+GT2H2)
 70       CONTINUE
          DO 80 IGPT=1,NGPT
C----
C  TERM 5: INTEGRAL FROM 0 TO PI/4 * 0 TO 1/COS(THETA)
C    PVO3=RPVO3*(UT3**3)*CHAUT*PIO4
C    HOLLOW TUBE -> CONTRIBUTION  TO PVO UTOP=UT3C
C    FILLED TUBE -> CONTRIBUTION  TO PVO UTOP=UT3C
C----
            CTHETA=COS(PIO4*XGAUS(IGPT))
            CTHET2=CTHETA*CTHETA
            STHETA=SIN(PIO4*XGAUS(IGPT))
            STHET2=1.0-CTHET2
            FACT1=UT3C*CTHET2
            FACT2=UT3C*STHET2
            CASL1=FACT1+CHAUT*STHET2
            CASL2=FACT2+CHAUT*CTHET2
            SL1=SQRT(CASL1)
            SL2=SQRT(CASL2)
            SSL1=-SL1*SIGT
            SSL2=-SL2*SIGT
            RRINT1=0.0
            RRINT2=0.0
            IF(SIGTOP.GT.0.1) THEN
C----
C  GAUSS-LEGENDRE INTEGRATION OVER R
C  1- FOR SIGTOP > 0.1 -> INTEGRATE DIRECTLY FROM
C     0 TO 1/(COS(THETA))
C----
              DO 81 JGPT=1,NGPT
                RE1=XGAUS(JGPT)/CTHETA
                RE1C=RE1*RE1
                RRINT1=RRINT1+(1.0-RE1*STHETA)*SEXP(SSL1*RE1)
     >                *SQRT(VEEC-FACT1*RE1C)*WGAUS(JGPT)
                RRINT2=RRINT2+(1.0-RE1*CTHETA)*SEXP(SSL2*RE1)
     >                *SQRT(VEEC-FACT2*RE1C)*WGAUS(JGPT)
 81           CONTINUE
              RPVO3(1)=RPVO3(1)+WGAUS(IGPT)*(RRINT1*CTHET2/(CASL1*SL1)
     >             +RRINT2*STHET2/(CASL2*SL2))/CTHETA
            ELSE
C----
C  GAUSS-LEGENDRE INTEGRATION OVER R
C  2- FOR SIGTOP <= 0.1 -> INTEGRATE FIRST FROM 0 TO SIGTOP
C     AND THEN FROM SIGTOP TO 1/(COS(THETA))
C----
              DO 82 JGPT=1,NGPT
                RE1=XGAUS(JGPT)*SIGTOP
                RE1C=RE1*RE1
                WTF=1.0/CTHETA-SIGTOP
                RE2=XGAUS(JGPT)*WTF+SIGTOP
                RE2C=RE2*RE2
                RRINT1=RRINT1+(1.0-RE1*STHETA)*SEXP(SSL1*RE1)
     >                *SQRT(VEEC-FACT1*RE1C)*WGAUS(JGPT)*SIGTOP
     >                       +(1.0-RE2*STHETA)*SEXP(SSL1*RE2)
     >                *SQRT(VEEC-FACT1*RE2C)*WGAUS(JGPT)*WTF
                RRINT2=RRINT2+(1.0-RE1*CTHETA)*SEXP(SSL2*RE1)
     >                *SQRT(VEEC-FACT2*RE1C)*WGAUS(JGPT)*SIGTOP
     >                       +(1.0-RE2*CTHETA)*SEXP(SSL2*RE2)
     >                *SQRT(VEEC-FACT2*RE2C)*WGAUS(JGPT)*WTF
 82           CONTINUE
              RPVO3(1)=RPVO3(1)+WGAUS(IGPT)*(RRINT1*CTHET2/(CASL1*SL1)
     >                                +RRINT2*STHET2/(CASL2*SL2))
            ENDIF
80        CONTINUE
          IF(ITY.EQ.2) THEN
            DO 170 IGPT=1,NGPT
C----
C  TERM 1: INTEGRAL FROM 0 TO UT1=SQRT(ALPHA*DELTA)
C  PHH1=RPHH1(1)*UT1
C  PVH1=RPVH1(1)*UT1
C  PHI1=RPHI1(1)*UT1
C  HOLLOW TUBE -> CONTRIBUTIONS TO PHH, PHI AND PVH
C  FILLED TUBE -> NO CONTRIBUTION
C----
              XE=XGAUS(IGPT)*UT1
              CXE=XE*XE
              CT2H2=CHAUT+CXE
              GT2  =TABEN(3,SIGT*XE)
              GT2H2=TABEN(3,SIGT*SQRT(CT2H2))
              RPHH1(1)=RPHH1(1)+WGAUS(IGPT)*(FH2-GT2H2/CT2H2)
              RPVH1(1)=RPVH1(1)+WGAUS(IGPT)*(GZERO-GH2-GT2+GT2H2)
              RPHI1(1)=RPHI1(1)+WGAUS(IGPT)*(GT2-GT2H2*CXE/CT2H2)
C----
C  TERM 2: INTEGRAL FROM 0 TO UT2=SQRT(ALPHA*DELTA-DELTA)
C  PHH2=RPHH2(1)*UT2
C  PVH2=RPVH2(1)*UT2
C  PHI2=RPHI2(1)*UT2
C  HOLLOW TUBE -> CONTRIBUTIONS TO PHH, PHI AND PVH
C  FILLED TUBE -> NO CONTRIBUTION
C----
              XE=XGAUS(IGPT)*UT2
              CXE=XE*XE
              CT2=CXE+CDELTA
              XIFAC=WGAUS(IGPT)*CXE*SQRT(QR2R1-CXE)/CT2
              CT2H2=CHAUT+CT2
              GT2  =TABEN(3,SIGT*SQRT(CT2))
              GT2H2=TABEN(3,SIGT*SQRT(CT2H2))
              RPHH2(1)=RPHH2(1)+XIFAC*(FH2-GT2H2/CT2H2)
              RPVH2(1)=RPVH2(1)+XIFAC*(GZERO-GH2-GT2+GT2H2)
              RPHI2(1)=RPHI2(1)+XIFAC*(GT2-GT2H2*CT2/CT2H2)
C----
C  TERM 3': INTEGRAL FROM 0 TO UT3=2.0*SQRT(ALPHA*DELTA)
C  PHH4=RPHH4*UT3*TWOR1
C  PVH4=RPVH4*UT3*TWOR1
C  HOLLOW TUBE -> CONTRIBUTIONS TO PHH, PVH
C  FILLED TUBE -> NO CONTRIBUTION
C----
            XE=XGAUS(IGPT)*UT3
            CXE=XE*XE
            CT2H2=CHAUT+CXE
            GT2   = TABEN(3,SIGT*XE)
            GT2H2 = TABEN(3,SIGT*SQRT(CT2H2))
            RPHH4=RPHH4+WGAUS(IGPT)*(FH2-GT2H2/CT2H2)
            RPVH4=RPVH4+WGAUS(IGPT)*(GZERO-GH2-GT2+GT2H2)
C----
C  TERM 4: INTEGRAL FROM 0 TO PI/4
C  HOLLOW TUBE ->
C        1 CONTRIBUTION  TO PVI : UTOP=UT1C
C        2 CONTRIBUTIONS TO PVO : UTOP=CDELTA AND UTOP=UT3C
C  FILLED TUBE -> NO CONTRIBUTION
C----
              CTHETA=COS(PIO4*XGAUS(IGPT))
              CTHET2=CTHETA*CTHETA
              STHETA=SIN(PIO4*XGAUS(IGPT))
              STHET2=1.0-CTHET2
C----
C  UTOP=UT1C
C  PVI1=RPVI1*(UT1**3)*CHAUT*PIO4/SIGT
C----
              CASL1=UT1C*CTHET2+STHET2*CHAUT
              CASL2=UT1C*STHET2+CTHET2*CHAUT
              SL1=SQRT(CASL1)
              SL2=SQRT(CASL2)
              FACT1=STHETA/(SIGT*SL1)
              FACT2=CTHETA/(SIGT*SL2)
              RPVI1(1)=RPVI1(1)+WGAUS(IGPT)*
     >              (CTHET2*(1.0-FACT1
     >              +SEXP(-SIGT*SL1/CTHETA)*(FACT1+STHETA/CTHETA-1.0))
     >              /(CASL1*CASL1)
     >              +STHET2*(1.0-FACT2
     >              +SEXP(-SIGT*SL2/CTHETA)*FACT2)/(CASL2*CASL2))
C----
C  UTOP=UT3C
C  PVO1=RPVO1*(UT3**3)*CHAUT*PIO4/SIGT
C----
              CASL1=UT3C*CTHET2+STHET2*CHAUT
              CASL2=UT3C*STHET2+CTHET2*CHAUT
              SL1=SQRT(CASL1)
              SL2=SQRT(CASL2)
              FACT1=STHETA/(SIGT*SL1)
              FACT2=CTHETA/(SIGT*SL2)
              RPVO1(1)=RPVO1(1)+WGAUS(IGPT)*
     >              (CTHET2*(1.0-FACT1
     >              +SEXP(-SIGT*SL1/CTHETA)*(FACT1+STHETA/CTHETA-1.0))
     >              /(CASL1*CASL1)
     >              +STHET2*(1.0-FACT2
     >              +SEXP(-SIGT*SL2/CTHETA)*FACT2)/(CASL2*CASL2))
 170        CONTINUE
            DO 180 IGPT=1,NGPT
C----
C  TERM 6: INTEGRAL FROM 0 TO PI/4 * 0 TO 1/COS(THETA)
C    PVI2=RPVI2*(UT2**3)*CHAUT*PIO4
C    HOLLOW TUBE -> CONTRIBUTION  TO PVO, PVI : UTOP=UT2C
C    FILLED TUBE -> NO CONTRIBUTION
C----
              CTHETA=COS(PIO4*XGAUS(IGPT))
              CTHET2=CTHETA*CTHETA
              STHETA=SIN(PIO4*XGAUS(IGPT))
              STHET2=1.0-CTHET2
              FACT1=UT2C*CTHET2
              FACT2=UT2C*STHET2
              FSL1=FACT1+CHAUT*STHET2
              FSL2=FACT2+CHAUT*CTHET2
              RRINT1=0.0
              RRINT2=0.0
              IF(SIGTOP.GT.0.1) THEN
C----
C  GAUSS-LEGENDRE INTEGRATION OVER R
C  1- FOR SIGTOP > 0.1 -> INTEGRATE DIRECTLY FROM
C     0 TO 1/(COS(THETA))
C----
                DO 181 JGPT=1,NGPT
                  RE1=XGAUS(JGPT)/CTHETA
                  RE1C=RE1*RE1
                  RE1CU=RE1C*RE1
                  CASL1=FSL1*RE1C+CDELTA
                  CASL2=FSL2*RE1C+CDELTA
                  SL1=SQRT(CASL1)
                  SL2=SQRT(CASL2)
                  RRINT1=RRINT1+(1.0-RE1*STHETA)*SEXP(-SL1*SIGT)
     >                  *SQRT(QR2R1-FACT1*RE1C)
     >                  *RE1CU*WGAUS(JGPT)/(CASL1*SL1)
                  RRINT2=RRINT2+(1.0-RE1*CTHETA)*SEXP(-SL2*SIGT)
     >                  *SQRT(QR2R1-FACT2*RE1C)
     >                  *RE1CU*WGAUS(JGPT)/(CASL2*SL2)
 181            CONTINUE
                RPVI2(1)=RPVI2(1)+WGAUS(IGPT)
     >               *(RRINT1*CTHET2+RRINT2*STHET2)/CTHETA
              ELSE
C----
C  GAUSS-LEGENDRE INTEGRATION OVER R
C  2- FOR SIGTOP <= 0.1 -> INTEGRATE FIRST FROM 0 TO SIGTOP
C     AND THEN FROM SIGTOP TO 1/(COS(THETA))
C----
                DO 182 JGPT=1,NGPT
                  RE1=XGAUS(JGPT)*SIGTOP
                  RE1C=RE1*RE1
                  RE1CU=RE1C*RE1
                  CASL1=FSL1*RE1C+CDELTA
                  CASL2=FSL2*RE1C+CDELTA
                  SL1=SQRT(CASL1)
                  SL2=SQRT(CASL2)
                  WTF=1.0/CTHETA-SIGTOP
                  RE2=XGAUS(JGPT)*WTF+SIGTOP
                  RE2C=RE2*RE2
                  RE2CU=RE2C*RE2
                  CASL3=FSL1*RE2C+CDELTA
                  CASL4=FSL2*RE2C+CDELTA
                  SL3=SQRT(CASL3)
                  SL4=SQRT(CASL4)
                  RRINT1=RRINT1+(1.0-RE1*STHETA)*SEXP(-SIGT*SL1)
     > *SQRT(QR2R1-FACT1*RE1C)*RE1CU*SIGTOP*WGAUS(JGPT)/(CASL1*SL1)
     >                         +(1.0-RE2*STHETA)*SEXP(-SIGT*SL3)
     > *SQRT(QR2R1-FACT1*RE2C)*RE2CU*WTF*WGAUS(JGPT)/(CASL3*SL3)
                  RRINT2=RRINT2+(1.0-RE1*CTHETA)*SEXP(-SIGT*SL2)
     > *SQRT(QR2R1-FACT2*RE1C)*RE1CU*SIGTOP*WGAUS(JGPT)/(CASL2*SL2)
     >                       +(1.0-RE2*CTHETA)*SEXP(-SIGT*SL4)
     > *SQRT(QR2R1-FACT2*RE2C)*RE2CU*WTF*WGAUS(JGPT)/(CASL4*SL4)
 182            CONTINUE
                RPVI2(1)=RPVI2(1)+WGAUS(IGPT)
     >                  *(RRINT1*CTHET2+RRINT2*STHET2)
              ENDIF
 180        CONTINUE
          ENDIF
C----
C  RECONSTRUCT PVS AND PSS MATRIX
C----
          PVS(IVS(1,ITY))=CHAUT*PIO4*(TWOR1*UT1C*UT1*RPVI1(1)/SIGT
     >       -UT2*UT2C*RPVI2(1))/VOL
          PVS(IVS(2,ITY))=CHAUT*PIO4*(UT3C*UT3*RPVO3(1)
     >       -UT2*UT2C*RPVI2(1)-TWOR1*(UT3C*UT3*RPVO1(1)
     >       -UT1C*UT1*RPVI1(1))/SIGT)/VOL
          PVS(IVS(3,ITY))=(U3W*RPVH3(1)-0.5*UT3*TWOR1*RPVH4
     >       +TWOR1*UT1*RPVH1(1)-UT2*RPVH2(1))/(VOL*SIGT)
          PVV=(1.0-PVS(IVS(1,ITY))-PVS(IVS(2,ITY))
     >       -2.0*PVS(IVS(3,ITY)))/SIGT
          PVS(IVS(4,ITY))=PVS(IVS(3,ITY))
          PSS(ISS( 1,ITY))=0.0
          PSS(ISS(11,ITY))=0.0
          PSS(ISS( 3,ITY))=2.0*(TWOR1*UT1*RPHI1(1)-UT2*RPHI2(1))/SURD
          PSS(ISS( 4,ITY))=PSS(ISS(3,ITY))
          PSS(ISS( 9,ITY))=OSURI*SURD*PSS(ISS(3,ITY))
          PSS(ISS(13,ITY))=PSS(ISS(9,ITY))
          PSS(ISS( 5,ITY))=1.0-2.0*PSS(ISS(9,ITY))
     >                    -QSIGV*PVS(IVS(1,ITY))*OSURI
          PSS(ISS( 2,ITY))=SURI*PSS(ISS( 5,ITY))/SURO
          PSS(ISS(12,ITY))=2.0*CHAUT*(2.*U3W*RPHH3(1)-UT3*TWOR1*RPHH4
     >       +2.0*TWOR1*UT1*RPHH1(1)-2.0*UT2*RPHH2(1))/SURD
          PSS(ISS( 7,ITY))=1.0-PSS(ISS(12,ITY))-PSS(ISS( 3,ITY))
     >                    -QSIGV*PVS(IVS(3,ITY))/SURD
          PSS(ISS( 8,ITY))=PSS(ISS( 7,ITY))
          PSS(ISS(10,ITY))=SURD*PSS(ISS( 7,ITY))/SURO
          PSS(ISS(14,ITY))=PSS(ISS(10,ITY))
          PSS(ISS(15,ITY))=PSS(ISS(12,ITY))
          PSS(ISS( 6,ITY))=1.0-PSS(ISS(2,ITY))-2.0*PSS(ISS(10,ITY))
     >                    -QSIGV*PVS(IVS(2,ITY))/SURO
          PSS(ISS(16,ITY))=0.0
      ELSE
C----
C  VOID LIMIT
C  FIRST CONSIDER TERMS THAT CONTRIBUTE TO BOTH HOLLOW AND FILLED TUBES
C  F1(1)=0.5; F1(2)=SIGT, F1(3)=-SIGT**2/4
C  F2(1)=1.0; F2(2)=-SIGT/4.0
C
C  TERM 3: INTEGRAL FROM 0 TO UT3=2.0*SQRT(ALPHA*DELTA)
C  PHH3(I)=RPHH3(I)*UT3
C  PVH3(I)=RPVH3(I)*UT3
C  PVO3(I)=RPVO3(I)*UT3
C  HOLLOW TUBE -> CONTRIBUTIONS TO PHH, PVH, PVO
C  FILLED TUBE -> CONTRIBUTIONS TO PHH, PVH, PVO
C----
          SVMU3 =SQRT(VEEC-UT3C)
          ASVOU3=ASIN(UT3/VEE)
          RPHH3(2)=-0.5*(UT3*SVMU3+VEEC*ASVOU3)/(HAUT*2.0*U3W)
          RPVH3(1)=(VEEC*VEE/3.0+0.5*HAUT*VEEC*ASVOU3
     >   +(0.5*HAUT*UT3+(UT3C-VEEC)/3.0)*SVMU3)/(2.0*U3W)
          DO 90 IGPT=1,NGPT
            XE=XGAUS(IGPT)*U3W+U3LB
            CXE=XE*XE
            XIFAC=WGAUS(IGPT)*CXE*SQRT(2*VEE-CXE)
            ZE=VEE-CXE
            CZE=ZE*ZE
            CT2H2 =1.0/(CHAUT+CZE)
            T2H2  =SQRT(CT2H2)
            AT2H2 =ALOG(CHAUT*CT2H2)
            ATT2H2 =ALOG(CZE*CT2H2)
            RPHH3(1)=RPHH3(1)+XIFAC*CZE*CT2H2
            RPHH3(2)=RPHH3(2)+XIFAC*T2H2
            RPHH3(3)=RPHH3(3)-XIFAC*AT2H2
            RPVH3(1)=RPVH3(1)-XIFAC/T2H2
            RPVH3(2)=RPVH3(2)-XIFAC*(CHAUT*AT2H2+CZE*ATT2H2)
            RPVO3(1)=RPVO3(1)+XIFAC*((CHAUT+CZE)*T2H2-ZE)
            RPVO3(2)=RPVO3(2)+XIFAC*(ZE*HAUT*ATAN(HAUT/ZE)
     >     +0.5*CZE*ATT2H2)
90        CONTINUE
          IF(ITY.EQ.2) THEN
C----
C  HOLLOW TUBE -> CONTRIBUTIONS TO PHH, PHI AND PVH
C  FILLED TUBE -> NO CONTRIBUTION
C  PHH1(I)=RPHH1(I)
C----
            CUHAUT=HAUT*CHAUT
            CUT1=UT1*UT1C
            CUT3=UT3*UT3C
            ATU1OH=ATAN(UT1/HAUT)
            ATU3OH=ATAN(UT3/HAUT)
            SU3PH=SQRT(UT3C+CHAUT)
            SU1PH=SQRT(UT1C+CHAUT)
            SDPH=SQRT(CDELTA+CHAUT)
            ALU1OH=ALOG(SU1PH/HAUT)
            ALU1OT=ALOG(SU1PH/UT1)
            ALU3OH=ALOG(SU3PH/HAUT)
            ALU3OT=ALOG(SU3PH/UT3)
            ALDOT=ALOG(SDPH/DELTA)
            ALS1OH=ALOG((UT1+SU1PH)/HAUT)
            ALS3OH=ALOG((UT3+SU3PH)/HAUT)
            ALSDOH=ALOG((DELTA+SDPH)/HAUT)
            RPHH1(1)=HAUT*ATU3OH-2.0*HAUT*ATU1OH
            RPHH1(2)=2.0*ALS1OH-ALOG((UT3+SU3PH)/HAUT)
            RPHH1(3)=2.0*UT3*(ALU1OH-ALU3OH)+4.0*HAUT*ATU1OH
     >     -2.0*HAUT*ATU3OH
C----
C  PHI1(I)=RPHI1(I)
C----
            RPHI1(1)=ATU1OH/HAUT
            RPHI1(2)=0.5*(UT1*SU1PH-UT1C-CHAUT*ALS1OH)
            RPHI1(3)=2.0*(CUT1*ALU1OT-CUHAUT*ATU1OH+UT1*CHAUT)/3.0
C----
C  PVH1(I)=RPVH1(I)
C----
            RPVH1(1)=0.5*(UT1*(SU3PH-SU1PH)-UT1C-CHAUT*(ALS1OH
     >     -0.5*ALS3OH))
            RPVH1(2)=(2.0*CUHAUT*(2.0*ATU1OH-ATU3OH)
     >     +6.0*UT1*CHAUT*(ALU1OH-ALU3OH)
     >     +2.0*CUT1*ALU1OT-CUT3*ALU3OT)/3.0
C----
C  TERM 4: INTEGRAL FROM 0 TO UT1C
C  HOLLOW TUBE -> CONTRIBUTION  TO PVI : UTOP=UT1C
C  FILLED TUBE -> NO CONTRIBUTION
C  PVI1(I)=RPVI1(I)
C----
            RPVI1(1)=0.5*(CHAUT*ALS1OH+UT1*SU1PH-UT1C)
            RPVI1(2)=(CHAUT*UT1-CUHAUT*ATU1OH
     >         +3.0*HAUT*UT1C*ATAN(HAUT/UT1)-2.0*CUT1*ALU1OT)/6.0
            ATU1OH=ATAN(UT1/HAUT)
C----
C  TERM 4: INTEGRAL FROM CDELTA TO UT3C
C  HOLLOW TUBE -> CONTRIBUTIONS TO PVO
C  FILLED TUBE -> NO CONTRIBUTION
C  PVO1(I)=RPVO1(I)
C----
            RPVO1(1)=0.5*(CHAUT*ALS3OH+UT3*SU3PH-UT3C)
            RPVO1(2)=(CHAUT*UT3-CUHAUT*ATU3OH
     >        +3.0*HAUT*UT3C*ATAN(HAUT/UT3) -2.0*CUT3*ALU3OT)/6.0
            DO 190 IGPT=1,NGPT
C----
C  TERM 2: INTEGRAL FROM 0 TO UT2=SQRT(ALPHA*DELTA-DELTA)
C  HOLLOW TUBE -> CONTRIBUTIONS TO PHH, PHI AND PVH
C  FILLED TUBE -> NO CONTRIBUTION
C----
              XE=XGAUS(IGPT)*UT2
              CXE=XE*XE
              CT2=CXE+CDELTA
              T2=SQRT(CT2)
              CT2H2=CHAUT+CT2
              T2H2=SQRT(CT2H2)
              ALH=ALOG(CT2H2/CHAUT)
              ALT=ALOG(CT2H2/CT2)
              XIFAC=WGAUS(IGPT)*CXE*SQRT(QR2R1-CXE)
              RPHH2(1)=RPHH2(1)+XIFAC/CT2H2
              RPHI2(2)=RPHI2(2)+XIFAC*(1.0/T2H2-1.0/T2)
              RPHI2(3)=RPHI2(3)+XIFAC*ALT
              RPVI2(1)=RPVI2(1)+XIFAC*((CHAUT/CT2+1.0)/T2H2-1.0/T2)
              RPVI2(2)=RPVI2(2)+XIFAC*(HAUT*ATAN(HAUT/T2)/T2
     >       -0.5*ALT)
              XIFAC=XIFAC/CT2
              RPHH2(2)=RPHH2(2)+XIFAC*(1.0/T2H2-1.0/HAUT)
              RPHH2(3)=RPHH2(3)+XIFAC*ALH
              RPHI2(1)=RPHI2(1)+XIFAC/CT2H2
              RPVH2(1)=RPVH2(1)+XIFAC*(T2+HAUT-T2H2)
              RPVH2(2)=RPVH2(2)+XIFAC*(CHAUT*ALH+CT2*ALT)
 190        CONTINUE
          ENDIF
C----
C  RECONSTRUCT PVS AND PSS MATRIX
C----
          PVS(IVS(1,ITY))=( TWOR1*RPVI1(1)-UT2*RPVI2(1)
     >       - SIGT*( TWOR1*RPVI1(2)-UT2*RPVI2(2) ) ) /VOL
          PVS(IVS(2,ITY))=( 2.0*U3W*RPVO3(1)-TWOR1*(RPVO1(1)-RPVI1(1))
     >       -UT2*RPVI2(1)-SIGT*(2.0*U3W*RPVO3(2)
     >       -TWOR1*(RPVO1(2)-RPVI1(2))
     >       -UT2*RPVI2(2) ) )/VOL
          PVS(IVS(3,ITY))=( U3W*RPVH3(1)+TWOR1*RPVH1(1)
     >       -UT2*RPVH2(1)- 0.25*SIGT*( U3W*RPVH3(2)
     >       +TWOR1*RPVH1(2)-UT2*RPVH2(2) ) )/VOL
          PVV1=VOL-(TWOR1*RPVI1(1)-UT2*RPVI2(1))-(2.0*U3W*RPVO3(1)
     >       -TWOR1*(RPVO1(1)-RPVI1(1))-UT2*RPVI2(1))
     >       -2.0*(U3W*RPVH3(1)+TWOR1*RPVH1(1)-UT2*RPVH2(1))
          PVV=(TWOR1*RPVI1(2)-UT2*RPVI2(2)+(2.0*U3W*RPVO3(2)
     >       -TWOR1*(RPVO1(2)-RPVI1(2))-UT2*RPVI2(2))
     >       +0.5*(U3W*RPVH3(2)+TWOR1*RPVH1(2)-UT2*RPVH2(2)))/VOL
          PVS(IVS(4,ITY))=PVS(IVS(3,ITY))
          PSS(ISS( 1,ITY))=0.0
          PSS(ISS(11,ITY))=0.0
          PSS(ISS( 3,ITY))=( CHAUT*( TWOR1*RPHI1(1)-UT2*RPHI2(1) )
     >       +2.0*SIGT*( TWOR1*RPHI1(2)-UT2*RPHI2(2)
     >       + 0.25*SIGT*( TWOR1*RPHI1(3)-UT2*RPHI2(3) ) ) )/SURD
          PSS(ISS( 4,ITY))=PSS(ISS(3,ITY))
          PSS(ISS( 9,ITY))=OSURI*SURD*PSS(ISS(3,ITY))
          PSS(ISS(13,ITY))=PSS(ISS(9,ITY))
          PSS(ISS( 5,ITY))=1.0-2.0*PSS(ISS(9,ITY))
     >                    -QSIGV*PVS(IVS(1,ITY))*OSURI
          PSS(ISS( 2,ITY))=SURI*PSS(ISS( 5,ITY))/SURO
          PSS(ISS(12,ITY))=( 2.0*U3W*RPHH3(1)+TWOR1*RPHH1(1)
     >       -2.0*UT2*RPHH2(1)+ 2.0*CHAUT*SIGT*( 2.0*U3W*RPHH3(2)
     >       +TWOR1*RPHH1(2)-2.0*UT2*RPHH2(2)+ 0.25*SIGT
     >       *(2.0*U3W*RPHH3(3) +TWOR1*RPHH1(3)-2.0*UT2*RPHH2(3)
     >        ) ) )/SURD
          PSS(ISS( 7,ITY))=1.0-PSS(ISS(12,ITY))-PSS(ISS( 3,ITY))
     >                    -QSIGV*PVS(IVS(3,ITY))/SURD
          PSS(ISS( 8,ITY))=PSS(ISS( 7,ITY))
          PSS(ISS(10,ITY))=SURD*PSS(ISS( 7,ITY))/SURO
          PSS(ISS(14,ITY))=PSS(ISS(10,ITY))
          PSS(ISS(15,ITY))=PSS(ISS(12,ITY))
          PSS(ISS( 6,ITY))=1.0-PSS(ISS(2,ITY))-2.0*PSS(ISS(10,ITY))
     >                    -QSIGV*PVS(IVS(2,ITY))/SURO
          PSS(ISS(16,ITY))=0.0
      ENDIF
      RETURN
      END
