*DECK XCSCVS
      SUBROUTINE XCSCVS(IAPPR,IROT,IOR,IVAR,NSURF,CHORD,SIGT,
     >                  PII,PIS,PSS)
C
C---------------------------  XCSCVS  ---------------------------------
C
C 1-  PROGRAMME STATISTICS:
C
C   NAME       :  XCSCVS
C   FUNCTION   :  COMPLETE PSS AND EVALUATE PIS AND PII
C                 FROM SYMMETRY AND CONSERVATION
C   VERSION    :  1.0
C   REVISED    :  13-11-1989
C   AUTHOR     :  G.MARLEAU
C
C 2-  PROGRAMME PARAMETERS:
C
C   IAPPR   : ANGULAR EXPANSION ORDER                    I
C             IAPP = 1 -> DP0 ALL
C                         NSUR=3, NDIR=1, NPRB=3
C             IAPP = 2 -> DP1 ALL OR DP1 INSIDE DP0 OUTSIDE
C                         NSUR=3, NDIR=2, NPRB=NSUR*NDIR
C   IROT    : TYPE OF RECONSTRUCTION
C             IROT = 1 -> DIRECT RECONSTRUCTION
C             IROT = 2 -> ROT2 RECONSTRUCTION
C   IOR     : LEVEL OF CONSERVATION USED                 I
C             IOR = 1 NO CONSERVATION RULES
C             IOR = 2 PSS CONSERVATION
C             IOR = 3 PSS AND PIS CONSERVATION RULES
C   IVAR    : TYPE OF SUBGEOMETRY                        I
C             IVAR = 1 FILLED TUBE
C             IVAR = 2 ROD CLUSTER WITHOUT INNER REGION
C             IVAR = 3 TUBE
C             IVAR = 4 ROD CLUSTER WITH INNER REGION
C   NSURF   : NUMBER OF SURFACES                         I
C   CHORD   : CHORD PER SURFACE (4V/S)                   R(NSURF)
C   SIGT    : REGION TOTAL CROSS SECTION                 R
C             FOR VOID REGION SIGT = 1.0
C   PII     : COLLISION PROBABILITY                      R
C   PIS     : LEAKAGE PROBABILITY                        R(NPRB)
C   PSS     : TRANSMISSION PROBABILITY                   R(NPRB*NPRB)
C
C 3-  PARAMETERS IN ROUTINE:
C
C   IPC,JPC  : PSS POSITION VECTOR FOR SYMMETRY RULES
C              SURFACE 1 TO 2 AND 3
C              IPC   -- FINAL POSITION IN PSS             I(3,2,2)
C              JPC   -- INITIAL POSITION IN PSS           I(3,2,2)
C   IIPC,JJPC: PSS POSITION VECTOR FOR SYMMETRY RULES
C             SURFACE 2 TO 3
C             IIPC   -- FINAL POSITION IN PSS             I(2,2,2)
C             JJPC   -- INITIAL POSITION IN PSS           I(2,2,2)
C   CENTRAL REGION ABSENT, IROT=1 (IVAR=2)
C    IAPPR = 1     IAPPR = 2
C     1 JJ=2           1  2  JJ= 3  4
C  II=3    4           5  6      7  8
C                  II= 9 10     11 12
C                     13 14     15 16
C   CENTRAL REGION ABSENT, IROT=2 (IVAR=2)
C       IAPPR = 1       IAPPR = 2
C      1    2    3       1   2      3   4      5   6
C      4    5 JJ=6       7   8      9  10     11  12
C      7 II=8    9      13  14     15  16  JJ=17  18
C                       19  20     21  22     23  24
C                       25  26  II=27  28     29  30
C                       31  32     33  34     35  36
C   RODS ABSENT, IROT=1, IROT=2, (IVAR=3)
C    IAPPR = 1     IAPPR = 2
C     1  J=2           1  2   J= 3  4
C   I=3    4           5  6      7  8
C                   I= 9 10     11 12
C                     13 14     15 16
C   ROD PRESENT, IROT=1 (IVAR=4)
C       IAPPR = 1       IAPPR = 2
C      1  J=2  J=3       1   2   J= 3   4   J= 5   6
C    I=4    5 JJ=6       7   8      9  10     11  12
C    I=7 II=8    9    I=13  14     15  16  JJ=17  18
C                       19  20     21  22     23  24
C                     I=25  26  II=27  28     29  30
C                       31  32     33  34     35  36
C   RODS PRESENT, IROT=2 (IVAR=4)
C       IAPPR = 1       IAPPR = 2
C      1 J=2     3   J=4       1   2   J= 3   4      5   6   J= 7   8
C   I= 5   6     7     8       9  10     11  12     13  14     15  16
C      9  10    11 JJ=12    I=17  18     19  20     21  22     23  24
C   I=13  14 II=15    16      25  26     27  28     29  30     31  32
C                             33  34     35  36     37  38  JJ=39  40
C                             41  42     43  44     45  46     47  48
C                           I=49  50     51  52  II=53  54     55  56
C                             57  58     59  60     61  62     63  64
C   RODS PRESENT, IROT=2
C       IAPPR = 1       IAPPR = 2
C      1 J=2     3   J=4       1   2   J= 3   4      5   6   J= 7   8
C   I= 5   6     7     8       9  10     11  12     13  14     15  16
C      9  10    11 JJ=12    I=17  18     19  20     21  22     23  24
C   I=13  14 II=15    16      25  26     27  28     29  30     31  32
C                             33  34     35  36     37  38  JJ=39  40
C                             41  42     43  44     45  46     47  48
C                           I=49  50     51  52  II=53  54     55  56
C                             57  58     59  60     61  62     63  64
C
C---------------------------   XCSCVS  --------------------------------
C
      SAVE       IPC,JPC,IIPC,JJPC
      PARAMETER (NTYP=6,PI=3.1415926535898,ERRM=0.0001,
     >           PIO2=PI/2.0,NDIR=2,NSUR=3,NZVAR=3)
      INTEGER    IAPPR,IOR,IVAR,NSURF,
     >           IPC(NZVAR,2,2),JPC(NZVAR,2,2),IIPC(2,2,2),JJPC(2,2,2)
      REAL       CHORD(NSURF),SIGT,PII,PIS(*),PSS(*)
C----
C  SYMMETRY
C----
      DATA  JPC  / 2, 2, 3, 3, 3, 5, 2, 2, 4, 3, 3, 7/
      DATA  IPC  / 3, 4, 7, 9,13,25, 3, 5,13, 9,17,49/
      DATA JJPC  / 2, 6, 3,17, 6,12,17,39/
      DATA IIPC  / 3, 8, 9,27, 8,15,27,53/
C----
C  IMPOSE SYMMETRY FOR P(2->3) AND P(3->2)
C----
      IF(MOD(IVAR,2).EQ.0) THEN
        JK=IVAR/2
        IP00=IIPC(JK,IAPPR,IROT)
        JP00=JJPC(JK,IAPPR,IROT)
        IF(IVAR.EQ.2) THEN
          RS=CHORD(1)/CHORD(IAPPR*IROT+1)
        ELSE
          RS=CHORD(IAPPR+1)/CHORD(IAPPR*(IROT+1)+1)
        ENDIF
        PSSX1=0.5*(RS*PSS(JP00)+PSS(IP00))
C       EX1=ABS((PSSX1-PSS(IP00))/PSSX1)
        PSS(JP00)=PSSX1/RS
        PSS(IP00)=PSSX1
        IF(IAPPR.EQ.2) THEN
          IP10=IP00+1
          IP11=IP10+NSURF
          IP01=IP11-1
          JP10=JP00+1
          JP11=JP10+NSURF
          JP01=JP11-1
          PSSX1=0.5*(RS*PSS(JP01)+PSS(IP10))
C         EX1=EX1+ABS((PSSX1-PSS(IP10))/PSSX1)
          PSS(JP01)=PSSX1/RS
          PSS(IP10)=PSSX1
          PSSX1=0.5*(RS*PSS(JP10)+PSS(IP01))
C         EX1=EX1+ABS((PSSX1-PSS(IP01))/PSSX1)
          PSS(JP10)=PSSX1/RS
          PSS(IP01)=PSSX1
          PSSX1=0.5*(RS*PSS(JP11)+PSS(IP11))
C         EX1=EX1+ABS((PSSX1-PSS(IP11))/PSSX1)
          PSS(JP11)=PSSX1/RS
          PSS(IP11)=PSSX1
        ENDIF
      ENDIF
C----
C  COMPLETE PSS MATRIX FROM SYMMETRY
C----
      IZVAR=IVAR-2
      IF(IZVAR.GT.0) THEN
        JJSUR=IAPPR+1
        DO 100 JJ=IZVAR,2*IZVAR-1
          RSUF=CHORD(1)/CHORD(JJSUR)
          IP00=IPC(JJ,IAPPR,IROT)
          JP00=JPC(JJ,IAPPR,IROT)
          PSS(IP00)=RSUF*PSS(JP00)
          IF(IAPPR.EQ.2) THEN
            IP10=IP00+1
            IP11=IP10+NSURF
            IP01=IP11-1
            JP10=JP00+1
            JP11=JP10+NSURF
            JP01=JP11-1
            PSS(IP01)=RSUF*PSS(JP10)
            PSS(IP10)=RSUF*PSS(JP01)
            PSS(IP11)=RSUF*PSS(JP11)
          ENDIF
          JJSUR=JJSUR+IROT*IAPPR
 100    CONTINUE
      ENDIF
      IST=IAPPR*NSURF
      NPSS=NSURF*NSURF
      IF(IOR.GT.1) THEN
        XVOL=1.0/SIGT
        DO 200 ISUR=1,NSURF
          DO 210 JSUR=ISUR,NPSS,IST
            PIS(ISUR)=PIS(ISUR)-PSS(JSUR)
 210      CONTINUE
          PIS(ISUR)=PIS(ISUR)*XVOL/CHORD(ISUR)
 200    CONTINUE
        IF(IOR.GT.2) THEN
          DO 240 JSUR=1,NSURF,IAPPR
            PII=PII-PIS(JSUR)
 240      CONTINUE
          PII=PII*XVOL
        ENDIF
      ENDIF
      RETURN
C----
C  FORMAT
C----
C6000 FORMAT(' P(2->3)/P(3->2) SYMMETRY ERROR =',1P,E15.7,'%')
      END
