*DECK XCSPIJ
      SUBROUTINE XCSPIJ(IAPP,IROT,NGEN,NBAN,NRT,MSROD,NRINFO,RAN,
     >                  NRODS,RODS,RODR,NINT,IFTRAK,CHORD,NSURF,
     >                  ISURF,NSOUT,COTE,MASK,TRAK,SIGT,PII,PIS,
     >                  PSS,IMPX)
C
C----------------------------------------------------------------------
C
C COMPUTE ALL PROBABILITIES ASSOCIATED TO THIS CLUSTER GEOMETRY
C
C 1-  SUBROUTINE STATISTICS:
C
C          NAME      -> XCSPIJ
C          VERSION   -> 3.0
C          DATE      -> 28-06-1990
C          AUTHOR    -> G. MARLEAU
C
C 2-  PARAMETERS:
C
C INPUT
C  IAPP    : ANGULAR EXPANSION ORDER                       I
C             IAPP = 1 -> DP0
C             IAPP = 2 -> DP1
C             IAPP = 3 -> DP1 INSIDE DP0 OUTSIDE
C  IROT    : TYPE OF RECONSTRUCTION                        I
C             IROT = 1 -> DIRECT RECONSTRUCTION
C             IROT = 2 -> ROT2 RECONSTRUCTION
C  NGEN    : TOTAL NUMBER OF REGIONS                       I
C  NBAN    : NUMBER OF CONCENTRIC REGIONS                  I
C  NRT     : NUMBER OF ROD TYPES                           I
C  MSROD   : MAXIMUM NUMBER OF SUBROD PER RODS             I
C  NRINFO  : TYPE OF CONCENTRIC REGION                     I(2,NBAN)
C            NSIDES(1,IAN) = NEW REGION NUMBER
C            NSIDES(2,IAN) = ASSOCIATED CLUSTER NUMBER
C                          = 0 NO CLUSTER ASSOCIATED
C  RAN     : RADIUS / LATTICE SIDE OF CONCENTRIC REGION    R(NBAN)
C  NRODS   : INTEGER DESCRIPTION OF ROD OF A GIVEN TYPE    I(3,NRT)
C            NRODS(1,IRT) = NUMBER OF ROD
C            NRODS(2,IRT) = NUMBER OF SUBRODS IN ROD
C            NRODS(3,IRT) = ASSOCIATED CONCENTRIC REGION
C  RODS    : REAL DESCRIPTION OF ROD OF A GIVEN TYPE       R(2,NRT)
C            RODS(1,IRT) = ROD CENTER RADIUS
C            RODS(2,IRT) = ANGULAR POSITION OF FIRST ROD
C  RODR    : SUBROD RADIUS                            R(MSROD,NRT)
C  NINT    : INTEGRATION PARAMETERS                        I(2)
C            NINT(1) = NUMBER OF ANGLES CONSIDERED
C            NINT(2) = NUMBER OF PARALLEL TRACKS PER ANGLE
C            NINT(3) = TUBES TRACKING PARAMETER
C  IFTRAK  : UNIT ALLOCATED TO TRACKING FILE               C*7
C  CHORD   : MEAN CHORD FOR EACH CURRENT                   R(NSURF)
C  NSURF   : NUMBER OF CURRENTS ( NSURF<MAXJ )             I
C  ISURF   : NUMBER OF SURFACES PER REGIONS                I(NGEN)
C  NSOUT   : NUMBER OF OUTER SURFACES                      I
C  COTE    : DIMENSION OF Y SIDE FOR RECTANGLE             R
C  MASK    : COMPUTE PROBABILITIES                         L(NGEN)
C            ONLY IN CELL WITH MASK=.TRUE.
C  TRAK    : VECTOR TO STORE TRACKS                        R(*)
C  SIGT    : MIXTURE CROSS SECTIONS                        R(NGEN)
C  PII     : COLLISION PROBABILITY                         R(NGEN)
C  PIS     : LEAKAGE PROBABILITY                           R(NSURF)
C  PSS     : TRANSMISSION PROBABILITY                    R(NSURF,NSURF)
C  IMPX    : IMPRESSION LEVEL                              I
C
C 3- INTERNAL PARAMETERS
C TRONC   : VOID LIMIT = 1.0E-2
C----------------------------------------------------------------------
C
      INTEGER    IOUT
      REAL       TRONC
      PARAMETER (IOUT=6,TRONC=1.0E-2)
      INTEGER    IAPP,IROT,NGEN,NBAN,NRT,MSROD,NRINFO(2,NBAN),
     >           NRODS(3,NRT),NINT(3),NSURF,NSOUT,ISURF(NGEN),
     >           IMPX,NTRAK(6)
      LOGICAL    MASK(NGEN)
      REAL       RAN(NBAN),RODS(2,NRT),RODR(MSROD,NRT),CHORD(NSURF),
     >           TRAK(*),SIGT(NGEN),PII(NGEN),COTE,
     >           PIS(*),PSS(*),RAYRE(2),RAYRR(3),VD(6,2,2,3)
      COMMON     BASE(1)
      IF(IAPP.EQ.3) THEN
        IAPPR=2
      ELSE
        IAPPR=IAPP
      ENDIF
      RAYRE(2)=0.0
      ISF=1
      ISF2=1
      IF(NSOUT.EQ.1) THEN
        NTAN=NBAN
      ELSE
        NTAN=NBAN-1
      ENDIF
      DO 110 IAN=1,NTAN
        IREG=NRINFO(1,IAN)
        IRT=NRINFO(2,IAN)
        RAYRE(1)=RAYRE(2)
        RAYRE(2)=RAN(IAN)
        IF(IMPX.GE.10) THEN
          WRITE(IOUT,6000) IAN,IREG,ISURF(IREG),RAYRE(1),RAYRE(2)
        ENDIF
        IF(IRT.GT.0) THEN
          NRAN=NRODS(2,IRT)
          RAYRR(2)=0.0
          IRREG=IREG-NRAN
          IF(IMPX.GE.10) THEN
            WRITE(IOUT,6001) IRT,NRODS(1,IRT),NRAN,RODR(NRAN,IRT)
          ENDIF
          DO 120 IRAN=1,NRAN
            RAYRR(1)=RAYRR(2)
            RAYRR(2)=RODR(IRAN,IRT)
            IF(MASK(IRREG)) THEN
C----
C  PRINT INTEGRATION INFORMATION IF REQUIRED
C----
              IF(IAPPR.EQ.1) THEN
                CALL TUBE1(NINT(3),RAYRR,SIGT(IRREG),TRONC,PII(IRREG),
     >                     PIS(ISF),PSS(ISF2))
              ELSE
                CALL TUBE2(NINT(3),RAYRR,SIGT(IRREG),TRONC,PII(IRREG),
     >                     PIS(ISF),PSS(ISF2))
              ENDIF
              IF(IROT.EQ.2) THEN
                CALL XCSRNU(IAPPR,ISURF(IRREG),PIS(ISF),PSS(ISF2))
              ENDIF
            ENDIF
            IF(IMPX.GE.10) THEN
              WRITE(IOUT,6002) IRAN,IRREG,ISURF(IRREG),
     >                         RAYRR(1),RAYRR(2)
              WRITE(IOUT,6010) SIGT(IRREG),PII(IRREG)
            ENDIF
            ISF=ISF+ISURF(IRREG)
            ISF2=ISF2+ISURF(IRREG)*ISURF(IRREG)
            IRREG=IRREG+1
 120      CONTINUE
          READ(IFTRAK) NTRK,NTRAK,IVAR,VD
          IFIN=0
          DO 130 ITRK=1,6
            IF(NTRAK(ITRK).GT.0) THEN
              IDEB=IFIN+1
              IFIN=IFIN+4*NTRAK(ITRK)
              READ(IFTRAK) (TRAK(JJ),JJ=IDEB,IFIN)
            ENDIF
 130      CONTINUE
          IF(MASK(IREG)) THEN
            CALL XCSINT(IAPPR,IROT,NRODS(1,IRT),ISURF(IREG),CHORD(ISF),
     >                IVAR,NTRAK,TRAK,VD,SIGT(IREG),TRONC,
     >                PII(IREG),PIS(ISF),PSS(ISF2))
          ENDIF
          IF(IMPX.GE.10) THEN
            WRITE(IOUT,6011) SIGT(IREG),PII(IREG)
          ENDIF
          ISF=ISF+ISURF(IREG)
          ISF2=ISF2+ISURF(IREG)*ISURF(IREG)
        ELSE IF(IRT.LT.0) THEN
          IRT=ABS(IRT)
          NRAN=NRODS(2,IRT)
          RAYRR(2)=0.0
          IRREG=IREG-NRAN
          IF(IMPX.GE.10) THEN
            WRITE(IOUT,6001) IRT,NRODS(1,IRT),NRAN,RODR(NRAN,IRT)
          ENDIF
          DO 125 IRAN=1,NRAN
            RAYRR(1)=RAYRR(2)
            RAYRR(2)=RODR(IRAN,IRT)
            IF(MASK(IRREG)) THEN
C----
C  PRINT INTEGRATION INFORMATION IF REQUIRED
C----
              IF(IAPPR.EQ.1) THEN
                CALL TUBE1(NINT(3),RAYRR,SIGT(IRREG),TRONC,PII(IRREG),
     >                     PIS(ISF),PSS(ISF2))
              ELSE
                CALL TUBE2(NINT(3),RAYRR,SIGT(IRREG),TRONC,PII(IRREG),
     >                     PIS(ISF),PSS(ISF2))
              ENDIF
            ENDIF
            IF(IMPX.GE.10) THEN
              WRITE(IOUT,6002) IRAN,IRREG,ISURF(IRREG),
     >                         RAYRR(1),RAYRR(2)
              WRITE(IOUT,6010) SIGT(IRREG),PII(IRREG)
            ENDIF
            ISF=ISF+ISURF(IRREG)
            ISF2=ISF2+ISURF(IRREG)*ISURF(IRREG)
            IRREG=IRREG+1
 125      CONTINUE
          RAYRE(1)=RAYRR(2)
          IF(MASK(IREG)) THEN
            IF(IAPPR.EQ.1) THEN
              CALL TUBE1(NINT(3),RAYRE,SIGT(IREG),TRONC,PII(IREG),
     >                   PIS(ISF),PSS(ISF2))
            ELSE
              CALL TUBE2(NINT(3),RAYRE,SIGT(IREG),TRONC,PII(IREG),
     >                   PIS(ISF),PSS(ISF2))
            ENDIF
          ENDIF
          IF(IMPX.GE.10) THEN
            WRITE(IOUT,6011) SIGT(IREG),PII(IREG)
          ENDIF
          ISF=ISF+ISURF(IREG)
          ISF2=ISF2+ISURF(IREG)*ISURF(IREG)
        ELSE
          IF(MASK(IREG)) THEN
            IF(IAPPR.EQ.1) THEN
              CALL TUBE1(NINT(3),RAYRE,SIGT(IREG),TRONC,PII(IREG),
     >                   PIS(ISF),PSS(ISF2))
            ELSE
              CALL TUBE2(NINT(3),RAYRE,SIGT(IREG),TRONC,PII(IREG),
     >                   PIS(ISF),PSS(ISF2))
            ENDIF
          ENDIF
          IF(IMPX.GE.10) THEN
            WRITE(IOUT,6011) SIGT(IREG),PII(IREG)
          ENDIF
          ISF=ISF+ISURF(IREG)
          ISF2=ISF2+ISURF(IREG)*ISURF(IREG)
        ENDIF
 110  CONTINUE
      IF(NSOUT.EQ.4) THEN
        IREG=NRINFO(1,NBAN)
        RAYRR(1)=RAN(NTAN)
        RAYRR(2)=RAN(NBAN)
        IF(COTE.EQ.0.0) THEN
          RAYRR(3)=RAN(NBAN)
        ELSE
          RAYRR(3)=COTE
        ENDIF
        IF(MASK(IREG)) THEN
          READ(IFTRAK) LZI,NLZ
          CALL SETARA(BASE,LZI,IZ)
          CALL SETARA(BASE,NLZ,JZ)
          READ(IFTRAK) (BASE(JJ),JJ=IZ,IZ+LZI-1)
          READ(IFTRAK) (BASE(JJ),JJ=JZ,JZ+NLZ-1)
          IF(IAPPR.EQ.1) THEN
            CALL CROWR1(IAPPR,RAYRR,BASE(JZ),BASE(IZ),SIGT(IREG),
     >                  TRONC,PII(IREG),PIS(ISF),PSS(ISF2))
          ELSE
            CALL CROWR2(IAPPR,RAYRR,BASE(JZ),BASE(IZ),SIGT(IREG),
     >                  TRONC,PII(IREG),PIS(ISF),PSS(ISF2))
          ENDIF
          CALL RLSARA(BASE(JZ))
          CALL RLSARA(BASE(IZ))
        ENDIF
        IF(IMPX.GE.10) THEN
          WRITE(IOUT,6003) NBAN,IREG,ISURF(IREG),
     >                     RAYRR(1),RAYRR(2),RAYRR(3)
          WRITE(IOUT,6011) SIGT(IREG),PII(IREG)
        ENDIF
      ELSE IF(NSOUT.EQ.6) THEN
        IREG=NRINFO(1,NBAN)
        IF(MASK(IREG)) THEN
          READ(IFTRAK) LZI,NLZ
          CALL SETARA(BASE,LZI,IZ)
          CALL SETARA(BASE,NLZ,JZ)
          READ(IFTRAK) (BASE(JJ),JJ=IZ,IZ+LZI-1)
          READ(IFTRAK) (BASE(JJ),JJ=JZ,JZ+NLZ-1)
          IF(IAPPR.EQ.1) THEN
          CALL CROWH1(BASE(JZ),BASE(IZ),RAN(NBAN),SIGT(IREG),RAN(NTAN),
     >                IAPPR,TRONC,PIS(ISF),PSS(ISF2),PII(IREG))
          ELSE
          CALL CROWH2(BASE(JZ),BASE(IZ),RAN(NBAN),SIGT(IREG),RAN(NTAN),
     >                IAPPR,TRONC,PIS(ISF),PSS(ISF2),PII(IREG))
          ENDIF
          CALL RLSARA(BASE(JZ))
          CALL RLSARA(BASE(IZ))
        ENDIF
        IF(IMPX.GE.10) THEN
          WRITE(IOUT,6004) NBAN,IREG,ISURF(IREG),
     >                     RAN(NTAN),RAN(NBAN)
          WRITE(IOUT,6011) SIGT(IREG),PII(IREG)
        ENDIF
      ENDIF
      RETURN
C----
C  FORMATS
C----
 6000 FORMAT(/' INTEGRATING ANNULUS  :',I10/
     >        ' ANNULUS REGION NUMBER:',I10/
     >        ' NUMBER OF SURFACES   :',I10/
     >        ' INNER RADIUS         :',1P,E15.7/
     >        ' OUTER RADIUS         :',1P,E15.7)
 6001 FORMAT(' ROD TYPE             :',I10/
     >       ' NUMBER OF RODS       :',I10/
     >       ' NUMBER OF SUBRODS    :',I10/
     >       ' ROD OUTER RADIUS     :',1P,E15.7)
 6002 FORMAT(/20X,' INTEGRATING SUBROD   :',I10/
     >        20X,' REGION NUMBER        :',I10/
     >        20X,' NUMBER OF SURFACES   :',I10/
     >        20X,' INNER RADIUS         :',1P,E15.7/
     >        20X,' OUTER RADIUS         :',1P,E15.7)
 6003 FORMAT(/' INTEGRATING RECTANGULAR CROWN :',I10/
     >        ' ANNULUS REGION NUMBER         :',I10/
     >        ' NUMBER OF SURFACES            :',I10/
     >        ' INNER RADIUS                  :',1P,E15.7/
     >        ' SIDE X OF RECTANGLE           :',1P,E15.7/
     >        ' SIDE Y OF RECTANGLE           :',1P,E15.7)
 6004 FORMAT(/' INTEGRATING HEXAGONAL CROWN :',I10/
     >        ' ANNULUS REGION NUMBER       :',I10/
     >        ' NUMBER OF SURFACES          :',I10/
     >        ' INNER RADIUS                :',1P,E15.7/
     >        ' SIDE OF HEXAGONE            :',1P,E15.7)
 6010 FORMAT(/20X,' SIGT       =',1P,E15.7/
     >        20X,' PII        =',1P,E15.7)
 6011 FORMAT(/ ' SIGT       =',1P,E15.7/
     >         ' PII        =',1P,E15.7)
      END
