*DECK XCSSEG
      SUBROUTINE XCSSEG(MRTRK,NOREG,DISREG,ZTRAK,XTRAK,KTRAK,NREG)
C
C---------------------------  XCSSEG  ---------------------------------
C
C 1-  PROGRAMME STATISTICS:
C
C   NAME       :  XCSSEG
C   FUNCTION   :  SEGMENT TOTAL INTEGRATION ANGLES FOR ROD CLUSTER
C                 ACCORDING TO REGION ENCOUNTERED
C   VERSION    :  1.0
C   REVISED    :  03-04-1990
C   AUTHOR     :  G.MARLEAU
C
C 2-  PROGRAMME PARAMETERS:
C
C PARAMETERS.
C   MRTRK   : MAXIMUM NUMBER OF SEGMENT FOR REGION       I
C   NOREG   : REGION NUMBER CURRENTLY ANALYSING          I
C   DISREG  : REGION DISTANCE FROM STARTUP SURFACE       R
C   ZTRAK   : INITIAL BOTTOM AND TOP ANGLES PERMITTED    R(2,MRTRK)
C             FOR NEW REGION.  ALSO USED AS DUMMY VECTOR
C             AS NEW REGION WHEN SEGMENTATION OCCURS.
C   XTRAK   : BOTTOM, TOP ANGLES AND DISTANCE OF SEGMENT R(3,MRTRK)
C             ALREADY TRACKED. NEW SEGMENT ARISING FROM
C             ZTRAK VECTOR ARE INCLUDED HERE.
C   KTRAK   : REGION TYPE OF SEGMENT OF ANGLE            I(MRTRK)
C   NREG    : INITIAL/FINAL NUMBER OF SEGMENT            I
C
C---------------------------   XCSSEG  --------------------------------
C
      INTEGER    MRTRK,NOREG,KTRAK(MRTRK),NREG
      REAL       DISREG,ZTRAK(2,MRTRK),XTRAK(3,MRTRK)
      LOGICAL    LTOP,LBOT,LRSHAD
      NIREG=NREG
      NREGR=1
      NIREGR=NREGR
      DO 320 IRG=1,NIREG
        IF(XTRAK(3,IRG).GT.DISREG) THEN
          LRSHAD=.TRUE.
        ELSE
          LRSHAD=.FALSE.
        ENDIF
        DO 330 IRGR=1,NIREGR
C----
C  FIND IF REGION ANGLE OVERLAPP ROD ANGLES
C----
          IF     (ZTRAK(2,IRGR).GE.XTRAK(2,IRG)) THEN
            LTOP=.TRUE.
          ELSE IF(ZTRAK(2,IRGR).GT.XTRAK(1,IRG)) THEN
            LTOP=.FALSE.
          ELSE
C----
C  ROD OUT OF RANGE
C----
            GO TO 340
          ENDIF
          IF     (ZTRAK(1,IRGR).LE.XTRAK(1,IRG)) THEN
            LBOT=.TRUE.
          ELSE IF(ZTRAK(1,IRGR).LT.XTRAK(2,IRG)) THEN
            LBOT=.FALSE.
          ELSE
C----
C  ROD OUT OF RANGE
C----
            GO TO 340
          ENDIF
C----
C  FIND IF NEW ROD CAST SHADOWS ON REGION ALREADY CONSIDERED
C        XTRAK(3,IRG) > ZTRAK(3,IRGR)
C  OR IF REGION ALREADY CONSIDERED CAST SHADOW ON NEW ROD
C        XTRAK(3,IRG) < ZTRAK(3,IRGR)
C----
          IF(LTOP.AND.LBOT) THEN
C----
C  ROD ANGLES COVERS COMPLETE ANGLE
C  IF LRSHAD IS TRUE -> ROD COVERS ALL REGION
C  IF LRSHAD IS FALSE-> REGION COVERS PART OF ROD
C                       NREGR=NREGR+1
C----
            NREGR=NREGR+1
            ZTRAK(1,NREGR)=XTRAK(2,IRG)
            ZTRAK(2,NREGR)=ZTRAK(2,IRGR)
            ZTRAK(2,IRGR)  =XTRAK(1,IRG)
            IF(LRSHAD) THEN
              NREG=NREG+1
              XTRAK(1,NREG)=XTRAK(1,IRG)
              XTRAK(2,NREG)=XTRAK(2,IRG)
              XTRAK(3,NREG)=DISREG
              KTRAK(NREG)  =NOREG
              XTRAK(2,IRG)  =XTRAK(1,IRG)
            ENDIF
          ELSE IF(LTOP) THEN
C----
C  ROD ANGLES COVERS BOTTOM PART OF ANGLE
C  IF LRSHAD IS TRUE -> ROD COVERS BOTTOM REGION
C  IF LRSHAD IS FALSE-> REGION COVERS TOP PART OF ROD
C                       NREG=NREG+1
C----
            XTBUF=ZTRAK(1,IRGR)
            ZTRAK(1,IRGR) =XTRAK(2,IRG)
            IF(LRSHAD) THEN
              NREG=NREG+1
              XTRAK(1,NREG)=XTBUF
              XTRAK(2,NREG)=XTRAK(2,IRG)
              XTRAK(3,NREG)=DISREG
              KTRAK(NREG)  =NOREG
              XTRAK(2,IRG)  =XTBUF
            ENDIF
          ELSE IF(LBOT) THEN
C----
C  ROD ANGLES COVERS TOP PART OF ANGLE
C  IF LRSHAD IS TRUE -> ROD COVERS TOP REGION
C  IF LRSHAD IS FALSE-> REGION COVERS BOTTOM PART OF ROD
C                       NREG=NREG+1
C----
            XTBUF=ZTRAK(2,IRGR)
            ZTRAK(2,IRGR) =XTRAK(1,IRG)
            IF(LRSHAD) THEN
              NREG=NREG+1
              XTRAK(1,NREG)=XTRAK(1,IRG)
              XTRAK(2,NREG)=XTBUF
              XTRAK(3,NREG)=DISREG
              KTRAK(NREG)  =NOREG
              XTRAK(1,IRG) =XTBUF
            ENDIF
          ELSE
C----
C  ROD ANGLE LOCATED BETWEEN TOP AND BOTTOM PART OF ANGLE
C  IF LRSHAD IS TRUE -> ROD COVERS CENTRAL REGION
C                       TWO ADDITIONAL REGIONS CREATED
C  IF LRSHAD IS FALSE-> ROD DISAPPEARS COMPLETELY
C                       NREG=NREG+2
            IF(LRSHAD) THEN
              NREG=NREG+1
              XTRAK(1,NREG)=ZTRAK(2,IRGR)
              XTRAK(2,NREG)=XTRAK(2,IRG)
              XTRAK(3,NREG)=XTRAK(3,IRG)
              KTRAK(NREG)  =KTRAK(IRG)
C
              NREG=NREG+1
              XTRAK(1,NREG)=ZTRAK(1,IRGR)
              XTRAK(2,NREG)=ZTRAK(2,IRGR)
              XTRAK(3,NREG)=DISREG
              KTRAK(NREG)  =NOREG
C
              XTRAK(2,IRG)  =ZTRAK(1,IRGR)
            ENDIF
            ZTRAK(2,IRGR)=ZTRAK(1,IRGR)
          ENDIF
 340      CONTINUE
 330    CONTINUE
C----
C  COMPRESS ZTRAK FOR REGIONS ALREADY ELIMINATED
C----
        NIREGR=0
        DO 350 IRGR=1,NREGR
          IF(ZTRAK(2,IRGR).NE.ZTRAK(1,IRGR)) THEN
            NIREGR=NIREGR+1
            ZTRAK(1,NIREGR)=ZTRAK(1,IRGR)
            ZTRAK(2,NIREGR)=ZTRAK(2,IRGR)
          ENDIF
 350    CONTINUE
C----
C  ONCE ALL ANGLES SPANNED BY ROS HAVE BEEN TREATED
C  COMPRESS XTRAK FOR ADJACENT ROD TYPES REGIONS
C  TO AVOID FRAGMENTATION, COMPRESS XTRAK COMPLETELY AND EXIT
C  TO START NEW ROD ANALYSIS
C----
        IF(NIREGR.EQ.0) THEN
          DO 360 IRG1=NIREG+1,NREG-1
            REFA=XTRAK(2,IRG1)
            IF(REFA.NE.XTRAK(1,IRG1)) THEN
              DO 370 IRG2=IRG1+1,NREG
                IF( (KTRAK(IRG2) .EQ. KTRAK(IRG1)   ).AND.
     >              (REFA        .EQ. XTRAK(1,IRG2) )     ) THEN
                  REFA=XTRAK(2,IRG2)
                  XTRAK(2,IRG1)=REFA
                  XTRAK(1,IRG2)=XTRAK(2,IRG2)
                ENDIF
 370          CONTINUE
            ENDIF
 360      CONTINUE
          GO TO 380
        ENDIF
        NREGR=NIREGR
 320  CONTINUE
 380  CONTINUE
C----
C  COMPRESS REGIONS WHERE TOP AND BOTTOM ANGLE ARE IDENTICAL
C  AND RESET TOTAL NUMBER OF REGIONS TO TRACK
C----
      NIREG=0
      DO 390 IRG=1,NREG
        IF(XTRAK(2,IRG).NE.XTRAK(1,IRG)) THEN
          NIREG=NIREG+1
          XTRAK(1,NIREG)=XTRAK(1,IRG)
          XTRAK(2,NIREG)=XTRAK(2,IRG)
          XTRAK(3,NIREG)=XTRAK(3,IRG)
          KTRAK(NIREG)  =KTRAK(IRG)
        ENDIF
 390  CONTINUE
      NREG=NIREG
      RETURN
      END
