*DECK XCSTRK
      SUBROUTINE XCSTRK(NGPT1,NGPT2,ZGPT,WGPT,RINER,ROUTR,NRODS,
     >                  RODRAD,RODCEN,NTRAK,TRAK,NMTRAK,VD,
     >                  XTRAK,KTRAK,ZTRAK,MRTRK,IMPX,IVAR)
C
C---------------------------  XCSTRK  ---------------------------------
C
C 1-  PROGRAMME STATISTICS:
C
C   NAME       :  XCSTRK
C   FUNCTION   :  TRACK ROD CLUSTER FOR TRANSMISSION
C                 PROBABILITY CALCULATIONS
C   VERSION    :  1.0
C   REVISED    :  13-11-1989
C   AUTHOR     :  G.MARLEAU
C
C 2-  PROGRAMME PARAMETERS:
C
C  INPUT PARAMETERS.
C   NGPT1   : NUMBER OF GAUSS INTEGRATION POINTS FOR 1D
C             INTEGRATION OF LAST INTEGRAL IN 2D         I
C   NGPT2   : NUMBER OF GAUSS INTEGRATION POINTS FOR
C             FIRST INTEGRAL IN 2D                       I
C             NTGPT=NGPT1         IF NGPT1=NGPT2
C             NTGPT=NGPT1+NGPT2   OTHERWISE
C             IN GENERAL NGPT2 >= NGPT1
C   ZGPT    : GAUSS INTEGRATION POINTS                   R(NTGPT)
C   WGPT    : GAUSS INTEGRATION WEIGHTS                  R(NTGPT)
C   RINER   : RADIUS OF INNER SHELL                      R
C   ROUTR   : RADIUS OF OUTER SHELL                      R
C   NRODS   : NUMBER OF EQUALLY SPACED RODS BETWEEN      I
C             INNER AND OUTER SHELL
C   RODRAD  : RADIUS OF RODS                             R
C   RODCEN  : DISTANCE OF ROD CENTER FROM SHELL CENTER   R
C  OUTPUT PARAMETERS.
C   NTRAK   : NUMBER OF TRACK FOR EACH OF THE 5 TYPES    I(6)
C             OF PROBABILITIES
C   TRAK    : TRACKS AND WEIGHT FOR INTEGRATION          R(4,NMTRAK)
C             FOR TRAK IL:   TRAK(1,IL) WEIGHT
C                            TRAK(2,IL) LENGHT
C                            TRAK(3,IL) DIR(MU)
C                            TRAK(4,IL) DIR(NU)
C             TRAK  IL=1,2*NGPT1          -> PS3S3 (ITYP=1)
C             TRAK  IL=2*NGPT1+1,3*NGPT1  -> PS3S1 (ITYP=2)
C             TRAK  IL=3*NGPT1+1,NPS3S2   -> PS3S2 (ITYP=3)
C                      NPS3S2=3*NGPT1+NTRAK(3)
C                      NTRAK(3)=NREG32*NGPT2*NGPT1
C             TRAK  IL=NPS3S2+1,NPS2S3   -> PS2S3 (ITYP=4)
C                      NPS2S3=NPS3S2+NTRAK(4)
C                      NTRAK(4)=NREG23*NGPT2*NGPT1
C             TRAK  IL=NPS2S3+1,NPS2S1   -> PS2S1 (ITYP=5)
C                      NPS2S1=NPS2S3+NTRAK(5)
C                      NTRAK(5)=NREG21*NGPT2*NGPT1
C             TRAK  IL=NPS2S1+1,NPS2S2   -> PS2S2 (ITYP=6)
C                      NPS2S2=NPS2S1+NTRAK(6)
C                      NTRAK(6)=NREG22*NGPT2*NGPT1
C   NMTRAK  : MAXIMUM NUMBER OF TRACK REGION PERMITTED   I
C             NMTRAK = 4*(2*NRODS+3)*NGPT1*NGPT2 + 3*MAX(NGPT1,NGPT2)
C   VD        SERIES EXPANSION OF BICKLEY FUNCTIONS
C             CONTRIBUTION                               R(ITYP,J,K,L)
C             ITYP -- FOR TYPE OF PROBABILITIES (ITYP=1,6)
C             J    -- INITIAL DIRECTION         (J=1,2)
C             K    -- FINAL DIRECTION           (K=1,2)
C             L    -- ORDER OF SERIES EXPANSION (L=1,3)
C  DUMMY PARAMETERS.
C   XTRAK   : TOP ANGLE, BOTTOM ANGLE AND  AND DISTANCE
C             FOR INTEGRATION REGION                     R(3,MRTRK)
C   KTRAK   : ROD NUMBER FOR INTEGRATION REGION          I(MTRK)
C   ZTRAK   : ANGLE LIMITS FOR ROD CONSIDERED            R(2,MRTRK)
C   MRTRK   : NUMBER OF ROD REGION PERMITTED             I
C   IMPX    : PRINT LEVEL >=15 PRINT TRACKING            I
C   IVAR    : TYPE OF CLUSTERS                           I
C             =1  NO RODS, NO INNER ANNULUS (FILLED TUBE)
C             =2  RODS, NO INNER ANNULUS    (FILLED CLUSTER)
C             =3  NO RODS, INNER ANNULUS    (HOLLOW TUBE)
C             =4  RODS AND INNER ANNULUS    (HOLLOW CLUSTER)
C
C 3-  ROUTINES CALLED BY THIS PROGRAMME:
C
C   XCSSEG  : SEGMENT PROJECTIVE REGION FOR CURRENT ROD
C   XCSQRT  : SAFE SQUARE ROOT (INTERNAL FUNCTION)
C
C 4-  PARAMETERS IN ROUTINE:
C
C   CI      : BICKLEY FUNCTION EXPANSION COEFFICIENT     R(L,J+K-1)
C             L -- ORDER OF SERIES EXPANSION (L=1,3)
C             J -- INITIAL DIRECTION         (J=1,2)
C             K -- FINAL DIRECTION           (K=1,2)
C
C---------------------------   XCSTRK  --------------------------------
C
      SAVE       CI
      PARAMETER (NTYP=6,PI=3.1415926535898,PIO2=PI/2.0,EOPI=8.0/PI,
     >           NDIR=2,NKI=3,NOR=3,IOUT=6)
      INTEGER    NGPT1,NGPT2,NRODS,MRTRK,NTRAK(NTYP),KTRAK(MRTRK),
     >           NMTRAK,IMPX,ITR(NTYP),ITF(NTYP)
      REAL       ZGPT(*),WGPT(*),RINER,ROUTR,RODRAD,RODCEN,
     >           TRAK(4,NMTRAK),XTRAK(3,MRTRK),ZTRAK(2,MRTRK),
     >           VD(NTYP,NDIR,NDIR,NOR),EB(2),EH(2),CI(NOR,NKI)
      DATA       CI
     >          /0.78539816339745,-1.00000000000000,0.78539816339745,
     >           0.66666666666667,-0.78539816339745,0.50000000000000,
     >           0.58904862254809,-0.66666666666667,0.39269908169873/
      XCSQRT(ARG)=SQRT(MAX(0.0,ARG))
      IF(NGPT1.EQ.NGPT2) THEN
        IR2=0
      ELSE
        IR2=NGPT1
      ENDIF
      XNRODS=FLOAT(NRODS)
C----
C  FIND TYPE OF GEOMETRY IVAR
C----
      IVAR=4
      IF(RINER.EQ.0.0) THEN
        IVAR=2
      ELSE
        RRI=RODRAD/RINER
        RCI=RODCEN/RINER
      ENDIF
      IF( (NRODS.EQ.0) .OR. (RODRAD.EQ.0.0) ) THEN
        IVAR=IVAR-1
      ELSE
        IF(RODCEN.GT.0.0) THEN
          RODRAP=RODRAD/RODCEN
          RODOUT=ROUTR/RODCEN
        ENDIF
        GAMMA=PI/XNRODS
        RCO=RODCEN/ROUTR
        RRO=RODRAD/ROUTR
        ROR=ROUTR/RODRAD
        RCR=RODCEN/RODRAD
      ENDIF
      XKAPPA=RINER/ROUTR
      UCHV=1.0-XKAPPA
      UCHV2=XCSQRT(UCHV)
      YKAPPA=(RODRAD+RODCEN)/ROUTR
      UYCHV=1.0-YKAPPA
      UYCHV2=XCSQRT(UYCHV)
      MRODS= 2*(NRODS+1)
C----
C  INITIALIZE POINTERS FOR TRACKS
C----
      NGPTM=MAX(NGPT1,NGPT2)
      ITR(1) = 1
      ITR(2) = 2*NGPTM+ITR(1)
      ITR(3) = NGPTM+ITR(2)
      ITR(4) = NGPT1*NGPT2*MRODS+ITR(3)
      ITR(5) = NGPT1*NGPT2*MRODS+ITR(4)
      ITR(6) = NGPT1*NGPT2*MRODS+ITR(5)
      DO 90 IIT=1,6
        ITF(IIT)  =ITR(IIT)
 90   CONTINUE
C----
C  FIND Y=SIN(PHI) INTEGRATION LIMITS FOR CLUSTER
C  FOR (3 -> 3) SURFACES        YKAPPA < Y < 1
C  WITH YKAPPA=RINER/ROUTR            FOR CLUSTER ABSENT
C  WITH YKAPPA=(RODRAD+RODCEN)/ROUTR  FOR CLUSTER PRESENT
C----
      IF( (IVAR.EQ.1) .OR. (IVAR.EQ.3) ) THEN
        YKAPPA=XKAPPA
        UYCHV=UCHV
        UYCHV2=UCHV2
      ENDIF
      WFACO=EOPI*UYCHV2
      DO 200 JGPT=1,NGPT1
C----
C  PS3S3
C----
        UJ=ZGPT(JGPT)*UYCHV2
        U2J=UJ*UJ
        TRAK(4,ITF(1))=UJ*XCSQRT(2.0-U2J)
        TRAK(3,ITF(1))=TRAK(4,ITF(1))
        TRAK(2,ITF(1))=2.0*ROUTR*TRAK(4,ITF(1))
        TRAK(1,ITF(1))=WFACO*WGPT(JGPT)*UJ
        ITF(1)=ITF(1)+1
 200  CONTINUE
      IF(IVAR.EQ.3) THEN
        WFACI=EOPI*YKAPPA
C----
C  PS3S1
C----
        DO 201 JGPT=1,NGPT1
          U2J=ZGPT(JGPT)*ZGPT(JGPT)
          TRAK(4,ITF(2))=ZGPT(JGPT)*XCSQRT(2.0-U2J)
          XKU2J=YKAPPA*(1.0-U2J)
          TRAK(3,ITF(2))=XCSQRT(1.0-XKU2J*XKU2J)
          TRAK(2,ITF(2))=ROUTR*(TRAK(3,ITF(2))-YKAPPA*TRAK(4,ITF(2)))
          TRAK(1,ITF(2))=WFACI*WGPT(JGPT)*ZGPT(JGPT)
          ITF(2)=ITF(2)+1
 201    CONTINUE
      ELSE IF( (IVAR.EQ.2) .OR. (IVAR.EQ.4) ) THEN
        DELGAM=2.0*GAMMA
        QOPI2=4.0/(PI*PI)
        QNOPI2=QOPI2*XNRODS
C----
C  ROD CLUSTER ANALYSIS
C  (3 -> 3) + (3 -> 2) :  XKAPPA <= SIN(PHI) <= YKAPPA
C  INTEGRATE OVER U=SQRT(1-SIN(PHI)) WITH LIMITS
C  SQRT(1-YKAPPA) < U < UCHV2=SQRT(1-XKAPPA) USING GAUSS QUADRATURE.
C----
        WSCALE=(UCHV2-UYCHV2)
        WFACT1=QNOPI2*WSCALE
        DO 210 JGPT=IR2+1,IR2+NGPT2
          UJ=ZGPT(JGPT)*WSCALE+UYCHV2
          U2J=UJ*UJ
          SXJ=(1.0-U2J)
          CXJ=UJ*XCSQRT(2.0-U2J)
          PHI  =ASIN(SXJ)
          NREG=1
          XTRAK(1,NREG)=0.0
          XTRAK(2,NREG)=DELGAM
          XTRAK(3,NREG)=2.0*ROUTR
          KTRAK(NREG)=0
          IF(RODCEN.EQ.0.0) THEN
            KTRAK(NREG)=1
            XL3=ROR*SXJ
            XL30=XCSQRT(1.0-XL3*XL3)
          ELSE
            AXMB =ACOS(RODOUT*SXJ-RODRAP)
            AXPB =RODOUT*SXJ+RODRAP
C----
C  LOCATE RANGE OF ANGLES EPSILON PERMITTED FOR A GIVEN VALUE OF PHI
C  FOR RODS LOCATED ON AN ANNULUS CENTERED AT RODCEN
C    EITHER: AXMB  =(ROUTR*SXJ+RODRAD)/ RODCEN > 1
C            -> DELGAM-EPSB1 <= EPSILON <= DELGAM-EPST1
C    OR    : AXMB  =(ROUTR*SXJ+RODRAD)/ RODCEN < 1
C            -> DELGAM-EPSB1 <- EPSILON <= DELGAM-EPSB2
C               DELGAM-EPST2 <- EPSILON <= DELGAM-EPST1
C----
            EPSC1=PIO2-PHI
            EPST1=(EPSC1-AXMB)
            EPSB1=(EPSC1+AXMB)
            EPSR=0.5*AXMB
            NEPSR=1
            EB(1)=EPSB1
            IF(AXPB.LT.1.0) THEN
              AXPB =ACOS(AXPB)
              EPST2=(EPSC1-AXPB)
              EPSB2=(EPSC1+AXPB)
              EPSR=0.5*AXPB
              EH(1)=EPSB2
              EB(2)=EPST2
              NEPSR=2
            ENDIF
            EPSRB=EPSC1+EPSR
            EPSRT=EPSC1-EPSR
            EH(NEPSR)=EPST1
            RODANG=GAMMA
            DO 212 IROD=1,NRODS
C----
C  FIND TOP AND BOTTOM ANGLE FOR EACH RODS SEPARATED BY ANGLE DELGAM
C----
              NOREG=IROD
              DO 213 IEPSR=1,NEPSR
                ZTRAK(2,1)=MIN(RODANG-EH(IEPSR),DELGAM)
                ZTRAK(1,1)=MAX(RODANG-EB(IEPSR),0.0)
                IF(ZTRAK(2,1).GT.ZTRAK(1,1)) THEN
                  XLN1=ROUTR-RODCEN*COS(RODANG-GAMMA)
                  XLN2=RODCEN*SIN(RODANG-GAMMA)
                  DISREG=XCSQRT(XLN2*XLN2+XLN1*XLN1)
                  CALL XCSSEG(MRTRK,NOREG,DISREG,ZTRAK,XTRAK,
     >                        KTRAK,NREG)
                ENDIF
                NOREG=-NOREG
 213          CONTINUE
              RODANG=RODANG+DELGAM
 212        CONTINUE
          ENDIF
C----
C  COMPLETE EPSILON SPANNED FOR THIS PHI FINISH EVALUATION OF TRACKS
C  FOR P33 AND P32(1) INTEGRATE OVER EPSILON EXPLICITELY
C----
          IF(IMPX.GE.15) THEN
            WRITE(IOUT,6001) PHI,NREG
            WRITE(IOUT,6000) (KTRAK(IIJJ),XTRAK(1,IIJJ),XTRAK(2,IIJJ),
     >      (XTRAK(1,IIJJ)+XTRAK(2,IIJJ))/2.0,IIJJ=1,NREG)
          ENDIF
          TOTEPS=0.0
          WFACO=WGPT(JGPT)*WFACT1*UJ
          DO 400 IRG=1,NREG
            IF(KTRAK(IRG).EQ.0) THEN
              TOTEPS=TOTEPS+(XTRAK(2,IRG)-XTRAK(1,IRG))
            ELSE
              IF(RODCEN.EQ.0.0) THEN
                TRAK(4,ITF(3))=XL30
                TRAK(3,ITF(3))=CXJ
                TRAK(2,ITF(3))=RODRAD*(ROR*CXJ-TRAK(4,ITF(3)))
                TRAK(1,ITF(3))=WFACO*(XTRAK(2,IRG)-XTRAK(1,IRG))
                ITF(3)=ITF(3)+1
              ELSE
C----
C  DISCRETIZE EPSILON INTEGRAL
C----
                RODANG=REAL(ABS(KTRAK(IRG)))*DELGAM-GAMMA
                DELEPS=(XTRAK(2,IRG)-XTRAK(1,IRG))
                WFACR=WFACO*DELEPS
                DO 220 IGPT=1,NGPT1
                  EPSCUR=DELEPS*ZGPT(IGPT)+XTRAK(1,IRG)
                  XL1=ROR-RCR*COS(RODANG-EPSCUR)
                  XL2=RCR*SIN(RODANG-EPSCUR)
                  XL3=XL1*SXJ-XL2*CXJ
                  TRAK(4,ITF(3))=XCSQRT(1.0-XL3*XL3)
                  TRAK(3,ITF(3))=CXJ
                  TRAK(2,ITF(3))=RODRAD*(XL1*CXJ+XL2*SXJ-TRAK(4,ITF(3)))
                  TRAK(1,ITF(3))=WFACR*WGPT(IGPT)
                  ITF(3)=ITF(3)+1
 220            CONTINUE
              ENDIF
            ENDIF
 400      CONTINUE
          IF(TOTEPS.GT.0.0) THEN
            TRAK(4,ITF(1))=CXJ
            TRAK(3,ITF(1))=CXJ
            TRAK(2,ITF(1))=2*ROUTR*CXJ
            TRAK(1,ITF(1))=WFACO*TOTEPS
            ITF(1)=ITF(1)+1
          ENDIF
 210    CONTINUE
C----
C  ROD CLUSTER ANALYSIS
C  (3 -> 2) + (3 -> 1) :     0   <= SIN(PHI) <= XKAPPA
C  INTEGRATE OVER U=SQRT(1-SIN(PHI)/XKAPPA)
C  WITH LIMITS  0 < U < 1  USING GAUSS QUADRATURE.
C----
        IF(XKAPPA.GT.0.0) THEN
          WFACT2=QNOPI2*XKAPPA
          DO 230 JGPT=IR2+1,IR2+NGPT2
            UJ=ZGPT(JGPT)
            U2J=UJ*UJ
            SXJ=(1.0-U2J)*XKAPPA
            CXJ  =XCSQRT(1.0-SXJ*SXJ)
            NREG=1
            XTRAK(1,NREG)=0.0
            XTRAK(2,NREG)=DELGAM
            XTRAK(3,NREG)=ROUTR
            KTRAK(NREG)=0
            PHI  =ASIN(SXJ)
            AXMB =ACOS(RODOUT*SXJ-RODRAP)
            AXPB =ACOS(RODOUT*SXJ+RODRAP)
C----
C  LOCATE RANGE OF ANGLES EPSILON PERMITTED FOR A GIVEN VALUE OF PHI
C  FOR RODS LOCATED ON AN ANNULUS CENTERED AT RODCEN
C    EITHER: AXMB  =(ROUTR*SXJ+RODRAD)/ RODCEN > 1
C            -> DELGAM-EPSB1 <= EPSILON <= DELGAM-EPST1
C    OR    : AXMB  =(ROUTR*SXJ+RODRAD)/ RODCEN < 1
C            -> DELGAM-EPSB1 <- EPSILON <= DELGAM-EPSB2
C               DELGAM-EPST2 <- EPSILON <= DELGAM-EPST1
C----
            NEPSR=2
            EB(1)=(PIO2-PHI+AXMB)
            EH(1)=(PIO2-PHI+AXPB)
            EB(2)=(PIO2-PHI-AXPB)
            EH(2)=(PIO2-PHI-AXMB)
            RODANG=GAMMA
            DO 231 IROD=1,NRODS
C----
C  FIND TOP AND BOTTOM ANGLE FOR EACH RODS SEPARATED BY ANGLE DELGAM
C----
              NOREG=IROD
              DO 232 IEPSR=1,NEPSR
                ZTRAK(2,1)=MIN(RODANG-EH(IEPSR),DELGAM)
                ZTRAK(1,1)=MAX(RODANG-EB(IEPSR),0.0)
                IF(ZTRAK(2,1).GT.ZTRAK(1,1)) THEN
                  XLN1=ROUTR-RODCEN*COS(RODANG-GAMMA)
                  XLN2=RODCEN*SIN(RODANG-GAMMA)
                  DISREG=XCSQRT(XLN2*XLN2+XLN1*XLN1)
                  IF(DISREG.LT.ROUTR) THEN
                    CALL XCSSEG(MRTRK,NOREG,DISREG,ZTRAK,XTRAK,
     >                          KTRAK,NREG)
                  ENDIF
                ENDIF
                NOREG=-NOREG
 232          CONTINUE
              RODANG=RODANG+DELGAM
 231        CONTINUE
C----
C  COMPLETE EPSILON SPANNED FOR THIS PHI FINISH EVALUATION OF TRACKS
C  FOR P31 AND P32(2) INTEGRATE OVER EPSILON EXPLICITELY
C----
            IF(IMPX.GE.15) THEN
              WRITE(IOUT,6002) PHI,NREG
              WRITE(IOUT,6000) (KTRAK(IIJJ),XTRAK(1,IIJJ),XTRAK(2,IIJJ),
     >        (XTRAK(1,IIJJ)+XTRAK(2,IIJJ))/2.0,IIJJ=1,NREG)
            ENDIF
            WFACI=WGPT(JGPT)*WFACT2*UJ
            TOTEPS=0.0
            DO 233 IRG=1,NREG
              IF(KTRAK(IRG).EQ.0) THEN
                TOTEPS=TOTEPS+(XTRAK(2,IRG)-XTRAK(1,IRG))
              ELSE
C----
C  DISCRETIZE EPSILON INTEGRAL
C----
                RODANG=REAL(ABS(KTRAK(IRG)))*DELGAM-GAMMA
                DELEPS=(XTRAK(2,IRG)-XTRAK(1,IRG))
                WFACR=WFACI*DELEPS
                DO 240 IGPT=1,NGPT1
                  EPSCUR=DELEPS*ZGPT(IGPT)+XTRAK(1,IRG)
                  XL1=ROR-RCR*COS(RODANG-EPSCUR)
                  XL2=RCR*SIN(RODANG-EPSCUR)
                  XL3=XL1*SXJ-XL2*CXJ
                  TRAK(4,ITF(3))=XCSQRT(1.0-XL3*XL3)
                  TRAK(3,ITF(3))=CXJ
                  TRAK(2,ITF(3))=RODRAD*(XL1*CXJ+XL2*SXJ-TRAK(4,ITF(3)))
                  TRAK(1,ITF(3))=WFACR*WGPT(IGPT)
                  ITF(3)=ITF(3)+1
 240            CONTINUE
              ENDIF
 233        CONTINUE
            IF(TOTEPS.GT.0.0) THEN
              TRAK(4,ITF(2))=UJ*XCSQRT(2.0-U2J)
              XKU2J=XKAPPA*(1.0-U2J)
              TRAK(3,ITF(2))=XCSQRT(1.0-XKU2J*XKU2J)
              TRAK(2,ITF(2))=ROUTR*( TRAK( 3,ITF(2) )
     >                      -XKAPPA* TRAK( 4,ITF(2) ) )
              TRAK(1,ITF(2))=WFACI*TOTEPS
              ITF(2)=ITF(2)+1
            ENDIF
 230      CONTINUE
        ENDIF
C----
C  TRANSMISSION FROM SURFACE 2
C  INTEGRATE OVER U=SQRT(1-SIN(PHI)) USING GAUSS-JACOBI
C  QUADRATURE.
C  THE  INTEGRATION  IS FOR  0 < U < 1
C----
        DO 300 JGPT=IR2+1,IR2+NGPT2
          SXJ=(1.0-ZGPT(JGPT)**2)
          CXJ=XCSQRT(1.0-SXJ*SXJ)
          PHI=ASIN(SXJ)
C----
C  LOCATE CENTRAL REGION
C----
          IF(RINER.EQ.0.0) THEN
            XTRAK(1,1)=-PI
            XTRAK(2,1)=PI
            XTRAK(3,1)=2.0*ROUTR
            KTRAK(1)=0
            NREG=1
          ELSE
            ANGM=-ASIN((RINER-RODRAD*SXJ)/RODCEN)-PHI
            ANGP= ASIN((RINER+RODRAD*SXJ)/RODCEN)-PHI
            XTRAK(1,1)=-PI
            XTRAK(2,1)=ANGM
            XTRAK(3,1)=2.0*ROUTR
            KTRAK(1)=0
            XTRAK(1,2)=ANGM
            XTRAK(2,2)=ANGP
            OMEGA=ANGM+PIO2+PHI
            SOMEG=SIN(OMEGA)
            COMEG=COS(OMEGA)
            XL1=RRI*SIN(ANGM)
            XL2=RCI-RRI*COS(ANGM)
            XL3=XL1*SOMEG-XL2*COMEG
            XL3=RINER*(XL1*COMEG+XL2*SOMEG-XCSQRT(1.0-XL3*XL3))
            OMEGA=ANGP+PIO2+PHI
            SOMEG=SIN(OMEGA)
            COMEG=COS(OMEGA)
            XL1=RRI*SIN(ANGP)
            XL2=RCI-RRI*COS(ANGP)
            XL4=XL1*SOMEG-XL2*COMEG
            XL4=RINER*(XL1*COMEG+XL2*SOMEG-XCSQRT(1.0-XL4*XL4))
            XTRAK(3,2)=0.5*(XL3+XL4)
            KTRAK(2)=1
            XTRAK(1,3)=ANGP
            XTRAK(2,3)=PI
            XTRAK(3,3)=2.0*ROUTR
            KTRAK(3)=0
            NREG=3
          ENDIF
          RODANG=DELGAM
          DO 310 NOREG=2,NRODS
C----
C  FIND TOP AND BOTTOM ANGLE FOR EACH RODS SEPARATED BY ANGLE DELGAM
C  ALSO LOCATE DISATANCE FROM ROD TO SURFACE FOR
C  INNER CLUSTER COMPARISON
C----
            DISREG=2*RODCEN*SIN(RODANG/2.0)
            ZTRAK(1,1)=(PI-RODANG)/2.0-PHI
     >                -ASIN((RODRAD-RODRAD*SXJ)/DISREG)
            ZTRAK(2,1)=(PI-RODANG)/2.0-PHI
     >                +ASIN((RODRAD+RODRAD*SXJ)/DISREG)
C----
C  CALL XCSSEG TO SEGMENT TRAKING ANGLES ACCORDING TO DISTANCE
C----
            CALL XCSSEG(MRTRK,NOREG,DISREG,ZTRAK,XTRAK,
     >                  KTRAK,NREG)
            RODANG=RODANG+DELGAM
 310      CONTINUE
C----
C  ALL RODS HAVE BEEN TREATED
C  DISCRETIZE EPSILON INTEGRAL
C----
          IF(IMPX.GE.15) THEN
            WRITE(IOUT,6003) PHI,NREG
            WRITE(IOUT,6000) (KTRAK(IIJJ),XTRAK(1,IIJJ),XTRAK(2,IIJJ),
     >      (XTRAK(1,IIJJ)+XTRAK(2,IIJJ))/2.0,IIJJ=1,NREG)
          ENDIF
          WFACT1=WGPT(JGPT)*QOPI2*ZGPT(JGPT)
          DO 320 IRG=1,NREG
            DELEPS=(XTRAK(2,IRG)-XTRAK(1,IRG))
            WFAC=WFACT1*DELEPS
            IF(KTRAK(IRG).EQ.0) THEN
C----
C  PS2S3
C----
              DO 330 IGPT=1,NGPT1
                EPSCUR=DELEPS*ZGPT(IGPT)+XTRAK(1,IRG)
                OMEGA=EPSCUR+PHI
                SOMEG=SIN(OMEGA)
                COMEG=COS(OMEGA)
                XL1=RRO*SIN(EPSCUR)
                XL2=RCO-RRO*COS(EPSCUR)
                XL3=XL1*COMEG+XL2*SOMEG
                TRAK(4,ITF(4))=XCSQRT(1.0-XL3*XL3)
                TRAK(3,ITF(4))=CXJ
                TRAK(2,ITF(4))=ROUTR*(TRAK(4,ITF(4))
     >                        -(XL1*SOMEG-XL2*COMEG))
                TRAK(1,ITF(4))=WFAC*WGPT(IGPT)
                ITF(4)=ITF(4)+1
 330          CONTINUE
            ELSE IF(KTRAK(IRG).EQ.1) THEN
C----
C  PS2S1
C----
              DO 340 IGPT=1,NGPT1
                EPSCUR=DELEPS*ZGPT(IGPT)+XTRAK(1,IRG)
                OMEGA=EPSCUR+PIO2+PHI
                SOMEG=SIN(OMEGA)
                COMEG=COS(OMEGA)
                XL1=RRI*SIN(EPSCUR)
                XL2=RCI-RRI*COS(EPSCUR)
                XL3=XL1*SOMEG-XL2*COMEG
                TRAK(4,ITF(5))=XCSQRT(1.0-XL3*XL3)
                TRAK(3,ITF(5))=CXJ
                TRAK(2,ITF(5))=RINER*
     >                         (XL1*COMEG+XL2*SOMEG-TRAK(4,ITF(5)))
                TRAK(1,ITF(5))=WFAC*WGPT(IGPT)
                ITF(5)=ITF(5)+1
 340          CONTINUE
            ELSE
C----
C  PS2S2
C----
              RODANG=0.5*REAL(KTRAK(IRG)-1)*DELGAM
              DO 350 IGPT=1,NGPT1
                EPSCUR=DELEPS*ZGPT(IGPT)+XTRAK(1,IRG)
                OMEGA=EPSCUR+RODANG+PHI
                SOMEG=SIN(OMEGA)
                COMEG=COS(OMEGA)
                EPSLOC=EPSCUR+RODANG-PIO2
                XL1=SIN(EPSLOC)
                XL2=2.0*RCR*SIN(RODANG)-COS(EPSLOC)
                XL3=XL1*SOMEG-XL2*COMEG
                TRAK(4,ITF(6))=XCSQRT(1.0-XL3*XL3)
                TRAK(3,ITF(6))=CXJ
                TRAK(2,ITF(6))=RODRAD*
     >                         (XL1*COMEG+XL2*SOMEG-TRAK(4,ITF(6)))
                TRAK(1,ITF(6))=WFAC*WGPT(IGPT)
                ITF(6)=ITF(6)+1
 350          CONTINUE
            ENDIF
 320      CONTINUE
 300    CONTINUE
      ENDIF
C----
C  TRACKING COMPLETED
C  INTEGRATE EXPLICITELY SERIES EXPANSION FOR BICKLEY FUNCTION
C  COLLISION PROBABILITY GIVEN BY
C  PIJ(JJ,J,K)=VD(JJ,J,K,1)+VD(JJ,J,K,2)*XS+VD(JJ,J,K,3)*XS**2
C  AND COMPRESS TRAK VECTOR
C----
      DO 500 ITYP=1,NTYP
        DO 510 J=1,NDIR
          DO 520 K=1,NDIR
            DO 530 L=1,NOR
              VD(ITYP,J,K,L)=0.0
 530        CONTINUE
 520      CONTINUE
 510    CONTINUE
 500  CONTINUE
      JTRAK=0
      DO 600 ITYP=1,NTYP
        DO 610 ITRAK=ITR(ITYP),ITF(ITYP)-1
          D1=1.0
          DO 620 J=1,NDIR
            D2=1.0
            DO 630 K=1,NDIR
              XL=TRAK(1,ITRAK)*D1*D2
              DO 640 L=1,NOR
                VD(ITYP,J,K,L)=VD(ITYP,J,K,L)+CI(L,J+K-1)*XL
                XL=XL*TRAK(2,ITRAK)
 640          CONTINUE
              D2=D2*TRAK(4,ITRAK)
 630        CONTINUE
            D1=D1*TRAK(3,ITRAK)
 620      CONTINUE
C----
C  COMPRESS TRAK VECTOR
C----
          JTRAK=JTRAK+1
          DO 650 IIJ=1,4
            TRAK(IIJ,JTRAK)=TRAK(IIJ,ITRAK)
 650      CONTINUE
 610    CONTINUE
        NTRAK(ITYP)=ITF(ITYP)-ITR(ITYP)
 600  CONTINUE
      RETURN
C----
C  FORMATS
C----
 6000 FORMAT((1X,I3,5X,F12.7,3X,F12.7,3X,F12.7))
 6001 FORMAT(' FIRST REGION TRACKING  -- PS3S3(0) AND PS3S2 '/
     >       ' TRACKING ANGLE PHI =',F10.7,10X,
     >       'NUMBER OF REGIONS CROSSED =',I10/
     >       ' REGION   LOW EPSILON   HIGH EPSILON   MIDDLE EPSILON')
 6002 FORMAT(' SECOND REGION TRACKING  -- PS3S1(0) AND PS3S1 '/
     >       ' TRACKING ANGLE PHI =',F10.7,10X,
     >       'NUMBER OF REGIONS CROSSED =',I10/
     >       ' REGION   LOW EPSILON   HIGH EPSILON   MIDDLE EPSILON')
 6003 FORMAT(' THIRD REGION TRACKING  -- PS2S3(0), PS2S1(1) AND PS2S1'/
     >       ' TRACKING ANGLE PHI =',F10.7,10X,
     >       'NUMBER OF REGIONS CROSSED =',I10/
     >       ' REGION   LOW EPSILON   HIGH EPSILON   MIDDLE EPSILON')
      END