*DECK ALPLSF
      SUBROUTINE ALPLSF(NORIN,X,Y,EPSRID,LREAL,NOR,A,B,PREC)
*
*-----------------------------------------------------------------------
*
* COMPUTE THE POLYNOMIAL COEFFICIENTS OF A PADE APPROXIMATION USING A
* DIRECT LEAST SQUARE PROCEDURE.
*
* INPUT PARAMETERS:
*  NORIN  : 2*NORIN+1 IS THE NUMBER OF COLLOCATION POINTS.
*  X      : ABSCISSA OF THE COLLOCATION POINTS. DIMENSION X(0:2*NORIN)
*  Y      : ORDINATES OF THE COLLOCATION POINTS. DIMENSION Y(0:2*NORIN)
*  EPSRID : EPSILON USED IN POLYNOMIAL SIMPLIFICATION.
*  LREAL  : =.TRUE. TO GET RID OF COMPLEX ROOTS.
*
* OUTPUT PARAMETERS:
*  NOR    : ORDER OF THE POLYNOMIALS.
*  A      : POLYNOMIAL COEFFICIENTS OF THE NUMERATOR OF THE PADE
*           APPROXIMATION. A(0) IS THE CONSTANT TERM.
*  B      : POLYNOMIAL COEFFICIENTS OF THE DENOMINATOR OF THE PADE
*           APPROXIMATION. B(0) IS THE CONSTANT TERM.
*           DOUBLE PRECISION A(0:NOR),B(0:NOR)
*  PREC   : ACCURACY OF THE FIT.
*
*------------------------------------- AUTHOR: A. HEBERT  ; 10/01/96 ---
*---------------------------------- MODIFIED : G. MARLEAU ; 98/04/30 ---
*                                   REPLACE COMPLEX*16 BY
*                                   DOUBLE PRECISION AND CARRY OUT
*                                   EXPLICITELY COMPLEX ALGEBRA
*
      PARAMETER       (MAXNOR=10,MAXPTS=2*MAXNOR+1,IREAL=1,IMAGI=2)
      INTEGER          NORIN,NOR
      LOGICAL          LREAL
      REAL             X(0:2*NORIN),Y(0:2*NORIN),PREC
      DOUBLE PRECISION EPSRID,A(0:NORIN),B(0:NORIN)
C----
C  LOCAL PARAMETERS
C----
      DOUBLE PRECISION AA(MAXPTS,MAXPTS),BB(MAXPTS),
     >                 GAR,PARAM(MAXPTS),AGAR(0:MAXNOR),
     >                 BGAR(0:MAXNOR),CGAR(0:MAXNOR+1),
     >                 DENOM,V(MAXPTS,MAXPTS),W(MAXPTS),
     >                 RV1(MAXPTS),SGN,DABSX,DABSW
C----
C  DOUBLE PRECISION VECTOR SIMULATING COMPLEX*16 VECTOR
C----
      DOUBLE PRECISION SIGX0(2,MAXNOR+1),SIGXW(2,MAXNOR+1),
     >                 DDAGAR(2,0:MAXNOR),DDBGAR(2,0:MAXNOR),
     >                 WEIGHT(2,MAXNOR+1),DCTMP(2),CCTMP(2),
     >                 DDTMP(2),DETMP(2)
*
      IF(NORIN.GT.MAXNOR) CALL XABORT('ALPLSF: INSUFFICIENT MAXNOR.')
*
* NOR=0 CASE.
C----
C  TEST PRINT - 1 FOR VALIDATION
C      write(6,*) 'alplsf'
C      write(6,*) 'norin =',norin
C      write(6,*) 'x =',(x(jj),jj=0,2*norin)
C      write(6,*) 'y =',(y(jj),jj=0,2*norin)
C  END TEST PRINT - 1 FOR VALIDATION
C----
      NTEL=2*NORIN
      NTELM1=NTEL-1
      NTELP1=NTEL+1
      NOR=0
      PREC=0.0
      A(0)=DBLE(Y(NTEL))
      B(0)=1.0D0
      DO 10 I=0,NTELM1
   10 PREC=MAX(PREC,ABS(Y(NTEL)/Y(I)-1.0))
*
      DO 170 IINOR=1,NORIN
      INOR=IINOR
      IF(X(NTEL).GE.1.0E10) THEN
         DO 40 I=0,NTELM1
         GAR=1.0D0
         IOF=0
         DO 20 J=0,INOR-1
         IOF=IOF+1
         AA(I+1,IOF)=GAR
   20    GAR=GAR*X(I)
         GAR=1.0D0
         DO 30 J=0,INOR-1
         IOF=IOF+1
         AA(I+1,IOF)=-GAR*Y(I)
   30    GAR=GAR*X(I)
         BB(I+1)=Y(I)-Y(NTEL)
         DO 40 J=1,2*INOR
   40    AA(I+1,J)=AA(I+1,J)/GAR
         CALL ALSVDF(AA,NTEL,2*INOR,MAXPTS,MAXPTS,W,V,RV1)
         DO 45 J=1,2*INOR
         IF(W(J).EQ.0.0D0) CALL XABORT('ALPLSF: SVD FAILURE(1).')
   45    CONTINUE
         CALL ALSVDS(AA,W,V,NTEL,2*INOR,MAXPTS,MAXPTS,BB,PARAM,RV1)
         DO 50 I=0,INOR-1
         AGAR(I)=PARAM(I+1)
   50    BGAR(I)=PARAM(INOR+1+I)
         AGAR(INOR)=Y(NTEL)
         BGAR(INOR)=1.0D0
      ELSE
         DO 80 I=0,NTEL
         GAR=1.0D0
         IOF=0
         DO 60 J=0,INOR
         IOF=IOF+1
         AA(I+1,IOF)=GAR
   60    GAR=GAR*X(I)
         GAR=1.0D0
         DO 70 J=0,INOR-1
         IOF=IOF+1
         AA(I+1,IOF)=-GAR*Y(I)
   70    GAR=GAR*X(I)
         BB(I+1)=Y(I)
         DO 80 J=1,2*INOR+1
   80    AA(I+1,J)=AA(I+1,J)/GAR
         CALL ALSVDF(AA,NTELP1,2*INOR+1,MAXPTS,MAXPTS,W,V,RV1)
         DO 85 J=1,2*INOR
         IF(W(J).EQ.0.0D0) CALL XABORT('ALPLSF: SVD FAILURE(2).')
   85    CONTINUE
         CALL ALSVDS(AA,W,V,NTELP1,2*INOR+1,MAXPTS,MAXPTS,BB,PARAM,
     1   RV1)
         DO 90 I=0,INOR
         AGAR(I)=PARAM(I+1)
         IF(I.EQ.INOR) THEN
            BGAR(I)=1.0D0
         ELSE
            BGAR(I)=PARAM(INOR+2+I)
         ENDIF
   90    CONTINUE
      ENDIF
*
* POLYNOMIAL SIMPLIFICATION.
      DDAGAR(IREAL,0)=AGAR(INOR)
      DDAGAR(IMAGI,0)=0.0D0
      DDBGAR(IREAL,0)=BGAR(INOR)
      DDBGAR(IMAGI,0)=0.0D0
      CALL ALROOT(AGAR,INOR,SIGX0)
      CALL ALROOT(BGAR,INOR,SIGXW)
C----
C  TEST PRINT - 2 FOR VALIDATION
C      write(6,*) 'sigx0 =',(sigx0(1,jj),sigx0(2,jj),jj=1,inor)
C      write(6,*) 'sigxw =',(sigxw(1,jj),sigxw(2,jj),jj=1,inor)
C  END TEST PRINT - 2 FOR VALIDATION
C----
      IJINOR=1
   95 CONTINUE
      DCTMP(IREAL)=SIGXW(IREAL,IJINOR)-SIGX0(IREAL,IJINOR)
      DCTMP(IMAGI)=SIGXW(IMAGI,IJINOR)-SIGX0(IMAGI,IJINOR)
      CALL DCAABS(DABSX,DCTMP)
      CALL DCAABS(DABSW,SIGXW(1,IJINOR))
      IF( ( DABSX .LT. DBLE(EPSRID)*DABSW ) .OR.
     >    ( SIGXW(IREAL,IJINOR) .GT. 0.0  )      ) THEN
         INOR=INOR-1
         DO 100 I=IJINOR,INOR
           SIGX0(IREAL,I)=SIGX0(IREAL,I+1)
           SIGX0(IMAGI,I)=SIGX0(IMAGI,I+1)
           SIGXW(IREAL,I)=SIGXW(IREAL,I+1)
           SIGXW(IMAGI,I)=SIGXW(IMAGI,I+1)
  100    CONTINUE
      ELSE IF(SIGXW(IREAL,IJINOR).GT.0.) THEN
         GO TO 170
      ELSE
         IJINOR=IJINOR+1
      ENDIF
      IF(IJINOR.LE.INOR) GO TO 95
      IF(INOR.LT.0) CALL XABORT('ALPLSF: ALGORITHM FAILURE.')
      DO 120 I=1,INOR
        DDAGAR(IREAL,I)=DDAGAR(IREAL,I-1)
        DDAGAR(IMAGI,I)=DDAGAR(IMAGI,I-1)
        DDBGAR(IREAL,I)=DDBGAR(IREAL,I-1)
        DDBGAR(IMAGI,I)=DDBGAR(IMAGI,I-1)
        DO 110 J=I-1,1,-1
          DCTMP(IREAL)=DDAGAR(IREAL,J)*SIGX0(IREAL,I)-
     >                 DDAGAR(IMAGI,J)*SIGX0(IMAGI,I)
          DCTMP(IMAGI)=DDAGAR(IREAL,J)*SIGX0(IMAGI,I)+
     >                 DDAGAR(IMAGI,J)*SIGX0(IREAL,I)
          DDAGAR(IREAL,J)=DDAGAR(IREAL,J-1)-DCTMP(IREAL)
          DDAGAR(IMAGI,J)=DDAGAR(IMAGI,J-1)-DCTMP(IMAGI)
          DCTMP(IREAL)=DDBGAR(IREAL,J)*SIGXW(IREAL,I)-
     >                 DDBGAR(IMAGI,J)*SIGXW(IMAGI,I)
          DCTMP(IMAGI)=DDBGAR(IREAL,J)*SIGXW(IMAGI,I)+
     >                 DDBGAR(IMAGI,J)*SIGXW(IREAL,I)
          DDBGAR(IREAL,J)=DDBGAR(IREAL,J-1)-DCTMP(IREAL)
          DDBGAR(IMAGI,J)=DDBGAR(IMAGI,J-1)-DCTMP(IMAGI)
 110    CONTINUE
        DCTMP(IREAL)=DDAGAR(IREAL,0)*SIGX0(IREAL,I)-
     >               DDAGAR(IMAGI,0)*SIGX0(IMAGI,I)
        DCTMP(IMAGI)=DDAGAR(IREAL,0)*SIGX0(IMAGI,I)+
     >               DDAGAR(IMAGI,0)*SIGX0(IREAL,I)
        DDAGAR(IREAL,0)=-DCTMP(IREAL)
        DDAGAR(IMAGI,0)=-DCTMP(IMAGI)
        DCTMP(IREAL)=DDBGAR(IREAL,0)*SIGXW(IREAL,I)-
     >               DDBGAR(IMAGI,0)*SIGXW(IMAGI,I)
        DCTMP(IMAGI)=DDBGAR(IREAL,0)*SIGXW(IMAGI,I)+
     >               DDBGAR(IMAGI,0)*SIGXW(IREAL,I)
        DDBGAR(IREAL,0)=-DCTMP(IREAL)
        DDBGAR(IMAGI,0)=-DCTMP(IMAGI)
 120  CONTINUE
      DENOM=DDBGAR(IREAL,INOR)
      DO 130 I=0,INOR
        AGAR(I)=DDAGAR(IREAL,I)/DENOM
        BGAR(I)=DDBGAR(IREAL,I)/DENOM
C----
C  TEST PRINT - 3 FOR VALIDATION
C        write(6,*) 'I,A,B =',i,AGAR(i),BGAR(i)
C  END TEST PRINT - 3 FOR VALIDATION
C----
        IF(AGAR(I).LE.0.0D0) GO TO 170
        IF(BGAR(I).LE.0.0D0) GO TO 170
 130  CONTINUE
      SGN=1.0D0
      CGAR(0)=AGAR(0)
      DO 135 I=2,INOR+1
        SGN=-SGN
        CGAR(I-1)=SGN*(BGAR(I-2)+AGAR(I-1))
 135  CONTINUE
      CGAR(INOR+1)=-SGN
      CALL ALROOT(CGAR,INOR+1,SIGX0)
      IF(LREAL) THEN
        DO 140 I=1,INOR+1
          IF(SIGX0(IMAGI,I).NE.0.0) GO TO 170
 140    CONTINUE
      ENDIF
*
* COMPUTE THE WEIGHTS.
      DO 150 I=1,INOR+1
        CCTMP(IREAL)=1.0D0
        CCTMP(IMAGI)=0.0D0
        DDTMP(IREAL)=0.0D0
        DDTMP(IMAGI)=0.0D0
        DO 151 JNOR=0,INOR
          DDTMP(IREAL)=DDTMP(IREAL)+BGAR(JNOR)*CCTMP(IREAL)
          DDTMP(IMAGI)=DDTMP(IMAGI)+BGAR(JNOR)*CCTMP(IMAGI)
          DABSX       =-SIGX0(IREAL,I)*CCTMP(IREAL)
     >                 +SIGX0(IMAGI,I)*CCTMP(IMAGI)
          CCTMP(IMAGI)=-SIGX0(IREAL,I)*CCTMP(IMAGI)
     >                 -SIGX0(IMAGI,I)*CCTMP(IREAL)
          CCTMP(IREAL)=DABSX
 151    CONTINUE
        DO 152 J=1,INOR+1
          IF(J.NE.I) THEN
            CCTMP(IREAL)=SIGX0(IREAL,J)-SIGX0(IREAL,I)
            CCTMP(IMAGI)=SIGX0(IMAGI,J)-SIGX0(IMAGI,I)
            CALL DCADIV(DCTMP,CCTMP)
            DABSX       =DCTMP(IREAL)*DDTMP(IREAL)
     >                  -DCTMP(IMAGI)*DDTMP(IMAGI)
            DDTMP(IMAGI)=DCTMP(IREAL)*DDTMP(IMAGI)
     >                  +DCTMP(IMAGI)*DDTMP(IREAL)
            DDTMP(IREAL)=DABSX
          ENDIF
 152    CONTINUE
        WEIGHT(IREAL,I)=DDTMP(IREAL)
        WEIGHT(IMAGI,I)=DDTMP(IMAGI)
 150  CONTINUE
*
* TEST THE ACCURACY OF THE PADE APPROXIMATION.
      PREC1=0.0
      DO 160 I=0,NTEL
        CCTMP(IREAL)=0.0D0
        CCTMP(IMAGI)=0.0D0
        DDTMP(IREAL)=0.0D0
        DDTMP(IMAGI)=0.0D0
        DO 161 JNOR=1,INOR+1
          DETMP(IREAL)=SIGX0(IREAL,JNOR)+X(I)
          DETMP(IMAGI)=SIGX0(IMAGI,JNOR)
          CALL DCADIV(DCTMP,DETMP)
          DETMP(IREAL)=WEIGHT(IREAL,JNOR)*DCTMP(IREAL)
     >                -WEIGHT(IMAGI,JNOR)*DCTMP(IMAGI)
          DETMP(IMAGI)=WEIGHT(IREAL,JNOR)*DCTMP(IMAGI)
     >                +WEIGHT(IMAGI,JNOR)*DCTMP(IREAL)
          CCTMP(IREAL)=CCTMP(IREAL)+DETMP(IREAL)
          CCTMP(IMAGI)=CCTMP(IMAGI)+DETMP(IMAGI)
          DCTMP(IREAL)=SIGX0(IREAL,JNOR)*DETMP(IREAL)
     >                -SIGX0(IMAGI,JNOR)*DETMP(IMAGI)
          DCTMP(IMAGI)=SIGX0(IREAL,JNOR)*DETMP(IMAGI)
     >                +SIGX0(IMAGI,JNOR)*DETMP(IREAL)
          DDTMP(IREAL)=DDTMP(IREAL)+DCTMP(IREAL)
          DDTMP(IMAGI)=DDTMP(IMAGI)+DCTMP(IMAGI)
 161    CONTINUE
        CALL DCADIV(DCTMP,CCTMP)
        ERROR=REAL(DDTMP(IREAL)*DCTMP(IREAL)
     >            -DDTMP(IMAGI)*DCTMP(IMAGI))
        PREC1=MAX(PREC1,ABS(ERROR/Y(I)-1.0))
 160  CONTINUE
      IF(PREC1.LT.0.95*PREC) THEN
        NOR=INOR
        PREC=PREC1
        DO 162 I=0,NOR
          A(I)=AGAR(I)
          B(I)=BGAR(I)
C----
C  TEST PRINT - 4 FOR VALIDATION
C        write(6,*) 'I,A,B =',i,A(i),B(i)
C  END TEST PRINT - 4 FOR VALIDATION
C----
 162    CONTINUE
      ENDIF
 170  CONTINUE
      RETURN
      END
