*DECK ALROOT
      SUBROUTINE ALROOT(A,M,ROOTS)
*
*-----------------------------------------------------------------------
*
* FIND THE ROOTS OF A POLYNOMIAL.
*
* INPUT PARAMETERS:
*  A     : POLYNOMIAL COEFFICIENTS. DIMENSION A(M+1)
*  M     : POLYNOMIAL ORDER.
*
* OUTPUT PARAMETER:
*  ROOTS : DOUBLE PRECISION COMPLEX ROOTS.
*          ROOTS(1,M) IS REAL PART
*          ROOTS(2,M) IS IMAGINARY PART
*
*-----------------------------------------------------------------------
*
      IMPLICIT         NONE
      INTEGER          M
      DOUBLE PRECISION A(M+1),ROOTS(2,M)
*----
*  LOCAL VARIABLES
*----
      INTEGER          MAXM,IREAL,IMAGI,J,JJ,I,II,ITS
      PARAMETER       (MAXM=101,IREAL=1,IMAGI=2)
      DOUBLE PRECISION EPS,SQRTM3,RR,QQ,PP,AA,BB,AAA(2),BBB(2),
     >                 SQ1(2),AD(2,MAXM),X(2),B(2),C(2),DCTMP(2),
     >                 DPOWS2,DPOWS3
      PARAMETER       (EPS=1.0D-6,SQRTM3=1.73205080757D0,
     >                 DPOWS2=1.0D0/2.0D0,DPOWS3=1.0D0/3.0D0)
*
      IF(M+1.GT.MAXM) CALL XABORT('ALROOT: INSUFFICIENT STORAGE.')
      IF(M.EQ.1) THEN
        ROOTS(IREAL,1)=-A(1)/A(2)
        ROOTS(IMAGI,1)=0.0
      ELSE IF(M.EQ.2) THEN
        RR=A(1)/A(3)
        QQ=A(2)/(2.0D0*A(3))
        AAA(IREAL)=QQ*QQ-RR
        AAA(IMAGI)=0.0D0
        CALL DCAPOW(AAA,AAA,DPOWS2)
        ROOTS(IREAL,1)=-AAA(1)-QQ
        ROOTS(IMAGI,1)=-AAA(2)
        ROOTS(IREAL,2)= AAA(1)-QQ
        ROOTS(IMAGI,2)= AAA(2)
      ELSE IF(M.EQ.3) THEN
        RR=A(1)/A(4)
        QQ=A(2)/(2.0D0*A(4))
        PP=A(3)/(3.0D0*A(4))
        AA=(2.0D0*QQ-3.0D0*PP**2)/3.0D0
        BB=(PP**3-PP*QQ+RR/2.0D0)
        SQ1(IREAL)=BB**2+AA**3
        SQ1(IMAGI)=0.0D0
        CALL DCAPOW(SQ1,SQ1,DPOWS2)
        AAA(IREAL)=BB-SQ1(IREAL)
        AAA(IMAGI)=  -SQ1(IMAGI)
        IF(AAA(IREAL).GE.0.0D0) THEN
          CALL DCAPOW(AAA,AAA,DPOWS3)
          AAA(IREAL)=-AAA(IREAL)
          AAA(IMAGI)=-AAA(IMAGI)
        ELSE
          AAA(IREAL)=-AAA(IREAL)
          AAA(IMAGI)=-AAA(IMAGI)
          CALL DCAPOW(AAA,AAA,DPOWS3)
        ENDIF
        BBB(IREAL)=BB+SQ1(IREAL)
        BBB(IMAGI)=   SQ1(IMAGI)
        IF(BBB(IREAL).GE.0.0D0) THEN
          CALL DCAPOW(BBB,BBB,DPOWS3)
          BBB(IREAL)=-BBB(IREAL)
          BBB(IMAGI)=-BBB(IMAGI)
        ELSE
          BBB(IREAL)=-BBB(IREAL)
          BBB(IMAGI)=-BBB(IMAGI)
          CALL DCAPOW(BBB,BBB,DPOWS3)
        ENDIF
        QQ=AAA(IREAL)+BBB(IREAL)
        RR=-SQRTM3*(AAA(IMAGI)-BBB(IMAGI))
        ROOTS(IREAL,1)=QQ-PP
        ROOTS(IREAL,2)=( RR-QQ)/2.0D0-PP
        ROOTS(IREAL,3)=(-RR-QQ)/2.0D0-PP
        QQ=(AAA(IMAGI)+BBB(IMAGI))
        RR=SQRTM3*(AAA(IREAL)-BBB(IREAL))
        ROOTS(IMAGI,1)=QQ
        ROOTS(IMAGI,2)=( RR-QQ)/2.0D0
        ROOTS(IMAGI,3)=(-RR-QQ)/2.0D0
      ELSE
        DO 100 J=1,M+1
          AD(IREAL,J)=A(J)
          AD(IMAGI,J)=0.0D0
 100    CONTINUE
        DO 110 J=M,1,-1
          X(IREAL)=0.0D0
          X(IMAGI)=0.0D0
          CALL ALGUER(AD,J,X,ITS)
          IF(ABS(X(IMAGI)).LE.2.D0*EPS**2*ABS(X(IREAL))) THEN
            X(IMAGI)=0.0D0
          ENDIF
          ROOTS(IREAL,J)=X(IREAL)
          ROOTS(IMAGI,J)=X(IMAGI)
          B(IREAL)=AD(IREAL,J+1)
          B(IMAGI)=AD(IMAGI,J+1)
          DO 120 JJ=J,1,-1
            C(IREAL)=AD(IREAL,JJ)
            C(IMAGI)=AD(IMAGI,JJ)
            AD(IREAL,JJ)=B(IREAL)
            AD(IMAGI,JJ)=B(IMAGI)
            DCTMP(IREAL)=X(IREAL)*B(IREAL)-X(IMAGI)*B(IMAGI)
            DCTMP(IMAGI)=X(IREAL)*B(IMAGI)+X(IMAGI)*B(IREAL)
            B(IREAL)=C(IREAL)+DCTMP(IREAL)
            B(IMAGI)=C(IMAGI)+DCTMP(IMAGI)
 120      CONTINUE
 110    CONTINUE
        DO 130 J=1,M+1
          AD(IREAL,J)=A(J)
          AD(IMAGI,J)=0.0D0
 130    CONTINUE
        DO 140 J=1,M
          CALL ALGUER(AD,M,ROOTS(1,J),ITS)
 140    CONTINUE
      ENDIF
*
      DO 150 J=2,M
        X(IREAL)=ROOTS(IREAL,J)
        X(IMAGI)=ROOTS(IMAGI,J)
        DO 160 I=J-1,1,-1
          II=I
          IF(ROOTS(IREAL,II).LE.X(IREAL)) GO TO 155
          ROOTS(IREAL,II+1)=ROOTS(IREAL,II)
          ROOTS(IMAGI,II+1)=ROOTS(IMAGI,II)
 160    CONTINUE
        II=0
 155    CONTINUE
        ROOTS(IREAL,II+1)=X(IREAL)
        ROOTS(IMAGI,II+1)=X(IMAGI)
 150  CONTINUE
      RETURN
      END
