*DECK ALSBC
      SUBROUTINE ALSBC (N,IS,B,IER,MAX)
*
*-----------------------------------------------------------------------
*
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS USING GAUSSIAN ELIMINATION
* WITH PARTIAL PIVOTING. COMPLEX VERSION ARRANGED TO DOUBLE PRECISION.
*
* INPUT PARAMETERS:
*  N       : ORDER OF THE COEFFICIENT MATRIX.
*  IS      : NUMBER OF RIGHT HAND VECTORS.
*  B       : COEFFICIENT MATRIX AUGMENTED WITH THE RIGHT HAND VECTORS.
*            DOUBLE PRECISION B(2,MAX,N+IS)
*  MAX     : FIRST DIMENTION OF MATRIX B.
*
* OUTPUT PARAMETERS:
*  B       : SOLUTION VECTORS, STARTING AT B(1,N+1).
*  IER     : ERROR FLAG. EXECUTION FAILURE IF IER.NE.0.
*
*------------------------------------------------ AUTHOR: A. HEBERT ----
*
      IMPLICIT         NONE
      INTEGER          N,IS,IER,MAX,NM1,NP1
      INTEGER          IREAL,IMAGI,I,J,IN,JCOL,M,IP1,K
      DOUBLE PRECISION TEST,DABSB
      DOUBLE PRECISION B(2,MAX,*),PMX(2),PER(2),PINV(2),STO(2)
      PARAMETER       (IREAL=1,IMAGI=2)
      M=N+IS
      IER=0
      NM1=N-1
      NP1=N+1
      IF (N.EQ.1) GO TO 100
*
* SEARCH FOR MAXIMUM PIVOT ON COLUMN JCOL.
      DO 40 JCOL=1,NM1
         TEST=0.0D0
         DO 10 I=JCOL,N
            CALL DCAABS(DABSB,B(1,I,JCOL))
            IF (DABSB.LE.TEST) GO TO 10
            TEST=DABSB
            IN=I
10       CONTINUE
         IF (TEST.EQ.0.0D0) GO TO 120
*
* TRIANGULARIZATION.
         PMX(IREAL)=B(IREAL,IN,JCOL)
         PMX(IMAGI)=B(IMAGI,IN,JCOL)
         CALL DCADIV(PINV,PMX)
         B(IREAL,IN,JCOL)=B(IREAL,JCOL,JCOL)
         B(IMAGI,IN,JCOL)=B(IMAGI,JCOL,JCOL)
         IP1=JCOL+1
         DO 30 J=IP1,M
            PER(IREAL)= B(IREAL,IN,J)*PINV(IREAL)-
     >                  B(IMAGI,IN,J)*PINV(IMAGI)
            PER(IMAGI)= B(IREAL,IN,J)*PINV(IMAGI)+
     >                  B(IMAGI,IN,J)*PINV(IREAL)
            B(IREAL,IN,J)=B(IREAL,JCOL,J)
            B(IMAGI,IN,J)=B(IMAGI,JCOL,J)
            B(IREAL,JCOL,J)=PER(IREAL)
            B(IMAGI,JCOL,J)=PER(IMAGI)
            DO 20 I=IP1,N
               B(IREAL,I,J)=B(IREAL,I,J)-
     >                     (B(IREAL,I,JCOL)*PER(IREAL)-
     >                      B(IMAGI,I,JCOL)*PER(IMAGI))
               B(IMAGI,I,J)=B(IMAGI,I,J)-
     >                     (B(IREAL,I,JCOL)*PER(IMAGI)+
     >                      B(IMAGI,I,JCOL)*PER(IREAL))
20          CONTINUE
30       CONTINUE
40    CONTINUE
      PER(IREAL)=B(IREAL,N,N)
      PER(IMAGI)=B(IMAGI,N,N)
      CALL DCADIV(PINV,PER)
      DO 50 J=NP1,M
         STO(IREAL)= B(IREAL,N,J)
         STO(IMAGI)= B(IMAGI,N,J)
         B(IREAL,N,J)= STO(IREAL)*PINV(IREAL)-STO(IMAGI)*PINV(IMAGI)
         B(IMAGI,N,J)= STO(IREAL)*PINV(IMAGI)+STO(IMAGI)*PINV(IREAL)
50    CONTINUE
*
* BACK SUBSTITUTION.
      DO 90 IN=2,N
         I=N-IN+1
         IP1=I+1
         DO 80 J=NP1,M
            PER(IREAL)=B(IREAL,I,J)
            PER(IMAGI)=B(IMAGI,I,J)
            DO 70 K=IP1,N
               PER(IREAL)=PER(IREAL)-
     >                    (B(IREAL,I,K)*B(IREAL,K,J)-
     >                     B(IMAGI,I,K)*B(IMAGI,K,J))
               PER(IMAGI)=PER(IMAGI)-
     >                    (B(IREAL,I,K)*B(IMAGI,K,J)+
     >                     B(IMAGI,I,K)*B(IREAL,K,J))
70          CONTINUE
            B(IREAL,I,J)=PER(IREAL)
            B(IMAGI,I,J)=PER(IMAGI)
80       CONTINUE
90    CONTINUE
      RETURN
*
100   CONTINUE
      PER(IREAL)=B(IREAL,1,1)
      PER(IMAGI)=B(IMAGI,1,1)
      IF (PER(IREAL).EQ.0.0D0.AND.PER(IMAGI).EQ.0.0D0) GO TO 120
      CALL DCADIV(PINV,PER)
      DO 110 J=2,M
         STO(IREAL)= B(IREAL,1,J)
         STO(IMAGI)= B(IMAGI,1,J)
         B(IREAL,1,J)= STO(IREAL)*PINV(IREAL)-STO(IMAGI)*PINV(IMAGI)
         B(IMAGI,1,J)= STO(IREAL)*PINV(IMAGI)+STO(IMAGI)*PINV(IREAL)
110   CONTINUE
      RETURN
120   CONTINUE
      IER=1
      RETURN
      END
